Encoding: UTF-8
Package: RSiena
Type: Package
Title: Siena - Simulation Investigation for Empirical Network Analysis
Version: 1.6.6
Date: 2026-04-20
Authors@R: c(person("Tom A.B.", "Snijders", role = c("aut", "ctb"), comment = c(ORCID = "0000-0003-3157-4157")),
              person("Ruth M.", "Ripley", role = "aut"),
              person("Krists", "Boitmanis", role = c( "aut","ctb")),
              person("Christian", "Steglich", email = "c.e.g.steglich@rug.nl", role = c("cre", "aut"), comment = c(ORCID = "0000-0002-9097-0873")),
			     person("Johan", "Koskinen", role = c("ctb"), comment = c(ORCID = "0000-0002-6860-325X")),
              person("Nynke M.D.", "Niezink", role = c("aut","ctb"), comment = c(ORCID = "0000-0003-4199-4841")),
              person("Viviana", "Amati", role = c("aut","ctb"), comment = c(ORCID = "0000-0003-1190-1237")),
              person("Christoph", "Stadtfeld", role = "ctb", comment = c(ORCID = "0000-0002-2704-2134")),
              person("James", "Hollway", role = "ctb", comment = c("IHEID", ORCID = "0000-0002-8361-9647")),
              person("Per", "Block", role = "ctb", comment = c(ORCID = "0000-0002-7583-2392")),
              person("Robert", "Krause", role = "ctb", comment = c(ORCID = "0000-0003-4288-4732")),
              person("Charlotte", "Greenan", role = "ctb"),
              person("Josh", "Lospinoso", role = "ctb"),
              person("Michael", "Schweinberger", role = "ctb", comment = c(ORCID = "0000-0003-3649-5386")),
              person("Mark", "Huisman", role = "ctb",comment = c(ORCID = "0000-0002-9009-7859")),
              person("Felix", "Schoenenberger", role = c("aut","ctb")),
              person("Mark", "Ortmann", role = "ctb"),
              person("Marion", "Hoffman", role = "ctb", comment=c(ORCID = "0000-0002-0741-7760")),
              person("Robert", "Hellpap", role = "ctb"),
              person("Alvaro", "Uzaheta", role = "ctb", comment=c(ORCID = "0000-0003-4367-3670")),
              person("Steffen", "Triebel", role = "ctb"),
              person("Daniel", "Gotthardt", role = "ctb", comment=c(ORCID = "0000-0002-1294-9913")))
Maintainer: Christian Steglich <c.e.g.steglich@rug.nl>
Depends: R (>= 4.5.0)
Imports: Matrix, lattice, parallel, MASS, methods, xtable, network
Suggests: tools, codetools, tcltk
SystemRequirements: GNU make
Description: The main purpose of this package is to perform simulation-based
   estimation of stochastic actor-oriented models for longitudinal network
   data collected as panel data. Dependent variables can be single or
   multivariate networks, which can be directed, non-directed, or two-mode;
   and associated actor variables.
   There are also functions for testing parameters and checking goodness of fit.
   An overview of these models is given in Snijders (2017),
   <doi:10.1146/annurev-statistics-060116-054035>.
License: GPL-2 | GPL-3 | file LICENSE
LazyData: yes
Biarch: yes
NeedsCompilation: yes
BuildResaveData: no
URL: https://www.stats.ox.ac.uk/~snijders/siena/
BugReports: https://github.com/stocnet/rsiena/issues
Packaged: 2026-04-20 08:00:41 UTC; cegst
Author: Tom A.B. Snijders [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-3157-4157>),
  Ruth M. Ripley [aut],
  Krists Boitmanis [aut, ctb],
  Christian Steglich [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-9097-0873>),
  Johan Koskinen [ctb] (ORCID: <https://orcid.org/0000-0002-6860-325X>),
  Nynke M.D. Niezink [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-4199-4841>),
  Viviana Amati [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-1190-1237>),
  Christoph Stadtfeld [ctb] (ORCID:
    <https://orcid.org/0000-0002-2704-2134>),
  James Hollway [ctb] (IHEID, ORCID:
    <https://orcid.org/0000-0002-8361-9647>),
  Per Block [ctb] (ORCID: <https://orcid.org/0000-0002-7583-2392>),
  Robert Krause [ctb] (ORCID: <https://orcid.org/0000-0003-4288-4732>),
  Charlotte Greenan [ctb],
  Josh Lospinoso [ctb],
  Michael Schweinberger [ctb] (ORCID:
    <https://orcid.org/0000-0003-3649-5386>),
  Mark Huisman [ctb] (ORCID: <https://orcid.org/0000-0002-9009-7859>),
  Felix Schoenenberger [aut, ctb],
  Mark Ortmann [ctb],
  Marion Hoffman [ctb] (ORCID: <https://orcid.org/0000-0002-0741-7760>),
  Robert Hellpap [ctb],
  Alvaro Uzaheta [ctb] (ORCID: <https://orcid.org/0000-0003-4367-3670>),
  Steffen Triebel [ctb],
  Daniel Gotthardt [ctb] (ORCID: <https://orcid.org/0000-0002-1294-9913>)
Repository: CRAN
Date/Publication: 2026-04-20 11:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 09:06:26 UTC; unix
Archs: RSiena.so.dSYM

useDynLib(RSiena, .registration = TRUE, .fixes="C_")

export(
	coCovar, coDyadCovar, varCovar, varDyadCovar, as_covariate_rsiena,
	sienaNet, sienaDependent, as_dependent_rsiena,
	getEffects, print01Report, write_report, siena07, siena,
	sienaCompositionChange, updateTheta, updateSpecification,
	sienaCompositionChangeFromFile, sienaDataCreate, make_data_rsiena,
	sienaGroupCreate, make_group_rsiena,
	sienaModelCreate, sienaAlgorithmCreate, 
	set_model_saom, set_algorithm_saom, set_output_saom, 
	sienaNodeSet, as_nodeset_rsiena, xtable.sienaFit, setEffect,
	includeEffects, includeInteraction, includeGMoMStatistics, 
   effectsDocumentation, sienaDataConstraint, print.xtable.sienaFit, 
	siena08, meta_siena, iwlsm, sienaTimeTest,
	includeTimeDummy, sienaGOF, descriptives, interpret_size,
	interpret_size_dynamics, sparseMatrixExtraction, networkExtraction, behaviorExtraction,
	OutdegreeDistribution, IndegreeDistribution, BehaviorDistribution,
	TriadCensus, mixedTriadCensus, dyadicCov, egoAlterCombi, egoAlterCovarComb,
	siena_table, xtable, score.Test, Wald.RSiena, Multipar.RSiena, 
   testSame.RSiena, funnelPlot, meta.table,
	influenceTable, selectionTable,
    as_composition_rsiena,as_composition_file_rsiena, make_constraint, 
	make_specification, set_effect, set_interaction, 
   update_theta, update_specification, 
	test_parameter, write_result, test_time, test_gof,
	interpret_selection, interpret_influence, transformScript)
importFrom("Matrix", "Matrix", "sparseMatrix", "spMatrix", "drop0", 
           "rankMatrix", "mat2triplet")
importFrom("lattice", "xyplot", "trellis.par.get", "trellis.par.set", "bwplot",
           "panel.violin", "panel.bwplot", "panel.xyplot", "panel.text",
           "panel.abline") 
importFrom("parallel", "clusterCall", "parLapply", "parRapply",
           "clusterSetRNGStream", "makeCluster", "stopCluster")
importFrom("xtable", "xtable", "print.xtable")
importFrom("MASS", "cov.rob", "ginv")
importFrom("grDevices", "rgb", "xy.coords")
importFrom("graphics", "axis", "barplot", "layout", "lines",
           "mtext", "pairs", "panel.smooth", "par", "pie", "plot",
           "points", "strwidth", "text")
importFrom("methods", "as", "is", "hasArg")
importFrom("stats", ".getXlevels", "acf", "as.formula", "coef",
           "contr.helmert", "cor", "cor.test", "cov", "density",
           "dnorm", "ecdf", "hatvalues", "lm", "lm.wfit", "mad",
           "mahalanobis", "median", "model.matrix", "model.offset",
           "model.response", "model.weights", "na.omit", "naprint",
           "optim", "optimize", "pchisq", "plot.ts", "pnorm", 
           "predict.lm", "pt", "qchisq", "qnorm", "quantile",
           "rWishart", "rnorm", "runif", "sd", "setNames", "ts", "uniroot", 
           "var", "weighted.mean", "weights")
importFrom("utils", "browseURL", "edit",
           "flush.console", "getFromNamespace", "object.size",
           "packageDescription", "read.csv",
           "read.table", "write.csv", "write.table")
importFrom("network", "network.initialize", "network.edgelist")
S3method(print, sienadata)
S3method(print, siena)
S3method(make_specification, sienadata)
S3method(make_specification, siena)
S3method(make_specification, sienaGroup)
S3method(make_constraint, sienadata)
S3method(make_constraint, siena)
S3method(make_constraint, sienaGroup)
S3method(write_report, sienadata)
S3method(print, sienaGroup)
S3method(print, sienaDependent)
S3method(print, sienaFit)
S3method(summary, sienaFit)
S3method(coef, sienaFit)
S3method(vcov, sienaFit)
S3method(test_parameter, sienaFit)
S3method(write_result, sienaFit)
S3method(print, summary.sienaFit)
S3method(write_result, sienaFit)
S3method(test_time, sienaFit)
S3method(test_gof, sienaFit)
S3method(interpret_selection, sienaFit)
S3method(interpret_influence, sienaFit)
S3method(interpret_size, sienaFit)
S3method(print, sienaAlgorithm)
S3method(print, sienaMeta)
S3method(print, summary.sienaMeta)
S3method(summary, sienaMeta)
S3method(plot, sienaMeta)
S3method(addAttributes, coCovar)
S3method(addAttributes, varCovar)
S3method(addAttributes, coDyadCovar)
S3method(addAttributes, varDyadCovar)
S3method(print, sienaTest)
S3method(print, sienaTimeTest)
S3method(summary, sienaTimeTest)
S3method(print, summary.sienaTimeTest)
S3method(plot, sienaTimeTest)
S3method(print, sienaEffects)
S3method(summary, sienaEffects)
S3method(set_effect, sienaEffects)
S3method(set_interaction, sienaEffects)
S3method(print, summary.sienaEffects)
S3method(edit, sienaEffects)
S3method(interpret_size, sienaEffects)
S3method(update_theta, sienaEffects)
S3method(update_specification, sienaEffects)
S3method(print, xtable.sienaFit)
S3method(summary, sienaGOF)
S3method(print, sienaGOF)
S3method(plot, sienaGOF)
S3method(descriptives, sienaGOF)
S3method(summary, sienaRI)
S3method(print, sienaRI)
S3method(plot, sienaRI)
S3method(summary, sienaRIDynamics)
S3method(print, sienaRIDynamics)
S3method(plot, sienaRIDynamics)
S3method(predict, iwlsm)
S3method(print, iwlsm)
S3method(print, summary.iwlsm)
S3method(summary, iwlsm)
S3method(iwlsm, formula)
S3method(iwlsm, default)
S3method(se.contrast, iwlsm)
S3method(vcov, iwlsm)
S3method(print, chains.data.frame)
S3method(print, influenceTable)
S3method(print, selectionTable)

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: phase2.r
## *
## * Description: This module contains the functions phase2.1, proc2subphase
## * and doIterations which together perform a robbins-monro stochastic
## * approximation algorithm.
## * phase2.1 and proc2subphase are called from robmon in robmon.r.
## ****************************************************************************/
## args: z: internal control object
##       x: model object (readonly as not returned)
## Hidden options exist that are activated 
## if the algorithm object x has a component x$moreUpdates or x$phase2imp

##@storeinFRANstore siena07 Used to avoid Namespace problems with multiple processes
storeinFRANstore <- function(...)
{
    FRANstore(...)
}
##@phase2.1 siena07 Start phase 2
phase2.1<- function(z, x, ...)
{
		if (ifelse(is.null(x$phase2imp),FALSE,x$phase2imp))
		{
			z$addChainToStore <- TRUE
			z$nbrNodes <- 1
			z$storedChains <- TRUE
			z$nGroup <- 1
			z$thetaList <- list()
			z$nCount <- 1
		}
	
    #initialise phase2
    if (x$maxlike)
    {
        z$phase2fras <- array(0, dim=c(4, z$pp, 1000))
        z$rejectprops <- array(0, dim=c(4, 4, 1000))
    }
    z$Phase <- 2
    z$writefreq <- 1
    if (!is.batch())
    {
        tcltk::tkconfigure(z$tkvars$earlyEndPhase2,state='normal')
        tcltk::tkconfigure(z$tkvars$subphase,state='normal')
        tcltk::tkconfigure(z$tkvars$subphaselabel,state='normal')
    }
    z$Deriv <- FALSE
	msf <- as.matrix(cov(z$sf)) # as.matrix() just in case z$pp = 1
#	z$sd <- sqrt(diag(msf))
# Instead of the preceding line,
# the following is used for equality with earlier versions.
	z$sd <- sqrt(pmax(apply(z$sf, 2, function(x) sum(x^2) / nrow(z$sf) - mean(x)^2),0))
    Report("standardization : \n", cf)
	if (!z$gmm) 
	{
   	z$sd[z$fixed] <- 0
	  z$standardization <-
						1/sqrt(pmax(diag(as.matrix(z$dinvv %*% msf %*% t(z$dinvv))),0))
	PrtOutMat( cbind(z$requestedEffects[,"effectName"], round(z$standardization,4)), cf)
	}
	else
	{
	  z$sd[which(z$fixed & !z$gmmEffects)] <- 0
	  z$standardization <- 1/sqrt(pmax(diag(as.matrix(z$dinvv %*% msf %*% t(z$dinvv))),0))
	  Report(round(z$standardization,4), cf)
	}
	if (sum(z$fixed) < z$pp)
	{
		z$sf.invcov <-
		 solve(msf[!z$fixed, !z$fixed] + 0.0001 * diag(z$pp - sum(z$fixed)))
	}
    Report(paste('\nPhase 2 has', x$nsub, 'subphases.\n'), cf)
    z$gain <- x$firstg
    z$reduceg <- x$reduceg
    if (x$nsub <= 0)
    {
        Report('With 0 subphases, there is no phase 2.\n', cf)
    }
    else
    {
        if (z$maxrepeatsubphase >= 2)
        {
            Report(c('Each subphase can be repeated up to', z$maxrepeatsubphase,
                   'times\n'), cf)
        }
        z <- proc2subphase(z, x, 1, ...)
    }
    z
}
##@proc2subphase siena07 Do one subphase of phase 2
proc2subphase <- function(z, x, subphase, ...)
{
	## init subphase of phase 2
	z <- AnnouncePhase(z, x, subphase)
	Report(paste("\nStart phase ", z$Phase, ".", subphase, "\n", sep=""), cf)
	if (subphase <= 0)
	{
		z$n2min <- 5
		z$n2max <- 100
	}
	else
	{
		z$n2min <- z$n2minimum[subphase]
		z$n2max <- z$n2maximum[subphase]
	}
	Report(paste("Number of iterations minimum ", z$n2min, ", maximum", z$n2max, ".\n", sep=""), cf)
	z$repeatsubphase <- 0
	repeat
	{
		z$repeatsubphase <- z$repeatsubphase + 1
		z$truncated <- rep(FALSE, z$n2max)
		z$positivized <- rep(0, z$pp)
		z$ctime <- proc.time()[3]
		z$time1 <- proc.time()[3]
		z$thav <- z$theta
		z$thavn <- 1
		## cat(z$thav, z$theta, '\n')
		z$prod0 <- rep(0, z$pp)
		z$prod1 <- rep(0, z$pp)
		functionAv <- 0
		## ###############################################
		## do the iterations for this repeat of this subphase
		## ##############################################
		z <- doIterations(z, x, subphase, ...)
		if (!z$OK || UserInterruptFlag() || UserRestartFlag() ||
			EarlyEndPhase2Flag())
		{
			break
		}
		##
		## end processing for this repeat of this subphase
		##
		## report truncations and positivizations
		if (any(z$truncated))
		{
			msg<- paste('Intervention 2.', subphase,
				'.1: changes truncated, iterations: ',sep='')
			Report(msg, cf)
			Report(which(z$truncated), cf, fill=80)
		}
		if (any(z$positivized >= 1))
		{
			msg <- paste('Intervention 2.',subphase,
				'.2: positivity restriction:\n ',sep='')
			Report(msg,cf)
			subs <- which((z$positivized) > 0)
			msg <- paste('Positivized: ',
				paste((1:z$pp)[subs], ': ', z$positivized[subs], '; ',
					collapse=' '))
			#            msg<- sapply(subs, function(i, y)
			#                         paste('Observation:', i, 'Coordinate(s):',
			#                               paste((1:z$pp)[y[i,]], collapse = ' ')),
			#                         y = z$positivized)
			Report(msg, cf, fill=80)
		}
		if ((subphase >= 1) && (z$maxacor >= sqrt(2.0 / (z$nit + 1))))
		{
			Report('Note: an autocorrelation is positive at the end',cf)
			Report(' of this subphase.\n',cf)
			Report('Autocorrelations:\n',cf)
			prtmat <- z$prod1 / z$prod0
			PrtOutMat(as.matrix(prtmat), cf)
		}
		if ((z$nit >= z$n2max)
			|| (subphase <= 0)
			|| (z$maxacor < 1e-10)
			|| (z$repeatsubphase >= z$maxrepeatsubphase))
		{
			break
		}
	}
	##finalize the subphase
	if (!z$OK || UserInterruptFlag() || UserRestartFlag())
	{
		return(z)
	}
	if (EarlyEndPhase2Flag())
	{
		Report('The user asked for early end of phase 2.\n', outf)
	}
	##    cat('it',z$nit,'\n')
	##recalculate autocor using -1 instead of -2 as error
	ac <- ifelse (z$prod0 > 1e-12, z$prod1 / z$prod0, -1)
	maxacor <- max(-99, ac[!z$fixed]) ##note -1 > -99
	z$theta <- z$thav / z$thavn   # z$thavn = (z$nit + 1)

	DisplayThetaAutocor(z)
	Report(paste('Phase ', z$Phase,'.', subphase, ' ended after ', z$nit,
			' iterations.\n', sep = ''), cf)
	if (x$checktime)
	{
		time1 <- proc.time()[[3]] - z$ctime
		subphaseTime <- time1 / z$nit
		Report(paste('Time per iteration in phase ', z$Phase, '.', subphase,
				' = ', format(subphaseTime, nsmall=4, digits=4),
				'\n', sep=''), lf)
	}
	if ((maxacor >= sqrt(2 / (z$nit + 1))) && (subphase >= 1))
	{
		Report('Note. Autocorrelation criterion not satisfied.\n', cf)
	}
	WriteOutTheta(z)
	if (EarlyEndPhase2Flag())
	{
		return(z)
	}
	if (subphase == 2 && z$restart) ## this means we restarted in phase 1
		##because of epsilon and need to restart the whole thing again now
	{
		Report('Restart after subphase 2.2 from current parameter values\n', cf)
		Report('because the initial values used ', cf)
		Report('led to questionable epsilon values\n', cf)
		z$fixed[z$newfixed] <- FALSE
		z$restarted <- TRUE
	}
	## For the next subphase:
	if (x$maxlike)
	{
		z$gain <- z$gain * 0.25
	}
	else
	{
		z$gain <- z$gain * z$reduceg
	}
	z
} ##end of this subphase

##@doIterations siena07 Do all iterations for 1 repeat of 1 subphase of phase 2
doIterations<- function(z, x, subphase,...)
{
	z$nit <- 0
	ac <- 0
	z$maxacor <- -1
	z$minacor <- 1
	xsmall <- NULL
	zsmall <- makeZsmall(z)
	z$returnDeps <- FALSE
	sumfra <- 0.0
	if (z$returnThetas)
	{
		thetas <- NULL
		sfs <- NULL
	}
	repeat
	{
		z$n <- z$n+1
		z$nit <- z$nit + 1
		if (z$returnThetas)
		{
			thetas <- rbind(thetas, c(subphase, z$theta))
		}
		if (subphase == 1 && z$nit == 2)
			z$time1 <- proc.time()[[3]]
		if (subphase == 1 && z$nit == 11)
		{
			time1 <- proc.time()[[3]] - z$time1
			if (time1 > 1e-5)
			{
				z$writefreq <- max(1, round(20.0 / time1))
			}
			else
			{
				z$writefreq <- 20
			}
			z$writefreq <- roundfreq(z$writefreq)
			if (is.batch())
			{
				z$writefreq <-  z$writefreq * 10 ##compensation for it
				## running faster with no tcl/tk
			}
		}
		zsmall$nit <- z$nit
		if (x$dolby) {zsmall$Deriv <- TRUE} ## include scores in FRAN
		

#Report(paste("z$theta: ", "\n"), cf)
#PrtOutMat(as.matrix(z$targets), cf)
#PrtOutMat(as.matrix(z$theta), cf)
#PrtOutMat(as.matrix(zsmall$theta), cf)


		if (z$int == 1) ## then no parallel runs at this level
		{
			if (ifelse(is.null(x$phase2imp),FALSE,x$phase2imp))
			{
				zz <- x$FRAN(zsmall, xsmall,  returnLoglik=TRUE, returnChains=TRUE)
				z$myloglik2 <- c(z$myloglik2, sum(zz$loglik))
			} else {
				zz <- x$FRAN(zsmall, xsmall)
			}
			fra <- colSums(zz$fra) - z$targets
			
			
#Report(paste("fra (1): ", "\n"), cf)
#PrtOutMat(as.matrix(fra), cf)

			if (!zz$OK)
			{
				z$OK <- zz$OK
				break
			}
			if (z$returnThetas)
			{
				sfs <- rbind(sfs, c(subphase, fra))
			}
			if (x$dolby)
				## subtract regression on scores;
				## permitted because fra is used in phase2 only for updates
			{
				ssc <- zz$sc # scores;  periods by variables
				ssc <- colSums(ssc) # add over periods
				fra <- fra - (z$regrCoef * ssc)
			}
		}
		else
		{
			zz <- clusterCall(z$cl, simstats0c, zsmall, xsmall)
			if (x$dolby)
				## subtract regression on scores;
			{
				fra <- sapply(zz, function(x){
					ssc <- x$sc # scores;  periods by variables
					ssc <- colSums(ssc) # add over periods
					colSums(x$fra)- z$targets - (z$regrCoef * ssc)
			})
			}
			else
			{
				fra <- sapply(zz, function(x) colSums(x$fra)- z$targets)
			}
			dim(fra) <- c(z$pp, z$int)
			fra <- rowMeans(fra)
			zz$OK <- sapply(zz, function(x) x$OK)
			if (!all(zz$OK))
			{
				z$OK <- FALSE
				break
			}

		}
		
		if ((z$nit <= 10) || (z$nit %% z$writefreq ==0))
		{
			DisplayIteration(z)
			if (is.batch())
			{
				val <- getProgressBar(z$pb)
				increment <- ifelse(z$nit <= 10, 1, z$writefreq)
				Report(paste('Phase ', z$Phase, ' Subphase ', subphase,
						' Iteration ', z$nit,' Progress: ',
						round((increment + val) / z$pb$pbmax * 100),
						'%\n', sep = ''))
				z$pb <- setProgressBar(z$pb, val + increment)
			}
			else
			{
				if (z$nit>1)
				{
					DisplayDeviations(z, fra)
				}
				if  (z$nit %% z$writefreq == 0)
				{
					val <- getProgressBar(z$pb)
					z$pb <- setProgressBar(z$pb, val + z$writefreq)
				}
			}
		}
		## setup check for end of phase. either store or calculate
		if (z$nit %% 2 == 1)
		{
			prev.fra <- fra
		}
		else
		{
			z$prod0 <- z$prod0 + fra * fra
			z$prod1 <- z$prod1 + fra * prev.fra
			ac <- ifelse (z$prod0 > 1e-12, z$prod1 / z$prod0, -2)
			z$maxacor <- max(-99, ac[!z$fixed]) ##note -2 > -99
			z$minacor <- min(1, ac[(!z$fixed) & ac > -1.0])
			z$ac <- ac
			if  (z$nit %% z$writefreq == 0)
			{
				DisplayThetaAutocor(z)
			}
		}
		## limit change.  Reporting is delayed to end of phase.
		# The truncation has been different from version 1.1-227 to 1.1-243,
		# due to a misunderstanding.
		# In version 1.1-244 it was changed back to the old Siena 3 way.
		# Except now the threshold is 5 instead of 10.
		# In version 1.1-285 the threshold is made a parameter from sienaModelCreate,
		# still with default 5;
		# and for the case !x$diagg, a multivariate truncation is used.

		if (x$diagg)
		{
			maxRatio <- max(ifelse(z$fixed, 1.0, abs(fra)/ z$sd), na.rm=TRUE)
			# note: this is a number, not a vector
		}
		else
		{
			if (z$pp > sum(z$fixed))
			{
			  if(!z$gmm)
			  {
			    maxRatio <-
			      max(sqrt((t(fra[!z$fixed]) %*% z$sf.invcov %*% fra[!z$fixed]) /
			                 (z$pp-sum(z$fixed))))			    
			  }
			  else
			  {
			    maxRatio <-
			      max(sqrt((t(fra[!z$fixed & !z$gmmEffects]) %*% z$sf.invcov %*% fra[!z$fixed & !z$gmmEffects]) /
			                 (z$pp-sum(!z$fixed & !z$gmmEffects)))) 
			   }
				# the max() is just to turn the (1,1) matrix into a number.
			}
			else
			{
				maxRatio <- 1.0
			}
		}
		if ((is.na(maxRatio)) || (is.nan(maxRatio)))
		{
			maxRatio <- 1.0
		}
		if (maxRatio > x$truncation)
		{
			fra <- x$truncation*fra/maxRatio
			z$truncated[z$nit] <- TRUE
		}
		if (subphase > x$doubleAveraging)
		{
			sumfra <- sumfra + fra
			fra <- sumfra
		}

#Report(paste("fra: ", "\n"), cf)
#PrtOutMat(as.matrix(fra), cf)

		if (x$standardizeVar)
		{
			if (x$diagg)
			{
				changestep <- fra / z$sd
			}
			else
			{
				if (x$standardizeWithTruncation)
				{
				  if (!z$gmm)
				  {
				    changestep <- (as.vector(z$dinvv %*% fra))*pmin(z$standardization, 1)
				  }
				  else
				  {
				    change <- (as.vector(z$dinvv %*% fra))*pmin(z$standardization, 1)
				    changestep <- rep(0,z$pp)
				    changestep[!z$gmmEffect] <- change
				  }
				}
				else
				{
				  if (!z$gmm)
				  {
				    changestep <- (as.vector(z$dinvv %*% fra))*z$standardization
				  }
				  else
				  {
				    changestep <- (as.vector(z$dinvv %*% fra))*z$standardization
				    changestep <- rep(0,z$pp)
				    changestep[!z$gmmEffect] <- change
		
				  }
				}
			}
		}
		else
		{
			if (x$diagg & !z$gmm)
			{
				changestep <- fra / diag(z$dfra)
			}
		  	else if (z$gmm)
		  	{
			    change <- (as.vector(z$dinvv %*% fra))
			    changestep <- rep(0,z$pp)
			    changestep[!z$gmmEffect] <- change 
			}
			else
			{
				changestep <- as.vector(z$dinvv %*% fra)
			}
		}
		changestep[z$fixed] <- 0.0
		fchange <- as.vector(z$gain * changestep)

		## check positivity restriction
		fchange[is.na(fchange)] <- 0
		z$positivized[fchange > z$theta] <- z$positivized[fchange > z$theta] +1
		z$positivized[!z$posj] <- 0
		fchange <- ifelse(z$posj & (fchange > z$theta), z$theta * 0.5, fchange)
		# make update step
		if (subphase > x$doubleAveraging)
		{
			zsmall$theta <- (z$thav/z$thavn) - fchange
		}
		else
		{
				zsmall$theta <- zsmall$theta - fchange
		}
		z$theta <- zsmall$theta
		z$thav <- z$thav + zsmall$theta
		z$thavn <- z$thavn + 1
 
#Report(paste("thavs: ", round(z$thavn), "\n"), cf)
#PrtOutMat(as.matrix(z$thav), cf)
		
		if (any(!z$fixed))
		{
			if (max(abs(z$theta[!z$fixed])) > z$thetaBound)
			{
				cat("The update steps have led to a parameter with maximum absolute value", 
						max(abs(z$theta[!z$fixed])), 
						",\nwhich is larger than thetaBound =", z$thetaBound, ".\n")
				larger <- rep("", length(z$theta))
				larger[((!z$fixed)&(abs(z$theta > z$thetaBound)))] <- " *****"
				print(cbind(z$requestedEffects$effectName, round(z$theta, 4), larger), quote=FALSE)
				if (interactive())
				{
					cat("If you wish to continue estimation in this session,")
					cat("\ngive a higher value for thetaBound.\n")
					thetaBound0 <- z$thetaBound
					z$thetaBound <- as.numeric(readline(prompt="Give a number: "))
					if (is.na(z$thetaBound)) 
					{
						stop("You gave no numeric answer, therefore the estimation stops.")
					}
					if (z$thetaBound > thetaBound0)
					{
						cat("OK, estimation continues.\n")
						flush.console()
					}
					else
					{
						stop("You gave a lower number, therefore the estimation stops.")
					}
				}
				else
				{
					stop("thetaBound should be set higher.")
				}
			}
		}

# This is a hidden option: 
# it is activated if the algorithm object x has a component x$moreUpdates
		if (x$maxlike && !is.null(x$moreUpdates) && x$moreUpdates > 0)
		{
			z <- doMoreUpdates(z, x, x$moreUpdates * subphase)
			zsmall$theta <- z$theta
		}
	
		if (ifelse(is.null(x$phase2imp),FALSE,x$phase2imp))
		{
			z$thetaList[[z$nCount]] <- z$theta
			z$nCount <- z$nCount + 1
		}
		
		##check for user interrupt
		CheckBreaks()
		if (UserInterruptFlag() || UserRestartFlag() || EarlyEndPhase2Flag())
		{
			break
		}
		## do we stop?
		if (!(is.na(z$minacor) || is.na(z$maxacor)))
		{
			if ((z$nit >= z$n2min && z$maxacor < 1e-10)||
				(z$nit >= z$n2max)
			|| ((z$nit >= 50) && (z$minacor < -0.8) &&
				(z$repeatsubphase < z$maxrepeatsubphase)))
			{
				break
			}
		}
	}
	if (ifelse(is.null(x$phase2imp),FALSE,x$phase2imp))
	{
		cat(z$nit)
		z$nAll <- c(z$nAll, z$nit)
		if (subphase==x$nsub)
		{
			z <- stdError(z,x,subphase=4, ...)
		}
	}
	
	if (z$returnThetas)
	{
		z$thetas <- rbind(z$thetas, thetas)
		z$sfs <- rbind(z$sfs, sfs)
	}
	z
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: siena08.r
# *
# * Description: This module contains the code for the meta analysis of a
# * collection of Siena fits.
# *****************************************************************************/

##@estimate_meta_saom new name for siena08
meta_siena <- function(..., outputName = "sienaMeta", bound = 10, alpha = 0.05, maxit=20)
{
	siena08(..., projname = outputName, bound = bound, alpha = alpha, maxit=maxit)
}


##@siena08 siena08
siena08 <- function(..., projname="sienaMeta", bound=10, alpha=0.05, maxit=20)
{
	fitList <- (!inherits(list(...)[[1]], 'sienaFit'))
	if (fitList)
	{
		dots <- (list(...))[[1]]
	}
	else
	{
    dots <- as.list(substitute(list(...)))[-1] ##first entry is the word 'list'
	}
    if (length(dots) == 0)
    {
        stop('need some sienafits')
    }
    nm <- names(dots)
    if (is.null(nm))
    {
        fixup <- seq(along=dots)
    }
    else
    {
        fixup <- nm == ''
    }
	if (fitList)
	{
		listName <- deparse(substitute(fitList))
# Check if it looks like a pipe chain 
		if (any(grepl("%>%|\\|>", listName)))
		{
			stop("Cannot auto-generate filename from piped data. Please apply this function to an object with a name.") 
		}
		dep <- paste(listName, seq(along=fitList), sep='')
	}
	else
	{
    dep <- sapply(dots[fixup], function(x) deparse(x)[1])
		dots <- list(...)
	}
    if (is.null(nm))
    {
        nm <- dep
    }
    else if (length(dep) > 0)
    {
        nm[fixup] <- dep
    }
    names(dots) <- nm
    if (any(duplicated(nm)))
    {
        stop('names must be unique')
    }
    ex <- dots
    projnames <- sapply(ex, function(x)x$x$projname)
    if (any(duplicated(projnames)))
    {
        projnames <- paste(projnames, "(", names(ex), ")", sep="")
    }

    ##first make a data frame
    mydf <- do.call(rbind, lapply(1:length(ex), function(i, y)
                              {
                                  x <- y[[i]]
                                  n <- length(x$theta)
                                  scoretests <- rep(NA, length(x$theta))
                                  if (!is.null(x$testresulto))
                                  {
                                      scoretests[x$test] <-
                                          x$testresulto[!is.na(x$testresulto)]
                                  }
                                  data.frame(projname=rep(projnames[i], n),
                                             theta=x$theta,
                                             effects=
                                             paste(format(x$requestedEffects$type,
                                                          width=5),
                                                   x$requestedEffects$effectName,
                                                   sep=": "),
                                             tconv=x$tconv,
                                             version=rep(x$version, n),
                                             scoretests=scoretests,
                                             ## add anything more needed
                                             ## for the report
                                             se=sqrt(diag(x$covtheta)))

                              }, y=ex
                                  )
                    )
    ## make sure the effects are in the right order.
    mydf$effects <- factor(mydf$effects, levels=unique(mydf$effects))

    ## produce meta analysis object
    dometa <- function(x)
    {
        ##tidy up the data frame x first so theta and sj2 match in length
        ##not actually necessary for iwlsm, as model.frame deals with it.
        ## but easier for the remainder...
        x1 <- x[!is.na(x$theta) & !is.na(x$se) & x$se < bound,]
        if (any(x1$theta != 0))
        {
            if (sum((x1$se < bound)) >= 3)
            {
                suppressWarnings(check.correl <- cor.test(x1$theta, x1$se,
                                                          method="spearman"))
                ## warnings will be given in case of ties, not important here
            }
            else
            {
                check.correl <- data.frame(estimate=NA, p.value=NA,
                                           method="no correlation test")
            }
            regfit <- iwlsm(theta ~ 1, psi=psi.iwlsm, data=x1,
                            ses=x1$se^2, maxit=maxit)
            regfit$terms <- NA
            regfit$model <- NULL
            regfit$psi <- NULL
            ## symbols ttilde, Qstat, Tsq as in Snijders & Baerveldt (2003),
            ##(18), (17), (15)
            Tsq <- sum((x1$theta / x1$se)^2)
            regsummary <- summary(regfit)
            tratio <- regsummary$coef[1, 3]
            ttilde <- sum(x1$theta / x1$se^2) / sqrt(sum(1 / x1$se^2))
            Qstat <- Tsq - ttilde^2
            cjplus <- -2 * sum(pnorm(x1$theta / x1$se, lower.tail=FALSE,
                                     log.p=TRUE))
            cjminus <- -2 * sum(pnorm(x1$theta / x1$se, log.p=TRUE))
            cjplusp <- 1 - pchisq(cjplus, 2 * nrow(x1))
            cjminusp <- 1 - pchisq(cjminus, 2 * nrow(x1))
            ## ML estimates and confidence intervals
            maxxlik <- maxlik(x1$theta, x1$se)
            cmu  <- confint.mu(x1$theta, x1$se, alpha)
            csig <- confint.sig(x1$theta, x1$se, alpha)
            ret1 <- list(cor.est=check.correl$estimate,
                         cor.pval=check.correl$p.value,
                         cor.meth=check.correl$method,
                         regfit=regfit, regsummary=regsummary,
                         Tsq=Tsq, pTsq=1 - pchisq(Tsq, nrow(x1)),
                         tratio=tratio,
                         ptratio=2 * pnorm(abs(tratio), lower.tail=FALSE),
                         Qstat=Qstat,
                         pttilde=1 - pchisq(Qstat, nrow(x1) - 1),
                         cjplus=cjplus, cjminus=cjminus,
                         cjplusp=cjplusp, cjminusp=cjminusp, n1=nrow(x1),
                         mu.ml=maxxlik$mu, sigma.ml=maxxlik$sigma,
                         mu.ml.se=maxxlik$se.mu,
                         mu.confint=cmu, sigma.confint=csig)
        }
        else
        {
            ret1 <- NULL
            ret1$n1 <- 0
        }
        if (any(!is.na(x$scoretests)))
        {
            n <- sum(!is.na(x$scoretests))
            cjplus <- -2 * sum(pnorm(x$scoretests, lower.tail=FALSE,
                                     log.p=TRUE), na.rm=TRUE)
            cjminus <- -2 * sum(pnorm(x$scoretests, log.p=TRUE),
                                na.rm=TRUE)
            cjplusp <- 1 - pchisq(cjplus, 2 * n)
            cjminusp <- 1 - pchisq(cjminus, 2 * n)
            ret1$scoreplus <- cjplus
            ret1$scoreminus <- cjminus
            ret1$scoreplusp <- cjplusp
            ret1$scoreminusp <- cjminusp
            ret1$ns <- n
        }
        else
        {
            ret1$ns <- 0
        }
        ret1
    }

    meta <- by(mydf, mydf$effects, dometa)
# copy effect specification and ML estimates
# to components requestedEffects, theta and se
	requestedEffects <- ex[[1]]$requestedEffects
	if (dim(requestedEffects)[1] != length(unique(mydf$effects))){
		warning('\nWarning: length requestedEffects incorrect.\n')
		print(requestedEffects)
		print(mydf$effects)
	}
	meta.theta <- sapply(meta,
					function(x){ifelse(is.null(x$mu.ml),NA,x$mu.ml)})
	meta.theta[is.na(meta.theta)] <- 0
	meta.se <- sapply(meta,
					function(x){ifelse(is.null(x$mu.ml.se),NA,x$mu.ml.se)})
	names(meta.theta) <- rep('', length(meta.theta))
	names(meta.se) <- rep('', length(meta.se))
# Put the IWLS estimates easily accessible on the meta object
	neff <- length(unique((mydf$effects))) # number of effects
	muhat <- rep(NA, neff)
	se.muhat <- rep(NA, neff)
	for (i in 1:neff)
	{
	if (!is.null(meta[[i]]$regsummary))
		{
			muhat[i] <- meta[[i]]$regsummary$coefficients[1, 1]
			se.muhat[i] <- meta[[i]]$regsummary$coefficients[1, 2]
		}
	}
	meta$muhat <- muhat
	meta$se.muhat <- se.muhat
# add everything to the meta object created
    meta$thetadf <- mydf
    class(meta) <- "sienaMeta"
	attr(meta, "version") <- packageDescription(pkgname, fields = "Version")
    meta$projname <- projname
    meta$bound <- bound
    ## count the score tests
    meta$scores <- by(mydf, mydf$effects,
					function(x){any(!is.na(x$scoretests))})
	meta$requestedEffects <- requestedEffects
	meta$theta <- meta.theta
	meta$se <- meta.se
	meta$startingDate <- date()
    meta
}



## methods

##@print.sienaMeta Methods
print.sienaMeta <- function(x, file=FALSE, reportEstimates=FALSE, ...)
{
    exitfn <- function()
    {
        if (file)
        {
            ## close the report file
            Report(closefiles=TRUE)
        }

    }
    on.exit(exitfn())
    projname <- x$projname
    if (file)
    {
        Report(openfiles=TRUE, type="a", projname=projname) # initialise a file
    }
    else
    {
        Report(openfiles=TRUE, type="n") #initialise with no file
    }
    ## projnames <- unique(x$thetadf$projname)
    ## nProjects <- length(projnames)
    effects <- unique(x$thetadf$effects)
    ## nEffects <- length(effects)
    ## results

    ## estimates

    Report(c("\nTests for mean parameters use a t-distribution with N-1 d.f.,\n" ,
	         "where N = number of included groups.\n"), sep="", outf)
    Report(c("\nUpper bound used for standard error is",
             format(round(x$bound, 4), width=9, nsmall=2), ".\n"), sep="", outf)

    dashes <- paste(rep("-", 80), collapse="")
    x$thetadf$excl <- ifelse(x$thetadf$se > x$bound,
                             " EXCLUDED from meta-analysis", "")
    by(x$thetadf, x$thetadf$effects, function(x, y)
   {
       i <- match(x$effects[1], effects)
       y <- y[[effects[i]]]
       Report(c("\n", dashes, "\nParameter ", i, ": ",
                as.character(x$effects[1]), "\n", dashes, "\n"),
              sep="", outf)
	   if (reportEstimates)
	   {
       tmp <- paste("Data set ", 1:nrow(x), ", ", format(x$projname),
                    " :  Estimate ",
                    format(round(x$theta, 4), width=12),
                    " (standard error ",
                    format(round(x$se, 4), nsmall=4,
                           width=12), ")", x$excl, "\n", sep="")
       Report(c(tmp, "\n"), sep="", outf)
	   }
       Report(c(" ", y$n1, " datasets used.\n\n"), sep="", outf)
       if (y$n1 > 0)
       {
           Report("Test that estimates and standard errors are uncorrelated",
                  outf)
           if (is.na(y$cor.est))
           {
               Report("\ncannot be performed.\n\n", outf)
           }
           else
           {
               Report(c(": \n", y$cor.meth, " =", format(round(y$cor.est, 4),
                                                         width=9),
                        ", two-sided ",reportp(y$cor.pval,3), "\n\n"), sep="",
                      outf)
           }
           Report(c("Estimates and test based on IWLS modification of",
                    "Snijders & Baerveldt (2003)\n"),
                  outf)
           Report(c("---------------------------------------------------",
                    "-------------------------\n"), sep="",
                  outf)
           Report(c("Test that all parameters are 0 : \n"), outf)
           Report(c("chi-squared =", format(round(y$Tsq, 4), width=9),
                    ", d.f. = ", y$n1, ", ",
                    reportp(y$pTsq, 3), "\n\n"), sep="", outf)
           Report(c("Estimated mean parameter",
                    format(round(y$regsummary$coefficients[1, 1], 4), width=9),
                    " (s.e.", format(round(y$regsummary$coefficients[1, 2], 4),
                                     width=9), "), two-sided ",
                    reportp(2 * pt(-abs(y$regsummary$coefficients[1, 3]),
                                   y$n1 - 1), 3), "\n\n"), sep="", outf)
           Report(c("Estimated standard deviation",
                    format(round(y$regsummary$stddev, 4), width=9)), outf)
           Report("\nTest that variance of parameter is 0 :\n", outf)
           Report(c("Chi-squared = ", format(round(y$Qstat, 4), width=9),
                    " (d.f. = ", y$n1-1, "), ", reportp(y$pttilde, 3),
                    "\n\n"), sep="", outf)
           Report(c("Estimates and confidence intervals under normality",
                    "assumptions\n"),
                  outf)
           Report(c("-------------------------------------------------------",
                    "-------\n"), outf)
           Report(c("Estimated mean parameter",
                    format(round(y$mu.ml, 4), width=9),
                    " (s.e.",format(round(y$mu.ml.se, 4), width=9),
                    "), two-sided ",
                    reportp(2 * pt(-abs(y$mu.ml/y$mu.ml.se),
                                   y$n1 - 1), 3), "\n"), sep="", outf)
           Report(c(format(round(y$mu.confint[3], 2), width=4),
                    "level confidence interval [",
                    format(round(y$mu.confint[1], 4), width=7),
                    ",",
                    format(round(y$mu.confint[2], 4), width=7), "]\n"), outf)
           Report(c("Estimated standard deviation",
                    ifelse((y$sigma.ml > 0.0001) | (y$sigma.ml < 0.0000001),
                           format(round(y$sigma.ml, 4), width=9), " < 0.0001"),
                     "\n"), outf)
           Report(c(format(round(y$sigma.confint[3], 2), width=4),
                    "level confidence interval [",
                    format(round(y$sigma.confint[1], 4), width=7),
                    ",",
                    format(round(y$sigma.confint[2], 4), width=7), "]\n\n"), outf)
           Report("Fisher's combination of one-sided tests\n", outf)
           Report("----------------------------------------\n", outf)
           Report("Combination of right one-sided p-values:\n", outf)
           Report(c("Chi-squared = ", format(round(y$cjplus, 4), width=9),
                    " (d.f. = ", 2 * y$n1, "), ", reportp(y$cjplusp, 3),
                    "\n"), sep="", outf)
           Report("Combination of left one-sided p-values:\n", outf)
           Report(c("Chi-squared = ", format(round(y$cjminus, 4), width=9),
                    " (d.f. = ", 2 * y$n1, "), ", reportp(y$cjminusp, 3),
                    "\n"), sep="", outf)
       }
       else
       {
           Report(c("There were no data sets satisfying the bounds for",
                    "this parameter.\n No combined output is given.\n"), outf)
       }
   }, y=x)
    ##score tests
    if (any(x$scores))
    {
        Report(c("\n\n", paste(rep("-", 65), collapse=""),
                 "\nScore tests:\nFisher combination\n",
                 paste(rep("-", 65), collapse=""), "\n"), sep="", outf)

        by(x$thetadf, x$thetadf$effects, function(x, y)
       {
           i <- match(x$effects[1], effects)
           y <- y[[effects[i]]]
           if (y$ns > 0)
           {
               Report(c("\n", "(", i, ")   ",
                        as.character(x$effects[1]), "\n"), sep="", outf)
               tmp <- paste("Data set ", 1:nrow(x), ", ", format(x$projname),
                            " : z = ", ifelse(is.na(x$scoretests), "NA",
                                              format(round(x$scoretests, 4),
                                                     width=12)),
                            "\n", sep="")
               Report(c(tmp, "\n"), sep="", outf)
               Report("Combination of right one-sided p-values:\n", outf)
               Report(c("Chi-squared = ", format(round(y$scoreplus, 4),
                                                 width=9),
                        " (d.f. = ", 2 * y$ns, "), ",
                        reportp(y$scoreplusp, 3), "\n"), sep="", outf)
               Report("Combination of left one-sided p-values:\n", outf)
               Report(c("Chi-squared = ",
                        format(round(y$scoreminus, 4), width=9),
                        " (d.f. = ", 2 * y$ns, "), ",
                        reportp(y$scoreminusp, 3), "\n"), sep="", outf)
           }
       }, y=x)
    }
    invisible(x)
}

##@reportp Miscellaneous
reportp <- function(p, ndec)
{
    thr <- exp(-ndec * log(10.0));
    if (is.na(p))
    {
        "p undefined"
    }
    else if (p > thr)
    {
        c("p = ", format(round(p, ndec), width=ndec+2, scientific=FALSE,
                         nsmall=ndec))
    }
    else
    {
        c("p < ", format(round(thr, ndec), width=ndec+2, scientific=FALSE,
                         nsmall=ndec))
    }

}

##@plot.sienaMeta Methods
plot.sienaMeta <- function(x, ..., which = 1:length(x$theta), useBound=TRUE, layout = c(2,2))
{
    ## library(lattice)
	if (useBound)
	{
		usedLines <- is.na(x$thetadf$scoretests) & (x$thetadf$se < x$bound)
	}
	else
	{
		usedLines <- is.na(x$thetadf$scoretests)
	}
    tmp <- xyplot(theta ~ se|effects[which],
                  data=x$thetadf[usedLines,],
                  ylab="estimates",
                  xlab="standard errors", layout=layout,
                  panel=function(x, y)
              {
                  panel.xyplot(x, y)
                  panel.abline(0, qnorm(0.025))
                  panel.abline(0, qnorm(0.975))
              },
                  prepanel=function(x,y)
              {   list(xlim=c(min(0,min(x)),max(0,max(x))),
                       ylim=c(min(0,min(y)),max(0,max(y))))
              },
                scales="free")
    tmp[!sapply(tmp$y.limits, function(x)all(is.na(x)))]
}

##@summary.sienaMeta Methods
summary.sienaMeta <- function(object, file=FALSE, extra=TRUE, ...)
{
    object$file <- file
    object$extra <- extra
    class(object) <- c("summary.sienaMeta", class(object))
    object
}

##@print.summary.sienaMeta Methods
print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
{
	exitfn <- function()
	{
		if (file)
		{
			## close the report file
			Report(closefiles=TRUE)
		}

	}
	on.exit(exitfn())
	if (!is.null(x$file))
	{
		file <- x$file
	}
	if (!is.null(x$extra))
	{
		extra <- x$extra
	}
	projname <- x$projname
	if (file)
	{
		Report(openfiles=TRUE, type="a", projname=projname) # initialise a file
	}
	else
	{
		Report(openfiles=TRUE, type="n") #initialise with no file
	}
	if (file)
	{
		## do some sums for the heading

		namelen <- nchar(projname) + 4
		##linelen <- 80
		astlen <- min(namelen + 10, 80)
		nBlanks <- if (astlen < 80)
		{
			(80 - astlen) %/% 2
		}
			else
			{
				0
			}
		nBlanks2 <- if (namelen < 80)
		{
			(80 - namelen) %/% 2
		}
			else
			{
				0
			}
		Report(c(rep(" ", nBlanks), rep("*", astlen), "\n"), sep="", outf)
		Report(c(rep(" ", nBlanks2), projname, ".txt\n"), sep="", outf)
		Report(c(rep(" ", nBlanks), rep("*", astlen), "\n"), sep="", outf)
		Report(c("Filename is ", projname, ".txt.\n\n"), sep="", outf)
		Report(c("This file contains primary output for SIENA project <<",
				projname, ">>.\n\n"), sep="", outf)
		Report(c("Date and time:", format(Sys.time(),
					"%d/%m/%Y %X"), "\n\n"), outf)
		packageValues <- packageDescription(pkgname,
			fields=c("Version", "Date"))
		Report(c(paste(pkgname, "version "), packageValues[[1]], " (",
				format(as.Date(packageValues[[2]]), "%d %m %Y"), ")\n\n"), sep="", outf)
	}
	Report(c("================================= SIENA08 ",
			"================================================\n",
			"Multilevel use of Siena algorithms according to ",
			"Snijders & Baerveldt (2003) with extension\n",
			"=================================================",
			"=========================================\n\n"), sep="", outf)
	projnames <- unique(x$thetadf$projname)
	nProjects <- length(projnames)
	effects <- unique(x$thetadf$effects)
	nEffects <- length(effects)
	Report(c("Number of projects in the list is " ,
			length(projnames), ".\n"), sep="", outf)
	Report("The names of these projects are :\n", outf)
	tmp <- paste("project", 1:nProjects, ": <", projnames,
		">\n", sep="")
	Report(tmp, sep="", outf)
	Report(c("\nOptions for running Siena08:\n",
			"-> Parameters are excluded from the meta-analysis when their ",
			"standard\n", "   error exceeds an upper bound of ",
			round(x$bound,digits=2), ".\n"), sep="", outf)
	if (extra)
	{
		Report("-> Extra output requested\n", outf)
	}
	else
	{
		Report("-> No extra output requested\n", outf)
	}
	## RSiena version
	Report(c("\nThe RSiena Version of the first fit object is ",
			x$thetadf$version[1], ".\n\n"), sep="", outf)
	## project names
	by(x$thetadf, x$thetadf$projname, function(x)
		{
			Report(c("Object <", x$projname[1], "> contains estimates of ",
					nrow(x), " parameters.\n"), sep="", outf)
			Report(c("The number of valid score tests found was ",
					sum(is.na(x$scoretests)), ".\n"),
				sep="", outf);
		})
	##parameters:
	Report(c("\nA total of", nEffects, "parameters in", nProjects,
			"projects :\n"),   outf)
	Report(paste(format(1:length(effects)), ". " , effects, "\n", sep=""),
		sep="", outf)
	Report(c("\nThe projects contain the parameters as follows",
			"(1=present, 0=absent):\n\n"), outf)
	row1 <- (1:nEffects)
	rows <- do.call(rbind, tapply(x$thetadf$effects,
			x$thetadf$projname, function(x)
			{
				as.numeric(effects %in% x)
			}
			)
		)
	rows <- format(rbind(row1, rows), width=3)
	row2 <- rep(paste(rep("-", nchar(rows[1, 1])), collapse=""), nEffects)
	rows <- rbind(rows[1,], row2, rows[-1, ])
	col1 <- format(rbind("Project",
			paste(rep("-", 7), collapse=""),
			cbind(1:nProjects)), justify="centre")
	col2 <- format(rbind( "|",  "--+--",  cbind(rep("|", nProjects))),
		justify="centre")
	rows <- cbind(col1, col2, rows)
	Report(t(rows), sep = c(rep.int("", ncol(rows) - 1), "\n"), outf)
	by(x$thetadf, x$thetadf$projname, function(x)
		{
			Report(c("\nProject", x$projname[1], "\n"), outf)
			tmp <- paste("par.", 1:nEffects, "estimate",
				format(round(x$theta, 4), width=12), "(s.e. ",
				format(round(x$se, 4), width=11), ") (conv_t",
				format(round(x$tconv, 4), width=9), ")\n")
			Report(format(tmp), sep="", outf)
			Report(c("\nMaximal absolute convergence t-statistic = ",
					format(round(max(abs(x$tconv)), 4), width=9), "\n"), outf)
			## score tests
		})
	## results

	## estimates
	Report(c("\n\n", paste(rep("=", 29), collapse=""),
			"\nResults of the meta-analysis:\n",
			paste(rep("=", 29), collapse=""), "\n"), sep="", outf)

	Report(c("\nUpper bound used for standard error is",
			format(round(x$bound, 4), width=9, nsmall=2), ".\n"), sep="", outf)

	dashes <- paste(rep("-", 80), collapse="")
	x$thetadf$excl <- ifelse(x$thetadf$se > x$bound,
		" EXCLUDED from meta-analysis", "")
	by(x$thetadf, x$thetadf$effects, function(x, y)
		{
			i <- match(x$effects[1], effects)
			y <- y[[effects[i]]]
			Report(c("\n", dashes, "\nParameter ", i, ": ",
					as.character(x$effects[1]), "\n", dashes, "\n"),
				sep="", outf)
			tmp <- paste("Data set ", 1:nrow(x), ", ", format(x$projname),
				" :  Estimate ",
				format(round(x$theta, 4), width=12),
				" (standard error ",
				format(round(x$se, 2), nsmall=2,
					width=11), ")", x$excl, "\n", sep="")
			Report(c(tmp, "\n"), sep="", outf)
			Report(c(" ", y$n1, " datasets used.\n\n"), sep="", outf)
			if (y$n1 > 0)
			{
				if (extra)
				{
					Report(c("IWLS modification of Snijders-Baerveldt (2003) method ",
							"of combining estimates"), outf)
					Report(c("\n--------------------------------------------",
							"---------------------------------\n"), sep="", outf)
					Report(c("This method assumes that true parameters and",
							" standard errors are uncorrelated.\n",
							"This can be checked by the plot method ",
							"and the test below.\n\n"), sep="", outf)
				}
				Report("Test that estimates and standard errors are uncorrelated",
					outf)
				if (is.na(y$cor.est))
				{
					Report("\ncannot be performed.\n\n", outf)
				}
				else
				{
					Report(c(": \n", y$cor.meth, " =", format(round(y$cor.est, 4),
								width=9),
							", two-sided ",reportp(y$cor.pval,3), "\n\n"),
						sep="", outf)
				}
				Report(c("Test that all parameters are 0 : \n"), outf)
				Report(c("chi-squared =", format(round(y$Tsq, 4), width=9),
						", d.f. = ", y$n1, ", ",
						reportp(y$pTsq, 3), "\n\n"), sep="", outf)
				Report(c("Estimated mean parameter",
						format(round(y$regsummary$coefficients[1, 1], 4), width=9),
						" (s.e.", format(round(y$regsummary$coefficients[1, 2], 4),
							width=9), "), two-sided ",
						reportp(2*pt(-abs(y$regsummary$coefficients[1, 3]),
								y$n1 - 1), 3), "\n"), sep="", outf)
				Report(c("based on IWLS modification of Snijders & Baerveldt (2003). ",
						"\n\n"), sep="", outf)
				Report(c("Residual standard error",
						format(round(y$regsummary$stddev, 4), width=9)), outf)
				Report("\nTest that variance of parameter is 0 :\n",outf)
				Report(c("Chi-squared = ", format(round(y$Qstat, 4), width=9),
						" (d.f. = ", y$n1-1, "), ", reportp(y$pttilde, 3),
						"\n"), sep="", outf)
				Report(c("based on IWLS modification of Snijders & Baerveldt (2003).",
						"\n\n"), sep="", outf)

				Report(c("Estimates and confidence intervals under normality",
                    "assumptions\n"), outf)
				Report(c("-------------------------------------------------------",
                    "-------\n"), outf)
				Report(c("Estimated mean parameter",
                    format(round(y$mu.ml, 4), width=9),
                    " (s.e.",format(round(y$mu.ml.se, 4), width=9),
                    "), two-sided ",
                    reportp(2 * pt(-abs(y$mu.ml/y$mu.ml.se),
                                   y$n1 - 1), 3), "\n"), sep="", outf)
				Report(c(format(round(y$mu.confint[3], 2), width=4),
                    "level confidence interval [",
                    format(round(y$mu.confint[1], 4), width=7),
                    ",",
                    format(round(y$mu.confint[2], 4), width=7), "]\n"), outf)
				Report(c("Estimated standard deviation",
                    ifelse((y$sigma.ml > 0.0001) | (y$sigma.ml < 0.0000001),
                           format(round(y$sigma.ml, 4), width=9), " < 0.0001"),
                     "\n"), outf)
				Report(c(format(round(y$sigma.confint[3], 2), width=4),
                    "level confidence interval [",
                    format(round(y$sigma.confint[1], 4), width=7),
                    ",",
                    format(round(y$sigma.confint[2], 4), width=7), "]\n\n"), outf)

				Report("Fisher's combination of one-sided tests\n", outf)
				Report("----------------------------------------\n", outf)
				Report("Combination of right one-sided p-values:\n", outf)
				Report(c("Chi-squared = ", format(round(y$cjplus, 4), width=9),
						" (d.f. = ", 2 * y$n1, "), ", reportp(y$cjplusp, 3),
						"\n"), sep="", outf)
				Report("Combination of left one-sided p-values:\n", outf)
				Report(c("Chi-squared = ", format(round(y$cjminus, 4), width=9),
						" (d.f. = ", 2 * y$n1, "), ", reportp(y$cjminusp, 3),
						"\n"), sep="", outf)
			}
			else
			{
				Report(c("There were no data sets satisfying the bounds for",
						"this parameter.\n No combined output is given.\n"), outf)
			}
		}, y=x)
	##score tests
	if (any(x$scores))
	{
		Report(c("\n\n", paste(rep("-", 65), collapse=""),
				"\nScore tests:\nFisher combination\n",
				paste(rep("-", 65), collapse=""), "\n"), sep="", outf)

		invisible(by(x$thetadf, x$thetadf$effects, function(x, y)
				{
					i <- match(x$effects[1], effects)
					y <- y[[effects[i]]]
					if (y$ns > 0)
					{
						Report(c("\n", "(", i, ")   ",
								as.character(x$effects[1]), "\n"), sep="", outf)
						tmp <- paste("Data set ", 1:nrow(x), ", ", format(x$projname),
							" : z = ",
							ifelse(is.na(x$scoretests), "NA",
								format(round(x$scoretests, 4), width=12)),
							"\n", sep="")
						Report(c(tmp, "\n"), sep="", outf)
						Report("Combination of right one-sided p-values:\n", outf)
						Report(c("Chi-squared = ", format(round(y$scoreplus, 4),
									width=9),
								" (d.f. = ", 2 * y$ns, "), ",
								reportp(y$scoreplusp, 3), "\n"), sep="", outf)
						Report("Bonferroni combination of left and right one-sided p-values:\n",
							outf)
						Report(c(reportp(2*min(y$scoreminusp, y$scoreplusp), 3), "\n"),
							sep="", outf)
					}
				}, y=x))
	}
}

funnelPlot <- function(anslist, k, threshold=NULL, origin=TRUE,
				plotAboveThreshold=TRUE, verbose=TRUE, ...)
{
	if (!inherits(anslist, 'list'))
	{
		stop('this function needs a list of sienaFit objects.')
	}
	if (!inherits(anslist[[1]], 'sienaFit'))
	{
		stop('this function needs a list of sienaFit objects.')
	}
	if (k <= 0)
	{
		stop('k should be positive.')
	}
	if (k > min(sapply(anslist, function(x){x$pp})))
	{
		stop(paste('k is too large, should be less than or equal to ',
			min(sapply(anslist, function(x){x$pp})),'.', sep=''))
	}
	use <- sapply(anslist, function(x){!x$fixed[k]})
	if (sum(use, na.rm=TRUE) == 0)
	{
		stop(paste('no estimations of parameter ',k,'.', sep=''))
	}
	notUsed <- sum(!use, na.rm=TRUE)
	efNames <- sapply(anslist, function(x){x$effects$effectName[k]})
	if (!all(efNames == efNames[1]))
	{
		warning('not all tested effect names are the same.')
	}
	use0 <- use
	th0 <- sapply(anslist, function(x){x$theta[k]}, USE.NAMES=FALSE)
	se0 <- sapply(anslist, function(x){x$se[k]}, USE.NAMES=FALSE)
	th <- th0[use]
	se <- se0[use]
	if (is.null(threshold))
	{
		threshold <- (1.1 * range(se, na.rm=TRUE)[2]) + 0.01
	}
	use <- use[se < threshold]
	tooLarge <- sum(se >= threshold, na.rm=TRUE)
	use <- (se < threshold)
	xxlim <- 1.1*range(th[use], na.rm=TRUE)
	yylim <- c(0,1.1*range(se[use], na.rm=TRUE)[2])
	if ((plotAboveThreshold) & any(!use))
	{
		yylim[2] <- threshold+ 0.05
	}
	if (origin)
	{
		xxlim[1] <- min(xxlim[1], 0)
		xxlim[2] <- max(xxlim[2], 0)
	}
	plot(th[use], se[use], xlim=xxlim, ylim=yylim, main=efNames[1],
			xlab='theta', ylab='se', ...)
	ma <- xxlim[2]
	mi <- xxlim[1]
	lines(c(0,ma), c(0, ma/1.96), col='red')
	lines(c(0,mi), c(0,  mi/(-1.96)), col='red')
	if ((plotAboveThreshold) & any(!use))
	{
		points(th[!use], rep(threshold, sum(!use, na.rm=TRUE)), pch=8)
	}
	if ((notUsed > 0) & verbose)
	{
		cat(notUsed, 'groups had no estimation of parameter', k,'.\n')
		cat('Table of parameter values for these groups : \n')
	#	print(round(cbind((th0[!use0]), se0[!use0]), 4))
		t0 <- table(round(th0[!use0], 4))
		t <- matrix(t0, 1, length(t0))
		colnames(t) <- names(t0)
		rownames(t) <- 'count'
		print(t)
		cat('\n')
	}
	if ((tooLarge > 0) & verbose)
	{
		cat(tooLarge, 'groups had standard errors larger than', threshold,'.\n')
		cat('Rounded parameter values for these groups : \n')
		t <- as.matrix(round(rbind(th[!use], se[!use]), 2))
		rownames(t) <- c('par', 'se')
		print(t)
		flush.console()
	}
	invisible(cbind(th[use],se[use]))
}

meta.table <- function(x, d=3, option=2,
	filename=paste(deparse(substitute(x)),'_global.tex',sep=""), align=TRUE)
{
# Produces three latex tables with summaries of sienaMeta object:
# with normality assumptions: parameters;
# with normality assumptions: confidence intervals;
# and the Fisher right and left combinations.
# d is number of digits after decimal point.
	code <- ifelse(align, "r@{.}l", "c")
	sepsign <- ifelse(align, "&", ".")
	numdig <- ifelse(align, 2, 1)
	num2dig <- ifelse(align, 4, 2)
    fromObjectToLaTeX <- function(a, mw=NULL){
		b <- as.character(a)
		b <- gsub('->', '$\\rightarrow$', fixed=TRUE, b)
		b <- gsub('<-', '$\\leftarrow$', fixed=TRUE, b)
		b <- gsub('<>', '$\\leftrightarrow$', fixed=TRUE, b)
		b <- gsub('=>', '$\\Rightarrow$', fixed=TRUE, b)
		b <- gsub('>=', '$\\geq$', fixed=TRUE, b)
		b <- gsub('<=', '$\\leq$', fixed=TRUE, b)
# Note: R changes \\ to \ but still displays \\ in printing the string.
		b <- gsub('^(1/1)', '', fixed=TRUE, b)
		b <- gsub('^(1/2)', '(sqrt)', fixed=TRUE, b)
		b <- gsub('^', '', fixed=TRUE, b)
		b <- gsub('_', '-', fixed=TRUE, b)
		b <- gsub('#', '.', fixed=TRUE, b)
		b <- gsub('&', '.', fixed=TRUE, b)
		format(b, width=mw)
	}
	if (is.null(x$startingDate))
	{
		startdate <- NULL
	}
	else
	{
		startdate <- x$startingDate
	}
	max.eff.width <- max(sapply(x$thetadf$effects,
				function(x){nchar(as.character(fromObjectToLaTeX(x)))}))
# header
	line <- c(paste("% Table based on sienaMeta object",
                               deparse(substitute(x))))
	cat("\n",line, "\n", file=filename, append = TRUE)
	cat("% combined sienaFit objects", file=filename, append = TRUE)
	line <- as.character(unique(x$thetadf$projname))
	cat("\n%",line, "\n", file=filename, append = TRUE)
	line <- c(paste("Estimation date",startdate))
	cat("\n%",line, "\n", file=filename, append = TRUE)
	neff <- length(unique((x$thetadf$effects))) # number of effects
	nscores <- 0
	if (option == 1)
 {
# begin table 1
 	cat("Without assumptions of normality \\\\ \n", file=filename, append = TRUE)
	cat("\n", sep="", file=filename, append = TRUE)
	line <- "\\begin{tabular}{l r"
	for (i in 1:7) {line <- paste(line, code)}
	line <- paste(line,"}")
	cat(line, "\n", sep="", file=filename, append = TRUE)
	cat("Effect & $N$ & \\multicolumn{", numdig, "}{c}{$T^2$} &  \\multicolumn{",
					numdig, "}{c}{($p_{T^2}$)} ",
					file=filename, append = TRUE)
	cat(" &  \\multicolumn{", numdig, "}{c}{$\\hat\\mu_k$} &  \\multicolumn{", numdig, "}{c}{(s.e.)}",
					file=filename, append = TRUE)
	cat(" &  \\multicolumn{", numdig, "}{c}{$\\hat\\sigma_k$} &  \\multicolumn{", numdig, "}{c}{$Q$}",
					file=filename, append = TRUE)
	cat(" &  \\multicolumn{", numdig, "}{c}{$p_{Q}$}", file=filename, append = TRUE)
	cat(" \\\\", "\n", sep="", file=filename, append=TRUE)
	cat("\\hline \n", file=filename, append=TRUE)
# body 1
	for (i in 1:neff)
	{
	line <- paste(fromObjectToLaTeX(x$thetadf$effects[i], max.eff.width), "&")
	line <- paste(line, x[[i]]$n1, " & ")
	if (x[[i]]$n1 >= 2)
	{
		line <- paste(line, formatC(x[[i]]$Tsq, digits=1, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " &  (", formatC(x[[i]]$pTsq, digits=3, format="f",
					decimal.mark=sepsign), ")", sep="")
		line <- paste(line, " & ", formatC(x[[i]]$regsummary$coefficients[1, 1], digits=d, format="f",
				decimal.mark=sepsign), sep="")
		line <- paste(line, " & \ (",
				formatC(x[[i]]$regsummary$coefficients[1, 2], digits=d, format="f",
				decimal.mark=sepsign), ")", sep="")
		line <- paste(line, " & ", formatC(x[[i]]$regsummary$stddev, digits=d, format="f",
				decimal.mark=sepsign), sep="")
		line <- paste(line, " & ", formatC(x[[i]]$Qstat, digits=d, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " &  (",
				formatC(x[[i]]$pttilde, digits=3, format="f", decimal.mark=sepsign), ")", sep="")
	}
	cat(line,"\\\\\n", file=filename, append = TRUE)
	nscores <- nscores + x[[i]]$ns
	}
# tailer 1
	cat("\\end{tabular}\n", file=filename, append=TRUE)
	if (nscores > 0)
	{
	cat("% There also were score tests.\\\\ \n", file=filename, append=TRUE)
	}
	cat("\\bigskip\n\n", file=filename, append=TRUE)
  }
  	if (option == 2)
  {
# begin table 2
	cat("With assumptions of normality \\\\ \n", file=filename, append = TRUE)
	cat("\n", sep="", file=filename, append = TRUE)
	line <- "\\begin{tabular}{l r"
	for (i in 1:8) {line <- paste(line, code)}
	line <- paste(line,"}")
	cat(line, "\n", sep="", file=filename, append = TRUE)
	cat("Effect & $N$ & \\multicolumn{", numdig, "}{c}{$T^2$} &  \\multicolumn{",
			numdig, "}{c}{($p_{T^2}$)} ",
			file=filename, append = TRUE)
	cat(" &  \\multicolumn{", numdig, "}{c}{$\\hat\\mu_k$} &  \\multicolumn{", num2dig,
			"}{c}{(conf. int.)}",
			file=filename, append = TRUE)
	cat(" &  \\multicolumn{", numdig, "}{c}{$\\hat\\sigma_k$} &  \\multicolumn{", num2dig,
			"}{c}{(conf. int.)}",
			file=filename, append = TRUE)
	cat(" \\\\", "\n", sep="", file=filename, append=TRUE)
	cat("\\hline \n", file=filename, append=TRUE)
	neff <- length(unique((x$thetadf$effects))) # number of effects
# body 2
	for (i in 1:neff)
	{
	line <- paste(fromObjectToLaTeX(x$thetadf$effects[i], max.eff.width), "&")
	line <- paste(line, x[[i]]$n1, " & ")
	if (x[[i]]$n1 >= 2)
	{
		line <- paste(line, formatC(x[[i]]$Tsq, digits=1, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " & (", formatC(x[[i]]$pTsq, digits=3, format="f",
				decimal.mark=sepsign), ")", sep="")
		line <- paste(line, " & ", formatC(x[[i]]$mu.ml, digits=d, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " & (", formatC(x[[i]]$mu.confint[1], digits=d, format="f",
				decimal.mark=sepsign),
					", &  ", formatC(x[[i]]$mu.confint[2], digits=d, format="f",
					decimal.mark=sepsign),")", sep="")
		line <- paste(line, " & ", formatC(x[[i]]$sigma.ml, digits=d, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " & (", formatC(x[[i]]$sigma.confint[1],
					digits=d, format="f", decimal.mark=sepsign),
						", & ", formatC(x[[i]]$sigma.confint[2],
						digits=d, format="f", decimal.mark=sepsign),")", sep="")
	}
	cat(line,"\\\\\n", file=filename, append = TRUE)
	nscores <- nscores + x[[i]]$ns
	}
# tailer 2
	cat("\\end{tabular}\n", file=filename, append=TRUE)
	if (nscores > 0)
	{
	cat("% There also were score tests.\n", file=filename, append=TRUE)
	}
	cat("\\bigskip \n\n\n", file=filename, append=TRUE)
  }

	if (option == 3)
  {
# header 3
	cat("With assumptions of normality \\\\ \n", file=filename, append = TRUE)
	cat("\n", sep="", file=filename, append = TRUE)
	line <- "\\begin{tabular}{l r"
	for (i in 1:7) {line <- paste(line, code)}
	line <- paste(line,"}")
	cat(line, "\n", sep="", file=filename, append = TRUE)
	cat("Effect & $N$ & \\multicolumn{", numdig, "}{c}{$T^2$} &  \\multicolumn{",
			numdig, "}{c}{($p_{T^2}$)} ",
			file=filename, append = TRUE)
	cat(" &  \\multicolumn{", numdig, "}{c}{$\\hat\\mu_k$} &  \\multicolumn{", numdig, "}{c}{(s.e.)}",
			file=filename, append = TRUE)
	cat(" &  \\multicolumn{", numdig, "}{c}{$\\hat\\sigma_k$} &  \\multicolumn{", numdig, "}{c}{$Q$}",
			file=filename, append = TRUE)
	cat(" &  \\multicolumn{", numdig, "}{c}{($p$)} \\\\ \n", file=filename, append = TRUE)
	cat(" \\\\", "\n", sep="", file=filename, append=TRUE)
	cat("\\hline \n", file=filename, append=TRUE)
# body 3
	for (i in 1:neff)
	{
	line <- paste(fromObjectToLaTeX(x$thetadf$effects[i], max.eff.width), "&")
	line <- paste(line, x[[i]]$n1, " & ")
	if (x[[i]]$n1 >= 2)
	{
		line <- paste(line, formatC(x[[i]]$Tsq, digits=1, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " & (", formatC(x[[i]]$pTsq, digits=3, format="f",
					decimal.mark=sepsign), ")", sep="")
		line <- paste(line, " & ", formatC(x[[i]]$mu.ml, digits=d, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " & (",
				formatC(x[[i]]$mu.ml.se, digits=d, format="f", decimal.mark=sepsign), ")", sep="")
		line <- paste(line, " & ", formatC(x[[i]]$sigma.ml, digits=d, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " & ", formatC(x[[i]]$Qstat, digits=d, format="f", decimal.mark=sepsign), sep="")
		line <- paste(line, " &  (",
				formatC(x[[i]]$pttilde, digits=3, format="f", decimal.mark=sepsign), ")", sep="")
	}
	cat(line,"\\\\\n", file=filename, append = TRUE)
	}
# tailer 3
	if (nscores > 0)
	{
	cat("% There also were score tests.\n", file=filename, append=TRUE)
	}
	cat("\\end{tabular}\\\\ \n", file=filename, append=TRUE)
	cat("\\bigskip \n\n", file=filename, append=TRUE)
 }

# header 4
	cat("Fisher combinations \\\\ \n", file=filename, append = TRUE)
	cat("\n", sep="", file=filename, append = TRUE)
	line <- "\\begin{tabular}{l "
	for (i in 1:2) {line <- paste(line, code)}
	line <- paste(line,"}")
	cat(line, "\n", sep="", file=filename, append = TRUE)
	cat("Effect & \\multicolumn{", numdig, "}{c}{$p$ (right-sided)} &  \\multicolumn{",
			numdig, "}{c}{$p$ (left-sided)} \\\\ \n", file=filename, append = TRUE)
	cat("\\hline \n", file=filename, append=TRUE)
# body 4
	for (i in 1:neff)
	{
	line <- paste(fromObjectToLaTeX(x$thetadf$effects[i], max.eff.width), "&")
	line <- paste(line, formatC(x[[i]]$cjplusp, digits=3, format="f", decimal.mark=sepsign),
					" & ", sep="")
	line <- paste(line, formatC(x[[i]]$cjminusp, digits=3, format="f", decimal.mark=sepsign),
					sep="")
	cat(line,"\\\\\n", file=filename, append = TRUE)
	}
# tailer 4
	cat("\\end{tabular}\n\n\n", file=filename, append=TRUE)
	if (filename > "")
	{
		cat('Results written to file', filename,'.\n')
	}
}

##*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: sienaTimeTest.r
## *
## * Description: This file contains the sienaTimeTest code for testing the
## * significance of additional time dummies interacted with effects, and
## * sienaTimeFix, which is called to set up time dummy interacted effects.
## ****************************************************************************/

test_time <- function(x, ...) UseMethod("test_time")

##@test_time.sienaFit method for sienaFit
test_time.sienaFit <- function(x, effects=NULL, excludedEffects=NULL,
			condition=FALSE, ...)
{
	sienaTimeTest(x, effects, excludedEffects, condition)
}

##@sienaTimeTest siena07 Does test for time homogeneity of effects
sienaTimeTest <- function (x, effects=NULL, excludedEffects=NULL,
	condition=FALSE)
{
	if (!inherits(x, "sienaFit"))
	{
		stop("not a legitimate sienaFit object")
	}
	if (!x$OK)
	{
		stop("Error end of estimation algorithm")
	}
	else if (x$termination == "UserInterrupt")
	{
		stop("User interrupted run, object possibly incomplete")
	}
	if (!is.null(x$gmm))
	{
		if (x$gmm)
		{
			stop("sienaTimeTest cannot be applied to results of GMoM estimation")
		}
	}
	if (!is.null(x$sf2.byIteration))
	{
		if (!x$sf2.byIteration)
		{
			stop("sienaTimeTest needs sf2 by iterations (use lessMem=FALSE)")
		}
	}
	waveNumbers <- attr(x$f, "periodNos")
	nWaves <- length(waveNumbers)

	fitEffects <- x$requestedEffects

	## There must be more than 2 periods (more than 1 wave)
	## to do a time test!
	if (nWaves < 2)
	{
		stop("You must have at least two time periods (three waves) to test ",
			"for non-heterogeneity across time.")
	}
	## if this is a maximum likelihood fit or finite differences we need
	## the optional objects sdf2
	if (x$maxlike || x$FinDiff.method)
	{
		if (is.null(x$sdf2[[1]][[1]]))
		{
			stop("rerun siena07 with the byWave option TRUE")
		}
	}
	## get the desired effects
	if (!is.null(effects))
	{
		## a little validation helps ensure we have at least one effect we want
		if (any(!is.numeric(effects)))
		{
			stop("non numeric effect number requested")
		}
		if (any(!effects %in% 1:nrow(fitEffects)))
		{
			stop("Some effect number requested not found")
		}
		use <- 1:nrow(fitEffects) %in% effects
		if (any(fitEffects$basicRate[use]))
		{
			stop("siena time tests are inappropriate for basic rates")
		}
		#        if (any(grepl("Dummy", fitEffects$effectName[use])))
		#        {
		#            stop("siena time tests are inappropriate for time dummy effects")
		#        }
	}
	else
	{
		use <- !fitEffects$basicRate
	}

	## if (x$maxlike || x$FinDiff.method)
	## {
	##     stop("Not yet implemented for finite differences or maxlike")
	## }
	## Identify the effects which will potentially be tested
	baseInFit <- use & !grepl("Dummy", fitEffects$effectName)

	nBaseEffects <- sum(baseInFit)
	if (nBaseEffects == 0)
	{
		stop("No effects available to test")
	}

	fixedDummies <- fitEffects$shortName=='egoX' &
		fitEffects$fix & grepl("Dummy", fitEffects$effectName)

	## establish effects for top left of derivative matrix D
	estimatedInFit <- use &  !fixedDummies
	topleftEffectNumbers <- fitEffects$effectNumber[estimatedInFit]

	nRowsToTest <- nBaseEffects * nWaves

	## construct the data frame that will be the basis for all calculations
	toTest <- data.frame(baseEffect=rep(1:nBaseEffects, each=nWaves),
		effectNumber=rep(fitEffects$effectNumber[baseInFit],
			each=nWaves),
		type=rep(fitEffects$type[baseInFit], each=nWaves),
		period = rep(waveNumbers, nBaseEffects),
		period1 = rep(1:nWaves, nBaseEffects),
		toTest=rep(TRUE, nRowsToTest),
		baseRowInD=rep(0, nRowsToTest),
		effectName=rep(fitEffects$effectName[baseInFit],
			each=nWaves),
		rowInD=rep(0, nRowsToTest))
	toTest$toTest[toTest$period == 1] <- FALSE
	toTest$rowInD <- match(toTest$effectNumber,
		topleftEffectNumbers, nomatch=0)
	toTest$baseRowInD <- toTest$rowInD
	toTest$rowInD[toTest$period > 1]  <- 0
	## Go through each effect which is an estimated time dummy, and
	## incorporate this information into the toTest data frame. i.e.
	## if a time dummy was estimated, set
	## its element in toTest equal to FALSE so that we do not time test it
	for (i in which(grepl("isDummy", fitEffects$timeDummy) & estimatedInFit))
	{
		tmp <- toString(fitEffects$timeDummy[i])
		tmp <- strsplit(tmp, split=",", fixed=TRUE)[[1]]
		if (any(tmp != ""))
		{
			## Dont test the dummy for the corresponding effect
			thisRow <- toTest$effectNumber == tmp[3] & toTest$period == tmp[2]
			toTest[thisRow, "toTest"] <- FALSE
			## We want to be able to reference this effect given an
			## index for the base effect and a time period, so store
			## this information in rowInD -- this is used
			## extensively in plot.sienaTimeTest
			toTest[thisRow, "rowInD"] <-
				match(fitEffects$effectNumber[i],
					topleftEffectNumbers)
		}
	}	
	##  nEffects, nSims, nameslist, nDummies convert commonly used ingredients
	##  from x into an easily accessed form based on the screens
	##  set up above
	nEffects <- sum(!toTest$toTest)
	## add row indices in D to the rows to be tested
	toTest$rowInD[toTest$toTest] <- nEffects + 1:sum(toTest$toTest)
	toTest <- toTest[order(toTest$rowInD), ]

	nSims <- x$Phase3nits
	nameslist <- list(Iteration=paste("it", 1:nSims, sep=""),
		Wave=paste("Wave", waveNumbers, sep=""),
		Effect=fitEffects$effectName[estimatedInFit]
		)
	nDummies <- sum(toTest$toTest)
	ttt <- toTest$toTest
	type <- ifelse(toTest$type == "eval", "",
		paste(" (", toTest$type, ")", sep=""))
	toTest$dummyNames[!ttt] <- paste(fitEffects$effectName[estimatedInFit],
		type[!ttt], sep="")
	toTest$dummyNames[ttt] <- paste("(*)Dummy", toTest$period[ttt], ":",
		toTest$effectName[ttt], type[ttt], sep="")

	## obsStats, moment, scores are the crucial ingredients from x which
	## screen for the base effects and make the rest of the code clean
	## be careful as the original logical vectors cannot be used once rows
	## have been extracted.
	obsStats <- t(x$targets2[estimatedInFit, , drop=FALSE])
	moment <- x$sf2[, , estimatedInFit, drop=FALSE] -
		rep(obsStats, each=nSims)
	G <- array(0, dim=c(nSims, nWaves, nEffects + nDummies))
	## Set the base effects G equal to the moments from x
	G[, , 1:nEffects] <- moment
	## copy over the others
	subs1 <- cbind(rep(1:nSims, nDummies), rep(toTest$period1[ttt], each=nSims),
		rep(toTest$rowInD[ttt], each=nSims))
	subs2 <- cbind(rep(1:nSims, nDummies), rep(toTest$period1[ttt], each=nSims),
		rep(toTest$baseRowInD[ttt], each=nSims))
	G[subs1] <- moment[subs2]
	## Put names onto G for easy? reference
	## Use dimnames(G) <- list(NULL, NULL) to view the data as a matrix!
	dimnames(G) <- list(nameslist$Iteration, nameslist$Wave, toTest$dummyNames)
	## Make the covariance matrix for the new moments
	sigma <- cov(apply(G, c(1, 3), sum))
	if (length(sigma) == 1)
	{
		sigma <- matrix(sigma,1,1)
	}
	if (!(x$maxlike || x$FinDiff.method))
	{
		scores <- x$ssc[ , , estimatedInFit, drop=FALSE]
		SF <- array(0, dim=c(nSims, nWaves, nEffects + nDummies))
		SF[, , 1:nEffects] <- scores
		SF[subs1] <- scores[subs2]
		## Put names onto SF for easy reference
		dimnames(SF) <- dimnames(G)
		D <- derivativeFromScoresAndDeviations(SF, G, , , , TRUE, )
	}
	else
	{
		if (x$FinDiff.method)
		{
			##x$sdf2 is a list of arrays, one for each iteration
			derivs <- t(sapply(x$sdf2, function(xr)
					xr[, estimatedInFit, estimatedInFit, drop=FALSE]))
		}
		else
		{
			##x$sdf2 is a list of lists of Matrix objects
			derivs <-
				t(sapply(x$sdf2, function(xx)
						array(t(sapply(xx, function(xr)
									as.matrix(xr[estimatedInFit,
										estimatedInFit,
										drop=FALSE]))),
						dim=c(nWaves, nEffects, nEffects))))
		}
		derivs <- array(derivs, dim=c(nSims, nWaves, nEffects, nEffects))
		DF <- array(0, dim=c(nSims, nWaves, nEffects + nDummies,
				nEffects + nDummies))
		DF[, , 1:nEffects, 1:nEffects] <- derivs
		for (wave in 2:nWaves)
		{
			thisWave <- toTest$period == wave & toTest$toTest
			subs1 <- (1: (nEffects + nDummies))[thisWave]
			subs2 <- toTest$baseRowInD[thisWave]
			DF[, wave, subs1, subs1] <- derivs[, wave, subs2, subs2]

			DF[, wave, subs1, 1:nEffects] <- derivs[, wave, subs2, 1:nEffects]
			DF[, wave, 1:nEffects, subs1] <- derivs[, wave, 1:nEffects, subs2]
		}

		D <- t(apply(DF, c(3, 4), mean))
	}

	## The following parts about excluding effects were added later
	## (preceding code is mainly from JL; parts about excluding are from TS).
	## This leads to some duplication in functionality;
	## but since the preceding construction works well,
	## I (TS) wanted to keep it mostly as it was/is.

	## Check for fixed effects and exclude them from testing.
	if (is.null(excludedEffects))
	{
		excludedEffects <- which(fitEffects$fix)
	}
	else
	{
		excludedEffects <-
			unique(c(excludedEffects, which(fitEffects$fix)))
	}

	## In case some effects are excluded from testing, this is done now:
	if ((!is.null(excludedEffects)) & (length(excludedEffects) > 0))
	{
		useRows <- rep(TRUE, dim(toTest)[1])
		for (i in seq_along(excludedEffects))
		{
			useRows[toTest$baseRowInD==excludedEffects[i]] <- FALSE
		}
		useRows[1:nBaseEffects] <- TRUE
		toTest <- toTest[useRows,]
		D <- D[useRows, useRows]
		sigma <- sigma[useRows, useRows, drop=FALSE]
		nDummies <- sum(toTest$toTest)
		G <- G[,,useRows]
		nRowsToTest <- dim(toTest)[1]
		exclusions <- 1:nBaseEffects %in% excludedEffects
	}
	else
	{
		exclusions <- rep(FALSE, nBaseEffects)
	}

	## Check if there are any collinearities in the effects and dummies;
	## note that time dummies added by sienaTimeFix were already excluded.
	rankSigma <- qr(sigma)$rank
	extraExclusions <- rep(FALSE, nBaseEffects)
	if (rankSigma < dim(sigma)[1])
	{
		es <- ifelse((nRowsToTest - nDummies == 1), '', 's')
		message("TimeTest constructed a null hypothesis with ",
							nRowsToTest - nDummies, " estimated parameter", es)
		message("and ", nDummies," dummy variables to be tested.")
		message("However, there are ", dim(sigma)[1] - rankSigma,
							" linear dependencies between these.\n")
		message("This may be because some of the parameters are already")
		message("interactions with time dummies or other time variables.\n")
		message("Look in the help page for the use of the <effects> parameter.\n")

		subset0 <- rep(FALSE, dim(sigma)[1])
		subset0[1:nBaseEffects] <- TRUE
		for (i in 1:nBaseEffects)
		{
			if (!(i %in% excludedEffects))
				## Check if the dummies for this effect
				## are dependent on the base effects.
			{
				subset <- subset0 | (toTest$baseRowInD==i)
				if (qr(sigma[subset,subset])$rank
					< dim(sigma[subset,subset])[1])
				{
					extraExclusions[i] <- TRUE
				}
			}
		}
		if (any(extraExclusions))
		{
			useRows <- rep(TRUE, dim(toTest)[1])
			for (i in 1:nBaseEffects)
			{
				if (extraExclusions[i])
				{
					useRows[toTest$baseRowInD==i] <- FALSE
				}
				useRows[1:nBaseEffects] <- TRUE
			}
			toTest <- toTest[useRows,, drop=FALSE]
			D <- D[useRows, useRows, drop=FALSE]
			sigma <- sigma[useRows, useRows, drop=FALSE]
			nDummies <- sum(toTest$toTest)
			G <- G[,,useRows, drop=FALSE]
			nRowsToTest <- dim(toTest)[1]
			excludedNumber <- sum(extraExclusions)
			es <- ifelse((excludedNumber == 1), '', 's')
			efNames <- fitEffects[baseInFit,"effectName"]
	cat("Automatic discovery of dependencies yielded the exclusion of effect", 
							es, sep="")
			if (excludedNumber <= 1)
			{			
				cat(" ", efNames[extraExclusions],".\n ")
			}
			else
			{
				cat(" ", paste(efNames[extraExclusions], collapse=", "),".\n ")
			}
			rankSigma <- qr(sigma)$rank
			if (rankSigma < dim(sigma)[1])
			{
				cat("After these exclusions, there still are linear dependencies.\n")
				cat("Advice: use test_time with a smaller set of effects.\n")
				cat("Look in the help page for the use of the <effects> parameter.\n\n")
				stop("Please rerun test_time with appropriate excluded effects.")
			}
		}
		else
		{
			cat("Automatic discovery of dependencies had no effect.\n")
			cat("Advice: use test_time with a smaller set of effects.\n")
			cat("Look in the help page for the use of the <effects> parameter.\n\n")
			stop("Please rerun test_time with appropriate excluded effects.")
		}
	}
	## We have now set up all of the ingredients properly, so we may proceed
	## with the score type test of Schweinberger (2007-2012)
	fra <- apply(G, 3, sum) / nSims
	doTests <- toTest$toTest
	redundant <- rep(FALSE, length(doTests))
	jointTest <- ScoreTest(nrow(toTest), D, sigma, fra, doTests, redundant,
		maxlike=x$maxlike)
	jointTestP <- 1 - pchisq(jointTest$testresOverall, nDummies)
	if (! condition)
	{
		individualTest <- jointTest$testresulto[1:nDummies]
	}
	else
	{
		individualTest <- sapply(1:nDummies, function (i)
			{
				doTests <- rep(FALSE, nEffects + nDummies)
				doTests[nEffects + i] <- TRUE
				test <- ScoreTest(nrow(toTest), D, sigma,
					fra, doTests, redundant, maxlike=x$maxlike)
				test$testresulto[1]
			}
			)
	}
	individualTestP <- 2 * (1- pnorm(abs(individualTest)))
	if (!is.na(jointTestP))
	{
		rownames(jointTestP) <- "Joint Significant Test"
		colnames(jointTestP) <- "p-Val"
	}
	thetaOneStep <- c(x$theta[estimatedInFit, drop=FALSE], rep(0, nDummies)) +
		jointTest$oneStep
	## Define the null hypothesis for the later tests (effect-wise and group-wise)
	## as the null hypothesis tested in the overall test:
	nullHyp <- 	!toTest$toTest
	effectTest <- as.vector(by(toTest, toTest$baseEffect, function (xr)
				{
					doTests <- rep(FALSE, nEffects + nDummies)
					if (any(xr$toTest))
					{
						if (!condition)
						{
							## Control for all estimated effects.
							doTests[toTest$baseEffect == (xr$baseEffect[1]) &
								toTest$toTest] <- TRUE
							testresult <- partial.scoreTest(D, sigma, fra,
								doTests, nullHyp, maxlike=x$maxlike)
							testresult
						}
						else
						{
							## Control for everything.
							doTests[toTest$baseEffect == (xr$baseEffect[1]) &
								toTest$toTest] <- TRUE
							redundant <- rep(FALSE, length(doTests))
							test <- ScoreTest(nEffects + nDummies, D, sigma,
								fra, doTests, redundant, maxlike=x$maxlike)
							test$testresOverall
						}
					}
					else
					{
						NA
					}
				}
				))
	dim(effectTest) <- c(length(effectTest), 1)
	effectTestP <- matrix(NA, dim(effectTest)[1], 3)
	effectTestP[,1] <- round(effectTest[,1],2)
	effectTestP[,2] <- tapply(toTest$toTest, toTest$baseEffect, sum)
	effectTestP[,3] <- round(1 - pchisq(effectTestP[,1], effectTestP[,2]), 3)
	rownames(effectTestP) <- toTest$dummyNames[toTest$period == 1]
	colnames(effectTestP) <- c("chi-sq.", "df", "p-value")
	pvalues <-
		round(c(2 * (1 -
					pnorm(abs(x$theta[estimatedInFit] /
							sqrt(diag(x$covtheta)[estimatedInFit])))),
				individualTestP), 3)
	thetaStar <- cbind(c(x$theta[estimatedInFit], rep(0, nDummies)),
		thetaOneStep, pvalues)
	colnames(thetaStar) <- c("Initial Est.", "One Step Est.", "p-Value")
	rownames(thetaStar) <- dimnames(G)[[3]]
	## put things on toTest to make plot easier
	toTest[, c("InitialEst", "OneStepEst", "p.value")] <- thetaStar
	toTest$effectTest <- NA
	toTest$effectTest <- effectTestP[toTest$baseEffect, 1]
	type <- ifelse(toTest$type =="eval", "", paste(" (",
			as.character(toTest$type), ")", sep=""))
	toTest$effectName <-
		factor(paste(toTest$effectName, type,
				" \n(chisq=", toTest$effectTest, ")", sep=""))
	toTest$valsplus <- toTest$OneStepEst +
		ifelse(toTest$period == 1, 0, toTest$OneStepEst[toTest$baseEffect])
	toTest$dummysd <- abs(toTest$OneStepEst / qnorm(1 - toTest$p.value / 2))
	toTest$dummysd[toTest$period == 1] <-
		sqrt(diag(x$covtheta))[estimatedInFit][toTest$period == 1]

	# Added by Tom: groupwise tests.
	cef <- attr(x$f,"condEffects")
	if (inherits(cef,"sienaGroupEffects"))
	{
		# This is a multi-group result
		groupsKind <- "Group"
		groupSizes <- attr(x$f,"groupPeriods") - 1
		nGroups <- x$f$nGroup
		if (nGroups != length(groupSizes)) {stop("No good group sizes 1.")}
		groups <- rep(1:nGroups, groupSizes)
		if (nWaves != length(groups)) {stop("No good group sizes 2.")}
		toTest$whichGroup <- groups[toTest$period1]
	}
	else
	{
		groupsKind <- "Period"
		nGroups <- x$observations-1
		toTest$whichGroup <- toTest$period
	}
	groupTest <-
		as.vector(by(toTest, toTest$whichGroup, function (xr)
				{
					doTests <- rep(FALSE, nEffects + nDummies)
					if (any(xr$toTest))
					{
						doTests[toTest$whichGroup == (xr$whichGroup[1]) &
							toTest$toTest] <- TRUE
						testresult <- partial.scoreTest(D, sigma, fra, doTests,
							nullHyp, maxlike=x$maxlike)
						testresult
					}
					else
					{
						NA
					}
				}
				))
	# The preceding does not yield the test for the first group/period,
	# because it was used as the reference category.
	# This is a test for the joint contribution 
	# of the dummies for the other groups.
	# Therefore matrix A is created as the design matrix for this test.	
	remainingEffects <- unique(toTest$baseEffect[toTest$toTest])
	nRemainingEffects <- length(remainingEffects)
	if (nRemainingEffects >= 1)
	{
		A <- matrix(0, nRemainingEffects, nRowsToTest)
		for (i in seq_along(remainingEffects))
		{
			A[i, which((toTest$baseEffect==remainingEffects[i]) & (!nullHyp))] <- 1
		}		
		groupTest[1] <- transformed.scoreTest(D, sigma, fra, A,
			nullHyp, maxlike=x$maxlike)
		dim(groupTest) <- c(nGroups, 1)
		groupTestP <- matrix(NA, nGroups, 3)
		groupTestP[,1] <- round(groupTest[,1], 2)
		groupTestP[,2] <- tapply(toTest$toTest, toTest$whichGroup, sum)
		groupTestP[1,2] <- nRemainingEffects
		groupTestP[,3] <- round(1 - pchisq(groupTest[,1], groupTestP[,2]), 3)
		rownames(groupTestP) <- paste(groupsKind, 1:nGroups)
		colnames(groupTestP) <- c("chi-sq.", "df", "p-value")
	}
	else
	{
		groupTest[1] <- NA
		groupTestP <- NA
		cat('No first dummy tested ...\n')
	}
	sienaFitName <- deparse(substitute(x))
# Check if it looks like a pipe chain 
	if (grepl("%>%|\\|>", sienaFitName)) 
	{
		sienaFitName <- "sienaFitObject"
	}
	returnObj <- list(
		sienaFitName=sienaFitName,
		JointTest=jointTestP,
		EffectTest=effectTestP,
		IndividualTest=thetaStar,
		JointTestStatistics=jointTest,
		EffectTestStatistics=effectTest,
		IndividualTestStatistics=individualTest,
		GroupTest=groupTestP,
		GroupTestStatistics=groupTest,
		GroupsKind=groupsKind,
		CovDummyEst=jointTest$covMatrix,
		Moments=G,
		Deriv=D,
		BaseRowInD=match(which(baseInFit),
			which(estimatedInFit)),
		Waves=dim(G)[2],
		Sims=dim(G)[1],
		Effects=dim(G)[3],
		DummyStdErr=sqrt(diag(jointTest$covMatrix)),
		OriginalEffects=nEffects,
		OriginalThetaStderr=
			sqrt(diag(x$covtheta))[estimatedInFit],
		ToTest=toTest,
		ScreenedEffects=which(!use),
		RequestedExclusions=exclusions,
		ExtraExclusions=extraExclusions,
		WaveNumbers=waveNumbers,
		IndividualTestsOrthogonalized=condition
		)
	class(returnObj) <- "sienaTimeTest"
	attr(returnObj, "version") <- packageDescription(pkgname, fields = "Version")
	returnObj
}
##@transformed.scoreTest siena07 score test for part of the additional effects
transformed.scoreTest <- function(dfra, msf, fra, A, nullHyp, maxlike)
	# score test of A %*% fra;
	# Boolean vector nullHyp defines vector of estimated effects
	# non-zero columns of A should not overlap with nullHyp
{
	test <- colSums(abs(A)) > 0
	if (any(test & nullHyp))
	{
		stop("Null hyp and test in transformed score test should not overlap")
	}
	if (dim(A)[1] + sum(nullHyp) < 2)
	{
		stop("Meaningless transformed score test")
	}
	A1 <- matrix(0, sum(nullHyp), dim(dfra)[1])
	A1[cbind(1:sum(nullHyp), which(nullHyp))] <- 1
	AA <- rbind(A1,A)
	D <- AA %*% dfra %*% t(AA)
	sig <- AA %*% msf %*% t(AA)
	g <- AA %*% fra
	tested <- rep(FALSE, dim(AA)[1])
	tested[(sum(nullHyp)+1):dim(AA)[1]] <- TRUE
	redundant <- rep(FALSE, dim(AA)[1])
	EvaluateTestStatistic(maxlike, tested, redundant, D, sig, g)$cvalue
}
##@partial.scoreTest siena07 score test for part of the additional effects
partial.scoreTest <- function(dfra, msf, fra, test, nullHyp, maxlike)
	# test gives vector of tested effects,
	# nullHyp gives vector of estimated effects
{
	if (any(test & nullHyp))
	{
		stop("Null hypothesis and test in partial score test should not overlap")
	}
	if (!any(test)){stop("Nothing to be tested in partial score test")}
	A <- matrix(0, sum(test), dim(dfra)[1])
	A[cbind(1:sum(test),which(test))] <- 1
	transformed.scoreTest(dfra, msf, fra, A, nullHyp, maxlike)
}
##@summary.sienaTimeTest siena07 summary method for sienaTimeTest objects
summary.sienaTimeTest <- function(object, ...)
{
	if (!inherits(object, "sienaTimeTest"))
	{
		stop("not a legitimate Siena time test object")
	}
	class(object) <- c("summary.sienaTimeTest", class(object))
	object
}
##@print.summary.sienaTimeTest siena07 print method for summary.sienaTimeTest
print.summary.sienaTimeTest <- function(x, ...)
{
	if (!inherits(x, "summary.sienaTimeTest"))
	{
		stop("not a legitimate Siena time test summary object")
	}
	print.sienaTimeTest(x)
	## Additional output to the print will go in here:
	## The first condition is merely for compatibility with older versions.
	if ((!is.null(x$RequestedExclusions))||(!is.null(x$ExtraExclusions)))
	{
		if (any(x$RequestedExclusions) || any(x$ExtraExclusions))
		{
			cat("\nSome effects were excluded from being tested.\n")
		}

		if (any(x$RequestedExclusions))
		{
			if (sum(x$RequestedExclusions) <= 1)
			{
				cat("The effect with number", which(x$RequestedExclusions))
				cat(" in", x$sienaFitName, "was excluded.\n")
			}
			else
			{
				cat("\nRequested exclusions were the effects numbered\n")
				cat(which(x$RequestedExclusions)[1],
					paste(", ",
						which(x$RequestedExclusions)[2:sum(x$RequestedExclusions)],
						sep=""),
					" in ", x$sienaFitName, ".\n", sep="")
			}
		}

		if (any(x$ExtraExclusions))
		{
			if (sum(x$ExtraExclusions) <= 1)
			{
				cat("The effect with number", which(x$ExtraExclusions),
					" in", x$sienaFitName,
					" was excluded automatically because of collinearity.\n")
			}
			else
			{
				cat("\nThe effects ")
				cat(which(x$ExtraExclusions)[1], paste(", ",
						which(x$ExtraExclusions)[2:sum(x$ExtraExclusions)],
						sep=""),
					" in ", x$sienaFitName, ".\n", sep="")
				cat("were excluded automatically because of collinearity.\n")
			}
		}
	}
	cat("\nIndividual significance tests and one-step estimators:\n")
	print(format(round(x$IndividualTest, digits=4)), quote=F)
	cat("\nEffect-wise joint significance tests\n")
	cat("(i.e. each effect across all dummies):\n")
	print(x$EffectTest)
	if (x$Waves <=2 && ! x$IndividualTestsOrthogonalized)
	{
		#		cat("\n\nNote that these effect-wise tests have a different form")
		#		cat("\nthan the individual tests, thus testing with 3 observations")
		#		cat("\nmay yield different individual and effect-wise values.\n\n")
	}
	else
	{
		if (x$IndividualTestsOrthogonalized)
		{
			cat("\nNote that the 1-df and effect-wise test statistics were",
				"\northogonalized with respect to each other (condition=TRUE).")
			cat("\n")
		}
	}
	if (!is.null(x$GroupTest))
	{
		cat(paste("\n",x$GroupsKind,"-wise joint significance tests\n", sep=""))
		cat(paste("(i.e. each", tolower(x$GroupsKind),
				"across all parameters):\n"))
		print(x$GroupTest)
	}

	tmp <- paste(" (", 1:length(x$BaseRowInD), ") ",
		rownames(x$IndividualTest)[x$BaseRowInD], "\n", sep="")
	cat("\nUse the following indices for plotting:\n",
		tmp[!is.na(x$EffectTestStatistics)])
	cat("\nIf you would like to fit time dummies to your model,")
	cat("\nuse the includeTimeDummy function.")
	cat("\nType \"?test_time\" for more information on this output.\n")
	invisible(x)
}
##@print.sienaTimeTest siena07 print method for sienaTimeTest objects
print.sienaTimeTest <- function(x, ...)
{
	if (!inherits(x, "sienaTimeTest"))
	{
		stop("not a legitimate Siena time test object")
	}
	effectNames <- rownames(x$IndividualTest)
	dummies <- x$ToTest$toTest
	dummyIndex <- paste(" (", format(1:sum(dummies)), ") ",
		effectNames[dummies], "\n", sep="")
	pvalue <- ifelse((x$JointTest < 0.0001)," < 0.0001",
		paste("=",sprintf("%6.4f", x$JointTest)))
	cat("Joint significance test of time heterogeneity:\n",
		"chi-squared = ",
		sprintf("%6.2f", x$JointTestStatistics$testresOverall),
		", d.f. = ", sprintf("%1d", length(x$IndividualTestStatistics)),
		", p", pvalue,
		", \nwhere H0: The following parameters are zero:\n",
		dummyIndex, sep="")
	invisible(x)
}
##@plot.sienaTimeTest siena07 plot method for sienaTimeTest objects
plot.sienaTimeTest <- function(x, pairwise=FALSE, effects,
	scale=0.2, plevels=c(0.1, 0.05, 0.025), ...)
{
	## require(lattice)
	tmp <- paste(" (", 1:length(x$BaseRowInD), ") ",
		rownames(x$IndividualTest)[x$BaseRowInD], "\n", sep="")
	if (missing(effects))
	{
		effects<- (1:length(tmp))[!is.na(x$EffectTestStatistics)]
	}
	if (any(!effects %in% 1:length(tmp)))
	{
		cat("Detected an error with the effects included. For a
			parameter-plot, use the following indices:")
			stop("\nUse the following indices for plotting the pairwise
				moment correlations:\n", tmp)
	}
	if (pairwise)
	{
		## On a "pairwise" call, print a pairwise plot of moments
		if (length(effects) == 0)
		{
			x <- x$Moments
		}
		else
		{
			x <- x$Moments[, , effects, drop=FALSE]
		}
		panel.cor <- function(x, y, digits=2, prefix="", cex.cor, ...)
		{
#			usr <- par("usr"); # on.exit(par("usr" = usr))
			par(usr = c(0, 1, 0, 1))
			r <- abs(cor(x, y))
			txt <- format(c(r, 0.123456789), digits=digits)[1]
			txt <- paste(prefix, txt, sep="")
			if(missing(cex.cor)) cex.cor <- 0.8 / strwidth(txt)
			text(0.5, 0.5, txt, cex = cex.cor * r)
		}
		pairs(apply(x, c(1, 3), sum),
			lower.panel=panel.smooth,
			upper.panel=panel.cor,
			pch=5, ...) ##pch=5 is a diamond
	}
	else
	{
		# require(grid)
		## Otherwise, make the parameter plots:
		## create a vector to split the IndividualTests by
		toTest <- x$ToTest
		waveNumbers <- x$WaveNumbers
		nWaves <- x$Waves
		nLevels <- length(plevels)
		cis <- lapply(1:nLevels, function(i)
			{
				cbind(toTest$valsplus - abs(qnorm(plevels[i]  /  2,
							sd=toTest$dummysd)),
					toTest$valsplus  + abs(qnorm(plevels[i] / 2,
							sd=toTest$dummysd)))
			}
			)
		toTest[, paste(rep(c("lower", "upper"), nLevels),
			rep(1:nLevels, each=2), sep="")] <-
				do.call(cbind, cis)
			## subset the data frame
			toTest <- toTest[toTest$baseEffect %in% effects, ]
			toTest$effectName <- factor(toTest$effectName,
				levels=unique(toTest$effectName))
			if (nlevels(toTest$effectName) != length(unique(toTest$baseEffect)))
			{
				stop("non unique effect names: contact RSiena help")
			}
			#devAskNewPage(TRUE)
			xyplot(valsplus ~ period1 | effectName, data=toTest,
				type = "p", as.table=TRUE, bty="n",
				xlab="Wave", ylab="Parameter Value", auto.key=TRUE,
				scales=list(relation='free', x=list(at=0:(nWaves+1),
						labels=c(" ", waveNumbers, " "))), subscripts=TRUE,
				par.strip.text=list(lines=2.2), #layout=layout,
				xlim=c(0, nWaves + 1), prepanel=function(x, y, ...)
				{
					ymin <- min(y - scale * abs(y))
					ymax <- max(y + scale * abs(y))
					list(ylim=c(ymin, ymax))
				},
				panel=function(x, y, subscripts, ...)
				{
					sapply(1:length(x), function(i, subscripts)
						{ ## pch 45 is a hyphen, pch 20 a filled circle
							tmp <- if(toTest$toTest[subscripts][i]) "red" else "gray"
							panel.xyplot(c(x[i], x[i]),
								c(toTest$lower1[subscripts][i],
									toTest$upper1[subscripts][i]),
								col=tmp, alpha=.50,
								type="l", lend=1, lwd=10)
							panel.xyplot(c(x[i], x[i]),
								c(toTest$lower1[subscripts][i],
									toTest$upper1[subscripts][i]),
								col=tmp, alpha=.75,
								type="p", pch=45, cex=3)
							panel.xyplot(c(x[i], x[i]),
								c(toTest$lower2[subscripts][i],
									toTest$upper2[subscripts][i]),
								col=tmp, alpha=.50,
								type="l", lend=1, lwd=10)
							panel.xyplot(c(x[i], x[i]),
								c(toTest$lower2[subscripts][i],
									toTest$upper2[subscripts][i]),
								col=tmp, alpha=.75,
								type="p", pch=45, cex=3)
							panel.xyplot(c(x[i], x[i]),
								c(toTest$lower3[subscripts][i],
									toTest$upper3[subscripts][i]),
								col=tmp, alpha=.25,
								type="l", lend=1, lwd=10)
						}, subscripts=subscripts)
					panel.xyplot(x, y, type="s",
						col="black", alpha=.75, pch=2)
					panel.xyplot(x, y, type="p",  pch=20,
						col=1)
					panel.abline(a=toTest[subscripts[1], "initial"],
						reference=TRUE,
						col="black", lwd=2, alpha=.75)
					# require grid	
					# grid.text(paste("p = ", toTest$effectTest[subscripts[1]]),
					#           0.02, 0.1, just="left")
				}, ...)
	}
}

##@sienaTimeFix siena07 Adds time dummy terms to the effects object
sienaTimeFix <- function(effects, data=NULL, getDocumentation=FALSE)
{
	##@addEffect internal sienaTimeFix add one or more effects
	addEffect <- function(newEffects, i, newname, effectGroup, shortName,
		timeDummy, fix=FALSE, include=TRUE, yName=NULL)
	{
		tmprows <- createEffects(effectGroup, xName=newname, yName=yName,
			name=effects$name[i][1],
			groupName=effects$groupName[i][1],
			group=effects$group[i][1],
			netType=effects$netType[i][1])
		tmprows <- tmprows[tmprows$shortName==shortName &
			tmprows$type %in% effects$type[i], ]
		tmprows$fix <- fix
		tmprows$include <- include
		tmprows$effectNumber <- max(newEffects$effectNumber) +
			(1:nrow(tmprows))
		tmprows$timeDummy <- timeDummy
		rownames(tmprows) <- paste(newname, effects$type[i], sep=".")
		newEffects <- rbind(newEffects, tmprows)
		newEffects
	}

	##@addVarCovar internal sienaTimeFix add a varying covariate
	addVarCovar <- function(data, base, nodeSet, newname, dataSub)
	{
		base <- varCovar(base, nodeSet=nodeSet, warn=FALSE)
		base <- addAttributes.varCovar(base, name=dname)
		data[[dataSub]]$vCovars <- c(data[[dataSub]]$vCovars, list(base))
		n <- length(data[[dataSub]]$vCovars)
		names(data[[dataSub]]$vCovars)[n] <- dname
		data
	}
	if (getDocumentation)
	{
		return(getInternals())
	}
	nGroups <- max(unique(effects$group))
	# note that period is a character and must be changed to numeric
	groupPeriods <- as.numeric(tapply(as.numeric(effects$period), effects$group,
			max, na.rm=TRUE)) + 1
	intervals <- rep(1:length(groupPeriods), groupPeriods)
	localPeriodNos <- unlist(sapply(groupPeriods, function(x)1:x))
	nGroups <- length(groupPeriods)
	v1 <- do.call(c, lapply(groupPeriods, function(x)1:x))
	v2 <- rep(cumsum(c(0, groupPeriods))[1:nGroups], groupPeriods)
	v3 <- v1 + v2
	periodNos <- v3[!v3 %in% cumsum(groupPeriods)]

	if (!is.null(data))
	{
		atts <- attributes(data)
	}
	if ((length(periodNos) < 2) && any(effects$timeDummy != ","))
	{
		warning("Time dummies not relevant with only 2 periods")
		effects$timeDummy <- ","
	}

	structuralRate <- effects$type == "rate" & effects$rateType %in%
		"structural"
	if (any(effects$timeDummy[structuralRate] != ","))
	{
		warning("Time dummy effects are not implemented",
			" for structural rate effects.")
		effects$timeDummy[structuralRate] <- ","
	}
	# JAL: Implementing these 20-FEB-2011 in RSienaTest
	# TODO: Behavioral interactions need to be implemented for these to work.
	# Once they are, we can comment lines 575-577 and 717-720 out.
	##	behaviorNonRateX <- effects$netType =="behavior" & effects$type != "rate"
	##   if (any(effects$timeDummy[behaviorNonRateX] != ","))
	##  {
	## 		warning("Time dummy effects are not implemented",
	##                " for behavior effects of type eval or endow.")
	##        effects$timeDummy[behaviorNonRateX] <- ","
	##    }

	if (all(effects$timeDummy == ",") )
	{
		##		No time dummy interactions to add, so kick the inputs back.
		return(list(effects=effects, data=data))
	}
	else
	{
		effects$timeDummy[effects$timeDummy=="all"]  <-
			paste(periodNos[-1], collapse = ",")

		alreadyDummied <- grepl("isDummy", effects$timeDummy)
		if (length(alreadyDummied)  >  0)
		{
			## Just remove those effects that have already been dummied so as to
			## not messy things up. The assumption is that the user will retain
			## all of the previous dummied effects within the column.
			effects <- effects[!alreadyDummied, ]
		}

		timesd <- strsplit(effects$timeDummy, ",| ")

		opar <- options(warn=-1)
		vals <- sapply(timesd, function(x) any(x=="" | !is.na(as.numeric(x))))
		if (any(!vals))
		{
			stop("Invalid input in timeDummy column", ": ",
				paste(effects$timeDummy[!vals], collapse=" and "))
		}
		options(opar)

		timesd <- lapply(timesd, function(x)as.numeric(x[x %in% periodNos]))
		dummiedEffects <- sapply(timesd, function(x)length(x) > 0)

		baseType <- list(number=which(dummiedEffects),
			type=effects$netType[dummiedEffects],
			name=effects$name[dummiedEffects])

		rateXDummies <- effects$shortName == "RateX" & dummiedEffects

		newEffects <- effects

		##@getCovar internal sienaTimeFix find the covariate
		getCovar <- function(cdvind, vdvind, bdvind, p)
		{
			if (!is.na(cdvind))
			{
				tmp <- data[[dataSub]]$cCovars[[cdvind]]
				val <- tmp
			}
			else if (!is.na(vdvind))
			{
				tmp <- data[[dataSub]]$vCovars[[vdvind]]
				val <- tmp[, p]
			}
			else
			{
				tmp <- data[[dataSub]]$depvars[[bdvind]]
				val <- tmp[, , p]
			}
			list(covar=tmp, val=val)
		}
		for (i in which(rateXDummies))
		{
			effect <- newEffects[i, ]
			## Figure out the base values:
			if (!is.null(data))
			{
				cdvind <- match(effect$interaction1, atts$cCovars)
				vdvind <- match(effect$interaction1, atts$vCovars)
				bdvind <- match(effect$interaction1, atts$netnames)
				if (any((is.na(cdvind) & is.na(vdvind) &
					(is.na(bdvind))))) #|| atts$types[bdvind] != "behavior"))
				{
					stop("Having trouble finding the covariate for your rate ",
						"effect. Please contact the developers.")
				}
			}
			for (p in timesd[[i]])
			{
				dname <-  paste(effect$interaction1, "Dummy", p, ":",
					effect$name, sep="")
				dataSub <- intervals[p]
				pp <- localPeriodNos[p]
				if (!is.null(data))
				{
					nPer <- attr(data[[dataSub]]$depvars[[effect$name]],
						"netdims")[3]
					covar <- getCovar(cdvind, vdvind, bdvind, pp)
					## add a new varCovar:
					nodeSet <- attr(covar$covar, "nodeSet")
					base <- matrix(0, nrow=length(covar$val), ncol=nPer - 1)
					base[, pp] <- covar$val
					data <- addVarCovar(data, base, nodeSet, dname, dataSub)

					## also add to other data objects as all must have same set

					for (ii in 1:length(data))
					{
						if (ii != dataSub)
						{
							nActors <-
								length(data[[ii]]$nodeSets[[nodeSet]])
							nPer <-
								attr(data[[ii]]$depvars[[effect$name]],
									"netdims")[3]
							base <- matrix(0, nrow=nActors, ncol=nPer - 1)
							data <- addVarCovar(data, base, nodeSet, dname, ii)
						}
					}
				}
				## add the rate effect row:
				newEffects <-
					addEffect(newEffects, i, dname, "covarNonSymmetricRate",
						'RateX', paste('isDummy', p, i, sep=','))
			}
		}
		## now the non rateX effects

		## first add dummies for all requested periods for each
		## dependent variable

		## then construct the interaction effects with the other effects
		## the dummies are fixed unless we need dummies for the density effect

		## find which periods we need
		use <- sapply(timesd, length) > 0 & !rateXDummies
		timeslist <- split(timesd[use], effects$name[use])
		timeslist <- lapply(timeslist, function(x)sort(unique(unlist(x))))
		timesTypelist <- split(timesd[use],
			list(effects$name[use], effects$type[use]))
		timesTypelist <- lapply(timesTypelist,
			function(x)unique(unlist(x)))

		types <- unique(effects$type[use ])
		for (depvar in names(timeslist))
		{
			##if (!is.null(data) && atts$types[[depvar]] == "behavior")
			##{
			##    stop ("Function is not specified for behavior effects")
			##}
			for (p in timeslist[[depvar]])
			{
				## create the dummy covariate
				pp <- localPeriodNos[p]
				dname <- paste("Dummy", p, ":", depvar, sep="")
				dataSub <- intervals[p]
				if (!is.null(data))
				{
					dims <- attr(data[[dataSub]]$depvars[[depvar]],
						"netdims")
					nodeSet <- attr(data[[dataSub]]$depvars[[depvar]],
						"nodeSet")[1]
					nActors <- dims[1]
					nPer <- dims[3]
					tmp <- matrix(0, nActors, nPer - 1)
					tmp[, pp] <- 1
					data <- addVarCovar(data, tmp, nodeSet, dname, dataSub)

					## also add to other data objects as all must be the same

					for (i in 1:length(data))
					{
						if ( i != dataSub)
						{
							dims <- attr(data[[i]]$depvars[[depvar]],
								"netdims")
							nodeSet <-
								attr(data[[i]]$depvars[[depvar]],
									"nodeSet")[1]
							nActors <- dims[1]
							nPer <- dims[3]
							base <- matrix(0, nActors, nPer - 1)
							data <- addVarCovar(data, base, nodeSet, dname, i)
						}
					}
				}
				## This is where the behaviors are treated differently
				if (baseType$type[baseType$name==depvar][1] == "behavior")
				{
					##find the linear effect rows for this depvar
					i <- which(effects$name == depvar &
						effects$shortName=="linear" &
						effects$type %in% types)

					if (length(i) == 0)
					{
						stop("Cannot find 'effect from' effect(s) for ", depvar,
							" ", types)
					}

					## establish whether we want the effFroms
					## included, fixed or not
					fix <- sapply(timesd[i], function(x)!p %in% x)
					typesNames <- paste(depvar, effects$type[i], sep=".")
					includeTypes <- !sapply(timesTypelist[typesNames], is.null)

					## add one or more effects (depending on types requested)
					newEffects <- addEffect(newEffects, i, dname,
						"covarBehaviorObjective", "effFrom",
						paste('isDummy', p,
							effects$effectNumber[i],
							sep=','),
						fix=fix, include=includeTypes,
						yName=dname)
				}
				else
				{
					##find the density rows for this depvar
					i <- which(effects$name == depvar &
						effects$shortName=="density" &
						effects$type %in% types)

					if (length(i) == 0)
					{
						stop("Cannot find density effect(s) for ", depvar,
							" ", types)
					}

					## establish whether we want the egoXs included,
					## fixed or not
					fix <- sapply(timesd[i], function(x)!p %in% x)
					typesNames <- paste(depvar, effects$type[i], sep=".")
					includeTypes <- !sapply(timesTypelist[typesNames], is.null)

					## add one or more effects (depending on types requested)
					newEffects <-
						addEffect(newEffects, i, dname,
							"covarNonSymmetricObjective", "egoX",
							paste('isDummy', p,
								effects$effectNumber[i], sep=','),
							fix=fix, include=includeTypes)
				}
			}

			## now the interactions for any dummied effects for this
			## dependent variable
			for (j in
				which(dummiedEffects & !rateXDummies &
					effects$name==depvar &
					effects$shortName != "density" &
					effects$shortName != "linear"))
			{
				for (p in timesd[[j]])
				{
					dname <- paste("Dummy", p, ":", depvar, sep="")
					effect <- effects[j, ]
					if (baseType$type[baseType$name==depvar][1] == "behavior") {
						newEffects <-
							includeInteraction(newEffects,
								effect$shortName, "effFrom",
								character=TRUE,
								type=effect$type,
								interaction1= c(effect$interaction1,
									dname),
								interaction2=effect$interaction2,
								name=depvar, verbose=FALSE)
					} else {
						newEffects <-
							includeInteraction(newEffects,
								effect$shortName, "egoX",
								character=TRUE,
								type=effect$type,
								interaction1= c(effect$interaction1,
									dname),
								interaction2=effect$interaction2,
								name=depvar, verbose=FALSE)
					}
					## find the row altered
					newrow <- newEffects$effect1 == j &
						newEffects$effect2 ==
						newEffects[paste(dname, effect$type, sep="."),
					"effectNumber"]
						newEffects$timeDummy[newrow] <-
							paste('isDummy', p, j, sep=',')
				}
			}
		}
		## reorder the effects so Dummies come before interactions and
		## rates where they are expected
		e0 <- split(newEffects, newEffects$name)

		e0 <- lapply(e0, function(x)
			{
				e1 <- !grepl("unspecified interaction effect",
					x$effectName)
				e2 <- grepl("unspecified interaction effect",
					x$effectName)
				e3 <- x$type == "rate"
				rbind(x[e3, ], x[e1 & ! e3, ], x[e2 & ! e3, ])
			})
		## sort into order of names in original effect
		effNames <- unique(effects$name)
		order1 <- match(effNames, names(e0))
		e0 <- e0[order1]
		newEffects <- do.call(rbind, e0)
		## now reconstruct the group object to make sure all attributes are
		## correct after adding covariates
		if (!is.null(data))
		{
			data <- sienaGroupCreate(data, singleOK=TRUE)
		}
		list(effects=newEffects, data=data)
	}
}
##@includeTimeDummy DataCreate
includeTimeDummy <- function(myeff, ..., timeDummy="all", depvar=myeff$name[1],
	type="eval", covar1="", covar2="", include=TRUE,
	character=FALSE)
{
	if (character)
	{
		dots <- sapply(list(...), function(x)x)
	}
	else
	{
		dots <- substitute(list(...))[-1] ##first entry is the word 'list'
	}
	if (length(dots) == 0)
	{
		stop("need some effect short names")
	}
	if (!character)
	{
		effectNames <- sapply(dots, function(x)deparse(x))
	}
	else
	{
		effectNames <- dots
	}
	if (hasArg("name"))
	{
		stop("The name of the dependent variable now should be given as 'depvar', not 'name'.
     See the help file for includeTimeDummy.  ")
	}
	if ((hasArg("interaction1")) | (hasArg("interaction2")))
	{
		stop("The name of the explanatory variable now should be given 
     as 'covar1' and 'covar2', not 'interaction1' and 'interaction2'.
     See the help file for includeTimeDummy.  ")
	}
	use <- myeff$shortName %in% effectNames &
		myeff$type==type &
		myeff$name==depvar &
		myeff$interaction1 == covar1 &
		myeff$interaction2 == covar2
	if (any(use))
	{
		myeff[use, "timeDummy"] <- timeDummy
		myeff[use, "include"] <- include
		print.data.frame(myeff[use,])
	}
	else
	{
		warning("No effects could be found with this specification.")
	}
	myeff
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: zzz.r
# *
# * Description: This module contains the code for package attachment and
# * detachment.
# *****************************************************************************/

#dllpath <- ''
##@imagepath Objects/Path Path for image of Siena
imagepath <- ""

##csvpath <- ''
## or .onAttach?
##@pkgpath Objects/Path Path for installation of siena.exe
pkgpath <- ""

##@pkgname Objects/Package name of package (RSiena or RSienaTest)
pkgname <- ""

##@pkgversion Objects/Package version of package
pkgvers <- ""

##@.onLoad Miscellaneous Start-up processing
.onLoad <- function(libname, pkgname) {
#   dllpath <<- if (nzchar(.Platform$r_arch))
##       file.path(libname, pkgname, "libs", .Platform$r_arch,
#                 paste('RSiena', .Platform$dynlib.ext, sep=''))
#   else
#       file.path(libname,pkgname, "libs",
 #                paste('RSiena', .Platform$dynlib.ext, sep=''))
#  #  data('sysdata',package='RSiena')
  imagepath <<- file.path(libname, pkgname, paste("ilcampo.gif"))
  pkgpath <<- file.path(libname, pkgname)
  pkgname <<- pkgname
  pkgvers <<- utils::packageDescription(pkgname, fields = c("Version", "Date"))
#cat(pkgname,pkgpath,'\n')
 ##  csvpath<<- file.path(libname,pkgname)
 #   library.dynam("RSiena",package=pkgname)
 #  cat (libname,pkgname,'\n')
}

##@.onUnload Miscellaneous Unload processing
.onUnload <- function(libpath) {
  library.dynam.unload(pkgname, libpath)
}

#.Last.lib <- function(libpath)
#{
#    cat(libpath,'\n')
#}

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: influenceTables.r
## *
## ****************************************************************************/

#############################################################################
# R script InfluenceTables.r                                                #
# written by Tom A.B. Snijders                                              #
# October 12, 2023                                                          #
# (Gratitude to Steffen Triebel and Rene Veenstra for corrections)          #
#############################################################################

#############################################################################
#                                                                           #
# These are functions for constructing and presenting influence tables      #
# for the interpretation of results for network and behavior dynamics       #
# obtained with the RSiena or multiSiena packages.                          #
# Also consult the manual, s                         #
#                                                                           #
# The main function to use is                                               #
# influenceMatrix <- function(x,xd,netname,behname,levls, levls.alt=levls)  #
# which creates a matrix containing the influence table for                 #
# siena data set xd, sienaFit or sienaMeta or sienaBayesFit object x,       #
# network variable netname (should be a character string),                  #
# dependent actor variable behname (also a character string),               #
# levels for ego levls, levels for alter levls.alt.                         #
# If levls is NULL (the default), it is taken as the integer range of       #
# the dependent actor variable.                                             #
# Mostly levls.alt will be the same as levls,                               #
# in which case it does not have to be specified.                           #
#                                                                           #
# For models utilizing creation and/or endowment effects,                   #
# by specifying include.endow=TRUE                                          #
# the sum of evaluation and endowment effects is used;                      #
# by specifying include.creation=TRUE                                       #
# the sum of evaluation and creation effects is used;                       #
# the first is relevant for maintaining, i.e., not decreasing,              #
# the values of the dependent variable,                                     #
# the second for increasing them.                                           #
#                                                                           #
# Other functions to use (see below):                                       #
# influenceTable.se                                                         #
# Computes standard error of a linear combination of elements               #
# of the influence table.                                                   #
# Not for sienaMeta or sienaBayesFit objects.                               #
#                                                                           #
# influenceTable.plot                                                       #
# Constructs a plot of the influence table using ggplot2.                   #
#                                                                           #
# If these functions are used for a sienaMeta object x,                     #
# (produced by siena08, as opposed to a sienaFit object produced by siena07)#
# xd should be one of the individual data sets used for creating x,         #
# (a single-group instead of multi-group data set)                          #
# with an average ('representative') value                                  #
# for the mean of variable <behname>.                                       #
#                                                                           #
#############################################################################

##@influenceTable.basis  Basis for influenceTable
influenceTable.basis <- function(x, xd, netname, behname,
                            levls=NULL, levls.alt=levls,
                            out.ego=1, silent=FALSE, nfirst=x$nwarm+1,
                            include.endow=FALSE, include.creation=FALSE){
# Creates basic materials for a influence table for
# siena data set xd, sienaFit or sienaBayesFit object x,
# network variable netname (should be a character string),
# dependent actor variable behname (also a character string),
# levels for ego levls, levels for alter levls.alt.
# For effect avAlt, levls.alt refers to average values of alter's behavior;
# for effect totAlt, levls.alt refers to total values of alter's behavior;
# For effects avSim and totSim, levls.alt refers to
#                    constant values of alter's behavior.
# For effects totSim and totAlt, out.ego is the presumed outdegree of ego
# for which the contributions to the objective function are calculated.
#
# This function can be used also for a sienaMeta object x.
# Then xd must be one of the data sets used for creating x,
# with an average ('representative') value for the mean of variable <behname>.
#
	if(!silent)
	{
		cat("Network",netname, "; dependent behavior",behname,".\n")
	}
    if (!(inherits(x, "sienaFit") | inherits(x, "sienaBayesFit")))
    {
        stop("x should be a sienaFit or sienaBayesFit object")
    }
    if (!(inherits(xd, "sienadata") | inherits(xd, "siena")))
    {
        stop("xd should be a siena data set")
    }
    if (include.endow && include.creation)
    {
        warning(paste('It is not meaningful to include creation',
                ' and maintenance effects simultaneously.'))
    }
# Obtain estimate
    if (inherits(x, "sienaBayesFit"))
    {
        theta <- sienaFitThetaTable(x, fromBayes=TRUE,
                    groupOnly=0, nfirst=nfirst)$mydf$value
        theEffects <- x$requestedEffects
        if (length(theta) != dim(theEffects)[1])
        {
            stop('mismatch between theta and effect names')
        }
# Is this correct for all model specifications?
    }
    else
    {
        theta <- x$theta
        theEffects <- x$requestedEffects
    }
    theparm <- theEffects$parm
# Note that theta and theparm now have the same length.
# Obtain means
    if (inherits(xd, "sienaGroup"))
    {
        cat("A sienaGroup data object was given.\n")
        # Does the variable exist, and is it a dependent variable?
        thebeh <- xd[[1]]$depvars[[behname]]
        thenet <- xd[[1]]$depvars[[netname]]
    }
    else
    {
        thebeh <- xd$depvars[[behname]]
        thenet <- xd$depvars[[netname]]
    }
    if (is.null(thenet)){stop(paste('There is no network <',netname,'>.'))}
    if (is.null(thebeh)){
            stop(paste('There is no dependent behaviour variable <',behname,'>.'))}

    if (inherits(xd, "sienaGroup"))
    {
#       zmean  <- mean(sapply(xd, function(z){mean(z$depvars[[behname]], na.rm=TRUE)}))
        ztot   <- sum(sapply(xd, function(z){sum(z$depvars[[behname]], na.rm=TRUE)}))
        ztotN  <- sum(sapply(xd, function(z){sum(!is.na(z$depvars[[behname]]))}))
        zmean  <- ztot/ztotN
        zsmean <- attr(xd, "bSim")[[behname]]
        Delta  <- attr(xd, "behRange")[,behname]
        if (is.null(levls))
        {
            levls <- Delta[1] : Delta[2]
        }
        Delta  <- Delta[2] - Delta[1]
    }
    else
    {
        zmean  <- mean(colMeans(thebeh, na.rm=TRUE))
        zsmean <- attr(thebeh, 'simMean')
        Delta  <- attr(thebeh, 'range')
        if (is.null(levls))
        {
            range2 <- attr(thebeh, 'range2')
            levls <- range2[1] : range2[2]
        }
    }
    if (is.null(levls.alt))
    {
        levls.alt <- levls
# I'm not sure this works if the effect used is totAlt and out.ego is not 1.
    }
# perhaps attr(thebeh, 'simMean') fails if there are more than one dependent network?
# is this attribute then a vector of length >= 2?
# Note that zsmean is used only if the model includes avSim or totSim.
    replace0 <- function(k){ifelse(length(k)==0,0,k)}
    efNames <- c('linear','quad','avAlt','avSim','totAlt','totSim',
                    'avAttHigher', 'avAttLower',
                    'threshold', 'threshold2', 'threshold3', 'threshold4')
    zeff.eval <- sapply(efNames, function(s)
        {replace0(which(theEffects$name == behname &
                (theEffects$type == 'eval') &
                (theEffects$interaction1 %in% c(netname,'')) &
                theEffects$include &
                theEffects$shortName==s))})
    zeff.endow <- sapply(efNames, function(s)
        {replace0(which(theEffects$name == behname &
                (theEffects$type == 'endow') &
                (theEffects$interaction1 %in% c(netname,'')) &
                theEffects$include &
                theEffects$shortName==s))})
    zeff.creation <- sapply(efNames, function(s)
        {replace0(which(theEffects$name == behname &
                (theEffects$type == 'creation') &
                (theEffects$interaction1 %in% c(netname,'')) &
                theEffects$include &
                theEffects$shortName==s))})
    zeff.parm <- sapply(efNames, function(s)
        {replace0(which(theEffects$name == behname &
                (theEffects$interaction1 %in% c(netname,'')) &
                theEffects$include &
                theEffects$shortName==s))})
# zeff gives the indicators of the effects in x
    taketheta <- function(k){ifelse(k==0,0,theta[k])}
    takeparm <- function(k){ifelse(k==0,0,theparm[k])}
    ztheta <- sapply(zeff.eval,taketheta)
    zparm <- sapply(zeff.parm,takeparm)
    if (include.endow)
    {
        ztheta <- ztheta + sapply(zeff.endow,taketheta)
    }
    if (include.creation)
    {
        ztheta <- ztheta + sapply(zeff.creation,taketheta)
    }
    if (all(ztheta == 0))
    {
        cat('All parameters found for the effect of', netname, 'on',
                                                behname, 'are 0.\n')
    stop('there seems to be something wrong with x, xd, netname, or behname.')
    }
    if (!silent){
        cat('Parameters found are\n')
        print(noquote(format(round(ztheta[ztheta != 0.0],4))))
        if (include.endow)
        {
            cat('This includes evaluation and endowment (maintenance) effects.\n')
        }
        if (include.creation)
        {
            cat('This includes evaluation and creation effects.\n')
        }
    }
    found <- (ztheta != 0.0)
    if (!silent){
		if (any(found[9:12]))
		{
			cat("Threshold internal effect parameters are ",
                        zparm[intersect(which(found), 9:12)], '\n')
		}
		if (sum(found[3:8]) == 0)
		{
			cat("Note: no influence effect was found.\n")
		}
		if (sum(found[3:8]) >= 2)
		{
			cat("Note: more that one influence effect was found.\n")
        }
		if (found[5] | found[6])
		{
			cat("Note that influence in this model depends on ego's outdegree. \n")
			cat("Outdegree of ego is given as ", out.ego, "; can be changed.\n")
        }
		if (found[4] | found[6] | found[7] | found[8])
		{
			cat("Levels of alter refer to constant values of alter's behavior.\n\n")
        } 
		else if (found[3] | found[5])
		{
			cat("Levels of alter refer to average values of alter's behavior.\n\n")
        }
		cat('\n')
		flush.console()
	}
    quad <- !any(found[c(4,6:12)])
    # These effects do not lead to quadratic functions
# ztheta contains the parameter values in x
    K <- length(levls)
    KA <- length(levls.alt)
    zalter <- rep(levls.alt,each=KA)
    zego <- rep(levls,K)
    fact <- 1:K
    alter <- factor(rep(fact,each=KA))
    coeffs <- matrix(NA, K*KA, length(efNames))
    coeffs[,1] <- (zego - zmean)
    coeffs[,2] <- (zego - zmean)*(zego - zmean)
    coeffs[,3] <- (zego - zmean)*(zalter - zmean)
    coeffs[,4] <- (1-(abs(zalter - zego)/Delta)-zsmean)
    coeffs[,5] <- out.ego*(zego - zmean)*(zalter - zmean)
    coeffs[,6] <- out.ego*(1-(abs(zalter - zego)/Delta)-zsmean)
    coeffs[,7] <- (1-(pmax(zalter - zego,0)/Delta)-zsmean)
    coeffs[,8] <- (1-(pmax(zego - zalter,0)/Delta)-zsmean)
    coeffs[,9] <- 1*(zego >= zparm[9])
    coeffs[,10] <- 1*(zego >= zparm[10])
    coeffs[,11] <- 1*(zego >= zparm[11])
    coeffs[,12] <- 1*(zego >= zparm[12])
    select <- coeffs %*% ztheta
    df <- data.frame(alter, zalter, zego, select=(coeffs%*%ztheta))
    list(df=df, zeff.eval=zeff.eval, zeff.endow=zeff.endow,
         zeff.creation=zeff.creation, ztheta=ztheta, zmean=zmean, zsmean=zsmean,
         Delta=Delta, coeffs=coeffs, levls=levls, levls.alt=levls.alt, quad=quad)
}

##@influenceTable  influenceTable
influenceTable <- function(x, xd, netname, behname,
                      as.matrix=FALSE,
                       levls=NULL, levls.alt=levls, out.ego=1, 
						silent=FALSE, nfirst=x$nwarm+1,
                        include.endow=FALSE, include.creation=FALSE){
# Creates a data frame containing the influence table for
# siena data set xd, sienaFit object x,
# network netname (should be a character string),
# dependent actor variable behname (also a character string),
# levels for ego levls, levels for alter levls.alt.
    infl.t <- influenceTable.basis(x, xd, netname, behname,
                   levls=levls, levls.alt=levls.alt, out.ego=out.ego, 
				   silent=silent, nfirst=nfirst,
                   include.endow=include.endow,
					include.creation=include.creation)
    df <- infl.t$df
    df$alter <- as.character(df$zalter)
    df$zego <- as.numeric(as.character(df$zego))
    df$select <- as.numeric(as.character(df$select))
    attr(df, "quad") <- infl.t$quad
	class(df) <- c("influenceTable", class(df))
	attr(df, "netname") <- netname
	attr(df, "behname") <- behname
	attr(df, "levls") <- infl.t$levls
	attr(df, "levls.alt") <- infl.t$levls.alt
	if (as.matrix)
	{
	    {
			levls <- unique(df$zego)
		}
		if (is.null(levls.alt))
		{
			levls.alt <- unique(df$alter)
		}
		mat <- matrix(as.numeric(as.character(df$select)),
            length(unique(df$zalter)),
            length(unique(df$zego)), byrow=TRUE)
		colnames(mat) <- infl.t$levls
		rownames(mat) <- infl.t$levls.alt
		class(mat) <- c("influenceTable", class(mat))
		return(mat)
	}
	else
	{
		return(df)
	}
}

influenceTable.se <- function(x, xd, netname, behname, levls, ww, levls.alt=levls){
# Calculates the standard error for a linear combination
# of elements of the influence table for siena data set xd, sienaFit object x,
# dependent actor variable behname (also a character string),
# network variable netname (should be a character string),
# levels for ego levls, levels for alter levls.alt.
# Coefficients of the linear combination are in matrix ww,
# which is assumed to have rows corresponding to levls and columns to levls.alt.
# The linear combination for which the standard error is computed is
# sum_{h,k} ww[h,k] * influenceTable[h,k].
    if (!all(dim(ww) == c(length(levls), length(levls.alt)))){
        stop('Dimension of ww should be the lengths of levls and levls.alt.\n')
    }
    if (inherits(x, "sienaBayesFit")){
        stop('This function does not work for sienaBayesFit objects.\n')
    }
    cat("Requested cell weights (row = alter's, col = ego's behavior value):\n")
    print(cbind(which(ww != 0, arr.ind=TRUE),
            value=ww[which(ww != 0, arr.ind=TRUE)]))
    if (!all(rowSums(ww) == 0)){
        cat("Warning: not all row sums of cell weights are 0.\n Row sums are:\n")
        print(rowSums(ww))
        cat("Only contrasts are meaningful. Change weight matrix.\n\n")
    }
    inftb <- influenceTable.basis(x=x, xd=xd, netname=netname, 
				behname=behname, levls=levls, levls.alt=levls.alt,
                silent=TRUE)
    zeff.r <- inftb$zeff.eval[inftb$zeff.eval != 0] # effects included in x
    cth <- x$covtheta[zeff.r, zeff.r] # their covariance matrix
    cth[is.na(cth)] <- 0 # if any effects were fixed, they have NA in covtheta
    lincomb <- sum(as.vector(t(ww)) * inftb$df["select"])
    # the desired linear combination of cell values of the influence matrix
    wt <- colSums(as.vector(t(ww)) * inftb$coeff)
    # Vector of weights for the linear combination of parameters
    names(wt) <- names(inftb$zeff.eval)
    wt.r <- wt[names(zeff.r)] # should be restricted to what is in the model
    cat("Parameter estimates and their resulting weights are \n")
    print(rbind('param'=inftb$ztheta[inftb$zeff.eval != 0], 'weight'=wt.r))
    cat("Linear combination of cells of influence matrix", round(lincomb,4),
        "\nStandard error\n")
    print(sqrt((wt.r %*% cth %*% wt.r)[1,1]))
}


##@print.influenceTable Methods
print.influenceTable <- function(x, ...)
{
	if (!inherits(x, "influenceTable"))
	{
		stop("not a legitimate influenceTable object")
	}
	if (inherits(x, "matrix"))
	{
		mat <- x
		class(mat) <- "matrix"
		print(mat)
	}
	else if (inherits(x, "data.frame"))
	{		
		mat <- matrix(as.numeric(as.character(x$select)),
            length(unique(x$zalter)),
            length(unique(x$zego)), byrow=TRUE)
		colnames(mat) <- attr(x, "levls") 
		rownames(mat) <- attr(x, "levls.alt")
		print(mat)
	}
	invisible(x)
}
	
	
interpret_influence <- function(x, ...) UseMethod("interpret_influence", x)

##@interpret_influence.sienaFit method for siena.Fit
interpret_influence.sienaFit <- function(x, xd, netname, behname,
                    as.matrix=FALSE,
                    levls=NULL, levls.alt=levls, out.ego=1, 
					silent=FALSE, nfirst=x$nwarm+1,
                    include.endow=FALSE, include.creation=FALSE, ...)
{
	influenceTable(x, xd, netname, behname,
                    as.matrix=as.matrix,
                    levls=levls, levls.alt=levls.alt, out.ego=out.ego, 
					silent=silent, nfirst=nfirst,
                    include.endow=include.endow, include.creation=include.creation, ...)
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: initializeFRAN.r
# *
# * Description: This module contains the code for setting up the data in C++
# * and for ML making the initial chain.
# *****************************************************************************/

##@initializeFRAN siena07 reformat data and send to C. get targets.
initializeFRAN <- function(z, x, data, effects, prevAns=NULL, initC,
	profileData=FALSE, returnDeps=FALSE,
	returnChains=FALSE, returnChangeContributions=FALSE,
	byGroup=FALSE, 
	returnDataFrame=FALSE, byWave=FALSE,
	returnLoglik=FALSE, onlyLoglik=FALSE)
{
	##@checkNames used in initializeFRAN for siena07
	checkNames <- function(xx, types) {
		# checks whether names of named vectors are in names of dependent variables.
		# returns xx; if is.null(xx), returns named vector
		# with variable names and all values 1.
		if (('oneMode' %in% types) || ('bipartite' %in% types))
		{
			thetype <- 'network'
		}
		else
		{
			thetype <- 'behavior'
		}
		if (inherits(data, "sienaGroup"))
		{
			theVars <- sapply(data[[1]]$depvars, function(x){attr(x,'type') %in% types})
			theVarNames <- names(data[[1]]$depvars)[theVars]
		}
		else
		{
			theVars <- sapply(data$depvars, function(x){attr(x,'type') %in% types})
			theVarNames <- names(data$depvars)[theVars]
		}
		if ((length(xx) >= 1) && (!(length(xx) == sum(theVars))))
		{
			cat(deparse(substitute(xx)), ' =  (',
					paste(names(xx), xx, sep=" = ", collapse="; ") ,') ,\n')
			if (sum(theVars)==1)
			{
				cat('but there is only one dependent variable.\n')
			}
			else
			{
				cat('but there are ', sum(theVars), ' dependent variables.\n')
			}
			cat('Length of',deparse(substitute(xx)),
				'should be equal to number of', thetype,'variables.\n')
			stop('Invalid algorithm-data combination.')
		}
		# For this check:
		# if there are no names, then (names(xx) %in% names(data$depvars))
		# will be logical(0), and all(logical(0)) is TRUE.
		if (is.null(xx))
		{
			wrongName <- FALSE
			xx <- rep(1, sum(theVars))
			names(xx) <- theVarNames
		}
		else
		{
			wrongName <- (!all(names(xx) == theVarNames))
			if (is.null(names(xx)))
			{
				wrongName <- TRUE
			}
			else
			{
				if (any(is.na(names(xx))))
				{
					wrongName <- TRUE
				}
			}
			if (wrongName)
			{
				cat(deparse(substitute(xx)),
					'in the algorithm object x should be a named vector with\n')
				cat(' the names of dependent', thetype, 'variables')
				cat(' in the data set, in the same order.\n')
				cat(' Names in algorithm object: ', names(xx), '\n')
				cat(' Names in data set: ', theVarNames, '\n')
				stop('Invalid algorithm-data combination.')
			}
		}
		xx
	}

	# start of initializeFRAN proper
	z$effectsName <- deparse(substitute(effects))
	## fix up the interface so can call from outside robmon framework
	if (is.null(z$FinDiff.method))
	{
		z$FinDiff.method <- FALSE
	}
	if (is.null(z$int))
	{
		z$int <- 1
	}
	if (is.null(z$int2))
	{
		z$int2 <- 1
	}
	if (!is.null(prevAns))
	{
		if (!inherits(prevAns, "sienaFit"))
		{
			stop("prevAns should be a sienaFit object")
		}
		if (x$gmm & (!prevAns$gmm))
		{
			cat("Use of Generalized Method of Moments also required for prevAns.\n")
			stop("Algorithm object specifies gmm, but prevAns did not use it.")
		}	
	}
	if (!initC) ## i.e. first time round
	{
		if ((!inherits(data,"siena")) && (!inherits(data, 'sienadata')))
		{
			stop("not valid siena data object")
		}
		## check the effects object
		defaultEffects <- checkVersion(data, effects)
		if (is.null(effects))
		{
			cat("You specified no effects. The default effects are used.\n")
			effects <- defaultEffects
		}
		else
		{
			## check that the effects match the data dependent variables
			userlist <- apply(effects[effects$include,], 1, function(x)
				paste(x[c("name", "shortName",
						"type", "groupName")],
					collapse="|"))
			deflist <- apply(defaultEffects, 1, function(x)
				paste(x[c("name", "shortName",
						"type", "groupName")],
					collapse="|"))
			if (!all(userlist %in% deflist))
			{
				bad <- which(!(userlist %in% deflist))
				print(userlist[bad])
				cat("invalid effect requested: see above; \n")
				cat("there seems to be a mismatch between data set and effects object.\n")
				cat("This may have been caused by the use of different versions of RSiena")
				cat("for creating the effects object and now running siena07.\n")
				stop("Try creating the effects object with the current version of RSiena.")
			}
		}
		if (!inherits(effects, "data.frame"))
		{
			stop("effects is not a data.frame")
		}
		if (x$useStdInits)
		{
		# The restriction to effects with shortname not unspInt, behUnspInt, or 
		# contUnspInt is because of the possibility to call getEffects with
		# non-default values of nintn and behNintn.
			effectsr <- (!(effects$shortName %in% c("unspInt","behUnspInt","contUnspInt")))
			defEffectsr <- (!(defaultEffects$shortName %in% c("unspInt","behUnspInt",
			                                                  "contUnspInt")))
			if (any(effects$shortName[effectsr] != defaultEffects$shortName[defEffectsr]))
			{
				cat("There seems to be a mismatch between data set and effects object.\n")
				cat("This may have been caused by the use of different versions of RSiena")
				cat("for creating the effects object and now running siena07.\n")
				cat("Try creating the effects object with the current version of RSiena.")
				stop("Cannot use standard initialisation with a ",
					"different effect list")
			}
			effects$initialValue[effectsr] <- defaultEffects$initialValue[defEffectsr]
		}
		if ((sum(!effects$fix[effects$include]) == 0) & (!x$simOnly))
		{
			stop('All parameters are fixed, none are estimated, but simOnly is FALSE.')
		}
		# Check that the following attributes have correct names
		# and change NULL to default values for x$modelType and x$behModelType
		# For symmetric networks, default is changed below from 1 to 2.
		checkNames(x$MaxDegree, c('oneMode','bipartite'))
		x$modelType <- checkNames(x$modelType, c('oneMode','bipartite'))
		x$behModelType <- checkNames(x$behModelType, 'behavior')
		checkNames(x$UniversalOffset, c('oneMode','bipartite'))
		# The following error will occur if ML estimation is requested
		# and there are any impossible changes from structural values
		# to different observed values.
		if (x$maxlike)
		{
			if (inherits(data, "sienaGroup"))
			{
				impossible <- FALSE
				zerochange <- FALSE
				imp.change <- lapply(data, checkImpossibleChanges)
				if (sum(unlist(imp.change)) > 0)
				{
					cat('For some groups, there are impossible changes;\n')
					cat('This occurs for groups\n', which(sapply(imp.change,
								function(ic){(sum(ic) > 0)})),'\n')
					impossible <- TRUE
				}
				z.changes <- sapply(data, checkZeroChanges)
				if (sum(z.changes >= 1))
				{
					cat(' For some groups and some period, there is no change.\n')
					cat('This occurs for groups ', which(sapply(z.changes,
								function(zc){(zc > 0)})),'\n')
					zerochange <- TRUE
				}
			}
			else
			{
				imp.change <- checkImpossibleChanges(data)
				if (imp.change %in% c(1,3))
				{
					cat('There are some changes from structural zero to observed 1.\n')
				}
				if (imp.change %in% c(2,3))
				{
					cat('There are some changes from structural one to observed 0.\n')
				}
				if (imp.change >= 4)
				{
					cat('There are some impossible changes between structural and observed values,\n')
					cat('some through intermediary NA.\n')
				}
				impossible <- (imp.change > 0)
				zerochange <- FALSE
				z.changes <- checkZeroChanges(data)
				if (sum(z.changes >= 1))
				{
					cat(' For some period, there is no change.\n')
					zerochange <- TRUE
				}
			}
			if (impossible || zerochange)
			{
				cat('This is not allowed for likelihood-based estimation.\n')
				if (impossible)
				{
					cat('Possible remedies are: represent periods by multiple groups,\n')
					cat('or change the offending values.\n')
				}
				stop('Impossible changes')
			}
		}

		## get data object into group format to save coping with two
		## different formats
		if (inherits(data, "sienaGroup"))
		{
			nGroup <- length(data)
		}
		else
		{
			nGroup <- 1
			data <- sienaGroupCreate(list(data), singleOK=TRUE)
		}
		## add any effects needed for time dummies
		tmp <- sienaTimeFix(effects, data)
		data <- tmp$data
		effects <- tmp$effects
		if (!x$useStdInits)
		{
			if (!is.null(prevAns) && inherits(prevAns, "sienaFit"))
			{
				effects <- updateTheta(effects, prevAns)
			}
		}
		## add any effects needed for settings model
		# this now is replaced by adding the settings in getEffects,
		# which is the more logical place.
		# If all works, this can be deleted,
		# and also the function addSettingsEffects can be deleted.
		# I used this function as a template for the change to getEffects.
		# I wonder why the next 8 lines cannot be dropped;
		# gives error message "cannot find setting col".
		#	if (!is.null(x$settings))
		#	{
		#		effects <- addSettingsEffects(effects, x)
		#	}
		#	else
		#	{
		#		effects$setting <- rep("", nrow(effects))
		#	}
		## find any effects not included which are needed for interactions
		tmpEffects <- effects[effects$include, ]
		interactionNos <- unique(c(tmpEffects$effect1, tmpEffects$effect2,
				tmpEffects$effect3))
		interactionNos <- interactionNos[interactionNos > 0]
		interactions <- effects$effectNumber %in%
			interactionNos
		effects$requested <- effects$include
		requestedEffects <- effects[effects$include, ]

		effects$include[interactions] <- TRUE
		effects <- effects[effects$include, ]

		## split and rejoin both versions before continuing
		depvarnames <- names(data[[1]]$depvars)

		if (x$maxlike & (length(depvarnames) > 1))
		{
			if(x$pridg + x$prcdg + x$prper + x$pripr + x$prdpr + x$prirms +
				x$prdrms < 1)
			{
				cat("Likelihood estimation with more than one dependent variable\n")
				cat("is impossible with prML=2. Try prML=1.\n")
				stop("Impossible algorithm-data combination.")
			}
		}

		effects1 <- split(requestedEffects, requestedEffects$name)
		effects1order <- match(c(depvarnames, "sde"), names(effects1))
		requestedEffects <- do.call(rbind, effects1[effects1order])
		row.names(requestedEffects) <- 1:nrow(requestedEffects)

		effects1 <- split(effects, effects$name)
		effects1order <- match(c(depvarnames, "sde"), names(effects1))
		effects <- do.call(rbind, effects1[effects1order])
		row.names(effects) <- 1:nrow(effects)

		## now set up z provisionally
		z$theta <- requestedEffects$initialValue
		z$fixed <- requestedEffects$fix
		z$test <- requestedEffects$test
		z$pp <- length(z$test)
		z$posj <- rep(FALSE, z$pp)
		z$posj[requestedEffects$basicRate] <- TRUE
		z$BasicRateFunction <- z$posj
		z$gmmEffects <- ((requestedEffects$type=="gmm") & requestedEffects$fix) # hhoho
#browser()
		## bugfix to allow use of intitializeFran without siena07
		if(is.null(z$thetaBound)) z$thetaBound <- 50
		if (any(!z$fixed))
		{
			if (max(abs(z$theta[!z$fixed])) > z$thetaBound)
			{
				cat("The maximum absolute value of the initial parameters is", 
						max(abs(z$theta[!z$fixed])), 
						",\nwhich is larger than thetaBound =", z$thetaBound, ".\n")
				stop("thetaBound in the call of siena07 should be set higher.")
			}
		}

		## sort out names of user specified interaction effects
		effects <- fixUpEffectNames(effects)#, defaultEffects)
		## copy interaction names to the requested effects
		requestedEffects$effectName <- effects[effects$requested,
			"effectName"]
		requestedEffects$functionName <- effects[effects$requested,
			"functionName"]

		if (attr(data, "compositionChange"))
		{
			if (x$maxlike)
			{
				stop("Not possible to use maximum likelihood estimation ",
					"with composition change")
			}
		}

		## if not specified whether conditional or nor, set to conditional
		## iff there is only one dependent variable (therefore number 1)
		## and not maxlike
		if (is.na(x$cconditional))
		{
			x$cconditional <- !x$maxlike && (length(depvarnames) == 1)
			if (x$cconditional)
			{
				x$condvarno <- 1
			}
		}

		if (any(hasSettings(data)))
		{
			if (x$cconditional)
			{
				stop("Estimation for a settings model requires ",
						"conditional estimation")
			}
		}
		types <- sapply(data[[1]]$depvars, function(x) attr(x, "type"))
		## now check if conditional estimation is OK and copy to z if so
		z$cconditional <- FALSE
		if ((x$cconditional) & (!(attr(data, "compositionChange"))))
		{
			if (x$maxlike)
			{
				stop("Conditional estimation is not possible with",
					"maximum likelihood method")
			}
			##  if (nets == 1) not sure if this is necessary
			##  {
			z$cconditional <- TRUE
			## find the conditioning variable
			observations <- attr(data, "observations")
			## this is actual number of waves to process
			if (x$condname != "")
			{
				z$condvarno <- match(x$condname, attr(data, "netnames"))
				if (is.na(z$condvarno))
				{
					cat("\nNo match between variable name in algorithm object\n")
					cat("and those in data set.\n")
					cat("Algorithm object: ", x$condname, "\n")
					cat("Data set: ", attr(data, "netnames"), "\n")
					stop("Incorrect variable name.\n")
				}
				z$condname <- x$condname
			}
			else
			{
				z$condvarno <- x$condvarno
				z$condname <- attr(data, "netnames")[x$condvarno]
			}
			z$condtype <- attr(data, "types")[z$condvarno]
			if (z$condtype == "oneMode")
				z$symmetric  <-  attr(data, "symmetric")[[z$condvarno]]
			else
				z$symmetric <- FALSE
			## find the positions of basic rate effects for this network
			z$condvar <-
				(1:nrow(requestedEffects))[requestedEffects$name==
				z$condname][1:observations]
			z$theta<- z$theta[-z$condvar]
			z$fixed<- z$fixed[-z$condvar]
			z$gmmEffects <- z$gmmEffects[-z$condvar]
			z$test<- z$test[-z$condvar]
			z$pp<- z$pp - length(z$condvar)
			z$scale<- z$scale[-z$condvar]
			z$BasicRateFunction <- z$posj[-z$condvar]
			z$posj <- z$posj[-z$condvar]
			z$theta[z$posj] <-
				z$theta[z$posj] / requestedEffects$initialValue[z$condvar]
		}
		z$qq <- z$pp

		## unpack data and put onto f anything we may need next time round.
#		f <- lapply(data, function(xx, x) unpackData(xx, x), x=x)
		f <- lapply(data, function(xx) unpackData(xx))
		attr(f, "netnames") <- attr(data, "netnames")
		attr(f, "symmetric") <- attr(data, "symmetric")
		attr(f, "allUpOnly") <- attr(data, "allUpOnly")
		attr(f, "allDownOnly") <- attr(data, "allDownOnly")
		attr(f, "allHigher") <- attr(data, "allHigher")
		attr(f, "allDisjoint") <- attr(data, "allDisjoint")
		attr(f, "allAtLeastOne") <- attr(data, "allAtLeastOne")
		attr(f, "anyUpOnly") <- attr(data, "anyUpOnly")
		attr(f, "anyDownOnly") <- attr(data, "anyDownOnly")
		attr(f, "anyHigher") <- attr(data, "anyHigher")
		attr(f, "anyDisjoint") <- attr(data, "anyDisjoint")
		attr(f, "anyAtLeastOne") <- attr(data, "anyAtLeastOne")
		attr(f, "types") <- attr(data, "types")
		attr(f, "observations") <- attr(data, "observations")
		attr(f, "compositionChange") <- attr(data, "compositionChange")
		attr(f, "exooptions") <- attr(data, "exooptions")
		attr(f, "groupPeriods") <- attr(data, "groupPeriods")
		attr(f, "periodNos") <- attr(data, "periodNos")
		attr(f, "numberNonMissingNetwork") <-
			attr(data, "numberNonMissingNetwork")
		attr(f, "numberMissingNetwork") <- attr(data, "numberMissingNetwork")
		attr(f, "numberNonMissingBehavior") <-
			attr(data, "numberNonMissingBehavior")
		attr(f, "numberMissingBehavior") <- attr(data, "numberMissingBehavior")

		##  attr(f, "totalMissings") <- attr(data, "totalMissings")

		if (x$maxlike && x$FinDiff.method)
		{
			stop("Finite difference method for derivatives not available",
				"with Maximum likelihood method")
		}
		## maxlike not available for symmetric networks; or is it?.
		## check model type: default for symmetric is type 2 (forcing model).
		## maxlike only for some model types? ABCD
		syms <- attr(data,"symmetric")[ attr(data,"types") %in% c("oneMode","bipartite")]
		z$FinDiffBecauseSymmetric <- FALSE
		z$modelType <- x$modelType
		z$behModelType <- x$behModelType
		if (any(!is.na(syms) & syms))
		{
			##     z$FinDiff.method <- TRUE
			##     z$FinDiffBecauseSymmetric <- TRUE
			if (x$maxlike)
			{
				#                stop("Maximum likelihood method not implemented",
				#                     "for symmetric networks")
			}
			symms <- syms
			symms[is.na(symms)] <- FALSE
			z$modelType[(z$modelType == 1) & symms] <- 2
		}
		if (z$cconditional)
		{
			attr(f, "change") <-
				sapply(f, function(xx)as.integer(attr(xx$depvars[[z$condname]],
							"distance")))
			attr(f,"condEffects") <- requestedEffects[z$condvar,]
			effcondvar <-
				(1:nrow(effects))[effects$name==
				z$condname][1:observations]
			effects <- effects[-effcondvar, ]
			requestedEffects <- requestedEffects[-z$condvar,]
		}
		## use previous dfra only if everything matches including method
		if (!is.null(prevAns) && inherits(prevAns, "sienaFit") &&
			prevAns$maxlike == z$maxlike)
		{
			if (!is.null(prevAns$dfra) &&
				nrow(prevAns$dfra) == nrow(requestedEffects) &&
				all(rownames(prevAns$dfra) == requestedEffects$shortName) &&
				all(prevAns$fix == requestedEffects$fix) &&
				!is.null(prevAns$sf))
			{
				z$haveDfra <- TRUE
				z$dfra <- prevAns$dfra
				z$dinv <- prevAns$dinv
				# z$dinvv must not be taken from prevAns,
				# because the values of diagonalize and splitDepvars
				# is defined in x and may have changed.
				# Therefore here we copy the corresponding lines
				# from phase1.r.
				# Partial diagonalization of derivative matrix
				# for use if 0 < x$diagonalize < 1.
				splitDepvars <- "splitDepvars" %in% names(x)
				if (splitDepvars)
				{
					splitDepvars <- x$splitDepvars
				}
				if (any(!z$gmm)) 
				{
				  temp <- (1-x$diagonalize)*z$dfra +
				    x$diagonalize*diag(diag(z$dfra), nrow=dim(z$dfra)[1])
				  temp[z$fixed, ] <- 0.0
				  temp[, z$fixed] <- 0.0
				  diag(temp)[z$fixed] <- 1.0
				  if (splitDepvars)
				  {	
						temp[outer(effects$name, effects$name, '!=')] <- 0
				  }
				  # Invert this matrix
				  z$dinvv <- solve(temp)
				}
				else
				{
				  z$B <- prevAns$B
				  z$D0 <- prevAns$D0
				  z$gamma <- prevAns$gamma
				  temp <- (1-x$diagonalize)*z$D0 +
				    x$diagonalize*diag(diag(z$D0), nrow=dim(z$D0)[1])
				  temp[z$fixed[!z$gmmEffects], ] <- 0.0
				  temp[, z$fixed[!z$gmmEffects]] <- 0.0
				  diag(temp)[z$fixed[!z$gmmEffects]] <- 1.0
				  # Invert this matrix
				  z$dinvv <- solve(temp)%*%z$B
				}
				z$sf <- prevAns$sf
				# check for backward compatibility with pre-1.1-220 versions:
				if (is.null(prevAns$regrCoef))
				{
					z$regrCoef <- rep(0, z$pp)
					z$regrCor <- rep(0, z$pp)
				}
				else
				{
					z$regrCoef <- prevAns$regrCoef
					z$regrCor <- prevAns$regrCor
				}
			}
		}
		z$effects <- effects
		z$requestedEffects <- requestedEffects
	}
	else ## initC, i.e just send already set up data into new processes
	{
		f <- FRANstore()
		## Would like f to be just the data objects plus the attributes
		## but need the effects later. Also a few other things,
		## which probably could be attributes but are not!
		## They will be automatically removed: if needed they must be readded
		ff <- f
		nGroup <- f$nGroup
		f[(nGroup + 1): length(f)] <- NULL
	}
	pData <- .Call(C_setupData, PACKAGE=pkgname,
		lapply(f, function(x)(as.integer(x$observations))),
		lapply(f, function(x)(x$nodeSets)))
	ans <- .Call(C_OneMode, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$nets))
	ans <- .Call(C_Bipartite, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$bipartites))
	ans <- .Call(C_Behavior, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$behavs))
	ans <- .Call(C_Continuous, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$contbehavs))
	ans <-.Call(C_ConstantCovariates, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$cCovars))
	ans <-.Call(C_ChangingCovariates, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$vCovars))
	ans <-.Call(C_DyadicCovariates, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$dycCovars))
	ans <-.Call(C_ChangingDyadicCovariates, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$dyvCovars))
	ans <-.Call(C_ExogEvent, PACKAGE=pkgname,
		pData, lapply(f, function(x)x$exog))
	## split the names of the constraints
	higher <- attr(f, "allHigher")
	disjoint <- attr(f, "allDisjoint")
	atLeastOne <- attr(f, "allAtLeastOne")
	froms <- sapply(strsplit(names(higher), ","), function(x)x[1])
	tos <- sapply(strsplit(names(higher), ","), function(x)x[2])
	ans <- .Call(C_Constraints, PACKAGE=pkgname,
		pData, froms[higher], tos[higher],
		froms[disjoint], tos[disjoint],
		froms[atLeastOne], tos[atLeastOne])

	##store the address
	f$pData <- pData
	## register a finalizer
	ans <- reg.finalizer(f$pData, clearData, onexit = FALSE)

	if (!initC)
	{
		storage.mode(effects$parm) <- "integer"
		storage.mode(effects$group) <- "integer"
		storage.mode(effects$period) <- "integer"
		effects$effectPtr <- rep(NA, nrow(effects))

		if (any(attr(f,"types") == "continuous"))
		{
			if (z$cconditional)
			{
				stop("For models with continuous behavior variables, conditional estimation is impossible")
			}
			splitFactor <- factor(effects$name, levels=c(attr(f, "netnames"), "sde"))
		}
		else
		{
		splitFactor <- factor(effects$name, levels=attr(f, "netnames"))
		}
		if (!all(attr(f,"netnames") %in% effects$name))
		{
			stop("Must have at least one effect for each dependent variable")
		}
		myeffects <- split(effects, splitFactor)
		myCompleteEffects <- myeffects
		## remove interaction effects and save till later
		basicEffects <-
			lapply(myeffects, function(x)
				{
					x[!x$shortName %in% c("unspInt", "behUnspInt", "contUnspInt"), ]
				}
				)
		basicEffectsl <-
			lapply(myeffects, function(x)
				{
					!x$shortName %in% c("unspInt", "behUnspInt", "contUnspInt")
				}
				)

		interactionEffects <-
			lapply(myeffects, function(x)
				{
					x[x$shortName %in% c("unspInt", "behUnspInt", "contUnspInt"), ]
				}
				)
		interactionEffectsl <-
			lapply(myeffects, function(x)
				{
					x$shortName %in% c("unspInt", "behUnspInt", "contUnspInt")
				}
				)
		## store effects objects as we may need to recreate them
		f$interactionEffects <- interactionEffects
		f$basicEffects <- basicEffects
		f$interactionEffectsl <- interactionEffectsl
		f$basicEffectsl <- basicEffectsl
	}
	else
	{
		myCompleteEffects <- ff$myCompleteEffects
		basicEffects <- ff$basicEffects
		interactionEffects <- ff$interactionEffects
		basicEffectsl <- ff$basicEffectsl
		interactionEffectsl <- ff$interactionEffectsl
		types <- ff$types
	}
	ans <- .Call(C_effects, PACKAGE=pkgname, pData, basicEffects)
	pModel <- ans[[1]][[1]]
	for (i in seq(along=(ans[[2]]))) ## ans[[2]] is a list of lists of
		## pointers to effects. Each list, here i, corresponds to one
		## dependent variable
		## The first are NULL, for rate effects, which are not used for interactions.
	{
		effectPtr <- ans[[2]][[i]]
		basicEffects[[i]]$effectPtr <- effectPtr
		interactionEffects[[i]]$effect1 <-
			basicEffects[[i]]$effectPtr[match(interactionEffects[[i]]$effect1,
				basicEffects[[i]]$effectNumber)]
		interactionEffects[[i]]$effect2 <-
			basicEffects[[i]]$effectPtr[match(interactionEffects[[i]]$effect2,
				basicEffects[[i]]$effectNumber)]
		interactionEffects[[i]]$effect3 <-
			basicEffects[[i]]$effectPtr[match(interactionEffects[[i]]$effect3,
				basicEffects[[i]]$effectNumber)]
	}
	ans <- .Call(C_interactionEffects, PACKAGE=pkgname,
		pModel, interactionEffects)
	## copy these pointers to the interaction effects and then insert in
	## effects object in the same rows for later use
	for (i in 1:length(ans[[1]])) ## ans is a list of lists of
		## pointers to effects. Each list corresponds to one
		## dependent variable
	{
		if (nrow(interactionEffects[[i]]) > 0)
		{
			effectPtr <- ans[[1]][[i]]
			interactionEffects[[i]]$effectPtr <- effectPtr
		}
		myCompleteEffects[[i]][basicEffectsl[[i]], ] <- basicEffects[[i]]
		myCompleteEffects[[i]][interactionEffectsl[[i]],] <-
			interactionEffects[[i]]
		##myeffects[[i]] <- myeffects[[i]][order(myeffects[[i]]$effectNumber),]
	}
	## remove the effects only created as underlying effects
	## for interaction effects. first store the original for use next time
	myeffects <- lapply(myCompleteEffects, function(x)
		{
			x[x$requested, ]
		}
		)

	##store address of model
	f$pModel <- pModel
	ans <- reg.finalizer(f$pModel, clearModel, onexit = FALSE)
	if (all(x$MaxDegree == 0) || is.null(x$MaxDegree))
	{
		MAXDEGREE <- NULL
	}
	else
	{
		MAXDEGREE <- x$MaxDegree
		storage.mode(MAXDEGREE) <- "integer"
	}
	if (all(x$UniversalOffset == 0) || is.null(x$UniversalOffset))
	{
		UNIVERSALOFFSET <- NULL
	}
	else
	{
		UNIVERSALOFFSET <- x$UniversalOffset
		storage.mode(UNIVERSALOFFSET) <- "double"
	}
	if ((length(x$modelType) == 0)|| (all(x$modelType == 0)) || is.null(x$modelType))
	{
		MODELTYPE <-  NULL
	}
	else
	{
		MODELTYPE <- x$modelType
		storage.mode(MODELTYPE) <- "integer"
	}
	if ((length(x$behModelType) == 0)|| (all(x$behModelType == 0)) || is.null(x$behModelType))
	{
		BEHMODELTYPE <-  NULL
	}
	else
	{
		BEHMODELTYPE <- x$behModelType
		storage.mode(BEHMODELTYPE) <- "integer"
	}
	if (z$cconditional)
	{
		CONDVAR <- z$condname
		CONDTARGET <- attr(f, "change")
		##   cat(CONDTARGET, "\n")
	}
	else
	{
		CONDVAR <- NULL
		CONDTARGET <- NULL
	}

	simpleRates <- TRUE
	if (any(!z$effects$basicRate & z$effects$type =="rate"))
	{
		simpleRates <- FALSE
	}
	z$simpleRates <- simpleRates
	ans <- .Call(C_setupModelOptions, PACKAGE=pkgname,
		pData, pModel, MAXDEGREE, UNIVERSALOFFSET, CONDVAR, CONDTARGET,
		profileData, z$parallelTesting, MODELTYPE, BEHMODELTYPE,
		z$simpleRates, x$normSetRates)
	if (!initC)
	{
		ans <- .Call(C_getTargets, PACKAGE=pkgname, pData, pModel, myeffects,
			z$parallelTesting, returnActorStatistics=FALSE,
			returnStaticChangeContributions=FALSE)
		##stop("done")
		## create a grid of periods with group names in case want to
		## parallelize using this or to access chains easily
		groupPeriods <- attr(f, "groupPeriods")
		z$callGrid <- cbind(rep(1:nGroup, groupPeriods - 1),
			as.vector(unlist(sapply(groupPeriods - 1,
						function(x) 1:x))))
		if (!x$maxlike)
		{
			z$targets <- rowSums(ans)
			attr(z$targets, "fromData") <- TRUE
			z$targets2 <- ans
# If targets is given in the call of siena07,
# this replaces the targets calculated from the data
# by user-defined targets.
			if ((!is.null(x$targets[1])) & (nGroup == 1) & (groupPeriods[1] == 2))
			{
				if (length(z$targets) == length(x$targets))
				{
					z$targets <- x$targets
					z$targets2 <- matrix(x$targets, length(x$targets), 1)
					attr(z$targets, "fromData") <- FALSE
					message('Note: targets used as given in call of siena07.')
					cat('\n')
					print(z$targets)
					cat('\n')
				}
				else
				{
					message("length of given targets = ", x$targets, ",")
					message("but there are ", length(z$targets), " parameters to be estimated.")
					warning("targets as given in the call of siena07 have incorrect length")
				}
			}
		}
		else
		{
			z$targets <- rep(0, z$pp)			
			attr(z$targets, "fromData") <- TRUE
			z$targets2 <- ans
			z$targets2[] <- 0
			z$maxlikeTargets <- rowSums(ans)
			z$maxlikeTargets2 <- ans
			z$mult <- x$mult
			length.nrunMH <- length(colSums(z$maxlikeTargets2[z$requestedEffects$basicRate,
					, drop=FALSE ]))
			if ((length(z$mult) >= 2) &&
				(length(z$mult) != length.nrunMH))
			{
				stop(paste('Length of parameter mult in the algorithm object is incorrect',
				'(should be 1 or', length.nrunMH, ').'))
			}
			z$nrunMH <- floor(z$mult *
				colSums(z$maxlikeTargets2[z$requestedEffects$basicRate,
					, drop=FALSE ]))
			## make the number pretty
			z$nrunMH <- ifelse (z$nrunMH > 100,
				round(z$nrunMH / 100) * 100, z$nrunMH)
			##thetaMat is to allow different thetas for each group in Bayes
			z$thetaMat <- matrix(z$theta, nrow=nGroup, ncol=z$pp, byrow=TRUE)
		}
	}
	if (x$maxlike)
	{
		if (!initC)
		{
			## set up chains and do initial steps

			types <- attr(f, "types")
			nbrNonMissNet <- attr(f, "numberNonMissingNetwork")
			nbrMissNet <- attr(f, "numberMissingNetwork")
			nbrNonMissBeh <- attr(f, "numberNonMissingBehavior")
			nbrMissBeh <- attr(f, "numberMissingBehavior")

			if (sum(nbrMissNet + nbrNonMissNet) > 0)
			{
				z$prmin <- nbrMissNet/ (nbrMissNet + nbrNonMissNet)
			}
			else
			{
				z$prmin <- rep(0, length(nbrMissNet))
			}
			if (sum(nbrMissBeh + nbrNonMissBeh) > 0)
			{
				z$prmib <-   nbrMissBeh/ (nbrMissBeh + nbrNonMissBeh)
			}
			else
			{
				z$prmib <- rep(0, length(nbrMissBeh))
			}

			## localML

			if (is.null(x$localML))
			{
				z$localML <- FALSE
			} else {
				z$localML <- x$localML
			}

			local <- ifelse(is.na(effects$local[effects$include]),
				FALSE, effects$local[effects$include])

			if (z$localML & any(!local))
			{
				stop("Non-local effect chosen.")
			}

			z$probs <- c(x$pridg, x$prcdg, x$prper, x$pripr, x$prdpr, x$prirms,
				x$prdrms)
			ans <- .Call(C_mlMakeChains, PACKAGE=pkgname, pData, pModel,
				z$probs, z$prmin, z$prmib,
				x$minimumPermutationLength,
				x$maximumPermutationLength,
				x$initialPermutationLength,
				z$localML)
			f$minimalChain <- ans[[1]]
			f$chain <- ans[[2]]
		}
		else ## set up the initial chains in the sub processes
		{
			ans <- .Call(C_mlInitializeSubProcesses,
				PACKAGE=pkgname, pData, pModel,
				z$probs, z$prmin, z$prmib,
				x$minimumPermutationLength,
				x$maximumPermutationLength,
				x$initialPermutationLength, ff$chain,
				z$localML)
			f$chain <- ff$chain
		}
	}

	f$simpleRates <- simpleRates
	f$myeffects <- myeffects
	f$myCompleteEffects <- myCompleteEffects
	if (!initC)
	{
		if (is.null(z$print) || z$print)
		{
			DataReport(z, x, f)
		}
		f$randomseed2 <- z$randomseed2
	}
	else
	{
		f$randomseed2 <- ff$randomseed2
	}
	f$observations <- attr(f, "observations") + 1
	f$depNames <- names(f[[1]]$depvars)
	f$groupNames <- names(f)[1:nGroup]
	f$nGroup <- nGroup
	f$types <- types
	if (!initC)
	{
		z$f <- f
		##z <- initForAlgorithms(z)
		z$periodNos <- attr(data, "periodNos")
		z$f$myeffects <- NULL
		z$f$myCompleteEffects <- NULL
		if (!returnDeps && !any(z$f$types == "continuous"))
		{
			z$f[1:nGroup] <- NULL
		}
	}
	if (initC || (z$int == 1 && z$int2 == 1 &&
			(is.null(z$nbrNodes) || z$nbrNodes == 1)))
	{
		f[1:nGroup] <- NULL
	}
	FRANstore(f) ## store f in FRANstore
	z$returnDeps <- returnDeps
	z$returnDepsStored <- returnDeps
	z$observations <- f$observations
	z$returnChains <- returnChains
	z$returnChangeContributions <- returnChangeContributions
	z$byGroup <- byGroup
	z$byWave <- byWave
	z$returnDataFrame <- returnDataFrame
	z$nDependentVariables <- length(z$f$depNames)
	z$x <- x # some elements -- named vectors -- may have changed
	if (initC)
	{
		NULL
	}
	else
	{
		z
	}
}

##@createEdgeLists siena07 Reformat data for C++
createEdgeLists<- function(mat, matorig, bipartite)
{
	## mat1 is basic values, with missings and structurals replaced
	tmp <- lapply(1 : nrow(mat), function(x, y)
		{
			mymat <- matrix(0, nrow = sum(y[x, ] > 0), ncol = 3)
			mymat[, 1] <- x
			mymat[, 2] <- which(y[x, ] != 0)
			mymat[, 3] <- y[x, mymat[, 2]]
			mymat
		}, y = mat)
	mat1 <- do.call(rbind, tmp)
	## mat2 reverts to matorig to get the missing values
	tmp <- lapply(1 : nrow(matorig), function(x, y)
		{
			mymat <- matrix(0, nrow = sum(is.na(y[x, ])), ncol = 3)
			mymat[, 1] <- x
			mymat[, 2] <- which(is.na(y[x, ]))
			mymat[, 3] <- 1
			mymat
		}, y = matorig)
	mat2 <- do.call(rbind, tmp)
	## remove the diagonal if not bipartite
	if (!bipartite)
	{
		mat2 <- mat2[mat2[, 1] != mat2[, 2], , drop=FALSE]
	}
	## mat3 structurals
	struct <- mat1[,3] %in% c(10, 11)
	mat1[struct, 3] <- mat1[struct,3] - 10
	mat3 <- mat1[struct, , drop=FALSE]
	mat3[, 3] <- 1
	mat1 <- mat1[!mat1[,3] == 0, , drop=FALSE] ##remove any zeros just created
	##fix up storage mode to be integer
	storage.mode(mat1) <- "integer"
	storage.mode(mat2) <- "integer"
	storage.mode(mat3) <- "integer"
	## add attribute of size
	if (bipartite)
	{
		attr(mat1, "nActors") <- c(nrow(mat), ncol(mat))
		attr(mat2, "nActors") <- c(nrow(mat), ncol(mat))
		attr(mat3, "nActors") <- c(nrow(mat), ncol(mat))
	}
	else
	{
		attr(mat1, "nActors") <- nrow(mat)
		attr(mat2, "nActors") <- nrow(mat)
		attr(mat3, "nActors") <- nrow(mat)
	}

	list(mat1 = t(mat1), mat2 = t(mat2), mat3 = t(mat3))
}

##@createCovarEdgeLists siena07 Reformat data for C++
createCovarEdgeList<- function(mat, matorig)
{
	tmp <- lapply(1 : nrow(mat), function(x, y)
		{
			mymat <- matrix(0, nrow = sum(y[x, ] != 0), ncol = 3)
			mymat[, 1] <- x
			mymat[, 2] <- which(y[x, ] != 0)
			mymat[, 3] <- y[x, mymat[, 2]]
			mymat
		}, y = mat)
	mat1 <- do.call(rbind, tmp)
	##mat2 reverts to matorig to get the missing values
	tmp <- lapply(1 : nrow(matorig), function(x, y)
		{
			mymat <- matrix(0, nrow = sum(is.na(y[x, ])), ncol = 3)
			mymat[, 1] <- x
			mymat[, 2] <- which(is.na(y[x, ]))
			mymat[, 3] <- 1
			mymat
		}, y = matorig)
	mat2 <- do.call(rbind, tmp)
	## add attribute of size
	attr(mat1, "nActors1") <- nrow(mat)
	attr(mat1, "nActors2") <- ncol(mat)
	list(mat1=t(mat1), mat2=t(mat2))
}

##@unpackOneMode siena07 Reformat data for C++
unpackOneMode <- function(depvar, observations, compositionChange)
{
	edgeLists <- vector("list", observations)
	networks <- vector("list", observations)
	actorSet <- attr(depvar, "nodeSet")
	compActorSets <- sapply(compositionChange, function(x)attr(x, "nodeSet"))
	thisComp <- match(actorSet, compActorSets)
	compChange <- !is.na(thisComp)
	if (compChange)
	{
		action <- attr(compositionChange[[thisComp]], "action")
		ccOption <- attr(compositionChange[[thisComp]], "ccOption")
	}
	else
	{
		ccOption <- 0
		action <- matrix(0, nrow=attr(depvar, "netdims")[1], ncol=observations)
	}
	## sort out composition change
	##      convertToStructuralZeros()?
	sparse <- attr(depvar, "sparse")
	allowOnly <- attr(depvar, "allowOnly")
	if (sparse)
	{
		## require(Matrix)
		## have a list of sparse matrices in triplet format
		## with missings and structurals embedded and 0 based indices!
		netmiss <- vector("list", observations)
		for (i in 1:observations)
		{
			## extract this matrix
			networks[[i]] <- depvar[[i]]
			nActors <- nrow(depvar[[i]])
			## stop if any duplicates
			netmat <- cbind(networks[[i]]@i+1, networks[[i]]@j+1,
				networks[[i]]@x)
			if (any(duplicated(netmat[, 1:2])))
			{
				stop("duplicate entries in sparse matrix")
			}
			## extract missing entries
			netmiss[[i]] <- netmat[is.na(netmat[,3]), , drop = FALSE]
			netmiss[[i]] <-
				netmiss[[i]][netmiss[[i]][, 1] != netmiss[[i]][, 2], ,
			drop=FALSE]
			## carry forward missing values if any
			if (i == 1)
			{
				netmat <- netmat[!is.na(netmat[,3]), , drop = FALSE]
				networks[[i]] <- spMatrix(nActors, nActors, netmat[, 1],
					netmat[, 2], netmat[,3])
			}
			else
			{
				netmiss1 <- netmiss[[i]][, 1:2]
				storage.mode(netmiss1) <- "integer"
				networks[[i]][netmiss1[, 1:2]] <-
					networks[[i-1]][netmiss1[, 1:2]]
			}
		}
		for (i in 1:observations)
		{
			mat1 <- networks[[i]]
			## drop the diagonal, if present
			diag(mat1) <- 0
			mat1 <- cbind(mat1@i + 1, mat1@j + 1, mat1@x)
			##missing edgelist
			mat2 <- netmiss[[i]]
			mat2[, 3] <- 1
			## rows of mat1 with structural values
			struct <- mat1[, 3] %in% c(10, 11)
			## reset real data
			mat1[struct, 3] <- mat1[struct, 3] - 10
			## copy reset data to structural edgelist
			mat3 <- mat1[struct, , drop = FALSE]
			mat3[, 3] <- 1
			## now remove the zeros from reset data
			mat1 <- mat1[!mat1[, 3] == 0, , drop=FALSE]
			## do comp change
			if (compChange)
			{
				## revert to sparse matrices temporarily
				mat1 <- spMatrix(nrow=nActors, ncol=nActors, i = mat1[, 1],
					j=mat1[, 2], x=mat1[, 3])
				mat2 <- spMatrix(nrow=nActors, ncol=nActors, i = mat2[, 1],
					j=mat2[, 2], x=mat2[, 3])
				mat3 <- spMatrix(nrow=nActors, ncol=nActors, i = mat3[, 1],
					j=mat3[, 2], x=mat3[, 3])
				ones <- which(action[, i] == 1)
				twos <- which(action[, i] == 2)
				threes <- which(action[, i] == 3)
				for (j in ones) ## False data is not preceded by anything real
				{
					if (ccOption %in% c(1, 2))
					{
						## find missing values for this actor
						use <- mat2[j, ] > 0
						## remove from real data (i.e. zero)
						mat1[j, use] <- 0
						mat1[use, j] <- 0
						## remove from missing data
						mat2[j, use] <- 0
						mat2[use, j] <- 0
						## remove from raw data for distances later
						depvar[[i]][j, use] <- 0 ## zero
						depvar[[i]][use, j] <- 0
						depvar[[i]][j, j] <- NA
					}
					else if (ccOption == 3)
					{
						## add the row and column to the missing data
						mat2[j, ] <- 1
						mat2[, j] <- 1
						mat2[j, j] <- 0
						## set to missing in raw data for distances later
						depvar[[i]][j, ] <- NA
						depvar[[i]][, j] <- NA
					}
				}
				for (j in threes) ## False data is preceded and followed by real
				{
					if (ccOption %in% c(1, 2))
					{
						## find missing values for this actor
						use <- mat2[j, ] > 0
						## remove these from mat2, the missing data
						mat2[j, use] <- 0
						mat2[use, j] <- 0
						## carry forward
						if (i == 1)
						{
							## 0 any matches from mat1, the real data
							mat1[j, use] <- 0
							mat1[use, j] <- 0
						}
						else
						{
							mat1[j, use] <- networks[[i-1]][j, use]
							mat1[use, j] <- networks[[i-1]][use, j]
						}
						depvar[[i]][j, use] <- 0 ##  not missing
						depvar[[i]][use, j] <- 0
						depvar[[i]][j, j] <- NA
					}
					else if (ccOption == 3)
					{
						## add the row and column to the missing data
						mat2[j, ] <- 1
						mat2[, j] <- 1
						mat2[j, j] <- 0
						depvar[[i]][j, ] <- NA
						depvar[[i]][, j] <- NA
					}
				}
				for (j in twos) ## False data is not followed by anything real
				{
					if (ccOption == 1)
					{
						## find missing values for this actor
						use <- mat2[j, ] > 0
						## remove these from mat2, the missing data
						mat2[j, use] <- 0
						mat2[use, j] <- 0
						depvar[[i]][j, use] <- 0 ##  not missing
						depvar[[i]][use, j] <- 0
						depvar[[i]][j, j] <- NA
						## carry forward
						if (i == 1)
						{
							## 0 any matches from mat1, the real data
							mat1[j, use] <- 0
							mat1[use, j] <- 0
						}
						else
						{
							mat1[j, use] <- networks[[i-1]][j , use]
							mat1[use, j] <- networks[[i-1]][use, j]
						}
					}
					else if (ccOption %in% c(2, 3))
					{
						## add the row and column to the missing data
						mat2[j, ] <- 1
						mat2[, j] <- 1
						mat2[j, j] <- 0
						depvar[[i]][j, ] <- NA
						depvar[[i]][, j] <- NA
					}
				}
				## now revert to triplet matrices, after updating networks
				networks[[i]] <- mat1
				mat1 <- cbind(mat1@i + 1, mat1@j + 1, mat1@x)
				mat2 <- cbind(mat2@i + 1, mat2@j + 1, mat2@x)
				mat3 <- cbind(mat3@i + 1, mat3@j + 1, mat3@x)
				if (any (mat1[, 3] == 0) || any (mat2[, 3] == 0) ||
					any (mat3[, 3] == 0))
				{
					stop("zero values in sparse matrices")
				}
				if (any (duplicated(mat1[, -3])) ||
					any (duplicated(mat2[, -3])) ||
					any (duplicated(mat3[, -3])))
				{
					stop("duplicate values in sparse matrices")
				}
				if (any (mat1[, 1] == mat1[, 2]) ||
					any (mat2[, 1] == mat2[, 2]) ||
					any (mat3[, 1] == mat3[, 2]))
				{
					stop("loop values in sparse matrices")
				}
			}
			##fix up storage mode to be integer
			storage.mode(mat1) <- "integer"
			storage.mode(mat2) <- "integer"
			storage.mode(mat3) <- "integer"
			## add attribute of size
			attr(mat1,"nActors") <- nActors
			attr(mat2,"nActors") <- nActors
			attr(mat3,"nActors") <- nActors
			if (i < observations)
			{
				## recreate the distance etc
				mymat1 <- depvar[[i]]
				mymat2 <- depvar[[i + 1]]
				##remove structural values
				x1 <- mymat1@x
				x2 <- mymat2@x
				x1[x1 %in% c(10, 11)] <- NA
				x2[x2 %in% c(10, 11)] <- NA
				mymat1@x <- x1
				mymat2@x <- x2
				diag(mymat1) <- 0
				diag(mymat2) <- 0
				mydiff <- mymat2 - mymat1
				attr(depvar, "distance")[i] <- sum(mydiff != 0,
					na.rm = TRUE)
				if (allowOnly)
				{
					if (all(mydiff@x >= 0, na.rm=TRUE))
					{
						attr(depvar, "uponly")[i] <- TRUE
					}
					if (all(mydiff@x <= 0, na.rm=TRUE))
					{
						attr(depvar, "downonly")[i] <- TRUE
					}
				}
			}
			edgeLists[[i]] <- list(mat1 = t(mat1), mat2 = t(mat2),
				mat3 = t(mat3))				
		}
	}
	else
	{
		for (i in 1:observations) ## carry missings forward if exist
		{
			networks[[i]] <- depvar[, , i]
			if (i == 1)
				networks[[i]][is.na(depvar[, , i])] <-0
			else ##carry missing forward!
				networks[[i]][is.na(depvar[, , i])] <-
					networks[[i-1]][is.na(depvar[, , i])]
		}
		for (i in 1:observations)
		{
			ones <- which(action[, i] == 1)
			twos <- which(action[, i] == 2)
			threes <- which(action[, i] == 3)
			for (j in ones) ## False data is not preceded by anything real
			{
				if (ccOption %in% c(1, 2))
				{
					use <- is.na(depvar[j, , i])
					depvar[j, use, i] <- 0 ## not missing
					depvar[use, j, i] <- 0
					depvar[j, j, i] <- NA
					networks[[i]][j, use] <- 0 ## zero
					networks[[i]][use, j] <- 0
				}
				else if (ccOption == 3)
				{
					depvar[j, , i] <- NA ## missing
					depvar[, j, i] <- NA
				}
			}
			for (j in threes) ## False data is preceded and followed by real
			{

				if (ccOption %in% c(1, 2))
				{
					use <- is.na(depvar[j, , i])
					depvar[j, use, i] <- 0 ##  not missing
					depvar[use, j, i] <- 0
					depvar[j, j, i] <- NA
					## carry forward already done
					if (i == 1)
					{
						networks[[i]][j, use] <- 0
						networks[[i]][use, j] <- 0
					}
					else
					{
						networks[[i]][j, use] <- networks[[i-1]][j, use]
						networks[[i]][use, j] <- networks[[i-1]][use, j]
					}
				}
				else if (ccOption == 3)
				{
					depvar[j, , i] <- NA ## missing
					depvar[, j, i] <- NA
				}
			}
			for (j in twos) ## False data is not followed by anything real
			{
				if (ccOption == 1)
				{
					use <- is.na(depvar[j, , i])
					depvar[j, use, i] <- 0 ##  not missing
					depvar[use, j, i] <- 0
					depvar[j, j, i] <- NA
					## carry forward already done
					if (i == 1)
					{
						networks[[i]][j, use] <- 0
						networks[[i]][use, j] <- 0
					}
					else
					{
						networks[[i]][j, use] <- networks[[i-1]][j, use]
						networks[[i]][use, j] <- networks[[i-1]][use, j]

					}
				}
				else if (ccOption %in% c(2, 3))
				{
					depvar[j, , i] <- NA ## missing
					depvar[, j, i] <- NA
				}
			}
		}
		for (i in 1:observations)
		{
			if (i < observations)
			{
				## recreate distances, as we have none in c++. (no longer true)
				mymat1 <- depvar[, ,i, drop=FALSE]
				mymat2 <- depvar[, ,i + 1, drop=FALSE]
				##remove structural values
				mymat1[mymat1 %in% c(10, 11)] <- NA
				mymat2[mymat2 %in% c(10, 11)] <- NA
				## and the diagonal
				diag(mymat1[, ,1]) <- 0
				diag(mymat2[, ,1]) <- 0
				mydiff <- mymat2 - mymat1
				attr(depvar, "distance")[i] <- sum(mydiff != 0,
					na.rm = TRUE)
				if (allowOnly)
				{
					if (all(mydiff >= 0, na.rm=TRUE))
					{
						attr(depvar, "uponly")[i] <- TRUE
					}
					if (all(mydiff <= 0, na.rm=TRUE))
					{
						attr(depvar, "downonly")[i] <- TRUE
					}
				}
			}
			diag(networks[[i]]) <- 0
			edgeLists[[i]] <- createEdgeLists(networks[[i]], depvar[, , i],
				FALSE)
		}
	}
	## add attribute of nodeset
	attr(edgeLists, "nodeSet") <- attr(depvar, "nodeSet")
	## add attribute of name
	attr(edgeLists, "name") <- attr(depvar, "name")
	## add attribute of distance
	attr(edgeLists, "distance") <- attr(depvar, "distance")
	## attr uponly and downonly
	attr(edgeLists, "uponly") <- attr(depvar, "uponly")
	attr(edgeLists, "downonly") <- attr(depvar, "downonly")
	## attr symmetric
	attr(edgeLists, "symmetric") <- attr(depvar, "symmetric")
	## attr balmean
	attr(edgeLists, "balmean") <- attr(depvar, "balmean")
	attr(edgeLists, "structmean") <- attr(depvar, "structmean")
	attr(edgeLists, "averageInDegree") <- attr(depvar, "averageInDegree")
	attr(edgeLists, "averageOutDegree") <- attr(depvar, "averageOutDegree")
	attr(edgeLists, "settingsinfo") <- attr(depvar, "settingsinfo")
	return(edgeLists = edgeLists)
}
##@unpackBipartite siena07 Reformat data for C++
unpackBipartite <- function(depvar, observations, compositionChange)
{
    edgeLists <- vector("list", observations)
    networks <- vector("list", observations)
    actorSet <- attr(depvar, "nodeSet")
    compActorSets <- sapply(compositionChange, function(x)attr(x, "nodeSet"))
    thisComp <- match(actorSet, compActorSets)
    compChange <- any(!is.na(thisComp))
    if (compChange)
    {
      #  stop("Composition change is not yet implemented for bipartite",
      #       "networks")
        action <- attr(compositionChange[[thisComp]], "action")
        ccOption <- attr(compositionChange[[thisComp]], "ccOption")
    }
    else
    {
        ccOption <- 0
        action <- matrix(0, nrow=attr(depvar, "netdims")[1], ncol=observations)
    }
    sparse <- attr(depvar, "sparse")
	allowOnly <- attr(depvar, "allowOnly")
    if (sparse)
    {
        ## require(Matrix)
        ## have a list of sparse matrices in triplet format
        ## with missings and structurals embedded and 0 based indices!
        netmiss <- vector("list", observations)
        for (i in 1:observations)
        {
            ## extract this matrix
            networks[[i]] <- depvar[[i]]
            nActors <- nrow(depvar[[i]])
            nReceivers <- ncol(depvar[[i]])
            ## stop if any duplicates
            netmat <- cbind(networks[[i]]@i+1, networks[[i]]@j+1,
                            networks[[i]]@x)
            if (any(duplicated(netmat[, 1:2])))
            {
                stop("duplicate entries in sparse matrix")
            }
            ## extract missing entries
            netmiss[[i]] <- netmat[is.na(netmat[, 3]), , drop = FALSE]
            ## carry forward missing values if any
            if (i == 1) # set missings to zero
            {
                netmat <- netmat[!is.na(netmat[,3]), ]
                networks[[i]] <- spMatrix(nActors, nReceivers, netmat[, 1],
                                          netmat[, 2], netmat[,3])
            }
            else
            {
                netmiss1 <- netmiss[[i]][, 1:2]
                storage.mode(netmiss1) <- "integer"
                networks[[i]][netmiss1[, 1:2]] <-
                    networks[[i-1]][netmiss1[, 1:2]]
            }
        }
        for (i in 1:observations)
        {
            mat1 <- networks[[i]]
            mat1 <- cbind(mat1@i + 1, mat1@j + 1, mat1@x)
            ##missing edgelist
            mat2 <- netmiss[[i]]
            mat2[, 3] <- 1
            ## rows of mat1 with structural values
            struct <- mat1[, 3] %in% c(10, 11)
            ## reset real data
            mat1[struct, 3] <- mat1[struct, 3] - 10
            ## copy reset data to structural edgelist
            mat3 <- mat1[struct, , drop = FALSE]
            ## now remove the zeros from reset data
            mat1 <- mat1[!mat1[, 3] == 0, ]
            ## do comp change
            if (compChange)
            {
                ## revert to sparse matrices temporarily
                mat1 <- spMatrix(nrow=nActors, ncol=nReceivers, i = mat1[, 1],
                                 j=mat1[, 2], x=mat1[, 3])
                mat2 <- spMatrix(nrow=nActors, ncol=nReceivers, i = mat2[, 1],
                                 j=mat2[, 2], x=mat2[, 3])
                mat3 <- spMatrix(nrow=nActors, ncol=nReceivers, i = mat3[, 1],
                                 j=mat3[, 2], x=mat3[, 3])
                ones <- which(action[, i] == 1)
                twos <- which(action[, i] == 2)
                threes <- which(action[, i] == 3)
                for (j in ones) ## False data is not preceded by anything real
                {
                    if (ccOption %in% c(1, 2))
                    {
                        ## find missing values for this actor
                        use <- mat2[j, ] > 0
                        ## remove from real data (i.e. zero)
                        mat1[j, use] <- 0
                        ## remove from missing data
                        mat2[j, use] <- 0
                        ## remove from raw data for distances later
                        depvar[[i]][j, use] <- 0 ## zero
                    }
                    else if (ccOption == 3)
                    {
                        ## add the row  to the missing data
                        mat2[j, ] <- 1
                        ## set to missing in raw data for distances later
                        depvar[[i]][j, ] <- NA
                    }
                }
                for (j in threes) ## False data is preceded and followed by real
                {
                    if (ccOption %in% c(1, 2))
                    {
                        ## find missing values for this actor
                        use <- mat2[j, ] > 0
                        ## remove these from mat2, the missing data
                        mat2[j, use] <- 0
                        ## carry forward
                        if (i == 1)
                        {
                            ## 0 any matches from mat1, the real data
                            mat1[j, use] <- 0
                        }
                        else
                        {
                            mat1[j, use] <- networks[[i-1]][j, use]
                        }
                        depvar[[i]][j, use] <- 0 ##  not missing
                    }
                    else if (ccOption == 3)
                    {
                        ## add the row to the missing data
                        mat2[j, ] <- 1
                        depvar[[i]][j, ] <- NA
                    }
                }
                for (j in twos) ## False data is not followed by anything real
                {
                    if (ccOption == 1)
                    {
                        ## find missing values for this actor
                        use <- mat2[j, ] > 0
                        ## remove these from mat2, the missing data
                        mat2[j, use] <- 0
                        depvar[[i]][j, use] <- 0 ##  not missing
                        ## carry forward
                        if (i == 1)
                        {
                            ## 0 any matches from mat1, the real data
                            mat1[j, use] <- 0
                        }
                        else
                        {
                            mat1[j, use] <- networks[[i-1]][j , use]
                        }
                    }
                    else if (ccOption %in% c(2, 3))
                    {
                        ## add the row  to the missing data
                        mat2[j, ] <- 1
                        depvar[[i]][j, ] <- NA
                    }
                }

                ## now revert to triplet matrices, after updating networks
                networks[[i]] <- mat1
                mat1 <- cbind(mat1@i + 1, mat1@j + 1, mat1@x)
                mat2 <- cbind(mat2@i + 1, mat2@j + 1, mat2@x)
                mat3 <- cbind(mat3@i + 1, mat3@j + 1, mat3@x)
                if (any (mat1[, 3] == 0) || any (mat2[, 3] == 0) ||
                    any (mat3[, 3] == 0))
                {
                    stop("zero values in sparse matrices")
                }
                if (any (duplicated(mat1[, -3])) ||
                    any (duplicated(mat2[, -3])) ||
                    any (duplicated(mat3[, -3])))
                {
                    stop("duplicate values in sparse matrices")
                }
            }
            ##fix up storage mode to be integer
            storage.mode(mat1) <- "integer"
            storage.mode(mat2) <- "integer"
            storage.mode(mat3) <- "integer"
            ## add attribute of size
            attr(mat1,"nActors") <- c(nActors, nReceivers)
            attr(mat2,"nActors") <- c(nActors, nReceivers)
            attr(mat3,"nActors") <- c(nActors, nReceivers)
            if (i < observations)
            {
                ## recreate the distance etc
                mymat1 <- depvar[[i]]
                mymat2 <- depvar[[i + 1]]
                ##remove structural values
                x1 <- mymat1@x
                x2 <- mymat2@x
                x1[x1 %in% c(10, 11)] <- NA
                x2[x2 %in% c(10, 11)] <- NA
                mymat1@x <- x1
                mymat2@x <- x2
                mydiff <- mymat2 - mymat1
                attr(depvar, "distance")[i] <- sum(mydiff != 0,
                                                   na.rm = TRUE)
				if (allowOnly)
				{
					if (all(mydiff@x >= 0, na.rm=TRUE))
					{
						attr(depvar, "uponly")[i] <- TRUE
					}
					if (all(mydiff@x <= 0, na.rm=TRUE))
					{
						attr(depvar, "downonly")[i] <- TRUE
					}
				}
            }
            edgeLists[[i]] <- list(mat1 = t(mat1), mat2 = t(mat2),
                                   mat3 = t(mat3))
        }
    }
    else
    {
        for (i in 1:observations) ## carry missings forward  if exist
        {
            networks[[i]] <- depvar[, , i]
            if (i == 1)
                networks[[i]][is.na(depvar[, , i])] <-0
            else ##carry missing forward!
                networks[[i]][is.na(depvar[, , i])] <-
                    networks[[i-1]][is.na(depvar[, , i])]
        }
        for (i in 1:observations)
        {
            ones <- which(action[, i] == 1)
            twos <- which(action[, i] == 2)
            threes <- which(action[, i] == 3)
            for (j in ones) ## False data is not preceded by anything real
            {
                if (ccOption %in% c(1, 2))
                {
                    use <- is.na(depvar[j, , i])
                    depvar[j, use, i] <- 0 ## not missing
                    networks[[i]][j, use] <- 0 ## zero
                }
                else if (ccOption == 3)
                {
                    depvar[j, , i] <- NA ## missing
                }
            }
            for (j in threes) ## False data is preceded and followed by real
            {

                if (ccOption %in% c(1, 2))
                {
                    use <- is.na(depvar[j, , i])
                    depvar[j, use, i] <- 0 ##  not missing
                    ## carry forward already done
                    if (i == 1)
                    {
                        networks[[i]][j, use] <- 0
                    }
                    else
                    {
                        networks[[i]][j, use] <- networks[[i-1]][j, use]
                    }
                }
                else if (ccOption == 3)
                {
                    depvar[j, , i] <- NA ## missing
                }
            }
            for (j in twos) ## False data is not followed by anything real
            {
                if (ccOption == 1)
                {
                    use <- is.na(depvar[j, , i])
                    depvar[j, use, i] <- 0 ##  not missing
                    ## carry forward already done
                    if (i == 1)
                    {
                        networks[[i]][j, use] <- 0
                    }
                    else
                    {
                        networks[[i]][j, use] <- networks[[i-1]][j, use]

                    }
                }
                else if (ccOption %in% c(2, 3))
                {
                    depvar[j, , i] <- NA ## missing
                }
            }
        }
        for (i in 1:observations)
        {
            if (i < observations)
            {
                ## recreate distances, as we have none in c++. (no longer true)
                mymat1 <- depvar[,,i, drop=FALSE]
                mymat2 <- depvar[,,i + 1,drop=FALSE]
                ##remove structural values
                mymat1[mymat1 %in% c(10,11)] <- NA
                mymat2[mymat2 %in% c(10,11)] <- NA
                mydiff <- mymat2 - mymat1
                attr(depvar, "distance")[i] <- sum(mydiff != 0,
                                                         na.rm = TRUE)
				if (allowOnly)
				{
					if (all(mydiff >= 0, na.rm=TRUE))
					{
						attr(depvar, "uponly")[i] <- TRUE
					}
					if (all(mydiff <= 0, na.rm=TRUE))
					{
						attr(depvar, "downonly")[i] <- TRUE
					}
				}
            }

            edgeLists[[i]] <- createEdgeLists(networks[[i]], depvar[, , i], TRUE)
        }
    }
    ## add attribute of nodeset
    attr(edgeLists, "nodeSet") <- attr(depvar, "nodeSet")
    ## add attribute of name
    attr(edgeLists, "name") <- attr(depvar, "name")
    ## add attribute of distance
    attr(edgeLists, "distance") <- attr(depvar, "distance")
    ## attr uponly and downonly
    attr(edgeLists, "uponly") <- attr(depvar, "uponly")
    attr(edgeLists, "downonly") <- attr(depvar, "downonly")
    ## attr symmetric
    attr(edgeLists, "symmetric") <- attr(depvar, "symmetric")
    ## attr balmean
    attr(edgeLists, "balmean") <- attr(depvar, "balmean")
    ## attr structmean
    attr(edgeLists, "structmean") <- attr(depvar, "structmean")
    attr(edgeLists, "averageOutDegree") <- attr(depvar, "averageOutDegree")
    return(edgeLists = edgeLists)
}
##@unpackBehavior siena07 Reformat data for C++
unpackBehavior<- function(depvar, observations)
{
    beh <- depvar[, 1, ]
    behmiss <- is.na(beh)
    allna <- apply(beh, 1, function(x)all(is.na(x)))
    modes <- attr(depvar, "modes")

    ## in case of missing data, use imputation values, provided by user
    ## else carry forward missings (otherwise use the mode / median)
    ## allNAs: use modes, in case of continuous behavior depvar$modes
	##         contains medians
    if (!is.null(attr(depvar, "imputationValues")) )
    {
        beh[behmiss] <- attr(depvar, "imputationValues")[behmiss]
    }
    else
    {
    beh[allna, ] <- rep(modes, each=sum(allna))
    for (i in 2:observations)
    {
            beh[is.na(beh[, i]), i] <-  beh[is.na(beh[, i]), i - 1]
    }
    for (i in (observations-1):1)
    {
            beh[is.na(beh[, i]), i] <-  beh[is.na(beh[, i]), i +1]
    }
    }
    ## need a better definition of structural for behavior variables
    ## struct <- beh %in% c(10, 11)
    ## beh[struct] <- beh[struct] - 10
    ## behstruct <- beh
    ## behstruct[!struct] <- 0

    ## add attribute of nodeset
    attr(beh, "nodeSet") <- attr(depvar, "nodeSet")
    ## add attribute of name
    attr(beh, "name") <- attr(depvar, "name")
    ## attr uponly and downonly
    attr(beh, "uponly") <- attr(depvar, "uponly")
    attr(beh, "downonly") <- attr(depvar, "downonly")
    ## attr symmetric
    attr(beh, "symmetric") <- attr(depvar, "symmetric")
    ## attr distance
    attr(beh, "distance") <- attr(depvar, "distance")
    ## attr simMean
    attr(beh, "simMean") <- attr(depvar, "simMean")
    ## attr simMeans
    attr(beh, "simMeans") <- attr(depvar, "simMeans")
    ## attr variance
    attr(beh, "variance") <- attr(depvar, "variance")
    if (attr(depvar, "type") == "behavior")
    {
        beh <- round(beh)
    storage.mode(beh) <- "integer"
    }
    else # type == "continuous"
    {
        storage.mode(beh) <- "double"
    }
    list(beh=beh, behmiss=behmiss)
}
##@convertToStructuralZeros Miscellaneous To be implemented
convertToStructuralZeros <- function()
{
}

##@unpackCDyad siena07 Reformat data for C++
unpackCDyad<- function(dycCovar)
{
    sparse <- attr(dycCovar, "sparse")
	bipartite <- attr(dycCovar, "type") == "bipartite"
    if (sparse)
    {
        ## have a list containing 1 sparse matrix in triplet format
        ## with missings embedded
        ## with 0 based indices!
        varmat <- cbind(dycCovar[[1]]@i+1, dycCovar[[1]]@j+1, dycCovar[[1]]@x)
        if (any(duplicated(varmat[, 1:2])))
        {
            stop("duplicate entries in sparse matrix dyadic covariate")
        }
        ##drop the diagonal, if present - not for bipartite
        if (!bipartite)
        {
            varmat <- varmat[varmat[,1] != varmat[, 2],]
        }
        mat1 <- varmat
        mat1[is.na(varmat[, 3]), 3] <- attr(dycCovar, "mean")
        mat1 <- mat1[!mat1[, 3] == 0, ]
        ## add attribute of dim
        attr(mat1, "nActors1") <- nrow(dycCovar[[1]])
        attr(mat1, "nActors2") <- ncol(dycCovar[[1]])
        mat2 <- varmat[is.na(varmat[, 3]), , drop=FALSE]
        mat2[, 3] <- 1
        ## add attribute of dim
        attr(mat2,"nActors1") <- nrow(dycCovar[[1]])
        attr(mat2,"nActors2") <- ncol(dycCovar[[1]])
        edgeLists <-  list(t(mat1), t(mat2))
    }
    else
    {
        if (!bipartite)
        {
            diag(dycCovar) <- 0
        }
        dycCovar1 <- dycCovar
        dycCovar1[is.na(dycCovar1)] <- attr(dycCovar, "mean")
        edgeLists <- createCovarEdgeList(dycCovar1, dycCovar)
    }
    ## add attribute of nodesets
    attr(edgeLists, "nodeSet") <- attr(dycCovar, "nodeSet")
    ## add attribute of type
    attr(edgeLists, "type") <- attr(dycCovar, "type")
    ## add attribute of name
    attr(edgeLists, "name") <- attr(dycCovar, "name")
    ## add attribute of mean
    attr(edgeLists, "mean") <- attr(dycCovar, "mean")
    return(edgeLists = edgeLists)
}


##@unpackVDyad siena07 Reformat data for C++
unpackVDyad<- function(dyvCovar, observations)
{
    edgeLists <- vector("list", observations)
    sparse <- attr(dyvCovar, "sparse")
    means <- attr(dyvCovar, "meanp")
	bipartite <- attr(dyvCovar, "type") == "bipartite"
    if (sparse)
    {
        ## have a list of sparse matrices in triplet format
        ## with 0 based indices!
        for (i in 1:(observations - 1))
        {
            thisvar <- dyvCovar[[i]]
            varmat <- cbind(thisvar@i+1, thisvar@j+1, thisvar@x)
            ## drop the diagonal, if present no - bipartite?
            if (!bipartite)
            {
                varmat <- varmat[varmat[,1] != varmat[, 2],]
            }
            mat1 <- varmat
            mat1[is.na(varmat[, 3]), 3] <- means[i]
            mat1 <- mat1[!mat1[, 3] == 0, ]
            mat2 <- varmat[is.na(varmat[, 3]),, drop=FALSE ]
            mat2[, 3] <- 1
            ## add attribute of size
            attr(mat1, "nActors1") <- nrow(dyvCovar[[i]])
            attr(mat1, "nActors2") <- ncol(dyvCovar[[i]])
            attr(mat2, "nActors1") <- nrow(dyvCovar[[i]])
            attr(mat2, "nActors2") <- ncol(dyvCovar[[i]])
            edgeLists[[i]] <- list(t(mat1), t(mat2))
        }
    }
    else
    {
        for (i in 1:(observations - 1))
        {
            if (!bipartite)
            {
                diag(dyvCovar[, , i]) <- 0
            }
            thisvar <- dyvCovar[, , i]
            thisvar[is.na(thisvar) ] <- means[i]
            edgeLists[[i]] <- createCovarEdgeList(thisvar, dyvCovar[, , i])
        }
    }
    ## add attribute of nodeset
    attr(edgeLists, "nodeSet") <- attr(dyvCovar, "nodeSet")
    ## add attribute of type
    attr(edgeLists, "type") <- attr(dyvCovar, "type")
    ## add attribute of name
    attr(edgeLists, "name") <- attr(dyvCovar, "name")
    ## add attribute of mean
    attr(edgeLists, "mean") <- attr(dyvCovar, "mean")
    return(edgeLists = edgeLists)
}

##@unpackData siena07 Reformat data for C++
unpackData <- function(data)
{
    f <- NULL
    observations<- data$observations
    types <- sapply(data$depvars, function(x) attr(x, "type"))
    f$nDepvars <- length(data$depvars)
    oneModes <- data$depvars[types == "oneMode"]
    Behaviors <- data$depvars[types == "behavior"]
    continuousBehaviors <- data$depvars[types == "continuous"]
    bipartites <- data$depvars[types == "bipartite"]
	## add the settings
    # oneModes <- lapply(oneModes, function(depvar) {
    #                    name <- attr(depvar, "name")
    #                    if (name %in% names(x$settings)) {
    #                      # attr(depvar, "settings") <- c("universal", "primary", x$settings[[name]])
    #                      attr(depvar, "settings") <- c(x$settings[[name]])
    #                    }
    #                    depvar
    #                    })
    f$nets <- lapply(oneModes, function(x, n, comp)
					 unpackOneMode(x, n, comp),
                     n = observations, comp=data$compositionChange)
    names(f$nets) <- names(oneModes)
    f$bipartites <- lapply(bipartites, function(x, n, comp)
                           unpackBipartite(x, n, comp),
                     n = observations, comp=data$compositionChange)
    names(f$bipartites) <- names(bipartites)
    f$behavs <-  lapply(Behaviors, function(x, n) unpackBehavior(x, n),
                        n = observations)
    names(f$behavs) <- names(Behaviors)
    f$contbehavs <- lapply(continuousBehaviors, function(x, n)
			unpackBehavior(x, n), n = observations)
    names(f$contbehavs) <- names(continuousBehaviors)
    f$observations <- observations
    f$seed<- vector("list", observations - 1)
    f$depvars <- data$depvars
    f$nodeSets <- data$nodeSets
    f$oneModes <- oneModes
    f$Behaviors <- Behaviors
    f$continuousBehaviors <- continuousBehaviors
    f$oneModeUpOnly <- sapply(oneModes, function(x) attr(x, "uponly"))
    f$oneModeDownOnly <- sapply(oneModes, function(x) attr(x, "downonly"))
    f$behaviorUpOnly <- sapply(Behaviors, function(x) attr(x, "uponly"))
    f$behaviorDownOnly <- sapply(Behaviors, function(x) attr(x,
                                                             "downonly"))
    f$distances <- sapply(data$depvars, function(x) attr(x, "distance"))
    f$cCovars <- data$cCovars
    f$vCovars <- data$vCovars
    ## dyadic covars need to be edgelists
    f$dycCovars <- lapply(data$dycCovars, function(x) unpackCDyad(x))
    f$dyvCovars <- lapply(data$dyvCovars, function(x,n) unpackVDyad(x,n),
                          n=observations)
    ## create the composition change event lists
    f$exog <- lapply(data$compositionChange, function(x)
                     unpackCompositionChange(x))
    f
}

##@unpackCompositionChange siena07 Reformat data for C++
unpackCompositionChange <- function(compositionChange)
{
    atts <- attributes(compositionChange)
    events <- atts$events
    activeStart <- atts$activeStart
    observations <- ncol(activeStart)
    ## check that there is someone there always
    for (i in 1:(observations - 1))
    {
        activeAll <- sum(activeStart[, i] & activeStart[, i + 1])
        if (activeAll < 2)
        {
            active <- sum(activeStart[, i])
            if (active == 1)
                stop("Only one active actor at start of period", i)
            else if (active == 0)
                stop("No active actors at start of period", i)
            perEvents <- events[events$period == i,]
            perEvents <- perEvents[order(perEvents$time),]
            changes <- c(1, -1)[as.numeric(as.character(perEvents$event))]
            active <- active + cumsum(changes)
            if (any(active < 2))
            {
                stop("No/only one active actor(s) left.")
            }
        }
    }
    events <- events[events$time > 1e-10,]
    exog <- list(events=events, activeStart=activeStart)
    attr(exog, "nodeSet") <- attr(compositionChange, "nodeSet")
    exog
}

##@updateTheta siena07 Copy theta values from previous fit
updateTheta <- function(effects, prevAns, varName=NULL)
{
	if (!inherits(effects, "data.frame"))
	{
		stop("effects is not a data.frame")
	}
	if ((!inherits(prevAns, "sienaFit")) && (!inherits(prevAns, "sienaBayesFit")))
	{
		stop("prevAns is not an RSiena fit object")
	}
  	if (!prevAns$gmm)
	{
		prevEffects <- prevAns$requestedEffects[which(prevAns$requestedEffects$type != 'gmm'),]
		prevEffects$initialValue <- prevAns$theta
	}
	else if (prevAns$gmm)
	{
		prevEffects <- prevAns$requestedEffects
    	prevEffects$initialValue <- prevAns$theta
	}
	if (prevAns$cconditional)
	{
		condEffects <- attr(prevAns$f, "condEffects")
		condEffects$initialValue <- prevAns$rate
		prevEffects <- rbind(prevEffects, condEffects)
	}
	if (!is.null(varName))
	{
		prevEffects$name[!( prevEffects$name %in% varName)] <- ' '
	}
	oldlist <- apply(prevEffects, 1, function(x)
					 paste(x[c("name", "shortName",
							   "type", "groupName",
							   "interaction1", "interaction2",
							   "period", "effect1", "effect2", "effect3")],
						   collapse="|"))
	efflist <- apply(effects, 1, function(x)
					 paste(x[c("name", "shortName",
							   "type", "groupName",
							   "interaction1", "interaction2",
							   "period", "effect1", "effect2", "effect3")],
						   collapse="|"))
	use <- efflist %in% oldlist
	effects$initialValue[use] <-
		prevEffects$initialValue[match(efflist, oldlist)][use]
	effects
}

update_theta  <- function(x, ...) UseMethod("update_theta", x)

##@updateTheta.sienaEffects Copy theta values from previous fit
update_theta.sienaEffects <- function(x, prevAns, varName=NULL, ...){
	updateTheta(x, prevAns=prevAns, varName=varName)
}

##@ numberIntn siena07 sienaBayes, number of network interaction effects used for getEffects
numberIntn <- function(myeff){
	if (!is.null(myeff)){	
		numnet <- length(unique(myeff$name[myeff$shortName=="density"])) # number of dependent networks
		nintn <- sum(myeff$shortName == 'unspInt')/3 # 3 for eval - creation - endow
	}
	else
	{
		numnet <- 0
	}	
	ifelse((numnet <= 0), 10, nintn/numnet) # 10 is the default in getEffects
}

##@ numberIntn siena07 sienaBayes, number of behavior interaction effects used for getEffects
numberBehIntn <- function(myeff){
	if (!is.null(myeff)){
		numbeh <- length(unique(myeff$name[myeff$shortName=="linear"])) # number of discrete behaviors
		nbehIntn <- sum(myeff$shortName == 'behUnspInt')/3 # 3 for eval - creation - endow
		numcont <- length(unique(myeff$name[myeff$shortName=="intercept"])) # nr. of continuous behaviors
		ncontIntn <- sum(myeff$shortName == 'contUnspInt')
	}
	else
	{
		numbeh <- 0
		numcont <- 0
	}		
	ifelse((numbeh == 0 && numcont == 0), 4,  # 4 is the default in getEffects
	       ifelse(numbeh == 0, max(4, ncontIntn/numcont),
           ifelse(numcont == 0, max(4, nbehIntn/numbeh),
	              max(nbehIntn/numbeh, ncontIntn/numcont))))
}


##@checkVersion siena07 Create default effects object and check RSiena version.
checkVersion <- function(dat, effs){
	if (!is.null(attr(effs, "onePeriodSde")))
	{
		oPS <- attr(effs, "onePeriodSde")
	}
	else
	{
		oPS <- FALSE
	}
	defaultEffects <- getEffects(dat, nintn=numberIntn(effs), 
						behNintn=numberBehIntn(effs),onePeriodSde=oPS)
	effectsVersion <- attr(effs, "version")
	if (is.null(effectsVersion))
	{
		differentVersions <- TRUE
	}
	else
	{
		differentVersions <- (effectsVersion != attr(defaultEffects, "version"))
	}
	if ((differentVersions) & 
		(any((effs$shortName %in% c("unspInt","behUnspInt", "contUnspInt"))&effs$include)))
		{
		warning("Your effects object contains interaction effects and was made
		  using a different RSiena version. 
		  Make sure the interaction effects are the same.")
	}
	defaultEffects 
}

##@addSettingseffects siena07 add extra rate effects for settings model
# addSettingsEffects <- function(effects, x)
# {
# 	## need a list of settings (constant dyadic covariate name) for some or all
# 	## dependent networks. If symmetric this is equivalent to a forcing model.
# 	## The covariate actor sets should match the network actor sets.
# 	## ?? would it ever make sense for bipartites? Allow it here for now and see!
# 	## maybe better to add the settings pars to the data object but for now
# 	## they are on the model with maxdegree. TODO write some validation
# 	varNames <- names(x$settings)
# 	nbrSettings <- sapply(x$settings, length)
# 	tmp <-
# 		lapply(1:length(x$settings), function(i)
# 		   {
# 			   ## find effects for this variable
# 			   varEffects <- effects[effects$name == varNames[i], ]
# 			   ## find relevant rate effects
# 			   dupl <- varEffects[varEffects$basicRate, ]
# 			   ## make extra copies
# 			   newEffects <- dupl[rep(1:nrow(dupl),
# 									  each = nbrSettings[i] + 2), ]
# 			   newEffects <- split(newEffects, list(newEffects$group,
# 								   newEffects$period))
# 			   newEffects <- lapply(newEffects, function(dd)
# 				  {
# 					  dd$setting <- c("universal", "primary",
# 											  x$settings[[i]])
# 					  i1 <- regexpr("rate", dd$effectName)
# 					  dd$effectName <-
# 						  paste(substr(dd$effectName, 1, i1 - 2),
# 											 dd$setting,
# 								substring(dd$effectName, i1))
# 					  dd
# 				  }
# 					  )
# 			   newEffects <- do.call(rbind, newEffects)
# 			   ## add extra column to non basic rate effects
# 			   varEffects$setting <- rep("", nrow(varEffects))
# 			   ## combine them
# 			   rbind(newEffects, varEffects[!varEffects$basicRate, ])
# 		   })
# 	## join them together again
# 	do.call(rbind, tmp)
# }

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: print07report.r
# *
# * Description: This module contains the code to produce the report on a
# * siena07 model fit.
# *****************************************************************************/
##@PrintReport siena07 Print report
PrintReport <- function(z, x)
{
	types <- attr(z$f, "types")
	Report('\n\n', outf)
	if ((x$nsub == 0)&(x$simOnly))
	{
		Heading(2, outf, "Simulation Results.")
	}
		else
	{
		Heading(2, outf, "Estimation Results.")
	}
	if (!z$OK)
	{
	Report("Error end of estimation algorithm", outf)
	}
	else
	{
		if (x$simOnly)
		{
			Report("Regular end of simulation algorithm.\n", outf)
			Report(c("Total of", z$n, "iteration steps.\n\n"), outf)
			Report(c("Total of", z$n, "iteration steps.\n\n"), bof)
			Heading(3, outf, "Parameter values")
		}
		else
		{
			Report("Regular end of estimation algorithm.\n", outf)
			Report(c("Total of", z$n, "iteration steps.\n\n"), outf)
			Report(c("Total of", z$n, "iteration steps.\n\n"), bof)
			Heading(3, outf, "Estimates and standard errors")
		}
		Heading(3, bof, "Estimates and standard errors")
		if (z$cconditional) ## deal with rate parameter
		{
			Report('Rate parameters:\n', outf)
			Report('Rate parameters:\n', bof)
			if (length(z$rate) == 1)
			{
				if (length(attr(z$f,'netnames')) == 1)
				{
					Report(format(' 0. Rate parameter', width = 43), outf)
					Report(format(' 0. Rate parameter', width = 43), bof)
				}
				else
				{
					Report(format(' 0. Rate parameter of conditioning variable',
									 width = 43), outf)
					Report(format(' 0. Rate parameter of conditioning variable',
									 width = 43), bof)
				}
				Report(format(round(z$rate[1], digits = 4), width = 9), outf)
				Report(format(round(z$rate[1], digits = 4), width = 9), bof)
				Report(c('  (', format(round(z$vrate[1], digits = 4),
										width = 9), ')\n'), sep = '', outf)
				Report(c('  (', format(round(z$vrate[1], digits = 4),
										width = 9), ')\n'), sep = '', bof)
			}
			else ## observations > 2
			{
				nn <- length(z$rate)
				nnstr <- format(as.character(1:nn),width=2,justify='left')
				if (z$f$nDepvars == 1 || is.null(z$f$nDepvars))
				{
					tmp <- paste(' 0.', nnstr, ' Rate parameter period ',
									1:nn, '  ',
									format(round(z$rate,4),width=9),
									'  (',format(round(z$vrate,4),width=9),
									')\n', sep = '')
				} else{
					tmp <- paste(' 0.', nnstr,
									'Rate parameter cond. variable period ',
									1:nn, '  ',
									format(round(z$rate,4),width=9),
									'  (',format(round(z$vrate,4),width=9),
									')\n',sep='')
				}
				Report(tmp, outf, sep='')
				Report(tmp, bof, sep='')
			}
			Report('\nOther parameters:\n', outf)
			Report('\nOther parameters:\n', bof)
		}
		nBehavs <- sum(types == "behavior")
		nNetworks <- length(types) - nBehavs
		if (nBehavs > 0 && nNetworks > 0)
		{
			Report("Network Dynamics\n", outf)
		}
		if (!x$simOnly & !gmm(x))
		{
			ses <- ifelse(diag(z$covtheta) >= 0.0 | z$fixed,
					paste('  (', sprintf("%9.4f",sqrt(diag(z$covtheta))),
										 ')', sep=''), '  ---')
			if (!all(z$fixed))
			{
				ses[z$fixed] <- '  (  fixed  )'
			}
			theta <- ifelse(diag(z$covtheta) >= 0.0 | z$fixed,
					sprintf("%9.4f",z$theta),
					' ---')
		}
		else if (!x$simOnly & gmm(x))
		{ 
		  ses <- ifelse(diag(z$covtheta) >= 0.0 | z$fixed[-which(z$gmmEffects==TRUE)],
		                paste('  (', sprintf("%9.4f",sqrt(diag(z$covtheta))),
		                      ')', sep=''), '  ---')
		  if (!all(z$fixed))
		  { 
		    ses[which(z$fixed & !z$gmmEffects)] <- '  (  fixed  )'
		  }
		  theta <- ifelse(diag(z$covtheta) >= 0.0 | z$fixed[-which(z$gmmEffects==TRUE)],
		                  sprintf("%9.4f",z$theta[-which(z$gmmEffects==TRUE)]),
		                  ' ---')
		}
		else
		{
			theta <- sprintf("%9.4f",z$theta)
		}
		if (nBehavs > 0)
		{
			behEffects <-
			z$requestedEffects[z$requestedEffects$netType == 'behavior',]
			behNames <- unique(behEffects$name)
			if (nBehavs > 1)
			{
				behEffects$effectName <- paste('<',
										 (1:nBehavs)[match(behEffects$name,
																behNames)],
											  '> ', behEffects$effectName,
											  sep='')
				z$requestedEffects$effectName[z$requestedEffects$netType==
											 'behavior'] <-
												 behEffects$effectName
			}
		}
		typesp <- ifelse (z$requestedEffects$type %in% c("eval", "rate"),
						 ":  ", ": ")
		typetxt <- ifelse (z$requestedEffects$type == "creation", "creat",
						  z$requestedEffects$type )
		tmp <- paste(typetxt, typesp, z$requestedEffects$effectName, sep = '')
		if (gmm(x))
		{
		  typesp <- typesp[-which(z$gmmEffects==TRUE)]
		  typetxt <- typetxt[-which(z$gmmEffects==TRUE)]
		  tmp <- paste(typetxt, typesp, z$requestedEffects$effectName[-which(z$gmmEffects==TRUE)], sep = '')
		}
		if (x$simOnly)
		{
			ses <- rep('  ', z$pp)
		}
		if(!gmm(x))
		{
		  tmp <- paste(sprintf("%2d", 1:length(z$requestedEffects$effectName)),
		               '. ', format(substr(tmp, 1, 60), width=60),
		               theta, ses, '\n', sep='', collapse = '')
		}
		else
		{
		  ngmmEffects <- sum(z$requestedEffects$type=="gmm")
		  tmp <- paste(sprintf("%2d", 1:(length(z$requestedEffects$effectName)-ngmmEffects)),
		               '. ', format(substr(tmp, 1, 60), width=60),
		               theta, ses, '\n', sep='', collapse = '')
		}
		if (nBehavs > 0 && nNetworks > 0)
		{
		  if(!gmm(x))
		  {
		      nNetworkEff <- nrow(z$requestedEffects) - nrow(behEffects)
		      tmpstr <- paste(nNetworkEff + 1, '. ', sep='')
		      tmpsub <- regexpr(tmpstr, tmp, fixed=TRUE)
		      tmp1 <- substring(tmp, 1, tmpsub - 2)
		      tmp2 <- substring(tmp, tmpsub - 1, nchar(tmp))
		      Report(tmp1, outf)
		      Report(tmp1, bof)
		      Report('\nBehavior Dynamics\n', outf)
		      Report(tmp2, outf)
		      Report(tmp2, bof)
		  }
		  else
		  {
		    nBehgmmEffects <- sum(z$requestedEffects$type!="gmm" & z$requestedEffects$netType == 'behavior')
		    nNetgmmEffects <- nrow(z$requestedEffects) - ngmmEffects - nBehgmmEffects 
		    tmpstr <- paste(nNetgmmEffects + 1, '. ', sep='')
		    tmpsub <- regexpr(tmpstr, tmp, fixed=TRUE)
		    tmp1 <- substring(tmp, 1, tmpsub - 2)
		    tmp2 <- substring(tmp, tmpsub - 1, nchar(tmp))
		    Report(tmp1, outf)
		    Report(tmp1, bof)
		    Report('\nBehavior Dynamics\n', outf)
		    Report(tmp2, outf)
		    Report(tmp2, bof) 
		  }
		}
		else
		{
			Report(tmp, outf)
			Report(tmp, bof)
		}
		Report('\n', outf)
		Report('\n', bof)
		if (z$cconditional && length(attr(z$f, 'netnames')) > 1)
		{
			Report(c('For conditional estimation, ',
						'the standard errors of rate parameters\n',
						'not used for conditioning are unreliable.'), outf)
		}
		if (x$simOnly)
		{
			Heading(3, outf, "Simulated statistics")
			if (z$thetaFromFile)
			{
				Report('Note: Parameters in Phase 3 were varying. \n', bof)
				Report('Note: Parameters in Phase 3 were varying. \n', outf)
			}
			Report('Estimated means and standard deviations, standard errors of the mean \n', bof)
			Report('Estimated means and standard deviations, standard errors of the mean \n', outf)
			dmsf <- diag(z$msf)
# sf and cov.dev may be dropped - just for now (07-10-13) I keep them in
#			sf <- colMeans(z$sf)
			mean.stats <- colMeans(z$sf) + z$targets
#			cov.dev <- z$msf
			sem <- sqrt(dmsf/dim(z$sf)[1])
			if ((x$dolby) & (!z$thetaFromFile))
			{
				scores <- apply(z$ssc, c(1,3), sum)  # z$nit by z$pp matrix
				mean.scores <- colMeans(scores)
				mean.stats <- mean.stats - (z$regrCoef * mean.scores)
				sem <- sem*sqrt(1 - (z$regrCor)^2)
			}
			mymess1 <- paste(format(1:z$pp,width=3), '. ',
				format(z$requestedEffects$functionName, width = 56),
				format(round(mean.stats, 3), width=8, nsmall=3), ' ',
				format(round(sqrt(dmsf), 3) ,width=8, nsmall=3), ' ',
				format(round(sem, 4) ,width=8, nsmall=4), sep='')
			PrtOutMat(as.matrix(mymess1), outf)
			PrtOutMat(as.matrix(mymess1), bof)
			if ((x$dolby) & (!z$thetaFromFile))
			{
				Report('Standard errors of the mean are less than s.d./sqrt(n) \n', outf)
				Report('because of regression on scores (Dolby option). \n', outf)
			}
		}
		else
		{
			Heading(3, outf, "Covariance matrices")
			if (all(is.na(z$covtheta)))
			{
				Report(c('There is a linear dependency between the parameter estimates\n',
						'therefore the covariance matrix should not be used.\n\n'),
					outf)
			}
			else
			{
				Report(c("Covariance matrix of estimates",
						"(correlations below diagonal):\n"), outf)
				covcor <- z$covtheta
				correl <- z$covtheta/sqrt(diag(z$covtheta))[row(z$covtheta)]/
					sqrt(diag(z$covtheta))[col(z$covtheta)]
				covcor[lower.tri(covcor)] <- correl[lower.tri(correl)]
				PrtOutMat(format(round(covcor, digits = 3), width = 10), outf)
				Report(c('Derivative matrix of expected statistics X by',
						'parameters and\n'), outf)
				Report(c("covariance/correlation matrix of X can be found using\n",
						"summary(ans) within R,",
						" or by using the 'verbose' option in Siena07.\n "),
					sep = "", outf)
				Report(c("Derivative matrix of expected statistics X by",
						"parameters:\n\n"), lf)
				PrtOutMat(z$dfrac, lf)
				Report('Covariance matrix of X (correlations below the diagonal):\n',
					lf)
				covcor <- z$msf
				correl <- z$msf/sqrt(diag(z$msf))[row(z$msf)]/
					sqrt(diag(z$msf))[col(z$msf)]
				covcor[lower.tri(covcor)] <- correl[lower.tri(correl)]
				PrtOutMat(format(round(covcor, digits = 3), width = 10), lf)
				Report('\n', outf)
				Report('\n', lf)
			}
		}
	}

}

#/******************************************************************************
#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: Sienatest.r
# *
# * Description: This module contains the function for instability analysis and
# * score tests.
# *
# *****************************************************************************/

test_parameter <- function(x, ...) UseMethod("test_parameter", x)

##@test_parameter.sienaFit test_parameter method for all tests for sienaFit 
test_parameter.sienaFit <- function(x, method=NULL, 
										tested=NULL, tested2=NULL, ...)
{
	if (is.null(method))
	{
		if (inherits(tested,"matrix")) 
		{
			Wald.RSiena(tested, x)
		}
		else if (is.vector(tested)) 		
		{
			Multipar.RSiena(x, tested)
		}
		else if (tested=="all")
		{
			Multipar.RSiena(x, which(!x$test))
		}
		else
		{
			stop("keyword 'tested' is not given properly")
		}
	}
	else if (method=="score")
	{
		if (is.null(tested))
		{
			tested <- x$test
		}
		score.Test(x, tested=tested)
	}
	else if (method=="same")
	{
		testSame.RSiena(x, tested, tested2)
	}
	else
	{
		stop(paste("There is no test_parameter.sienaFit with method = ", method))
	}
}

##@TestOutput siena07 Print test report
TestOutput <- function(z, x)
{
	testn <- sum(z$test)
	## browser()
	if (testn)
	{
		if (x$maxlike)
		{
			Heading(2, outf,'Score test <c>')
		}
		else
		{
			Heading(2, outf, 'Generalised score test <c>')
		}
		Report('Testing the goodness-of-fit of the model restricted by\n', outf)
		j <- 0
		for (k in 1:z$pp)
		{
			if (z$test[k])
			{
				j <- j + 1
				Report(c(" (", j, ")   ",
						format(paste(z$requestedEffects$type[k], ":  ",
								z$requestedEffects$effectName[k],
								sep=""),
							width=50), " = ",
						sprintf("%8.4f", z$theta[k]), "\n"),
					sep = "", outf)
			}
		}
		Report('_________________________________________________\n',outf)
		Report('                ',outf)
		Report('   \n',outf)
		if (testn > 1)
			Report('Joint test:\n-----------\n',outf)
		Report(c('   c = ',sprintf("%8.4f", z$testresOverall),
				'   d.f. = ',j,'   p-value '),sep='',outf)
		pvalue <- 1-pchisq(z$testresOverall,j)
		if (!is.na(pvalue))
		{
			if (pvalue < 0.0001)
			{
				Report('< 0.0001',outf)
			}
			else
			{
				Report(c('= ',sprintf("%8.4f",pvalue)), sep = '', outf)
			}
		}
		else
		{
			Report('  NA  ',outf)
		}
		if (testn==1)
			Report(c('\n   one-sided (normal variate): ',
					sprintf("%8.4f",z$testresulto[1])), sep = '', outf)
		if (testn> 1)
		{
			Report('\n\n',outf)
			for (k in 1:j)
			{
				Report(c('(',k,') tested separately:\n'),sep='',outf)
				Report('-----------------------\n',outf)
				Report(' - two-sided:\n',outf)
				Report(c('  c = ', sprintf("%8.4f", z$testresult[k]),
						'   d.f. = 1  p-value '), sep = '', outf)
				pvalue<- 1-pchisq(z$testresult[k],1)
				if (!is.na(pvalue))
				{
					if (pvalue < 0.0001)
					{
						Report('< 0.0001\n',outf)
					}
					else
					{
						Report(c('= ', sprintf("%8.4f", pvalue), '\n'), sep = '',
							outf)
					}
				}
				else
				{
					Report('  NA  ',outf)
				}
				Report(c(' - one-sided (normal variate): ',
						sprintf("%8.4f", z$testresulto[k])), sep = '', outf)
				if (k < j)
				{
					Report('\n\n',outf)
				}
			}
		}
		Report('    \n_________________________________________________\n\n',
			outf)
		Report('One-step estimates: \n\n', outf)
		for (i in 1 : z$pp)
		{
			onestepest <- z$oneStep[i] + z$theta[i]
			Report(c(format(paste(z$requestedEffects$type[i], ':  ',
							z$requestedEffects$effectName[i], sep = ''),
						width=50),
					sprintf("%8.4f", onestepest), '\n'), sep = '', outf)
		}
		Report('\n',outf)
	}
}

##@ScoreTest siena07 Do score tests
ScoreTest<- function(pp, dfra, msf, fra, test, redundant, maxlike)
{
	testresult <- rep(NA, pp) ##for chisq per parameter
	testresulto <- rep(NA, pp) ##for one-sided tests per parameter
	##first the general one
	ans <- EvaluateTestStatistic(maxlike, test, redundant, dfra, msf, fra)
	testresOverall <- ans$cvalue
	covMatrix <- ans$covMatrix
	if (sum(test) == 1)
	{
		testresulto[1] <- ans$oneSided
	}
	else
	{
		## single df tests
		use <- !test
		k <- 0
		for (i in 1:pp)
		{
			if (test[i])
			{
				k <- k + 1
				use[i] <- TRUE
				ans <- EvaluateTestStatistic(maxlike, test[use], redundant[use],
					dfra[use, use], msf[use, use], fra[use])
				testresult[k] <- ans$cvalue
				testresulto[k] <- ans$oneSided
				use[i] <- FALSE
			}
		}
	}
	##onestep estimator
	if (maxlike)
		dfra2 <- dfra + msf
	else
		dfra2 <- dfra
	if (inherits(try(dinv2 <- solve(dfra2), silent=TRUE), "try-error"))
	{
		Report("Error message for inversion to get onestep estimator: \n", cf)
		dinv2 <- dfra2
		dinv2[] <- NA
		oneStep <- rep(NA, nrow(dfra2))
	}
	else
	{
		oneStep<- -dinv2 %*% fra
	}
	list(testresult=testresult, testresulto=testresulto,
		testresOverall=testresOverall, covMatrix=covMatrix,
		oneStep=oneStep, dinv2= dinv2, dfra2=dfra2)
}
##@EvaluateTestStatistic siena07 Calculate score test statistics
EvaluateTestStatistic<- function(maxlike, test, redundant, dfra, msf, fra)
{
	##uses local arrays set up in the calling procedure
	d11 <- dfra[!(test|redundant), !(test|redundant), drop=FALSE]
	d22 <- dfra[test, test, drop=FALSE]
	d21 <- dfra[test, !(test|redundant), drop=FALSE]
	d12 <- t(d21)
	sigma11 <- msf[!(test|redundant), !(test|redundant), drop=FALSE]
	sigma22<- msf[test, test,drop=FALSE]
	sigma12 <- msf[!(test|redundant), test, drop=FALSE]
	sigma21<- t(sigma12)
	z1 <- fra[!(test|redundant)]
	z2 <- fra[test]
	oneSided <- NA
	if (inherits(try(id11 <- solve(d11), silent=TRUE), "try-error"))
	{
		warning('Score test: Error for inversion of d11 \n')
		v9 <- d22
		v9[] <- NA
		cvalue <- matrix(NA, 1, 1)
	}
	else
	{
		rg <- d21 %*% id11
		if (!maxlike)
		{
			##orthogonalise deviation vector
			ov <- z2 - rg %*% z1
			##compute var(ov) = sigma22 - (d21 %*% id11) %*% sigma12 -
			##      sigma21 %*% t(id11)%*% t(d21) +
			##      d21%*%id11 %*% sigma11 %*% t(id11) %*% t(d21)
			v2 <- sigma21 - rg %*% sigma11
			v6 <- v2 %*% t(id11) %*% t(d21)
			v9 <- sigma22 -  rg %*% sigma12 - v6
		}
		else
		{
			ov <- -z2
			v9 <- d22 - rg %*% d12
		}
		if (inherits(try(vav <- solve(v9), silent=TRUE), "try-error"))
			## vav is the inverse variance matrix of ov
		{
			warning('Score test: Error for inversion of v9\n')
			vav <- v9
			vav[] <- NA
			cvalue <- NA
		}
		else
		{
			cvalue <- t(ov) %*% vav %*% ov
			if (cvalue < 0) cvalue <- 0
			if (sum(test) == 1)
			{
				if (vav > 0)
				{
					oneSided <- ov * sqrt(vav)
				}
				else
				{
					oneSided <- 0
				}
				if (!maxlike) oneSided <- - oneSided
				## change the sign for intuition for users
			}
			else
			{
				oneSided <- 0
			}
		}
	}
	list(cvalue=cvalue, oneSided=oneSided, covMatrix=v9)
}


##@theEfNames internal RSiena function getting estimated effect names
theEfNames <- function(ans){
	if(length(unique(ans$requestedEffects$name)) > 1)
	{
		res <- paste(ans$requestedEffects$name, ans$requestedEffects$effectName, sep=': ')
	}
	else
	{
		res <- ans$requestedEffects$effectName
	}
	if (any(ans$requestedEffects$type != "eval"))
	{
		res <- paste(res, ans$requestedEffects$type, sep=' ')
	}
	res
}

##@scoreTest Calculate score tested
score.Test <- function(x, tested=x$test)
	# use: x must be a sienaFit object;
	# tested must be a boolean vector with length equal to the number of parameters of x,
	# or a vector of integer numbers between 1 and x$pp.
{
	if ((is.numeric(tested)) || (is.integer(tested)))
	{
		if (max(tested) > x$pp)
		{
			stop(paste('The maximum requested coordinate is too high:',
					max(tested)))
		}
		tested <- (1:x$pp) %in% tested
	}
	if (sum(tested) <= 0) stop(paste('Something should be tested, but the total requested is',
			sum(tested)))
	if (length(tested) != x$pp) stop('Dimensions of tested must agree')
	if (any(tested & (!x$fix))) warning('Warning: some tested parameters were not fixed; do you know what you are doing??? \n')
	fra <- colMeans(x$sf, na.rm=TRUE)
	redundant <- (x$fix & (!tested))
	tests <- EvaluateTestStatistic(x$maxlike, tested, redundant, x$dfra, x$msf, fra)
	teststat <- tests$cvalue
	df <- sum(tested)
	if (df == 1)
	{
		onesided <- tests$oneSided
	}
	else
	{
		onesided <- NULL
	}
	pval <- 1 - pchisq(teststat, df)
	efnames <- theEfNames(x)[tested]
	t_x <- list(chisquare=teststat, df=df, pvalue=pval, onesided=onesided, efnames=efnames)
	class(t_x) <- "sienaTest"
	attr(t_x, "version") <- packageDescription(pkgname, fields = "Version")
	t_x
}

##@Wald.RSiena  Calculate Wald test statistics
Wald.RSiena <- function(A, x)
{
	if (is.vector(A)){A <- matrix(A, nrow=1)}
	if (dim(A)[2] != x$pp){stop(paste('A must have', x$pp, 'columns.'))}
	sigma <- x$covtheta
	if (any(is.na(sigma))) {
		# happens when some coordinates were fixed
		# in the call of siena07 leading to x;
		# then the non-used part of sigma,
		# which partially consists of NA,
		# is replaced by the identity matrix.
		zero.cols <- apply(A, 2, function(colum){all(colum==0)})
		sigma[zero.cols, ] <- 0
		sigma[, zero.cols] <- 0
		diag(sigma)[zero.cols] <- 1
	}
	th <- A %*% x$theta
	covmat <- A %*% sigma %*% t(A)
	chisq <- drop(t(th) %*% solve(covmat) %*% th)
	df <- nrow(A)
	pval <- 1 - pchisq(chisq, df)
	if (df == 1)
	{
		onesided <- sign(th) * sqrt(chisq)
		ster <- sqrt(covmat[1,1])
	}
	else
	{
		onesided <- NULL
		ster <- NULL
	}
	t_x <- list(chisquare=chisq, df=df, pvalue=pval, onesided=onesided, sterror=ster)
	class(t_x) <- "sienaTest"
	attr(t_x, "version") <- packageDescription(pkgname, fields = "Version")
	t_x
}

##@Multipar.RSiena  Calculate Wald test statistic for hypothesis that subvector = 0.
Multipar.RSiena <- function(x, tested)
{
	p <- length(x$theta)
	efnames <- theEfNames(x)[tested]
	k <- length(tested)
	A <- matrix(0, nrow=k, ncol=p)
	A[cbind(1:k,tested)] <- 1
	t_x <- Wald.RSiena(A, x)
	t_x$efnames <- efnames
	t_x
}

##@testSame.RSiena Test that two subvectors of parameter are identical
testSame.RSiena <- function(x, e1, e2){
	if (length(e1) != length(e2))
	{
		stop('Tested parameters should have the same length')
	}
	p <- x$pp
	if ((min(e1) < 1) | (min(e2) < 1) | (max(e1) > p) | (max(e2) > p))
	{
		stop(paste('Tested parameters should have numbers between 1 and', p))
	}
	if (any(x$fixed[c(e1,e2)]))
	{
		stop('Fixed parameters cannot be tested')
	}
	t <- length(e1)
	efnames1 <- theEfNames(x)[e1]
	efnames2 <- theEfNames(x)[e2]
	if (t > 1)
	{
		max.width <- max(c(nchar(efnames1)))
		efnames1 <- format(efnames1, width=max.width)
	}
	efnames <- paste(efnames1, ' == ', efnames2)
	mat <- matrix(0,t,p)
	for (i in seq_along(1:t)){mat[i,e1[i]] <- 1}
	for (i in seq_along(1:t)){mat[i,e2[i]] <- -1}
	t_x <- Wald.RSiena(mat, x)
	t_x$efnames <- efnames
	t_x
}

##@print.sienaTest Methods
print.sienaTest <- function(x, ...)
{
	if (!inherits(x, "sienaTest"))
	{
		stop("not a legitimate sienaTest object")
	}
	if (any(is.na(unlist(x))))
	{
		warning("some estimates or standard errors are NA")
	}
	if (!is.null(x$efnames))
	{
		cat('Tested effects:\n ')
		cat(paste(x$efnames,'\n'))
	}
	cat(paste('chi-squared = ',
		format(round(x$chisquare, digits=2), nsmall = 2),
		', d.f. = ', x$df, '; ', sep=''))
	if ((x$df == 1) & (!is.null(x$onesided)) & (!is.null(x$sterror)))
	{
		cat(paste( 'standard error of linear combination = ',
			format(round(x$sterror, digits=3), nsmall = 2), '; ', sep=''))
		cat(paste('one-sided Z = ',
			format(round(x$onesided, digits=2), nsmall = 2), '; ', sep=''))
	}
	cat('two-sided')
	if (x$pvalue < 0.001)
	{
		cat(' p < 0.001. \n')
	}
	else
	{
		cat(paste(' p = ',
			format(round(x$pvalue, digits=3), nsmall = 2), '. \n', sep=''))
	}
	invisible(x)
}

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: effectsMethods.r
## *
## * Description: This file contains the print and edit methods for the
## * class sienaEffects
## *
## ****************************************************************************/

set_effect <- function(x, ...) UseMethod("set_effect", x)

##@set_effect.sienaEffects Methods new names
# points to setEffect and includeEffects
set_effect.sienaEffects <- function(x, shortNames, type="eval",
			depvar=x$name[1], covar1="", covar2="",
			effect1=0, effect2=0, effect3=0, period=1, group=1, 
			parameter=NULL, initialValue = 0, random=FALSE,
			timeDummy = ",", 
			include=TRUE, fix=FALSE, test=FALSE, verbose=TRUE, ...)
{
	if (!hasArg(shortNames))
	{
		stop("Function set_effect needs one or more shortNames")
	}
# Test whether the shortNames are given as characters that are not objects:
	suppressWarnings(notObjects <- inherits(try(shortNames[[1]], silent=TRUE), 
										"try-error"), classes = "warning")
	if (notObjects)
	{
		if (length(substitute(shortNames))==1)
		{
			shn1 <- substitute(shortNames)
		}
		else
		{
			shn1 <- substitute(shortNames)[-1]
		}
		shn2 <- sapply(shn1, function(x) deparse(x))	
	}
	else
	{
		if (length(shortNames)==1)
		{
			shn2 <- shortNames
		}
# else test whether the shortNames are {names of objects} or {character strings}:
# (of course those objects also must be character strings)
		else if (is.symbol(substitute(shortNames)[[2]])) # names of objects
		{
			shn2 <- unlist(shortNames)
		}
		else # character strings
		{
			shn1 <- substitute(shortNames)[-1]
			shn2 <- sapply(shn1, function(x) x)	
		}
	}
	if (length(shn2) == 0)
	{
		stop("Some shortNames are needed")
	}
	else if (length(shn2) == 1)
	{
		setEffect(x, shn2, parameter = parameter,
			fix=fix, test=test, random=random, initialValue = initialValue,
			timeDummy = timeDummy, include = include,
			name = depvar, type = type, interaction1 = covar1, interaction2 = covar2, 
			effect1=effect1, effect2=effect2, effect3=effect3,
			period=period, group=group, character=TRUE, verbose = verbose)
	}
	else
	{
		if (hasArg(parameter))
		{
			warning("For more than one shortName, function set_effect does not consider parameters")
		}		
		includeEffects(x, shn2, include = TRUE, name = depvar, type = type,
			interaction1 = covar1, interaction2 = covar2, fix=fix, test=test, character=TRUE,
			verbose = verbose)
	}
}


set_interaction <- function(x, ...) UseMethod("set_interaction", x)

##@set_interaction.sienaEffects Methods new names
set_interaction.sienaEffects <- function(x, shortNames, type="eval",
			depvar=x$name[1], covar1=rep("", 3), covar2=rep("", 3), 
			initialValue = 0, random=FALSE,
			include=TRUE, fix=FALSE, test=FALSE, verbose=TRUE, ...)
{
# Same beginning as set_effect;
# I (TS) did not succeed in making this part into a common function.
	stop23 <- "shortNames should be a list of 2 or 3 shortNames"
	if (!hasArg(shortNames))
	{
		stop("Function set_interaction needs a list of two or three shortNames")
	}
# Test whether the shortNames are given as characters that are not objects:
	suppressWarnings(notObjects <- inherits(try(shortNames[[1]], silent=TRUE), 
										"try-error"), classes = "warning")
	if (notObjects)
	{
		if (length(substitute(shortNames))==1)
		{
			shn1 <- substitute(shortNames)
			stop(stop23)
		}
		else
		{
			shn1 <- substitute(shortNames)[-1]
		}
		shn2 <- sapply(shn1, function(x) deparse(x))	
	}
	else
	{
		if (length(shortNames)==1)
		{
			shn2 <- shortNames
			if (length(shn2) <= 1)
			{
				stop(stop23)
			}
		}
# else test whether the shortNames are {names of objects} or {character strings}:
# (of course those objects also must be character strings)
		else if (is.symbol(substitute(shortNames)[[2]])) # names of objects
		{
			shn2 <- unlist(shortNames)
			if (length(shn2) <= 1)
			{
				stop(stop23)
			}
		}
		else # character strings
		{
			shn1 <- substitute(shortNames)[-1]
			shn2 <- sapply(shn1, function(x) x)	
		}
	}
	if (hasArg("parameter"))
	{
		warning("Function set_interaction does not consider parameters")
	}		
	includeInteraction(x, shn2, include = include, name = depvar,
    type = type, interaction1 = covar1, interaction2 =covar2,
    fix=fix, test=test, random=random, initialValue=initialValue,
    character = TRUE, verbose = verbose)
}

##@print.sienaEffects Methods
print.sienaEffects <- function(x, fileName=NULL, includeOnly=TRUE, 
    expandDummies=FALSE, includeRandoms=FALSE, dropRates=FALSE, includeShortNames=FALSE, ...)
{
    if (!inherits(x, "sienaEffects"))
        stop("not a legitimate Siena effects object")

    if (typeof(fileName)=="character")
    {
        sink(fileName, split=TRUE)
    }

    interactions <- x[x$shortName %in% c("unspInt", "behUnspInt", "contUnspInt") & x$include &
        x$effect1 > 0, ]
    if (expandDummies)
    {
        if(includeOnly && !all(x[x$include, "timeDummy"] == ",")
            || !all(x[, "timeDummy"] == "," ))
        {
            x <- sienaTimeFix(x)$effects
#            x <- fixUpEffectNames(x)
        }
        else
        {
            if (nrow(interactions) > 0)
            {
#                x <- fixUpEffectNames(x)
            }
        }
    }
    if (nrow(x) > 0)
    {
        nDependents <- length(unique(x$name))
        userSpecifieds <- x$shortName[x$include] %in%
            c("unspInt", "behUnspInt", "contUnspInt")
        endowments <- !x$type[x$include] %in% c("rate", "eval", "gmm")
        # includes creations and gmm
        gmm <- any(x$type[x$include] %in% "gmm")
        timeDummies <- !x$timeDummy[x$include] == ","
		if (includeShortNames)
		{
			specs <- as.data.frame(x[, c("effectNumber", "name", "effectName", 
				"shortName", "include", "fix",
                "test", "initialValue", "parm")])	
		}
		else
		{
			specs <- as.data.frame(x[, c("name", "effectName", "include", "fix",
                "test", "initialValue", "parm")])
		}
        if (includeOnly)
        {
            included <- x$include
        }
        else
        {
            included <- rep(TRUE, nrow(x))
        }
        if (dropRates)
        {
            included <- included & !x$basicRate
        }
        specs <- specs[included, ]
        if (nrow(specs) > 0)
        {
            row.names(specs) <- 1:nrow(specs)
        }
        if (nDependents == 1)
        {
			whichname <- which(names(specs)=="name")
            specs <- specs[, -whichname]  # drop name of dependent variable
        }
        if (any(endowments))
        {
            specs <- cbind(specs, type=x[included, "type"])
        }
        if (gmm)
        {
          specs <- cbind(specs, type=x[included, "type"])
        }
        if (any(timeDummies))
        {
            specs <- cbind(specs, timeDummy=x[included, "timeDummy"])
        }
        if (any(userSpecifieds))
        {
            specs <- cbind(specs, x[included, c("effect1", "effect2")])
            if (any (x$effect3[included] > 0))
            {
                specs <- cbind(specs, effect3=x[included, "effect3"])
            }
        }
        if (includeRandoms)
        {
            specs <- cbind(specs, randomEffects=x[included, "randomEffects"])
        }
        specs[, "initialValue"] <- format(round(specs$initialValue,digits=5),
            width=10)
        if (nrow(specs) > 0)
        {
          if (gmm)
              {
                cat('\n Effects and statistics for estimation by the Generalized Method of Moments\n')
                if (any(specs$type!='gmm'))
                {
                  cat('\n Effects\n')
                  specs1 <- specs[specs$type!='gmm',]
                  row.names(specs1) <- 1:nrow(specs1)
                  print(as.matrix(specs1), quote=FALSE)
                }
                if (any(specs$type=='gmm'))
                {
                  cat('\n Regular and GMoM statistics\n')
                  specs2 <- specs[specs$type=='gmm',]
                  specs3 <- rbind(specs1, specs2)
                  specs3$Statistic <- c(rep("Regular",nrow(specs1)),
                                        rep("GMoM",nrow(specs2)))
                  row.names(specs3) <- 1:nrow(specs3)
                  print(as.matrix(specs3[,c(1:2,9)]), quote=FALSE)
                }
              }
            else
            {
                print(as.matrix(specs), quote=FALSE)
            }
        }
        else
        {
            print.data.frame(specs)
        }
    }
    else
    {
        print.data.frame(x)
    }
    if (includeRandoms)
    {
        nfeff <- sum((!x$randomEffects) & x$include & (!x$fix) & (!x$basicRate))
        nreff <- sum(x$randomEffects & x$include & (!x$fix) & (!x$basicRate))
        nrate <- sum(x$basicRate & x$include & (x$group==2) & (!x$fix))
        if (sum(x$group != 1) > 0) # else there is only one group, and counting should be different.
        {
            cat('Length of priorSigEta for sienaBayes, if used, should be',
                nfeff, '.\n')
            cat('Dimensions of priorMu and priorSigma for sienaBayes should be',
                nreff, '+', nrate, '=', nreff+nrate,'.\n')
        }
        else if (dim(x)[1] > 1)
        # to avoid the following if called from includeInteraction or setEffect
        {
            cat('There are', nreff, 'random effects.\n')
        }
    }
    if (typeof(fileName)=="character")
    {
        sink()
    }
    invisible(x)
}

##@summary.sienaEffects Methods
summary.sienaEffects <- function(object, fileName=NULL, includeOnly=TRUE,
    expandDummies=FALSE, ...)
{
    if (!inherits(object, "sienaEffects"))
        stop("not a legitimate Siena effects object")
    if (expandDummies && (includeOnly && !all(object[object$include,
                "timeDummy"] == ",")
            || !all(object[, "timeDummy"] == ",")))
    {
        object <- sienaTimeFix(object)$effects
    }
    if (includeOnly)
    {
        object <- object[object$include, ]
    }
    class(object) <- c("summary.sienaEffects", class(object))
    object
}

##@print.summary.sienaEffects Methods
print.summary.sienaEffects <- function(x, fileName=NULL, ...)
{
    if (!inherits(x, "summary.sienaEffects"))
        stop("not a legitimate summary of a Siena effects object")
    ## find out if any columns need removing because they will not print

    problem <- sapply(x, function(x) {
        inherits(try(unlist(x[[1]]), silent=TRUE), "try-error")
    })
    if (any(problem))
    {
        x1 <- x[, problem]
        x <- x[, !problem]
    }
    if (typeof(fileName)=="character")
    {
        sink(fileName, split=TRUE)
    }
    print.data.frame(x)
    if (any(problem))
    {
        lapply(x1, print)
    }
    if (typeof(fileName)=="character")
    {
        sink()
    }
    invisible(x)
}

edit.sienaEffects <- function(name, ...)
{
    if (!interactive())
    {
        return(name)
    }
    ## store the original column order
    originalNames <- names(name)
    ## move function name and other things out of the way
    priorityColumns <- c("name", "effectName", "type", "include", "fix",
        "test", "initialValue", "parm", "shortName")
    priorityX <- name[, priorityColumns]
    notPriorityX <- name[, -c(match(priorityColumns, names(name)))]
    name <- cbind(priorityX, notPriorityX)

    ## get edit.data.frame to do the actual edit
    tmp <- NextMethod(, , edit.row.names=FALSE)

    ## re-sort the columns
    tmp <- tmp[, match(originalNames, names(tmp))]
    class(tmp) <- c("sienaEffects", class(tmp))
	 attr(tmp, "version") <- packageDescription(pkgname, fields = "Version")
    tmp
}

##@updateSpecification Methods add specified effects from other effects object
updateSpecification <- function(effects.to, effects.from,
									effects.extra=NULL, name.to=NULL, name.from=NULL)
{
    if (!inherits(effects.to, "sienaEffects"))
    {
        stop("effects.to is not an effects object")
    }
    if (!inherits(effects.from, "sienaEffects"))
    {
        stop("effects.from is not an effects object")
    }
    if (is.null(name.from))
    {
        prevEffects <-
            effects.from[which((effects.from$type != 'gmm')&((effects.from$include))),]
    }
    else
    {
        if (!is.character(name.from))
        {
            stop("name.from should be a string")
        }
        prevEffects <-
            effects.from[which((effects.from$type != 'gmm')&((effects.from$include))&
                (effects.from$name == name.from)),]
    }
    oldlist <- apply(prevEffects, 1, function(x)
                     paste(x[c("name", "shortName",
                               "type", "groupName",
                               "interaction1", "interaction2",
                               "period", "effect1", "effect2", "effect3")],
                           collapse="|"))
    efflist <- apply(effects.to, 1, function(x)
                     paste(x[c("name", "shortName",
                               "type", "groupName",
                               "interaction1", "interaction2",
                               "period", "effect1", "effect2", "effect3")],
                            collapse="|"))
    if (!(is.null(name.to)))
    {
        if (!is.character(name.to))
        {
            stop("name.to should be a string")
        }
        else if (name.to == "all") # omit matching on "name"
        {
            oldlist <- apply(prevEffects, 1, function(x)
                     paste(x[c("shortName",
                               "type", "groupName",
                               "interaction1", "interaction2",
                               "period", "effect1", "effect2", "effect3")],
                           collapse="|"))
            efflist <- apply(effects.to, 1, function(x)
                     paste(x[c("shortName",
                               "type", "groupName",
                               "interaction1", "interaction2",
                               "period", "effect1", "effect2", "effect3")],
                            collapse="|"))
        }
    }
    use <- (efflist %in% oldlist)
    correspondence <- match(efflist, oldlist)
    if (!(is.null(name.to)))
    {
        if (name.to != "all")
        {
            use <- (use & (effects.to$name == name.to))
        }
    }
    effects.to$include[use] <- TRUE
    effects.to$fix[use] <- prevEffects$fix[correspondence][use]
    effects.to$test[use] <- prevEffects$test[correspondence][use]
    effects.to$parm[use] <- prevEffects$parm[correspondence][use]
    effects.to$randomEffects[use] <- prevEffects$randomEffects[correspondence][use]
    effects.to$initialValue[use] <- prevEffects$initialValue[correspondence][use]
# the above does not transfer interaction effects.
# A lot of work is needed to get the information about the interacting effects.
    inter <- which(prevEffects$include &
						(prevEffects$shortName %in% c("unspInt","behUnspInt","contUnspInt")))
    if (length(inter) >= 1)
	{
	# look up the interacting main effects, and try to get information
	# about which are the interacting effects.
	    cat("Handling interactions ... ")
		efn1 <- prevEffects$effect1[inter]
		efn2 <- prevEffects$effect2[inter]
		efn3 <- prevEffects$effect3[inter]
	# prepare a stopmessage; this will possibly be used at various places.
		if (is.null(effects.extra))
		{
			stopMessage <- paste("Effects object ", deparse(substitute(effects.from)),
					" contains some interactions \n",
					" but there is no information ",
					"for the corresponding main effects.", sep="")
		}
		else
		{
			stopMessage <- paste("Effects object ", deparse(substitute(effects.from)),
					" contains some interactions \n",
					"  and neither this nor ", deparse(substitute(effects.extra)),
					"\n contains information for the corresponding main effects.", sep="")
		}
	# Note that some of efn3 may be 0,
	# and effects.from$effectNumber starts counting from 1.
	# First try to find the corresponding main effects in effects.from
		three <- (efn3 > 0)
		mefn1 <- match(efn1, effects.from$effectNumber)
		mefn2 <- match(efn2, effects.from$effectNumber)
		mefn3 <- ifelse(three, match(efn3, effects.from$effectNumber), 0)
		effects.fr <- effects.from
		if (any(is.na(c(mefn1,mefn2,mefn3))))
		{
			if (is.null(effects.extra))
			{
				stop("Effects object ", deparse(substitute(effects.from)),
					" contains some interactions \n",
					"  without information for the corresponding main effects,\n",
					"  and there is no effects.extra.")
			}
			else
			{
				if (!inherits(effects.extra, "sienaEffects"))
				{
					stop("effects.extra is needed, and is not an effects object")
				}
	# Now try to find the corresponding main effects in effects.extra
				version1 <- attr(effects.from, "version")
				version2 <- attr(effects.extra, "version")
				sameversion <- FALSE
				if ((!is.null(version1)) & (!is.null(version2)))
				{
					sameversion <- (version1==version2)
				}
				if (!sameversion)
				{
					warning("RSiena versions of effects.from and effects.extra",
						" are different;\n",
						"  check that the effects object",
						" generated by updateSpecification is correct.")
				}
				mefn1 <- match(efn1, effects.extra$effectNumber)
				mefn2 <- match(efn2, effects.extra$effectNumber)
				mefn3 <- ifelse(three, match(efn3, effects.extra$effectNumber), 0)
				if (any(is.na(c(mefn1,mefn2,mefn3))))
				{
					stop(stopMessage)
				}
				effects.fr <- effects.extra
			}
		}
        shn1  <- effects.fr[mefn1,"shortName"]
        shn2  <- effects.fr[mefn2,"shortName"]
		shn3  <- rep('', length(three))
        shn3[three]  <- effects.fr[mefn3[three],"shortName"]
        int11  <- effects.fr[mefn1,"interaction1"]
        int12  <- effects.fr[mefn2,"interaction1"]
		int13  <- rep('', length(three))
        int13[three]  <- effects.fr[mefn3[three],"interaction1"]
        int21  <- effects.fr[mefn1,"interaction2"]
        int22  <- effects.fr[mefn2,"interaction2"]
		int23  <- rep('', length(three))
        int23[three]  <- effects.fr[mefn3[three],"interaction2"]
        nam  <- prevEffects[inter,"name"]
        typ   <- prevEffects[inter,"type"]
        fixx  <- prevEffects[inter,"fix"]
        tests <- prevEffects[inter,"test"]
        rand  <- prevEffects[inter,"randomEffects"]
		initv <- prevEffects[inter,"initialValue"]
#browser()
        for (k in seq_along(inter)){
			cat(k," ")
			flush.console()
			if (three[k])
			{
				if (inherits(try(
						effects.to <- includeInteraction(effects.to, shn1[k], shn2[k], shn3[k],
						name=nam[k],
						interaction1=c(int11[k],int12[k],int13[k]),
						interaction2=c(int21[k],int22[k],int23[k]),
						initialValue=initv[k],
						type=typ[k], fix=fixx[k], test=tests[k], random=rand[k],
						verbose=FALSE, character=TRUE), 
					silent=TRUE), "try-error"))
				{
					stop(stopMessage)
				}
			}
			else
			{
				if (inherits(try(
					effects.to <- includeInteraction(effects.to, shn1[k], shn2[k],
						name=nam[k],
						interaction1=c(int11[k],int12[k]),
						interaction2=c(int21[k],int22[k]),
						initialValue=initv[k],
						type=typ[k], fix=fixx[k], test=tests[k], random=rand[k],
						verbose=FALSE, character=TRUE), 
					silent=TRUE), "try-error"))
				{
					stop(stopMessage)
				}
			}
        }
		cat("\n")
    }
    effects.to
}

update_specification <- function(x, ...) UseMethod("update_specification", x)

##@update_specification Methods add specified effects from other effects object
update_specification.sienaEffects <- function(x, effects.from,
					effects.extra=NULL, name.to=NULL, name.from=NULL, ...){
	updateSpecification(effects.to=x, effects.from=effects.from,
		effects.extra=effects.extra, name.to=name.to, name.from=name.from)
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: sienautils.r
# *
# * Description: This module contains utilities for creating siena special
# * objects from vectors etc
# *****************************************************************************/

##@sienaCompositionChange Create
sienaCompositionChange <- function(changelist, nodeSet="Actors", option=1)
{
    if (!is.list(changelist))
	{
        stop("changelist must be a list")
	}
    warn <- getOption("warn")
    options(warn = -1)
    changelist <- lapply(changelist, as.numeric)
    options(warn = warn)
    if (any(sapply(changelist,function(x)any(is.na(x)))))
	{
        stop("Non numeric data")
	}
    out <- changelist
    class(out) <- "compositionChange"
	 attr(out, "version") <- packageDescription(pkgname, fields = "Version")
    if (is.vector(nodeSet) && length(nodeSet) > 1)
	{
        stop ("only one node set relevant")
	}
    if (!is.character(nodeSet))
	{
		stop ("nodeset should be a character string")
	}
    if (!is.numeric(option))
	{
        stop("option should be numeric")
	}
    if (option < 1 || option > 4)
	{
        stop("option should be between 1 and 4")
	}
    attr(out, "nodeSet") <- nodeSet
    attr(out, "ccOption") <- option
    out
}
##@sienaCompositionChangeFromFile Create
sienaCompositionChangeFromFile <- function(filename, nodeSet="Actors",
                                           fileobj=NULL, option=1)
{
    if (is.null(fileobj))
	{
        tmp <- readLines(filename)
	}
    else
	{
        tmp <- fileobj
	}
    changelist <- lapply(tmp, function(x)
                     {
						 ##  cat("x1",x)
                         x <- sub("^ +", "", x)
						 ##  cat("x2",x, "x")
                         x <- unlist(strsplit(x, " +"))
						 ##   print(x)
                         x
                     })
    warn <- getOption("warn")
    options(warn = -1)
    changelist <- lapply(changelist, as.numeric)
    options(warn = warn)
    if (any(sapply(changelist, function(x)any(is.na(x)))))
	{
        stop("Non numeric data")
	}
    out <- changelist
    class(out) <- "compositionChange"
	 attr(out, "version") <- packageDescription(pkgname, fields = "Version")
    if (is.vector(nodeSet) && length(nodeSet) > 1)
	{
        stop ("only one node set relevant")
	}
    if (!is.character(nodeSet))
	{
        stop ("nodeset should be a character string")
	}
    if (!is.numeric(option))
	{
        stop("option should be numeric")
	}
    if (option < 1 || option > 4)
	{
        stop("option should be between 1 and 4")
	}
    attr(out, "nodeSet") <- nodeSet
    attr(out, "ccOption") <- option
    out
}

##@as_composition_rsiena Create
as_composition_rsiena <- sienaCompositionChange

##@as_composition_rsiena Create
as_composition_file_rsiena <- sienaCompositionChangeFromFile

##@as_nodeset_rsiena Create
as_nodeset_rsiena <- function(n, nodeSetName="Actors", names=NULL)
{
    if (!is.numeric(n))
	{
        stop("n must be numeric")
	}
    if (!is.character(nodeSetName))
	{
        stop("node set name must be a character string")
	}
    if (!is.null(names) && length(names) != n)
	{
        stop("node names incorrect length")
	}
    out <- 1:n
    attributes(out) <- list(class="sienaNodeSet", nodeSetName=nodeSetName)
    if (!is.null(names))
	{
        names(out) <- names
	}
    out
}

sienaNodeSet <- as_nodeset_rsiena


##@as_covariate_rsiena create covariate
as_covariate_rsiena <- function(val, type="monadic", centered=TRUE, 
								nodeSet="Actors", warn=TRUE, 
								imputationValues=NULL)
# Defines val as a covariate.
# It figures out whether it is a monadic or dyadic covariate,
# and if dyadic, whether it is a list of sparse matrices.
{
	if ((is.null(dim(val)) | (length(dim(val))==1)) &  (type=="monadic")) # Then val should be a numeric vector
	{
		if (is.null(centered))
		{
			centered <- TRUE
		}
		return(coCovar(val, centered=centered, nodeSet=nodeSet, warn=warn, 
								imputationValues=imputationValues))
	}
	else
	{
		if (length(dim(val))==2) # Then val should be a numeric matrix
		{
			if (is.null(centered))
			{
				centered <- TRUE
			}
			if (type=="monadic")
			{
				return(varCovar(val, centered=centered, nodeSet=nodeSet, 
								warn=warn, imputationValues=imputationValues))
			}
			else if (type %in% c("oneMode", "bipartite")) 
							# then it is a dyadic covariate
			{
				if (dim(val)[1] == dim(val)[2])
				{
					if (length(nodeSet)==1)
					{
						nodeSet <- c(nodeSet, nodeSet)
					}
					else if (type=="oneMode")
					{
	stop("The one-mode dyadic covariate should be represented by a square matrix")
					}
				}
				return(coDyadCovar(val, centered=centered, nodeSets=nodeSet, 
											warn=warn, type=type))
# The check of sparse should follow from the default in coDyadCovar
			}
			else
			{
stop("For a matrix of values, a type (monadic, oneMode or bipartite) should be given")
			}
		}
		else
		{
			if ((length(dim(val))==3) | is.list(val)) 
			# Then val is a 3-dimensional array or list and the covariate should be dyadic
			{
				if (length(nodeSet)==1)
				{
					nodeSet <- c(nodeSet, nodeSet)
				}
				symmetric <- ifelse(is.list(val), (dim(val[[1]]) == dim(val[[2]])),
												 (dim(val)[1] == dim(val)[2]))
				if ((!symmetric) & (type=="oneMode"))
				{
		stop("The one-mode dyadic covariate should be represented by square matrices")
				}
				if (is.list(val) & is.null(centered))
				{
					centered <- FALSE
				}
				return(varDyadCovar(val, centered=centered, nodeSets=nodeSet, 
															warn=warn, type=type))
# The check of sparse should follow from the default in coDyadCovar
			}
			else
			{
				stop("For this value, impossible to choose between covariate types")
			}		
		}
	}
}

##@coCovar Create
coCovar <- function(val, centered=TRUE, nodeSet="Actors", warn=TRUE, 
													imputationValues=NULL)
{
    ##val should be vector, numeric or factor
	if (all(is.na(val)))
	{
		stop("all values are missing")
	}
    if (!is.vector(val))
	{
        stop("val must be a vector")
	}
    if (!(is.numeric(val) || is.factor(val) || (all(is.na(val)))))
	{
        stop("val must be numeric or a factor")
	}
	if (warn)
	{
		if (sum(!is.na(val)) == 1)
		{
			warning("Note: only one value is non-missing.")
		}
		else if (!is.factor(val)) 
		{
			if (var(val, na.rm=TRUE)==0)
			{
				warning('Note: all non-missing values are identical to ', 
							mean(val, na.rm=TRUE),'.')
			}
		}
	}
    if (!is.character(nodeSet))
	{
        stop ("nodeset should be a character string")
	}
	if (!is.null(imputationValues))
    {
		if (!(is.numeric(imputationValues) || is.factor(imputationValues)))
		{
			stop("imputationValues must be numeric or a factor")
		}
		if (anyNA(imputationValues))
		{
			stop("imputationValues must not contain any NA values")
		}
		if (length(imputationValues) != length(val))
		{
			stop("imputationValues must have the same length as val")
		}
	}
    out <- val
    class(out) <- "coCovar"
	 attr(out, "version") <- packageDescription(pkgname, fields = "Version")
    attr(out, "centered") <- centered
    attr(out, "nodeSet") <- nodeSet
    attr(out, "imputationValues") <- imputationValues
    out
}

##@varCovar Create
varCovar<- function(val, centered=TRUE, nodeSet="Actors", warn=TRUE, imputationValues=NULL)
{
    ##matrix, numeric or factor, nrow = nactors and cols = observations-1
	if (all(is.na(val)))
	{
		stop("all values are missing")
	}
    if (!is.matrix(val))
	{
        stop("val must be a matrix")
	}
    if (!(is.numeric(val) || is.factor(val)))
	{
        stop("val must be numeric or a factor")
	}
	if (warn)
	{
		if (sum(!is.na(val)) == 1)
		{
			warning("Note: only one value is non-missing.")
		}
		else if (!is.factor(val)) 
		{
			if (var(as.vector(val), na.rm=TRUE)==0)
			{
				warning('Note: all non-missing values are identical to ', 
							mean(val, na.rm=TRUE),'.')
			}
		}
	}
    if (!is.character(nodeSet))
	{
        stop ("nodeset should be a character string")
	}
    if (!is.null(imputationValues))
    {
		if (!(is.numeric(imputationValues) || is.factor(imputationValues)))
		{
			stop("imputationValues must be numeric or a factor")
		}
		if (any(is.na(imputationValues)))
		{
			stop("imputationValues must not contain any NA values")
		}
		if (any(dim(imputationValues) != dim(val)))
		{
			stop("imputationValues must have the same dimension as val")
		}
	}
    out <- val
    class(out) <- "varCovar"
	 attr(out, "version") <- packageDescription(pkgname, fields = "Version")
    attr(out, "centered") <- centered
    attr(out, "nodeSet") <- nodeSet
    attr(out, "imputationValues") <- imputationValues
    out
}

##@coDyadCovar Create
coDyadCovar<- function(val, centered=TRUE, nodeSets=c("Actors","Actors"),
					   warn=TRUE, sparse=inherits(val,"TsparseMatrix"),
					   type=c("oneMode", "bipartite"))
{
    ##matrix, numeric or factor, dims= those of net - must validate later or
	## sparse matrix
    if (!sparse)
    {
        if (!is.matrix(val))
        {
            stop("val must be a matrix")
        }
        if (!(is.numeric(val) || is.factor(val)))
        {
            stop("val must be numeric or a factor")
        }
    }
	if ((!is.factor(val)) & warn)
	{
		if (max(val, na.rm=TRUE) == min(val, na.rm=TRUE))
		{
			warning('Note: all values are identical to ', max(val, na.rm=TRUE),'.')
		}
	}
    if (sparse)
    {
        if (!inherits(val, "TsparseMatrix"))
        {
            stop("not a sparse triples matrix")
        }
        val <- list(val)
    }
	if ((sum(!is.na(val))==0) & warn)
	{
		warning('Note: all values are missing.')
	}
    vardims <- dim(val)
    if (length(nodeSets) > 2)
    {
        stop("nodeSets may only have one or two elements")
    }
    if (!is.character(nodeSets))
    {
        stop("nodeSets must be a vector of character strings")
    }
	if (missing(type))
	{
		if (nodeSets[1] == nodeSets[2])
		{
			type <- "oneMode"
		}
		else
		{
			type <- "bipartite"
		}
	}
	else
	{
		type <- match.arg(type)
	}
	if (type == "bipartite")
	{
		if (nodeSets[1] == nodeSets[2])
		{
			stop("bipartite covariate needs two different node sets")
		}
	}
	else
	{
		if (nodeSets[1] != nodeSets[2])
		{
			stop("Both node sets must be the same for a oneMode covariate")
		}
	}
    out <- val
    class(out) <- "coDyadCovar"
	 attr(out, "version") <- packageDescription(pkgname, fields = "Version")
	attr(out, "type") <- type
    attr(out, "centered") <- centered
    attr(out, "nodeSet") <- nodeSets
    attr(out, "sparse") <- sparse
    attr(out, "vardims") <- vardims
    out
}
##@varDyadCovar Create
varDyadCovar<- function(val, centered=TRUE, nodeSets=c("Actors","Actors"), 
						warn=TRUE, sparse=is.list(val),
						type=c("oneMode", "bipartite"))
{
    ##array, numeric or factor, dims= those of net by observations-1 -
    ##must validate later or list of sparse matrices
    if (!sparse)
    {
        if (!is.array(val) || !(length(dim(val)) == 3))
            stop("val must be a 3d array")
        if (!(is.numeric(val) || is.factor(val)))
            stop("val must be numeric or a factor")
        vardims <- dim(val)
		if ((!is.factor(val)) & warn)
		{
			if (max(val, na.rm=TRUE) == min(val, na.rm=TRUE))
			{
				warning('Note: all values are identical to ', max(val, na.rm=TRUE),'.')
			}
		}
    }
    else
    {
         if (!is.list(val))
            stop("values must be an array or a list of sparse matrices")
        if (!all(sapply(val, function(x){inherits(x,"TsparseMatrix", which = FALSE)})))
            stop("not a list of sparse triples matrices")
        vardims <- sapply(val, dim) ## dimensions of matrices in columns
        if (any(vardims != vardims[, 1]))
            stop("all matrices must have the same dimension")
        vardims <- vardims[, 1]
        vardims[3] <- length(val)
    }
	if ((sum(!is.na(val))==0) & warn)
	{
		warning('Note: all values are missing.')
	}
    if (length(nodeSets) > 2)
        stop("nodeSets may only have one or two elements")
    if (!is.character(nodeSets))
        stop("nodeSets must be a vector of character strings")
    if (length(nodeSets) == 1)
        nodeSets <- c(nodeSets, nodeSets)
	if (missing(type))
	{
		if (nodeSets[1] == nodeSets[2])
		{
			type <- "oneMode"
		}
		else
		{
			type <- "bipartite"
		}
	}
	else
	{
		type <- match.arg(type)
	}
	if (type == "bipartite")
	{
		if (nodeSets[1] == nodeSets[2])
		{
			stop("bipartite covariate needs two different node sets")
		}
	}
	else
	{
		if (nodeSets[1] != nodeSets[2])
		{
			stop("Both node sets must be the same for a oneMode covariate")
		}
	}
    out <- val
    class(out) <- "varDyadCovar"
	 attr(out, "version") <- packageDescription(pkgname, fields = "Version")
	attr(out, "type") <- type
    attr(out, "centered") <- centered
    attr(out, "nodeSet") <- nodeSets
    attr(out, "sparse") <- sparse
    attr(out, "vardims") <- vardims
    out
}

##@sienaDependent Create
sienaDependent <- function(netarray, type=c("oneMode","bipartite","behavior",
					"continuous"), nodeSet="Actors", sparse=is.list(netarray),
					allowOnly=TRUE, imputationValues=NULL)
{
	if (inherits(netarray,'data.frame'))
	{
		stop("The first argument must not be a data.frame, but an array or a list of sparse matrices.")
	}
	if (!sparse)
    {
        if (!is.array(netarray))
		{
            stop("The first argument must be an matrix, array, or list of sparse matrices")
		}
        netdims <- dim(netarray)
        if (length(netdims) == 2) ## assume behavior network
        {
            dim(netarray) <- c(netdims[1], 1, netdims[2])
            netdims <- dim(netarray)
        }
        if (!is.numeric(netarray))
		{
            stop("entries must be numeric")
		}
    }
    else
    {
      #  require(Matrix)
        if (!is.list(netarray))
		{
            stop("The first argument must be an array or a list of sparse matrices")
		}
        if (!all(sapply(netarray, function(x){inherits(x,"TsparseMatrix", which = FALSE)})))
		{
            stop("this is not a list of sparse triples matrices")
		}
        netdims <- sapply(netarray, dim) ## dimensions of network in columns
        if (any(netdims != netdims[, 1]))
		{
            stop("all matrices must have the same dimension")
		}
        netdims <- netdims[, 1]
        netdims[3] <- length(netarray)
    }
	if (sum(!is.na(netarray))==0)
	{
		warning('Note: all values are missing.')
	}
	observations <- netdims[3]
    if (observations < 2)
	{
        stop("netarray must have at least two observations")
	}
    if (netdims[2] == 1) ## how we defined a behavior array earlier
    {
        if (missing(type))
		{
            type <- "behavior"
		}
        else
        {
            type <- match.arg(type)
            if (type != "behavior" && type != "continuous")
			{
                stop("incorrect type")

			}
		}
    }
    else if (netdims[1] != netdims[2])
    {
        if (missing(type))
		{
            type <- "bipartite"
		}
        else
        {
            type <- match.arg(type)
            if (type != "bipartite")
			{
                stop("incorrect type")
			}
		}
    }
    else
    {
        type <- match.arg(type)
        if (type == "behavior")
		{
            stop("incorrect type")
		}
    }
    if (!is.character(nodeSet))
	{
        stop ("nodeSet should be a character string")
	}
    if (type == "bipartite")
    {
        if (!is.vector(nodeSet) || length(nodeSet) != 2)
		{
            stop ("need 2 node sets for a bipartite network")
		}
        if (!is.character(nodeSet[[1]]) || ! is.character(nodeSet[[2]]))
		{
            stop ("nodesets should be character strings")
		}
		if (nodeSet[[1]] == nodeSet[[2]])
		{
            stop ("nodesets should be different from each other")
		}
    }
    else
     {
        if (is.vector(nodeSet) && length(nodeSet) > 1)
		{
            stop ("only one node set for this network")
		}
        if (!is.character(nodeSet))
		{
            stop ("nodeSet should be a character string")
		}
    }
    if (type != "behavior" && type != "continuous")
    {
        if (sparse)
        {
            netarray <- lapply(netarray, function(x)as(drop0(x), "TsparseMatrix"))
            if (!all(sapply(netarray, function(x)
                        {
                            tmp <- x@x
                            all(is.na(tmp) | tmp == 1 | tmp == 10 |
                                    tmp == 11 )
                            }
                                )))
                 stop("entries in networks must be 0, 1, 10, 11, or NA")
         }
        else
        {
            if (!all(netarray %in% c(0, 1, 10, 11) | is.na(netarray)))
			{
                stop("entries in networks must be 0, 1, 10 or 11")
			}
        }
    }

  	if (!is.null(imputationValues))
    {
        if (type != "behavior" && type != "continuous")
        {
            stop("imputationValues only implemented for dependent behavior variables")
        }
		if (!(is.numeric(imputationValues)))
		{
			stop("imputationValues must be numeric or a factor")
		}
		if (any(is.na(imputationValues)))
		{
			stop("imputationValues may not contain any NA values")
		}
		if (nrow(imputationValues) != netdims[1] || ncol(imputationValues) != netdims[3])
		{
			stop("imputationValues and data must have the same dimension")
		}
    }

    obj <- netarray
    class(obj) <- ("sienaDependent")
	attr(obj, "version") <- packageDescription(pkgname, fields = "Version")
    attr(obj, "type") <- type
    attr(obj, "sparse") <- sparse
    attr(obj, "nodeSet") <- nodeSet
    attr(obj, "netdims") <- netdims
	attr(obj, "allowOnly") <- allowOnly
    if (!is.null(imputationValues))
    {
        attr(obj, "imputationValues") <- imputationValues
    }
    obj
}

##@sienaNet sienaDependent synonym
sienaNet <- sienaDependent

##@as_dependent_rsiena sienaDependent synonym
as_dependent_rsiena <- sienaDependent

##@sienaDataConstraint DataCreate
sienaDataConstraint <- function(x, net1, net2, 
		type, value=FALSE)
{
 #   type <- match.arg(type)
    net1 <- deparse(substitute(net1))
    net2 <- deparse(substitute(net2))	
 # Check if it looks like a pipe chain 
	if (grepl("%>%|\\|>", net1) ||  grepl("%>%|\\|>", net2))
	{
		stop("Cannot auto-generate filename from piped data. Please apply this function to net1 and net2 with a name.") 
	}
	sienaDataConstraintFromStrings(x, net1, net2, type, value)
}


##@sienaDataConstraintFromStrings DataCreate
sienaDataConstraintFromStrings <- function(x, net1, net2, 
		type, value=FALSE)
{
    pairname <- paste(net1, net2, sep=",")
    atts <- attr(x, type)
    if (is.null(atts))
    {
        stop("No constraints calculated: Need to recreate data object")
    }
    if (!(pairname %in% names(atts)))
    {
        stop("No such constraint found")
    }

    if (value == attr(x, type)[pairname])
    {
        message("No change in constraint")
    }
    else
    {
        attr(x, type)[pairname] <- value
    }

	if (type %in% c("disjoint", "atLeastOne"))
	{
		pairname <- paste(net2, net1, sep=",")
		attr(x, type)[pairname] <- value
	}
    x
}

make_constraint <- function(x, ...) UseMethod("make_constraint", x)

##@make_constraint  method for siena data objects
make_constraint.sienadata <- function(x, net1, net2, 
		type, value=FALSE, ...)
{
    net1 <- deparse(substitute(net1))
    net2 <- deparse(substitute(net2))	
 # Check if it looks like a pipe chain 
	if (grepl("%>%|\\|>", net1) ||  grepl("%>%|\\|>", net2))
	{
		stop("Cannot auto-generate filename from piped data. Please apply this function to net1 and net2 with a name.") 
	}
	sienaDataConstraintFromStrings(x, net1, net2, type, value)
}

##@make_constraint  method for sienaGroup data objects
make_constraint.sienaGroup <- function(x, net1, net2, 
		type, value=FALSE, ...)
{
    net1 <- deparse(substitute(net1))
    net2 <- deparse(substitute(net2))	
 # Check if it looks like a pipe chain 
	if (grepl("%>%|\\|>", net1) ||  grepl("%>%|\\|>", net2))
	{
		stop("Cannot auto-generate filename from piped data. Please apply this function to net1 and net2 with a name.") 
	}
	sienaDataConstraintFromStrings(x, net1, net2, type, value)
}

## extra for backward compatibility with siena objects
## created before version 1.6:

##@make_constraint.siena Methods
make_constraint.siena <- make_constraint.sienadata

library(RSiena)

# When new effects are added, the numbering of effects changes.
# This will have consequences for the output of set_interaction,
# and will require adaptation of parallel.Rout.save.

##test3
mynet1 <- as_dependent_rsiena(array(c(tmp3, tmp4),dim=c(32, 32, 2)))
mydata <- make_data_rsiena(mynet1)
myeff<- make_specification(mydata)
print('test3')
alg_alg <- set_algorithm_saom(cond=FALSE, seed=3, n3=50, nsub=2, findiff=TRUE)
ans <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
ans
(myeff <- set_effect(myeff, list(transTrip, cycle4)))
(myeff <- set_effect(myeff, cycle4, include=FALSE))
##test4
print('test4')
alg_alg <- set_algorithm_saom(cond=TRUE, condvarno=1, seed=3, n3=50, nsub=2, findiff=TRUE)
ans <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
ans
##test5
mynet1 <- as_dependent_rsiena(array(c(tmp3,tmp4),dim=c(32,32,2)))
mydata <- make_data_rsiena(mynet1)
myeff<- make_specification(mydata)
print('test5')
alg_alg <- set_algorithm_saom(cond=FALSE, seed=5, n3=50, nsub=2)
ans <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
ans
(myeff <- set_effect(myeff, list(recip, inPop)))
(myeff <- set_effect(myeff, outAct, fix=TRUE, test=TRUE))
(myeff <- set_interaction(myeff, list(recip, inPop), fix=TRUE, test=TRUE))
alg_alg <- set_algorithm_saom(cond=FALSE, seed=5, n3=50, nsub=2)
ans <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg, returnDeps=TRUE)
ans
test_parameter(ans, method='score')
(goft <- test_gof(ans, IndegreeDistribution, verbose=TRUE, varName="mynet1",
          test=NULL))
summary(goft)
##test6
mynet1 <- as_dependent_rsiena(array(c(tmp3,tmp4),dim=c(32,32,2)))
mydata <- make_data_rsiena(mynet1)
myeff<- make_specification(mydata)
print('test6')
alg_alg <- set_algorithm_saom(cond=FALSE, seed=5, n3=50, nsub=2)
ans <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
ans
myeff <- set_effect(myeff, recip, include=FALSE)
myeff <- set_effect(myeff, recip, type="endow")
myeff <- set_effect(myeff, recip, type="creation")
alg_alg <- set_algorithm_saom(cond=FALSE, seed=5, n3=50, nsub=2)
ans <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
ans
test_parameter(ans, method='same', tested=3, tested2=4)
##test7
mynet1 <- as_dependent_rsiena(array(c(tmp3,tmp4),dim=c(32,32,2)))
mydata <- make_data_rsiena(mynet1)
myeff<- make_specification(mydata)
print('test7')
alg_alg <- set_algorithm_saom(cond=FALSE, seed=5, n3=50, nsub=2,
         diagonalize=0.5)
ans <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
ans
##test8
print('test8')
alg_alg <- set_algorithm_saom(cond=TRUE, condvarno=1, seed=5, n3=50, nsub=1)
ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
ans
##test9
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mynet2 <- as_dependent_rsiena(s50a,type='behavior')
mydata <- make_data_rsiena(mynet1, mynet2)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, linear, depvar="mynet2", initialValue=0.34699930338)
myeff <- set_effect(myeff, avAlt, depvar="mynet2", covar1="mynet1")
##test10
print('test10')
alg_alg <- set_algorithm_saom(cond=TRUE, condvarno=1, seed=5, n3=50, nsub=1)
ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
alg_alg2 <- set_algorithm_saom(cond=TRUE, condvarno=1, seed=5, n3=50, nsub=1,
			splitDepvars=1)
ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg2)
##, verbose=TRUE)
ans
##test11
print('test11')
alg_model <- set_model_saom(behModelType=c(mynet2=2))
alg_alg <- set_algorithm_saom(seed=6, n3=50, nsub=1)
(ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_model=alg_model, control_algo=alg_alg))
##test12
print('test12')
use<- 1:30
mynet1 <- as_dependent_rsiena(array(c(s501[use,], s502[use,], s503[use,]),
                         dim=c(length(use), 50,3)), type='bipartite',
                         nodeSet=c('Senders','receivers'))
receivers <- as_nodeset_rsiena(50,'receivers')
senders <- as_nodeset_rsiena(30,'Senders')
myvar1 <- as_covariate_rsiena(s50a[1:30,2], nodeSet='Senders')
mydata <- make_data_rsiena(mynet1, myvar1, nodeSets=list(senders, receivers))
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, inPop)
myeff <- set_effect(myeff, altInDist2, covar1="myvar1", parameter=1)
alg_alg <- set_algorithm_saom(seed=1, n3=50, nsub=2)
ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
ans
tt <- test_time(ans)
summary(tt)
##test13
print('test13')
use<- 1:30
mynet1 <- as_dependent_rsiena(array(c(s502[,use], s503[,use]),
                         dim=c(50, length(use), 2)), type='bipartite',
                         nodeSet=c('Senders','receivers'))
receivers <- as_nodeset_rsiena(30,'receivers')
senders <- as_nodeset_rsiena(50,'Senders')
myvar1 <- as_covariate_rsiena(s50a[1:50,2], nodeSet='Senders')
mydata <- make_data_rsiena(mynet1, myvar1, nodeSets=list(senders, receivers))
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, altInDist2, covar1="myvar1", parameter=1)
myeff <- set_effect(myeff, egoX, covar1="myvar1")
alg_alg <- set_algorithm_saom(seed=1, n3=50, nsub=2)
(ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg))
##test14
print('test14')
net <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
dataset <- make_data_rsiena(net)
myeff <- make_specification(dataset)
myeff <- set_effect(myeff, inPop)
alg_alg <- set_algorithm_saom(maxlike=TRUE, cond=FALSE, seed=15, n3=20, nsub=1,
         diagonalize=0, mult=1)
(ans  <- siena(data=dataset, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg))
##test 15
print('test15')
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mynet2 <- as_dependent_rsiena(s50a,type='behavior')
mydata <- make_data_rsiena(mynet1, mynet2)
myeff <- make_specification(mydata)
(myeff <- set_effect(myeff, transTrip))
(myeff <- set_effect(myeff, list(egoX, simX), covar1="mynet2"))
(myeff <- set_effect(myeff, avSim, depvar="mynet2", covar1="mynet1"))
(myeff <- includeGMoMStatistics(myeff, simX_gmm, covar1="mynet2"))
alg_alg <- set_algorithm_saom(gmm=TRUE, seed=6, n3=100, nsub=1)
(ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg))
##test16
print('test16')
set.seed(123) # simulate behavior data according to dZ(t) = [-0.1 Z + 1] dt + 1 dW(t)
y1 <- rnorm(50, 0,3)
y2 <- exp(-0.1) * y1 + (1-exp(-0.1)) * 1/ -0.1 + rnorm(50, 0, (exp(-0.2)- 1) / -0.2 * 1^2)
friend <- as_dependent_rsiena(array(c(s501, s502), dim = c(50,50,2)))
behavior <- as_dependent_rsiena(matrix(c(y1,y2), 50,2), type = "continuous")
(mydata <- make_data_rsiena(friend, behavior))
(myeff <- make_specification(mydata, onePeriodSde = TRUE))
alg_alg <- set_algorithm_saom(seed=321, n3=20, nsub=1)
(ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg))
##test17
print('test17')
mynet <- sienaNet(array(c(s501, s502), dim=c(50, 50, 2)))
sm1 <- 1*(s50s[,2] >= 2)
sm2 <- 1*(s50s[,3] >= 2)
sm2 <- pmax(sm1,sm2)
sm2[c(33,28,29,44)] <- 1
mybeh <- as_dependent_rsiena(cbind(sm1,sm2), type="behavior")
(mydata <- make_data_rsiena(mynet, mybeh))
myeff <- make_specification(mydata)
(myeff <- set_effect(myeff, avExposure, type="rate", depvar="mybeh",
         covar1="mynet", parameter=2))
alg_alg <- set_algorithm_saom(seed=1234, n3=10, nsub=1, firstg=0.001)
(ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg))
##test18
print('test18')
mynet1 <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
cova <- as_covariate_rsiena(1:32)
cova2 <- as_covariate_rsiena(rep(0,32), warn=FALSE)
mydata <- make_data_rsiena(mynet1, cova)
mydata2 <- make_data_rsiena(mynet1, cova=cova2)
mygroup <- make_group_rsiena(list(mydata,mydata2))
myeff <- make_specification(mygroup)
myeff <- set_effect(myeff, simX, covar1="cova")
alg_alg <- set_algorithm_saom(seed=1293, n3=10, nsub=1)
(ans  <- siena(data=mygroup, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg))
##test19
print('test19')
mynet <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
alc <- as_covariate_rsiena(s50a[,1])
smoke <- as_covariate_rsiena(s50s[,1])
mydata <- make_data_rsiena(mynet, alc, smoke)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, gwespFF)
myeff <- set_effect(myeff, gwespFF, parameter=20)
myeff <- set_effect(myeff, outTrunc, parameter=2, include=FALSE)
myeff <- set_interaction(myeff, list(outTrunc, egoX, egoX), covar1=c("",
         "smoke", "alc"))
alg_alg <- set_algorithm_saom(seed=1943, n3=20, nsub=1)
(ans  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg))
##test20
print('test20')
mynet1 <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mynet2 <- as_dependent_rsiena(array(c(s503, s502), dim=c(50, 50, 2)))
mydata <- make_data_rsiena(mynet1, mynet2)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, crprod, depvar="mynet2", covar1="mynet1")
myeff <- set_effect(myeff, from, depvar="mynet1", covar1="mynet2")
(myeff <- includeGMoMStatistics(myeff, from_gmm, depvar='mynet1',
                                covar1='mynet2'))
alg_alg <- set_algorithm_saom(seed=1293, n3=50, nsub=2)
(ans  <- siena(data=mydata, effects=myeff[myeff$type != "gmm", ], batch=TRUE,
         silent=TRUE, control_algo=alg_alg))
alg_alg <- set_algorithm_saom(gmm=TRUE, seed=1293, n3=50, nsub=2)
(ans1  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         control_algo=alg_alg))
alg_alg <- set_algorithm_saom(gmm=TRUE, seed=1293, n3=50, nsub=2)
(ans2  <- siena(data=mydata, effects=myeff, batch=TRUE, silent=TRUE,
         prevAns=ans1, control_algo=alg_alg))

##test21
# Run simple test model ----
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim = c(50, 50, 3)))
mydata <- make_data_rsiena(mynet)
mymodel <- make_specification(mydata)
## TransitiveTriplets model
mymodel <- set_effect(mymodel, transTrip, depvar="mynet")
# Test returnChangeContributions when running siena directly ----

print('test21')
alg_out <- set_output_saom(returnChangeContributions=TRUE)
alg_alg <- set_algorithm_saom(cond=FALSE, seed=42, n3=60, nsub=1)
ans  <- siena(data=mydata, effects=mymodel, batch=TRUE, silent=TRUE,
         control_algo=alg_alg, control_out=alg_out)
length(ans$changeContributions) # 60 as expected
head(ans$changeContributions[[1]][[1]][[1]])
dim(ans$changeContributions[[1]][[1]][[1]][[1]]) # 3 x 50 as expected

# Test returnChangeContributions when setting nsub = 0 and prevAns = ans, 
#using batch mode
alg_out <- set_output_saom(returnChangeContributions=TRUE)
alg_alg <- set_algorithm_saom(cond=FALSE, seed=42, n3=60, nsub=1)
ans2  <- siena(data=mydata, effects=mymodel, batch=TRUE, prevAns=ans,
         control_algo=alg_alg, control_out=alg_out)
length(ans$changeContributions) # 60 chains as expected
head(ans$changeContributions[[1]][[1]][[1]])
##test22
# Test interpret_size_dynamics ----
## Unconditional Estimation ----
### Use changeContributions from ans ----
print('test22')
RIDynamics1 <- interpret_size_dynamics(data=mydata, 
     ans=ans, 
     useChangeContributions=TRUE, 
     intervalsPerPeriod=10)
RIDynamics1
### Unconditional Estimation with model rerun
mycontrols2 <- sienaAlgorithmCreate(nsub=2, n3=60, cond=FALSE, seed = 84)
mynet2 <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
mydata2 <- make_data_rsiena(mynet2)
mymodel2 <- make_specification(mydata2)
mymodel2 <- set_effect(mymodel2, list(density, recip, transTies))
alg_alg <- set_algorithm_saom(cond=FALSE, seed=84, n3=60, nsub=2)
ans2  <- siena(data=mydata2, effects=mymodel2, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
RIDynamics2 <-  interpret_size_dynamics(mydata2, ans=ans2)
RIDynamics2
### Don't use ans but previously estimated coefficients ----
RIDynamics3 <- interpret_size_dynamics(data=mydata2, theta=c(ans2$rate,ans2$theta),
             algorithm=mycontrols2, effects=mymodel2, intervalsPerPeriod=10)
RIDynamics3
## Conditional Estimation ----
alg_alg <- set_algorithm_saom(cond=TRUE, condvarno=1, seed=4242, n3=60, nsub=2)
ans3  <- siena(data=mydata2, effects=mymodel2, batch=TRUE, silent=TRUE,
         control_algo=alg_alg)
RIDynamics4 <- interpret_size_dynamics(mydata2, ans=ans3, effects = mymodel2)
RIDynamics4
##test23
## Test interpret_size_dynamics with behavior variable ----
mynet3 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <- as_dependent_rsiena(s50a, type="behavior")
mydata3 <- make_data_rsiena(mynet3, mybeh)
mymodel3 <- make_specification(mydata3)
mymodel3 <- set_effect(mymodel3, list(density, recip, transTies, transTrip))
print('test23')
alg_out <- set_output_saom(returnChangeContributions=TRUE)
alg_alg <- set_algorithm_saom(seed=8484, n3=60, nsub=2)
ans4  <- siena(data=mydata3, effects=mymodel3, batch=TRUE, silent=TRUE,
         control_algo=alg_alg, control_out=alg_out)
length(ans4$changeContributions) # 60 chains as expected
length(ans4$changeContributions[[1]][[1]]) # 2 periods as expected
beh_steps <- sapply(ans4$changeContributions[[1]][[1]][[2]], 
     function(ministep) attr(ministep, "networkName")) %in%  "mybeh"
any(beh_steps) # TRUE as expected
ministeps <- ans4$changeContributions[[1]][[1]][[1]]
getDepvarName <- function(ministep) attr(ministep, "networkName")
beh_steps <- Filter(function(ministep) getDepvarName(ministep)  == "mybeh", 
     ans4$changeContributions[[1]][[1]][[1]]) # 1st period, 1st group, 1st chain, behavior steps
RIDynamics5 <- interpret_size_dynamics(mydata3, ans=ans4, depvar="mybeh",
                               useChangeContributions=TRUE)
RIDynamics5
net_steps <- Filter(function(ministep) getDepvarName(ministep)  == "mynet3", 
     ans4$changeContributions[[1]][[1]][[1]]) # 1st period, 1st group, 1st chain, network steps
RIDynamics6 <- interpret_size_dynamics(mydata3, ans=ans4, depvar="mynet3")
RIDynamics6
##test24
mynet1 <- as_dependent_rsiena(array(c(tmp3,tmp4),dim=c(32,32,2)))
mydata <- make_data_rsiena(mynet1)
myeff<- make_specification(mydata)
myeff <- set_effect(myeff, list(recip, inPop, outAct))
thv <- matrix(NA, 10, 5)
thv[,1] <- 3 +0.1*(1:10)
thv[,2] <- c(-2, -2.4, -2.3, -2.5, -2 + (1:6)*0.15)
thv[,3] <- rep(2, 10)
thv[,4] <- 0.02 * (1:10)
thv[,5] <- 0.02 * (10:1) 
myalg <- set_algorithm_saom(nsub=2, n3=50, cond=FALSE, seed=5, simOnly=TRUE,
				thetaValue=thv)
print('test24')
(ans <- siena(mymodel, data=mydata, effects=myeff, control_algo=myalg, batch=TRUE, silent=TRUE))
## delete output files
if (file.exists('mydata_out.txt')){unlink('mydata_out.txt')}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: sienaDataCreate.r
# *
# * Description: This module contains the code to create
# * Siena data objects and group data objects.
# *****************************************************************************/
##@ make_data_rsiena DataCreate new name
make_data_rsiena <- function(..., nodeSets=NULL)
{
	sienaDataCreate(..., nodeSets=nodeSets)
}

##@ make_group DataCreate new name
make_group_rsiena <- function(objlist, singleOK = FALSE)
{
	sienaGroupCreate(objlist, singleOK=singleOK)
}

##@addAttributes DataCreate method for attaching attributes to objects
# This is used when creating the data object.
addAttributes <- function(x, name, ...) UseMethod("addAttributes")
# Note: this is used when creating the data object;
# the attributes are not part of the variables.

##@lowIntegers utility function DataCreate
lowIntegers <- function(vals, centr){
	is.wholenumber <- function(x, tol = 1e-6){abs(x - round(x)) < tol}
	all(is.wholenumber(vals), na.rm=TRUE) && (min(vals, na.rm=TRUE) >= 0) &&
					(max(vals, na.rm=TRUE) <= 20)  && (!centr)
}


##@addAttributes.coCovar DataCreate
addAttributes.coCovar <- function(x, name, ...)
{
	storage.mode(x) <- 'double'
	varmean <- mean(x, na.rm=TRUE)
	range2 <- range(x, na.rm=TRUE)
	attr(x, 'moreThan2') <- length(table(x)) > 2
	vartotal <- sum(x, na.rm=TRUE)
	nonMissingCount <- sum(!is.na(x))
	if (attr(x, "centered"))
	{
		x <- x - varmean
	}
	else
	{
		x <- x - 0.0
	}
	attr(x, 'mean') <- varmean
	rr <- rangeAndSimilarity(x, range2)
	if (rr$range[2] == rr$range[1] && !any(is.na(x)))
	{
		attr(x, 'poszvar') <- FALSE
	}
	else
	{
		attr(x, 'poszvar') <- TRUE
	}
	attr(x, 'range') <- ifelse((rr$range[2] - rr$range[1]==0), 1, rr$range[2] - rr$range[1])
	storage.mode(attr(x, 'range')) <- 'double'
	attr(x, 'range2') <- range2
	## attr(x, 'simTotal') <- rr$simTotal
	attr(x, 'simMean') <- rr$simMean
	## attr(x, 'simCnt') <- rr$simCnt
	attr(x, "name") <- name
	attr(x, "vartotal") <- vartotal
	attr(x, "nonMissingCount") <- nonMissingCount
	attr(x, "lowIntegers") <- lowIntegers(x, attr(x, "centered"))
	if ((!is.null(attr(x, "imputationValues"))) && (attr(x, "centered")))
	{
		attr(x, "imputationValues") <- attr(x, "imputationValues") - varmean
	}
	x
}

##@addAttributes.varCovar DataCreate
addAttributes.varCovar <- function(x, name, ...)
{
	tmpmat <- x
	varmean <- mean(x, na.rm=TRUE)
	vartotal <- sum(x, na.rm=TRUE)
	nonMissingCount <- sum(!is.na(x))
	attr(x, "rangep") <- apply(x, 2, range, na.rm=TRUE)
	attr(x, "meanp") <- colMeans(x, na.rm=TRUE)
	cr <- range(x, na.rm=TRUE)
	attr(x, 'range') <- cr[2] - cr[1]
	storage.mode(attr(x, 'range')) <- 'double'
	attr(x, 'mean') <- varmean
	if (attr(x, "centered"))
	{
		x <- x - varmean
	}
	else
	{
		x <- x - 0.0
	}
	rr <- rangeAndSimilarity(tmpmat, cr)
	if (rr$range[2] == rr$range[1] && !any(is.na(tmpmat)))
	{
		attr(x, 'poszvar') <- FALSE
	}
	else
	{
		attr(x, 'poszvar') <- TRUE
	}
	attr(x, 'simMean') <- rr$simMean
	attr(x, 'moreThan2') <- length(unique(x)) > 2
	attr(x, 'name') <- name
	attr(x, "vartotal") <- vartotal
	attr(x, "nonMissingCount") <- nonMissingCount
	attr(x, "lowIntegers") <- lowIntegers(x, attr(x, "centered"))
    if ((!is.null(attr(x, "imputationValues"))) && (attr(x, "centered")))
	{
		attr(x, "imputationValues") <- attr(x, "imputationValues") - varmean
	}
	x
}
##@addAttributes.coDyadCovar DataCreate
addAttributes.coDyadCovar <- function(x, name, bipartite, ...)
{
	sparse <- attr(x, "sparse")
	if (!bipartite) ## remove diagonal for calculation of mean
	{
		if (!sparse)
		{
			diag(x) <- NA
		}
		else
		{
			diag(x[[1]])  <-  NA
		}
	}
	if (sparse)
	{
		nonMissingCount <- sum(!is.na(x[[1]]))
		varmean <- sum(x[[1]], na.rm=TRUE) / nonMissingCount
		## sparse mean is incorrect
		rr <-  range(x[[1]], na.rm=TRUE)
	}
	else
	{
		varmean <- mean(x, na.rm=TRUE)
		rr <-  range(x, na.rm=TRUE)
		nonMissingCount <- sum(!is.na(x))
	}
	attr(x,'mean') <- ifelse(attr(x,'centered'), varmean, 0)
	attr(x,'range') <- rr[2] - rr[1]
	storage.mode(attr(x, 'range')) <- 'double'
	attr(x,'range2') <- rr
	attr(x, 'name') <- name
	attr(x, "nonMissingCount") <- nonMissingCount
	if (!bipartite) #zero the diagonal
	{
		if (sparse)
		{
			diag(x[[1]]) <- 0
		}
		else
		{
			diag(x) <- 0
		}
	}
	x
}
##@addAttributes.varDyadCovar DataCreate
addAttributes.varDyadCovar <- function(x, name, bipartite, ...)
{
	sparse <- attr(x, "sparse")
	vardims <- attr(x, "vardims")
	if (!bipartite) ## remove the diagonal before calculating the mean
	{
		for (obs in 1:vardims[3])
		{
			if (sparse)
			{
				diag(x[[obs]]) <- NA
			}
			else
			{
				diag(x[, , obs]) <- NA
			}
		}
	}
	if (sparse)
	{
		totalValue <- 0
		totalCount <- 0
		meanp <- rep(NA, vardims[3])
		nonMissingCounts <- rep(NA, vardims[3])
		for (obs in 1:vardims[3])
		{
			totalValue <- totalValue + sum(x[[obs]], na.rm=TRUE)
			nonMissingCounts[obs] <- sum(!is.na(x[[obs]]))
			totalCount <- totalCount +	nonMissingCounts[obs]
			meanp[obs] <- sum(x[[obs]], na.rm=TRUE) /
				nonMissingCounts[obs]
	  }
		varmean <- totalValue / totalCount
		rr <- range(sapply(x, range, na.rm=TRUE), na.rm=TRUE)
		attr(x, "meanp") <- meanp
	}
	else
	{
		varmean <- mean(x, na.rm=TRUE)
		rr <-  range(x, na.rm=TRUE)
		attr(x, "meanp") <- colMeans(x, dims=2, na.rm=TRUE)
		nonMissingCounts <- colSums(!is.na(x), dims=2)
	}
	attr(x, "mean") <- ifelse(attr(x, "centered"), varmean, 0)
	attr(x, "range") <- ifelse((rr[2] - rr[1]==0), 1, rr[2] - rr[1])
	storage.mode(attr(x, "range")) <- "double"
	attr(x, "name") <- name
	attr(x, "nonMissingCount") <- nonMissingCounts
	if (!bipartite) ## put diagonal to zero
	{
		for (obs in 1:vardims[3])
		{
			if (!sparse)
			{
				diag(x[, , obs]) <- 0
			}
			else
			{
				diag(x[[obs]]) <- 0
			}
		}
	}
	x
}
##@sienaDataCreate DataCreate
sienaDataCreate<- function(..., nodeSets=NULL, getDocumentation=FALSE)
{
	##@validNodeSet internal sienaDataCreate
	## suppressPackageStartupMessages(require(network))
	validNodeSet <- function(nodeSetName, n)
	{
		sub <- match(nodeSetName, nodeSetNames)
		if (is.na(sub))
		{
			warning(paste("node set",nodeSetName,"not found in",nodeSetNames,'\n'),
					immediate. = TRUE)
		}
		n == length(nodeSets[[sub]])
	}
	if (getDocumentation)
	{
		return(getInternals())
	}
	narg <- nargs()
	## find a set of names for the objects: either the names given in the
	## argument list or the names of the objects in the argument list
	dots <- as.list(substitute(list(...)))[-1] ##first entry is the word 'list'
	if (length(dots) == 0)
	{
		stop('need some objects')
	}
	if (length(dots) == 1)
	{
		ldots <- list(...)
		dotsIsList <- ((is.list(ldots[[1]])) & 
						(! inherits((ldots[[1]]), "sienaDependent")))
# If dotsIsList, it needs to be a list of variables
# The second condition is to rule out the case of a single dependent network
# given as a list of sparse matrices.
		if (dotsIsList) 
		{
			dots <- as.list(substitute(...))[-1]
			narg <- length(ldots)
		}
	}
	else
	{
		dotsIsList <- FALSE
	}
	nm <- names(dots)
	if (is.null(nm))
	{
		fixup <- seq(along=dots)
	}
	else
	{
		fixup <- nm == ''
	}
	dep <- sapply(dots[fixup], function(x) deparse(x)[1])
	if (is.null(nm))
	{
		nm <- dep
	}
	else if (length(dep) > 0)
	{
		nm[fixup] <- dep
	}
	if (!dotsIsList)
	{
		dots <- list(...)
	}
	else
	{
		dots <- (...)
	}
	names(dots) <- nm
	if (any(duplicated(nm)))
	{
		stop('names must be unique')
	}
	## process the inputs: check dimensions,
	## sort out missings and structural zeros and symmetric etc
	## check sizes match the corresponding nodeSets
	observations <- 0
	depvars <- vector('list',narg)
	cCovars <- vector('list',narg)
	vCovars <- vector('list',narg)
	dycCovars <- vector('list',narg)
	dyvCovars <- vector('list',narg)
	compositionChange <- vector('list',narg)
	v1 <- 0; v2 <- 0; v3 <- 0; v4 <- 0; v5 <- 0; v6 <- 0
	for (i in seq(along = dots))
		switch(class(dots[[i]])[1],
			   sienaDependent = {
				   if (attr(dots[[i]],'sparse'))
				   {
					   ##  require(Matrix)
					   netdims <- c(dim(dots[[i]][[1]]), length(dots[[i]]))
				   }
				   else
				   {
					   netdims <- dim(dots[[i]])
				   }
				   if (observations == 0)
				   {
					   observations <- netdims[3]
				   }
				   else if (observations != netdims[3])
				   {
					   stop('differing number of observations')
				   }
				   v1 <- v1 + 1
				   depvars[[v1]] <- dots[[i]]
				   names(depvars)[v1] <- nm[i]
			   },
			   coCovar = {
				   v2 <- v2 + 1
				   cCovars[[v2]] <- dots[[i]]
				   names(cCovars)[v2] <- nm[i]
			   },
			   varCovar = {
				   v3 <- v3 + 1
				   vCovars[[v3]] <- dots[[i]]
				   names(vCovars)[v3] <- nm[i]
			   },
			   coDyadCovar = {
				   v4 <- v4 + 1
				   dycCovars[[v4]] <- dots[[i]]
				   names(dycCovars)[v4] <- nm[i]
			   },
			   varDyadCovar = {
				   v5 <- v5 + 1
				   dyvCovars[[v5]] <- dots[[i]]
				   names(dyvCovars)[v5] <- nm[i]
			   },
			   compositionChange = {
				   v6 <- v6 + 1
				   compositionChange[[v6]] <- dots[[i]]
				   names(compositionChange)[v6] <- nm[i]
			   },
			   stop(paste("invalid object in sienaDataCreate: argument number",
					i, "is of class ", class(dots[[i]]),
					", which is not a valid ... argument."), call.=FALSE)
			   )
	if (v1 == 0)
	{
		stop("need a dependent variable")
	}
	depvars <- depvars[1:v1]
	if (is.null(nodeSets))
	{
		nodeSets <- list(sienaNodeSet(attr(depvars[[1]], "netdims")[1]))
	}
	else
	{
		if (!(inherits(nodeSets, "sienaNodeSet") || inherits(nodeSets[[1]], "sienaNodeSet")))
		{
			stop("nodeSets should be a sienaNodeSet object or a list of such objects")
		}
	}
	nodeSetNames <- sapply(nodeSets,function(x) attr(x,"nodeSetName"))
	names(nodeSets) <- nodeSetNames
	if (v2 == 0)
	{
		cCovars <- list()
	}
	else
	{
		cCovars <- cCovars[1:v2]
	}
	if (v3 == 0)
	{
		vCovars <- list()
	}
	else
	{
		vCovars <- vCovars[1:v3]
	}
	if (v4 == 0)
	{
		dycCovars <- list()
	}
	else
	{
		dycCovars <- dycCovars[1:v4]
	}
	if (v5 == 0)
	{
		dyvCovars <- list()
	}
	else
	{
		dyvCovars <- dyvCovars[1:v5]
	}
	if (v6 == 0)
	{
		compositionChange <- list()
	}
	else
	{
		compositionChange <- compositionChange[1:v6]
	}
	##now can check dimensions and find ranges
	for (i in seq(along = cCovars))
	{
		if (!validNodeSet(attr(cCovars[[i]], 'nodeSet'), length(cCovars[[i]])))
		{
			stop('constant covariate incorrect node set: ', names(cCovars)[i])
		}
		cCovars[[i]] <- addAttributes(cCovars[[i]], names(cCovars)[i])
	}
	for (i in seq(along=vCovars)) ## note that behaviour variables are not here!
	{
		if (observations < 3)
		{
			stop("Changing covariates are not possible with only two waves")
		}
		if (!validNodeSet(attr(vCovars[[i]], 'nodeSet'), nrow(vCovars[[i]])))
			stop('changing covariate incorrect size: ', names(vCovars)[i])
		if (ncol(vCovars[[i]]) < (observations - 1))
			stop('changing covariate not enough columns')
		if (ncol(vCovars[[i]]) != (observations - 1))
		{
			tmpatt <- attributes(vCovars[[i]])
			vCovars[[i]] <- vCovars[[i]][, 1:(observations - 1), drop=FALSE]
			attnames <- names(tmpatt)
			for (att in seq(along=attnames))
			{
				if (!attnames[att] %in% c('dim', 'dimnames'))
				{
					attr(vCovars[[i]], attnames[att]) <- tmpatt[[att]]
				}
			}
		}
		vCovars[[i]] <- addAttributes(vCovars[[i]], names(vCovars)[i])
	}
	for (i in seq(along=dycCovars))
	{
		nattr <- attr(dycCovars[[i]], 'nodeSet')
		bipartite <- attr(dycCovars[[i]], "type") == "bipartite"
		if (attr(dycCovars[[i]], "sparse"))
		{
			thisdycCovar <- dycCovars[[i]][[1]]
		}
		else
		{
			thisdycCovar <- dycCovars[[i]]
		}
		if (!validNodeSet(nattr[1], nrow(thisdycCovar)))
		{
			stop("dyadic covariate incorrect nbr rows ", names(dycCovars)[i])
		}
		if (!validNodeSet(nattr[2], ncol(thisdycCovar)))
		{
			 stop("dyadic covariate incorrect nbr columns ",
				  names(dycCovars)[i])
		 }
		dycCovars[[i]] <- addAttributes(dycCovars[[i]], names(dycCovars)[i],
										bipartite)
	}
	for (i in seq(along=dyvCovars))
	{
		if (observations < 3)
		{
			stop("Changing covariates are not possible with only two waves")
		}
		nattr <- attr(dyvCovars[[i]],'nodeSet')
		sparse <- attr(dyvCovars[[i]], "sparse")
		bipartite <- attr(dyvCovars[[i]], "type") == "bipartite"
		vardims <- attr(dyvCovars[[i]], "vardims")
		if (!validNodeSet(nattr[1], vardims[1]))
		{
			stop('dyadic changing covariate incorrect number of rows ',
				 names(dyvCovars)[i])
		}
		if (!validNodeSet(nattr[2], vardims[2]))
		{
			stop('dyadic changing covariate incorrect number of columns ',
				 names(dyvCovars)[i])
		}
		if (vardims[3] < (observations - 1))
		{
			stop('Dyadic changing covariate not enough observations')
		}
		if (vardims[3] != (observations - 1))
		{
			tmpatt <- attributes(dyvCovars[[i]])
			if (sparse)
			{
				dyvCovars[[i]] <- dyvCovars[[i]][1:(observations - 1)]
			}
			else
			{
				dyvCovars[[i]] <- dyvCovars[[i]][, 1:(observations - 1)]
			}
			attnames <- names(tmpatt)
			for (att in seq(along=attnames))
			{
				if (attnames[att] != "dim")
				{
					attr(dyvCovars[[i]], attnames[att]) <- tmpatt[[att]]
				}
			}
		}
		dyvCovars[[i]] <- addAttributes(dyvCovars[[i]], names(dyvCovars)[i],
										bipartite)
	}
	compnodesets <- sapply(compositionChange, function(x) attr(x, 'nodeSet'))
	if (any(duplicated(compnodesets)))
		stop('Only one composition change allowed for each nodeSet')
	for (i in seq(along = compositionChange))
	{
		thisNodeSet <- attr(compositionChange[[i]], 'nodeSet')
		nodeSetSize <- length(compositionChange[[i]])
		if (!validNodeSet(thisNodeSet, nodeSetSize))
			stop('composition change incorrect size: ',
				 names(compositionChange)[i])
		if (any(sapply(compositionChange[[i]], function(x)
					   any(x < 1.0 | x > observations))))
			stop("invalid times of composition change")
		if (!all(sapply(compositionChange[[i]], length) %% 2 == 0))
			stop(" Each composition change entry must have an ",
				 "even number of digits")
		## generate events and active flags
		activeStart <- matrix(FALSE, nrow=nodeSetSize, ncol=observations)
		action <- matrix(0, nrow=nodeSetSize, ncol=observations)
		events <- vector("list", nodeSetSize * 2 * observations)
		evSubs <- 1
		for (j in 1:nodeSetSize)
		{
			xsubs <- 1
			x <- compositionChange[[i]][[j]]
			repeat
			{
				##process one interval
				##start <- x[xsubs]
				##end <- x[xsubs+1]
				startIndex <- ceiling(x[xsubs])
				endIndex <- trunc(x[xsubs + 1])
			  #	 if (startIndex < observations && startIndex <= activeEndIndex)
			  #	 {
					activeStart[j, startIndex:endIndex] <- TRUE
			  #	 }
				if (x[xsubs] > 1.0)
				{
					period <- trunc(x[xsubs])
					evTime <- x[xsubs] - period
					events[[evSubs]] <- data.frame(event="join",
												   period=period,
												   actor = j, time=evTime)
					evSubs <- evSubs + 1
				}
				if (x[xsubs+1] < observations)
				{
					period <- trunc(x[xsubs+1])
					evTime <- x[xsubs+1] - period
					events[[evSubs]] <- data.frame(event="leave",
												   period=period,
												   actor = j, time=evTime)
					evSubs <- evSubs + 1
				}
				xsubs <- xsubs + 2
				if (xsubs > length(x))
				{
					break
				}
			}
		 #	 cat(j, 'active',activeStart[j,],  x, '\n')
			if (any(!activeStart[j, ]))
			{
				notActive <- which(!activeStart[j, ])
				for (jj in notActive)
				{
					precActive <- jj > 1 && sum(activeStart[j, 1:(jj - 1)]) > 0
					len <- length(activeStart[i,])
					succActive <- (jj < len) &&
					(sum(activeStart[j, (jj + 1):len]) > 0)

					if (!precActive)
					{
						action[j, jj] <- 1
					}
					else if (!succActive)
					{
						action[j, jj] <- 2
					}
					else
					{
						action[j, jj] <- 3
					}
				}
			}
		}
		if (evSubs > 1)
		{
			events <- do.call(rbind, events[1:(evSubs-1)])
		}
		else
		{
			events <- data.frame(event="join", period=1, actor = 1, time=0)[-1,]
		}
		events$event <- factor(events$event, levels=c('join','leave'))
		storage.mode(events$period) <- "integer"
		storage.mode(events$actor) <- "integer"
		attr(compositionChange[[i]], "events") <- events
		attr(compositionChange[[i]], "activeStart") <- activeStart
		attr(compositionChange[[i]], "action") <- action
	}
	## dependent variables. First we sort the list so discrete and then continuous
	## behavior are at the end
	types <- sapply(depvars, function(x)attr(x, "type"))
	depvars <- depvars[c(which(!(types %in% c('behavior', 'continuous'))),
						which(types == 'behavior'), which(types == "continuous"))]
	onemodes <- which(types == "oneMode")
	bipartites <- which(types == "bipartite")
	onemodes.mx <- max(c(onemodes, 0))
	bipartites.mn <- min(c(bipartites, v1+1))
	if (bipartites.mn < onemodes.mx)
	{
		stop("One-mode networks (if any) should be given before bipartite networks (if any).")
	}

	for (i in 1:v1) ## dependent variables
	{
		nattr <- attr(depvars[[i]], 'nodeSet')
		netdims <- attr(depvars[[i]], 'netdims')
		type <- attr(depvars[[i]], 'type')
		sparse <- attr(depvars[[i]], 'sparse')
		myarray <- depvars[[i]]
		if (!validNodeSet(nattr[1], netdims[1]))
			stop('1st net dimension wrong')
		if (type =='bipartite')
			if (!validNodeSet(nattr[2], netdims[2]))
				stop('2nd net dimension wrong')
		attr(depvars[[i]], 'uponly') <- rep(FALSE, observations - 1)
		attr(depvars[[i]], 'downonly') <- rep(FALSE, observations - 1)
		attr(depvars[[i]], 'distance') <- rep(FALSE, observations - 1)
		attr(depvars[[i]], 'vals') <- vector("list", observations)
		attr(depvars[[i]], 'nval') <- rep(NA, observations)
		attr(depvars[[i]], 'noMissing') <- rep(0, observations)
		attr(depvars[[i]], 'noMissingEither') <- rep(0, observations - 1)
		attr(depvars[[i]], 'nonMissingEither') <- rep(0, observations - 1)
		someOnly <- FALSE
		if (type == 'behavior' || type == 'continuous')
		{
			attr(depvars[[i]], 'noMissing') <- FALSE
			attr(depvars[[i]], 'symmetric') <- NA
			for (j in 1:(observations - 1))
			{
				myvector1 <- myarray[, , j]
				myvector2 <- myarray[, , j + 1]
				mydiff <- myvector1 - myvector2
				attr(depvars[[i]], "distance")[j] <- sum(abs(mydiff),
					na.rm=TRUE)
				attr(depvars[[i]], "vals")[[j]] <- table(myvector1,
					useNA="always")
				attr(depvars[[i]], "vals")[[j+1]] <- table(myvector2,
					useNA="always")

				attr(depvars[[i]], "nval")[j] <-  sum(!is.na(myvector1))
				attr(depvars[[i]], "nval")[j + 1] <-  sum(!is.na(myvector2))
				attr(depvars[[i]], 'noMissing')[j] <- sum(is.na(myvector1))
				attr(depvars[[i]], 'noMissing')[j+1] <- sum(is.na(myvector2))
				attr(depvars[[i]], 'noMissingEither')[j] <-
					sum(is.na(myvector2) | is.na(myvector1))
				attr(depvars[[i]], 'nonMissingEither')[j] <-
					sum(!(is.na(myvector2) | is.na(myvector1)))
				if (attr(depvars[[i]], 'allowOnly'))
				{
					if (all(mydiff >= 0, na.rm=TRUE))
					{
						attr(depvars[[i]], 'downonly')[j] <- TRUE
						someOnly <- TRUE
					}
					if (all(mydiff <= 0, na.rm=TRUE))
					{
						attr(depvars[[i]], 'uponly')[j] <- TRUE
						someOnly <- TRUE
					}
				}
			}
			rr <- range(depvars[[i]], na.rm=TRUE)
			if (rr[2] == rr[1] && !any(is.na(depvars[[i]])))
				attr(depvars[[i]], 'poszvar') <- FALSE
			else
				attr(depvars[[i]], 'poszvar') <- TRUE
			crange <- rr[2] - rr[1]
			attr(depvars[[i]],'range') <- crange
			attr(depvars[[i]], "range2") <- rr
			attr(depvars[[i]],'moreThan2') <- length(unique(depvars[[i]])) > 2
			if (type == 'behavior')
			{
			modes <- apply(depvars[[i]][, 1, ], 2, function(z)
					   {
						   taba <- table(round(z))
						   as.numeric(names(which.max(taba)))
					   }
						   )
			}
			else # type == 'continuous', input the medians here; these will be
				 #						 used for missing data imputation
			{
				modes <- apply(depvars[[i]][, 1, ], 2, median, na.rm=TRUE)
			}
			attr(depvars[[i]],'modes') <- modes
			attr(depvars[[i]], 'missing') <- any(is.na(depvars[[i]]))
			tmpmat <- depvars[[i]][, 1, ]
			rr <- rangeAndSimilarity(tmpmat[, - ncol(tmpmat)], rr)
			##	attr(depvars[[i]], 'simTotal') <- rr$simTotal
			##	attr(depvars[[i]], 'simCnt') <- rr$simCnt
			attr(depvars[[i]], 'simMean') <- rr$simMean
            attr(depvars[[i]], 'variance') <- rr$variance
			attr(depvars[[i]], 'structural') <- FALSE
			attr(depvars[[i]], 'balmean') <- NA
			attr(depvars[[i]], 'structmean') <- NA
	   }
		else
		{
			for (j in 1:(observations - 1))
			{
				if (sparse)
				{
					mymat1 <- myarray[[j]]
					mymat2 <- myarray[[j + 1]]
					## remove diagonals if not bipartite
					if (attr(depvars[[i]], "type") != "bipartite")
					{
						diag(mymat1) <- NA
						diag(mymat2) <- NA
					}
					attr(depvars[[i]], 'noMissingEither')[j] <-
						sum(is.na(mymat1) | is.na(mymat2))
					attr(depvars[[i]], 'nonMissingEither')[j] <-
						sum(!(is.na(mymat1) | is.na(mymat2)))
					## remove diagonals if not bipartite
					if (attr(depvars[[i]], "type") != "bipartite")
					{
						attr(depvars[[i]], 'noMissingEither')[j] <-
							attr(depvars[[i]], 'noMissingEither')[j] -
								nrow(mymat1)
					}
					##remove structural values
					x1 <- mymat1@x
					x2 <- mymat2@x
					x1[x1 %in% c(10, 11)] <- NA
					x2[x2 %in% c(10, 11)] <- NA
					mymat1@x <- x1
					mymat2@x <- x2
					mydiff <- mymat2 - mymat1
					attr(depvars[[i]], 'distance')[j] <- sum(mydiff != 0,
															 na.rm = TRUE)
					if (attr(depvars[[i]], 'allowOnly'))
						{
							if (all(mydiff@x >= 0, na.rm=TRUE))
							{
								attr(depvars[[i]], 'uponly')[j] <- TRUE
								someOnly <- TRUE
							}
							if (all(mydiff@x <= 0, na.rm=TRUE))
							{
								attr(depvars[[i]], 'downonly')[j] <- TRUE
								someOnly <- TRUE
							}
						}
				}
				else
				{
					mymat1 <- myarray[, , j]
					mymat2 <- myarray[, , j + 1]
					## remove diagonals if not bipartite
					if (attr(depvars[[i]], "type") != "bipartite")
					{
						diag(mymat1) <- NA
						diag(mymat2) <- NA
					}
					attr(depvars[[i]], 'noMissingEither')[j] <-
						sum(is.na(mymat1) | is.na(mymat2))
					attr(depvars[[i]], 'nonMissingEither')[j] <-
						sum(!(is.na(mymat1) | is.na(mymat2)))
					## remove diagonals if not bipartite as not really missing
					if (attr(depvars[[i]], "type") != "bipartite")
					{
						attr(depvars[[i]], 'noMissingEither')[j] <-
							attr(depvars[[i]], 'noMissingEither')[j] -
								nrow(mymat1)
					}
					##remove structural values
					mymat1[mymat1 %in% c(10,11)] <- NA
					mymat2[mymat2 %in% c(10,11)] <- NA
					mydiff <- mymat2 - mymat1
					attr(depvars[[i]], 'distance')[j] <- sum(mydiff != 0,
															 na.rm = TRUE)
					if (attr(depvars[[i]], 'allowOnly'))
						{
							if (all(mydiff >= 0, na.rm=TRUE))
							{
								attr(depvars[[i]], 'uponly')[j] <- TRUE
								someOnly <- TRUE
							}
							if (all(mydiff <= 0, na.rm=TRUE))
							{
								attr(depvars[[i]], 'downonly')[j] <- TRUE
								someOnly <- TRUE
							}
						}
				}
			}
			if (type == 'oneMode')
			{
				attr(depvars[[i]], 'balmean') <- calcBalmean(depvars[[i]])
				attr(depvars[[i]], 'structmean') <- calcStructmean(depvars[[i]])
				attr(depvars[[i]], 'simMean') <- NA
                attr(depvars[[i]], 'variance') <- NA
				attr(depvars[[i]], 'symmetric') <- TRUE
				attr(depvars[[i]], 'missing') <- FALSE
				attr(depvars[[i]], 'structural') <- FALSE
				maxObsOutDegree <- rep(NA, observations)
				for (j in 1:observations)
				{
					if (sparse)
					{
						mymat <- myarray[[j]]
						diag(mymat) <- NA
					}
					else
					{
						mymat <- myarray[, , j]
						diag(mymat) <- NA
					}
					if (suppressMessages(!isSymmetric(mymat)))
					{
						attr(depvars[[i]], 'symmetric') <- FALSE
					}
					if (sparse)
					{
						if (any(is.na(mymat@x[mymat@i != mymat@j])))
						{
							attr(depvars[[i]], 'missing') <- TRUE
						}
					}
					else if (any(is.na(mymat[row(mymat) != col(mymat)])))
					{
						attr(depvars[[i]], 'missing') <- TRUE
					}
					if (any(!is.na(mymat) & (mymat == 10 | mymat == 11)))
					{
						attr(depvars[[i]], "structural") <- TRUE
					}
					if (sparse)
					{
						nonZeros <- table(mymat@x, useNA="always")
						Zeros <- nrow(mymat) * nrow(mymat - 1) - sum(nonZeros)
						attr(depvars[[i]], "vals")[[j]] <-
							c(table(rep(0, Zeros)), nonZeros)
					}
					else
					{
						attr(depvars[[i]], "vals")[[j]] <-
							table(mymat, useNA="always")
					}
					attr(depvars[[i]], "nval")[j] <-
						sum(!is.na(mymat[row(mymat) != col(mymat)]))
					# Use %% (modulo) to handle structural values:
					maxObsOutDegree[j] <- max(rowSums(mymat %% 10, na.rm=TRUE))
				}
				### need to exclude the structurals here
				if (sparse)
				{
				   vals <- lapply(depvars[[i]], function(x)
								   c(x@x[!(is.na(x@x) |
										   x@x %in% c(10, 11))] , 0))
					attr(depvars[[i]], "range2") <-
						do.call(range, vals)
			   }
				else
				{
					tmp <- depvars[[i]]
					attr(depvars[[i]], "range2") <-
						range(tmp[!(is.na(tmp) | tmp %in% c(10, 11))])
				}
				## average degree etc.
				atts <- attributes(depvars[[i]])
				ones <- sapply(atts$vals, function(x)
						   {
							   if (is.na(x["11"]))
							   {
									if (is.na(x["1"]))
									{
										0
									}
									else
									x["1"]
							   }
							   else
							   {
								   x["1"] + x["11"]
							   }
						   }
							   )
				density <- ones / atts$nval
				degree <- (atts$netdims[1] - 1) * ones / atts$nval
				missings <- 1 - atts$nval/ atts$netdims[1] /
					(atts$netdims[1] - 1)
				noMissing <- atts$netdims[1] * (atts$netdims[1] - 1) -
					atts$nval
				attr(depvars[[i]], "ones") <- ones
				attr(depvars[[i]], "density") <- density
				attr(depvars[[i]], "degree") <- degree
				attr(depvars[[i]], "averageOutDegree") <- mean(degree)
				attr(depvars[[i]], "averageInDegree") <- mean(degree)
				attr(depvars[[i]], "maxObsOutDegree") <- maxObsOutDegree
				attr(depvars[[i]], "missings") <- missings
				attr(depvars[[i]], "noMissing") <- noMissing
		   }
			else #type=='bipartite' not sure what we need here,
				## but include diagonal
			{
				attr(depvars[[i]], 'balmean') <- NA
				attr(depvars[[i]], 'structmean') <- NA
				attr(depvars[[i]], 'simMean') <- NA
                attr(depvars[[i]], 'variance') <- NA
				attr(depvars[[i]], 'symmetric') <- FALSE
				attr(depvars[[i]], 'missing') <- FALSE
				attr(depvars[[i]], 'structural') <- FALSE
				maxObsOutDegree <- rep(NA, observations)
				for (j in 1:observations)
				{
					if (sparse)
					{
						mymat <- myarray[[j]]
					}
					else
					{
						mymat <- myarray[, , j]
					}
					if (any(is.na(mymat)))
					{
						attr(depvars[[i]], 'missing') <- TRUE
					}
					if (any(!is.na(mymat) & (mymat == 10 | mymat == 11)))
					{
						attr(depvars[[i]], "structural") <- TRUE
					}
					if (sparse)
					{
						nonZeros <- table(mymat@x, useNA="always")
						Zeros <- nrow(mymat) * ncol(mymat) - sum(nonZeros)
						attr(depvars[[i]], "vals")[[j]] <-
							c(table(rep(0, Zeros)), nonZeros)
					}
					else
					{
						attr(depvars[[i]], "vals")[[j]] <- table(mymat,
																 useNA="always")
					}
					attr(depvars[[i]], "nval")[j] <- sum(!is.na(mymat))
					# Use %% (modulo) to handle structural values:
					maxObsOutDegree[j] <- max(rowSums(mymat %% 10, na.rm=TRUE))
				}
				### need to exclude the structurals here
				if (sparse)
				{
				   vals <- lapply(depvars[[i]], function(x)
								   c(x@x[!(is.na(x@x) |
										   x@x %in% c(10, 11))] , 0))
					attr(depvars[[i]], "range") <-
						do.call(range, vals)
			   }
				else
				{
					tmp <- depvars[[i]]
					attr(depvars[[i]], "range") <-
						range(tmp[!(is.na(tmp) | tmp %in% c(10, 11))])
				}
				 ## average degree
				atts <- attributes(depvars[[i]])
				ones <- sapply(atts$vals, function(x)
						   {
							   if (is.na(x["11"]))
							   {
								   x["1"]
							   }
							   else
							   {
								   x["1"] + x["11"]
							   }
						   }
							   )
				density <- ones / atts$nval
				degree <- (atts$netdims[2]) * ones / atts$nval
				missings <- 1 - atts$nval/ atts$netdims[1] /
					atts$netdims[2]
				noMissing <- atts$netdims[1] * atts$netdims[2] - atts$nval
				attr(depvars[[i]], "ones") <- ones
				attr(depvars[[i]], "density") <- density
				attr(depvars[[i]], "degree") <- degree
				attr(depvars[[i]], "averageOutDegree") <- mean(degree)
				attr(depvars[[i]], "maxObsOutDegree") <- maxObsOutDegree
				attr(depvars[[i]], "missings") <- missings
				attr(depvars[[i]], "noMissing") <- noMissing
		   }
		}
			
		if (someOnly)
		{
		message("For dependent variable ", names(depvars)[i], ", in some periods,")
		message("there are only increases, or only decreases.")
		message("This will be respected in the simulations. ")
		message("If this is not desired,\n use allowOnly=FALSE when creating the dependent variable.")
		}
		attr(depvars[[i]], 'name') <- names(depvars)[i]
	}
	## create the object
	z <- NULL
	z$nodeSets <- nodeSets
	z$observations <- observations
	z$depvars <- depvars
	z$cCovars <- cCovars
	z$vCovars <- vCovars
	z$dycCovars <- dycCovars
	z$dyvCovars <- dyvCovars
	z$compositionChange <- compositionChange
	z <- checkConstraints(z)
	z <- covarDist2(z)
	attr(z, "version") <- packageDescription(pkgname, fields = "Version")
	class(z) <- "sienadata"
	z
}
##@checkConstraints DataCreate
checkConstraints <- function(z)
{
	types <- sapply(z$depvars, function(x)attr(x, "type"))
	symmetrics <- sapply(z$depvars, function(x)attr(x, "symmetric"))
	sparse <- sapply(z$depvars, function(x)attr(x, "sparse"))
	nodeSets <- lapply(z$depvars, function(x)attr(x, "nodeSet"))
	nNets <- length(z$depvars)

	pairsOfNets <- as.matrix(expand.grid(names(z$depvars), names(z$depvars)))
	pairsNames <- paste(pairsOfNets[, 1], pairsOfNets[, 2], sep=",")

	higher <- namedVector(FALSE, pairsNames )
	atLeastOne <- namedVector(FALSE, pairsNames )
	disjoint <- namedVector(FALSE, pairsNames )

	## identify any nets which may relate. These are those that
	## share both node sets and type.
	relates <- data.frame(name=names(z$depvars), type=types,
						  nodeSets=sapply(nodeSets, paste, collapse=","),
						  tn=paste(types, sapply(nodeSets, paste,
						  collapse=",")) , stringsAsFactors=FALSE)

	## just check we have only one row per network
	if (nrow(relates) != nNets)
	{
		stop("Error in checkConstraints")
	}
	## find the ones that do occur more than once
	use <- relates$tn %in% relates$tn[duplicated(relates$tn)]

	## create a working list to be compared, with names for ease of access
	nets <- namedVector(NA, names(z$depvars), listType=TRUE)

	for (net in names(z$depvars)[use])
	{
		if (types[[net]] != "behavior")
		{
			nets[[net]] <- z$depvars[[net]]
		}
	}

	for (i in 1:nrow(pairsOfNets))
	{
		if (pairsOfNets[i, 1] != pairsOfNets[i, 2])
		{
			net1 <- pairsOfNets[i, 1]
			net2 <- pairsOfNets[i, 2]

			type1 <- types[net1]
			type2 <- types[net2]
			nodes1 <- relates[net1, "nodeSets"]
			nodes2 <- relates[net2, "nodeSets"]

			symmetric1 <- symmetrics[net1]
			symmetric2 <- symmetrics[net2]

			if (type1 == type2 && type1 != "behavior" && nodes1 == nodes2
				&& symmetric1 == symmetric2 && type1 != "continuous"
				&& type2 != "continuous")
			{
				higher[i] <- TRUE
				disjoint[i] <- TRUE
				atLeastOne[i] <- TRUE
				depvar1 <- nets[[pairsOfNets[i, 1]]]
				depvar2 <- nets[[pairsOfNets[i, 2]]]
				for (obs in 1:z$observations)
				{
					if (sparse[net1])
					{
						var1 <- depvar1[[obs]]
					}
					else
					{
						var1 <- depvar1[,, obs]
					}
					if (sparse[net2])
					{
						var2 <- depvar2[[obs]]
					}
					else
					{
						var2 <- depvar2[,, obs]
					}
#					var1[var1 %in% c(10, 11)] <- var1[var1 %in% c(10, 11)] - 10
#					var2[var2 %in% c(10, 11)] <- var2[var2 %in% c(10, 11)] - 10
					var1[var1==10] <- 0
					var1[var1==11] <- 1
					var2[var2==10] <- 0
					var2[var2==11] <- 1
					## higher
					if (any(var1 - var2 < 0, na.rm=TRUE))
					{
						higher[i] <- FALSE
					}
					## disjoint
					if (sum(var1 * var2, na.rm=TRUE) > 0)
					{
						disjoint[i] <- FALSE
					}
					##atleastone
					if (any(var1 + var2 == 0, na.rm=TRUE))
					{
						atLeastOne[i] <- FALSE
					}
				}

			}
		}
	}
	attr(z, "higher") <- higher
	attr(z, "disjoint") <- disjoint
	attr(z, "atLeastOne") <- atLeastOne

	## report on constraints; adapted from print01Report
	if (any(higher))
	{
		higherSplit <- strsplit(names(higher)[higher], ",")
		report <- sapply(higherSplit, function(x)
		   {
			   paste(c("Network ", x[1], " is higher than network ", x[2],
						".\n"), sep="")
		  })
		message(report)
		cat("This will be respected in the simulations. ")
		cat("If this is not desired, change attribute 'higher'\n")
		cat("by function make_constraint.\n")
	}
	if (any(disjoint))
	{
		disjointSplit <- strsplit(names(disjoint)[disjoint],',')
		report <- sapply(disjointSplit, function(x)
		   {
			   paste(c("Network ", x[1], " is disjoint from network ",
						x[2], ".\n"), sep="")
		  })
		message(report)
		cat("This will be respected in the simulations.\n")
		cat("If this is not desired, change attribute 'disjoint'\n")
		cat("by function make_constraint.\n")

	}
	if (any(atLeastOne))
	{
		atLeastOneSplit <- strsplit(names(atLeastOne)[atLeastOne],',')
		report <- sapply(atLeastOneSplit, function(x)
		   {
			   paste(c("A link in at least one of networks ",
						x[1], " and", x[2],
					   " always exists.\n"), sep="")
		  })
		message(report)
		cat("This will be respected in the simulations.")
		cat("If this is not desired, change attribute 'atLeastOne'\n")
		cat("by function sienaDataConstraint.\n")
	}
	z
}

##@rangeAndSimilarity DataCreate
rangeAndSimilarity <- function(vals, rvals=NULL)
{
	zeroOrNA <- function(x){ifelse(is.na(x), TRUE, (x==0))}
	if (is.null(rvals))
	{
		rvals <- range(vals, na.rm=TRUE)
	}
	if (zeroOrNA(var(as.vector(vals), na.rm=TRUE)))
	{
		simTotal <- 0
		simCnt <- sum(!is.na(vals))^2
		simMean <- 0
	}
	else
	{
		vals <- as.matrix(vals)
		rvals1 <- rvals[2] - rvals[1]
		tmp <- apply(vals, 2, function(v)
			 {
				sapply(1: length(v), function(x, y, r)
					{
						z <- y
						z[x] <- NA
						##browser()
						tmp1 <- 1 - abs(y[x] - z) / r
						list(sum(tmp1, na.rm=TRUE), sum(!is.na(tmp1)))
					},
						y=v, r=rvals1)
			 }
					)
		tmp <- unlist(tmp)
		raw <- tmp[seq(1, length(tmp), by=2)]
		cnts <- tmp[seq(2, length(tmp), by=2)]
		simTotal <- sum(raw)
		simCnt <- sum(cnts)
		simMean <- ifelse(simCnt==0, 0, simTotal/simCnt)
    }
    
    # and variance
    sum <- sum(c(vals), na.rm = TRUE)
    sumSq <- sum(c(vals)^2, na.rm = TRUE)
    nonmis <- sum(!is.na(c(vals)))
    variance <- ifelse(nonmis==0, 0, (sumSq/nonmis) - (sum/nonmis)^2)

	list(simTotal=simTotal, simMean=simMean, range=rvals, simCnt=simCnt,
        sum=sum, sumSq=sumSq, variance=variance, varCnt=nonmis)
}
##@groupRangeAndSimilarityAndMean DataCreate
## calculates attributes at group level and re-centers actor covariates
groupRangeAndSimilarityAndMean <- function(group)
{
	atts <- attributes(group)
	##behavs <- atts$types == "behavior"
	netnames <- atts$netnames
	bRange <- namedVector(NA, netnames)
	behRange <- matrix(NA, ncol=length(netnames), nrow=2)
	colnames(behRange) <- netnames
	bSim <- namedVector(NA, netnames)
    bVar <- namedVector(NA, netnames)
	bPoszvar <- namedVector(NA, netnames)
	bMoreThan2 <- namedVector(NA, netnames)
	bAnyMissing <- namedVector(FALSE, netnames)
	for (net in which(atts$types %in% c("behavior", "continuous")))
	{
		simTotal <- 0
		simCnt <- 0
        sumTotal <- 0
        sumSqTotal <- 0
        varCnt <- 0
		anyMissing <- FALSE
		bPoszvar[net] <- TRUE
		thisrange <- matrix(NA, ncol=length(group), nrow=2)
		for (i in 1:length(group))
		{
			j <- match(netnames[net], names(group[[i]]$depvars))
			if (is.na(j))
				stop("network names not consistent")
			depvar <- group[[i]]$depvars[[j]][, 1, ]
			## this should be a matrix
			thisrange[, i] <- range(depvar, na.rm=TRUE)
			if (any(is.na(depvar)))
			{
				anyMissing <- TRUE
			}
		}
		behRange[, net] <- round(range(thisrange, na.rm=TRUE))
		bRange[net] <- behRange[, net][2] - behRange[, net][1]
		if (behRange[, net][2] == behRange[, net][1] && !anyMissing)
		{
			bPoszvar[net] <- FALSE
		}
		values <- NULL
		for (i in 1:length(group))
		{
			j <- match(netnames[net], names(group[[i]]$depvars))
			depvar <- group[[i]]$depvars[[j]][, 1, ]
			## this should be a matrix

			tmp <- rangeAndSimilarity(depvar[, -ncol(depvar)],
									  behRange[, net])
			simTotal <- simTotal + tmp$simTotal
			simCnt <- simCnt + tmp$simCnt
			values <- c(values, unique(depvar))
            
            sumTotal <- sumTotal + tmp$sum
            sumSqTotal <- sumSqTotal + tmp$sumSq
            varCnt <- varCnt + tmp$varCnt
		}
		simMean <- ifelse(simCnt==0, 0, simTotal/simCnt)
		bSim[net] <- simMean
		bMoreThan2[net] <- length(unique(values)) > 2
		if (anyMissing)
			bAnyMissing[net] <- TRUE
        variance <- ifelse(varCnt==0, 0, sumSqTotal/varCnt - (sumTotal/varCnt)^2)
        bVar[net] <- variance
	}
	## constant ones will not exist unless there is only one data object
	cCovarRange <- namedVector(NA, atts$cCovars)
	cCovarSim <- namedVector(NA, atts$cCovars)
	cCovarPoszvar <- namedVector(TRUE, atts$cCovars)
	cCovarMoreThan2 <- namedVector(FALSE, atts$cCovars)
	cCovarMean <- namedVector(NA, atts$cCovars)
	cCovarRange2 <- matrix(NA, ncol=length(atts$cCovars), nrow=2)
	colnames(cCovarRange2) <- atts$cCovars
	for (covar in seq(along=atts$cCovars))
	{
		if (length(group) > 1)
			stop("group create constant covariate error")
		simTotal <- 0
		simCnt <- 0
		anyMissing <- FALSE
		## first find the range
		thisrange <- matrix(NA, ncol=length(group),nrow=2)
		for (i in 1:length(group))
		{
			j <- match(atts$cCovars[covar], names(group[[i]]$cCovars))
			if (is.na(j))
			{
				stop("inconsistent actor covariate names")
			}
			thisrange[, i] <- range(group[[i]]$cCovars[[j]],
								   na.rm=TRUE)
			if (any(is.na(group[[i]]$cCovars[[j]])))
			{
				anyMissing <- TRUE
			}
		}
		rr <- range(thisrange, na.rm=TRUE)
		cCovarRange[covar] <- rr[2] - rr[1]
		if (rr[2] == rr[1] && !anyMissing)
				cCovarPoszvar[covar] <- FALSE
		##then calculate similarity
		for (i in 1:length(group))
		{
			j <- match(atts$cCovars[covar], names(group[[i]]$cCovars))
			tmp <- rangeAndSimilarity(group[[i]]$cCovars[[covar]],
									  rr)
			simTotal <- simTotal + tmp$simTotal
			simCnt <- simCnt + tmp$simCnt
		}
		simMean <- simTotal/simCnt
		cCovarSim[covar] <- simMean
		cCovarMoreThan2[covar] <- attr(group[[i]]$cCovars[[covar]], "moreThan2")
		cCovarMean[covar] <- attr(group[[i]]$cCovars[[covar]], "mean")
		cCovarRange2[, covar] <- attr(group[[i]]$cCovars[[covar]], "range2")
   }

	vCovarRange <- namedVector(NA, atts$vCovars)
	vCovarSim <- namedVector(NA, atts$vCovars)
	vCovarPoszvar <- namedVector(TRUE, atts$vCovars)
	vCovarMoreThan2 <- namedVector(FALSE, atts$vCovars)
	vCovarMean <- namedVector(NA, atts$vCovars)
	for (covar in seq(along=atts$vCovars))
	{
		vartotal <- 0
		nonMissingCount <- 0
		## need to re-centre these values
		for (i in 1:length(group))
		{
			j <- match(atts$vCovars[covar], names(group[[i]]$vCovars))
			j1 <- match(atts$vCovars[covar], names(group[[1]]$vCovars))
			if (is.na(j))
			{
				stop("inconsistent actor covariate names")
			}
			if (attr(group[[i]]$vCovars[[j]],"centered") != attr(group[[1]]$vCovars[[j]],"centered"))
			{
				stop(paste("Inconsistent centering for covariate", names(group[[i]]$vCovars)[j]))
			}
			vartotal <- vartotal + attr(group[[i]]$vCovars[[j]], "vartotal")
			nonMissingCount <- nonMissingCount +
					attr(group[[i]]$vCovars[[j]], "nonMissingCount")
			if (attr(group[[i]]$vCovars[[j]],"centered"))
			{
				group[[i]]$vCovars[[j]] <- group[[i]]$vCovars[[j]] +
					attr(group[[i]]$vCovars[[j]], "vartotal") /
						attr(group[[i]]$vCovars[[j]], "nonMissingCount")
			}
		}
		varmean <- vartotal / nonMissingCount
		j <- match(atts$vCovars[covar], names(group[[1]]$vCovars))
		if (attr(group[[1]]$vCovars[[j]],"centered"))
		{
			for (i in 1:length(group))
			{
				j <- match(atts$vCovars[covar], names(group[[i]]$vCovars))
				if (is.na(j))
				{
					stop("inconsistent actor covariate names")
				}
				group[[i]]$vCovars[[j]] <- group[[i]]$vCovars[[j]] -
					varmean
			}
		}
		simTotal <- 0
		simCnt <- 0
		anyMissing <- FALSE
		## first find the range
		thisrange <- matrix(NA, ncol=length(group), nrow=2)
		values <- NULL
		for (i in 1:length(group))
		{
			j <- match(atts$vCovars[covar], names(group[[i]]$vCovars))
			if (is.na(j))
			{
				stop("inconsistent actor covariate names")
			}
			thisrange[, i] <- range(group[[i]]$vCovars[[j]],
									na.rm=TRUE)
			if (any(is.na(group[[i]]$vCovars[[j]])))
			{
				anyMissing <- TRUE
			}
			values <- c(values, unique(group[[i]]$vCovars[[j]]))
		}
		rr <- range(thisrange, na.rm=TRUE)
		if (rr[2] == rr[1] && !anyMissing)
		{
			vCovarPoszvar[covar] <-	 FALSE
		}
		vCovarRange[covar] <- rr[2] - rr[1]
		##then calculate similarity. Note ignore final observation
		for (i in 1:length(group))
		{
			j <- match(atts$vCovars[covar], names(group[[i]]$vCovars))
			tmpmat <- group[[i]]$vCovars[[covar]]
			tmp <- rangeAndSimilarity(tmpmat, rr)
			simTotal <- simTotal + tmp$simTotal
			simCnt <- simCnt + tmp$simCnt
		}
		simMean <- simTotal/simCnt
		vCovarSim[covar] <- simMean
		## storage.mode(attr(vCovars[[i]], 'range')) <- 'double'
		vCovarMean[covar] <- varmean
		vCovarMoreThan2[covar] <- length(unique(values)) > 2
	}
	dycCovarMean <- namedVector(NA, atts$dycCovars)
	dycCovarRange <- namedVector(NA, atts$dycCovars)
	dycCovarRange2 <- matrix(NA, 2, length(atts$dycCovars))
	colnames(dycCovarRange2) <- atts$dycCovars
	for (covar in seq(along=atts$dycCovars))
	{
		if (length(group) > 1)
			stop("error in dyadic constant covariate, group create")
		j <- match(atts$dycCovars[covar], names(group[[1]]$dycCovars))
		if (is.na(j))
		{
			stop("inconsistent dyadic covariate names")
		}
		dycCovarMean[covar] <- attr(group[[1]]$dycCovars[[j]], "mean")
		dycCovarRange[covar] <- attr(group[[1]]$dycCovars[[j]], "range")
		dycCovarRange2[, covar] <- attr(group[[1]]$dycCovars[[j]], "range2")
	}
	dyvCovarMean <- namedVector(NA, atts$dyvCovars)
	dyvCovarRange <- namedVector(NA, atts$dyvCovars)
	dyvCovarRange2 <- matrix(NA, 2, length(atts$dyvCovars))
	colnames(dyvCovarRange2) <- atts$dyvCovars
	for (covar in seq(along=atts$dyvCovars))
	{
		vartotal <- 0
		nonMissingCount <- 0
		thisrange <- matrix(NA, ncol=length(group), nrow=2)
		for (i in 1:length(group))
		{
			j <- match(atts$dyvCovars[covar], names(group[[i]]$dyvCovars))
			if (is.na(j))
			{
				stop("inconsistent dyadic covariate names")
			}
			sparse <- attr(group[[i]]$dyvCovars[[j]], "sparse")
			vardims <- attr(group[[i]]$dyvCovars[[j]], "vardims")
			if (sparse)
			{
				for (obs in 1:vardims[3])
				{
					vartotal <- vartotal + sum(group[[i]]$dyvCovars[[j]][[obs]],
											   na.rm=TRUE)
					nonMissingCount <- nonMissingCount +
						sum(!is.na(group[[i]]$dyvCovars[[j]][[obs]]))
				}
				thisrange[, i] <- range(sapply(group[[i]]$dyvCovars[[j]], range,
											   na.rm=TRUE),
										na.rm=TRUE)
			}
			else
			{
				vartotal <- vartotal + sum(group[[i]]$dyvCovars[[j]], na.rm=TRUE)
				nonMissingCount <- nonMissingCount +
					sum(!is.na(group[[i]]$dyvCovars[[j]]))
				thisrange[, i] <- range(group[[i]]$dyvCovars[[j]],
										na.rm=TRUE)
			}
			centered <- attr(group[[i]]$dyvCovars[[j]], "centered")
			if (i <= 1) {centered1 <- centered}
			if (centered != centered1)
			{
				stop("inconsistent centering of dyadic covariates")
			}
		}
		dyvCovarMean[covar] <- ifelse(centered, vartotal / nonMissingCount, 0)
		rr <- range(thisrange, na.rm=TRUE)
		dyvCovarRange2[,covar] <- rr
		dyvCovarRange[covar] <- rr[2] - rr[1]
   }
	attr(group, "bRange") <- bRange
	attr(group, "behRange") <- behRange
	attr(group, "bSim") <- bSim
	attr(group, "bPoszvar") <- bPoszvar
	attr(group, "bMoreThan2") <- bMoreThan2
	attr(group, "bAnyMissing") <- bAnyMissing
	attr(group, "cCovarPoszvar") <- cCovarPoszvar
	attr(group, "cCovarMoreThan2") <- cCovarMoreThan2
	attr(group, "cCovarRange") <- cCovarRange
	attr(group, "cCovarRange2") <- cCovarRange2
	attr(group, "cCovarSim") <- cCovarSim
	attr(group, "cCovarMean") <- cCovarMean
	attr(group, "vCovarRange") <- vCovarRange
	attr(group, "vCovarSim") <- vCovarSim
	attr(group, "vCovarMoreThan2") <- vCovarMoreThan2
	attr(group, "vCovarPoszvar") <- vCovarPoszvar
	attr(group, "vCovarMean") <- vCovarMean
	attr(group, "dycCovarMean") <- dycCovarMean
	attr(group, "dycCovarRange") <- dycCovarRange
	attr(group, "dycCovarRange2") <- dycCovarRange2
	attr(group, "dyvCovarRange") <- dyvCovarRange
	attr(group, "dyvCovarRange2") <- dyvCovarRange2
	attr(group, "dyvCovarMean") <- dyvCovarMean
	group
}
##@namedVector DataCreate Utility to create a vector with appropriate names
namedVector <- function(vectorValue, vectorNames, listType=FALSE)
{
	if (listType)
	{
		tmp <- vector("list", length(vectorNames))
	}
	else
	{
		tmp <- rep(vectorValue, length(vectorNames))
	}
	names(tmp) <- vectorNames
	tmp
}


##@createSettings DataCreate
# create a settings structure for sienaData object x
createSettings <- function(x, varName=1, model=TRUE)
##
{
	if (!(inherits(x, "sienadata") | inherits(x, "siena")))
	{
		stop('x should be a siena data object')
	}
	if (is.null(x$depvars[[varName]]))
	{
		stop('varName should refer to a dependent network variable in x')
	}
	if ((attr(x$depvars[[varName]], 'type')) != 'oneMode')
	{
		stop('varName should refer to a dependent network variable in x')
	}
	universalOnly <- ifelse(model, "up", "none")
	attr(x$depvars[[varName]], 'settingsinfo') <- list(
		list(id="universal", type="universal", only=universalOnly, covariate=""),
		list(id="primary", type="primary", only="both", covariate=""))
# universal MUST be mentioned first here, and primary second.
	x
}

##@hasSettings DataCreate
# do the dependent variables in sienaData object x have a settings structure
# if varName is specified (number or name),
# this refers only to the indicated variable.
hasSettings <- function(x, varName=NULL)
{
	if (!(inherits(x, "sienadata") | inherits(x, "siena")))
	{
		stop('x should be a siena data object')
	}
	if (is.null(varName))
	{
		varNames <- 1:length(x$depvars)
	}
	else
	{
		varNames <- varName
		if (is.numeric(varName))
		{
			if ((varName < 1) | (varName > length(x$depvars)))
			{
				stop('varName should indicate one among the dependent variables')
			}
		}
		else
		{
			if (is.null(x$depvars[[varName]]))
			{
				stop('varName should indicate one among the dependent variables')
			}
		}
	}
	hasSettingsi <- rep(FALSE, length(varNames))
	for (i in seq(along = varNames))
	{
		hasSettingsi[i] <- (!is.null(attr(x$depvars[[varNames[i]]], 'settingsinfo')))
	}
	as.logical(hasSettingsi)
}

##@describeTheSetting DataCreate
# gives a description of the settings structure of depvar
describeTheSetting <- function(depvar)
{
	if (!inherits(depvar, 'sienaDependent'))
	{
		stop('depvar should be a dependent variable')
	}
	settingsinfo <- attr(depvar, 'settingsinfo')
	if (is.null(settingsinfo))
	{
		dts <- ""
	}
	else
	{
		dts <- list()
		for (i in seq_along(settingsinfo))
		{
			dts[[i]] <- unlist(c(name=attr(depvar, 'name'), settingsinfo[[i]]))
		}
		dtsn <- sapply(dts, function(x){x['name']})
		dtsid <- sapply(dts, function(x){x['id']})
		dtstype <- sapply(dts, function(x){x['type']})
		dtscovar <- sapply(dts, function(x){x['covar']})
		dtscovar <- sapply(dtscovar, function(x){ifelse(is.na(x),'none',x)})
		dtsonly <- sapply(dts, function(x){x['only']})
		dtsonly  <- sapply(dtsonly, function(x){ifelse(is.na(x),'both',x)})
		dts <- cbind(dtsn, dtsid, dtstype, dtscovar, dtsonly)
		colnames(dts) <- c('dependent variable', 'setting', 'type', 'covariate', 'direction')
		rownames(dts) <- 1:seq_len(dtsn)
	}
	dts
}



##@sienaGroupCreate DataCreate
sienaGroupCreate <- function(objlist, singleOK=FALSE, getDocumentation=FALSE)
{
	## suppressPackageStartupMessages(require(network))
	##@copyAttributes internal sienaGroupCreate
	copyAttributes <- function(x, y)
	{
		atts <- attributes(y)
		attr(x, "rangep") <- matrix(atts$range2, ncol=ncol(x), nrow=2)
		attr(x, "meanp") <- rep(atts$mean, ncol(x))
		attr(x, "range") <- atts$range
		attr(x, 'mean') <- atts$mean
		attr(x, 'centered') <- atts$centered
		attr(x, 'vartotal') <- atts$vartotal
		attr(x, 'nonMissingCount') <- atts$nonMissingCount
		attr(x, 'simMeans') <- atts$simMeans
		rr <- rangeAndSimilarity(x, atts$range2)
		attr(x, 'poszvar') <- atts$poszvar
		attr(x, 'similarity') <- rr$simValues
		attr(x, 'simMean') <- rr$simMean
		attr(x, 'moreThan2') <- atts$moreThan2
		attr(x, 'name') <- atts$name
		## storage.mode(attr(vCovars[[i]], 'range')) <- 'double'
	   x
	}
	if (getDocumentation)
	{
		return(getInternals())
	}
	if (!is.list(objlist))
	{
		stop('Need a list of objects')
	} 
	if (any (sapply(objlist, function(x) (!(inherits(x, "sienadata") | inherits(x, "siena"))))))
	{
		stop('Not a list of valid sienadata objects')
	}
	if (length(objlist) == 1 && !singleOK)
	{
		stop('Need more than one siena object')
	}
	## get hold of the attributes from the networks and lists of periods etc.
	group <- objlist
	## get the names of the nets and other objects

	netnames <-	 names(objlist[[1]]$depvars)
	pairsnames <- names(attr(objlist[[1]], "higher"))
	cCovars <- names(objlist[[1]]$cCovars)
	if (is.null(cCovars))
	{
		cCovars <- character(0)
	}
	vCovars <- names(objlist[[1]]$vCovars)
	if (is.null(vCovars))
	{
		vCovars <- character(0)
	}
	dycCovars <-  names(objlist[[1]]$dycCovars)
	if (is.null(dycCovars))
	{
		dycCovars <- character(0)
	}
	dyvCovars <- names(objlist[[1]]$dyvCovars)
	if (is.null(dyvCovars))
	{
		dyvCovars <- character(0)
	}
	## nNetnames <- length(netnames)
	anyMissing <- namedVector(FALSE, netnames)
	symmetric <- namedVector(TRUE, netnames)
	structural <- namedVector(FALSE, netnames)
	allUpOnly <- namedVector(TRUE, netnames)
	allDownOnly <- namedVector(TRUE, netnames)
	anyUpOnly <- namedVector(FALSE, netnames)
	anyDownOnly <- namedVector(FALSE, netnames)
	allHigher <- namedVector(TRUE,pairsnames )
	allDisjoint <- namedVector(TRUE, pairsnames)
	allAtLeastOne <- namedVector(TRUE, pairsnames)
	anyHigher <- namedVector(FALSE, pairsnames)
	anyDisjoint <- namedVector(FALSE, pairsnames)
	anyAtLeastOne <- namedVector(FALSE, pairsnames)
	types <- namedVector(NA, netnames)
	nodeSets <- namedVector(NA, netnames, listType=TRUE)
	ccnodeSets <- namedVector(NA, cCovars)
	cvnodeSets <- namedVector(NA, vCovars)
	dycnodeSets <- namedVector(NA, dycCovars, listType=TRUE)
	dyvnodeSets <- namedVector(NA, dyvCovars, listType=TRUE)
	dyctype <- namedVector(NA, dycCovars)
	dyvtype <- namedVector(NA, dyvCovars)
  #	 totalMissings <- namedVector(0, netnames, listType=TRUE)
  #	 nonMissingCount <- namedVector(0, netnames, listType=TRUE)
	observations <- 0
	periodNos <- rep(NA, 2)
	numberMissingNetwork <- rep(0, 2)
	numberMissingBehavior <- rep(0, 2)
	numberNonMissingNetwork <- rep(0, 2)
	numberNonMissingBehavior <- rep(0, 2)
	groupPeriods <- namedVector(NA, names(objlist))
	for (i in 1:length(objlist))
	{
		newobs <- objlist[[i]]$observations
		periodNos[observations + (1 : (newobs - 1))] <-
			observations + i - 1 + (1 : (newobs - 1))
		numberMissingBehavior[observations + (1 : (newobs - 1))] <- 0
		numberNonMissingBehavior[observations + (1 : (newobs - 1))] <- 0
		numberMissingNetwork[observations + (1 : (newobs - 1))] <- 0
		numberNonMissingNetwork[observations + (1 : (newobs - 1))] <- 0
		for (j in 1:length(objlist[[i]]$depvars))
		{
			varname <- names(objlist[[i]]$depvars)[j]
			netnamesub <- match(varname, netnames)
			if (is.na(netnamesub))
			{
				stop('object names inconsistent')
			}
			attribs <- attributes(objlist[[i]]$depvars[[j]])
			if (is.na(types[netnamesub]))
			{
				types[netnamesub] <- attribs[['type']]
			}
			else if (types[netnamesub] != attribs[['type']])
			{
				stop('Inconsistent network types')
			}
			if (is.null(nodeSets[[netnamesub]]))
			{
				  nodeSets[[netnamesub]] <- attribs[['nodeSet']]
			}
			else if (any(nodeSets[[netnamesub]] != attribs[['nodeSet']]))
			{
				stop('Inconsistent node Sets')
			}
			if (attribs[['type']] == 'oneMode')
			{
				if (!attribs[['symmetric']])
					symmetric[netnamesub] <- FALSE
			}
			if (any(attribs[['uponly']]))
			{
				anyUpOnly[netnamesub] <- TRUE
			}
			if (any(attribs[['downonly']]))
			{
				anyDownOnly[netnamesub] <- TRUE
			}
			if (!all(attribs[['uponly']]))
			{
				allUpOnly[netnamesub] <- FALSE
			}
			if (!all(attribs[['downonly']]))
			{
				allDownOnly[netnamesub] <- FALSE
			}
			if (attribs[["missing"]])
			{
				anyMissing[netnamesub] <- TRUE
			}
			if (attribs[["structural"]])
			{
				structural[netnamesub] <- TRUE
			}
			if (attribs[["type"]] == "behavior")
			{
				numberMissingBehavior[observations + (1 : (newobs - 1))] <-
					numberMissingBehavior[observations + (1 : (newobs - 1))] +
						attribs[["noMissingEither"]]
				numberNonMissingBehavior[observations + (1 : (newobs - 1))] <-
					numberNonMissingBehavior[observations +
											 (1 : (newobs - 1))] +
												 attribs[["nonMissingEither"]]
			}
			else
			{
				numberMissingNetwork[observations + (1 : (newobs - 1))] <-
					numberMissingNetwork[observations + (1 : (newobs - 1))] +
						attribs[["noMissingEither"]]
				numberNonMissingNetwork[observations + (1 : (newobs - 1))] <-
					numberNonMissingNetwork[observations + (1 : (newobs - 1))] +
						attribs[["nonMissingEither"]]
			}
		}
		thisHigher <- attr(objlist[[i]], "higher")
		thisDisjoint <- attr(objlist[[i]], "disjoint")
		thisAtLeastOne <- attr(objlist[[i]], "atLeastOne")

		anyHigher[names(thisHigher)[thisHigher]] <- TRUE
		anyDisjoint[names(thisDisjoint)[thisDisjoint]] <- TRUE
		anyAtLeastOne[names(thisAtLeastOne)[thisAtLeastOne]] <- TRUE
		allHigher[names(thisHigher)[!thisHigher]] <- FALSE
		allDisjoint[names(thisDisjoint)[!thisDisjoint]] <- FALSE
		allAtLeastOne[names(thisAtLeastOne)[!thisAtLeastOne]] <- FALSE
		for (j in seq(along=objlist[[i]]$cCovars))
		{
			varname <- names(objlist[[i]]$cCovars)[j]
			covarsub <- match(varname, cCovars)
			if (is.na(covarsub))
			{
				stop('actor covariate names inconsistent')
			}
			attribs <- attributes(objlist[[i]]$cCovars[[j]])
			if (is.na(ccnodeSets[covarsub]))
			{
				ccnodeSets[covarsub] <- attribs[['nodeSet']]
			}
			else if (ccnodeSets[covarsub] != attribs[['nodeSet']])
			{
				stop('Inconsistent node Sets')
			}
		}
		for (j in seq(along=objlist[[i]]$vCovars))
		{
			varname <- names(objlist[[i]]$vCovars)[j]
			covarsub <- match(varname, vCovars)
			if (is.na(covarsub))
			{
				stop('covariate names inconsistent')
			}
			attribs <- attributes(objlist[[i]]$vCovars[[j]])
			if (is.na(cvnodeSets[covarsub]))
			{
				cvnodeSets[covarsub] <- attribs[['nodeSet']]
			}
			else if (cvnodeSets[covarsub] != attribs[['nodeSet']])
			{
				stop('Inconsistent node Sets')
			}
		}

		 for (j in seq(along=objlist[[i]]$dycCovars))
		{
			varname <- names(objlist[[i]]$dycCovars)[j]
			covarsub <- match(varname, dycCovars)
			if (is.na(covarsub))
			{
				stop('dyadic covariate names inconsistent')
			}
			attribs <- attributes(objlist[[i]]$dycCovars[[j]])
			if (is.null(dycnodeSets[[covarsub]]))
			{
				dycnodeSets[[covarsub]] <- attribs[['nodeSet']]
				dyctype[[covarsub]] <- attribs[['type']]
			}
			else
			{
				if (any(dycnodeSets[[covarsub]] != attribs[['nodeSet']]))
				{
					stop('Inconsistent node Sets')
				}
				if (dyctype[[covarsub]] != attribs[['type']])
				{
					stop("Inconsistent constant dyadic covariate types")
				}
			}
		}
		for (j in seq(along=objlist[[i]]$dyvCovars))
		{
			varname <- names(objlist[[i]]$dyvCovars)[j]
			covarsub <- match(varname, dyvCovars)
			if (is.na(covarsub))
			{
				stop('dyadic covariate names inconsistent')
			}
			attribs <- attributes(objlist[[i]]$dyvCovars[[j]])
			if (is.null(dyvnodeSets[[covarsub]]))
			{
				dyvnodeSets[[covarsub]] <- attribs[['nodeSet']]
				dyvtype[[covarsub]] <- attribs[['type']]
			}
			else
			{
				if (any(dyvnodeSets[[covarsub]] != attribs[['nodeSet']]))
				{
					stop('Inconsistent node Sets')
				}
				if (dyvtype[[covarsub]] != attribs[['type']])
				{
					stop("Inconsistent changing dyadic covariate types")
				}
			}
		}
		observations <- observations + objlist[[i]]$observations - 1
		groupPeriods[i] <- newobs
	}
	## if more than one object, now create the group proper
	if (length(objlist) > 1)
	{
		for (i in 1:length(objlist))
		{
			## constant covars turn into changing ones, probably because the
			## actors change. Change on the individual data object.
			const <- objlist[[i]]$cCovars
			vars <- objlist[[i]]$vCovars
			oldnames <- names(vars)
			nVCovar <- length(vars)
			for (j in seq(along=const))
			{
				newcovar <-
					varCovar(matrix(const[[j]],
									ncol = objlist[[i]]$observations - 1,
									nrow=length(const[[j]])),
							 nodeSet=attr(const[[j]], "nodeSet"), warn=FALSE)
				newcovar <- copyAttributes(newcovar, const[[j]])
				nVCovar <- nVCovar + 1
				vars[[nVCovar]] <- newcovar
			}
			names(vars) <- c(oldnames, names(const))
			objlist[[i]]$vCovars <- vars
			objlist[[i]]$cCovars <- list()

			## now the dyadic ones similarly.

			const <- objlist[[i]]$dycCovars
			vars <- objlist[[i]]$dyvCovars
			oldnames <- names(vars)
			nVCovar <- length(vars)
			for (j in seq(along=const))
			{
				oneCentered <- FALSE
				oneNonCentered <- FALSE
				dim3 <- objlist[[i]]$observations - 1
				newcovar <-
					varDyadCovar(array(const[[j]], dim=c(dim(const[[j]]),
												   dim3)),
								 nodeSets=attr(const[[j]], "nodeSet"), warn=FALSE)
				attr(newcovar, "vartotal") <- attr(const[[j]], "vartotal")
				attr(newcovar, "nonMissingCount") <-
					 attr(const[[j]], "nonMissingCount")
				attr(newcovar, "mean") <- attr(const[[j]], "mean")
				attr(newcovar, "meanp") <- rep(attr(const[[j]], "mean"), dim3)
				attr(newcovar, "centered") <- attr(const[[j]], "centered")
				attr(newcovar, "range") <- attr(const[[j]], "range")
				attr(newcovar, "rangep") <- rep(attr(const[[j]], "range"),
												dim3)
				attr(newcovar, "range2") <- matrix(attr(const[[j]], "range2"),
												   ncol=dim3, nrow=2)
				attr(newcovar, 'name') <- attr(const[[j]], "name")
				nVCovar <- nVCovar + 1
				vars[[nVCovar]] <- newcovar
			}
			names(vars) <- c(oldnames, names(const))
			objlist[[i]]$dyvCovars <- vars
			objlist[[i]]$dycCovars <- list()
		}
		## update the working lists as well
		cCovars <- names(objlist[[1]]$cCovars)
		dycCovars <- names(objlist[[1]]$dycCovars)
		vCovars <- names(objlist[[1]]$vCovars)
		dyvCovars <- names(objlist[[1]]$dyvCovars)
	}
	symmetric[types=='behavior'] <- NA
	symmetric[types=='bipartite'] <- FALSE
	group <-  objlist
	attr(group, 'netnames') <- netnames
	attr(group, 'symmetric') <- symmetric
	attr(group, 'structural') <- structural
 #	 attr(group, "totalMissings") <- totalMissings
	attr(group, "numberNonMissingNetwork") <-
		numberNonMissingNetwork[1:observations]
	attr(group, "numberMissingNetwork") <-
		numberMissingNetwork[1:observations]
	attr(group, "numberNonMissingBehavior") <-
		numberNonMissingBehavior[1:observations]
	attr(group, "numberMissingBehavior") <-
		numberMissingBehavior[1:observations]
	attr(group, 'allUpOnly') <- allUpOnly
	attr(group, 'allDownOnly') <- allDownOnly
	attr(group, 'anyUpOnly') <- anyUpOnly
	attr(group, 'anyDownOnly') <- anyDownOnly
	attr(group, 'allHigher') <- allHigher
	attr(group, 'allDisjoint') <- allDisjoint
	attr(group, 'allAtLeastOne') <- allAtLeastOne
	attr(group, 'anyHigher') <- anyHigher
	attr(group, 'anyDisjoint') <- anyDisjoint
	attr(group, 'anyAtLeastOne') <- anyAtLeastOne
	attr(group, 'types') <- types
	attr(group, 'observations') <- observations
	attr(group, 'periodNos') <- periodNos[1:observations]
	attr(group, 'groupPeriods') <- groupPeriods
	attr(group, 'netnodeSets') <- nodeSets
	attr(group, 'cCovars') <- cCovars
	attr(group, 'vCovars') <- vCovars
	attr(group, 'dycCovars') <- dycCovars
	attr(group, 'dyvCovars') <- dyvCovars
	attr(group, 'ccnodeSets') <- ccnodeSets
	attr(group, 'cvnodeSets') <- cvnodeSets
	attr(group, 'dycnodeSets') <- dycnodeSets
	attr(group, 'dyvnodeSets') <- dyvnodeSets
	attr(group, 'compositionChange') <-
		any( sapply(objlist, function(x) length(x$compositionChange)>0))
	## in fact assume all the same - validate earlier!
	if (attr(group, 'compositionChange'))
	{
		exooptions <- sapply(objlist[[1]]$compositionChange, function(x)
							 attr(x, "ccOption"))
		names(exooptions) <- sapply(objlist[[1]]$compositionChange, function(x)
									attr(x, "nodeSet"))
		attr(group, 'exooptions') <- exooptions
	}
	else
	{
		attr(group, 'exooptions') <- character(0)
	}
	if (is.null(names(group)))
	{
		names(group) <- paste('Data', 1:length(group), sep="")
	}
# This is where the names Data1 etc. are created.
	class(group)<- c("sienaGroup", "siena")
	attr(group, "version") <- packageDescription(pkgname, fields = "Version")
	balmeans <- calcBalmeanGroup (group)
	names(balmeans) <- netnames
	attr(group, "balmean") <- balmeans
	structmeans <- calcStructmeanGroup (group)
	names(structmeans) <- netnames
	attr(group, "structmean") <- structmeans
	## calculate overall degree averages
	atts <- attributes(group)
	netnames <- atts$netnames
	types <- atts$types
	## cat(types,'\n')
	degrees <- namedVector(NA, netnames)
	for (net in seq(along=netnames))
	{
		if (types[net] != "behavior")
		{
			degree <- 0
			nDegree <- 0
			for (i in 1: length(group))
			{
				j <- match(netnames[net], names(group[[i]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- group[[i]]$depvars[[j]]
				degs <- attr(depvar, "degree")
				degree <- degree + sum(degs)
				nDegree <- nDegree + length(degs)
			}
			degrees[net] <- degree / nDegree
		}
	}
	attr(group, "averageOutDegree") <- degrees
	attr(group, "averageInDegree") <- degrees
	group <- groupRangeAndSimilarityAndMean(group)
	bAnyMissing <- attr(group, "bAnyMissing")
	attr(group, "anyMissing") <- anyMissing | bAnyMissing
	attr(group, "bAnyMissing") <- NULL
	tmp <- getGroupNetRanges(group)
	colnames(tmp) <- netnames
	attr(group, "netRanges") <- tmp
	group <- copyGroupAttributes(group, "depvars", "behRange", "behRange")
	if (length(group) >= 2)
	{
		##copy the global attributes down to individual level where appropriate
		##group <- copyGroupAttributes(group, "depvars", "balmean", "balmean")
		##group <- copyGroupAttributes(group, "depvars", "structmean",
		## "structmean")
		group <- copyGroupAttributes(group, "depvars", "symmetric", "symmetric")
		##group <- copyGroupAttributes(group, "depvars", "averageInDegree",
		##							   "averageInDegree")
		##group <- copyGroupAttributes(group, "depvars", "averageOutDegree",
		##							   "averageOutDegree")
		##group <- copyGroupAttributes(group, "depvars", "bSim", "simMean")
		group <- copyGroupAttributes(group, "depvars", "bposzvar", "poszvar")
		group <- copyGroupAttributes(group, "depvars", "bRange", "range")
		group <- copyGroupAttributes(group, "depvars", "bMoreThan2",
								 "moreThan2")
		group <- copyGroupAttributes(group, "depvars", "anyMissing", "missing")
		group <- copyGroupAttributes(group, "depvars", "structural",
								 "structural")
		##group <- copyGroupAttributes(group, "vCovars", "vCovarSim", "simMean")
		group <- copyGroupAttributes(group, "vCovars", "vCovarRange", "range",
								 TRUE)
		##group <- copyGroupAttributes(group, "vCovars", "vCovarMean", "mean",
		## TRUE)
		group <- copyGroupAttributes(group, "vCovars", "vCovarPoszvar",
								 "poszvar")
		group <- copyGroupAttributes(group, "vCovars", "vCovarMoreThan2",
								 "moreThan2")
		##group <- copyGroupAttributes(group, "dycCovars", "dycCovarMean",
		##	"mean")
		group <- copyGroupAttributes(group, "dycCovars", "dycCovarRange2",
								 "range2", TRUE)
		##group <- copyGroupAttributes(group, "dyvCovars", "dyvCovarMean",
		## "mean")
		group <- copyGroupAttributes(group, "dyvCovars", "dyvCovarRange",
								 "range", TRUE)
		group <- copyGroupAttributes(group, "dyvCovars", "dyvCovarRange2",
								 "range2", TRUE)
	}
	group
}
##@copyGroupAttributes DataCreate
## copies attribute groupattrname from the top level to the attribute
## attrname of the corresponding objects of vartype in the subsidiary data
## objects.
copyGroupAttributes <- function(group, vartype, groupattrname, attrname,
								storage.mode=FALSE)
{
	attributeValue <- attr(group, groupattrname)
	for (i in names(group))
	{
		for (j in names(group[[i]][[vartype]]))
		{
			if (is.matrix(attributeValue))
			{
				attr(group[[i]][[vartype]][[j]], attrname) <-
					as.vector(attributeValue[, j])
			}
			else
			{
				attr(group[[i]][[vartype]][[j]], attrname) <-
					as.vector(attributeValue[j])
			}
			if (storage.mode)
				 storage.mode(attr(group[[i]][[vartype]][[j]], attrname)) <-
					 "double"
		}
	}
	group
}

##@calcBalmeanGroup DataCreate
calcBalmeanGroup <- function(data)
{
	atts <- attributes(data)
	netnames <- atts$netnames
	types <- atts$types
   ## cat(types,'\n')
	balmeans <- rep(NA, length(netnames))
	for (net in seq(along=netnames))
	{
		if (types[net] == "oneMode")
		{
			tempra <- 0
			temprb <- 0
			for (i in 1: length(data))
			{
				j <- match(netnames[net], names(data[[i]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[i]]$depvars[[j]]
				dims <- attr(depvar, "netdims")
				for (k in 1 : (dims[3] - 1))
				{
					## remove structural values here?
					if (attr(depvar, "sparse"))
					{
						tmp <- depvar[[k]]
						diag(tmp)  <-  NA ## just in case
						x1 <- tmp@x
						struct <- !is.na(x1) & x1 %in% c(10, 11)
						x1[struct] <- x1[struct] - 10
						tmp@x <- x1
						tmp1 <- colSums(is.na(tmp))
						tempra <- tempra +
							sum(2 * colSums(tmp, na.rm=TRUE) *
								(dims[1] - tmp1 - colSums(tmp, na.rm=TRUE)),
								na.rm=TRUE)
						tmp2 <- colSums(!is.na(tmp))
						temprb <- temprb + sum(tmp2 * (tmp2 - 1))
					}
					else
					{
						tmp <- depvar[, , k]
						diag(tmp) <- NA ## just in case
						struct <- !is.na(tmp) & tmp %in% c(10, 11)
						tmp[struct] <- tmp[struct] - 10
						tempra <- tempra +
							sum(2 * colSums(tmp, na.rm=TRUE) *
								colSums(1 - tmp, na.rm=TRUE),
								na.rm=TRUE)
						tmp1 <- colSums(is.na(tmp))
						tmp2 <- colSums(!is.na(tmp))
						temprb <- temprb + sum(tmp2 * (tmp2 - 1))
					}
				}
			}
			balmeans[net] <- tempra / temprb
		}
	}
	balmeans
}
##@calcStructmeanGroup DataCreate
calcStructmeanGroup <- function(data)
{
	atts <- attributes(data)
	netnames <- atts$netnames
	types <- atts$types
   ## cat(types,'\n')
	structmeans <- rep(NA, length(netnames))
	for (net in seq(along=netnames))
	{
		if (types[net] == "oneMode")
		{
			tempra <- 0
			temprb <- 0
			for (i in 1: length(data))
			{
				j <- match(netnames[net], names(data[[i]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[i]]$depvars[[j]]
				dims <- attr(depvar, "netdims")
				for (k in 1 : (dims[3] - 1))
				{
					## remove structural values here?
					if (attr(depvar, "sparse"))
					{
						tmp <- depvar[[k]]
						diag(tmp)  <-  NA ## just in case
						x1 <- tmp@x
						struct <- !is.na(x1) & x1 %in% c(10, 11)
						x1[struct] <- x1[struct] - 10
						tmp@x <- x1
						tmp1 <- rowSums(is.na(tmp))
						tempra <- tempra +
							sum(2 * rowSums(tmp, na.rm=TRUE) *
								(dims[1] - tmp1 - rowSums(tmp, na.rm=TRUE)),
								na.rm=TRUE)
						tmp2 <- rowSums(!is.na(tmp))
						temprb <- temprb + sum(tmp2 * (tmp2 - 1))
					}
					else
					{
						tmp <- depvar[, , k]
						diag(tmp) <- NA ## just in case
						struct <- !is.na(tmp) & tmp %in% c(10, 11)
						tmp[struct] <- tmp[struct] - 10
						tempra <- tempra +
							sum(2 * rowSums(tmp, na.rm=TRUE) *
								rowSums(1 - tmp, na.rm=TRUE),
								na.rm=TRUE)
						tmp1 <- rowSums(is.na(tmp))
						tmp2 <- rowSums(!is.na(tmp))
						temprb <- temprb + sum(tmp2 * (tmp2 - 1))
					}
				}
			}
			structmeans[net] <- tempra / temprb
		}
	}
	structmeans
}
##@calcStructmean DataCreate
calcStructmean <- function(depvar)
{
	tempra <- 0
	temprb <- 0
	dims <- attr(depvar, "netdims")
	for (k in 1 : (dims[3] - 1))
	{
		if (attr(depvar, "sparse"))
		{
			tmp <- depvar[[k]]
			diag(tmp)  <-  NA ## just in case
			x1 <- tmp@x
			struct <- !is.na(x1) & x1 %in% c(10, 11)
			x1[struct] <- x1[struct] - 10
			tmp@x <- x1
			tmp1 <- rowSums(is.na(tmp))
			tempra <- tempra +
				sum(2 * rowSums(tmp, na.rm=TRUE) *
					(dims[1] - tmp1 - rowSums(tmp, na.rm=TRUE)),
					na.rm=TRUE)
			tmp2 <- rowSums(!is.na(tmp))
			temprb <- temprb + sum(tmp2 * (tmp2 - 1))
	 }
		else
		{
			##extract this observation
			tmp <- depvar[, , k]
			##remove diagonal
			diag(tmp) <- NA ## just in case
			##subtract 10 from structurals
			struct <- !is.na(tmp) & tmp %in% c(10, 11)
			tmp[struct] <- tmp[struct] - 10
			## rowSums here counts how many 1's or 0's in tmp
			tempra <- tempra +
				sum(2 * rowSums(tmp, na.rm=TRUE) *
					rowSums(1 - tmp, na.rm=TRUE),
					na.rm=TRUE)
			tmp2 <- rowSums(!is.na(tmp)) ## counts non-missings in row
			temprb <- temprb + sum(tmp2 * (tmp2 - 1))
		}
	}
	structmean <- tempra / temprb
	#  cat(tempra, temprb, structmean,'\n')
	structmean
}
##@calcBalmean DataCreate
calcBalmean <- function(depvar)
{
	tempra <- 0
	temprb <- 0
	dims <- attr(depvar, "netdims")
	for (k in 1 : (dims[3] - 1))
	{
		if (attr(depvar, "sparse"))
		{
			tmp <- depvar[[k]]
			diag(tmp)  <-  NA ## just in case
			x1 <- tmp@x
			struct <- !is.na(x1) & x1 %in% c(10, 11)
			x1[struct] <- x1[struct] - 10
			tmp@x <- x1
			tmp1 <- colSums(is.na(tmp))
			tempra <- tempra +
				sum(2 * colSums(tmp, na.rm=TRUE) *
					(dims[1] - tmp1 - colSums(tmp, na.rm=TRUE)),
					na.rm=TRUE)
			tmp2 <- colSums(!is.na(tmp))
			temprb <- temprb + sum(tmp2 * (tmp2 - 1))
	 }
		else
		{
			##extract this observation
			tmp <- depvar[, , k]
			##remove diagonal
			diag(tmp) <- NA ## just in case
			##subtract 10 from structurals
			struct <- !is.na(tmp) & tmp %in% c(10, 11)
			tmp[struct] <- tmp[struct] - 10
			## colSums here counts how many 1's or 0's in tmp
			tempra <- tempra +
				sum(2 * colSums(tmp, na.rm=TRUE) *
					colSums(1 - tmp, na.rm=TRUE),
					na.rm=TRUE)
			tmp2 <- colSums(!is.na(tmp)) ## counts non-missings in column
			temprb <- temprb + sum(tmp2 * (tmp2 - 1))
		}
	}
	balmean <- tempra / temprb
	#  cat(tempra, temprb,balmean,'\n')
	balmean
}

##@getGroupNetRanges DataCreate
getGroupNetRanges <- function(data)
{
	atts <- attributes(data)
	netnames <- atts$netnames
	types <- atts$types
   ## cat(types,'\n')
	ranges <- matrix(NA, ncol=length(netnames), nrow=2)
	varmin <- NA
	varmax <- NA
	for (net in seq(along=netnames))
	{
		if (types[net] %in% c("oneMode", "bipartite"))
		{
			varmin <- NA
			varmax <- NA
			for (i in 1: length(data))
			{
				j <- match(netnames[net], names(data[[i]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[i]]$depvars[[j]]
				### need to exclude structural values
				if (attr(depvar, "sparse"))
				{
			  ##   varmin <- min(varmin, sapply(depvar, function(x)
			  ##							   {
			  ##								   tmp <- x@x
			  ##								   min(tmp[!(is.na(tmp) |
			  ##										 tmp %in% c(10, 11))])
			  ##							   }), na.rm=TRUE)
					varmin <- 0 ## in sparse matrices 0's are not there
					varmax <- max(varmax, sapply(depvar, function(x)
											 {
												 tmp <- x@x
												 ifelse(length(tmp)==0,0,
												 max(tmp[!(is.na(tmp) |
														   tmp %in% c(10, 11))]))
											 }), na.rm=TRUE)
				}
				else
				{
					varmin <- min(varmin, depvar[!(is.na(depvar) |
												   depvar %in% c(10,11))],
								  na.rm=TRUE)
					varmax <- max(varmax, depvar[!(is.na(depvar) |
												   depvar %in% c(10,11))],
								  na.rm=TRUE)
				}
			}
			ranges[, net] <- c(varmin, varmax)
		}
	}
	ranges
}

##@covarDist2 DataCreate calculate average alter values for covariate wrt each net
covarDist2 <- function(z)
{
	netNames <- names(z$depvars)
	netTypes <- sapply(z$depvars, function(x)attr(x, "type"))
	netActorSet <- sapply(z$depvars, function(x)
					  {
						  if (attr(x, "type") == "bipartite")
						  {
							  attr(x, "nodeSet")[2]
						  }
						  else
						  {
							  attr(x, "nodeSet")
						  }
					  }
						  )
	for (i in seq(along=z$cCovars))
	{
		nodeSet <- attr(z$cCovars[[i]], "nodeSet")
		use <- (!(netTypes %in% c("behavior", "continuous")) & (netActorSet == nodeSet))
		simMeans <- namedVector(NA, netNames[use])
		for (j in which(use))
		{
			simMeans[netNames[j]] <-
				calcCovarDist2(matrix(z$cCovars[[i]], ncol=1),
							   z$depvars[[j]])
		}
		attr(z$cCovars[[i]], "simMeans") <- simMeans
	}
	for (i in seq(along=z$vCovars))
	{
		nodeSet <- attr(z$vCovars[[i]], "nodeSet")
		use <- (!(netTypes %in% c("behavior", "continuous")) & (netActorSet == nodeSet))
		simMeans <- namedVector(NA, netNames[use])
		for (j in which(use))
		{
			simMeans[netNames[j]] <-
				calcCovarDist2(z$vCovars[[i]], z$depvars[[j]])
		}
		attr(z$vCovars[[i]], "simMeans") <- simMeans

	}
	for (i in seq(along=z$depvars))
	{
		if (netTypes[i] %in% c("behavior", "continuous"))
		{
			beh <- z$depvars[[i]][, 1, ]
			## take off the mean NB no structurals yet!
			beh <- beh - mean(beh, na.rm=TRUE)
			nodeSet <- netActorSet[i]
			use <- (!(netTypes %in% c("behavior", "continuous"))
			        & netActorSet == nodeSet)
			simMeans <- namedVector(NA, netNames[use])
			for (j in which(use))
			{
				simMeans[netNames[j]] <-
					calcCovarDist2(beh[, -ncol(beh), drop=FALSE],
								   z$depvars[[j]], rval=range(beh, na.rm=TRUE))
			}
			attr(z$depvars[[i]], "simMeans") <- simMeans
		}
	}
	z
}
##@calcCovarDist2 DataCreate similarity mean for alter of this depvar
calcCovarDist2 <- function(covar, depvar, rval=NULL)
{
	## remove final obs from depvars
	observations <- attr(depvar, "netdims")[3] - 1

	simTotal <- rep(NA, observations)
	simCnt <- rep(NA, observations)

	for (i in 1:observations)
	{
		if (ncol(covar) == 1) # maybe constant covariate used for each obs
		{
			xx <- covar[, 1]
		}
		else
		{
			xx <- covar[, i]
		}
		if (attr(depvar, "sparse"))
		{
			dep <- depvar[[i]]
			dep@x[dep@x %in% c(10, 11)] <- dep@x[dep@x %in% c(10, 11)] - 10
		}
		else
		{
			dep <- depvar[, , i, drop=FALSE]
			dep[dep %in% c(10, 11)] <- dep[dep %in% c(10, 11)] - 10
		}
		vi <- apply(dep, 1, function(x)
				{
					if (sum(x, na.rm=TRUE) > 0)
					{
						nonMissing <- sum(!is.na(xx[!is.na(x) & x > 0]))
						if (nonMissing > 0)
						{
							sum(x * xx, na.rm=TRUE)/ sum(x, na.rm=TRUE)
						}
						else
						{
							NA
						}
					}
					else
					{
						0
					}

				}
					)

		tmp <- rangeAndSimilarity(vi, rval)
		simTotal[i] <- tmp$simTotal
		simCnt[i] <- tmp$simCnt
	}
	ifelse(sum(simCnt)==0, 0, sum(simTotal)/sum(simCnt))
}

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: selectionTables.r
## *
## *
## * Description: This file contains functions for making selection tables
## *
## ****************************************************************************/

########################################################################
# R script SelectionTables.r                                           #
# written by Tom A.B. Snijders                                         #
# September 5, 2025                                                    #
########################################################################
########################################################################

########################################################################
#                                                                      #
# These are functions for constructing and presenting selection tables #
# for the interpretation of results for network dynamics               #
# obtained with the RSiena program.                                    #
# Also consult the manual, Sections 13.1 and 13.3!                     #
#                                                                      #
# It is recommended that you download this file and read it;           #
# in particular, for the functions you wish to use,                    #
# read the comments that are written at the start of the functions.    #
#                                                                      #
# The main functions to use are                                        #
# selectionTable.plot <- function(x, xd, name, vname, levls,           #
# and many other arguments, as shown below;                            #
# which makes a selection plot;                                        #
# and                                                                  #
# selectionMatrix <- function(x,xd,name,vname,levls, levls.alt=levls)  #
# which creates a matrix containing the selection table                #
# for siena or sienaGroup data set xd,                                 #
# sienaFit, sienaBayesFit, or sienaMeta object x,                      #
# actor covariate vname (should be a character string),                #
# dependent network variable name (also a character string),           #
# levels for ego levls, levels for alter levls.alt.                    #
# Mostly levls.alt will be the same as levls,                          #
# in which case it does not have to be specified.                      #
#                                                                      #
# These functions use the parameter estimates given by x$theta.        #
#                                                                      #
# You can use selectionTable.plot directly,                            #
# without first using some of the other functions.                     #
#                                                                      #
# For models utilizing creation and/or endowment effects,              #
# by specifying include.endow=TRUE                                     #
# the sum of evaluation and endowment effects is used;                 #
# by specifying include.creation=TRUE                                  #
# the sum of evaluation and creation effects is used;                  #
# the first is relevant for maintaining existing ties,                 #
# the second for creating new ties.                                    #
#                                                                      #
# The data set is only used to get the means and similarity means      #
# which are subtracted somewhere in the effects.                       #
#                                                                      #
# It is assumed variable <vname> is centered;                          #
# if it is not, some things below need to be changed.                  #
#                                                                      #
# Other functions to use (see below):                                  #
# selectionTable.se (only for sienaFit objects)                        #
# Computes standard error of a linear combination of elements          #
# of the selection table.                                              #
# This function uses the covariance matrix of parameter estimates      #
# given by x$covtheta.                                                 #
#                                                                      #
# selectionTableWithMax                                                #
# Creates a data frame of the selection table together with            #
# the maximum per ego of the selection function over alters.           #
#                                                                      #
# selectionTable.norm  (only for sienaFit objects)                     #
# Computes the location of the social norm and its standard error      #
# for the model discussed in Snijders & Lomi (2019).                   #
# This also uses the covariance matrix of parameter estimates          #
# given by x$covtheta.                                                 #
#                                                                      #
# selectionTable.plot uses ggplot2.                                    #
#                                                                      #
# If these functions are used                                          #
# for a sienaMeta object x constructed by siena08                      #
# (as opposed to a sienaFit object constructed by siena07),            #
# xd should be one of the individual data sets used for creating x,    #
# (a single-group instead of multi-group data set)                     #
# with an average ('representative') value                             #
# for the mean of variable <name>.                                     #
#                                                                      #
# If these functions are used                                          #
# for a sienaBayesFit object x  as created by sienaBayes               #
# (as opposed to a sienaFit object),                                   #
# information about the mean and similarity mean for variable vname    #
# is taken from the first data set in the list xd.                     #
########################################################################


##@selectionTable.basis  Basis for selectionTable
selectionTable.basis <- function(x, xd, name, vname,
                    levls=NULL, levls.alt=levls, nfirst=x$nwarm+1,
                    multiplier=1,
                    include.endow=FALSE, include.creation=FALSE,
                    silent=FALSE){
# Creates basic materials for a selection table for
# siena data set xd, sienaFit object x,
# actor variable vname (should be a character string),
# dependent variable name (also a character string),
# levels for ego levls, levels for alter levls.alt.
# The multiplier is used in case
# the variable vname has a different natural scale,
# and the values for ego and alter as reported should be multiplied.
# The values given for levls and levls.alt are before this multiplication,
# i.e., they are the values as in the data set.
#
# The function can be used also for a sienaMeta or sienaBayesFit object x.
# Then xd must be one of the data sets used for creating x,
# with an average ('representative') value for the mean of variable <vname>.
# nfirst is used only for sienaBayesFit objects x.
#
    dsign <- function(d){
        0.5*(ifelse(d > 0, 1, 0) + ifelse(d >= 0, 1, 0))
    }
	if (!silent)
    {
		cat("Dependent network",name, "; actor variable",vname,".\n")
	}
# Obtain estimate
    if (inherits(x, "sienaBayesFit"))
    {
        theta <- sienaFitThetaTable(x, fromBayes=TRUE,
                    groupOnly=0, nfirst=nfirst)$mydf$value
        theEffects <- x$requestedEffects
        if (length(theta) != dim(theEffects)[1])
        {
            stop('mismatch between theta and effect names')
        }
# Is this correct for all model specifications?
    }
    else
    {
        theta <- x$theta
        theEffects <- x$requestedEffects
    }
# Obtain means
    depvar <- FALSE
    if (inherits(xd, "sienaGroup"))
    {
        cat("A sienaGroup data object was given.\n")
        # Does the variable exist, and is it a dependent variable?
        xd1 <- xd[[1]]
        thevar <- xd1$cCovars[[vname]]
        if (is.null(thevar)) {thevar <- xd1$vCovars[[vname]]}
        if (is.null(thevar))
            {
                depvar <- TRUE
                thevar <- xd1$depvars[[vname]]
            }
        if (is.null(thevar)){stop(paste('There is no actor variable <',vname,'>.'))}
        if (depvar)
        {
#           vmean <- mean(sapply(xd, function(z){mean(z$depvars[[vname]], na.rm=TRUE)}))
            vtot <- sum(sapply(xd, function(z){sum(z$depvars[[vname]], na.rm=TRUE)}))
            vtotN <- sum(sapply(xd, function(z){sum(!is.na(z$depvars[[vname]]))}))
            vmean <- vtot/vtotN
            vsmean <- attr(xd, "bSim")[[vname]]
        }
        else
        { # constant covariates are transformed by sienaGroupCreate to varying covariates
            vmean <- attr(xd, "vCovarMean")[vname]
            vsmean <- attr(xd, "vCovarSim")[vname]
        }
    }
    else
    {
        thevar <- xd$cCovars[[vname]]
        if (is.null(thevar)) {thevar <- xd$vCovars[[vname]]}
        if (is.null(thevar))
            {
                depvar <- TRUE
                thevar <- xd$depvars[[vname]]
            }
        if (is.null(thevar)){stop(paste('There is no actor variable <',vname,'>.'))}
        if (depvar)
        {# Then the mean is not stored as an attribute
            means <- colMeans(thevar, na.rm=TRUE)
            vmean <- mean(means)
        }
        else
        {
            vmean <- attr(thevar, 'mean')
        }
        vsmean <- attr(thevar, 'simMean')
    }
    if (!depvar)
    {
        if (!attr(thevar, 'centered'))
        {
            cat('Warning: this variable is not centered. The mean is subtracted anyway; this may be wrong.\n')
        }
    }
    Delta  <- attr(thevar, 'range')
	Delta12  <- attr(thevar, 'range2')
	if (is.null(Delta12))
	{
		Delta12  <- range(attr(thevar, 'rangep'))
	}
    if (is.null(levls))
    {
        levls <- floor(Delta12[1]) : ceiling(Delta12[2])
    }
    if (is.null(levls.alt))
    {
        levls.alt <- levls
    }
    replace0 <- function(k){ifelse(length(k)==0,0,k)}
# If additional efNames are going to be included,
# adapt also the coeffs matrix below and function selectionTableWithMax.
    efNames <- c('altX','altSqX','egoX','egoSqX','egoXaltX',
                    'simX','diffX','diffSqX','higher','sameX','egoDiffX','egoPlusAltX')
    veff.eval <- sapply(efNames, function(s){replace0(which(theEffects$name == name &
                    theEffects$interaction1 == vname &
                    (theEffects$type == 'eval') &
                    theEffects$include &
                    theEffects$shortName==s))})
    veff.endow <- sapply(efNames, function(s){replace0(which(theEffects$name == name &
                    theEffects$interaction1 == vname &
                    (theEffects$type == 'endow') &
                    theEffects$include &
                    theEffects$shortName==s))})
    veff.creation <- sapply(efNames, function(s){replace0(which(theEffects$name == name &
                    theEffects$interaction1 == vname &
                    (theEffects$type == 'creation') &
                    theEffects$include &
                    theEffects$shortName==s))})

# veff gives the indicators of the effects in x
    taketheta <- function(k){ifelse(k==0,0,theta[k])}
    vtheta <- sapply(veff.eval, taketheta)
    if (include.endow)
    {
        vtheta <- vtheta + sapply(veff.endow,taketheta)
    }
    if (include.creation)
    {
        vtheta <- vtheta + sapply(veff.creation,taketheta)
    }
    if (all(vtheta == 0))
    {
        cat('All parameters found for the effect of', vname, 'on', name, 'are 0.\n')
        stop('there seems to be something wrong with x, xd, name, or vname.')
    }

    if (!silent){
        cat('Parameters found are\n')
        print(vtheta[vtheta != 0.0])
        cat('Check that these parameter values are correct!\n')
        if (include.endow)
        {
            cat('This includes evaluation and endowment (maintenance) effects.\n')
        }
        if (include.creation)
        {
            cat('This includes evaluation and creation effects.\n')
        }
    }
    if (!silent)
    {
        cat('The subtracted mean value of', vname, 'is', vmean,'.\n')
        if (vtheta["simX"] != 0)
		{
			cat('The subtracted similarity mean value of', vname, 'is', vsmean,
															'.\n')
		}
    }
    flush.console()
# vtheta contains the parameter values in x
    K <- length(levls)
    KA <- length(levls.alt)
    valter <- rep(levls.alt,K)
    vego <- rep(levls,each=KA)
# coeffs duplicates the information in f6 in selectionTableWithMax.
# This duplication is undesirable, but I do this anyway.
# When maintaining this script, coeffs and f6 in selectionTable.WithMax
# must stay in line.
    coeffs <- matrix(NA, K*KA, length(efNames))
    coeffs[,1] <- (valter - vmean)
    coeffs[,2] <- (valter - vmean)*(valter - vmean)
    coeffs[,3] <- (vego - vmean)
    coeffs[,4] <- (vego - vmean)*(vego - vmean)
    coeffs[,5] <- (vego - vmean)*(valter - vmean)
    coeffs[,6] <- (1-(abs(valter - vego)/Delta)-vsmean)
    coeffs[,7] <- (valter - vego)
    coeffs[,8] <- (valter - vego)*(valter - vego)
    coeffs[,9] <- dsign(vego - valter)
    coeffs[,10] <- 1*(vego == valter)
    coeffs[,11] <- vego*(valter - vego)
    coeffs[,12] <- (vego - vmean) + (valter - vmean)
    select <- coeffs %*% vtheta
    fact <- multiplier*(1:K)
    ego <- factor(rep(fact,each=KA), ordered=TRUE)
    vego <- multiplier*vego
    valter <- multiplier*valter
	quad <- (sum(abs(vtheta[c(6,9,10)]))==0) # Then it is a quadratic function
    df <- data.frame(ego,vego,valter,select)
    list(df=df, veff.eval=veff.eval, veff.endow=veff.endow,
         veff.creation=veff.creation, vtheta=vtheta, vmean=vmean, vsmean=vsmean,
         Delta=Delta, coeffs=coeffs, levls=levls, levls.alt=levls.alt, quad=quad)
}


##@selectionTable  selectionTable
selectionTable <- function(x, xd, name, vname,
                    as.matrix=FALSE,
                    levls=NULL, levls.alt=levls, nfirst=x$nwarm+1,
                    multiplier=1,
					include.endow=FALSE, include.creation=FALSE,
					silent=FALSE){
# Creates a data frame containing the selection table for
# siena data set xd, sienaFit object x,
# actor covariate vname (should be a character string),
# dependent variable name (also a character string),
# levels for ego levls, levels for alter levls.alt.
    sel.t <- selectionTable.basis(x=x, xd=xd, name=name, vname=vname,
				levls=levls, levls.alt=levls.alt, nfirst=nfirst,
                multiplier=multiplier,
				include.endow=include.endow,
				include.creation=include.creation,
				silent=silent)
	df <- sel.t$df
    vals <- as.character(unique(df$vego))
    df$ego <- factor(as.character(df$vego), levels=vals, ordered=TRUE)
    df$valter <- as.numeric(as.character(df$valter))
    df$select <- as.numeric(as.character(df$select))
	class(df) <- c("selectionTable", class(df))
	attr(df, "name") <- name
	attr(df, "vname") <- vname
	attr(df, "multiplier") <- multiplier
	attr(df, "levls") <- sel.t$levls
	attr(df, "levls.alt") <- sel.t$levls.alt
	attr(df, "quad") <- sel.t$quad
	if (as.matrix)
	{
		mat <- matrix(as.numeric(as.character(df$select)),
            length(unique(df$vego)),
            length(unique(df$valter)), byrow=TRUE)
		colnames(mat) <- sel.t$levls.alt
		rownames(mat) <- sel.t$levls
		class(mat) <- c("selectionTable" ,class(mat))
		return(mat)
	}
	else
	{
		return(df)
	}
}

selectionTableWithMax <- function(x, xd, name, vname,
                levls=NULL, levls.alt=levls, nfirst=x$nwarm+1,
                multiplier=1, discrete=TRUE,
                include.endow=FALSE, include.creation=FALSE,
                silent=FALSE){
# Creates a data frame including the selection table for
# siena data set xd, sienaFit object x,
# actor covariate vname (should be a character string),
# dependent variable name (also a character string),
# levels for ego levls, levels for alter levls.alt,
# to which is appended the values for the maximum across alter,
# where alter ranges over levls.alt
# (a grid with 200 points between minimum and maximum is used).
# The variable named 'kind' is 1 for the selection table and 2 for the maximum.
    dsign <- function(d){
        0.5*(ifelse(d > 0, 1, 0) + ifelse(d >= 0, 1, 0))
    }
    st <- selectionTable.basis(x, xd, name, vname, levls, levls.alt, nfirst=nfirst,
                                    multiplier,
				include.endow=include.endow,
				include.creation=include.creation,
				silent=silent)
    df1 <- st$df
    df1$kind <- rep(1,dim(df1)[1])
    vtheta <- st$vtheta
    vmean <- st$vmean
    Delta <- st$Delta
    vsmean <- st$vsmean
# f6 duplicates the information in coeffs in selectionTable.basis.
# This duplication is undesirable as programming style, but I do this anyway.
# When maintaining this script, f6 and coeffs in selectionTable.basis
# must stay in line.
    f6 <- function(ve,va){
        contr <- vtheta[1]*(va-vmean) + vtheta[2]*(va-vmean)*(va-vmean) +
            vtheta[3]*(ve-vmean) +
            vtheta[4]*(ve-vmean)*(ve-vmean) + vtheta[5]*(ve-vmean)*(va-vmean) +
            vtheta[6]*(1-(abs(va-ve)/Delta)-vsmean) +
            vtheta[7]*(va-ve) + vtheta[8]*(va-ve)*(va-ve) +
            vtheta[9]*dsign(ve-va) +
            vtheta[10]*(1*(ve == va)) +
            vtheta[11]*ve*(va-ve) +
            vtheta[12]*((va-ve) + (va-ve))
        names(contr) <- 'contribution'
        contr}
# vtheta contains the parameter values in x
    K <- length(levls)
    KA <- length(levls.alt)
    valter <- rep(levls.alt,K)
    vego <- rep(levls,each=KA)
    fact <- 1:K
    ego <- factor(rep(fact,each=KA))
# Now calculate the maximum.
    minv <- min(levls.alt)
    maxv <- max(levls.alt)
    gridv <- minv + (0:200)*((maxv-minv)/200)
# vsmean is not important because it is an additive constant
    if (discrete){
        altm <- as.integer(round(minv)):as.integer(round(maxv))
        egom <- rep(levls[1], length(altm)) # dummy
        maxm <- sapply(altm, function(x){max(f6(x, gridv))})
    } else {
        egom <- rep(NA, K*4)
        altm <- rep(NA, K*4)
        maxm <- rep(NA, K*4)
        for (i in 1:K){
            for (j in 1:4){
            egom[4*(i-1) + j] <- levls[i]
            x1 <- ifelse(i <= 1, levls[1], 0.5*(levls[i-1]+levls[i]))
            x2 <- ifelse(i >= K, levls[K], 0.5*(levls[i+1]+levls[i]))
            dd <- x2-x1
            alterij <- ((j-1)/3)*x2 + (1 - (j-1)/3)*x1
            altm[4*(i-1) + j] <- alterij
#           maxm[4*(i-1) + j]  <- max(f6(alterij, gridv))
            maxm[4*(i-1) + j]  <- max(f6(levls[i], gridv))
            }
        }
        }
    df1$ego <- as.character(df1$vego)
    df2 <- data.frame(ego=egom,vego=egom,valter=altm,select=maxm,kind=2)
    df <- rbind(df1,df2)
    df$ego <- as.character(df$vego)
	attr(df, "name") <- name
	attr(df, "vname") <- vname
	attr(df, "multiplier") <- multiplier
	attr(df, "levls") <- st$levls
	attr(df, "levls.alt") <- st$levls.alt
	attr(df, "quad")<- st$quad
    df
}

selectionTable.se <- function(x, xd, name, vname,
                    levls=NULL, ww, levls.alt=levls, nfirst=x$nwarm+1,
                    multiplier=1){
# Calculates standard errors for a linear combination
# of elements of the selection table for siena data set xd, sienaFit object x,
# actor covariate vname (should be a character string),
# dependent variable name (also a character string),
# levels for ego levls, levels for alter levls.alt,
# Coefficients of the linear combination are in matrix ww,
# which is assumed to have rows corresponding to levls and columns to levls.alt.
# The linear combination for which the standard error is computed is
# sum_{h,k} ww[h,k] * selectionTable[h,k].
    if (inherits(x, "sienaBayesFit")){
        stop('This function does not work for sienaBayesFit objects.\n')
    }
    if (!all(dim(ww) == c(length(levls), length(levls.alt)))){
        stop('Dimension of ww should be the lengths of levls and levls.alt.\n')
    }
    cat("Requested cell weights (row = ego; col = alter):\n")
    print(cbind(which(ww != 0, arr.ind=TRUE),
            value=ww[which(ww != 0, arr.ind=TRUE)]))
    sb <- selectionTable.basis(x, xd, name, vname, levls, levls.alt,
                nfirst=nfirst, multiplier, silent=TRUE)
    veff.r <- sb$veff.eval[sb$veff.eval != 0] # effects included in x
    cth <- x$covtheta[veff.r, veff.r] # their covariance matrix
    cth[is.na(cth)] <- 0 # if any effects were fixed, they have NA in covtheta
    lincomb <- sum(as.vector(t(ww)) * sb$df["select"])
    # the desired linear combination of cell values of the selection matrix
    wt <- colSums(as.vector(t(ww)) * sb$coeff)
    # Vector of weights for the linear combination of parameters
    names(wt) <- names(sb$veff.eval)
    wt.r <- wt[names(veff.r)] # should be restricted to what is in the model
    cat("Parameter estimates and their resulting weights are \n")
    print(rbind('param'=sb$vtheta[sb$veff.eval != 0], 'weight'=wt.r))
    cat("Linear combination of cells of selection matrix", round(lincomb,4),
        "\nStandard error\n")
    print(sqrt((wt.r %*% cth %*% wt.r)[1,1]))
}

selectionTable.norm <- function(x, xd, name=attr(xd$depvars,"name")[1], vname,
                                    nfirst=x$nwarm+1, multiplier=1){
# Calculates the location of the social norm and its standard error
# in the attraction function for siena data set xd, sienaFit object x,
# actor covariate vname (should be a character string),
# dependent variable name (also a character string),
# if the attraction function with terms
# 'altX','altSqX','egoX','egoSqX','diffSqX' is used.
# Note that the attraction function should not include the 'egoXaltX' effect!
    stab <- selectionTable.basis(x,xd,name,vname,1:2, nfirst=nfirst, multiplier)
# the 1:2 is arbitrary, not used
    vtheta <- stab$vtheta
    vmean <- stab$vmean
    veff <- stab$veff
	veff.eval <- stab$veff.eval
    if (vtheta["egoXaltX"] != 0){
        cat('Warning: effect of egoXaltX is ',vtheta["egoXaltX"],
             ', not equal to 0; \n')
        cat(' this function does not apply to such a sienaFit object.\n')
        }
# calculate gradient
    grad <- matrix(0, length(x$theta), 1)
    grad[veff.eval["altX"],1] <- -1/(2*vtheta["altSqX"])
    grad[veff.eval["altSqX"],1] <- vtheta["altX"]/(2*vtheta["altSqX"]*vtheta["altSqX"])
    covtheta <- x$covtheta
    covtheta[is.na(covtheta)] <- 0
    if (vtheta["altSqX"] > 0){
        cat('Warning: the coefficient of alter squared is positive.\n')
        cat('This means the extremum of the attraction function is a minimum,\n')
        cat('and cannot be interpreted as a social norm.\n')
        flush.console()
        }
    list(vnorm = vmean - (vtheta["altX"]/(2*vtheta["altSqX"])),
        se.vnorm = sqrt(t(grad) %*% covtheta %*% grad))
}


##@print.selectionTable Methods
print.selectionTable <- function(x, ...)
{
	if (!inherits(x, "selectionTable"))
	{
		stop("not a legitimate selectionTable object")
	}
	if (inherits(x, "matrix"))
	{
		mat <- x
		class(mat) <- "matrix"
		print(mat)
	}
	else if (inherits(x, "data.frame"))
	{
		mat <- matrix(as.numeric(as.character(x$select)),
            length(unique(x$vego)),
            length(unique(x$valter)), byrow=TRUE)
		colnames(mat) <- attr(x, "levls.alt")
		rownames(mat) <- attr(x, "levls")
		print(mat)
	}
	invisible(x)
}


	
interpret_selection <- function(x, ...) UseMethod("interpret_selection", x)

##@interpret_influence.sienaFit method for siena.Fit
interpret_selection.sienaFit <- function(x, xd, name, vname,
                    as.matrix=FALSE,
                    levls=NULL, levls.alt=levls, nfirst=x$nwarm+1,
                    multiplier=1,
					include.endow=FALSE, include.creation=FALSE,
					silent=FALSE, ...)
{ 	
		selectionTable(x, xd, name, vname,
                    as.matrix=as.matrix,
                    levls=levls, levls.alt=levls.alt, nfirst=nfirst,
                    multiplier=multiplier,
					include.endow=include.endow, include.creation=include.creation,
					silent=silent, ...)
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: sienaModelCreate.r
# *
# * Description: This module contains the function for creating model objects.
# *
# *****************************************************************************/


##@set_model_saom ModelCreate
set_model_saom <- function(modelType=NULL, behModelType=NULL,
						MaxDegree = NULL, Offset = NULL)
# Sets details of SAOM model specification
{
	model <- list()
	if (!is.null(modelType))
	{
		if (any(!(modelType %in% 1:11)))
		{
			stop('modelType can only have integer values from 1 to 11\n')
		}
		if (any(is.null(names(modelType))))
		{
			stop('modelType must have names of dependent networks')
		}
		model$modelType <- modelType
	}
	if (!is.null(behModelType))
	{
		if (any(!(behModelType %in% c(1,2))))
		{
			warning('behModelType can only have values 1 or 2; other values changed to 1\n')
			model$behModelType[!(behModelType %in% c(1,2))] <- 1
		}
		if (any(is.null(names(behModelType))))
		{
			stop('behModelType must have names of dependent behavioral variables')
		}
		model$behModelType <- behModelType
	}
	model$MaxDegree <- MaxDegree
	if (!is.null(Offset))
	{
		if (any(is.null(names(Offset))))
		{
			stop('Offset must have names of dependent behavioral variables')
		}
		model$UniversalOffset <- Offset
	}
	class(model) <- "sienaModel"
	attr(model, "version") <- packageDescription(pkgname, fields = "Version")
	model
}


##@set_algorithm_saom ModelCreate
set_algorithm_saom <- function(
# Sets SAOM estimation procedure:
	maxlike = FALSE, gmm = FALSE,
	cond = NA, condvarno = 0, condname = "",
	simOnly = FALSE,
	targets = NULL, thetaValues = NULL, 
	seed = NULL,
# Specification of Robbins Monro algorithm:
	n3 = 1000, nsub = 4, n2start = NULL, firstg = 0.2, reduceg = 0.5,
	truncation = 5, doubleAveraging = 0,
	diagonalize = 0.2*!maxlike, standardizeVar = (diagonalize<1), dolby = TRUE,
	splitDepvars = 0,
	useStdInits = FALSE, findiff = FALSE,
# Specification of likelihood algorithm,
# relevant only for maximum likelihood and Bayesian estimation:
	mult=5, prML=1, maximumPermutationLength=40,
	minimumPermutationLength=2, initialPermutationLength=20, localML=FALSE)
# Note: sienaModelCreate also contains the keyword `fn`,
# which allows to use a function for one simulation step in the RM algorithm
# different from "simstats0c" and "maxlikec".
{
	model <- list()
	model$maxlike <- maxlike
	model$gmm <- gmm
	if (maxlike)
	{
		model$FRANname <- "maxlikec"
		if (is.na(cond))
		{
			cond <- FALSE
		}
		if (cond)
		{
			stop("Conditional estimation is not possible with",
				"maximum likelihood estimation")
		}
	}
	else
	{
		model$FRANname <- "simstats0c"
	}
	model$cconditional <- cond
	if (!is.na(cond) && cond && condvarno == 0 && condname == "")
	{
		model$condvarno <-  1
		model$condname <- ""
	}
	else
	{
		model$condvarno <-  condvarno
		model$condname <- condname
	}
	model$simOnly <- simOnly
	model$targets <- targets
	## create theta values for phase 3, if necessary
	if (is.null(thetaValues))
	{
		model$thetaValues <- NULL
		model$thetaFromFile <- FALSE
	}
	else
	{
		if (!model$simOnly)
		{
			cat('The thetaValues parameter was given,\n')
			cat('but simOnly was not specified.\n')
			cat('This is inconsistent.\n')
			stop('Inconsistent combination simOnly - thetaValues')
		}
		if (!is.matrix(thetaValues))
		{
			stop('thetaValues should be a matrix.')
		}
		model$thetaValues <- thetaValues
		model$thetaFromFile <- TRUE
	}
	model$useStdInits <- useStdInits
	model$checktime <- TRUE
	model$n3 <- n3
	model$firstg <- firstg
	model$reduceg <- reduceg
	model$maxrat <- 1.0
	model$normSetRates <- FALSE
	model$FinDiff.method <-  findiff
	model$nsub <- nsub
	model$n2start <- n2start
	model$dolby <- (dolby && (!maxlike)&& (!gmm))
	model$splitDepvars <- splitDepvars
	if (diagonalize < 0) {diagonalize <- 0}
	if (diagonalize > 1) {diagonalize <- 1}
	model$diagg <- (diagonalize >= 0.9999)
	model$diagonalize <- diagonalize
	model$randomSeed <- seed
	model$mult <- mult
	model$prML <- prML
	if (length (prML) == 1)
	{
		if (prML <= 1) # old default
#	pridg=0.05, prcdg=0.05, prper=0.2, pripr=0.3, prdpr=0.3,
#	prirms=0.05, prdrms=0.05,
		{
			model$pridg <-  0.05   # insert diagonal
			model$prcdg <-  0.05   # cancel diagonal
			model$prper <-  0.2    # permute
			model$pripr <-  0.3    # insert permute (CCP)
			model$prdpr <-  0.3    # delete permute (CCP)
			model$prirms <- 0.05   # insert random missing
			model$prdrms <- 0.05   # delete random missing
			# prob(move) = 0
		}
		else  # prML == 2
		{
			model$pridg <-  0.05   # insert diagonal
			model$prcdg <-  0.05   # cancel diagonal
			model$prper <-  0      # permute
			model$pripr <-  0.3    # insert permute (CCP)
			model$prdpr <-  0.3    # delete permute (CCP)
			model$prirms <- 0.05   # insert random missing
			model$prdrms <- 0.05   # delete random missing
			# prob(move) = 0.2
		}
	}
	else
	{
		if (length(prML) != 7)
		{
			stop("prML should have length 1 or 7")
		}
		if ((sum(prML)> 1) | (min(prML) < 0))
		{
			stop("prML, if given as a 7-vector, should have nonnegative numbers with sum <= 1")
		}
		model$pridg <- prML[1]    # insert diagonal
		model$prcdg <- prML[2]    # cancel diagonal
		model$prper <- prML[3]    # permute
		model$pripr <- prML[4]    # insert permute (CCP)
		model$prdpr <- prML[5]    # delete permute (CCP)
		model$prirms <- prML[6]   # insert random missing
		model$prdrms <- prML[7]   # delete random missing
	}
	model$maximumPermutationLength <- maximumPermutationLength
	model$minimumPermutationLength <- minimumPermutationLength
	model$initialPermutationLength <- initialPermutationLength
	model$truncation <- truncation
	model$doubleAveraging <- doubleAveraging
	# The following options are used only in phase2.r.
	model$standardizeWithTruncation <- standardizeVar
	model$standardizeVar <- standardizeVar
	# The difference between these two is a hidden, non-documented option,
	# which perhaps could be tried out.
	model$noAggregation <- FALSE
	# This also is a hidden, non-documented option, perhaps for being tried out.
	#  \item{noAggregation}{Logical:
	#   do not replace current parameter value after subphase 1
	#   by the mean over subphase 1, if some quasi-autocorrelation
	#   then is larger than .5. May be helpful if initial value was very far away.
	class(model) <- "sienaAlgorithmSettings"
	attr(model, "version") <- packageDescription(pkgname, fields = "Version")
	model
}


##@set_output_saom ModelCreate
set_output_saom <- function(outputName=NULL, lessMem=FALSE,
	returnThetas=FALSE, 
#	returnDeps=FALSE, Now in ... for siena
	returnChains=FALSE, returnDataFrame=FALSE,
	returnChangeContributions=FALSE)
# Sets output options for SAOM estimation.
# How is it with returnLoglik, onlyLoglik?
# Can be put in ... or here
{
	model <- list()
	model$outputName <- outputName
	model$projname <- outputName
	model$sf2.byIteration <- !lessMem
	model$returnThetas=returnThetas
#	model$returnDeps=returnDeps
	model$returnChains=returnChains
	model$returnDataFrame=returnDataFrame
	if (returnDataFrame & ( !returnChains))
	{
		warning("Specifying returnDataFrame without returnChains has no effect")
	}
	model$returnChangeContributions=returnChangeContributions
	class(model) <- "sienaOutputOptions"
	attr(model, "version") <- packageDescription(pkgname, fields = "Version")
	model
}


##@sienaModelCreate ModelCreate
sienaModelCreate <- function(fn,
	projname="Siena", MaxDegree=NULL, Offset=NULL,
	useStdInits=FALSE,
	n3=1000, nsub=4, n2start = NULL, dolby=TRUE,
	splitDepvars=0,
	maxlike=FALSE, gmm=FALSE, diagonalize=0.2*!maxlike,
	condvarno=0, condname='',
	firstg=0.2, reduceg=0.5, cond=NA, findiff=FALSE,  seed=NULL,
	prML=1,
#	pridg=0.05, prcdg=0.05, prper=0.2, pripr=0.3, prdpr=0.3,
#	prirms=0.05, prdrms=0.05,
	maximumPermutationLength=40,
	minimumPermutationLength=2, initialPermutationLength=20,
	modelType=NULL, behModelType=NULL, mult=5, simOnly=FALSE, localML=FALSE,
	truncation=5, doubleAveraging=0, standardizeVar=(diagonalize<1),
	lessMem=FALSE, silent=FALSE)
{
	model <- NULL
	checking <- any(grepl("_R_CHECK", names(Sys.getenv())))
	fromTransformFunction <- (exists("FromTransformScript", parent.frame()))
# TRUE means that this function was called from TransformScript,
# and then FromTransformScript is TRUE.
# This will lead to output that can be written to file.
	if (fromTransformFunction)
	{
		silent <- TRUE
	}
	if (maxlike && (!is.null(MaxDegree)))
	{
		warning("maxlike and MaxDegree are incompatible")
	}
	if (is.null(projname) | checking)
	{
		model$projname <- tempfile("Siena")
		if (!silent)
		{
			if (checking)
			{
	cat('If you use this algorithm object, siena07 will create/use an output file',
				paste('Siena','.txt',sep=''),'.\n')
			}
				else
			{
	cat('If you use this algorithm object, siena07 will create/use an output file',
				paste(model$projname,'.txt',sep=''),'.\n')
			cat('This is a temporary file for this R session.\n')
			}
		}
	}
	else
	{
		if (is.character(projname))
		{
			model$projname <- projname
			if (!silent)
			{
			cat('If you use this algorithm object, siena07 will create/use an output file',
				paste(model$projname,'.txt',sep=''),'.\n')
			}
		}
		else
		{
			stop('projname should be a character string.')
		}
	}
	model$useStdInits <- useStdInits
	model$checktime <- TRUE
	if (fromTransformFunction)
	{
		model$n3 <- deparse1(substitute(n3))
	}
	else
	{
		model$n3 <- n3
	}
	model$firstg <- firstg
	model$reduceg <- reduceg
	model$maxrat <- 1.0
	model$normSetRates <- FALSE
	model$maxlike <- maxlike
	model$gmm <- gmm
	model$simOnly <- simOnly
	model$localML <- localML
	model$FRANname <- deparse(substitute(fn))
	if (maxlike)
	{
		if (missing(fn))
		{
			model$FRANname <- "maxlikec"
		}
		if (is.na(cond))
		{
			cond <- FALSE
		}
		if (cond)
		{
			stop("Conditional estimation is not possible with",
				"maximum likelihood estimation")
		}
		if (findiff)
		{
			stop("Finite differences estimation of derivatives",
				"is not possible with maximum likelihood estimation")
		}
	}
	else
	{
		if (missing(fn))
		{
			model$FRANname <- "simstats0c"
		}
	}
	model$cconditional <- cond
	if (!is.na(cond) && cond && condvarno == 0 && condname == "")
	{
		model$condvarno <-  1
		model$condname <- ""
	}
	else
	{
		model$condvarno <-  condvarno
		model$condname <- condname
	}
	model$FinDiff.method <-  findiff
	if (fromTransformFunction)
	{
		model$nsub <- deparse1(substitute(nsub))
	}
	else
	{
		model$nsub <- nsub
	}
	if (fromTransformFunction)
	{
		model$n2start <- deparse1(substitute(n2start))
	}
	else
	{
		model$n2start <- n2start
	}
	if (fromTransformFunction)
	{
		model$dolby <- deparse1(substitute(dolby))
	}
	else
	{
		model$dolby <- (dolby && (!maxlike)&& (!gmm))
	}	
	model$splitDepvars <- splitDepvars
	if (diagonalize < 0) {diagonalize <- 0}
	if (diagonalize > 1) {diagonalize <- 1}
	model$diagg <- (diagonalize >= 0.9999)
	model$diagonalize <- diagonalize

	if (!is.null(modelType))
	{
		if (any(!(modelType %in% 1:11)))
		{
			stop('modelType can only have integer values from 1 to 11\n')
		}
		if ((maxlike) & (any(modelType %in% 7:10)))
		{
			stop('Double step model type incompatible with maximum likelihood estimation')
		}
		if (any(is.null(names(modelType))))
		{
			stop('modelType must have names of dependent networks')
		}
		model$modelType <- modelType
	}
	if (!is.null(behModelType))
	{
		if (any(!(behModelType %in% c(1,2))))
		{
			warning('behModelType can only have values 1 or 2; other values changed to 1\n')
			model$behModelType[!(behModelType %in% c(1,2))] <- 1
		}
		if (any(is.null(names(behModelType))))
		{
			stop('behModelType must have names of dependent behavioral variables')
		}
		model$behModelType <- behModelType
	}
	model$MaxDegree <- MaxDegree
	if (!is.null(Offset))
	{
		if (any(is.null(names(Offset))))
		{
			stop('Offset must have names of dependent behavioral variables')
		}
		model$UniversalOffset <- Offset
	}
#	model$randomSeed <- seed
	
	if (fromTransformFunction)
	{
		model$randomSeed <- deparse1(substitute(seed))
	}
	else
	{
		model$randomSeed <- seed
	}
	model$prML <- prML
	if (length (prML) == 1)
	{
		if (prML <= 1) # old default
#	pridg=0.05, prcdg=0.05, prper=0.2, pripr=0.3, prdpr=0.3,
#	prirms=0.05, prdrms=0.05,
		{
			model$pridg <-  0.05   # insert diagonal
			model$prcdg <-  0.05   # cancel diagonal
			model$prper <-  0.2    # permute
			model$pripr <-  0.3    # insert permute (CCP)
			model$prdpr <-  0.3    # delete permute (CCP)
			model$prirms <- 0.05   # insert random missing
			model$prdrms <- 0.05   # delete random missing
			# prob(move) = 0
		}
		else  # prML == 2
		{
			model$pridg <-  0.05   # insert diagonal
			model$prcdg <-  0.05   # cancel diagonal
			model$prper <-  0      # permute
			model$pripr <-  0.3    # insert permute (CCP)
			model$prdpr <-  0.3    # delete permute (CCP)
			model$prirms <- 0.05   # insert random missing
			model$prdrms <- 0.05   # delete random missing
			# prob(move) = 0.2
		}
	}
	else
	{
		if (length(prML) != 7)
		{
			stop("prML should have length 1 or 7")
		}
		if ((sum(prML)> 1) | (min(prML) < 0))
		{
			stop("prML, if given as a 7-vector, should have nonnegative numbers with sum <= 1")
		}
		model$pridg <- prML[1]    # insert diagonal
		model$prcdg <- prML[2]    # cancel diagonal
		model$prper <- prML[3]    # permute
		model$pripr <- prML[4]    # insert permute (CCP)
		model$prdpr <- prML[5]    # delete permute (CCP)
		model$prirms <- prML[6]   # insert random missing
		model$prdrms <- prML[7]   # delete random missing
	}
	model$maximumPermutationLength <- maximumPermutationLength
	model$minimumPermutationLength <- minimumPermutationLength
	model$initialPermutationLength <- initialPermutationLength
	if (exists(deparse1(substitute(mult))))
	{
		model$mult <- mult
	}
	else
	{
		model$mult <- substitute(mult)
	}
	model$truncation <- truncation
	model$doubleAveraging <- doubleAveraging
	model$sf2.byIteration <- !lessMem
	model$standardizeWithTruncation <- standardizeVar
	model$standardizeVar <- standardizeVar
	# The difference between these two is a hidden, non-documented option,
	# perhaps for being tried out
	# by later modification of the sienaAlgorithm object.
	model$noAggregation <- FALSE
	# This also is a hidden, non-documented option, perhaps for being tried out.
	#  \item{noAggregation}{Logical:
	#   do not replace current parameter value after subphase 1
	#   by the mean over subphase 1, if some quasi-autocorrelation
	#   then is larger than .5. May be helpful if initial value was very far away.
	# The two options model$noAggregation and model$standardizeWithTruncation
	# are used only in phase2.r.
	class(model) <- "sienaAlgorithm"
	attr(model, "version") <- packageDescription(pkgname, fields = "Version")
	model
}

##@sienaAlgorithmCreate DataCreate
sienaAlgorithmCreate <- sienaModelCreate

##@ModelTypeStrings DataCreate
ModelTypeStrings <- function(i){
	ifelse(((i >= 1) && (i <= 11) && (!is.null(i))),
		switch(i,
			"Standard actor-oriented model",
			"Forcing model",
			"Initiative model",
			"Pairwise forcing model",
			"Pairwise mutual model",
			"Pairwise joint model",
			"Double Step Model 0.25",
			"Double Step Model 0.50",
			"Double Step Model 0.75",
			"Double Step Model 1.00",
			"Contemporaneous evaluation statistics model"), "")
}

##@BehaviorModelTypeStrings DataCreate
BehaviorModelTypeStrings <- function(i){
	ifelse(((i %in% 1:2) && (!is.null(i))),
		switch(i,
			"Standard behavior actor-oriented model ('restrict')",
			"Boundary-absorbing behavior model"),
		"")
}

##@getTargets Miscellaneous Written for Krista. Use as RSiena:::getTargets
getTargets <- function(data, effects) {
    setup <- sienaSetupForCpp(data, effects,
                              includeBehavior = TRUE,
                              includeBipartite = TRUE,
                              returnActorStatistics = FALSE,
                              returnStaticChangeContributions = FALSE,
                              parallelrun = FALSE)
    ans <- .Call(C_getTargets, PACKAGE=pkgname,
                 setup$pData, setup$pModel, setup$myeffects,
                 parallelrun = FALSE,
                 returnActorStatistics = FALSE,
                 returnStaticChangeContributions = FALSE)
    ans
}

##@actorTargets Calculates the actor statistics at a wave (which cannot be
## the last wave), for effects associated with a specified behavior.  Optionally
## replaces the actual data for the wave for this behaviour with imputedData.
## For use with multiple imputation of behavior data.
actorTargets <- function(data, effects, behaviorName, wave,
                         imputedData = NULL, rate = FALSE)
{
    depvarsNo <- length(data$depvars)
    behNo <- c(1:depvarsNo)[names(data$depvars) == behaviorName]
    beh <- data$depvars[[behNo]][,1,]
    n <- dim(data$depvars[[behNo]])[1]
    waves <- dim(data$depvars[[behNo]])[3]

    effects <- effects[effects$include,]
    effects$setting <- rep("", nrow(effects))
    effects <- effects[effects$name == behaviorName,]
    effects <- effects[!effects$basicRate,]
    noEff <- dim(effects)[1]
    ans2 <- array(NA, dim = c(n, noEff))

    w <- wave
    vars <- data$depvars
    if (!is.null(imputedData)) {
        vars[[behNo]][,,w] <- imputedData
        beh[,w] <- imputedData
    }

    for (j in 1:n) {
        if (rate) {
            data$depvars[[behNo]][,1,] <- beh
            data$depvars[[behNo]][,1, setdiff(1:waves, w)] <- NA
            data$depvars[[behNo]][j,1,(w+1):waves] <- rep(1, waves-w)
            data$depvars[[behNo]][j,1,1:w] <- rep(0, w)
        } else {
            for (k in 1:depvarsNo) {
                data$depvars[[k]][,,1] <- vars[[k]][,,waves]
                data$depvars[[k]][,,2:waves] <- vars[[k]][,,1:(waves-1)]
            }
            data$depvars[[behNo]][,1, setdiff(1:waves, w+1)] <- NA
            data$depvars[[behNo]][j,1, setdiff(1:waves, w+1)] <- 0
        }

        # Use sienaSetupForCpp for setup
        setup <- sienaSetupForCpp(data, effects,
                                  includeBehavior = TRUE,
                                  includeBipartite = FALSE,
                                  returnActorStatistics = FALSE,
                                  returnStaticChangeContributions = FALSE,
                                  parallelrun = FALSE)
        ans <- .Call(C_getTargets, PACKAGE=pkgname,
                     setup$pData, setup$pModel, setup$myeffects,
                     parallelrun = FALSE,
                     returnActorStatistics = FALSE,
                     returnStaticChangeContributions = FALSE)
        ans2[j,] <- ans[,w]
    }

    list(effects = effects$effectName, effectType = effects$type, targets = ans2)
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: RSienaRDocumentation.r
# *
# * Description: This module contains the code for documenting the
# * RSiena R source.
# *****************************************************************************/
##
##@getInternals Documentation
getInternals <- function()
{   
    fnlist <- utils::read.csv("RSienafnlist.csv", as.is=TRUE)
    mylist <- ls(parent.frame())
    ##  print(mylist)
    ## require(codetools)
	## fnlist has a row number at the front: column 3 contains the function names
    mylist <- mylist[mylist %in% fnlist[, 3]]
    mytt <- lapply(mylist, function(x)
       {
           x <- get(x, envir=parent.frame(3))
           if (is.function(x))
           {
               tt <- codetools::findGlobals(x, merge=FALSE)[[1]]
               tt2 <- codetools::findLocals(body(x))
               tt <- c(tt, tt2)
               tt[tt %in% fnlist[, 3]]
           }
           else
           {
                 NULL
           }
       }
           )
    names(mytt) <- mylist
    mytt
}
##@getRSienaDocumentation Documentation
getRSienaRDocumentation <- function(Rdir)
{
	## require(xtable)
	## require(codetools)

    thisdir <- getwd()
    ## temporarily move directory
	on.exit(setwd(thisdir))
    setwd(Rdir)

    ## extract comment lines
    if (.Platform$OS.type == "windows")
    {
        shell('grep "##@" *.r *.R > comments.lis')
    }
    else
    {
        system('grep "##@" *.r *.R > comments.lis')
    }
    ## read them in
    comms <- readLines('comments.lis')
    ## remove the file
    file.remove("comments.lis")
    ## remove the shell line
    comms <- comms[!grepl("comments.lis", comms)]
	## get rid of tabs
	comms <- gsub("\t", "", comms)
    ## split off
    mystr <- paste("##", "@", sep="")
    comms1 <- strsplit(comms, mystr)
    ## join up rest
    ## comms2 <- do.call(rbind, comms1)
    ## turn into dataframe
    comms3 <- sapply(comms1, function(x)
                 {
                     tmp <- strsplit(x[2], " ")[[1]]
                     if (tmp[2] == "internal")
                     {
                         c(x[1], tmp[1], tmp[2], paste('internal to', tmp[3],
                                                       collapse=" "))
                     }
                     else
                     {
                         c(x[1], tmp[1], tmp[2], paste(tmp[-c(1, 2)],
                                                       collapse=" "))
                     }
                 }
                     )
    comms3  <-  t(comms3)

    ## get the calls (global)
    codet <- lapply(comms3[,2], function(x)
                {
                    x <- try(getFromNamespace(x, pkgname), silent=TRUE)
                    if (is.function(x))
                    {
                        tmp1 <- codetools::findGlobals(x, merge=FALSE)[[1]]
                        tmp2 <- codetools::findLocals(body(x))
                        tmp <- c(tmp1, tmp2)
                    }
                    else
                        tmp <- NULL
                    unique(as.vector(tmp[tmp %in% comms3[,2]]))
                }
                    )
    names(codet) <- comms3[, 2]

    ## now the internal ones
    ## find the list of files from comms3
#	browser()
	internals <- comms3[grepl("internal to", comms3[, 4]), ]
    ttmp <- unique(comms3[grepl("internal to", comms3[, 4]), 4])
    ttmp1 <- sub("internal to ", "", ttmp)
	ttmp3 <- lapply(ttmp1, function(x)
				{
					tmp <- x
					while(!is.na(sub <- match(tmp[1], internals[, 2])))
					{
						tmp1 <- internals[sub, 4]
						tmp1 <- sub("internal to ", "", tmp1)
						tmp <- c(tmp1, tmp)
					}
					tmp
				})
    #ttmp2 <- comms3[match(ttmp1, comms3[, 2]), 1]
    #ttmp2 <- sub(":", "", ttmp2)
    ## write out the fnlist in the Rdir
    write.csv(data.frame(comms3), "RSienafnlist.csv")
    ## get the list of internals
	tt <- lapply(ttmp3, function(x)
			 {
				 yy <- getFromNamespace(x[1], pkgname)
                 targs <- formals(yy)
				 targs[1:length(targs)] <- 1
				 if (length(x) > 1)
				 {
					 targs['getDocumentation'] <- x[-1]
				 }
				 else
				 {
					 targs['getDocumentation'] <- TRUE
				 }
                 do.call(yy, targs)
			 })
    #tt <- lapply(1:length(ttmp3), function(x, y)
    #         {
    #             yy <- y[x]
    #             yy <- getFromNamespace(yy, pkgname)
    #             targs <- formals(yy)
    #             n <- length(targs)
    #             myargs <- targs
    #             for (i in 1:n)
#				 {
#                     myargs[[i]] <- 1#
				 #}
                 #myargs['getDocumentation'] <- TRUE
                 #do.call(yy, myargs)
             #}, y=ttmp)
    names(tt) <- ttmp
    ## remove the file
    file.remove("RSienafnlist.csv")
    ## reformat this
    ttt <- lapply(1:length(tt), function(x,y)
              {
                  yy <- y[[x]]
                  n <- length(y[[x]])
                  bb <- names(yy)
                  t1<- lapply(1:n, function(x,  b, a)
                          {
                              y <- a[[x]]
                              bb <- b[[x]]
                              n <- length(y)
                              if ( n > 0)
                                  cbind( rep(bb, n), y)
                              else
                                  c( bb, " ")
                          },  a=yy, b=bb)
                  do.call(rbind,t1)
              }, y=tt
                  )

    tttt <- as.data.frame(do.call(rbind,ttt))
    names(tttt) <- c('Function', 'Calls')

    ## create an object that will tabify to the right output
    tmp2 <- codet

    tmp4 <- lapply(1 : length(tmp2), function(x, y, z, a)
               {
                   n <- length(y[[x]])
                   if (n > 0)
                   {
                       cbind( rep(a[x, 1], n), rep(z[x], n),  y[[x]],
                             rep(a[x, 3], n), rep(a[x, 4], n))
                   }
                   else
                   {
                       cbind(a[x, 1], z[x], " ", a[x, 3], a[x, 4])
                   }

               }, y=tmp2, z=names(tmp2), a=comms3)

    tmp5 <- do.call(rbind, tmp4)
    tmp5 <- as.data.frame(tmp5, stringsAsFactors=FALSE)
    names(tmp5) <- c('Source File', 'Function', 'Calls', 'Type', 'Notes')

    ## now merge in the internals
    tmp5bit <- tmp5[tmp5$Function %in%tttt$Function,]
    tmerge <- merge(tmp5bit, tttt, by="Function")
    tmerge <- tmerge[, -3]
    tmerge <- tmerge[, c(2, 1, 5, 3, 4)]
    names(tmerge)[3] <- "Calls"
    tmp5new <- rbind(tmp5[!tmp5$Function %in% tttt$Function,], tmerge)
    tmp55 <- split(tmp5new, tmp5new$Function)

    ## same for called by
    tmp6 <- lapply(1 : length(tmp2), function(x, y, z)
               {
                   n <- length(y[[x]])
                   if (n > 0)
                   {
                       cbind( rep(z[x], n),  y[[x]] )
                   }
                   else
                   {
                       cbind(z[x], " ")
                   }

               }, y=tmp2, z=names(tmp2))

    tmp7 <- do.call(rbind, tmp6)

    tmp7 <- as.data.frame(tmp7, stringsAsFactors=FALSE)
    names(tmp7) <- c( 'Called from', 'Function')

    tmp7 <- tmp7[order(tmp7[,2],tmp7[,1]), ]

    tttt7 <- tttt
    names(tttt7) <- c("Called from", "Function")
    tttt7 <- tttt7[order(tttt7[,2],tttt7[,1]), ]

   ## tmp7bit <- tmp7[tmp7$Function %in% tttt7$Function, ]

    tmp7new <- merge(tmp7, tttt7, by=c("Function", "Called from"), all=TRUE)

    tmp7new <- tmp7new[order(tmp7new[,1], tmp7new[,2]),]

    tmp77new <- split(tmp7new, tmp7new$Function)

    tmp77new <- tmp77new[-1]

    ## create desired output format
    tmp11 <- lapply(1:length(names(tmp55)), function(x,y,z)
                {
                    thisone <- names(tmp55)[x]
                    yy <- y[[thisone]]
                    zz <- z[[thisone]]
                    d <- max(nrow(yy), nrow(zz))
                    fn <- yy$Function[1]
                    src<- yy$`Source File`[1]
                    type<- yy$Type[1]
                    notes<- yy$Notes[1]
                    if (!is.null(zz))
                    {
                        called <- c(zz[,2], rep(' ', d-nrow(zz)))
                    }
                    else
                    {
                        called <- rep(' ', d)
                    }
                    tmp <- data.frame(src=rep(src,d),
                                      fun=rep(fn, d),
                                      type=rep(type, d),
                                      notes=rep(notes, d),
                                      calls=c(yy[,3], rep(' ', d-nrow(yy))),
                                      called=called, stringsAsFactors=FALSE)
                    tmp
                }, y=tmp55, z=tmp77new)
    ## join into a data frame
    tmp12 <- do.call(rbind, tmp11)
    names(tmp12)[2] <- "Function"

    tmp12 <- tmp12[order(tmp12[, "type"], as.numeric(row.names(tmp12))), ]
    tmp12 <- tmp12[, c(3, 2, 5, 6, 4, 1)]
    ff <- xtable::xtable(tmp12)
    ## go back to start directory
    setwd(thisdir)
    print(ff, tabular.environment="longtable",
          file="RSienaRDocumentation.tex", floating=FALSE)

    write.csv(tmp12, "RSienaRDocumentation.csv")
}

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: checkImpossibleChanges.r
## *
## * Description: This file contains the function checkImpossibleChanges
## * which checks for impossible changes from structural values to
## * different observed values.
## * Used for maximum likelihood and Bayesian estimation.
## *
## ****************************************************************************/
##args:x: Siena data object
## checkImpossibleChanges returns 1*I{any structural zero changed to 1} +
##                          2*I{any structural one changed to 0} +
##       10*I{any structural zero changed to NA and then changed to 1} +
##       20*I{any structural one  changed to NA and then changed to 0} 


notNetwork <- function(depv){
	ifelse(length(dim(depv))<= 2, TRUE, 			
		(!(attr(depv, 'type') %in% c("oneMode", "bipartite"))))
}

##@checkImpossibleChanges checks for likelihood calculations
checkImpossibleChanges <- function(x)
{
	if (!(inherits(x, "sienadata") | inherits(x, "siena")))
	{
		stop('checkImpossibleChanges can only be applied to siena data objects')
	}
	xd <- x$depvars
	impossibleChangeOne <- function(dv)
	{
		ifelse(notNetwork(dv), 0, 
			1*any(sapply((2:dim(dv)[3]),
					FUN=function(i){any((dv[,,i-1] == 10) &
					                    (dv[,,i] %in% c(1,11)), na.rm=TRUE)})))
	}
	impossibleChangeZero <- function(dv)
	{
		ifelse(notNetwork(dv), 0, 
			1*any(sapply((2:dim(dv)[3]),
					FUN=function(i){any((dv[,,i-1] == 11) &
					                  (dv[,,i] %in% c(0,10)), na.rm=TRUE)})))
	}
	impossibleChangeNAOne <- function(dv)
	{
		ifelse(notNetwork(dv), 0, 
			ifelse((dim(dv)[3] >= 3),
			1*any(sapply((3:dim(dv)[3]),
					FUN=function(i){
		any((dv[,,i-2] == 10) & (is.na(dv[,,i-1])) & (dv[,,i] %in% c(1,11)),
								na.rm=TRUE)})), 0))
					
	}
	impossibleChangeNAZero <- function(dv)
	{
		ifelse(notNetwork(dv), 0, 
			ifelse((dim(dv)[3] >= 3),
			1*any(sapply((3:dim(dv)[3]),
					FUN=function(i){
		any(((dv[,,i-2] == 11) & (is.na(dv[,,i-1])) &
					(dv[,,i] %in% c(0,10))), na.rm=TRUE)})), 0))					
	}
# This is not complete; see the manual;
# it does not check for patterns 10, NA, ..., NA, 1
# where ... are all NA; neither for patterns 11, NA, ..., NA, 0
	max(sapply(xd,impossibleChangeOne)) + 2*max(sapply(xd,impossibleChangeZero)) +
			10*max(sapply(xd,impossibleChangeNAOne)) + 
			20*max(sapply(xd,impossibleChangeNAZero))
}


##@checkZeroChanges checks for likelihood calculations
checkZeroChanges <- function(x)
{
	if (!(inherits(x, "sienadata") | inherits(x, "siena")))
	{
		stop('checkZeroChanges can only be applied to siena data objects')
	}
	zeroChange <- function(dv){
		if (attr(dv, 'type') %in% c("oneMode", "bipartite"))
		{
			dv[dv==10] <- 0
			dv[dv==11] <- 1
		}
		ifelse((length(dim(dv))==3),
			1*any(sapply((2:dim(dv)[3]),
					FUN=function(i){all(dv[,,i-1] == dv[,,i], na.rm = TRUE)})), 0)
	}
	sum(sapply(x$depvars,zeroChange))
}

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: sienaprint.r
## *
## * Description: This file contains the print and summary modules for the
## * classes siena, sienaFit, sienaDependent, and sienaAlgorithm
## * Note: $requestedEffects used everywhere instead of $effects,
## * because the latter also includes all main effects corresponding to
## * included interaction effects,
## * even if these main effects were not requested.
## *
## ****************************************************************************/
##@print.sienadata Methods
print.sienadata <- function(x, ...)
{
    ##@onlys internal print.siena; prints matrix attributes of x$depvar
	onlys <- function(x, attrib){
	# function to print the attribute attrib (character string)
	# of object x$depvars
	# This attribute must be a logical matrix
	# with named columns, and rows indicating periods.
		textattr <- deparse(substitute(attrib),  backtick=FALSE)
		uponlys <- as.matrix(sapply(x$depvars, function(y){attr(y,attrib)}))
		if (x$observations == 2)
		# only one period
		{
			uponlys <- t(uponlys)
		}
		# now rows are periods; columns are dependent variables
		for (j in seq_len(dim(uponlys)[2]))
		{
			if (any(uponlys[,j]))
			{
				cat(textattr)
				cat(":  ", colnames(uponlys)[j], ":    ", sep="")
				if (all(uponlys[,j]))
				{
					cat("all periods\n")
				}
				else
				{
					word <-
						ifelse((sum(uponlys[,j]) <= 1), "period ", "periods ")
					cat(word)
					cat(which(uponlys[,j]), sep=", ")
					cat("\n")
				}
			}
		}
	}
# begin main method siena.print proper
	if ((!inherits(x, "sienadata")) & (!inherits(x, "siena")))
	{
        stop("not a legitimate Siena data object")
	}
	cat('Dependent variables: ', paste(names(x$depvars), collapse=", "), "\n")
	cat('Number of observations:', x$observations, "\n\n")
	if (length(x$compositionChange) > 0)
	{
		cat('With composition change.\n')
	}
	if (!is.null(x$nodeSet))
	{
		tmp <- sapply(x$nodeSet, length)
		if (length(x$nodeSet) <= 1)
		{
			tmp <- c('Nodeset        ' = 'Number of nodes', tmp)
		}
		else
		{
			tmp <- c('Nodesets       ' = 'Number of nodes', tmp)
		}
		print(tmp, quote=FALSE)
		cat("\n")
	}

	has.set <- hasSettings(x)
	for (j in (1:length(x$depvars)))
	{
		xj <- x$depvars[[j]]
		mymat <- matrix("", 7, 2)
		mymat[1,] <- c("Dependent variable", attr(xj, "name"))
		mymat[2,] <- c("Type",               attr(xj, "type"))
		if (attr(xj,"type") == "oneMode")
		{
			if (attr(xj,"symmetric"))
			{
				mymat[2,2] <- paste(mymat[2,2],", symmetric", sep="")
			}
		}
		mymat[3,] <- c("Observations",       attr(xj,  "netdims")[3])
		if (attr(xj, "type") == "bipartite")
		{
			mymat[4,] <- c("First nodeset ", attr(xj, "nodeSet")[1])
			mymat[5,] <- c("Second nodeset ",attr(xj, "nodeSet")[2])
			nrows <- 5
		}
		else
		{
			mymat[4,] <- c("Nodeset ", attr(xj, "nodeSet"))
			nrows <- 4
		}
		if (attr(xj, "type") %in% c("behavior", "continuous"))
		{
			mymat[nrows+1,] <- c("Range",
				paste(signif(attr(xj, "range2"), 4),  collapse=" - "))
		}
		else
		{
			mymat[nrows+1,] <- c("Densities",
				paste(signif(attr(xj, "density"), 2),  collapse=" "))
		}
		mymat2 <- apply(mymat[0:nrows+1,], 2, format)
		write.table(mymat2, row.names=FALSE, col.names=FALSE, quote=FALSE)
		if (has.set[j]){
			cat("\nSettings\n")
			dts <- describeTheSetting(xj)
			dts <- matrix(c(colnames(dts), t(dts)), 3, 5, byrow=TRUE)
			mymat3 <- apply(dts[,1:5], 2, format)
			write.table(mymat3, row.names=FALSE, col.names=FALSE, quote=FALSE)
		}
		cat("\n")
	}

	if (length(x$cCovars) > 0)
	{
		cat('Constant covariates: ', paste(names(x$cCovars), collapse=", "), "\n")
	}
	if (length(x$vCovars) > 0)
	{
		cat('Changing covariates: ',
			paste(names(x$vCovars), collapse = ", "), "\n")
	}
	if (length(x$dycCovars) > 0)
	{
		cat('Constant dyadic covariates: ',
			paste(names(x$dycCovars), collapse=", "), "\n")
	}
	if (length(x$dyvCovars) > 0)
	{
		cat('Changing dyadic covariates: ',
			paste(names(x$dyvCovars), collapse=", "), "\n")
	}

	onlys(x, 'uponly')
	onlys(x, 'downonly')

	attrs <- attributes(x)
	higher <- attrs[["higher"]]
	disjoint <- attrs[["disjoint"]]
	atleastone <- attrs[["atLeastOne"]]
	if (any(higher))
	{
		higherSplit <- strsplit(names(higher)[higher], ",")
		report <- sapply(higherSplit, function(x)
		   {
			   paste(c("Network ", x[1], " is higher than network ", x[2],
						".\n"), sep="")
		  })
		message("\n", report)
		message("This will be respected in the simulations. \n")
		message("If this is not desired, change attribute 'higher'.\n")
	}
	if (any(disjoint))
	{
		disjointSplit <- strsplit(names(disjoint)[disjoint],',')
		report <- sapply(disjointSplit, function(x)
		   {
			   paste(c("Network ", x[1], " is disjoint from network ",
						x[2], ".\n"), sep="")
		  })
		message("\n", report)
		message("This will be respected in the simulations. \n")
		message("If this is not desired, change attribute 'disjoint'.\n")
	}
	if (any(atleastone))
	{
		atLeastOneSplit <- strsplit(names(atleastone)[atleastone],',')
		report <- sapply(atLeastOneSplit, function(x)
		   {
			   paste(c("A link in at least one of networks ",
						x[1], " and", x[2],
					   " always exists.\n"), sep="")
		  })
		message("\n", report)
		message("This will be respected in the simulations. \n")
		message("If this is not desired, change attribute 'atLeastOne'.\n")
	}
	invisible(x)
}
##@print.sienaGroup Methods
print.sienaGroup <- function(x, ...)
{
	if (!inherits(x, "sienaGroup"))
	{
        stop("not a legitimate Siena group data object")
	}
	att <- attributes(x)
	cat('Dependent variables: \n')
	cat(paste(att$netnames, ":", att$types,'\n'))
	cat('Total number of groups:', length(x),'\n')
	cat('Total number of periods:', att$observations,'\n')
	if (length(x[[1]]$vCovars) > 0)
	{
		cat('Changing covariates: ',
			paste(names(x[[1]]$vCovars), collapse = ", "), "\n")
	}
	if (length(x[[1]]$dyvCovars) > 0)
	{
		cat('Changing dyadic covariates: ',
			paste(names(x[[1]]$dyvCovars), collapse=", "), "\n")
	}
	cat("\n")
	invisible(x)
}

print.sienaDependent <- function(x, ...)
{
	if (!inherits(x, "sienaDependent"))
	{
        stop("not a legitimate Siena dependent variable object")
	}
	mymat <- matrix("", 4, 2)
	mymat[1,] <- c("Type",               attr(x, "type"))
	mymat[2,] <- c("Observations",       attr(x,  "netdims")[3])
	if (attr(x, "type") == "bipartite")
	{
		mymat[3,] <- c("First nodeset ",
						paste(attr(x, "nodeSet")[1], " (", attr(x, "netdims")[1],
							" elements)", sep=""))
		mymat[4,] <- c("Second nodeset ",
						paste(attr(x, "nodeSet")[2], " (", attr(x, "netdims")[2],
							" elements)", sep=""))
		nrows <- 4
	}
	else
	{
		mymat[3,] <- c("Nodeset ",
						paste(attr(x, "nodeSet"), " (", attr(x, "netdims")[1],
							" elements)", sep=""))
		nrows <- 3
	}

	mymat <- apply(mymat, 2, format)
	write.table(mymat[1:nrows,], row.names=FALSE, col.names=FALSE, quote=FALSE)
	cat("\n")
	invisible(x)
}

##@print.sienaFit Methods
print.sienaFit <- function(x, tstat=TRUE, ...)
{
	objectName <- deparse(substitute(x))
# Check if it looks like a pipe chain 
	if (grepl("%>%|\\|>", objectName)) 
	{
		stop("Cannot auto-generate filename from piped data. Please apply this print to an object with a name.") 
	}
	if (!inherits(x, "sienaFit"))
	{
        stop("not a legitimate Siena model fit")
	}
	if (!x$OK)
	{
		message("Error end of estimation algorithm\n")
	}
	else if (x$termination == "UserInterrupt")
	{
		message("User interrupted run, object possibly incomplete\n")
	}
	else
	{
		if (is.null(x$thetaFromFile))
		{
			x$thetaFromFile <- FALSE
		}
		if(x$x$simOnly)
		{
		cat("Parameter values used for simulations\n\n")
		}
		else
		{
			if (gmm(x))
			{
				cat("Estimated by Generalized Method of Moments\n\n")
			}
			if (x$maxlike)
			{
				cat("Estimated by Maximum Likelihood\n\n")
			}
		cat("Estimates, standard errors and convergence t-ratios\n\n")
		}
		tmp <- sienaFitThetaTable(x, tstat=tstat)
		mydf <- tmp$mydf
		mymat <- as.matrix(mydf)
		mymat[, 'value'] <- format(round(mydf$value, digits=4))
		if(x$x$simOnly)
		{
			mymat[, 'tstat'] <- ' '
		}
		else
		{
			mymat[, 'tstat'] <- paste(" ", format(round(mydf$tstat, digits=4)))
			mymat[is.na(mydf$tstat), 'tstat'] <- ' '
		}
		mymat[, 'se'] <- format(round(mydf$se, digits=4))
		mymat[, 'type'] <- format(mymat[, 'type'])
		mymat[, 'text'] <- format(mymat[, 'text'])
		mymat[mydf$row < 1, 'row'] <-
			format(mydf[mydf$row < 1, 'row'])
		mymat[mydf[,'row'] >= 1, 'row'] <-
			paste(format(mydf[mydf$row >= 1, 'row']), '.', sep='')
		if(x$x$simOnly)
		{
			if (x$thetaFromFile)
			{
				mymat <- rbind(c(rep("", 4), "Mean", "", "Standard", "",
						 ""),
					   c(rep("", 4), "  value", "", "Deviation", rep("", 2)), mymat)
			}
			else
			{
				mymat <- rbind(c(rep("", 4), "Parameter", "", "Standard", "",
						 ""),
					   c(rep("", 4), "  value", "", "Deviation", rep("", 2)), mymat)
			}
		}
		else
		{
			mymat <- rbind(c(rep("", 4), "Estimate", "", "Standard", "",
						 "Convergence"),
					   c(rep("", 6),  "  Error", "", "  t-ratio"), mymat)
		}
		mymat <- apply(mymat, 2, format)
		tmp1 <- apply(mymat, 1, function(x) paste(x, collapse=" "))
		addtorow <- tmp$addtorow
		for (i in 1:length(tmp1))
		{
			if (length(addtorow$command) > 0)
			{
				for (j in 1:length(addtorow$command))
				{
					ii <- match(i-1, addtorow$pos[[j]])
					if (!is.na(ii))
						if (i == 2 | addtorow$command[j] == 'Network Dynamics')
							cat( addtorow$command[j], '\n')
						else
							cat('\n', addtorow$command[j], '\n', sep='')
				}
			}
			cat(tmp1[i], '\n')
		}

		if(x$x$simOnly)
		{
			cat('\nSimulated means and standard deviations')
			if (x$thetaFromFile)
			{
				cat('\n')
			}
			else
			{
				cat(', standard errors of the mean \n')
			}
			if (x$x$maxlike)
			{
				cat('Note: statistics for ML simulations are score functions.\n')
			}
			dmsf <- diag(x$msf)
			mean.stats <- colMeans(x$sf) + x$targets
#  cov.dev may be dropped
#			cov.dev <- x$msf
			sem <- sqrt(dmsf/dim(x$sf)[1])
			if ((x$x$dolby) & (!x$thetaFromFile))
			{
				scores <- apply(x$ssc, c(1,3), sum)	 # x$nit by x$qq matrix
				mean.scores <- colMeans(scores)
				mean.stats <- mean.stats - (x$regrCoef * mean.scores)
				sem <- sem*sqrt(1 - (x$regrCor)^2)
			}
# mean.stats is exactly the same as x$estMeans
			if (gmm(x))
			{
				selection <- which(x$requestedEffects$type=='gmm' |
								x$requestedEffects$type=='rate')
			}
			else
			{
				selection <- 1:nrow(x$requestedEffects)
			}
			mymess1 <- paste(format(1:x$qq,width=3), '. ',
					format(x$requestedEffects$functionName[selection], width = 56),
					format(round(mean.stats, 3), width=8, nsmall=3), ' ',
				format(round(sqrt(dmsf), 3) ,width=8, nsmall=3), ' ', sep='')
			if (x$thetaFromFile)
			{
				mymess1 <- paste(mymess1, rep('\n',x$qq), sep='')
			}
			else
			{
				mymess1 <- paste(mymess1,
					format(round(sem, 4) ,width=8, nsmall=4),
					rep('\n',x$qq), sep='')
			}
			cat(as.matrix(mymess1),'\n', sep='')
			cat("\nSimulated statistics are in ", objectName,'$sf',sep="")
			if (x$returnDeps)
			{
				cat("\nand simulated dependent variables in ", objectName,
					'$sims, where ', objectName,' is the created object.\n',sep="")
			}
			else
			{
				cat(", where", objectName, "is the created object.\n")
			}
		}
		else
		{
			cat(c("\nOverall maximum convergence ratio: ",
					sprintf("%8.4f", x$tconv.max), "\n\n"))
		}
		if (sum(x$test) > 0)
		{
			sc.t <- score.Test(x)
			letter <- ifelse(sc.t$df == 1, '', 's')
			cat('\nScore test for ', sc.t$df, ' parameter', letter, ':\n',
			    'chi-squared = ', round(sc.t$chisquare,2),
				', p = ', sprintf("%6.4f", sc.t$pvalue),'.\n', sep='')
		}

		if (any(x$x$MaxDegree > 0)) {
			cat('\nDegrees constrained to maximum values:\n')
			for (i in 1:length(x$x$MaxDegree)){
					cat(names(x$x$MaxDegree)[i],':',x$x$MaxDegree[i],'\n')}
			cat('\n')
		}
		if (any(x$x$UniversalOffset > 0))
		{
	cat(' Offsets for symmetric networks, and for settings model (if any): \n')
			for (i in 1:length(x$x$UniversalOffset))
			{
				cat(names(x$x$UniversalOffset)[i],':',x$x$UniversalOffset[i],'\n')
			}
		}
		if (any(x$x$modelType != 1))
			{
			cat('\n Model Type:\n')
				for (i in 1:length(x$x$modelType))
				{
					cat(names(x$x$modelType)[i],':',
							ModelTypeStrings(x$x$modelType[i]),'\n')
				}
			cat('\n')
		}
		if (any(x$x$behModelType != 1))
			{
			cat('\n Behavioral Model Type:\n')
				for (i in 1:length(x$x$behModelType))
				{
					cat(names(x$x$behModelType)[i],':',
						BehaviorModelTypeStrings(x$x$behModelType[i]),'\n')
				}
			cat('\n')
		}

		try(if (x$errorMessage.cov > '')
				{warning('\nWarning:', x$errorMessage.cov, '\n')}, silent=TRUE)
			# "Try" for compatibility with previous versions
		cat("\nTotal of", x$n, "iteration steps.\n\n")

		if ((x$x$dolby) & (x$x$simOnly) & (!x$thetaFromFile))
		{
			cat('(Standard errors of the mean are less than s.d./',
						sqrt(x$n),' \n', sep='')
		cat('because of regression on scores (Dolby option).) \n')
		}
		if (x$termination == "UserInterrupt")
		{
			warning(" \n*** Warning ***",
				"Estimation terminated early at user request.\n")
	}
	}
	invisible(x)
}

##@summary.sienaFit Methods
summary.sienaFit <- function(object, ...)
{
	if (!inherits(object, "sienaFit"))
	{
		stop("not a legitimate Siena model fit")
	}
	class(object) <- c("summary.sienaFit", class(object))
	object
}

##@print.summary.sienaFit Methods
print.summary.sienaFit <- function(x, matrices=TRUE, ...)
{
	if (!inherits(x, "summary.sienaFit"))
	{
        stop("not a legitimate summary of a Siena model fit")
	}
	print.sienaFit(x)

    if (x$maxlike)
    {
        cat('Autocorrelations during phase 3 : \n')
        sfl <- apply(x$sf, 2,
                       function(y)acf(y, plot=FALSE, lag.max=1)[[1]][[2]])
        cat(paste(format(1:x$pp, width=3), '. ',
                     format(sfl, width=8, digits=4),
                     '\n', collapse="", sep=""))
        cat ('\n')
    }

	if (sum(x$test) > 0) ## we have some score tests
	{
		testn <- sum(x$test)
		if (x$maxlike)
		{
			cat("Score test <c>\n\n")
		}
		else
		{
			cat("Generalised score test <c>\n\n")
		}
		cat("Testing the goodness-of-fit of the model restricted by\n")
		j <- 0
		for (k in 1:x$pp)
		{
			if (x$test[k])
			{
				j <- j + 1
				cat(c(" (", j, ")   ",
					  format(paste(x$requestedEffects$type[k], ":  ",
								   x$requestedEffects$effectName[k],
								   sep=""),
							 width=50), " = ",
					  sprintf("%8.4f", x$theta[k]),"\n"),
					sep = "")
			}
		}
		cat("_________________________________________________\n")
		cat("                ")
		cat("   \n")
		if (testn > 1)
		{
			cat('Joint test:\n-----------\n')
		}
		cat(c('   c = ',sprintf("%8.4f", x$testresOverall),
			  '   d.f. = ',j,'   p-value '), sep='')
		pvalue <- 1 - pchisq(x$testresOverall, j)
		if (!is.na(pvalue))
		{
			if (pvalue < 0.0001)
			{
				cat('< 0.0001\n')
			}
			else
			{
				cat(c('= ', sprintf("%8.4f\n", pvalue)), sep = '')
			}
		}
		else
		{
			Report('  NA  ',outf)
		}
		if (testn==1)
		{
			cat(c('\n   one-sided (normal variate): ',
					sprintf("%8.4f", x$testresulto[1])), sep = '')
		}
		if (testn> 1)
		{
			cat('\n\n')
			for (k in 1:j)
			{
				cat(c('(', k, ') tested separately:\n'), sep='')
				cat('-----------------------\n')
				cat(' - two-sided:\n')
				cat(c('  c = ', sprintf("%8.4f", x$testresult[k]),
						'   d.f. = 1  p-value '), sep = '')
				pvalue<- 1-pchisq(x$testresult[k],1)
				if (!is.na(pvalue))
				{
					if (pvalue < 0.0001)
					{
						cat('< 0.0001\n')
					}
					else
					{
						cat(c('= ', sprintf("%8.4f", pvalue), '\n'), sep = '')
					}
				}
				else
				{
					Report('  NA  ',outf)
				}
				cat(c(' - one-sided (normal variate): ',
						sprintf("%8.4f", x$testresulto[k])), sep = '')
				if (k < j)
				{
					cat('\n\n')
				}
			}
		}
		cat('    \n_________________________________________________\n\n')
		cat('One-step estimates: \n\n')
		for (i in 1 : x$pp)
		{
			onestepest <- x$oneStep[i] + x$theta[i]
			cat(c(format(paste(x$requestedEffects$type[i], ':  ',
							x$requestedEffects$effectName[i], sep = ''),
						width=50),
					sprintf("%8.4f", onestepest), '\n'), sep = "")
		}
		cat('\n')
	}
	if ((matrices)&(x$OK)&(!is.null(x$covtheta)))
	{
		cat("Covariance matrix of estimates (correlations below diagonal)\n\n")
		covcor <- x$covtheta
		correl <- x$covtheta / sqrt(diag(x$covtheta))[row(x$covtheta)] /
			sqrt(diag(x$covtheta))[col(x$covtheta)]
		covcor[lower.tri(covcor)] <- correl[lower.tri(correl)]
		printMatrix(format(round(t(covcor),digits=3),width=12))
		cat("\nDerivative matrix of expected statistics X by parameters:\n\n")
		if (gmm(x))
		{
			printMatrix(format(round(x$gamma,digits=3),width=12))
		}
		else
		{
			printMatrix(format(round(x$dfra,digits=3),width=12))
		}
		cat("\nCovariance matrix of X (correlations below diagonal):\n\n")
		covcor <- x$msf
		correl <- x$msf / sqrt(diag(x$msf))[row(x$msf)] /
			sqrt(diag(x$msf))[col(x$msf)]
		covcor[lower.tri(covcor)] <- correl[lower.tri(correl)]
		printMatrix(format(round(t(covcor),digits=3),width=12))
	}
	invisible(x)
}

##@printMatrix Miscellaneous
printMatrix <- function(mat)
{
	cat(mat, sep=c(rep.int(' ', ncol(mat) - 1), '\n'))
}

##@print.sienaAlgorithm Methods
print.sienaAlgorithm <- function(x, ...)
{
	cat(' Siena Algorithm specification.\n')
	cat(' Project name:', x$projname, '\n')
	cat(' Use standard initial values:', x$useStdInits, '\n')
	cat(' Random seed:', objectOrNull(x$randomSeed),'\n')
	cat(' Number of subphases in phase 2:', x$nsub, '\n')
	if (x$simOnly)
	{
		cat(' Simulation only', '\n')
	}
	else
	{
		if (!is.null(x$n2start))cat(' Length of first subphase:', x$n2start, '\n')
		cat(' Starting value of gain parameter:', x$firstg, '\n')
		cat(' Reduction factor for gain parameter:', objectOrNull(x$reduceg), '\n')
		cat(' Diagonalization parameter:', x$diagonalize, '\n')
		cat(' Double averaging after subphase:', x$doubleAveraging, '\n')
	}
	cat(' Dolby noise reduction:', x$dolby, '\n')
	if (any(x$MaxDegree > 0))
	{
		cat(' Restrictions on degree in simulations: ')
		cat(x$MaxDegree,'\n')
	}
	if (any(x$UniversalOffset > 0))
	{
		cat(' Offsets for universal setting: ')
		cat(x$UniversalOffset,'\n')
	}
	cat(' Method for calculation of derivatives:',
		c('Scores', 'Finite Differences')[as.numeric(x$FinDiff.method) + 1],
		'\n')
	cat(' Number of subphases in phase 2:', x$nsub, '\n')
	cat(' Number of iterations in phase 3:', x$n3, '\n')
	if (x$maxlike)
	{
		cat(" Estimation by maximum likelihood\n")
		cat(' multiplication factor:', x$mult, '\n')
	}
	else
	{
		if (is.na(x$cconditional))
		{
			cat(" Unconditional simulation if more than one dependent",
				"variable\n")
		}
		else
		{
			if (x$cconditional)
			{
				cat(" Conditional simulation:")
				if (x$condname != '')
				{
					cat('conditioned on', x$condname, '\n')
				}
				else
				{
					if (x$condvarno > 0)
					{
						cat('conditioned on variable number', x$condvarno, '\n')
					}
				}
			}
			else
			{
				cat(" Unconditional simulation\n")
			}
		}
	}
	if (length(x$modelType) >= 1)
	{
		cat(" Model Type:\n")
		if (is.null(names(x$modelType)))
		{
			names(x$modelType) <- 1:length(x$modelType)
		}
		for (i in 1:length(x$modelType))
		{
			cat(sprintf(" %s: %d %s \n", names(x$modelType)[i], (x$modelType)[i],
					ModelTypeStrings(x$modelType[i])))
		}
	}
	if (length(x$behModelType) >= 1)
	{
		cat(" Behavioral Model Type:\n")
		if (is.null(names(x$behModelType)))
		{
			names(x$behModelType) <- 1:length(x$behModelType)
		}
		for (i in 1:length(x$behModelType))
		{
			cat(sprintf(" %s: %s\n",
					names(x$behModelType)[i], BehaviorModelTypeStrings(x$behModelType[i])))
		}
	}
	invisible(x)
}

##@objectOrNull Miscellaneous
objectOrNull <- function(x)
{
	if (is.null(x))
	{
		'NULL'
	}
	else
	{
		x
	}
}

##@averageTheta.last Miscellaneous
averageTheta.last <- function(z, groupOnly=0, nfirst=z$nwarm+1)
{
	ntot <- sum(!is.na(z$ThinPosteriorMu[,1]))
	ntott <- sum(!is.na(z$ThinParameters[,1,1]))

	if (nfirst > ntot)
	{
		stop('Sample did not come beyond warming')
	}
	if (nfirst > ntott)
	{
		stop(paste('nfirst =',nfirst,' > first dimension ThinParameters =',ntott))
	}
	thetaMean <- rep(NA, z$pp)
	postVarMean <- rep(NA, z$pp)
	for (group in 1:z$nGroup)
	{
		thetaMean[z$ratePositions[[group]]] <- 	colMeans(
				z$ThinParameters[nfirst:ntott, group,
					!z$generalParametersInGroup, drop=FALSE], na.rm=TRUE)
		postVarMean[z$ratePositions[[group]]] <- apply(
				z$ThinParameters[nfirst:ntott,
					group, !z$generalParametersInGroup, drop=FALSE], 3,
					var, na.rm=TRUE)
	}

	if (is.null(z$priorRatesFromData))
	{
		z$priorRatesFromData <- 2
	}
# 2 is here the default; this is to achieve compatibility with earlier versions
	if (groupOnly != 0)
	{
		thetaMean[(z$set1)&(!z$basicRate)] <- colMeans(
				z$ThinParameters[nfirst:ntott, groupOnly,
				z$varyingGeneralParametersInGroup, drop=FALSE], na.rm=TRUE)
	}
	else
	{
		if ((z$priorRatesFromData <0) | z$incidentalBasicRates)
		{
		thetaMean[(z$set1)&(!z$basicRate)] <- colMeans(
				z$ThinPosteriorMu[nfirst:ntot, , drop=FALSE], na.rm=TRUE)
			postVarMean[z$varyingObjectiveParameters] <-
				sapply(1:(dim(z$ThinPosteriorSigma)[2]),
					function(i){mean(z$ThinPosteriorSigma[nfirst:ntot,i,i], na.rm=TRUE)})
		}
		else
		{
	thetaMean[(z$set1)&(!z$basicRate)] <- colMeans(
				z$ThinPosteriorMu[nfirst:ntot,
				z$objectiveInVarying, drop=FALSE], na.rm=TRUE)
			postVarMean[z$varyingObjectiveParameters] <-
			sapply(1:(dim(z$ThinPosteriorSigma)[2]),
				function(i){mean(z$ThinPosteriorSigma[nfirst:ntot,i,i], na.rm=TRUE)}
														)[z$objectiveInVarying]
	}
	}
		thetaMean[z$set2] <-
			colMeans(z$ThinPosteriorEta[nfirst:ntot,, drop=FALSE], na.rm=TRUE)
	thetaMean[z$fix & (!z$basicRate)] <- z$thetaMat[1,z$fix & (!z$basicRate)]
	list(thetaMean, postVarMean)
}

##@sdTheta.last Miscellaneous
sdTheta.last <- function(z, groupOnly=0, nfirst=z$nwarm+1)
{
	ntot <- sum(!is.na(z$ThinPosteriorMu[,1]))
	ntott <- sum(!is.na(z$ThinParameters[,1,1]))
	if (nfirst >= ntot-1)
	{
		stop('Sample did not come beyond warming')
	}
	sdTheta<- rep(NA, z$pp)
	for (group in 1:z$nGroup)
	{
		sdTheta[z$ratePositions[[group]]] <- apply(
					z$ThinParameters[nfirst:ntott,
					group, !z$generalParametersInGroup, drop=FALSE], 3, sd)
	}
	if (is.null(z$priorRatesFromData))
	{
		z$priorRatesFromData <- 2
	}
# 2 is here the default; this is to achieve compatibility with earlier versions
	if (groupOnly != 0)
	{
		sdTheta[(z$set1)&(!z$basicRate)] <- apply(
				z$ThinParameters[nfirst:ntott, groupOnly,
				z$varyingGeneralParametersInGroup, drop=FALSE], 3, sd)
	}
	else
	{
		if ((z$priorRatesFromData <0) | z$incidentalBasicRates)
		{
		sdTheta[(z$set1)&(!z$basicRate)] <- apply(
				z$ThinPosteriorMu[nfirst:ntot, , drop=FALSE], 2, sd)
		}
		else
		{
		sdTheta[(z$set1)&(!z$basicRate)] <- apply(
				z$ThinPosteriorMu[nfirst:ntot,
				z$objectiveInVarying, drop=FALSE], 2, sd)
	}
	}
		sdTheta[z$set2] <-
			apply(z$ThinPosteriorEta[nfirst:ntot,, drop=FALSE], 2, sd)
	sdTheta
}

##@credValues Miscellaneous
credValues <- function(z, theProbs = c(0.025,0.975), tested = 0,
						groupOnly=0, nfirst=z$nwarm+1)
{
	ntot <- sum(!is.na(z$ThinPosteriorMu[,1]))
	ntott <- dim(z$ThinParameters)[1]
	if (nfirst >= ntot-1)
	{
		stop('Sample did not come beyond warming')
	}
	credVals <- matrix(NA, length(z$set1), 5)
	cvp <- function(x, test0){c(quantile(x, probs=theProbs, na.rm=TRUE),
									1-(ecdf(x))(test0))}
	if (is.null(z$priorRatesFromData))
	{
		z$priorRatesFromData <- 2
	}
# 2 is here the default; this is to achieve compatibility with earlier versions
	if (groupOnly != 0)
	{
		credVals[(z$set1)&(!z$basicRate), 1:3] <- t(apply(
				z$ThinParameters[nfirst:ntott, groupOnly,
				z$varyingGeneralParametersInGroup, drop=FALSE], 3,
						cvp, test0 = tested))
	}
	else
	{
		if ((z$priorRatesFromData <0) | z$incidentalBasicRates)
		{
			credVals[(z$set1)&(!z$basicRate), 1:3] <-
				t(apply(z$ThinPosteriorMu[nfirst:ntott,
					, drop=FALSE], 2, cvp, test0 = tested))
			credVals[z$varyingObjectiveParameters, 4:5] <-
				t(sapply(1:(dim(z$ThinPosteriorSigma)[2]),
		function(i){cvp(z$ThinPosteriorSigma[nfirst:dim(z$ThinPosteriorSigma)[1],i,i],
									10)[1:2]} ))
		}
		else
		{
		credVals[(z$set1)&(!z$basicRate), 1:3] <-
			t(apply(z$ThinPosteriorMu[nfirst:ntott,
				z$objectiveInVarying, drop=FALSE], 2, cvp, test0 = tested))
		credVals[z$varyingObjectiveParameters, 4:5] <-
			t(sapply(1:(dim(z$ThinPosteriorSigma)[2]),
		function(i){cvp(z$ThinPosteriorSigma[nfirst:dim(z$ThinPosteriorSigma)[1],i,i],
									10)[1:2]}
		)[,z$objectiveInVarying])
	}
	}
	for (group in 1:z$nGroup)
	{
		credVals[z$ratePositions[[group]], 1:2] <-
			t(sapply(which(!z$generalParametersInGroup),
			function(xx){cvp(z$ThinParameters[nfirst:ntott, group,
												xx, drop=FALSE],0)[1:2]}))
	}
	if (any(z$set2))
	{
		credVals[z$set2,1:3 ] <-
			t(apply(z$ThinPosteriorEta[nfirst:dim(z$ThinPosteriorEta)[1],, drop=FALSE],
																2, cvp, test0 = tested))
	}
	credVals
}

##@sienaFitThetaTable Miscellaneous
sienaFitThetaTable <- function(x, fromBayes=FALSE, tstat=FALSE, groupOnly=0, nfirst)
{
	if (fromBayes)
	{
	    theEffects <- x$initialResults$requestedEffects
	}
	else
	{
		theEffects <- x$requestedEffects
    	theEffects <- theEffects[which(theEffects$type!='gmm'),]
	}
	pp <- dim(theEffects)[1]
	if (is.null(x$thetaFromFile))
	{
		x$thetaFromFile <- FALSE
	}
    if (x$cconditional)
    {
        nrates <- length(x$rate)
    }
    else
    {
        nrates <- 0
    }
	if (!gmm(x))
	{
	  xp <- pp
	  pp <- pp + nrates
	}
	else
	{
	  pp <- sum(x$requestedEffects$include==TRUE)
	  xp <- pp - sum(x$requestedEffects$type=="gmm")
	  pp <- xp + nrates
	}
    ## mydf stores the data before formatting
	if (fromBayes)
	{
		mydf <- data.frame(dummy=rep(" ", pp),
                       row=rep(0, pp),
                       type=rep("", pp),
                       text=rep(" ", pp),
                       value = rep(0, pp),
                       lParenthesis = rep("(", pp),
                       se = rep(0, pp),
                       rParenthesis = rep(")", pp),
                       tstat=rep(NA, pp),
                       cFrom=rep(NA, pp),
                       cTo=rep(NA, pp),
                       p=rep(NA, pp),
					   random=rep("", pp),
                       stringsAsFactors =FALSE)
		if (groupOnly == 0)
		{
			mydf <- cbind(mydf, data.frame(postSd=rep("", pp),
					   cSdFrom=rep("", pp),
					   cSdTo=rep("", pp),
                       stringsAsFactors =FALSE))
		}
	}
	else
	{
		mydf <- data.frame(dummy=rep(" ", pp),
                       row=rep(0, pp),
                       type=rep("", pp),
                       text=rep(" ", pp),
                       value = rep(0, pp),
                       lParenthesis = rep("(", pp),
                       se = rep(0, pp),
                       rParenthesis = rep(")", pp),
                       tstat=rep(NA, pp),
                       stringsAsFactors =FALSE)
	}

    ## add to row is the extra lines to put in to table if you wish
    addtorow <- list()
    addtorow$pos <- list()
    addsub <- 1
    if (nrates > 0) ## conditional
    {
        addtorow$command[addsub] <-
            'Rate parameters: '
        addtorow$pos[[addsub]] <- 2
        addsub <- addsub + 1
        if (length(x$rate) == 1)
        {
            mydf[1, 'row'] <- 0
            if (length(attr(x$f,'netnames')) == 1)
            {
                mydf[1, 'text'] <- 'Rate parameter'
            }
            else
            {
                mydf[1, 'text'] <- 'Rate parameter of conditioning variable'
            }
			if (x$x$simOnly)
			{
				mydf[1, 'se'] <- NA
				mydf[1, 'value'] <- NA
			}
			else
			{
				mydf[1, 'value'] <- x$rate[1]
				mydf[1, 'se'] <- x$vrate[1]
			}
            addtorow$command[addsub] <- 'Other parameters: '
            addtorow$pos[[addsub]] <- 3
            addsub <- addsub + 1
        }
        else ## observations > 2
        {
            nn <- length(x$rate)
            nnstr <- as.numeric(paste('0.', as.character(1:nn), sep=""))
            mydf[1:nn, 'row'] <- nnstr
			if (x$x$simOnly)
			{
				mydf[1:nn, 'se'] <- rep(NA, nn)
				mydf[1:nn, 'value'] <- rep(NA, nn)
			}
			else
			{
				mydf[1:nn, 'value'] <- x$rate
				mydf[1:nn, 'se'] <- x$vrate
			}
            if (length(x$f$types) == 1)
            {
                mydf[1:nn, 'text'] <- paste('Rate parameter period', 1:nn)
            }
            else
            {
                mydf[1:nn, 'text'] <-
                    paste('Rate parameter cond. variable period', 1:nn)
            }
            addtorow$command[addsub] <- 'Other parameters: '
            addtorow$pos[[addsub]] <- nn + 2
            addsub <- addsub + 1
        }
    }
    nBehavs <- sum(x$f$types == "behavior")
	nConts <- sum(x$f$types == "continuous")
    nNetworks <- length(x$f$types) - nBehavs - nConts
    if ((nConts > 0 || nBehavs > 0) && nNetworks > 0)
    {
        addtorow$command[addsub] <- "Network Dynamics"
        addtorow$pos[[addsub]] <- nrates + 2
        addsub <- addsub + 1
    }
	if (!is.null(x$covtheta))
	{
	  if (!gmm(x))
	  {
	    ses <- sqrt(diag(x$covtheta))
	    ses[x$fixed] <- NA
	  }
	  else
	  {
	    ses <- sqrt(diag(x$covtheta))
	    ses[x$fixed[-which(x$requestedEffects$type=="gmm")]] <- NA
	  }
	}
	if (fromBayes)
	{
		atl <- averageTheta.last(x, groupOnly, nfirst = nfirst)
		theta <- atl[[1]]
		postSd <- sqrt(atl[[2]])
	}
	else
	{
		if (gmm(x))
		{
		theta <- x$theta[-which(x$requestedEffects$type=="gmm")]
		}
		else
		{
			theta <- x$theta
		}
		if (!is.null(x$covtheta))
		{
			theta[diag(x$covtheta) < 0.0] <- NA
		}
	}

	if (nBehavs > 0)
	{
		behEffects <- theEffects[theEffects$netType == 'behavior',]
		behNames <- unique(behEffects$name)
	}
	if (nBehavs > 1)
	{
		behEffects$effectName <- paste('<',
			(1:nBehavs)[match(behEffects$name,
				behNames)],
			'> ', behEffects$effectName,
			sep='')
		theEffects$effectName[theEffects$netType=='behavior'] <-
			behEffects$effectName
	}
    if (nConts > 0)
    {
        contEffects <- theEffects[theEffects$netType == 'continuous',]
        contNames <- unique(contEffects$name)
    }
    if (nConts > 1)
    {
        contEffects$effectName <- paste('<',
            (1:nConts)[match(contEffects$name,
                contNames)],
            '> ', contEffects$effectName,
            sep='')
        theEffects$effectName[theEffects$netType=='continuous'] <-
            contEffects$effectName
    }
      mydf[nrates + (1:xp), 'row'] <-  1:xp
      mydf[nrates + (1:xp), 'type' ] <- ifelse(theEffects$type == "creation",
                                               "creat", theEffects$type)
      mydf[nrates + (1:xp), 'text' ] <- theEffects$effectName
      mydf[nrates + (1:xp), 'value' ] <- theta

	if (fromBayes) # then nrates=0
	{
		mydf[nrates + (1:xp), 'se' ] <- sdTheta.last(x, groupOnly, nfirst=nfirst)
		mydf[nrates + (1:xp), 'random' ] <- NA
		credVal <- credValues(x, groupOnly = groupOnly, nfirst = nfirst)
		mydf[nrates + (1:xp), 'cFrom' ] <- credVal[,1]
		mydf[nrates + (1:xp), 'cTo' ] <- credVal[,2]
		mydf[nrates + (1:xp), 'p' ] <- credVal[,3]
		if (groupOnly == 0)
		{
		mydf[nrates + (1:xp), 'postSd' ] <- postSd
		mydf[nrates + (1:xp), 'cSdFrom' ] <- sqrt(credVal[,4])
		mydf[nrates + (1:xp), 'cSdTo' ] <- sqrt(credVal[,5])
			mydf[1:nrates, 'cFrom'] <- credVal[1:nrates,1]
			mydf[1:nrates, 'cTo'] <- credVal[1:nrates,2]
		}
	}
	else
	{
		if (exists("ses"))
		{
			mydf[nrates + (1:xp), 'se' ] <- ses
		}
	}
	if (!is.null(x$tstat))
	{
		mydf[1:nrates, "tstat"] <- NA
		mydf[nrates + (1:xp), 'tstat' ] <- x$tstat
	}

	if (nBehavs > 0 && nNetworks > 0)
	{
		nNetworkEff <- nrow(theEffects) - nrow(behEffects) -
            sum(theEffects$netType %in% c('continuous', 'sde'))
		addtorow$command[addsub] <-
			'Behavior Dynamics'
		addtorow$pos[[addsub]] <- nrates + 2 + nNetworkEff
		addsub <- addsub + 1
	}
    if (nConts > 0 && nNetworks > 0)
    {
        nNetworkAndBehEff <- nrow(theEffects) -
            sum(theEffects$netType %in% c('continuous', 'sde'))
        addtorow$command[addsub] <-
            'Continuous Behavior Dynamics'
        addtorow$pos[[addsub]] <- nrates + 2 + nNetworkAndBehEff
        addsub <- addsub + 1
    }
	return(list(mydf=mydf, addtorow=addtorow))
} # end sienaFitThetaTable

##@sienaFitCovarianceCorrelation Miscellaneous
sienaFitCovarianceCorrelation <- function(x)
{
	covcor <- x
	correl <- x/sqrt(diag(x))[row(x)]/ sqrt(diag(x))[col(x)]
	covcor[lower.tri(covcor)] <- correl[lower.tri(correl)]
	return(covcor)
}

##@xtable fake in case package not loaded
xtable <- function(x, ...)
{
	xtable::xtable(x, ...)
}

##@xtable.sienaFit Methods
xtable.sienaFit <- function(x, caption = NULL, label = NULL, align = NULL,
                            digits = NULL, display = NULL, ...)
{
    tmp <- sienaFitThetaTable(x)
    mydf <- tmp$mydf
    addtorow <- tmp$addtorow
    ## find out whether the type is html or latex
    dots <- substitute(list(...))[-1] ##first entry is the word 'list'
    if (!is.null(dots[["type"]]))
    {
        type <- dots[["type"]]
    }
    else
    {
        type <- "latex"
    }
    if (!is.null(addtorow$command))
    {
        if (type =="latex")
        {
            use <- addtorow$command != 'Network Dynamics'
	## the \rule{0pt}{2.5ex}\relax gives a little bit extra vertical space
            addtorow$command <- paste('\\multicolumn{4}{l}{\rule{0pt}{2.5ex}\relax ', addtorow$command,
                                      '} \\\\ \n')
            use[1] <- FALSE
            addtorow$command[use] <- paste('\\\\ ', addtorow$command[use])
        }
        else ##html
        {
			## use <- addtorow$command != 'Network Dynamics'
            addtorow$command <- paste("<TR> <TD colspan=9 align=left>",
                                      addtorow$command,
                                      "</TD> </TR> <TR> </TR> \n")
			##  use[1] <- FALSE
			##  addtorow$command[use] <- paste('\\\\ ', addtorow$command[use])
        }
    }
    else
    {
        addtorow <- NULL
    }
    mydf[mydf$row < 1, 'row'] <-
        format(mydf[mydf$row < 1, 'row'])
    mydf[mydf[,'row'] >= 1, 'row'] <- paste(format(mydf[mydf$row >= 1,
             'row']), '.', sep='')
    tmp <- list(xtable::xtable(mydf, caption=caption, label=label, align=align,
							   digits=digits, display=display),
				add.to.row=addtorow,
                include.colnames=FALSE, include.rownames=FALSE, ...)
    class(tmp) <- c("xtable.sienaFit", "xtable")
    tmp
}
##@print.xtable.sienaFit Methods
print.xtable.sienaFit <- function(x, ...)
{
    addtorow <- x[["add.to.row"]]
    if (!is.null(addtorow))
    {
        x$add.to.row$pos <- lapply(x$add.to.row$pos, function(x)x-2)
        do.call("print", x)
    }
    else
    {
        do.call("print", x[-2])
    }
    invisible(x)
}

##@print.chains.data.frame Methods
print.chains.data.frame <- function(x, ...)
{
    NextMethod(x)
	initState <- attr(x, "initialStateDifferences")
	endState <- attr(x, "endStateDifferences")
	if (nrow(initState) > 0)
	{
		cat("\n Initial State Differences:\n")
		print(initState)
	}
	if (nrow(endState) > 0)
	{
		cat("\n End State Differences:\n")
		print(endState)
	}
}

## extra for backward compatibility with siena objects
## created before version 1.6:

##@print.siena Methods
print.siena <- print.sienadata

terminateFRAN <- function(z, x)
{
    if (z$cconditional)
    {
        z$rate<- colMeans(z$ntim, na.rm=TRUE)
        z$vrate <- apply(z$ntim, 2, sd, na.rm=TRUE)
        z$theta[z$posj] <- z$theta[z$posj] * z$rate
		if (!x$simOnly)
		{
		  if (!gmm(x))
		  {
		    z$covtheta[z$posj, ] <- z$covtheta[z$posj, ] * z$rate
		    z$covtheta[, z$posj] <- z$covtheta[,z$posj ] * z$rate
		  }
			else 
			{ 
			  z$covtheta[z$posj[-which(z$requestedEffects$type=="gmm")], ] <- z$covtheta[z$posj[-which(z$requestedEffects$type=="gmm")], ] * z$rate
			  z$covtheta[, z$posj[-which(z$requestedEffects$type=="gmm")]] <- z$covtheta[,z$posj[-which(z$requestedEffects$type=="gmm")] ] * z$rate  
			}
		}	
    }
    f <- FRANstore()
    f$pModel <- NULL
    f$pData <- NULL
    z$f$myeffects <- lapply(z$f$myeffects, function(x){x$effectPtr <- NULL;x})
    FRANstore(NULL) ## clear the stored object
    if (is.null(z$print) || z$print)
    {
        PrintReport(z, x)
    }
    if (sum(z$test))
    {
        z$fra <- colMeans(z$sf, na.rm=TRUE)
		z$redundant <- (z$fix & (!z$test))
        ans <- ScoreTest(z$pp, z$dfra, z$msf, z$fra, z$test, z$redundant, x$maxlike)
        z <- c(z, ans)
        TestOutput(z, x)
    }
    if (!is.null(z$dfra))
    {
        dimnames(z$dfra)[[1]] <- as.list(z$requestedEffects$shortName)
    }
    return(z)
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: observationErrors.r
# *
# * Description: This module contains various functions for parameter estimation
# * in models with observation errors
# * by maximum likelihood under normality,
# * extending Snijders & Baerveldt (2003), for use in siena08.
# *****************************************************************************/
##args:x: vector of observations
##     se: vector of same length as x: standard errors of observations
## returns updated z

##@deviance.observationErrors siena08: deviance under normality assumptions
deviance.observationErrors <- function(mu, sig, x, se)
{
    ## deviance.observationErrors: given vector observations x, assuming model
    ## x ~ normal(expected = mu, variance = sig^2 + se^2)
    ## required is length(x) = length(se)
    sig2 <- sig^2
    sum((x - mu)^2/(sig2 + se^2)) + sum(log(sig2 + se^2))
}

##@profdev.mu siena08 (profile deviance =) minus twice profile loglik for mu
profdev.mu <- function(mu,x,se)
{
    ## deviance.observationErrors maximized over sigma
    ra <- max(x) - min(x)  # easy upper bound for sigma
    optimize(function(sig)deviance.observationErrors(mu, sig, x, se),
             c(0, ra))
}


##@profdev.sig siena08 (profile deviance =) minus twice prof. loglik for sigma
profdev.sig <- function(sig,x,se)
{
    ## deviance.observationErrors maximized over mu
    optimize(function(mu)deviance.observationErrors(mu, sig, x, se),
             range(x))
}

##@maxlik siena08 maximum likelihood estimator
maxlik <- function(x,se)
{
    if (length(x) > 1)
    {
        ## deviance.observationErrors minimized over mu and sigma
        ## Minimize profile deviance for mu:
        opmu  <- optimize(function(mu)profdev.mu(mu, x, se)$objective,
                          range(x))
        ## MLE for mu:
        mu    <- opmu$minimum
        ## minimized deviance:
        dev   <- opmu$objective
        ## Location for minimum of deviance for this value of mu:
        sig   <- profdev.mu(mu, x , se)$minimum
        ## Standard error of MLE(mu):
        se.mu <- sqrt(1 / sum(1 / (se^2 + sig^2)))
    }
    else
    {
        mu <- x
        se.mu <- NA
        sig <- NA
        dev <- NA
    }
    return(list(mu = mu, se.mu = se.mu, sigma = sig, deviance = dev))
}

##@unisroot siena08 root finder when interval may be inadequate
unisroot <- function(f, interval, ..., left=TRUE)
{
    ## tries to solve f(x) = 0,
    ## first within interval,
    ## if endpoints do not have opposite signs
    ## then in interval extended to left or to right
    ## Return value is a list which includes root = location of the root,
    ## and f.root = function value at this location.
    if (f(interval[1]) * f(interval[2]) < 0)
    {
        uniroot(f, interval, ...)
    }
    else
    {
        x1 <- interval[1]
        x2 <- interval[2]
        ra <- x2 - x1
        for (it in (1:1000))
        {
            if (left)
            {
                x1 <- x1 - ra
            }
            else
            {
                x2 <- x2 + ra
            }
            if (f(x1) * f(x2) < 0)
            {
                break
            }
        }
        if (f(x1) * f(x2) < 0)
        {
            uniroot(f, c(x1, x2), ...)
        }
        else
        {
            list(root=NA, f.root=NA)
        }
    }
}

##@confint.mu siena08 confidence interval for mu
confint.mu <- function(x, se, alpha=0.05)
{
    ## confidence interval for mu in model
    ## x ~ normal(expected = mu, variance = sigma^2 + se^2)
    ## with length(x) = length(se).
    ## returns list consisting of bounds confidence interval and
    ## confidence level (1-alpha).
    ma <- max(x)
    mi <- min(x)
    maxli <- maxlik(x, se)        # ML estimators
    mindev <- maxli$deviance       # minimized deviance
    mlemu <- maxli$mu             # MLE for mu
    chidev <- qchisq(1 - alpha, 1) # critical value chi-squared distribution
    if (length(x) > 1)
    {
        ## The end points of the confidence interval are the values
        ## where the profile deviance for mu is equal to mindev + chidev.
        ## Now first the solution for the left side of the confidence interval:
        tmp1 <- unisroot(function(mu)
                     {
                         tmp <- profdev.mu(mu, x, se)
                         tmp$objective - mindev - chidev
                     },
                         c(mi, mlemu))
        mu1 <- tmp1$root
        ## and then the solution for the right side of the confidence interval:
        tmp1 <- unisroot(function(mu)
                     {
                         tmp <- profdev.mu(mu, x, se)
                         tmp$objective - mindev - chidev
                     },
                         c(mlemu, ma), left=FALSE)
        mu2 <- tmp1$root
    }
    else
    {
        mu1 <- NA
        mu2 <- NA
    }
    c(mu1, mu2, 1 - alpha)
}

##@confint.sig siena08 confidence interval for sigma
confint.sig <- function(x, se, alpha=0.05)
{
    ## confidence interval for sigma in model
    ## x ~ normal(expected = mu, variance = sigma^2 + se^2)
    ## with length(x) = length(se)
    ## returns list consisting of bounds confidence interval and
    ## confidence level (1 - alpha).     ma <- max(x)
    ra <- max(x) - min(x)          # easy upper bound for sigma
    maxli  <- maxlik(x, se)        # ML estimators
    mindev <- maxli$deviance       # minimized deviance
    mlesig <- maxli$sigma          # MLE for sigma
    chidev <- qchisq(1 - alpha, 1) # critical value chi-squared distribution
    ## The end points of the confidence interval are the values
    ## where the profile deviance for sigma is equal to mindev + chidev,
    ## unless the profile deviance in 0 is smaller than this.
    ## Now first the solution for the left side of the confidence interval;
    ## this may be 0.
    if (length(x) > 1)
    {
        if (profdev.sig(0, x, se)$objective <= mindev + chidev)
        {
            sig1 <- 0
        }
        else
        {
            sig1 <- uniroot(function(sig)
                            profdev.sig(sig, x, se)$objective - mindev - chidev,
                            c(0, mlesig))$root
        }
        ## and then the solution for the right side of the confidence interval:
        sig2 <- unisroot(function(sig)
                         profdev.sig(sig, x, se)$objective - mindev - chidev,
                         c(mlesig, ra), left=FALSE)$root
    }
    else
    {
        sig1 <- NA
        sig2 <- NA
    }
    return(c(sig1, sig2, 1 - alpha))
}

## /*****************************************************************************
##	* SIENA: Simulation Investigation for Empirical Network Analysis
##	*
##	* Web: https://www.stats.ox.ac.uk/~snijders/siena
##	*
##	* File: sienaGOF.r
##	*
##	* Description: This file contains the code to assess goodness of fit:
##	* the main function sienaGOF, the plot method,
##	* and auxiliary statistics and extractor functions.
##	* Written by Josh Lospinoso, modifications by Tom Snijders.
##	*
##	****************************************************************************/


test_gof <- function(object, ...) UseMethod("test_gof", object)

##@sienaGOF siena07 Does test for goodness of fit
sienaGOF <- function(
		object,	auxiliaryFunction,
		period=NULL, verbose=FALSE, join=TRUE, twoTailed=FALSE,
		cluster=NULL, robust=FALSE,
		groupName="Data1", varName, tested=FALSE, iterations=NULL,
		giveNAWarning=TRUE, ...)
	{
	## require(MASS)
	## require(Matrix)
	##	Check input
	# fitList indicates whether the object is a list of
	# sienaFit objects, or a single such object.
	fitList <- FALSE
	if (inherits(object, "sienaFit"))
	{
		sFO <- object
	}
	else
	{
		if (all(vapply(object, function(x){inherits(x, "sienaFit")}, FUN.VALUE=TRUE)))
		{
			sFO <- object[[1]]
# If fitList, it is assumed that all elements of this list
# have the same specification, and the first one is used
# to deduce this information.
			fitList <- TRUE
			if (!inherits(sFO, "sienaFit"))
			{
				stop("The first parameter of sienaGOF must be a sienaFit object or a list of such objects")
			}
			if (length(period) > 1)
			{
				stop("For operating on a list of sienaFit objects, only a single period can be used.")
			}
		}
	}

	if (sFO$maxlike)
	{
		stop(
	"sienaGOF can only operate on results from Method of Moments estimation.")
	}
	if (! sFO$returnDeps)
	{
		stop("You must instruct siena07 to return the simulated networks")
	}
	if (!is.null(sFO$sf2.byIteration))
	{
		if (!sFO$sf2.byIteration)
    	{
        	stop("sienaGOF needs sf2 by iterations (use lessMem=FALSE)")
    	}
	}
	if (is.null(iterations))
	{
		iterations <- length(sFO$sims)
	}
	else
	{
		iterations <- min(iterations, length(sFO$sims))
	}
	if (iterations < 1)
	{
		stop("You need at least one iteration.")
	}
	if (missing(varName))
	{
		stop("You need to supply the parameter <<varName>>.")
	}
	if (missing(auxiliaryFunction))
	{
		stop("You need to supply the parameter <<auxiliaryFunction>>.")
	}
# This should be captured for possible later use
# (when it will have been used the name is no longer available):
	auxfu <- deparse(substitute(auxiliaryFunction))
# There might be more than one varName:
	if (is.null(sFO$f[[groupName]]$depvars[[varName[1]]]))
	{
		stop("There is a mismatch between the object and the groupName or varName.")
	}

	groups <- length(sFO$f$groupNames)
	if (fitList)
	{
		tested <- FALSE
	}
	else if (is.null(tested))
	{
		tested <- sFO$test
	}
	else
	{
		if (!inherits(tested, "logical"))
		{
			stop('tested should be a logical vector')
		}
		if ((length(tested) != length(sFO$test)) | (all(tested == FALSE)))
		{
			tested <- rep(FALSE, length(sFO$test))
		}
		else
		{
			tested <- (tested & sFO$test)
		}
	}
	if (verbose)
	{
		if (groups <= 1)
		{
			message("Detected ", iterations, " iterations and ", groups, " group.")
		}
		else
		{
			message("Detected ", iterations, " iterations and ", groups, " groups.")
		}
		if (fitList)
		{
			message("The data for analysis is a list of ", length(object),
						" sienaFit objects.")
		}
	}

	if (fitList)
	{
		if (is.null(period) )
		{
			period <- 1
		}
		per <- period # can only be a single number
		auxFunction <- function(i, object, j, groupName, varName, ...){
			auxiliaryFunction(i, object[[j]]$f,
						object[[j]]$sims, per, groupName, varName, ...)}
		period <- seq_along(object)
		# from now on, period will denote the rank number of the sienaFit object.
	}
	else
	{
		if (is.null(period) )
		{
			period <- 1:(attr(object$f[[1]]$depvars[[1]], "netdims")[3] - 1)
		}
		auxFunction <- function(i, object, j, groupName, varName, ...){
			auxiliaryFunction(i, object$f,
						object$sims, j, groupName, varName, ...)}
	}

	obsStatsByPeriod <- lapply(period, function (j) {
						matrix(
						auxFunction(NULL,
								object, j, groupName, varName, ...)
								, nrow=1)
				})

	if (join)
	{
		obsStats <- Reduce("+", obsStatsByPeriod)
		obsStats <- list(Joint=obsStats)
	}
	else
	{
		obsStats <- obsStatsByPeriod
		names(obsStats) <- paste("Period", period)
	}
	plotKey <- names(auxiliaryFunction(NULL, sFO$f,
				sFO$sims, 1, groupName, varName, ...))
	EA <- attr(auxiliaryFunction(NULL, sFO$f,
				sFO$sims, 1, groupName, varName, ...), "EgoAlter")
	EA <- ifelse(is.null(EA), FALSE, EA)
	class(obsStats) <- "observedAuxiliaryStatistics"
	attr(obsStats,"auxiliaryStatisticName") <-
			deparse(substitute(auxiliaryFunction))
	attr(obsStats,"joint") <- join
	if (attr(obsStats,"auxiliaryStatisticName") == "egoAlterCombi")
	{
		EA <- TRUE
	}
	##	Calculate the simulated auxiliary statistics
	if (verbose)
	{
		if (length(period) <= 1)
		{
			cat("Calculating auxiliary statistics for period ", period, ".\n")
		}
		else
		{
			cat("Calculating auxiliary statistics for periods ", period, ".\n")
		}
	}
	if (!is.null(cluster))
	{
		ttcSimulation <- system.time(simStatsByPeriod <-
			lapply(period, function (j) {
				simStatsByPeriod <- parLapply(cluster, 1:iterations,
					function (i){auxFunction(i, object,
										j, groupName, varName, ...)})
				simStatsByPeriod <- matrix(unlist(simStatsByPeriod), ncol=iterations)
				dimnames(simStatsByPeriod)[[1]] <-	plotKey
				dimnames(simStatsByPeriod)[[2]] <-	1:iterations
				t(simStatsByPeriod)
				}))
	}
	else
	{
		ttcSimulation <- system.time( simStatsByPeriod <- lapply(period,
					function (j) {
						if (verbose)
						{
							message("  Period ", j, "\n")
							flush.console()
						}
						simStatsByPeriod <- lapply(1:iterations, function (i)
						{
							if (verbose && (i %% 100 == 0) )
								{
								cat("  > Completed ", i,
										" calculations\r")
								flush.console()
								}
								auxFunction(i, object,
										j, groupName, varName, ...)
						})
					if (verbose)
					{
						cat("  > Completed ", iterations, " calculations\n\n")
					}
					flush.console()
					if (var(vapply(simStatsByPeriod, length, FUN.VALUE=0))>1e-8)
					{
						stop("Function ", auxfu,
							" does not always give vectors of the same length")
					}
					simStatsByPeriod <-
							matrix(unlist(simStatsByPeriod), ncol=iterations)
					dimnames(simStatsByPeriod)[[1]] <-	plotKey
					dimnames(simStatsByPeriod)[[2]] <-	1:iterations
					t(simStatsByPeriod)
					})
	  )
	}

	## Give a warning in case of missings.
	nmissings <- vapply(simStatsByPeriod,
		function(sp){apply(sp, 2, function(x){sum(is.na(x))})},
					FUN.VALUE=rep(0,dim(simStatsByPeriod[[1]])[2]))
	rownames(nmissings) <- plotKey
	if ((sum(nmissings) > 0) & giveNAWarning)
	{
		cat("Number of missing values in the simulated functions:\n")
		print(t(nmissings))
		warning("Some simulated values are missing.")
	}

	## Aggregate by period if necessary to produce simStats
	if (join)
	{
			simStats <- Reduce("+", simStatsByPeriod)
		simStats <- list(Joint=simStats)
	}
	else
	{
		simStats <- simStatsByPeriod
		names(simStats) <- paste("Period",period)
	}
	class(simStats) <- "simulatedAuxiliaryStatistics"
	attr(simStats,"auxiliaryStatisticName") <-
			deparse(substitute(auxiliaryFunction))
	attr(simStats,"joint") <- join
	attr(simStats,"time") <- ttcSimulation

	applyTest <-  function (observed, simulated)
# Test using Mahalanobis distances
	{
		if (!inherits(simulated,"matrix"))
		{
			stop("Invalid input.")
		}
		if (!inherits(simulated,"matrix"))
		{
			observed <- matrix(observed,nrow=1)
		}
		if (!inherits(simulated,"matrix"))
		{
			stop("Observation must be a matrix.")
		}
		if (ncol(observed) != ncol(simulated))
		{
			stop("Dimensionality of function parameters do not match.")
		}
		observations <- nrow(observed)
	#	simulations<-nrow(simulated)
		variates<-ncol(simulated)
		if (robust) {
			a <- cov.rob(simulated)$cov
		}
		else
		{
			a <- cov(simulated, use="pairwise.complete.obs")
			a[is.na(a)] <- 0
		}
		ainv <- ginv(a)
		arank <- rankMatrix(a)
		expectation <- colMeans(simulated)
		centeredSimulations <- scale(simulated, scale=FALSE)
		if (variates==1)
		{
			centeredSimulations <- t(centeredSimulations)
		}
		mhd <- function(x) # Mahalanobis distance
		{
			x %*% ainv %*% x
		}
		simTestStat <- apply(centeredSimulations, 1, mhd)
		centeredObservations <- observed - expectation
		obsTestStat <- apply(centeredObservations, 1, mhd)
		if (twoTailed)
		{
			p <- sapply(1:observations, function (i)
						1 - abs(1 - 2 * sum(obsTestStat[i] <=
						simTestStat)/length(simTestStat)) )
		}
		else
		{
			p <- sapply(1:observations, function (i)
				sum(obsTestStat[i] <= simTestStat) /length(simTestStat))
		}

		ret <- list( p = p,
				SimulatedTestStat=simTestStat,
				ObservedTestStat=obsTestStat,
				TwoTailed=twoTailed,
				Simulations=simulated,
				Observations=observed,
				InvCovSimStats=a,
				Rank=arank)
		class(ret) <- "sienaGofTest"
		attr(ret, "version") <- packageDescription(pkgname, fields = "Version")
		attr(ret,"sienaFitName") <- deparse(substitute(object))
		attr(ret,"auxiliaryStatisticName") <-
				attr(obsStats,"auxiliaryStatisticName")
		attr(ret, "key") <- plotKey
		ret
	}

	res <- lapply(1:length(simStats),
					function (i) {
				 applyTest(obsStats[[i]], simStats[[i]]) })
	mhdTemplate <- rep(0, sum(tested))
	names(mhdTemplate) <- rep(0, sum(tested))

	JoinedOneStepMHD_old <- mhdTemplate
	OneStepMHD_old <- lapply(period, function(i) (mhdTemplate))
	JoinedOneStepMHD <- mhdTemplate
	OneStepMHD <- lapply(period, function(i) (mhdTemplate))

	obsMhd <- NULL

	ExpStat <-
		lapply(period, function(i) {colMeans(simStatsByPeriod[[i]])})
	simStatsByPeriod_tilde <-
		lapply(period, function(i) {
			t(apply(simStatsByPeriod[[i]],1, function(x){x - ExpStat[[i]]}))})

	OneStepSpecs <- matrix(0, ncol=sum(tested),
			nrow=length(object$theta))
	if (robust) {
		covInvByPeriod <- lapply(period, function(i) ginv(
							cov.rob(simStatsByPeriod[[i]]) ))
	}
	else
	{
		covInvByPeriod <- lapply(period, function(i){
				b <- cov(simStatsByPeriod[[i]], use="pairwise.complete.obs")
				b[is.na(b)] <- 0
				ginv(b)})
	}

	obsMhd <- sapply(period, function (i) {
				 (obsStatsByPeriod[[i]] - ExpStat[[i]])	 %*%
						covInvByPeriod[[i]] %*%
						t(obsStatsByPeriod[[i]] - ExpStat[[i]] )
			})

	if (sum(tested) > 0) {
		effectsObject <- object$requestedEffects
		for (i in period) {
			names(OneStepMHD_old[[i]]) <-
					effectsObject$effectName[tested]
			names(OneStepMHD[[i]]) <-
					effectsObject$effectName[tested]
		}
		names(JoinedOneStepMHD_old) <-
			effectsObject$effectName[tested]
		names(JoinedOneStepMHD) <-
				effectsObject$effectName[tested]

		rownames(OneStepSpecs) <- effectsObject$effectName
		colnames(OneStepSpecs) <- effectsObject$effectName[tested]
		counterTestEffects <- 0
		for(index in which(tested)) {
			if (verbose) {
				message("Estimating test statistic for model including ",
						effectsObject$effectName[index], "\n")
			}
			counterTestEffects <- counterTestEffects + 1
			effectsToInclude <- !tested
			effectsToInclude[index] <- TRUE
			theta0 <- object$theta
			names(theta0) <- effectsObject$effectName
			theta0 <- theta0[effectsToInclude]
			obsSuffStats <-
					t(object$targets2[effectsToInclude, , drop=FALSE])
			G <- object$sf2[, , effectsToInclude, drop=FALSE] -
					rep(obsSuffStats, each=iterations)
			sigma <- cov(apply(G, c(1, 3), sum), use="pairwise.complete.obs")
			SF <- object$ssc[ , , effectsToInclude, drop=FALSE]
			dimnames(SF)[[3]] <- effectsObject$effectName[effectsToInclude]
			dimnames(G) <- dimnames(SF)
			if (!(object$maxlike || object$FinDiff.method))
			{
				D <- derivativeFromScoresAndDeviations(SF, G, , , , TRUE, )
			}
			else
			{
				DF <- object$
						sdf2[ , , effectsToInclude, effectsToInclude,
						drop=FALSE]
				D <- t(apply(DF, c(3, 4), mean))
			}
			fra <- apply(G, 3, sum) / iterations
			doTests <- rep(FALSE, sum(effectsToInclude))
			names(doTests) <- effectsObject$effectName[effectsToInclude]
			doTests[effectsObject$effectName[index]] <- TRUE
			redundant <- rep(FALSE, length(doTests))
			mmThetaDelta <- as.numeric(ScoreTest(length(doTests), D,
							sigma, fra, doTests, redundant,
							maxlike=object$maxlike)$oneStep )

      # \mu'_\theta(X)
			JacobianExpStat_old <- lapply(period, function (i) {
				t(SF[1:iterations,i,]) %*% simStatsByPeriod[[i]]/ iterations	 })
			JacobianExpStat <- lapply(period, function (i) {
				t(SF[1:iterations,i,]) %*% simStatsByPeriod_tilde[[i]]/ iterations })
      # List structure: Period, effect index
      thetaIndices <- 1:sum(effectsToInclude)
	  # \Gamma_i(\theta)  i=period, j=parameter, k=replication
			ExpStatCovar_old <- lapply(period, function (i) {
            lapply(thetaIndices, function(j){
              Reduce("+", lapply(1:iterations,function(k){
                simStatsByPeriod[[i]][k,] %*% t(simStatsByPeriod[[i]][k,]) * SF[k,i,j]
              })) / iterations
				- JacobianExpStat[[i]][j,] %*%
			t(ExpStat[[i]]) - ExpStat[[i]] %*% t(JacobianExpStat[[i]][j,])
            })
        })
			ExpStatCovar <- lapply(period, function (i) {
				lapply(thetaIndices, function(j){
				Reduce("+", lapply(1:iterations,function(k){
			simStatsByPeriod_tilde[[i]][k,] %*%
				t(simStatsByPeriod_tilde[[i]][k,]) * SF[k,i,j] })) / iterations})})
      # \Xi_i(\theta)
			JacobianCovar_old <- lapply(period, function (i) {
				lapply(thetaIndices, function(j){
					-1 * covInvByPeriod[[i]] %*% ExpStatCovar_old[[i]][[j]] %*%
						covInvByPeriod[[i]] })
			})
      JacobianCovar <- lapply(period, function (i) {
        lapply(thetaIndices, function(j){
					-1 * covInvByPeriod[[i]] %*% ExpStatCovar[[i]][[j]] %*%
						covInvByPeriod[[i]] })
        })

			Gradient_old <- lapply(period, function(i) {
				sapply(thetaIndices, function(j){
					( obsStatsByPeriod[[i]] - ExpStat[[i]] ) %*%
						JacobianCovar_old[[i]][[j]] %*%
					t( obsStatsByPeriod[[i]] - ExpStat[[i]] )
					})
				-2 * JacobianExpStat_old[[i]] %*% covInvByPeriod[[i]] %*%
					t( obsStatsByPeriod[[i]] - ExpStat[[i]] )
				})
			Gradient <- lapply(period, function(i) {
          sapply(thetaIndices, function(j){
          ( obsStatsByPeriod[[i]] - ExpStat[[i]] ) %*%
            JacobianCovar[[i]][[j]] %*%
          t( obsStatsByPeriod[[i]] - ExpStat[[i]] )
          })
				-2 * JacobianExpStat[[i]] %*% covInvByPeriod[[i]] %*%
            t( obsStatsByPeriod[[i]] - ExpStat[[i]] )
					})

			OneStepSpecs[effectsToInclude,counterTestEffects] <-
								theta0 + mmThetaDelta

			for (i in 1:length(obsMhd)) {
				OneStepMHD_old[[i]][counterTestEffects] <-
					as.numeric(obsMhd[i] + mmThetaDelta %*% Gradient_old[[i]] )
      }
			for (i in 1:length(obsMhd)) {
				OneStepMHD[[i]][counterTestEffects] <-
					as.numeric(obsMhd[i] + mmThetaDelta %*% Gradient[[i]] )
		}
			JoinedOneStepMHD_old[counterTestEffects] <-
						Reduce("+",OneStepMHD_old)[counterTestEffects]
			JoinedOneStepMHD[counterTestEffects] <-
						Reduce("+",OneStepMHD)[counterTestEffects]
		} # end 'for index'
	}

	names(res) <- names(obsStats)
	class(res) <- "sienaGOF"
	attr(res, "version") <- packageDescription(pkgname, fields = "Version")
	attr(res, "scoreTest") <- (sum(tested) > 0)
	attr(res, "originalMahalanobisDistances") <- obsMhd
	attr(res, "oneStepMahalanobisDistances") <- OneStepMHD
	attr(res, "joinedOneStepMahalanobisDistances") <-
			JoinedOneStepMHD
	attr(res, "oneStepMahalanobisDistances_old") <- OneStepMHD_old
	attr(res, "joinedOneStepMahalanobisDistances_old") <-
			JoinedOneStepMHD_old
	attr(res, "oneStepSpecs") <- OneStepSpecs
	attr(res,"auxiliaryStatisticName") <-
			attr(obsStats,"auxiliaryStatisticName")
	attr(res, "simTime") <- attr(simStats,"time")
	attr(res, "twoTailed") <- twoTailed
	attr(res, "joined") <- join
	attr(res, "nmissings") <- nmissings
	attr(res, "EgoAlter") <- EA
	res
}

##@test_gof.sienaFit gof method for sienaFit
test_gof.sienaFit <- sienaGOF

##@print.sienaGOF siena07 Print method for sienaGOF
print.sienaGOF <- function (x, ...) {
	## require(Matrix)
	levls <- 1:length(x)
	pVals <- sapply(levls, function(i) x[[i]]$p)
	titleStr <- "Monte Carlo Mahalanobis distance test p-value: "

	if (! attr(x,"joined"))
	{
		cat("Siena Goodness of Fit (",
			attr(x,"auxiliaryStatisticName"),"),", length(levls)," periods\n=====\n")
		cat(" >",titleStr, "\n")
		for (i in 1:length(pVals))
		{
			cat(names(x)[i], ": ", round(pVals[i],3), "\n")
		}
		for (i in 1:length(pVals))
		{
			if (x[[i]]$Rank < dim(x[[i]]$Observations)[2])
			{
				cat(" * Note for", names(x)[i],
					": Only", x[[i]]$Rank, "statistics are",
					"necessary in the auxiliary function.\n")
			}
		}
	}
	else
	{
		cat("Siena Goodness of Fit (",
			attr(x,"auxiliaryStatisticName"),"), all periods\n=====\n")
		cat(titleStr, round(pVals[1],3), "\n")
		if (x[[1]]$Rank < dim(x[[1]]$Observations)[2])
			{
				cat("**Note: Only", x[[1]]$Rank, "statistics are",
				"necessary in the auxiliary function.\n")
			}
	}

	if ( attr(x, "twoTailed") )
	{
		cat("-----\nTwo tailed test used.")
	}
	else
	{
		cat("-----\nOne tailed test used ",
		"(i.e. estimated probability of greater distance than observation).\n")
	}
	originalMhd <- attr(x, "originalMahalanobisDistances")
	if (attr(x, "joined")) {
		cat("-----\nCalculated joint MHD = (",
				round(sum(originalMhd),2),") for current model.\n")
	}
	else
	{
		for (j in 1:length(originalMhd)) {
			cat("-----\nCalculated period ", j, " MHD = (",
					round(originalMhd[j],2),") for current model.\n")
		}
	}
	invisible(x)
}

##@summary.sienaGOF siena07 Summary method for sienaGOF
summary.sienaGOF <- function(object, ...) {
	x <- object
	print(x)
	if (sum(attr(x, "nmissings"))> 0){
		cat("\nThere were missing values in the simulated statistics.\n")
		cat("Their number (by period):\n")
		print(t(attr(x, "nmissings")))
	}
	if (attr(x, "scoreTest")) {
		oneStepSpecs <- attr(x, "oneStepSpecs")
		oneStepMhd <- attr(x, "oneStepMahalanobisDistances")
		joinedOneStepMhd <- attr(x, "joinedOneStepMahalanobisDistances")
		cat("\nOne-step estimates and predicted Mahalanobis distances")
		cat(" for modified models.\n")
		if (attr(x, "joined")) {
			for (i in 1:ncol(oneStepSpecs)) {
				a <- cbind(oneStepSpecs[,i, drop=FALSE] )
				b <- matrix( c(joinedOneStepMhd[i] ), ncol=1)
				rownames(b) <- c("MHD")
				a <- rbind(a, b)
				a <- round(a, 3)
				cat("\n**Model including", colnames(a)[1], "\n")
				colnames(a) <- "one-step"
				print(a)
			}
		}
		else
		{
			for (j in 1:length(oneStepMhd)) {
				for (i in 1:ncol(oneStepSpecs)) {
					a <- cbind( oneStepSpecs[,i, drop=FALSE] )
					b <- matrix( oneStepMhd[[j]][i], ncol=1 )
					rownames(b) <- c("MHD")
					a <- rbind(a, b)
					a <- round(a, 3)
					cat("\n**Model including", colnames(a)[1], "\n")
					colnames(a) <- c("one-step")
					print(a)
				}
			}
		}
		cat("\n-----")
	}
	invisible(x)
}

##@plot.sienaGOF siena07 Plot method for sienaGOF
plot.sienaGOF <- function (x, center=FALSE, scale=FALSE, violin=TRUE,
		key=NULL, perc=.05, period=1, showAll=FALSE,
		position=4, fontsize=12, ...)
{
	## require(lattice)
	args <- list(...)
	if (is.null(args$main))
	{
		main=paste("Goodness of Fit of",
				attr(x,"auxiliaryStatisticName"))
		if (!attr(x,"joined"))
		{
			main = paste(main, "Period", period)
		}
	}
	else
	{
		main=args$main
	}

	if (attr(x,"joined"))
	{
		x <- x[[1]]
	}
	else
	{
		x <- x[[period]]
	}
	sims <- x$Simulations
	obs <- x$Observations
	itns <- nrow(sims)
#	vars <- ncol(sims)
	## Need to check for useless statistics here:
	n.obs <- nrow(obs)

	if (showAll)
	{
		screen <- sapply(1:ncol(obs),function(i){
					(sum(is.nan(rbind(sims,obs)[,i])) == 0) })
	}
	else
	{
		screen <- sapply(1:ncol(obs),function(i){
						(sum(is.nan(rbind(sims,obs)[,i])) == 0) }) &
				(diag(var(rbind(sims,obs)))!=0)
		if (any((diag(var(rbind(sims,obs)))==0)))
		{
			cat("Note: some statistics are not plotted because their variance is 0.\n")
			cat("This holds for the statistic")
			if (sum(diag(var(rbind(sims,obs)))==0) > 1){cat("s")}
			cat(": ")
			cat(paste(attr(x,"key")[which(diag(var(rbind(sims,obs)))==0)], sep=", "))
			cat(".\n")
		}
	}

	sims <- sims[,screen, drop=FALSE]
	obs <- obs[,screen, drop=FALSE]
	obsLabels <- round(x$Observations[,screen, drop=FALSE],3)

	sims.min <- apply(sims, 2, min)
	sims.max <- apply(sims, 2, max)
	sims.min <- pmin(sims.min, obs)
	sims.max <- pmax(sims.max, obs)

	if (center)
	{
		sims.median <- apply(sims, 2, median)
		sims <- sapply(1:ncol(sims), function(i)
					(sims[,i] - sims.median[i]) )
		obs <- matrix(sapply(1:ncol(sims), function(i)
							(obs[,i] - sims.median[i])), nrow=n.obs )
		sims.min <- sims.min - sims.median
		sims.max <- sims.max - sims.median
	}
	if (scale)
	{
		sims.range <- sims.max - sims.min + 1e-6
		sims <- sapply(1:ncol(sims), function(i) sims[,i]/(sims.range[i]))
		obs <- matrix(sapply(1:ncol(sims), function(i) obs[,i]/(sims.range[i]))
				, nrow=n.obs )
		sims.min <- sims.min/sims.range
		sims.max <- sims.max/sims.range
	}

	ymin <- 1.05*min(sims.min) - 0.05*max(sims.max)
	ymax <- -0.05*min(sims.min) + 1.05*max(sims.max)

	if (is.null(args$ylab))
	{
		ylabel = "Statistic"
		if (center && scale) {
			ylabel = "Statistic (centered and scaled)"
		}
		else if (scale)
		{
			ylabel = "Statistic (scaled)"
		}
		else if (center)
		{
			ylabel = "Statistic (center)"
		}
		else
		{
			ylabel = "Statistic"
		}
	}
	else
	{
		ylabel = args$ylab
	}

	if (is.null(args$xlab))
	{
		xlabel = paste( paste("p:", round(x$p, 3),
						collapse = " "), collapse = "\n")
	}
	else
	{
		xlabel = args$xlab
	}

	if (is.null(args$cex))
	{
		cexpar <- par("cex")
	}
	else
	{
		cexpar <- args$cex
	}

	if (is.null(args$cex.axis))
	{
		cexaxispar <- par("cex.axis")
	}
	else
	{
		cexaxispar <- args$cex.axis
	}

	if (is.null(args$cex.main))
	{
		cexmainpar <- par("cex.main")
	}
	else
	{
		cexmainpar <- args$cex.main
	}

	if (is.null(args$cex.lab))
	{
		cexlabpar <- par("cex.lab")
	}
	else
	{
		cexlabpar <- args$cex.lab
	}

	if (is.null(args$cex.sub))
	{
		cexsubpar <- par("cex.sub")
	}
	else
	{
		cexsubpar <- args$cex.sub
	}

	xAxis <- (1:sum(screen))

	if (is.null(key))
	{
		if (is.null(attr(x, "key")))
		{
			key=xAxis
		}
		else
		{
			key <- attr(x,"key")[screen]
		}
	}
	else
	{
		key <- key[screen] ## added 1.1-244
		if (length(key) != ncol(obs))
		{
			stop("Key length does not match the number of variates.")
		}
	}

	br <- trellis.par.get("box.rectangle")
	br$col <- 1
	trellis.par.set("box.rectangle", br)
	bu <- trellis.par.get("box.umbrella")
	bu$col <- 1
	trellis.par.set("box.umbrella", bu)
	plot.symbol <- trellis.par.get("plot.symbol")
	plot.symbol$col <- "black"
	plot.symbol$pch <- 4
	plot.symbol$cex <- cexpar  # default 1
	trellis.par.set("plot.symbol", plot.symbol)

#	plot.axis <- trellis.par.get("axis.text")
#	plot.axis$cex <- cexaxispar # default 1
	trellis.par.set("axis.text", list(cex=cexaxispar))

#	plot.xlab <- trellis.par.get("par.xlab.text")
#	plot.xlab$cex <- cexlabpar # default 1
	trellis.par.set("par.xlab.text", list(cex=cexlabpar))

#	plot.ylab <- trellis.par.get("par.ylab.text")
#	plot.ylab$cex <- cexlabpar # default 1
	trellis.par.set("par.ylab.text", list(cex=cexlabpar))

#	plot.main <- trellis.par.get("par.main.text")
#	plot.main$cex <- cexmainpar # default 1.2
	trellis.par.set("par.main.text", list(cex=cexmainpar))

#	plot.font <- trellis.par.get("fontsize")
#	plot.font$text <- fontsize
	trellis.par.set("fontsize", list(text=fontsize))

	panelFunction <- function(..., x=x, y=y, box.ratio){
		ind.lower <- max( round(itns * perc/2), 1)
		ind.upper <- round(itns * (1-perc/2))
		yperc.lower <- sapply(1:ncol(sims), function(i)
					sort(sims[,i])[ind.lower]  )
		yperc.upper <- sapply(1:ncol(sims), function(i)
					sort(sims[,i])[ind.upper]  )
		if (violin) {
			panel.violin(x, y, box.ratio=box.ratio, col = "transparent", ...)
		}
		panel.bwplot(x, y, box.ratio=.1, fill = "gray", ...)
		panel.xyplot(xAxis, yperc.lower, lty=3, col = "gray", lwd=3, type="l",
				...)
		panel.xyplot(xAxis, yperc.upper, lty=3, col = "gray", lwd=3, type="l",
				...)
		for(i in 1:nrow(obs))
		{
			panel.xyplot(xAxis, obs[i,],  col="red", type="l", lwd=1, ...)
			panel.xyplot(xAxis, obs[i,],  col="red", type="p", lwd=3, pch=19,
					...)
			panel.text(xAxis, obs[i,], labels=obsLabels[i,], pos=position)
		}
	}
	bwplot(as.numeric(sims)~rep(xAxis, each=itns), horizontal=FALSE,
			panel = panelFunction, xlab=xlabel, ylab=ylabel, ylim=c(ymin,ymax),
			scales=list(x=list(labels=key), y=list(draw=FALSE)),
			main=main)
}



descriptives <- function(x, ...) UseMethod("descriptives", x)

##@descriptives.sienaGOF siena07 Gives numerical values in the plot.
descriptives.sienaGOF <- function (x, center=FALSE, scale=FALSE,
			perc=.05, key=NULL, period=1, showAll=FALSE, ...)
{
# adapted excerpt from plot.sienaGOF
	if (attr(x,"joined"))
	{
		x <- x[[1]]
	}
	else
	{
		x <- x[[period]]
	}

	sims <- x$Simulations
	obs <- x$Observations
	itns <- nrow(sims)
	screen <- sapply(1:ncol(obs),function(i){
						(sum(is.nan(rbind(sims,obs)[,i])) == 0) })
	if (!showAll)
	{
		screen <- screen & (diag(var(rbind(sims,obs)))!=0)
	}
	sims <- sims[,screen, drop=FALSE]
	obs <- obs[,screen, drop=FALSE]
	## Need to check for useless statistics here:
	n.obs <- nrow(obs)

	if (is.null(key))
	{
		if (is.null(attr(x, "key")))
		{
			key=(1:sum(screen))
		}
		else
		{
			key <- attr(x,"key")[screen]
		}
	}
	else
	{
		if (length(key) != ncol(obs))
		{
			stop("Key length does not match the number of variates.")
		}
		key <- key[screen]
	}

	sims.themin <- apply(sims, 2, min, na.rm=TRUE)
	sims.themax <- apply(sims, 2, max, na.rm=TRUE)
	sims.mean <- apply(sims, 2, mean, na.rm=TRUE)
	sims.sd <- apply(sims, 2, sd, na.rm=TRUE)
	sims.min <- pmin(sims.themin, obs)
	sims.max <- pmax(sims.themax, obs)

	if (center)
	{
		sims.median <- apply(sims, 2, median)
		sims <- sapply(1:ncol(sims), function(i)
					(sims[,i] - sims.median[i]) )
		obs <- matrix(sapply(1:ncol(sims), function(i)
							(obs[,i] - sims.median[i])), nrow=n.obs )
		sims.mean <- sims.mean - sims.median
		sims.min <- sims.min - sims.median
		sims.max <- sims.max - sims.median
	}

	if (scale)
	{
		sims.range <- sims.max - sims.min + 1e-6
		sims <- sapply(1:ncol(sims), function(i) sims[,i]/(sims.range[i]))
		obs <- matrix(sapply(1:ncol(sims), function(i) obs[,i]/(sims.range[i]))
				, nrow=n.obs )
		sims.mean <- sims.mean/sims.range
		sims.min <- sims.min/sims.range
		sims.max <- sims.max/sims.range
		sims.sd <- sims.sd/sims.range
	}

	screen <- sapply(1:ncol(obs),function(i){
						(sum(is.nan(rbind(sims,obs)[,i])) == 0) })
	if (!showAll)
	{
		screen <- screen & (diag(var(rbind(sims,obs)))!=0)
	}
	sims <- sims[,screen, drop=FALSE]
	obs <- obs[,screen, drop=FALSE]
	sims.themin <- sims.themin[screen, drop=FALSE]
	sims.themax <- sims.themax[screen, drop=FALSE]

	ind.lower = max( round(itns * perc/2), 1)
	ind.upper = round(itns * (1-perc/2))
	ind.median = round(itns * 0.5)
	yperc.mid = sapply(1:ncol(sims), function(i)
				sort(sims[,i])[ind.median])
	yperc.lower = sapply(1:ncol(sims), function(i)
				sort(sims[,i])[ind.lower]  )
	yperc.upper = sapply(1:ncol(sims), function(i)
				sort(sims[,i])[ind.upper]  )
	ypg <- sapply(1:ncol(sims), function(i)	mean(sims[,i] > obs[1,i]))
	ypp <- sapply(1:ncol(sims), function(i)	mean(sims[,i] >= obs[1,i]))
    violins <- matrix(NA, 10, ncol(sims))
	violins[1,] <- sims.themax
	violins[2,] <- yperc.upper
	violins[3,] <- sims.mean
	violins[4,] <- yperc.mid
	violins[5,] <- yperc.lower
	violins[6,] <- sims.themin
	violins[7,] <- sims.sd
	violins[8,] <- obs
    violins[9, ] <- ypg
    violins[10, ] <- ypp
    rownames(violins) <- c("max", "perc.upper", "mean", "median",
        "perc.lower", "min", "sd", "obs", "p>", "p>=")
	colnames(violins) <- key
	violins
}

##@changeToStructural sienaGOF Utility to change
# values in X to structural values in S
# X must have values 0, 1.
# NA values in X will be 0 in the result.
changeToStructural <- function(X, S) {
	if (any(S >= 10, na.rm=TRUE))
		{
			S[is.na(S)] <- 0
			S0 <- (S==10)
			S1 <- (S==11)
# the 1* turns the logical into numeric
			X <- 1*((X - S0 + S1)>=1)
		}
	X[is.na(X)] <- 0
	drop0(X)
}

##@changeToNewStructural sienaGOF Utility to change
# values in X to structural values in SAfter
# for tie variables that have no structural values in SBefore.
# X must have values 0, 1.
# NA values in X or SBefore or SAfter will be 0 in the result.
changeToNewStructural <- function(X, SBefore, SAfter) {
		SB <- (SBefore>=10)
		SA <- (SAfter>=10)
		difAB <- (SA > SB)
		if (any(difAB, na.rm=TRUE))
		{
			S0 <- (difAB)*(SAfter==10)
			S1 <- (difAB)*(SAfter==11)
# the 1* turns the logical into numeric
			X <- 1*((X - S0 + S1)>=1)
		}
	X[is.na(X)] <- 0
	drop0(X)
}

##@sparseMatrixExtraction sienaGOF Extracts simulated networks
# This function returns the simulated network as a dgCMatrix;
# this is the "standard" class for sparse numeric matrices
# in the Matrix package. See the help file for "dgCMatrix-class".
# Ties for ordered pairs with a missing value for wave=period or period+1
# are zeroed;
# note that this also is done in RSiena for calculation of target statistics.
# To obtain equality between observed and simulated tie values
# in the case of structurally determined values, the following is done.
# The difficulty lies in the possibility
# that there is change in structural values.
# The reasoning is as follows:
# structural values affect the following period.
# Therefore the simulated values at the end of the period
# should be compared with an observation containing the structural values
# present at the beginning of the period.
# This implies that observations (wave=period+1) should be modified to contain
# the structural values of the preceding observation (wave=period).
# But if there are any tie variables with
# structural values for wave=period+1 and free values for wave=period,
# then there is no valid reference value for the simulations in this period,
# and the simulated tie values should be set to
# the observed (structural) values for wave=period+1.
# Concluding:
# For ties that have a structurally determined value at wave=period,
# this value is used for the observation at the end of the period.
# For ties that have a structurally determined value at the end of the period
# and a free value at the start,
# the structurally determined value at wave=period+1 is used
# for the simulations at the end of the period.
# TODO: Calculate the matrix of structurals and of missings outside
# of this procedure, doing it only once. Perhaps in sienaGOF.
sparseMatrixExtraction <-
	function(i, obsData, sims, period, groupName, varName){
	# require(Matrix)
	isBipartite <- "bipartite" == attr(obsData[[groupName]]$depvars[[varName]], "type")
	dimsOfDepVar<- attr(obsData[[groupName]]$depvars[[varName]], "netdims")
	if (attr(obsData[[groupName]]$depvars[[varName]], "missing"))
	{
		if (attr(obsData[[groupName]]$depvars[[varName]], "sparse"))
		{
			missings <-
				(is.na(obsData[[groupName]]$depvars[[varName]][[period]]) |
				is.na(obsData[[groupName]]$depvars[[varName]][[period+1]]))*1
		}
		else
		{
			missings <- Matrix(
				(is.na(obsData[[groupName]]$depvars[[varName]][,,period]) |
				is.na(obsData[[groupName]]$depvars[[varName]][,,period+1]))*1)
		}
	}
	if (is.null(i))
	{
		# sienaGOF wants the observation;
		# transform structurally fixed values into regular values
		# by "modulo 10" (%%10) operation
		# If preceding observation contains structural values
		# use these to replace the observations at period+1.
		if (attr(obsData[[groupName]]$depvars[[varName]], "sparse"))
		{
			extractedMatrix <- drop0(Matrix(
				obsData[[groupName]]$depvars[[varName]][[period+1]] %% 10))
			extractedMatrix[is.na(extractedMatrix)] <- 0
			if (attr(obsData[[groupName]]$depvars[[varName]], "structural"))
			{
				extractedMatrix <- changeToStructural(extractedMatrix,
					Matrix(obsData[[groupName]]$depvars[[varName]][[period]]))
			}
		}
		else # not sparse
		{
			extractedMatrix <-
			 Matrix(obsData[[groupName]]$depvars[[varName]][,,period+1] %% 10)
			extractedMatrix[is.na(extractedMatrix)] <- 0
			if (attr(obsData[[groupName]]$depvars[[varName]], "structural"))
			{
				extractedMatrix <- changeToStructural(extractedMatrix,
				Matrix(obsData[[groupName]]$depvars[[varName]][,,period]))
			}
		}
		if(!isBipartite){ diag(extractedMatrix) <- 0} # not guaranteed by data input
	}
	else
	{
		# sienaGOF wants the i-th simulation:
		extractedMatrix <- sparseMatrix(
				sims[[i]][[groupName]][[varName]][[period]][,1],
				sims[[i]][[groupName]][[varName]][[period]][,2],
				x=sims[[i]][[groupName]][[varName]][[period]][,3],
				dims=dimsOfDepVar[1:2], repr="T")
		if (attr(obsData[[groupName]]$depvars[[varName]], "structural"))
		{
		# If observation at end of period contains structural values
		# use these to replace the simulations.
			if (attr(obsData[[groupName]]$depvars[[varName]], "sparse"))
			{
				extractedMatrix <- changeToNewStructural(extractedMatrix,
				Matrix(obsData[[groupName]]$depvars[[varName]][[period]]),
				Matrix(obsData[[groupName]]$depvars[[varName]][[period+1]]))
			}
			else # not sparse
			{
				extractedMatrix <- changeToNewStructural(extractedMatrix,
				Matrix(obsData[[groupName]]$depvars[[varName]][,,period]),
				Matrix(obsData[[groupName]]$depvars[[varName]][,,period+1]))
			}
		}
	}
	## Zero missings (the 1* turns the logical into numeric):
	if (attr(obsData[[groupName]]$depvars[[varName]], "missing"))
	{
		extractedMatrix <- 1*drop0((extractedMatrix - missings) > 0)
	}
	extractedMatrix
}

##@sparseMatrixExtraction0 sienaGOF Extracts simulated networks
## simplified version of sparseMatrixExtraction if there are no missings or structurals
sparseMatrixExtraction0 <-
	function(i, obsData, sims, period, groupName, varName){
	# require(Matrix)
	isBipartite <- "bipartite" == attr(obsData[[groupName]]$depvars[[varName]], "type")
	dimsOfDepVar<- attr(obsData[[groupName]]$depvars[[varName]], "netdims")
	if (is.null(i))
	{
		# sienaGOF wants the observation:
		if (attr(obsData[[groupName]]$depvars[[varName]], "sparse"))
		{
			extractedValue <- Matrix(
				obsData[[groupName]]$depvars[[varName]][[period+1]])
			extractedValue[is.na(extractedValue)] <- 0
		}
		else # not sparse
		{
			extractedValue <-
			 Matrix(obsData[[groupName]]$depvars[[varName]][,,period+1])
			extractedValue[is.na(extractedValue)] <- 0
		}
		diag(extractedValue) <- 0 # not guaranteed by data input
		extractedValue <- as(drop0(extractedValue), "sparseMatrix")
	}
	else
	{
		# sienaGOF wants the i-th simulation:
		extractedValue <- sparseMatrix(
				sims[[i]][[groupName]][[varName]][[period]][,1],
				sims[[i]][[groupName]][[varName]][[period]][,2],
				x=1,
				dims=dimsOfDepVar[1:2], repr="T")
	}
	extractedValue
}

##@networkExtraction sienaGOF Extracts simulated networks
# This function provides a standard way of extracting simulated and observed
# networks from the results of a siena07 run.
# It returns the network as an edge list of class "network"
# according to the <network> package (used for package sna).
# Ties for ordered pairs with a missing value for wave=period or period+1
# are zeroed;
# note that this also is done in RSiena for calculation of target statistics.
# Structural values are treated as in sparseMatrixExtraction.
networkExtraction <- function (i, obsData, sims, period, groupName, varName){
	## suppressPackageStartupMessages(require(network))
	dimsOfDepVar<- attr(obsData[[groupName]]$depvars[[varName]], "netdims")
	isbipartite <- (attr(obsData[[groupName]]$depvars[[varName]], "type")
						=="bipartite")
	# For bipartite networks in package <network>,
	# the number of nodes is equal to
	# the number of actors (rows) plus the number of events (columns)
	# with all actors preceding all events.
	# Therefore the bipartiteOffset will come in handy:
	bipartiteOffset <- ifelse(isbipartite, dimsOfDepVar[1], 0)

	# Initialize empty networks:
	if (isbipartite)
	{
		emptyNetwork <- network::network.initialize(dimsOfDepVar[1]+dimsOfDepVar[2],
											bipartite=dimsOfDepVar[1])
	}
	else
	{
		emptyNetwork <- network::network.initialize(dimsOfDepVar[1], bipartite=NULL)
	}
	# Use what was defined in the function above:
	matrixNetwork <- sparseMatrixExtraction(i, obsData, sims,
						period, groupName, varName)
	if (sum(matrixNetwork) <= 0) # else network.edgelist() below will not work
	{
		extractedValue <- emptyNetwork
	}
	else
	{
		tripEV <- mat2triplet(matrixNetwork)
		extractedValue <- network::network.edgelist(
					cbind(tripEV$i, tripEV$j + bipartiteOffset, 1),
					emptyNetwork)
	}
	extractedValue
}

##@behaviorExtraction sienaGOF Extracts simulated behavioral variables.
# This function provides a standard way of extracting simulated and observed
# dependent behavior variables from the results of a siena07 run.
# The result is an integer vector.
# Values for actors with a missing value for wave=period or period+1 are
# transformed to NA.
behaviorExtraction <- function (i, obsData, sims, period, groupName, varName) {
  missings <- is.na(obsData[[groupName]]$depvars[[varName]][,,period]) |
	is.na(obsData[[groupName]]$depvars[[varName]][,,period+1])
  if (is.null(i))
	{
		# sienaGOF wants the observation:
		original <- obsData[[groupName]]$depvars[[varName]][,,period+1]
		original[missings] <- NA
		extractedValue <- original
	}
	else
	{
		#sienaGOF wants the i-th simulation:
		extractedValue <- sims[[i]][[groupName]][[varName]][[period]]
		extractedValue[missings] <- NA
	}
	extractedValue
}

##@OutdegreeDistribution sienaGOF Calculates Outdegree distribution
OutdegreeDistribution <- function(i, obsData, sims, period, groupName, varName,
						levls=0:8, cumulative=TRUE) {
	if (!(attr(obsData[[groupName]]$depvars[[varName]],'missing') |
	      attr(obsData[[groupName]]$depvars[[varName]],'structural')))
	{
		x <- sparseMatrixExtraction0(i, obsData, sims, period, groupName, varName)
	}
	else
	{
	x <- sparseMatrixExtraction(i, obsData, sims, period, groupName, varName)
	}

	a <- apply(x, 1, sum)
	if (cumulative)
	{
		oddi <- sapply(levls, function(i){ sum(a<=i) })
	}
	else
	{
		oddi <- sapply(levls, function(i){ sum(a==i) })
	}
	names(oddi) <- as.character(levls)
	oddi
}

##@IndegreeDistribution sienaGOF Calculates Indegree distribution
IndegreeDistribution <- function (i, obsData, sims, period, groupName, varName,
						levls=0:8, cumulative=TRUE){
	if (!(attr(obsData[[groupName]]$depvars[[varName]],'missing') |
	      attr(obsData[[groupName]]$depvars[[varName]],'structural')))
	{
		x <- sparseMatrixExtraction0(i, obsData, sims, period, groupName, varName)
	}
	else
	{
  x <- sparseMatrixExtraction(i, obsData, sims, period, groupName, varName)
	}
  a <- apply(x, 2, sum)
  if (cumulative)
  {
	iddi <- sapply(levls, function(i){ sum(a<=i) })
  }
  else
  {
	iddi <- sapply(levls, function(i){ sum(a==i) })
  }
  names(iddi) <- as.character(levls)
  iddi
}

##@BehaviorDistribution sienaGOF Calculates behavior distribution
BehaviorDistribution <- function (i, obsData, sims, period, groupName, varName,
							levls=NULL, cumulative=TRUE){
	x <- behaviorExtraction(i, obsData, sims, period, groupName, varName)
	if (is.null(levls))
	{
		levls <- attr(obsData[[groupName]]$depvars[[varName]],"behRange")[1]:
					attr(obsData[[groupName]]$depvars[[varName]],"behRange")[2]
	}
	if (cumulative)
	{
		bdi <- sapply(levls, function(i){ sum(x<=i, na.rm=TRUE) })
	}
	else
	{
	bdi <- sapply(levls, function(i){ sum(x==i, na.rm=TRUE) })
	}
	names(bdi) <- as.character(levls)
	bdi
}

##@mixedTriadCensus sienaGOF Calculates mixed triad census
# Contributed by Christoph Stadtfeld.
# For more details see
# Hollway, J., Lomi, A., Pallotti, F., & Stadtfeld, C. (2017)
# Multilevel social spaces: The network dynamics of organizational fields
# Network Science, 5(2), 187-212. doi:10.1017/nws.2017.8
#
# https://www.cambridge.org/core/journals/network-science/article/multilevel-social-spaces-the-network-dynamics-of-organizational-fields/602BB810A44497EBDE2A111A6C2771A3
#
# The function is called with two varName parameters, e.g.
# gof <- sienaGOF(res, mixedTriadCensus, varName = c("oneModeNet", "twoModeNet"))
#
mixedTriadCensus <- function (i, obsData, sims, period, groupName, varName) {
  if (length(varName) != 2) stop("mixedTriadCensus expects two varName parameters")
  varName1 <- varName[1]
  varName2 <- varName[2]

  # get matrices
  m1 <- as.matrix(sparseMatrixExtraction(i, obsData, sims, period, groupName,
											varName = varName1))
  m2 <- as.matrix(sparseMatrixExtraction(i, obsData, sims, period, groupName,
											varName = varName2))

  # check if the first network is one-mode, the second (potentially) bipartite
  if (dim(m1)[1] != dim(m1)[2]) {
	stop("Error: The first element in varName must be one-mode")
  }
  if (dim(m1)[1] != dim(m2)[1]) {
	stop("Error: Both elements in varName must have the same node sets")
  }
  # complement of a binary matrix
  cp <- function(m) (-m + 1)

  # all ties of two-paths in the one mode network
  onemode.reciprocal <- m1 * t(m1)
  onemode.forward <- m1 * cp(t(m1))
  onemode.backward <- cp(m1) * t(m1)
  onemode.null <- cp(m1) * cp(t(m1))
  diag(onemode.forward) <- 0
  diag(onemode.backward) <- 0
  diag(onemode.null) <- 0

  # one mode projections to the first mode
  bipartite.twopath <- m2 %*% t(m2)
  bipartite.null <- cp(m2) %*% cp(t(m2))
  bipartite.onestep1 <- m2 %*% cp(t(m2))
  bipartite.onestep2 <- cp(m2) %*% t(m2)
  diag(bipartite.twopath) <- 0
  diag(bipartite.null) <- 0
  diag(bipartite.onestep1) <- 0
  diag(bipartite.onestep2) <- 0

  # The coding is explained in the above referenced paper, pages 191-192
  # The first digit refers to the number of two-mode ties,
  # the second to the number of one-mode ties in triadic configurations
  # with two first-mode nodes, and one second-mode node.
  res <- c("22" =  sum(onemode.reciprocal * bipartite.twopath) / 2,
           "21" = (sum(onemode.forward * bipartite.twopath) +
						sum(onemode.backward * bipartite.twopath)) / 2,
           "20" =  sum(onemode.null * bipartite.twopath) / 2,
           "12" = (sum(onemode.reciprocal * bipartite.onestep1) +
						sum(onemode.reciprocal * bipartite.onestep2)) / 2,
           "11D" = (sum(onemode.forward * bipartite.onestep1) +
						sum(onemode.backward * bipartite.onestep2)) / 2,
           "11U" = (sum(onemode.forward * bipartite.onestep2) +
						sum(onemode.backward * bipartite.onestep1)) / 2,
           "10" = (sum(onemode.null * bipartite.onestep2) +
						sum(onemode.null * bipartite.onestep1)) / 2,
           "02" =  sum(onemode.reciprocal * bipartite.null) / 2,
           "01" = (sum(onemode.forward * bipartite.null) +
						sum(onemode.backward * bipartite.null)) / 2,
           "00" = sum(onemode.null * bipartite.null) / 2
  )

  # An ad-hoc check. Error should never be thrown.
  dim1 <- dim(m2)[1]
  dim2 <- dim(m2)[2]
  nTriads <- dim2 * dim1 * (dim1 - 1) / 2
  if (sum(res) != nTriads){
  stop(paste("Error in calculation. More than", nTriads,
						"triads counted (", sum(res), ")"))
  }
  res
}

##@TriadCensus sienaGOF Calculates triad census
# Contributed by Christoph Stadtfeld.
#
# Implementation of the Batagelj-Mrvar (Social Networks, 2001) algorithm
# based on the summary in the thesis of Sindhuja
#
TriadCensus <- function (i, obsData, sims, period, groupName, varName, levls = 1:16) {
  # get matrix and prepare data
  mat <- as.matrix(sparseMatrixExtraction(i, obsData, sims, period, groupName, varName = varName))
  N <- nrow(mat)
  # matrix with reciprocal ties
  matReciprocal <- mat + t(mat)
  # matrix with direction information for triad
  matDirected <- mat - t(mat)
  matDirected[matDirected == -1] <- 2
  matDirected[matReciprocal == 2] <- 3
  matDirected <- matDirected + 1
  # reciproal matrix with ties from higher to lower IDs
  matHigher <- matReciprocal
  matHigher[lower.tri(matHigher)] <- 0
  # neighbors lookup
  neighbors<- apply(matReciprocal, 1, function(x) which(x > 0))
  # neighbors with lower ids
  neighborsHigher <- apply(matHigher, 1, function(x) which(x > 0))

  # lookup table for 64 triad types
  # i->j, j->k, i->k
  # 1: empty, 2: forward, 3: backward, 4: reciprocal
  # order as in vector tc
  lookup <- array(NA, dim = rep(4,3))
  lookup[1,1,1] <- 1
  lookup[2,1,1] <- lookup[1,2,1] <- lookup[1,1,2] <- lookup[3,1,1] <- lookup[1,3,1] <- lookup[1,1,3] <- 2
  lookup[4,1,1] <- lookup[1,4,1] <- lookup[1,1,4] <- 3
  lookup[2,1,2] <- lookup[3,2,1] <- lookup[1,3,3] <- 4
  lookup[2,3,1] <- lookup[3,1,3] <- lookup[1,2,2] <- 5
  lookup[2,2,1] <- lookup[3,3,1] <- lookup[2,1,3] <- lookup[3,1,2] <- lookup[1,2,3] <- lookup[1,3,2] <- 6
  lookup[4,3,1] <- lookup[4,1,3] <- lookup[2,4,1] <- lookup[1,4,2] <- lookup[3,1,4] <- lookup[1,2,4] <- 7
  lookup[4,2,1] <- lookup[4,1,2] <- lookup[3,4,1] <- lookup[1,4,3] <- lookup[2,1,4] <- lookup[1,3,4] <- 8
  lookup[2,2,2] <- lookup[2,3,3] <- lookup[2,3,2] <- lookup[3,3,3] <- lookup[3,2,2] <- lookup[3,2,3] <- 9
  lookup[2,2,3] <- lookup[3,3,2] <- 10  # 3-cycle
  lookup[4,4,1] <- lookup[4,1,4] <- lookup[1,4,4] <- 11
  lookup[2,4,2] <- lookup[3,2,4] <- lookup[4,3,3] <- 12
  lookup[2,3,4] <- lookup[3,4,3] <- lookup[4,2,2] <- 13
  lookup[2,2,4] <- lookup[3,3,4] <- lookup[2,4,3] <- lookup[3,4,2] <- lookup[4,2,3] <- lookup[4,3,2]<- 14
  lookup[2,4,4] <- lookup[4,2,4] <- lookup[4,4,2] <- lookup[3,4,4] <- lookup[4,3,4] <- lookup[4,4,3] <- 15
  lookup[4,4,4] <- 16

  # initialize triad census
  tc <- c("003"  = 0,
          "012"  = 0,
          "102"  = 0,
          "021D" = 0,
          "021U" = 0,
          "021C" = 0,
          "111D" = 0,
          "111U" = 0,
          "030T" = 0,
          "030C" = 0,
          "201"  = 0,
          "120D" = 0,
          "120U" = 0,
          "120C" = 0,
          "210"  = 0,
          "300"  = 0)

  # iterate through all non-empty dyads (from lower to higher ID)
  if (length(neighborsHigher) > 0){ # else mat is the zero matrix
	for(ii in 1:N){
		for(j in neighborsHigher[[ii]]){
      # set of nodes that are linked to ii and j
		third <- setdiff( union(neighbors[[ii]], neighbors[[j]]),
                    c(ii, j) )
		# store triads with just one tie
		triadType <- ifelse(matReciprocal[ii,j] == 2, 3, 2)
		tc[triadType] <- tc[triadType] + N - length(third) - 2
		for (k in third){
        # only store triads once
			if(j < k || ( ii < k && k < j && !(k %in% neighbors[[ii]]) ) ){
				t1 <- matDirected[ii,j]
				t2 <- matDirected[j,k]
				t3 <- matDirected[ii,k]
				triadType <- lookup[t1, t2, t3]
				tc[triadType] <- tc[triadType] + 1
				}
			}
		}
	}
  }
  # assign residual to empty triad count
  tc[1] <- 1/6 * N*(N-1)*(N-2) - sum(tc[2:16])
  tc[levls]
}


##@dyadicCov sienaGOF Auxiliary variable for dyadic covariate
#
# An auxiliary function calculating the number of ties
# for subsets of ordered pairs corresponding to
# certain values of the categorical dyadic covariate dc.
# dc should be a matrix of the same dimensions as
# the dependent variable,
# or an array where the third dimension is time.
# Frequencies of ties with dc == 0 are not counted.
dyadicCov <-  function (i, obsData, sims, period, groupName, varName, dc){
	m <- sparseMatrixExtraction(i, obsData, sims, period, groupName, varName)
    # note that m*dc is a sparse matrix, too:
	if (length(dim(dc))==3)
	{
		tmdyv <- table((m * dc[,,period])@x, useNA = "no")
	}
	else
	{
		tmdyv <- table((m * dc)@x, useNA = "no")
	}
	values <- unique(as.vector(dc))
	tdyv <- sort(values[!is.na(values)])
	# if 0 is included, take it out:
	zeros <- which(tdyv==0)
	if (length(zeros)>0)
	{
		tdyv <- tdyv[-zeros]
	}
	# Now we want to construct the table of numbers of m*dyv;
	# and categories in dc not represented in m*dyv should get a 0.
	# First make a named vector of the correct length with 0s in place.
	ttmdyv <- tdyv
	ttmdyv <- 0*tdyv
	names(ttmdyv) <- tdyv
	dims <- dimnames(tmdyv)[[1]]
	ttmdyv[dims] <- tmdyv # The other entries remain 0
	ttmdyv
}

egoAlterCombi <- function (i, obsData, sims, period, groupName, varName,
     trafo=NULL)
{
# An auxiliary function calculating the number of ties
# for each ego-alter combination of values of the dependent behavior variable;
# the dependent variable is transformed by trafo.
	if (length(varName) != 2){
		stop("egoAlterCombi expects two varName parameters")
	}
	if (is.null(trafo)){
		trafo <- function(x){x}
	}
	varName1 <- varName[1]
	varName2 <- varName[2]
	m <- sparseMatrixExtraction(i, obsData, sims, period, groupName,
		varName1)
	x <- behaviorExtraction(i, obsData, sims, period, groupName,
		varName2)
	m <- m[!is.na(x), !is.na(x)]
	x <- x[!is.na(x)]
	brange <- attr(obsData[[groupName]]$depvars[[varName2]],
			"behRange")[1]:attr(obsData[[groupName]]$depvars[[varName2]],
			"behRange")[2]
	combi.egoalter <- outer(10*trafo(x), trafo(x) ,'+')
	possible.pairs <-
		sort(unique(as.vector(outer(10*trafo(brange), trafo(brange), '+'))))
	if ((length(possible.pairs) >= 100) && is.null(i))
	{
	warning("The transformation used for egoAlterCombi has too many values.")
	}
	tmeax <- table((m * combi.egoalter)@x, useNA = "no")
	ppnames <- as.character(possible.pairs)
	teax <- setNames(0*possible.pairs, ppnames)
	teax[dimnames(tmeax)[[1]]] <- tmeax
	# pad names with leading 0s, if necessary:
	pp.names <- ifelse(nchar(ppnames)==1, paste("0",ppnames,sep=""),ppnames)
	names(teax) <- pp.names
	attr(teax, "EgoAlter") <- TRUE
	teax
}

egoAlterCovarComb <- function (i, obsData, sims, period, groupName, varName,
     covar)
{
# An auxiliary function calculating the number of ties
# for each ego-alter combination of values of covar.
	varName1 <- varName[1]
	varName2 <- varName[2]
	m <- sparseMatrixExtraction(i, obsData, sims, period, groupName,
		varName1)
	m <- m[!is.na(covar), !is.na(covar)]
	x <- covar[!is.na(covar)]
	branges <- range(covar, na.rm = TRUE)
	brange <- branges[1]:branges[2]
	combi.egoalter <- outer(10*x, x ,'+')
	possible.pairs <-
		sort(unique(as.vector(outer(10*brange, brange, '+'))))
	if ((length(possible.pairs) >= 100) && is.null(i))
	{
	warning("The variables used for egoAlterCovarComb has too many values.")
	}
	tmeax <- table((m * combi.egoalter)@x, useNA = "no")
	ppnames <- as.character(possible.pairs)
	teax <- setNames(0*possible.pairs, ppnames)
	teax[dimnames(tmeax)[[1]]] <- tmeax
	# pad names with leading 0s, if necessary:
	pp.names <- ifelse(nchar(ppnames)==1, paste("0",ppnames,sep=""),ppnames)
	names(teax) <- pp.names
	attr(teax, "EgoAlter") <- TRUE
	teax
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: effectsDocumentation.r
# *
# * Description: This module contains a function for documenting the shortNames
# * and other fields of an effects object.
# *****************************************************************************/

##@effectsDocumentation Documentation
effectsDocumentation <- function(effects= NULL, type="html",
	display=(type=="html"),
	filename=ifelse(is.null(effects), "effects", deparse(substitute(effects))))
{
	## require(xtable)
	if (is.null(effects)) {
		x <- RSiena::allEffects[, c("effectGroup", "effectName", "shortName",
			"endowment", "interaction1", "interaction2", "parm", "interactionType")]
	} else {
		if (!inherits(effects, "sienaEffects")) {
			stop(paste(deparse(substitute(effects)), "is not a sienaEffects object."))
		}
		x <- as.data.frame(effects[, c("name", "effectName", "shortName", "type",
			"interaction1", "interaction2", "parm", "interactionType")])
	}
#  if (any(x$type=="gmm"))
#  {
#    x <- x[-which(x$type=="gmm"),]
#  }
	storage.mode(x$parm) <- "integer"
	names(x)[4] <- ifelse(is.null(effects), "endow?", "type")
	names(x)[5] <- "covar1"
	names(x)[6] <- "covar2"
	names(x)[8] <- "interactionType"
	if (is.null(effects))
	{
		x$row <- as.integer(row.names(x))
	}
	else
	{
		x$row <- seq_len(dim(x)[1])
	}
	x <- x[, c(9, 1:8)]

	myorder <- c("nonSymmetricRate",
				"covarNonSymmetricRate",

				"symmetricRate",
				"covarSymmetricRate",

				"bipartiteRate",
				"covarBipartiteRate",

				"behaviorRate",
				"behaviorOneModeRate",
				"behaviorSymmetricRate",
				"covarBehaviorOneModeRate",
				"covarBehaviorSymmetricRate",
				"behaviorBipartiteRate",
				"covarBehaviorRate",
				"continuousRate",

				"nonSymmetricObjective",
				"dyadObjective",
				"covarNonSymmetricObjective",
				"unspecifiedNetInteraction",
				"nonSymmetricNonSymmetricObjective",
				"nonSymmetricSymmetricObjective",
				"nonSymmetricSymmetricSObjective",
				"nonSymmetricBipartiteObjective",
				"covarNetNetObjective",
				"covarANetNetObjective",
				"covarABNetNetObjective",
				"covarBNetNetObjective",
				"tripleNetworkObjective",
				"dyadANetNetObjective",
				"covarABipNetObjective",
				"dyadANetNetObjective",
				"dyadBNetNetObjective",
				"settingsObjective",

				"symmetricObjective",
				"covarSymmetricObjective",

				"bipartiteObjective",
				"dyadBipartiteObjective",
				"dyadSecondBipartiteObjective",
				"covarBipartiteObjective",
				"doubleCovarNetObjective", 
				"unspecifiedNetInteraction",
				"bipartiteNonSymmetricObjective",
				"bipartiteSymmetricObjective",
				"bipartiteBipartiteObjective",

				"unspecifiedNetInteraction",

				"behaviorObjective",
				"behaviorOneModeObjective",
				"behaviorSymmetricObjective",
				"behaviorBipartiteObjective",
				"behaviorOneOneModeObjective",
				"behaviorSymSymObjective",
				"behaviorOneModeSymObjective",
				"behaviorBipBipObjective",
				"covarBehaviorObjective",
				"covarBehaviorNetObjective",
				"dyadBehaviorNetObjective",
				"covarABehaviorBipartiteObjective",
				"covarBBehaviorBipartiteObjective",
				"unspecifiedBehaviorInteraction",
				"continuousFeedback",
				"continuousWiener",
				"continuousIntercept",
				"continuousOneModeObjective",
				"continuousBipartiteObjective",
				"unspecifiedContinuousInteraction")

	mytab <- table(RSiena::allEffects[,1])

	addtorowPos <- cumsum(c(0, mytab[myorder]))[1:length(myorder)]
	addtorowText <- names(mytab[myorder])
	x[is.na(x)] <- "FALSE" ## endow? field

	if (type == "latex") {
		addtorowText <- paste(" \\hline \\multicolumn{4}{l}{", addtorowText,
			"}\\\\ \\hline")
		addtorowText[1] <- paste(" \\endhead ", addtorowText[1], collapse = "")
	} else {
		x[x == ""] <- "<br>"
		addtorowText <- paste(" <TR> <TD colspan=\"8\" >", addtorowText,
			"</TD> </TR>")
	}
	add.to.row <- NULL
	x$interactionType[x$type=='gmm'] <- ''
	if (is.null(effects))
	{
		add.to.row$pos <- lapply(addtorowPos, function(x)x)
		add.to.row$command <- as.vector(sapply(addtorowText, function(x)x))
		order2 <- match(myorder, x[, 2])
		order3 <- as.vector(mytab[myorder])
		order4 <- unlist(apply(cbind(order2, order3), 1,
				function(x) x[1]:(max(x[1] + x[2] - 1, 1))))
		y <- x[order4, -2]
	}
	else
	{
		y <- rbind(x[x$type!='gmm',], x[x$type=='gmm',])
	}
	row.names(y) <- 1:nrow(y)

	if (type =="latex")
	{
		filename2 <- paste(filename,".tex", sep="", collapse="")
		includefile <- paste(filename,".include.tex", sep="", collapse="")
		includepart <- paste(filename,".include", sep="", collapse="")
		print(xtable::xtable(y), add.to.row=add.to.row, file=includefile,
			tabular.environment="longtable", hline.after=c(-1),
			floating=FALSE, include.rownames=FALSE)

		cat(file=filename2, "\\documentclass[12pt,a4paper]{article}\n",
			"\\usepackage[pdftex,dvipsnames]{color}\n",
			"\\usepackage[landscape]{geometry}\n",
			"\\usepackage{longtable}\n",
			"\\pagestyle{empty}\n",
			"\\textheight=7.25in\n",
			"\\topmargin=-1in\n",
			"\\evensidemargin=-0in\n",
			"\\oddsidemargin=-0in\n",
			"\\marginparwidth=-0in\n",
			"\\begin{document}\n",
			"\\include{", includepart, "}\n",
			"\\end{document}\n", sep=""
			)		
		cat("Effects documentation written to file", filename2, ".\n")
	}
	else
	{
		filename <- paste(filename,".html", sep="", collapse="")
		print(xtable::xtable(y), add.to.row=add.to.row,
			file=filename,
			type="html", hline.after=c(-1),
			sanitize.text.function=function(x){x},
			floating=FALSE, include.rownames=FALSE,
			html.table.attributes="border=1 cellpadding=3 cellspacing=0")
		if (display) {
			browseURL(paste("file://", getwd(), "/", filename, collapse = "",
				sep = ""))
			}
		cat("Effects documentation written to file", filename, ".\n")
	}
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: effects.r
# *
# * Description: This module contains the code for the creation of the
# * effects object to go with a Siena data object or group object.
# *****************************************************************************/
##@substituteNames DataCreate replace xxxxxx, yyyyyy, zzzzzz
substituteNames <- function(nameVectors, xName = NULL, yName = NULL, zName = NULL) {
	effects <- nameVectors[, c("effectName", "functionName", "interaction1",
		"interaction2")]
	if (!is.null(xName)) {
		effects <- sapply(effects, function(x) gsub("xxxxxx", xName, x))
	}
	if (!is.null(yName)) {
		effects <- sapply(effects, function(x) gsub("yyyyyy", yName, x))
	}
	if (!is.null(zName)) {
		effects <- sapply(effects, function(x) gsub("zzzzzz", zName, x))
	}
	nameVectors[, c("effectName", "functionName", "interaction1", "interaction2")] <- effects
	nameVectors
}

##@createEffects DataCreate Extract required rows and change text
createEffects <- function(effectGroup, xName=NULL, yName=NULL, zName = NULL,
	name, groupName, group, netType)
{
	effects <- RSiena::allEffects[RSiena::allEffects$effectGroup == effectGroup, ]
	if (nrow(effects) == 0)
	{
		stop("empty effect group", effectGroup)
	}
	if (any(is.na(effects$effectName)))
	{
		stop("missing effect name")
	}
	effects <- substituteNames(effects, xName, yName, zName)
	effects$effectGroup <- NULL
	nn <- nrow(effects)
	# ignore the endowment field of 'gmm' type effects
	effects$endowment[which(effects$type == 'gmm')] <- FALSE
	# If we have some with a valid endowment field in the selection.
	if (!all(is.na(effects$endowment))) {
		# create a new vector with the right dimension
		neweffects <- effects[rep(1:nn, times=(1 + 2 * as.numeric(effects$endowment))), ]
		# fill with eval, endow and creation effects
		neweffects$type <- unlist(by(effects, 1:nn,
				function(e) {
					if (e$endowment) {
						c('eval', 'endow', 'creation')
					} else {
						e$type
					}
				}))
		effects <- neweffects
		nn <- nrow(effects)
	}
	effects$endowment <-  NULL
	effectFn <- vector('list', nn)
	statisticFn <- vector('list', nn)
	effects$effectFn <- effectFn
	effects$statisticFn <- statisticFn
	effects$netType <- netType
	effects$groupName <- groupName
	effects$group <- group
	effectsname <- rep(name, nn)
	effects <- data.frame(name=effectsname, effects, stringsAsFactors=FALSE)
	effects
}

##@DoubleAttributesChecked utility function for getEffects
DoubleAttributesChecked <- function(cova1, cova2)
{
	if (is.null(attr(cova1,"lowIntegers")) | is.null(attr(cova2,"lowIntegers")))
	{
		stop("Please use function sienaDataCreate for RSiena version at least 1.4.10")
	}
	(attr(cova1,"lowIntegers") && attr(cova2,"lowIntegers"))
}

##@getEffects DataCreate create effects object
getEffects <- function(x, nintn = 10, behNintn=4, getDocumentation=FALSE, onePeriodSde=FALSE)
{
	##@duplicateDataFrameRow internal getEffects Put period numbers in
	duplicateDataFrameRow <- function(x, n)
	{
		tmp <- NULL
		for (i in 1:n)
		{
			xx <- x
			xx[, c("effectName", "functionName", "period")] <-
				sub("nnnnnn", periodNos[i], xx[, c("effectName", "functionName",
						"period")])
			tmp <- rbind(tmp, xx)
		}
		tmp
	}

	##@networkRateEffects internal getEffects create a set of rate effects
	networkRateEffects <- function(depvar, varname, symmetric, bipartite)
	{
		if (symmetric)
		{
			rateEffects <- createEffects("symmetricRate", varname, name=varname,
				groupName=groupName, group=group,
				netType=netType)
		}
		else if (bipartite)
		{
			rateEffects <- createEffects("bipartiteRate", varname, name=varname,
				groupName=groupName, group=group,
				netType=netType)
		}
		else
		{
			rateEffects <- createEffects("nonSymmetricRate", varname,
				name=varname,
				groupName=groupName, group=group,
				netType=netType)
		}
		if (observations == 1)
		{
			rateEffects <- rateEffects[-2, ] ## remove the extra period
		}
		else
		{
			## get correct number of rows
			rateEffects <- rbind(duplicateDataFrameRow(rateEffects[2, ],
					observations),
				rateEffects[-c(1, 2), ])
		}
		rateEffects
	}
	##@oneModeNet internal getEffects process a one mode net
	oneModeNet <- function(depvar, varname)
	{
		symmetric <- attr(depvar, "symmetric")
		nodeSet <- attr(depvar, 'nodeSet')

		rateEffects <- networkRateEffects(depvar, varname, symmetric=symmetric,
			bipartite=FALSE)

		if (symmetric)
		{
			objEffects <- createEffects("symmetricObjective", varname,
				name=varname,
				groupName=groupName, group=group,
				netType=netType)
		}
		else
		{
			objEffects <- createEffects("nonSymmetricObjective", varname,
				name=varname,
				groupName=groupName, group=group,
				netType=netType)
		}
		for (j in seq(along = xx$dycCovars))
		{
			if (attr(xx$dycCovars[[j]], "type") == "oneMode" &&
				attr(xx$dycCovars[[j]], 'nodeSet')[1] == nodeSet)
			{
				objEffects <- rbind(objEffects,
					createEffects("dyadObjective",
						names(xx$dycCovars)[j],
						name=varname,
						groupName=groupName,
						group=group,
						netType=netType))
			}
		}
		for (j in seq(along = xx$dyvCovars))
		{
			if (attr(xx$dyvCovars[[j]], "type") == "oneMode" &&
				attr(xx$dyvCovars[[j]], 'nodeSet')[1] == nodeSet)
			{
				objEffects <- rbind(objEffects,
					createEffects("dyadObjective",
						names(xx$dyvCovars)[j],
						name=varname,
						groupName=groupName,
						group=group,
						netType=netType))
			}
		}
		for (j in seq(along = xx$cCovars))
		{
			if (attr(xx$cCovars[[j]], 'nodeSet') == nodeSet)
			{
				tmp <- covarOneModeEff(names(xx$cCovars)[j],
					attr(xx$cCovars[[j]], 'poszvar'),
					attr(xx$cCovars[[j]], 'moreThan2'),
					symmetric, constant=TRUE, name=varname)
				objEffects <-  rbind(objEffects, tmp$objEff)
				rateEffects <- rbind(rateEffects, tmp$rateEff)

				for (jj in seq(along = xx$cCovars))
				{
					if ((jj != j) && 
					(DoubleAttributesChecked(xx$cCovars[[j]], xx$cCovars[[jj]])))
					{
						tmp <- doubleCovarNetEff(names(xx$cCovars)[j],
						names(xx$cCovars)[jj], name=varname)
						objEffects <-  rbind(objEffects, tmp$objEff)
					}
				}
				for (jj in seq(along = xx$vCovars))
				{
					if (DoubleAttributesChecked(xx$cCovars[[j]], xx$vCovars[[jj]]))
					{
						tmp <- doubleCovarNetEff(names(xx$cCovars)[j],
									names(xx$vCovars)[jj], name=varname)
						objEffects <-  rbind(objEffects, tmp$objEff)
					}		
				}
# I did not do the following, because it would lead to too many
# unneeded effects. Keep it as an example of how it could be done.
#				for (jj in seq(along = xx$depvars))
#				{
#					if (types[jj] =='behavior')
#					{
#						tmp <- doubleCovarNetEff(names(xx$cCovars)[j],
#									names(xx$depvars)[jj], name=varname)
#						objEffects <-  rbind(objEffects, tmp$objEff)
#					}
#				}
			}
		}
		for (j in seq(along=xx$depvars))
		{
			if (types[j] %in% c('behavior', 'continuous') &&
				attr(xx$depvars[[j]], 'nodeSet') == nodeSet)
			{
				tmp <- covarOneModeEff(names(xx$depvars)[j],
					poszvar=TRUE,
					attr(xx$depvars[[j]], 'moreThan2'),
					symmetric, constant=FALSE, name=varname)
				objEffects <- rbind(objEffects, tmp$objEff)
				rateEffects <- rbind(rateEffects, tmp$rateEff)
			}
		}
		for (j in seq(along=xx$vCovars))
		{			
			if (attr(xx$vCovars[[j]], 'nodeSet') == nodeSet)
			{
				tmp <- covarOneModeEff(names(xx$vCovars)[j],
					attr(xx$vCovars[[j]], 'poszvar'),
					attr(xx$vCovars[[j]], 'moreThan2'),
					symmetric, constant=FALSE, name=varname)
				objEffects <- rbind(objEffects,tmp$objEff)
				rateEffects<- rbind(rateEffects,tmp$rateEff)
			}
		}

		if (length(xx$cCovars) + length(xx$vCovars) +
			length(xx$dycCovars) + length(xx$dyvCovars) +
			length(types=='behavior') + length(types=='continuous') > 0)
		{
			interaction <- createEffects("unspecifiedNetInteraction",
				name=varname,
				groupName=groupName, group=group,
				netType=netType)
			objEffects <-  rbind(objEffects, interaction[rep(1:nrow(interaction), nintn), ])
		}

		for (j in seq(along=xx$depvars))
		{
			otherName <- names(xx$depvars)[j]
			if (types[j] == 'oneMode' &&
				attr(xx$depvars[[j]], 'nodeSet') == nodeSet &&
				varname != otherName)
			{
				if (attr(xx$depvars[[j]], "symmetric"))
				{
					objEffects <-
						rbind(objEffects,
							createEffects("nonSymmetricSymmetricSObjective",
								otherName, name=varname,
								groupName=groupName, group=group,
								netType=netType))
					objEffects <-
						rbind(objEffects,
							createEffects("nonSymmetricSymmetricObjective",
								otherName, name=varname,
								groupName=groupName, group=group,
								netType=netType))
				}
				else
				{
					objEffects <-
						rbind(objEffects,
							createEffects("nonSymmetricNonSymmetricObjective",
								otherName, name=varname,
								groupName=groupName, group=group,
								netType=netType))
				}
			}

			for (k in seq(along=xx$depvars))
			{
				# This is an effect group that can be used
				# for any three dependent networks, provided the first is one-mode;
				# the receiver sets of the second and third networks
				# must be the same.
				# Note that some or all of the three networks may be the same.
				if ((types[j] == "bipartite") && (types[k] == "bipartite"))
				{
					suitable <- (attr(xx$depvars[[j]], 'nodeSet')[2] ==
						attr(xx$depvars[[k]], 'nodeSet')[2])
				}
				else
				{
					suitable <- ((types[j] == "oneMode") && (types[k] == "oneMode"))
				}
				if (suitable)
				{
					thirdName <- names(xx$depvars)[k]
					objEffects <-
						rbind(objEffects,
							createEffects("tripleNetworkObjective",
								otherName, thirdName, name=varname,
								groupName=groupName, group=group,
								netType=netType))
				}
			}
			if (types[j] == 'bipartite' &&
				(attr(xx$depvars[[j]], 'nodeSet')[1] == nodeSet))
			{
				objEffects <-
					rbind(objEffects,
						createEffects("nonSymmetricBipartiteObjective",
							otherName, name=varname,
							groupName=groupName, group=group,
							netType=netType))
			}
			if ((!(types[j] %in% c('behavior', 'continuous'))) && (varname != otherName))
			{
				for (k in seq(along=xx$cCovars))
				{
					if (attr(xx$cCovars[[k]], 'nodeSet') == nodeSet)
					{
						objEffects <-
							rbind(objEffects,
								covarNetNetEff(otherName,
									covarname=names(xx$cCovars)[k],
									attr(xx$depvars[[j]], 'nodeSet'),
									attr(xx$cCovars[[k]], 'nodeSet'),
									attr(xx$cCovars[[k]], 'poszvar'),
									name=varname))
					}
				}
				for (k in seq(along=xx$vCovars))
				{
					if (attr(xx$vCovars[[k]], 'nodeSet') == nodeSet)
					{
						objEffects <-
							rbind(objEffects,
								covarNetNetEff(otherName,
									covarname=names(xx$vCovars)[k],
									attr(xx$depvars[[j]], 'nodeSet'),
									attr(xx$vCovars[[k]], 'nodeSet'),
									attr(xx$vCovars[[k]], 'poszvar'),
									name=varname))
					}
				}
				for (k in seq(along = xx$dycCovars))
				{
					if (attr(xx$dycCovars[[k]], "type") == "oneMode" &&
						attr(xx$dycCovars[[k]], 'nodeSet')[1] == nodeSet)
					{
						othervarname <- names(xx$dycCovars)[k]
						objEffects <- rbind(objEffects,
							createEffects("dyadANetNetObjective",
								varname, otherName, othervarname,
								name=varname, groupName=groupName,
								group=group, netType=netType))
					}
				}
				for (k in seq(along = xx$dyvCovars))
				{
					if (attr(xx$dyvCovars[[k]], "type") == "oneMode" &&
						attr(xx$dyvCovars[[k]], 'nodeSet')[1] == nodeSet)
					{
						othervarname <- names(xx$dyvCovars)[k]
						objEffects <- rbind(objEffects,
							createEffects("dyadANetNetObjective",
								varname, otherName, othervarname,
								name=varname, groupName=groupName,
								group=group, netType=netType))
					}
				}
				for (k in seq(along = xx$dycCovars))
				{
					nodset_j <- ifelse((attr(xx$depvars[[j]], "type") == "oneMode"), 
								attr(xx$depvars[[j]], 'nodeSet'), attr(xx$depvars[[j]], 'nodeSet')[2])
					if (attr(xx$dycCovars[[k]], 'nodeSet')[2] == nodset_j)
					{
						othervarname <- names(xx$dycCovars)[k]
						objEffects <- rbind(objEffects,
							createEffects("dyadBNetNetObjective",
								varname, otherName, othervarname,
								name=varname, groupName=groupName,
								group=group, netType=netType))
					}
				}
				for (k in seq(along = xx$dyvCovars))
				{
					nodset_j <- ifelse((attr(xx$depvars[[j]], "type") == "oneMode"), 
								attr(xx$depvars[[j]], 'nodeSet'), attr(xx$depvars[[j]], 'nodeSet')[2])
					if (attr(xx$dyvCovars[[k]], 'nodeSet')[2] == nodset_j)
					{
						othervarname <- names(xx$dyvCovars)[k]
						objEffects <- rbind(objEffects,
							createEffects("dyadBNetNetObjective",
								varname, otherName, othervarname,
								name=varname, groupName=groupName,
								group=group, netType=netType))
					}
				}
				for (k in seq(along=xx$depvars))
				{
					if ((types[k] %in% c('behavior', 'continuous')) &&
						(attr(xx$depvars[[k]], 'nodeSet') == nodeSet))
					{
						objEffects <-
							rbind(objEffects,
								covarNetNetEff(otherName,
									covarname=names(xx$depvars)[k],
									attr(xx$depvars[[j]], 'nodeSet'),
									attr(xx$depvars[[k]], 'nodeSet'),
									poszvar=TRUE,
									name=varname))
					}
				}

			}
		}
		if ((nOneModes + nBipartites) > 1) ## add the network name
		{
			objEffects$functionName <- paste(varname, ': ',
				objEffects$functionName, sep = '')
			objEffects$effectName <- paste(varname, ': ',
				objEffects$effectName, sep = '')
		}

		## replace the text for endowment and creation effects
		tmp <- objEffects$functionName[objEffects$type =='endow']
		tmp <- paste('Lost ties:', tmp)
		objEffects$functionName[objEffects$type == 'endow'] <- tmp
		tmp <- objEffects$functionName[objEffects$type =='creation']
		tmp <- paste('New ties:', tmp)
		objEffects$functionName[objEffects$type == 'creation'] <- tmp
		tmp <- objEffects$functionName[objEffects$type =='gmm']
		tmp <- paste('Gmm statistic:', tmp)
		objEffects$functionName[objEffects$type == 'gmm'] <- tmp

		## get starting values
		starts <- getNetworkStartingVals(depvar)
		##set defaults
		rateEffects[1:noPeriods, "include"] <- TRUE
		rateEffects[1:noPeriods, "initialValue"] <- starts$startRate
		rateEffects$basicRate[1:observations] <- TRUE

		objEffects[objEffects$shortName == "density" &
			objEffects$type == "eval",'randomEffects'] <- TRUE
		objEffects[objEffects$shortName == "linear" &
			objEffects$type == "eval",'randomEffects'] <- TRUE

		objEffects$untrimmedValue <- rep(0, nrow(objEffects))
		if ((attr(depvar,'allUpOnly') || attr(depvar, 'allDownOnly')))
		{
			objEffects <- objEffects[!objEffects$shortName == "density", ]
		}
		else
		{
			objEffects[objEffects$shortName == "density" &
				objEffects$type == 'eval',
			c('include', "initialValue", "untrimmedValue")] <-
				list(TRUE, starts$degree, starts$untrimmed)
		}
		if (!(attr(depvar,'symmetric')))
		{
			objEffects[objEffects$shortName == 'recip'&
				objEffects$type == 'eval', 'include'] <- TRUE
		}
		rateEffects$basicRate[1:observations] <- TRUE
		
		useSettingsModel <- (!is.null(attr(depvar,"settingsinfo")))
		if (useSettingsModel)
		{			
			objEffects <- rbind(objEffects, createEffects(
				"settingsObjective", varname, name=varname,
				groupName=groupName, group=group, netType=netType))
				# append effects with an interaction on the primary settings network of `varname`
			objEffects <- rbind(objEffects, createEffects(
				"nonSymmetricSymmetricSObjective", paste0("primary(", varname, ")") , name=varname,
				groupName=groupName, group=group, netType=netType))
			settingOnlys <- vapply(attr(depvar,"settingsinfo"), function(s){s$only},
								FUN.VALUE="a")
			if (settingOnlys[1] == 'none') 
			{
				useSettingsModel  <- FALSE
			}
		}

		if (useSettingsModel)
		{
			settingIds <- sapply(attr(depvar,"settingsinfo"), function(s) s$id)
			nbrSettings <- length(settingIds)

			if ("primary" %in% settingIds) {
			# see above, already done if useSettingsModel before this was reduced.
			}

			# duplicate rate effects, split by periods
			setRate <- rateEffects[1:observations, ]
			setRate <- setRate[rep(1:nrow(setRate), each=nbrSettings), ]
			setRateByPeriod <- split(setRate, list(setRate$group, setRate$period))

			# for each period, set "setting" and modify "effectName"
			setRateByPeriod <- lapply(setRateByPeriod, function(dd) {
					dd$setting <- settingIds
					dd$interaction1 <- settingIds
# also see below with 0.75
					dd$initialValue[dd$setting != 'primary'] <- 0.5
					dd$initialValue[dd$setting == 'primary'] <-
										0.75*dd$initialValue[dd$setting == 'primary']
					dd$effectName <- paste(dd$setting, ' setting rate ',
										varname,' (period ', dd$period, ')', sep='')
					dd$functionName <- paste('distance in ',dd$setting,
										' setting (period ', dd$period, ')', sep='')
					dd
				})
			setRate <- do.call(rbind, setRateByPeriod)
			## add the extra column also to the other effects
			rateEffects$setting <- rep("", nrow(rateEffects))
			objEffects$setting <- rep("", nrow(objEffects))
			# get the settings description
			settingsDescription <- describeTheSetting(depvar)
			rateEffects <- rbind(setRate, rateEffects[!rateEffects$basicRate, ])
		}
		else
		{
			settingsDescription <- ""
		}
		list(effects=rbind(rateEffects = rateEffects, objEffects = objEffects),
			starts=starts, settingsDescription=settingsDescription)
	}

	##@behaviornet internal getEffects
	behaviorNet <- function(depvar, varname)
	{
		nodeSet <- attr(depvar,'nodeSet')

		rateEffects <- createEffects("behaviorRate", varname, name=varname,
			groupName=groupName, group=group,
			netType=netType)
		if (observations == 1)
		{
			rateEffects <- rateEffects[-2, ] ## remove the extra period
		}
		else
		{
			## get correct number of rows
			rateEffects <- rbind(duplicateDataFrameRow(rateEffects[2, ],
					observations),
				rateEffects[-c(1, 2), ])
		}

		objEffects <- createEffects("behaviorObjective", varname, name=varname,
			groupName=groupName, group=group,
			netType=netType)

		for (j in seq(along=xx$depvars))
		{
			if (types[j] == "oneMode" &&
				attr(xx$depvars[[j]], "nodeSet") == nodeSet)
			{
				depvarname <- names(xx$depvars)[j]
				if (attr(xx$depvars[[j]], "symmetric"))
				{
					tmpObjEffects <-
						createEffects("behaviorSymmetricObjective",
							varname, depvarname, name=varname,
							groupName=groupName, group=group,
							netType=netType)
					tmpObjEffects2 <- NULL
					for (k in seq(along=xx$depvars))
					{
						if ((j != k) && (types[k] == "oneMode") &&
							(attr(xx$depvars[[k]], "symmetric")) &&
							(attr(xx$depvars[[k]], "nodeSet") == nodeSet))
						{
							othervarname <- names(xx$depvars)[k]
							tmpObjEffects3 <-
								createEffects("behaviorSymSymObjective",
									varname, depvarname, othervarname,
									name=varname,
									groupName=groupName, group=group,
									netType=netType)
							tmpObjEffects3$functionName <-
								paste(tmpObjEffects3$functionName,
									" (", depvarname, ",", othervarname, ")", sep="")
							tmpObjEffects3$effectName <-
								paste(tmpObjEffects3$effectName,
									" (", depvarname, ",", othervarname, ")", sep = "")
							tmpObjEffects2 <- rbind(tmpObjEffects2, tmpObjEffects3)
						}
					}
					tmpRateEffects <-
						createEffects("behaviorSymmetricRate",
							varname, depvarname, name=varname,
							groupName=groupName, group=group,
							netType=netType)
				}
				else # non-symmetric
				{
					tmpObjEffects <-
						createEffects("behaviorOneModeObjective",
							varname, depvarname, name=varname,
							groupName=groupName, group=group,
							netType=netType)
					tmpObjEffects2 <- NULL
					for (k in seq(along=xx$depvars))
					{
						if ((k != j) && (types[k] == "oneMode") &&
							(!attr(xx$depvars[[k]], "symmetric")) &&
							(attr(xx$depvars[[k]], "nodeSet") == nodeSet))
						{
							othervarname <- names(xx$depvars)[k]
							tmpObjEffects3 <-
								createEffects("behaviorOneOneModeObjective",
									varname, depvarname, othervarname,
									name=varname,
									groupName=groupName, group=group,
									netType=netType)
							tmpObjEffects3$functionName <-
								paste(tmpObjEffects3$functionName,
									" (", depvarname, ",", othervarname, ")", sep="")
							tmpObjEffects3$effectName <-
								paste(tmpObjEffects3$effectName,
									" (", depvarname, ",", othervarname, ")", sep = "")
							tmpObjEffects2 <- rbind(tmpObjEffects2, tmpObjEffects3)
						}
						if ((types[k] == "oneMode") &&
							(attr(xx$depvars[[k]], "symmetric")) &&
							(attr(xx$depvars[[k]], "nodeSet") == nodeSet))
						{
							othervarname <- names(xx$depvars)[k]
							tmpObjEffects3 <-
								createEffects("behaviorOneModeSymObjective",
									varname, depvarname, othervarname,
									name=varname,
									groupName=groupName, group=group,
									netType=netType)
							tmpObjEffects3$functionName <-
								paste(tmpObjEffects3$functionName,
									" (", depvarname, ",", othervarname, ")", sep="")
							tmpObjEffects3$effectName <-
								paste(tmpObjEffects3$effectName,
									" (", depvarname, ",", othervarname, ")", sep = "")
							tmpObjEffects2 <- rbind(tmpObjEffects2, tmpObjEffects3)
						}
					}
					tmpRateEffects <-
						createEffects("behaviorOneModeRate",
							varname, depvarname, name=varname,
							groupName=groupName, group=group,
							netType=netType)
				}
				# Now irrespective of (attr(xx$depvars[[j]], "symmetric"))
				for (k in seq(along = xx$dycCovars))
				{
					if (attr(xx$dycCovars[[k]], "type") == "oneMode" &&
						attr(xx$dycCovars[[k]], 'nodeSet')[1] == nodeSet)
					{
						othervarname <- names(xx$dycCovars)[k]
						tmpObjEffects3 <- createEffects("dyadBehaviorNetObjective",
							varname, depvarname, othervarname,
							name=varname, groupName=groupName,
							group=group, netType=netType)
						tmpObjEffects2 <- rbind(tmpObjEffects2, tmpObjEffects3)
					}
				}
				for (k in seq(along = xx$dyvCovars))
				{
					if (attr(xx$dyvCovars[[k]], "type") == "oneMode" &&
						attr(xx$dyvCovars[[k]], 'nodeSet')[1] == nodeSet)
					{
						othervarname <- names(xx$dyvCovars)[k]
						tmpObjEffects3 <- createEffects("dyadBehaviorNetObjective",
							varname, depvarname, othervarname,
							name=varname, groupName=groupName,
							group=group, netType=netType)
						tmpObjEffects2 <- rbind(tmpObjEffects2, tmpObjEffects3)
					}
				}
				if ((nOneModes + nBipartites) > 1) ## add the network name
				{
					tmpObjEffects$functionName <-
						paste(tmpObjEffects$functionName,
							" (", depvarname, ")", sep="")
					tmpObjEffects$effectName <-
						paste(tmpObjEffects$effectName,
							" (", depvarname, ")", sep = "")
					tmpRateEffects$functionName <-
						paste(tmpRateEffects$functionName,
							" (", depvarname, ")", sep="")
					tmpRateEffects$effectName <-
						paste(tmpRateEffects$effectName,
							" (", depvarname, ")", sep = "")
				}
				objEffects <- rbind(objEffects, tmpObjEffects, tmpObjEffects2)
				rateEffects <- rbind(rateEffects, tmpRateEffects)
			}
			if (types[j] == 'bipartite' &&
				(attr(xx$depvars[[j]], 'nodeSet')[1] == nodeSet))
			{
				depvarname <- names(xx$depvars)[j]
				tmpObjEffects <-
					createEffects("behaviorBipartiteObjective",
						varname, depvarname, name=varname,
						groupName=groupName, group=group,
						netType=netType)
				tmpObjEffects2 <- NULL
				for (k in seq(along=xx$depvars))
				{
					if ((k != j) && (types[k] == "bipartite") &&
						(attr(xx$depvars[[k]], "nodeSet")[1] == nodeSet) &&
						(attr(xx$depvars[[j]], "nodeSet")[2] ==
							attr(xx$depvars[[k]], "nodeSet")[2]))
					{
						othervarname <- names(xx$depvars)[k]
						tmpObjEffects3 <-
							createEffects("behaviorBipBipObjective",
								varname, depvarname, othervarname,
								name=varname,
								groupName=groupName, group=group,
								netType=netType)
						tmpObjEffects3$functionName <-
							paste(tmpObjEffects3$functionName,
								" (", depvarname, ",", othervarname, ")", sep="")
						tmpObjEffects3$effectName <-
							paste(tmpObjEffects3$effectName,
								" (", depvarname, ",", othervarname, ")", sep = "")
						tmpObjEffects2 <- rbind(tmpObjEffects2, tmpObjEffects3)
					}
				}
				tmpRateEffects <-
					createEffects("behaviorBipartiteRate",
						varname, depvarname, name=varname,
						groupName=groupName, group=group,
						netType=netType)
				if ((nOneModes + nBipartites) > 1) ## add the network name
				{
					tmpObjEffects$functionName <-
						paste(tmpObjEffects$functionName,
							" (", depvarname, ")", sep="")
					tmpObjEffects$effectName <-
						paste(tmpObjEffects$effectName,
							" (", depvarname, ")", sep = "")
					tmpRateEffects$functionName <-
						paste(tmpRateEffects$functionName,
							" (", depvarname, ")", sep="")
					tmpRateEffects$effectName <-
						paste(tmpRateEffects$effectName,
							" (", depvarname, ")", sep = "")
				}

				objEffects <- rbind(objEffects, tmpObjEffects, tmpObjEffects2)
				rateEffects <- rbind(rateEffects, tmpRateEffects)
			}
		}
		for (j in seq(along = xx$cCovars))
		{
			if (attr(xx$cCovars[[j]], 'nodeSet') == nodeSet)
			{
				tmp <- covBehEff(varname, names(xx$cCovars)[j], nodeSet,
					type='', name=varname)
				objEffects<- rbind(objEffects, tmp$objEff)
				rateEffects<- rbind(rateEffects, tmp$rateEff)
			}
			else
			{
				nodeSet2 <- attr(xx$cCovars[[j]], 'nodeSet')
				tmp <- covBBehEff(varname, names(xx$cCovars)[j], nodeSet2,
					name=varname)
				objEffects<- rbind(objEffects, tmp$objEff)
			}
		}
		for (j in seq(along=xx$depvars))
		{
			if (types[j] %in% c('behavior', 'continuous') &&
				attr(xx$depvars[[j]], 'nodeSet') == nodeSet)
			{
				tmp <- covBehEff(varname, names(xx$depvars)[j], nodeSet, j==i,
					type='Beh', name=varname)
				objEffects<- rbind(objEffects, tmp$objEff)
				rateEffects<- rbind(rateEffects, tmp$rateEff)
			}
		}
		for (j in seq(along=xx$vCovars))
		{
			if (attr(xx$vCovars[[j]], 'nodeSet') == nodeSet)
			{
				tmp <- covBehEff(varname, names(xx$vCovars)[j], nodeSet,
					type='Var', name=varname)
				objEffects<- rbind(objEffects, tmp$objEff)
				rateEffects<- rbind(rateEffects, tmp$rateEff)
			}
			else
			{
				nodeSet2 <- attr(xx$vCovars[[j]], 'nodeSet')
				tmp <- covBBehEff(varname, names(xx$vCovars)[j], nodeSet2,
					name=varname)
				objEffects<- rbind(objEffects, tmp$objEff)
			}
		}
		interaction <- createEffects("unspecifiedBehaviorInteraction",
			varname, name=varname,
			groupName=groupName, group=group,
			netType=netType)
		objEffects <- rbind(objEffects, interaction[rep(1:nrow(interaction), behNintn),])

		## get starting values
		starts <- getBehaviorStartingVals(depvar)
		## set defaults
		if (!(attr(depvar,'allUpOnly') || attr(depvar, 'allDownOnly')))
		{
			objEffects[grepl("linear shape", objEffects$effectName) &
				objEffects$type == 'eval',
				c('include', 'initialValue','untrimmedValue')]  <-
				list(TRUE, starts$tendency, starts$untrimmed)
		}
		else
		{
			objEffects <- objEffects[objEffects$shortName != "linear", ]
		}
		if (attr(depvar, "range") >= 2)
		{
			objEffects[grepl("quadratic shape", objEffects$effectName) &
				objEffects$type == 'eval' &
				objEffects$shortName != 'quad_cc' & objEffects$shortName != 'quad_nc','include'] <- TRUE
			## no starting value for quadratic effect
		}

		rateEffects[1:observations, 'include'] <- TRUE
		rateEffects[1:noPeriods, 'initialValue'] <- starts$startRate
		rateEffects$basicRate[1:observations] <- TRUE

		## alter the text for endowment and creation names
		objEffects$effectName[objEffects$type == 'endow'] <-
			sub('behavior', 'dec. beh.',
				objEffects$effectName[objEffects$type == 'endow'])
		objEffects$effectName[objEffects$type == 'creation'] <-
			sub('behavior', 'inc. beh.',
				objEffects$effectName[objEffects$type == 'creation'])

		list(effects = rbind(rateEffects = rateEffects,
				objEffects = objEffects), starts=starts)
	}

	##@continuousNet internal getEffects
    continuousNet <- function(depvars, varnames)
    {
        nodeSet <- attr(depvars[[1]],'nodeSet')	## NN: nodeset should be the same for
						##     all continuous depvars

        rateEffects <- createEffects("continuousRate", name="sde",
                                         groupName=groupName, group=group,
                                         netType=netType)
        if (observations == 1)
        {
            rateEffects <- rateEffects[-2, ] ## remove the extra period
        }
        else
        {
            ## get correct number of rows
            rateEffects <- rbind(duplicateDataFrameRow(rateEffects[2, ],
                                                       observations),
                                 rateEffects[-c(1, 2), ])
        }

        objEffects <- fbicEffects <- wEffects <- NULL # general effects, feedback
                                                      # and intercept, wiener
        for (j in seq(along=varnames)) # for all continuous variables
        {
			for (k in seq(along=varnames))
            {
				fbicEffects <- rbind(fbicEffects, createEffects("continuousFeedback",
                                xName = varnames[j], yName = varnames[k],
							    name=varnames[j], groupName=groupName, group=group,
                                netType=netType))
				if (j <= k)
				    wEffects <- rbind(wEffects, createEffects("continuousWiener",
                                xName = varnames[k], yName = varnames[j],
							    name=varnames[k], groupName=groupName, group=group,
                                netType=netType))
			}
            fbicEffects <- rbind(fbicEffects, createEffects("continuousIntercept",
							xName = varnames[j], name = varnames[j],
							groupName=groupName, group=group, netType=netType))

            for (k in seq(along=xx$depvars))  # corrected version 1.4.5.
            {
                if (types[k] == "oneMode" &&
                    attr(xx$depvars[[k]], "nodeSet") == nodeSet)
                {
                    depvarname <- names(xx$depvars)[k]
                    tmpObjEffects <-
                            createEffects("continuousOneModeObjective",
                                          varnames[j], depvarname, name=varnames[j],
                                          groupName=groupName, group=group,
                                          netType=netType)
                    if ((nOneModes) > 1) # add the network name, 
                    {
                        tmpObjEffects$functionName <-
                            paste(tmpObjEffects$functionName,
                                  " (", depvarname, ")", sep="")
                        tmpObjEffects$effectName <-
                            paste(tmpObjEffects$effectName,
                                  " (", depvarname, ")", sep = "")
                    }
					objEffects <- rbind(objEffects, tmpObjEffects)	
				}			
				
                if (types[k] == "bipartite" &&
                    attr(xx$depvars[[k]], "nodeSet")[1] == nodeSet)
                {
                    depvarname <- names(xx$depvars)[k]

                    tmpObjEffects <-
                            createEffects("continuousBipartiteObjective",
                                          varnames[j], depvarname, name=varnames[j],
                                          groupName=groupName, group=group,
                                          netType=netType)
                    if ((nBipartites) > 1) # add the network name, 
                    {
                        tmpObjEffects$functionName <-
                            paste(tmpObjEffects$functionName,
                                  " (", depvarname, ")", sep="")
                        tmpObjEffects$effectName <-
                            paste(tmpObjEffects$effectName,
                                  " (", depvarname, ")", sep = "")
                    }
					objEffects <- rbind(objEffects, tmpObjEffects)
				}
            }
            for (k in seq(along = xx$cCovars))
            {
                if (attr(xx$cCovars[[k]], 'nodeSet') == nodeSet)
                {
                    tmp <- covContEff(varnames[j], names(xx$cCovars)[k], nodeSet,
                                     type='', name=varnames[j])
                    objEffects <- rbind(objEffects, tmp$objEff)
                }
            }
            for (k in seq(along=xx$depvars))
            {
                if (types[k] %in% c('behavior', 'continuous') &&
                    attr(xx$depvars[[k]], 'nodeSet') == nodeSet)
                {
                    tmp <- covContEff(varnames[j], names(xx$depvars)[k], nodeSet,
                                     varnames[j] == names(xx$depvars)[k],
                                     type='Beh', name=varnames[j])
                    objEffects <- rbind(objEffects, tmp$objEff)
                }
            }
            for (k in seq(along=xx$vCovars))
            {
                if (attr(xx$vCovars[[k]], 'nodeSet') == nodeSet)
                {
                    tmp <- covContEff(varnames[j], names(xx$vCovars)[k], nodeSet,
                                     type='Var', name=varnames[j])
                    objEffects <- rbind(objEffects, tmp$objEff)
                }
            }
			interaction <- createEffects("unspecifiedContinuousInteraction",
                                     varnames[j], name=varnames[j],
									 groupName=groupName, group=group,
									 netType=netType)

			objEffects <- rbind(objEffects, interaction[rep(1, behNintn),])
		}

		fbicEffects$include <- TRUE

        if (onePeriodSde)
        {
            wEffects$include <- TRUE
            rateEffects$fix[1] <- TRUE
            rateEffects$include[1] <- TRUE
        }
        else
        {
            wEffects$fix[1] <- TRUE
            rateEffects$include[1:observations] <- TRUE
            rateEffects$basicRate[1:observations] <- TRUE
        }

		if (nContinuous > 1)
			wEffects$include <- TRUE

        starts <- getContinuousStartingVals(depvars, onePeriodSde)
        wEffects$initialValue <- starts$startWiener
		rateEffects$initialValue[1:noPeriods] <- starts$startScale
		fbicEffects$initialValue <- starts$startFbic

        list(effects = rbind(rateEffects, wEffects, fbicEffects, objEffects),
             starts = starts)
    }

	##@bipartiteNet internal getEffects
	bipartiteNet <- function(depvar, varname)
	{
		nodeSets <- attr(depvar, 'nodeSet')

		rateEffects <- networkRateEffects(depvar, varname, symmetric=FALSE,
			bipartite=TRUE)

		objEffects <- createEffects("bipartiteObjective", varname,
			name=varname,
			groupName=groupName, group=group,
			netType=netType)

		for (j in seq(along = xx$dycCovars))
		{
			if (attr(xx$dycCovars[[j]], "type") == "bipartite" &&
				all(nodeSets == attr(xx$dycCovars[[j]], 'nodeSet')))
			{
				objEffects <- rbind(objEffects,
					createEffects("dyadBipartiteObjective",
						names(xx$dycCovars)[j],
						name=varname,
						groupName=groupName, group=group,
						netType=netType))
			}
			if (attr(xx$dycCovars[[j]], "type") == "oneMode" &&
				(nodeSets[2] == attr(xx$dycCovars[[j]], 'nodeSet')[1]) &&
				(nodeSets[2] == attr(xx$dycCovars[[j]], 'nodeSet')[2]) )
			{
				objEffects <- rbind(objEffects,
					createEffects("dyadSecondBipartiteObjective",
						names(xx$dycCovars)[j],
						name=varname,
						groupName=groupName, group=group,
						netType=netType))
			}
		}
		for (j in seq(along = xx$dyvCovars))
		{
			if (attr(xx$dyvCovars[[j]], "type") == "bipartite" &&
				all(nodeSets == attr(xx$dyvCovars[[j]], 'nodeSet')))
			{
				objEffects <- rbind(objEffects,
					createEffects("dyadBipartiteObjective",
						names(xx$dyvCovars)[j],
						name=varname,
						groupName=groupName, group=group,
						netType=netType))
			}
			if (attr(xx$dyvCovars[[j]], "type") == "oneMode" &&
				(nodeSets[2] == attr(xx$dyvCovars[[j]], 'nodeSet')[1]) &&
				(nodeSets[2] == attr(xx$dyvCovars[[j]], 'nodeSet')[2]) )
			{ 
				objEffects <- rbind(objEffects,
					createEffects("dyadSecondBipartiteObjective",
						names(xx$dyvCovars)[j],
						name=varname,
						groupName=groupName, group=group,
						netType=netType))
			}
		}
		for (j in seq(along = xx$cCovars))
		{
			covNodeset <- match(attr(xx$cCovars[[j]], "nodeSet"),
				nodeSets)
			if (!is.na(covNodeset))
			{
				tmp <- covarBipartiteEff(names(xx$cCovars)[j],
					attr(xx$cCovars[[j]],
						'poszvar'),
					attr(xx$cCovars[[j]],
						'moreThan2'),
					covNodeset, name=varname)
				objEffects <- rbind(objEffects, tmp$objEff)
				rateEffects <- rbind(rateEffects, tmp$rateEff)				
				for (jj in seq(along = xx$cCovars))
				{
					if ((jj != j) & (nodeSets[1]==attr(xx$cCovars[[j]], "nodeSet"))
									& (nodeSets[2]==attr(xx$cCovars[[jj]], "nodeSet"))
							& (DoubleAttributesChecked(xx$cCovars[[j]], xx$cCovars[[jj]])))
					{
						tmp <- doubleCovarNetEff(names(xx$cCovars)[j],
									names(xx$cCovars)[jj], name=varname)
						objEffects <-  rbind(objEffects, tmp$objEff)
					}
				}
				for (jj in seq(along = xx$vCovars))
				{
					if ((nodeSets[1]==attr(xx$cCovars[[j]], "nodeSet"))
									& (nodeSets[2]==attr(xx$vCovars[[jj]], "nodeSet"))									
							& (DoubleAttributesChecked(xx$cCovars[[j]], xx$vCovars[[jj]])))
					{
						tmp <- doubleCovarNetEff(names(xx$cCovars)[j],
									names(xx$vCovars)[jj], name=varname)
						objEffects <-  rbind(objEffects, tmp$objEff)
					}
				}						
			}
		}
		for (j in seq(along=xx$depvars))
		{
			if (types[j] %in% c('behavior', 'continuous'))
			{
				covNodeset <- match(attr(xx$depvars[[j]], "nodeSet"),
					nodeSets)
				if (!is.na(covNodeset))
				{
					tmp <- covarBipartiteEff(names(xx$depvars)[j],
						poszvar=TRUE,
						attr(xx$depvars[[j]],
							'moreThan2'),
						covNodeset, name=varname)
					objEffects <- rbind(objEffects,tmp$objEff)
					rateEffects <- rbind(rateEffects,tmp$rateEff)
				}
			}
		}
		for (j in seq(along=xx$vCovars))
		{
			covNodeset <- match(attr(xx$vCovars[[j]], "nodeSet"),
				nodeSets)
			if (!is.na(covNodeset))
			{
				tmp <- covarBipartiteEff(names(xx$vCovars)[j],
					attr(xx$vCovars[[j]],
						'poszvar'),
					attr(xx$vCovars[[j]],
						'moreThan2'),
					covNodeset, name=varname)
				objEffects <- rbind(objEffects, tmp$objEff)
				rateEffects <- rbind(rateEffects, tmp$rateEff)
			}
		}
		if (length(xx$cCovars) + length(xx$vCovars) +
			length(xx$dycCovars) + length(xx$dyvCovars) +
			length(types=='behavior') + length(types=='continuous') > 0)
		{
			interaction <- createEffects("unspecifiedNetInteraction",
				name=varname,
				groupName=groupName, group=group,
				netType=netType)
			objEffects <-  rbind(objEffects, interaction[rep(1:nrow(interaction), nintn), ])
		}

		for (j in seq(along=xx$depvars))
		{
			otherName <- names(xx$depvars)[j]
			if (types[j] == 'oneMode' &&
				attr(xx$depvars[[j]], 'nodeSet') == nodeSets[1] )
			{
				if (attr(xx$depvars[[j]], "symmetric"))
				{
					objEffects <-
						rbind(objEffects,
							createEffects("bipartiteSymmetricObjective",
								names(xx$depvars)[[j]],
								name=varname,
								groupName=groupName, group=group,
								netType=netType))
				}
				else
				{
					objEffects <-
						rbind(objEffects,
							createEffects("bipartiteNonSymmetricObjective",
								names(xx$depvars)[[j]],
								name=varname,
								groupName=groupName, group=group,
								netType=netType))
				}
			}
			if (types[j] == 'bipartite' &&
				(attr(xx$depvars[[j]], 'nodeSet')[1] == nodeSets[1]) &&
				varname != otherName)
			{
				objEffects <-
					rbind(objEffects,
						createEffects("bipartiteBipartiteObjective",
							names(xx$depvars)[[j]],
							name=varname,
							groupName=groupName, group=group,
							netType=netType))
			}
			if (!(types[j] %in% c('behavior', 'continuous'))) # other networks, irrespective of onemode/symmetric/twomode
			{
				for (k in seq(along=xx$cCovars))
				{
					if (attr(xx$cCovars[[k]], 'nodeSet') == nodeSets[1])
					{
						objEffects <-
							rbind(objEffects,
								createEffects("covarABipNetObjective", otherName,
									names(xx$cCovars)[k], name=varname,
									groupName=groupName, group=group,
									netType=netType))
					}
				}
				for (k in seq(along=xx$vCovars))
				{
					if (attr(xx$vCovars[[k]], 'nodeSet') == nodeSets[1])
					{
						objEffects <-
							rbind(objEffects,
								createEffects("covarABipNetObjective", otherName,
									names(xx$vCovars)[k], name=varname,
									groupName=groupName, group=group,
									netType=netType))
					}
				}
				for (k in seq(along = xx$dycCovars))
				{
					nodset_j <- ifelse((attr(xx$depvars[[j]], "type") == "oneMode"), 
								attr(xx$depvars[[j]], 'nodeSet'), attr(xx$depvars[[j]], 'nodeSet')[2])
					if (attr(xx$dycCovars[[k]], 'nodeSet')[2] == nodset_j)
					{
						othervarname <- names(xx$dycCovars)[k]
						objEffects <- rbind(objEffects,
							createEffects("dyadBNetNetObjective",
								varname, otherName, othervarname,
								name=varname, groupName=groupName,
								group=group, netType=netType))
					}
				}
				for (k in seq(along = xx$dyvCovars))
				{
					nodset_j <- ifelse((attr(xx$depvars[[j]], "type") == "oneMode"), 
								attr(xx$depvars[[j]], 'nodeSet'), attr(xx$depvars[[j]], 'nodeSet')[2])
					if (attr(xx$dyvCovars[[k]], 'nodeSet')[2] == nodset_j)
					{
						othervarname <- names(xx$dyvCovars)[k]
						objEffects <- rbind(objEffects,
							createEffects("dyadBNetNetObjective",
								varname, otherName, othervarname,
								name=varname, groupName=groupName,
								group=group, netType=netType))
					}
				}
				for (k in seq(along=xx$depvars))
				{
					if (types[k] %in% c('behavior', 'continuous') &&
						attr(xx$depvars[[k]], 'nodeSet') == nodeSets[1])
					{
						objEffects <-
							rbind(objEffects,
								createEffects("covarABipNetObjective", otherName,
									names(xx$depvars)[k], name=varname,
									groupName=groupName, group=group,
									netType=netType))
					}
				}
			}
		}
		if ((nOneModes + nBipartites) > 1) ## add the network name
		{
			objEffects$functionName <- paste(varname, ': ',
				objEffects$functionName, sep = '')
			objEffects$effectName <- paste(varname, ': ',
				objEffects$effectName, sep = '')
		}

		objEffects$functionName[objEffects$type == 'endow'] <-
			paste('Lost ties:',
				objEffects$functionName[objEffects$type =='endow'])
		objEffects$functionName[objEffects$type == 'creation'] <-
			paste('New ties:',
				objEffects$functionName[objEffects$type =='creation'])

		## get starting values
		starts <- getBipartiteStartingVals(depvar)
		##set defaults
		rateEffects[1:observations, 'include'] <- TRUE
		rateEffects[1:noPeriods, 'initialValue'] <- starts$startRate
		rateEffects$basicRate[1:observations] <- TRUE

		if (!(attr(depvar,'allUpOnly') || attr(depvar, 'allDownOnly')))
		{
			objEffects[objEffects$shortName =='density' &
				objEffects$type == 'eval',
			c('include', 'initialValue', 'untrimmedValue')] <-
				list(TRUE, starts$degree, starts$untrimmed)
		}
		else
		{
			objEffects <-
				objEffects[!objEffects$shortName == "density", ]
		}

		rateEffects$basicRate[1:observations] <- TRUE

		list(effects=rbind(rateEffects = rateEffects, objEffects = objEffects),
			starts=starts)
	}

	##@covarOneModeEff internal getEffects
	covarOneModeEff<- function(covarname, poszvar, moreThan2, symmetric, constant,
		name)
	{
		if (symmetric)
		{
			covObjEffects <- createEffects("covarSymmetricObjective", covarname,
				name=name,
				groupName=groupName, group=group,
				netType=netType)
			covRateEffects <- createEffects("covarSymmetricRate", covarname,
				name=name,
				groupName=groupName, group=group,
				netType=netType)
		}
		else
		{
			covObjEffects <- createEffects("covarNonSymmetricObjective",
				covarname, name=name,
				groupName=groupName, group=group,
				netType=netType)
			covRateEffects <- createEffects("covarNonSymmetricRate", covarname,
				name=name,
				groupName=groupName, group=group,
				netType=netType)
		}
				
		if (constant)
		{
			covObjEffects <-
				covObjEffects[!(covObjEffects$shortName %in% c("avGroupEgoX", "totGroupEgoX")),]
		}

# these lines tentatively dropped version 1.2-5
#		if (!tr & (!poszvar))  # not (positive variance of z, or any z missing)
#		{
#			if (symmetric)
#			{
#				covObjEffects <-
#					covObjEffects[covObjEffects$shortName %in%
#					c("altX", "altSqX"), ]
#			}
#			else
#			{
#				covObjEffects <-
#					covObjEffects[covObjEffects$shortName %in%
#					c("egoX", "egoSqX"), ]
#			}
#		}
#		if (!tr & (!moreThan2))
#		{
#			covObjEffects <-
#				covObjEffects[!covObjEffects$shortName %in%
#				c("altSqX", "egoPlusAltSqX"), ]
#		}

		list(objEff=covObjEffects, rateEff=covRateEffects)
	}
	
	##@doubleCovarNetEff internal getEffects
	doubleCovarNetEff<- function(covarname1, covarname2, name)
	{
		covObjEffects <- createEffects("doubleCovarNetObjective", 
				covarname1,covarname2,
				name=name,
				groupName=groupName, group=group,
				netType=netType)
		list(objEff=covObjEffects, rateEff=NULL)
	}
	
	##@covarBipartiteEff internal getEffects
	covarBipartiteEff<- function(covarname, poszvar, moreThan2, nodesetNbr,
		name)
	{
		covRateEffects <- NULL
		if (nodesetNbr == 1)
		{
			covObjEffects <-
				createEffects("covarBipartiteObjective", covarname,
					name=varname,
					groupName=groupName, group=group,
					netType=netType)
			# restrict to covariates on first node set, but also include sameX
			covObjEffects <-
				covObjEffects[covObjEffects$shortName %in%
				c("egoX", "egoSqX", "sameX", "egoLThresholdX", "egoRThresholdX",
					"degAbsDiffX", "degPosDiffX", "degNegDiffX",
					"altInDist2", "totInDist2", "simEgoInDist2", 
					"sameEgoInDist2", "sameXInPop", "diffXInPop",
					"sameXOutAct", "diffXOutAct", "crossXOutAct",
					"sameXCycle4", "inPopX", "inActX", "avGroupEgoX", "totGroupEgoX"), ]
			covRateEffects <- createEffects("covarBipartiteRate", covarname,
				name=varname,
				groupName=groupName, group=group,
				netType=netType)
		}
		else if (tr | poszvar) # positive variance of z, or any z missing
		{
			covObjEffects <- createEffects("covarBipartiteObjective", covarname,
				name=varname,
				groupName=groupName, group=group,
				netType=netType)
			# restrict to covariates on second node set
			covObjEffects <-
				covObjEffects[covObjEffects$shortName %in%
				c("altX", "altSqX",  "altLThresholdX", "altRThresholdX",
					"homXOutAct", "homXOutAct2", "altXOutAct",
					"inActX", "outActX", "sameInXCycle4"), ]
			if (!tr & (!moreThan2))
			{
				covObjEffects <-
					covObjEffects[!covObjEffects$shortName %in% c("altSqX"), ]
			}
		}
		else
		{
			covObjEffects <- NULL
		}

		list(objEff=covObjEffects, rateEff=covRateEffects)
	}
	##@covBehEff internal getEffects
	## beware: this function does lots of things!
	covBehEff<- function(varname, covarname, nodeSet, same=FALSE,
		## same indicates that varname and covarname are
		## the same: just one rate effect required
		## type is no longer used
		type=c('', 'Var', 'Beh'), name)
	{
		covObjEffects <-  NULL
		if (!same)
		{
			covObjEffects <- createEffects("covarBehaviorObjective", varname,
				covarname, name=name,
				groupName=groupName, group=group,
				netType=netType)
		}

		covRateEffects <- createEffects("covarBehaviorRate", varname, covarname,
			name=name,
			groupName=groupName, group=group,
			netType=netType)
		## if we have a real covariate, need to add other effects for every
		## appropriate network
		if (!same)
		{
			for (j in seq(along=xx$depvars))
			{
				if (types[j] == 'oneMode' &&
					attr(xx$depvars[[j]], 'nodeSet') == nodeSet)
				{
					newEffects <-
						createEffects("covarBehaviorNetObjective", varname,
							covarname, names(xx$depvars)[j],
							groupName=groupName, group=group,
							netType=netType, name=name)

					covObjEffects <- rbind(covObjEffects, newEffects)
					if (attr(xx$depvars[[j]], "symmetric"))
					{
						covBehRateEffects <-
							createEffects("covarBehaviorSymmetricRate", varname,
								yName=names(xx$depvars)[j],
								zName=covarname,
								groupName=groupName, group=group,
								netType=netType, name=name)
					}
					else
					{
						covBehRateEffects <-
							createEffects("covarBehaviorOneModeRate", varname,
								yName=names(xx$depvars)[j],
								zName=covarname,
								groupName=groupName, group=group,
								netType=netType, name=name)
					}
					covRateEffects <- rbind(covRateEffects, covBehRateEffects)
				}
				if ((types[j] == "bipartite" &&
						attr(xx$depvars[[j]], 'nodeSet')[1] == nodeSet))
				{
					newEffects <-
						createEffects("covarABehaviorBipartiteObjective", varname,
							covarname, names(xx$depvars)[j],
							groupName=groupName, group=group,
							netType=netType, name=name)
					covObjEffects <- rbind(covObjEffects, newEffects)
				}
			}
		}

		list(objEff=covObjEffects, rateEff=covRateEffects)
	}
	##@covBBehEff internal getEffects
	covBBehEff<- function(varname, covarname, nodeSetB, name)
	{
		covObjEffects <-  NULL
		for (j in seq(along=xx$depvars))
		{
			if ((types[j] == "bipartite" &&
					attr(xx$depvars[[j]], 'nodeSet')[2] == nodeSetB))
			{
				newEffects <-
					createEffects("covarBBehaviorBipartiteObjective", varname,
						covarname, names(xx$depvars)[j],
						groupName=groupName, group=group,
						netType=netType, name=name)
				covObjEffects <- rbind(covObjEffects, newEffects)
			}
		}
		list(objEff=covObjEffects)
	}

	##@covarNetNetEff internal getEffects
	covarNetNetEff<- function(othernetname, covarname, nodeSetsj, nodeSetk, poszvar, name)
	{
		if (tr | poszvar) # positive variance of z, or any z missing
		{
			if (length(nodeSetsj) <= 1) ## second network onemode
			{
				processA <- TRUE
				processB <- TRUE
				objEffectsOneMode <- createEffects("covarNetNetObjective", othernetname,
					covarname, name=name,
					groupName=groupName, group=group,
					netType=netType)
			}
			else ## second network twomode
			{
				processA <- (nodeSetsj[[1]] == nodeSetk)
				processB <- (nodeSetsj[[2]] == nodeSetk)
				objEffectsOneMode <- NULL
			}
			if (processA)
			{
				objEffectsA <- createEffects("covarANetNetObjective", othernetname,
					covarname, name=name,
					groupName=groupName, group=group,
					netType=netType)
			}
			else
			{
				objEffectsA <- NULL
			}
			if (processB)
			{
				objEffectsB <- createEffects("covarBNetNetObjective", othernetname,
					covarname, name=name,
					groupName=groupName, group=group,
					netType=netType)
			}
			else
			{
				objEffectsB <- NULL
			}
			objEffectsAB <- createEffects("covarABNetNetObjective", othernetname,
				covarname, name=name,
				groupName=groupName, group=group,
				netType=netType)
			objEffects <- rbind(objEffectsOneMode, objEffectsAB, objEffectsA, objEffectsB)
		}
		else
		{
			objEffects <- NULL
		}
		objEffects
	}

	##@covContEff internal getEffects
	covContEff <- function(varname, covarname, nodeSet, same=FALSE,
                         ## same indicates that varname and covarname are
                         ## the same: just one rate effect required
                         ## type is no longer used
                           type=c('', 'Var', 'Beh'), name)
	{
        	covObjEffects <-  NULL
		if (!same)
		{
			covObjEffects <- createEffects("covarContinuousObjective", varname,
                                    covarname, name=name,
                                    groupName=groupName, group=group,
                                         netType=netType)
		}
		list(objEff=covObjEffects)
	}

	###################################
	## start of function getEffects
	##################################
	if (getDocumentation)
	{
		tt <- getInternals()
		return(tt)
	}
	if (!inherits(x, 'sienaGroup') && !inherits(x, 'sienadata') && !inherits(x, 'siena'))
	{
		stop('Not a valid siena data object or group')
	}
	if (inherits(x, 'sienaGroup'))
	{
		groupx <- TRUE
	}
	else
	{
		groupx <- FALSE
	}
	tr <- TRUE; # supersedes restrictions by poszvar and moreThan2; 1.1-306
	## validate the object?
	## find the total number of periods to be processed = local var observations
	## then process the first or only data object. Fill in starting values
	## for other periods from the other objects, if any.
	if (groupx)
	{
		groupNames <- names(x)
		observations <- attr(x, 'observations')
		xx <- x[[1]]
		periodNos <- attr(x, "periodNos")
	}
	else
	{
		groupNames <- 'Group1'
		observations <- x$observations - 1
		xx <- x
		periodNos <- 1:observations
	}
	n <- length(xx$depvars)
	types <- sapply(xx$depvars, function(x)attr(x, 'type'))
	#sparses <- sapply(xx$depvars, function(x)attr(x, 'sparse'))
	nOneModes <- sum(types == 'oneMode')
	#nBehaviors <- sum(types == 'behavior')
	nBipartites <- sum(types =='bipartite')
	nContinuous <- sum(types == 'continuous')
	effects <- vector('list',n+1) 			# n+1 th place for all sde parameters
	#nodeSetNames <- sapply(xx$nodeSets, function(x)attr(x, 'nodeSetName'))
	names(effects) <- names(xx$depvars) 	# n+1 th place has no name

	if (onePeriodSde && xx$observations > 2)
		stop('onePeriodSde only possible in case of 2 observations')

	if (onePeriodSde && groupx)
		stop('onePeriodSde not possible in combination with multi-group')

	for (i in 1:(n-nContinuous))
	{
		varname<- names(xx$depvars)[i]
		groupName <- groupNames[1]
		group <- 1
		noPeriods <- xx$observations - 1 ## xx is a single object
		depvar <- xx$depvars[[i]]
		if (groupx)
		{
			netnamesub <- match(varname, attr(x, 'netnames'))
			attr(depvar, 'allUpOnly') <- attr(x, 'allUpOnly')[netnamesub]
			attr(depvar, 'anyUpOnly') <- attr(x, 'anyUpOnly')[netnamesub]
			attr(depvar, 'anyDownOnly') <- attr(x, 'anyDownOnly')[netnamesub]
			attr(depvar, 'allDownOnly') <- attr(x, 'allDownOnly')[netnamesub]
			if (types[i] == 'oneMode')
				attr(depvar, 'symmetric') <- attr(x, 'symmetric')[netnamesub]
		}
		else
		{
			attr(depvar, 'allUpOnly') <- all(attr(depvar, 'uponly'))
			attr(depvar, 'anyUpOnly') <- any(attr(depvar, 'uponly'))
			attr(depvar, 'anyDownOnly') <- any(attr(depvar, 'downonly'))
			attr(depvar, 'allDownOnly') <- all(attr(depvar, 'downonly'))
		}
		switch(types[i],
			behavior =
			{
				netType <- "behavior"
				tmp <- behaviorNet(depvar, varname)
				effects[[i]] <- tmp$effects
				attr(effects[[i]], 'starts') <- tmp$starts
				attr(effects[[i]], 'settings') <- ''
			},
			oneMode =
			{
				netType <- "oneMode"
				tmp <- oneModeNet(depvar, varname)
				effects[[i]] <- tmp$effects
				attr(effects[[i]], 'starts') <- tmp$starts
				attr(effects[[i]], 'settings') <- tmp$settingsDescription
			},
			bipartite =
			{
				netType <- "bipartite"
				tmp <- bipartiteNet(depvar, varname)
				effects[[i]] <- tmp$effects
				attr(effects[[i]], 'starts') <- tmp$starts
				attr(effects[[i]], 'settings') <- ''
			},
			stop('error type'))
	}
	settingsList <- lapply(effects, function(ef){attr(ef,'settings')})
	if (nContinuous > 0)
	{
		groupName <- groupNames[1]
        	group <- 1
	        noPeriods <- xx$observations - 1
		netType <- "continuous"
		contIndices <- (n-nContinuous+1):n ## indicates continuous depvars
		varnames <- names(xx$depvars)[contIndices]
        	depvars <- xx$depvars[contIndices]
		tmp <- continuousNet(depvars,varnames)
		effects[[n+1]] <- tmp$effects
		attr(effects[[n+1]], 'starts') <- tmp$starts
		for (i in contIndices)
		{
			# all the continuous variable specific effects are currently
			# also part of effects[[n+1]]
		}
	}
	## add starting values for the other objects
	if (groupx && length(x) > 1)
	{
		period <-  xx$observations ##periods used so far

		for (group in 2:length(x))
		{
			xx <- x[[group]]
			n <- length(xx$depvars)
			types <- sapply(xx$depvars, function(x)attr(x, 'type'))
			noPeriods <- xx$observations - 1
			nContinuous <- sum(types == 'continuous')

			for (i in 1:(n-nContinuous))
			{
				varname<- names(xx$depvars)[i]
				depvar <- xx$depvars[[i]]
				netnamesub <- match(varname, attr(x, 'netnames'))
				if (types[i] == 'oneMode')
				{
					attr(depvar, 'symmetric') <- attr(x, 'symmetric')[netnamesub]
				}
					switch(types[i],
						behavior =
						{
							starts <-  getBehaviorStartingVals(depvar)
						## first for the rate parameters
							## find the appropriate set of effects
							eff <- match(varname, names(effects))
							if (is.na(eff))
						{
								stop("depvars don't match")
						}
							effectname <- paste('rate ', varname,' (period ',
							period + 1:noPeriods, ')',sep='')
						use <- effects[[eff]]$effectName %in% effectname
							effects[[eff]][use, c('include','initialValue',
								'groupName', 'group', 'period')] <-
									list(TRUE, starts$startRate,
									groupNames[group], group, 1:noPeriods)
									## now sort out the tendency and update the
									## attribute on the effects list:
						newdif <- c(starts$dif, attr(effects[[eff]], "starts")$dif)
									meandif <- mean(newdif, na.rm=TRUE)
									vardif <- var(as.vector(newdif), na.rm=TRUE)
									if (meandif < 0.9 * vardif)
									{
										tendency <- 0.5 * log((meandif + vardif)/
											(vardif - meandif))
									}
									else
									{
										tendency <- meandif / (vardif + 1)
									}
									untrimmed <- tendency
									tendency <- ifelse(tendency < -3.0, -3.0,
										ifelse(tendency > 3/0, 3.0, tendency))
									use <- (effects[[eff]]$shortName == "linear" &
										effects[[eff]]$type == "eval")
									effects[[eff]][use, c("include", "initialValue",
							"untrimmedValue")] <- list(TRUE, tendency, untrimmed)
											attr(effects[[eff]], 'starts')$dif <- newdif
						},
						oneMode =
						{
							starts <- getNetworkStartingVals(depvar)
						## first for the rate parameters
							## find the appropriate set of effects
							eff <- match(varname, names(effects))
							if (is.na(eff))
							{
								stop("depvars don't match")
							}
						thisSettingDescription <- paste(unlist(describeTheSetting(depvar)), collapse=" ")
						if (thisSettingDescription !=
									paste(unlist(settingsList[[varname]]), collapse=" "))
						{
							stop(paste('setting definitions do not match (group ',
								group,', variable ', varname,')', sep=''))

						}
						if (thisSettingDescription == "")
						{
							effectname <- paste('constant ', varname, ' rate (period ',
								period + 1:noPeriods,')', sep='')
							use <- effects[[eff]]$effectName %in% effectname
							effects[[eff]][use, c('include', 'initialValue',
								"groupName", "group", "period")] <-
									list(TRUE, starts$startRate,
									groupNames[group], group, 1:noPeriods)
						}
						else
						{
							effectname <- paste('setting rate ', varname, ' (period ',
								period + 1:noPeriods,')', sep='')
							use <- grep(effectname, effects[[eff]]$effectName, fixed=TRUE)
							effects[[eff]][use, c('include', 'initialValue',
								"groupName", "group", "period")] <-
									list(TRUE, starts$startRateSett,
										groupNames[group], group, 1:noPeriods)
						}
									## now sort out the degree and
									## update the attribute on the effects list
									oldstarts <- attr(effects[[eff]], "starts")
									alpha <- c(oldstarts$alpha, starts$alpha)
									prec <- c(oldstarts$prec, starts$prec)
									degree <- sum(alpha * prec) / sum(prec)
									untrimmed <- degree
									degree <- ifelse (degree < -3, -3,
										ifelse(degree > 3, 3, degree))
									attr(effects[[eff]], "starts")$alpha <- alpha
									attr(effects[[eff]], "starts")$prec <-  prec
									if (attr(depvar, 'symmetric'))
									{
								effects[[eff]][effects[[eff]]$shortName == 'density' &
											effects[[eff]]$type == 'eval',
										c('initialValue','untrimmedValue')] <-
											list(degree, untrimmed)
									}
									else
									{
								if (!(attr(x,'anyUpOnly')[netnamesub] || 
											attr(x, 'anyDownOnly')[netnamesub]))
										{
									effects[[eff]][effects[[eff]]$shortName == 'density' &
												effects[[eff]]$type == 'eval',
													c('initialValue', "untrimmedValue")] <-
													list(degree, untrimmed)
										}
									}
						},
						bipartite =
						{
							starts <- getBipartiteStartingVals(depvar)
						## first for the rate parameters
							## find the appropriate set of effects
							eff <- match(varname, names(effects))
							if (is.na(eff))
							{
								stop("depvars don't match")
							}
							effectname <- paste('constant ', varname,
								' rate (period ',
								period + 1:noPeriods,')', sep='')
							use <- effects[[eff]]$effectName %in% effectname
							effects[[eff]][use, c('include', 'initialValue',
								'groupName', 'group', 'period')] <-
									list(TRUE, starts$startRate,
										groupNames[group], group, 1:noPeriods)
									## now sort out the degree and
									## update the attribute on the effects list
									oldstarts <- attr(effects[[eff]], "starts")
									alpha <- c(oldstarts$alpha, starts$alpha)
									prec <- c(oldstarts$prec, starts$prec)
									degree <- sum(alpha * prec) / sum(prec)
									untrimmed <- degree
									degree <- ifelse (degree < -3, -3,
										ifelse(degree > 3, 3, degree))
									attr(effects[[eff]], "starts")$alpha <- alpha
									attr(effects[[eff]], "starts")$prec <-  prec
							if (!(attr(x,'anyUpOnly')[netnamesub] || 
									attr(x, 'anyDownOnly')[netnamesub]))											
									{
								effects[[eff]][effects[[eff]]$shortName == 'density' &
											effects[[eff]]$type == 'eval',
												c('initialValue', "untrimmedValue")] <-
												list(degree, untrimmed)
									}
						},
						stop('error type dependent variable'))
			}
			if (nContinuous > 0)
			{
				contIndices <- (n-nContinuous+1):n ## indicates continuous depvars
				varnames <- names(xx$depvars)[contIndices]
				depvars <- xx$depvars[contIndices]
				starts <- getContinuousStartingVals(depvars, onePeriodSde = FALSE)
				## At this point, onePeriodSde is always FALSE, as the combination
				## of multi-group and onePeriodSde = TRUE is not feasible
				for (i in 1:nContinuous) ## check whether all continous vars match
				{
					eff <- match(varnames[i], names(effects))
					if (is.na(eff))
					{
						stop("depvars don't match")
					}
				}
				effectname <- paste('scale parameter (period ',
									period + 1:noPeriods, ')', sep='')
				use <- effects[[n+1]]$effectName %in% effectname
				effects[[n+1]][use, c('include','initialValue',
										'groupName', 'group', 'period')] <-
					list(TRUE, starts$startScale, groupNames[group], group, 1:noPeriods)
			}

			period <-  period + xx$observations ##periods used so far
		}
	}
	effects <- do.call(rbind, effects)
	effects <- cbind(effects, effectNumber=1:nrow(effects))
	cl <- class(effects)
	if (groupx)
		class(effects) <- c('sienaGroupEffects','sienaEffects', cl)
	else
		class(effects) <- c('sienaEffects', cl)
	myrownames <- paste(sapply(strsplit(row.names(effects), ".", fixed=TRUE),
			function(x)paste(x[1:2], collapse='.')),
		effects$type, sep='.')
	mytab <- table(myrownames)
	for (i in 1:length(mytab))
	{
		myrownames[myrownames == names(mytab)[i]] <-
			paste(myrownames[myrownames == names(mytab)[i]], 1:mytab[i], sep=".")
	}
	myrownames <- sub("Effects", "", myrownames)
	myrownames <- sub("rate.rate", "rate", myrownames)
	rownames(effects) <- myrownames
	attr(effects, "version") <- packageDescription(pkgname, fields = "Version")
	attr(effects, "starts") <- NULL
	attr(effects, "onePeriodSde") <- onePeriodSde
	attr(effects, "settings") <- settingsList
	effects
}

##@getBehaviorStartingVals DataCreate
getBehaviorStartingVals <- function(depvar)
{
	drange <- attr(depvar, 'range')
	depvar <- depvar[,1,]
	nactors <- nrow(depvar)
	## rate
	if (round(drange) == 2)
	{
		rr <- range(depvar, na.rm=TRUE)
		dif <- t(diff(t(depvar))) ##calculate column differences
		tmp <- sapply(1:ncol(dif), function(x, y, z){
			mintab <- c("FALSE"=0, "TRUE" = 0)
			mintab[1] <- 1 + sum(dif[z[, x] == rr[1], x] == 0, na.rm=TRUE)
			mintab[2] <- 1 + sum(dif[z[, x] == rr[1], x] > 0, na.rm=TRUE)
			maxtab <- c("FALSE"=0, "TRUE" = 0)
			maxtab[1] <- 1 + sum(dif[z[, x] == rr[2], x] == 0, na.rm=TRUE)
			maxtab[2] <- 1 + sum(dif[z[, x] == rr[2], x] < 0, na.rm=TRUE)
			val <- mintab[2] / sum(mintab) + maxtab[2] / sum(maxtab)
			if (val > 0.9) val <- 0.5
			c(-log(1 - val), mintab, maxtab)
		}, z = depvar, y = dif)
		startRate <- tmp[1, ]
		##tendency
		tmp <- rowSums(tmp[-1, , drop=FALSE])
		tendency <- log((tmp[2] * (tmp[3] + tmp[4])) /
			(tmp[4] * (tmp[1] + tmp[2])))
		untrimmed <- tendency
		tendency <- ifelse(tendency < -2, -2, ifelse (tendency > 2, 2,
				tendency))
	}
	else
	{
		dif <- t(round(diff(t(depvar)))) ## get column differences
		absdif <- colSums(abs(dif), na.rm=TRUE) ##sum abs over columns
		# sqrdif <- colSums(dif*dif, na.rm=TRUE) ##sum sqr over columns
		# dif <- colSums(dif,na.rm=TRUE) ## sum over columns
		# startRate <- nactors / (nactors-1) * (sqrdif/nactors -
		#   dif * dif/nactors/nactors)
		startRate <- apply(dif, 2, var, na.rm=TRUE)
		startRate <- pmax(startRate, 0.1 + absdif/nactors)
		## tendency
		##  dif<- sum(dif/nactors,na.rm=TRUE)
		##  dif2<- nactors/(nactors-1)*(sum(sqrdif,na.rm=TRUE)/nactors-dif^2)
		dif1 <- mean(dif, na.rm=TRUE)
		dif2 <- var(as.vector(dif), na.rm=TRUE)
		if (is.na(dif2))
		{
			stop('Too little information in the dependent variable')
		}
		if (abs(dif1) < 0.9 * dif2)
		{
			tendency <- 0.5 * sign(dif1) * log((abs(dif1)+dif2)/(dif2-abs(dif1)))
		}
		else
		{
			tendency <- dif1/(dif2+1)
		}
	}
	untrimmed <- tendency
	tendency <- ifelse(tendency < -3, -3,
		ifelse (tendency > 3, 3, tendency))
	list(startRate=startRate, tendency=tendency, untrimmed = untrimmed, dif=dif)
}
##@getContinuousStartingVals DataCreate
getContinuousStartingVals <- function(depvars, onePeriodSde)
{
	depvar <- depvars[[1]][,1,]       # first continuous dependent variable
	nActors <- nrow(depvar)           # no. of actors
	nPeriods <- ncol(depvar) - 1	  # no. of periods
	nCont <- length(depvars)          # no. of continuous variables

	if(nCont > 1)
		stop("getContinuousStartingVals not defined for more than one continuous variable")

	# determine SDE parameters by regressing each observation on the
	# preceding observation, using the Bergstrom formula
	# note: wiener process parameter G is set to 1 for identifiability
	LL <- function(theta)
	{
		a <- theta[1]
		b0 <- theta[2]
		tau <- theta[3:(2+nPeriods)]
		minlogliks <- rep(0, nPeriods)

		for(i in 1:nPeriods)
		{	# actors present at time i and i+1
			act <- which(!is.na(depvar[,i] + depvar[,i+1]))
			R <- depvar[act,i+1] - exp(a * tau[i]) * depvar[act,i] -
				(exp(a*tau[i]) - 1) * b0 / a
			R <- suppressWarnings(dnorm(R, 0, sqrt((exp(2 * a * tau[i]) - 1) / (2*a)),
				log = TRUE))
			minlogliks[i] <- -sum(R)
		}
		sum(minlogliks)
    	}

	LLonePeriodSde <- function(theta)
	{
		a <- theta[1]
		b0 <- theta[2]
		g <- theta[3]
		# actors present at time 1 and 2
		act <- which(!is.na(depvar[,1] + depvar[,2]))
		R <- depvar[act,2] - exp(a) * depvar[act,1] -
			(exp(a) - 1) * b0 / a
		R <- suppressWarnings(dnorm(R, 0, sqrt((exp(2 * a) - 1) * g^2 / (2*a)),
			log = TRUE))
		-sum(R)
	}

	if (onePeriodSde)
		fit <- optim(theta <- c(-0.5, 3, 1), LLonePeriodSde, hessian = TRUE)
	else
		fit <- optim(theta <- c(-0.5, 3, rep(1, nPeriods)), LL, hessian = TRUE,
				method = "BFGS", control = list(maxit = 10000))

	if (fit$convergence != 0)
		stop("Initial SDE parameter values could not be determined.")

	## NN: still prints the SDE initial parameters and standard errors when getEffects
	##     is called
	cat("SDE init parameters: ", fit$par, '\n')
	cat("SDE par stand errors:", sqrt(diag(solve(fit$hessian))), '\n')

	if (onePeriodSde) # return: tau, g, a, b0
		list(startScale = 1, startWiener = fit$par[3], startFbic = fit$par[1:2])
	else
		list(startScale = fit$par[3:(2+nPeriods)], startWiener = 1,
			 startFbic = fit$par[1:2])
}
##@getNetworkStartingVals DataCreate
getNetworkStartingVals <- function(depvar)
{
	noPeriods <- attr(depvar, "netdims")[3] - 1
	##rate
	##get distance and number of valid links
	if (!attr(depvar,'sparse'))
	{
		nactors <- nrow(depvar)
		use <- !is.na(depvar) & (depvar == 10 | depvar == 11)
		depvar[use] <- depvar[use] - 10  ## remove structural values
		tmp <- sapply(1:noPeriods, function(x, z){
			diag(z[ , , x]) <- NA
			diag(z[, , x + 1]) <- NA
			matdiff <- sum(z[, , x + 1] != z[, , x], na.rm=TRUE)
			# matchange0 <- table(z[, , x + 1], z[, , x])
			# Changed to protect against zero rows or columns
			mc00 <- sum((1 - z[ , , x+1])*(1 - z[ , , x]), na.rm=TRUE)
			mc01 <- sum(z[ , , x+1]*(1 - z[ , , x]), na.rm=TRUE)
			mc10 <- sum((1 - z[ , , x+1])*z[ , , x], na.rm=TRUE)
			mc11 <- sum(z[ , , x+1]*z[ , , x], na.rm=TRUE)
			matchange <- matrix(c(mc00, mc01, mc10, mc11), 2, 2)
			#cat(matchange0,'\n',matchange,'\n')
			matcnt <- nactors * nactors -
				sum(is.na(z[, , x + 1]) | is.na(z[, , x]))
			tmp <- c(matcnt=matcnt, matdiff=matdiff, matchange=matchange)
			names(tmp) <- c("matcnt", "matdiff", "matchangeFrom0To0",
				"matchangeFrom0To1",
				"matchangeFrom1To0", "matchangeFrom1To1")
			tmp
		}, z=depvar)
	}
	else
	{
		nactors <- nrow(depvar[[1]])
		matdiff<- rep(NA, noPeriods)
		matcnt<- rep(NA, noPeriods)
		matchange<- matrix(NA, nrow=4, ncol=noPeriods)
		for (i in 1: noPeriods)
		{
			mymat1 <- depvar[[i]]
			mymat2 <- depvar[[i+1]]
			use <- mymat1@x %in% c(10, 11)
			mymat1@x[use] <- mymat1@x[use] - 10
			use <- mymat2@x %in% c(10, 11)
			mymat2@x[use] <- mymat2@x[use] - 10
			mymat1 <- drop0(mymat1)
			mymat2 <- drop0(mymat2)
			diag(mymat1) <- NA
			diag(mymat2) <- NA
			mydif <- mymat2 - mymat1
			matdiff[i] <- sum(abs(mydif), na.rm=TRUE)
			tmp <- table(mydif@x)
			dummy <- factor(NA, levels=c(-1,0,1))
			dummy <- table(dummy)
			dummy[names(tmp)] <- tmp
			tmp <- dummy
			tmp00 <- nactors * nactors - length(mydif@x)
			tmp <- c(tmp00, tmp[c(3, 1, 2)])
			matchange[, i] <- tmp
			matcnt[i] <- sum(tmp)
		}
		matchange <- data.frame(matchange)
		tmp <- as.matrix(rbind(matcnt=matcnt, matdiff=matdiff,
				matchange=matchange))
		row.names(tmp) <- c("matcnt", "matdiff", "matchangeFrom0To0",
			"matchangeFrom0To1",
			"matchangeFrom1To0", "matchangeFrom1To1")
	}
	distance <- attr(depvar, "distance" )
	if (attr(depvar,'symmetric'))
		startRate <- nactors * (0.2 + distance)/(tmp['matcnt',] %/% 2 +1)
	else
		startRate <- nactors * (0.2 + 2 * distance)/(tmp['matcnt',] + 1)
	startRate <- pmax(0.1, startRate)
	startRate <- pmin(100, startRate)
	if (length(attr(depvar,'settings')) >= 2) # also see above with 0.75
	{
		startR <- rep(startRate, length(attr(depvar,'settings')))
#		setR <- rep(sapply(attr(depvar,'settings'), function(x){x$id}), length(startRate))
		setR <- rep(attr(depvar,'settings'), length(startRate))
		startR[setR=='primary'] <- 0.75 * startR[setR=='primary']
		startR[setR!='primary'] <- 0.5
		startRateSett <- startR
	}
	else
	{
		startRateSett <- NA
	}
	##degree
	matchange<- as.matrix(tmp[grep("matchange", rownames(tmp)),,drop=FALSE])
	if (attr(depvar,'symmetric'))
	{
		matchange <- matchange %/% 2
	}
	p01 <- ifelse (matchange["matchangeFrom0To0", ] +
		matchange["matchangeFrom0To1", ] >=1,
	matchange["matchangeFrom0To1", ] /
		(matchange["matchangeFrom0To0", ] +
			matchange["matchangeFrom0To1", ]), 0.5)
	p10 <- ifelse (matchange["matchangeFrom1To0", ]
		+ matchange["matchangeFrom1To1", ] >=1,
		matchange["matchangeFrom1To0", ] /
			(matchange["matchangeFrom1To0", ] +
				matchange["matchangeFrom1To1", ]), 0.5)
	p01 <- pmax(0.02, p01)
	p10 <- pmax(0.02, p10)
	p01 <- pmin(0.98, p01)
	p10 <- pmin(0.98, p10)
	alpha <- 0.5 * log(p01 / p10)
	p00 <- ifelse (matchange["matchangeFrom0To0", ] +
		matchange["matchangeFrom0To1", ] >=1,
	matchange["matchangeFrom0To0", ] /
		(matchange["matchangeFrom0To0", ] +
			matchange["matchangeFrom0To1", ]), 0.0)
	p11 <- ifelse (matchange["matchangeFrom1To0", ]
		+ matchange["matchangeFrom1To1", ] >=1,
		matchange["matchangeFrom1To1", ] /
			(matchange["matchangeFrom1To0", ] +
				matchange["matchangeFrom1To1", ]), 0.0)
	p00 <- pmax(0.02, p00)
	p11 <- pmax(0.02, p11)
	p00 <- pmin(0.98, p00)
	p11 <- pmin(0.98, p11)
	prec <- ifelse(matchange["matchangeFrom0To1", ] *
		matchange["matchangeFrom1To0", ] >= 1,
	4 / ((p00 / matchange["matchangeFrom0To1", ]) +
		(p11 / matchange["matchangeFrom1To0", ])), 1e-6)
	alphaf1 <- sum(alpha * prec / sum(prec))
	untrimmed <- alphaf1
	alphaf1 <- ifelse(alphaf1 < -3, -3, ifelse(alphaf1 > 3, 3, alphaf1))
	list(startRate=startRate, degree=alphaf1, alpha=alpha, prec=prec, tmp=tmp,
		untrimmed = untrimmed, startRateSett=startRateSett)
}
##@getBipartiteStartingVals DataCreate
getBipartiteStartingVals <- function(depvar)
{
	noPeriods <- attr(depvar, "netdims")[3] - 1
	##rate
	##get distance and number of valid links
	if (!attr(depvar,'sparse'))
	{
		nsenders<- nrow(depvar)
		nreceivers <- ncol(depvar)
		use <- !is.na(depvar) & (depvar == 10 | depvar == 11)
		depvar[use] <- depvar[use] - 10  ## remove structural values
		tmp <- sapply(1:noPeriods, function(x, z){
			matdiff <- sum(z[, , x + 1] != z[, , x], na.rm=TRUE)
			# matchange0 <- table(z[, , x + 1], z[, , x])
			# Changed to protect against zero rows or columns
			mc00 <- sum((1 - z[ , , x+1])*(1 - z[ , , x]), na.rm=TRUE)
			mc01 <- sum(z[ , , x+1]*(1 - z[ , , x]), na.rm=TRUE)
			mc10 <- sum((1 - z[ , , x+1])*z[ , , x], na.rm=TRUE)
			mc11 <- sum(z[ , , x+1]*z[ , , x], na.rm=TRUE)
			matchange <- matrix(c(mc00, mc01, mc10, mc11), 2, 2)
			#cat(matchange0,'\n',matchange,'\n')
			matcnt <- nsenders * nreceivers -
				sum(is.na(z[, , x + 1]) | is.na(z[, , x]))
			tmp <- c(matcnt=matcnt, matdiff=matdiff, matchange=matchange)
			names(tmp) <- c("matcnt", "matdiff", "matchangeFrom0To0",
				"matchangeFrom0To1",
				"matchangeFrom1To0", "matchangeFrom1To1")
			tmp
		}, z=depvar)
	}
	else
	{
		nsenders <- nrow(depvar[[1]])
		nreceivers <- ncol(depvar[[1]]) # CS: Was 2, but why?
		matdiff<- rep(NA, noPeriods)
		matcnt<- rep(NA, noPeriods)
		matchange<- matrix(NA, nrow=4, ncol=noPeriods)
		for (i in 1: noPeriods)
		{
			mymat1 <- depvar[[i]]
			mymat2 <- depvar[[i+1]]
			use <- mymat1@x %in% c(10, 11)
			mymat1@x[use] <- mymat1@x[use] - 10
			use <- mymat2@x %in% c(10, 11)
			mymat2@x[use] <- mymat2@x[use] - 10
			mymat1 <- drop0(mymat1)
			mymat2 <- drop0(mymat2)
			mydif <- mymat2 - mymat1
			matdiff[i] <- sum(abs(mydif), na.rm=TRUE)
			tmp <- table(mydif@x)
			tmp00 <- nsenders * nreceivers - length(mydif@x)
			tmp <- c(tmp00, tmp[c(3, 1, 2)])
			matchange[,i] <- tmp
			matcnt[i] <- sum(tmp)
		}
		matchange <- data.frame(matchange)
		tmp <-as.matrix(rbind(matcnt=matcnt, matdiff=matdiff,
				matchange=matchange))
		row.names(tmp) <- c("matcnt", "matdiff", "matchangeFrom0To0",
			"matchangeFrom0To1",
			"matchangeFrom1To0", "matchangeFrom1To1")
	}
	distance <- attr(depvar, "distance" )
	startRate <- nreceivers * (0.2 + 2 * distance)/(tmp['matcnt',] + 1)
	# CS: the above used to be 'nsenders' instead of 'nreceivers';
	#     this was a wrong calculation and led to extremely high
	#     rate parameters for comparatively small receiver nodesets
	#     slowing down estimation and prohibiting identification
	#     of parameters.
	startRate <- pmax(0.1, startRate)
	startRate <- pmin(100, startRate)
	##degree
	matchange<- as.matrix(tmp[grep("matchange", rownames(tmp)),,drop=FALSE])
	p01 <- ifelse (matchange["matchangeFrom0To0", ] +
		matchange["matchangeFrom0To1", ] >=1,
	matchange["matchangeFrom0To1", ] /
		(matchange["matchangeFrom0To0", ] +
			matchange["matchangeFrom0To1", ]), 0.5)
	p10 <- ifelse (matchange["matchangeFrom1To0", ]
		+ matchange["matchangeFrom1To1", ] >=1,
		matchange["matchangeFrom1To0", ] /
			(matchange["matchangeFrom1To0", ] +
				matchange["matchangeFrom1To1", ]), 0.5)
	p01 <- pmax(0.02,p01)
	p10 <- pmax(0.02,p10)
	p01 <- pmin(0.98,p01)
	p10 <- pmin(0.98,p10)
	alpha <- 0.5 * log(p01/p10)
	p00 <- ifelse (matchange["matchangeFrom0To0", ] +
		matchange["matchangeFrom0To1", ] >=1,
	matchange["matchangeFrom0To0", ] /
		(matchange["matchangeFrom0To0", ] +
			matchange["matchangeFrom0To1", ]), 0.0)
	p11 <- ifelse (matchange["matchangeFrom1To0", ]
		+ matchange["matchangeFrom1To1", ] >=1,
		matchange["matchangeFrom1To1", ] /
			(matchange["matchangeFrom1To0", ] +
				matchange["matchangeFrom1To1", ]), 0.0)
	p00 <- pmax(0.02,p00)
	p11 <- pmax(0.02,p11)
	p00 <- pmin(0.98,p00)
	p11 <- pmin(0.98,p11)
	prec <- ifelse(matchange["matchangeFrom0To1", ] *
		matchange["matchangeFrom1To0", ] >= 1,
	4 / ((p00 / matchange["matchangeFrom0To1", ]) +
		(p11 / matchange["matchangeFrom1To0", ])), 1e-6)
	alphaf1 <- sum(alpha*prec/sum(prec))
	## }
	untrimmed <- alphaf1
	alphaf1 <- ifelse(alphaf1 < -3, -3, ifelse(alphaf1 > 3, 3, alphaf1))
	list(startRate=startRate, degree=alphaf1, alpha=alpha, prec=prec, tmp=tmp,
		untrimmed = untrimmed)
}


make_specification <- function(x, ...) UseMethod("make_specification", x)

##@ make_specification DataCreate method for siena data sets
make_specification.sienadata <- function(x, nintn = 10, behNintn=4, 
										onePeriodSde=FALSE, ...)
{
	getEffects(x, nintn = nintn, behNintn=behNintn, 
					getDocumentation=FALSE, onePeriodSde=onePeriodSde)
}

##@ make_specification DataCreate method for siena data sets
make_specification.sienaGroup <- function(x, nintn = 10, behNintn=4, 	
											onePeriodSde=FALSE, ...)
{
	getEffects(x, nintn = nintn, behNintn=behNintn, 
					getDocumentation=FALSE, onePeriodSde=onePeriodSde)
}
					
## extra for backward compatibility with siena objects
## created before version 1.6:

##@make_specification.siena Methods
make_specification.siena <- make_specification.sienadata

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: simstatsc.r
# *
# * Description: This module contains the code for simulating the process,
# * communicating with C++. Only subsidiary routines used for maximum likelihood
# *****************************************************************************/
##@simstats0c siena07 Simulation Module
simstats0c <- function(z, x, data=NULL, effects=NULL, fromFiniteDiff=FALSE,
                       returnDeps=FALSE, returnChains=FALSE, returnChangeContributions = FALSE,
                       byWave=FALSE, returnDataFrame=FALSE, returnLoglik=FALSE)
{
    ## retrieve stored information
    f <- FRANstore()
    ## browser()
    ## fix up the interface so can call from outside robmon framework
    if (is.null(z$Deriv))
    {
        z$Deriv <- FALSE
    }
    if (is.null(z$Phase))
    {
        z$Phase <- 1 ### nb be aware
    }
    if (fromFiniteDiff)
    {
        returnDeps <- FALSE
    }
    else
    {
        returnDeps <- z$returnDeps
    }
    if (is.null(z$returnActorStatistics))
	{
		z$returnActorStatistics <- FALSE
	}
	if (is.null(z$returnChangeContributions))
	{
		z$returnChangeContributions <- FALSE
	}
    if (is.null(f$seeds))
    {
        seeds <- NULL
    }
    else
    {
        seeds <- f$seeds
    }
    if (is.null(f$randomseed2))
    {
        randomseed2 <- NULL
    }
    else
    {
        if (fromFiniteDiff)
        {
            randomseed2 <- as.integer(f$storedseed)
        }
        else
        {
            randomseed2 <- as.integer(f$randomseed2)
            f$storedseed <- randomseed2
        }
        ## cat(randomseed2, '\n')
    }
	## The possibility to use snow now has been dropped
	## because RSiena requires R >= 2.15.0
	## and snow is superseded.
	## Therefore useStreams was dropped.
	#if (getRversion() < "2.14.0") ##fake this to repeat old results
	#	##	if (TRUE)
	#{
	#	useStreams <- TRUE
	#}
	#else
	#{
	useStreams <- FALSE
	#}
    ## z$int2 is the number of processors if iterating by period, so 1 means
    ## we are not. Now have removed option to parallelize by period
	ans <- .Call(C_forwardModel, PACKAGE=pkgname, z$Deriv, f$pData, seeds,
				 fromFiniteDiff, f$pModel, f$myeffects, z$theta,
				 randomseed2, returnDeps, z$FinDiff.method,
				 !is.null(z$cl) && useStreams, z$addChainToStore,
				  z$returnChains, returnLoglik, 
				  z$returnActorStatistics, z$returnChangeContributions,
                  z$returnDataFrame)
    if (!fromFiniteDiff)
    {
        if (z$FinDiff.method)
            f$seeds <- ans[[3]]
    }
    if (z$Deriv )
    {
        sc <- t(ans[[2]])
    }
    else
    {
        sc <-  NULL
    }
    ntim <- ans[[4]]
    fra <- t(ans[[1]])
    f$randomseed2 <- ans[[5]]#[c(1,4,3,2)]
    FRANstore(f)
    if (returnDeps)
    {
        sims <- ans[[6]]
    }
    else
    {
        sims <- NULL
    }
    if (z$returnChains)
    {
        chain <- ans[[7]]
    }
    else
    {
        chain <- NULL
    }
	if (returnLoglik)
	{
		loglik <- ans[[8]]
	}
	else
	{
		loglik <- NULL
	}
	if(z$returnChangeContributions)
	{	
		changeContributions <- ans[[9]]
	}
	else
	{
		changeContributions <- NULL
	}
	if(z$returnActorStatistics)
	{
		actorStatistics <- ans[[10]]
	}
	else
	{
		actorStatistics <- NULL
	}
    if (returnDeps)
    {
        ## attach the names
        names(sims) <- f$groupNames
        periodNo <- 1
        for (i in 1:length(sims))
        {
            names(sims[[i]]) <- f$depNames
            for (j in 1:length(sims[[i]]))
            {
                periodNos <- periodNo:(periodNo  + length(sims[[i]][[j]]) - 1)
                names(sims[[i]][[j]]) <- periodNos
            }
            periodNo <- periodNos[length(periodNos)] + 2
        }
    }
		 ## browser()
    list(sc = sc, fra = fra, ntim0 = ntim, feasible = TRUE, OK = TRUE,
         sims=sims, f$seeds, chain=chain, loglik=loglik,
		 actorStatistics = actorStatistics, changeContributions = changeContributions)
}

##@clearData siena07 Finalizer to clear Data object in C++
clearData <- function(pData)
{
    .Call(C_deleteData, PACKAGE=pkgname, pData)
}
##@clearModel siena07 Finalizer to clear Model object in C++
clearModel <- function(pModel)
{
    .Call(C_deleteModel, PACKAGE=pkgname, pModel)
}

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: phase1.r
## *
## * Description: Phase 1 processing is controlled by the functions phase1.1
## * and phase1.2. phase1.1 does the first 10 iterations and checks that
## * finite differencing is going OK. If not, alters epsilon and forces a
## * restart. Phase 1.2 does the rest of the iterations and then calculates
## * the derivative estimate. Both call doPhase1or3iterations which does a block
## * of iterations, to save copying large objects around.
## ****************************************************************************/
##args: x model object (readonly), z control object
##
##@phase1.1 siena07 Do first 10 iters (before check if using finite differences)
phase1.1 <- function(z, x, ...)
{
	## initialise phase 1
	z$SomeFixed <- FALSE
	z$Phase <-  1
	f <- FRANstore()
	int <- z$int # used for multiple processes: number of processes for MoM
	z <- AnnouncePhase(z, x)
	z$phase1Its <- 0
	if (z$returnThetas)
	{
		z$thetas <- c(0, z$theta)
	}
	## fix up iteration numbers if using multiple processes
	if (10 %% int == 0)
	{
		firstNit <- 10
	}
	else
	{
		firstNit <- 10 + int - 10 %% int
	}
	if ((z$n1 - firstNit) %% int == 0)
	{
		endNit <- z$n1
	}
	else
	{
		endNit <- z$n1  + int - (z$n1 - firstNit) %% int
	}
	z$n1 <- endNit
	z <- createSiena07stores(z, z$n1, f)
	z$writefreq <- 10
	z$DerivativeProblem <- FALSE
	z$Deriv <- !z$FinDiff.method ## can do both in phase 3 but not here!
	xsmall <- NULL
	zsmall <- makeZsmall(z)

	nits <- seq(1, firstNit, int)
	if (any(nits >= 6))
	{
		nits6 <- min(nits[nits >= 6 ])
	}
	else
	{
		nits6 <- -1
	}
	### call the subroutine to do the iterations
	z <- doPhase1or3Iterations(1, z, x, zsmall, xsmall, nits, nits6)

	if (UserInterruptFlag() || UserRestartFlag())
	{
		return(z)
	}
	if (z$FinDiff.method)
	{
		npos <- z$npos
		if (any(npos[!z$fixed] < 5))
		{
			j<- (1 : z$pp)[!z$fixed & npos < 5]
			for (i in 1 : length(j))
			{
				Report(c("After ", z$n, " iterations, only ", npos[j[i]],
						" differences in coordinate ", j[i], ".\n"), sep="",
					cf)
				Report(c("with epsilon = ", z$epsilon[j[i]], ".\n"), sep="", cf)
			}
			use <- !z$fixed & npos <= 2
			z$epsilon[use] <- ifelse(z$posj[use],
				3.0 * z$epsilon[use],
				10.0 * z$epsilon[use])
			use <- !z$fixed & npos > 2 & npos < 5
			z$epsilon[use] <- ifelse(z$posj[use],
				2.0 * z$epsilon[use],
				sqrt(10.0) * z$epsilon[use])
			use <- !z$fixed & npos < 5
			z$epsilon[use] <- pmin(100.0 * z$scale[use], z$epsilon[use])
			z$epsilon[use] <- pmax(0.1 * z$scale[use], z$epsilon[use])
			Report(c("New epsilon =", paste(" ", z$epsilon[use], collapse=""),
					".\n"), sep="", cf, fill=80)
			if (z$repeatsForEpsilon <= 4)
			{
				Report("Change value of epsilon and restart Phase 1.\n", cf)
				z$epsilonProblem <- TRUE
				return(z)
			}
			if (any(npos[!z$fixed] <= 1))
			{
				j0s <- which(!z$fixed & npos <= 1)
				for (j0 in j0s)
				{
					Report (c("Difficulties with parameter", j0, ".\n"),
						outf)
					Report(c("After ", z$n, " iterations, only", npos[j0],
							"differences in this coordinate.\n"), outf)
					Report("This parameter is fixed and not estimated.\n",
						outf)
					Report(c("Fix parameter", j0, ", in phase 1.\n"), cf)
				}
				z$newFixed[j0s] <- TRUE
				z$fixed[j0s] <- TRUE
				z$SomeFixed <- TRUE
			}
			if (all(z$fixed))
			{
				z$AllNowFixed <- TRUE
			}
		}
	}
	z
}


##@phase1.2 siena07 Do rest of phase 1 iterations
phase1.2 <- function(z, x, ...)
{
	##finish phase 1 iterations and do end-of-phase processing
	zsmall <- makeZsmall(z)
	xsmall <- NULL
	int <- z$int # used for multiple processes: number of processes for MoM
	if (z$n1 > z$phase1Its)
	{
		nits <- seq((z$phase1Its+1), z$n1, int)
		z <- doPhase1or3Iterations(1, z, x, zsmall, xsmall, nits)
	}
	if (UserInterrupt() || UserRestartFlag())
	{
		return(z)
	}
	# for dolby option: regress deviations fra on scores ssc
	z$regrCoef <- rep(0, z$pp)
	z$regrCor <- rep(0, z$pp)
	if (x$dolby && ((!is.null(z$ssc)) || (!is.null(z$scores))))
	{
		if ((!is.null(z$ssc)) & (z$sf2.byIteration))
		{
			scores <- apply(z$ssc, c(1,3), sum)
		}
		else
		{
			scores <- z$scores
		}
		oldwarn <- getOption("warn")
		options(warn = -1)
		for (i in 1:z$pp)
		{
			if (is.na(var(scores[,i])))
			{
				stop(paste('phase1.2: missing var(scores[,i]) for i =', i))
			}
			if (is.na(var(z$sf[,i])))
			{
				stop(paste('phase1.2: missing var(z$sf[,i]) for i =', i))
			}
			if ((var(scores[,i]) > 0)&&(var(z$sf[,i]) > 0))
			{
				z$regrCoef[i] <- cov(z$sf[,i], scores[,i])/var(scores[,i])
				z$regrCor[i] <- cor(z$sf[,i], scores[,i])
				#				covi <- sum(sapply(1:dim(z$sf2)[2], function(m){cov(z$sf2[,m,i], z$ssc[,m,i])}))
				#				vari.sc <- sum(sapply(1:dim(z$sf2)[2], function(m){var(z$ssc[,m,i])}))
				#				vari.fra <- sum(sapply(1:dim(z$sf2)[2], function(m){var(z$sf2[,m,i])}))
				#				z$regrCoef[i] <- covi/vari.sc
				#				z$regrCor[i] <- covi/sqrt(vari.sc * vari.fra)
			}
			if (is.na(z$regrCor[i])){z$regrCor[i] <- 0}
			if (is.na(z$regrCoef[i])){z$regrCoef[i] <- 0}
			if (!is.finite(z$regrCor[i])){z$regrCor[i] <- 0}
			if (!is.finite(z$regrCoef[i])){z$regrCoef[i] <- 0}
		}
		options(warn = oldwarn)
		Report('Correlations between scores and statistics:\n', cf)
		PrtOutMat(format(as.matrix(t(z$regrCor)), digits = 2, nsmall = 2), cf)
		Report('Regression coefficients:\n', cf)
		PrtOutMat(format(as.matrix(t(z$regrCoef)), digits = 2, nsmall = 2), cf)
	}

	z$timePhase1 <- (proc.time()['elapsed'] - z$ctime) / (z$nit - 1)
	if (x$checktime  && !is.na(z$timePhase1))
	{
		Report(c('Time per iteration in phase 1  =',
				format(z$timePhase1, digits = 4, nsmall = 4),'\n'), lf)
	}
	z$mnfra <- colMeans(z$sf)
	Report('Average deviations NR generated statistics and targets\n', cf)
	Report('after phase 1:\n', cf)
	PrtOutMat(format(as.matrix(z$mnfra), width = 15, nsmall = 6), cf)
	z <- CalculateDerivative(z, x)
	## browser()
	if (!z$OK || z$DerivativeProblem) ##longer phase 1 or use finite differences
	{
		return(z)
	}
	z$SomeFixed <- z$SomeFixed | z$cdSomeFixed
	if (z$SomeFixed)
	{
	  if (!z$gmm)
	  {
	    Report('(Values for fixed parameters are meaningless.)\n', cf)
	    z$dfra[outer(z$fixed, z$fixed, '|')] <- 0
	    diag(z$dfra)[z$fixed] <- 1.0
	    z$mnfra[z$fixed] <- 0.0
	    z$sf[ , z$fixed] <- 0
	  }
	  else
	  {
	    Report('(Values for fixed parameters are meaningless.)\n', cf)
	    z$dfra[outer(z$fixed & !z$gmmEffects, z$fixed & !z$gmmEffects, '|')] <- 0
	    diag(z$dfra)[z$fixed & !z$gmmEffects] <- 1.0
	    z$mnfra[z$fixed & !z$gmmEffects] <- 0.0
	    z$sf[ , z$fixed & !z$gmmEffects] <- 0
	  }
	}
	# Manage derivative matrix
	if (any(diag(z$dfra) < 1e-8))
	{
		sml <- (1 : z$pp)[diag(z$dfra) < 1e-8]
		diag(z$dfra)[sml] <- 1e-3
		Report(c('Diagonal elements(s)', sml,
				'of derivative matrix amended in phase 1.'), cf, fill=80)
	}
	# Invert derivative matrix and define step fchange 
	# that will be made at the end of Phase 1.
	if (inherits(try(dinv <- solve(z$dfra), silent=TRUE), "try-error") & !z$gmm)
	{
		Report('Error message for inversion of dfra: \n', cf)
		diag(z$dfra) <- diag(z$dfra) + 1
		Report('Intervention 1.4.1: ridge added after phase 1.\n', cf)
		if (inherits(try(dinv <- solve(z$dfra), silent=TRUE), "try-error"))
		{
			Report(c('Error. After phase 1, derivative matrix non-invertible',
					'even with a ridge.\n'), cf)
			fchange <- 0
			stop("Cannot proceed: derivative matrix not invertible")
		}
		else
		{
			fchange <- as.vector(dinv %*% z$mnfra)
			z$dinv <- dinv
		}
	}
	else if (z$gmm)
	{
	  # qxq covariance matrix of the statistics
	  sigmagmm <- cov(t(apply(z$sf2, 1, function(x) colSums(x)) - z$targets))
	  W <- solve(sigmagmm) # Matrix of GMoM weights
	  gamma <- z$dfra[,-which(z$gmmEffects==TRUE)] # gammaT qxp matrix of first order derivatives
	  B0 <- t(gamma) %*% W
	  B <- solve(diag(sqrt(rowSums(B0*B0)))) %*% B0 # Row-normalized matrix B
	  D0 <- B %*% gamma # Matrix D = B * gammaT
	  if (inherits(try(dinvGmm <- solve(D0), silent=TRUE), "try-error"))
	  {
	    Report('Error message for inversion of dfra: \n', cf)
	    diag(D0) <- diag(D0) + 1
	    Report('Intervention 1.4.1: ridge added after phase 1.\n', cf)
	    if (inherits(try(dinvGmm <- solve(D0), silent=TRUE), "try-error"))
	    {
	      Report(c('Error. After phase 1, derivative matrix non-invertible',
	               'even with a ridge.\n'), cf)
	      fchange <- 0
	      stop("Cannot proceed: derivative matrix not invertible")
	    }
	  }
	  else
	    {
	      dbGmm <- dinvGmm%*%B # D_0^{-1}*B_0
	      fchangeGmm <- as.vector(dbGmm %*% z$mnfra)
	      fchange <- rep(0,z$pp)
	      fchange[!z$gmmEffects] <- fchangeGmm
	      z$W <- W
	      z$B <- B
	      z$gamma <- gamma
	      z$D0 <- D0
	      z$dinv <- dbGmm
	    }
	}
	else
	{
		fchange <- as.vector(dinv %*% z$mnfra)
		z$dinv <- dinv
	}
	splitDepvars <- "splitDepvars" %in% names(x)
	if (splitDepvars)
	{
		 splitDepvars <- (x$splitDepvars >= 1)
		 oldNRstep <- (x$splitDepvars < 0)
	}
	else
	{
		oldNRstep <- FALSE
	}
	# Partial diagonalization of derivative matrix
	# for use if 0 < x$diagonalize < 1.
	if (!z$gmm)
	{
	  temp <- (1-x$diagonalize)*z$dfra +
	    x$diagonalize*diag(diag(z$dfra), nrow=dim(z$dfra)[1])
	  temp[z$fixed, ] <- 0.0
	  temp[, z$fixed] <- 0.0
	  diag(temp)[z$fixed] <- 1.0
      if ((splitDepvars) & (length(unique(z$effects$name)) > 1))
	  {
		temp[outer(z$effects$name, z$effects$name, '!=')] <- 0
		Report('Split updates between dependent variables \n', cf)
	  }
	  # Invert this matrix
	  z$dinvv <- solve(temp)
	}
	else
	{
	  temp <- (1-x$diagonalize)*dinvGmm +
	    x$diagonalize*diag(diag(dinvGmm), nrow=dim(dinvGmm)[1])
	  temp[z$fixed[!z$gmmEffects], ] <- 0.0  # changed version 1.4.19,
	                                         # also next 2 lines
	  temp[, z$fixed[!z$gmmEffects]] <- 0.0
	  diag(temp)[z$fixed[!z$gmmEffects]] <- 1.0
# Invert this matrix
	  z$dinvv <- solve(temp)%*%B
	}

	if (!z$gmm)
	{
		if (oldNRstep)
		{
# This was used before version 1.6.3
			fchange <- as.vector(z$dinv %*% z$mnfra)
		}
		else
		{
			fchange <- as.vector(z$dinvv %*% z$mnfra)
		}		
	}	

	Report('dfra :\n', cf)
	##  browser()
	PrtOutMat(z$dfra, cf)
	Report('inverse of dfra: dinv = \n', cf)
	PrtOutMat(dinv, cf)
	Report('Modified dinv: dinvv  = \n', cf)
	PrtOutMat(z$dinvv, cf)
	Report('Full Quasi-Newton-Raphson step after phase 1:\n', cf)
	Report(c(paste(1:z$pp, '. ', format(-fchange, width = 12, digits = 6,
					nsmall = 6), sep = '', collapse = '\n'),
			'\n'), cf)
	Report(c('This step is multiplied by the factor ',
			format(0.5 * x$firstg, digits = 5, nsmall = 5, width = 8), '.\n'),
		cf, sep='')
	fchange <- 0.5 * x$firstg * fchange
	if (z$gmm)
	{
		fchange[z$fixed & !z$gmmEffects] <- 0.0
	}
	else
	{
		fchange[z$fixed] <- 0.0
	}
	##check if jump is too large
	maxrat<- max(abs(fchange / z$scale))
	if (maxrat > 10.0)
	{
		fchange <- 10 * fchange / maxrat;
		Report(c('Intervention 1.4.2: jump after phase 1 decreased by factor',
				maxrat, '.\n'), cf)
	}
	##check positivity
	if (z$anyposj)
	{
		neg <- z$posj & fchange >= z$theta
		if (any(neg))
		{
			fchange[neg] <- 0.5 * z$theta[neg]
			Report(c('Intervention 1.4.3: positivity restriction after phase 1',
					'coordinate(s)', (1 : z$pp)[neg], '.'), cf, fill=80)
			Report('\n', cf)
		}
	}
	## make step
	if (x$nsub >= 1)
	{
		z$theta[!z$fixed] <- z$theta[!z$fixed] - fchange[!z$fixed]
	}
	Report(c('Phase 1 achieved after ', z$phase1Its, ' iterations.\n'), cf)
	WriteOutTheta(z)
	z$nitPhase1 <- z$phase1Its
	#	z$phase1devs <- z$sf
	#	z$phase1dfra <- z$frda
	#	z$phase1sdf <- z$sdf
	#	z$phase1sdf2 <- z$sdf2
	#	z$phase1scores <- z$ssc
	#	z$phase1accepts <- z$accepts
	#	z$phase1rejects <- z$rejects
	#	z$phase1aborts <- z$aborts
	##browser()
	z
}

##@CalculateDerivative siena07 Calculates derivative in Phase 1
CalculateDerivative <- function(z, x)
{
	if (z$FinDiff.method || x$maxlike)
	{
		dfra <- t(as.matrix(Reduce("+", z$sdf) / length(z$sdf)))
	}
	else
	{
		##note that warnings is never set as this piece of code is not executed
		##if force_fin_diff_phase_1 is true so warning is zero on entry
		dfra <- derivativeFromScoresAndDeviations(z$ssc, z$sf2,
			z$dfras, z$sscs, z$sf2s, z$sf2.byIteration, z$Phase1nits)
		if (sum(z$f$types == "continuous") > 1) {
		  stop("Analytic sde derivate in phase 1 not implemented for >1 continuous behavior")
		} else if (sum(z$f$types == "continuous") == 1) {
		  dfra <- updateDerivativeSde(z, dfra)
		}
		fromBayes <- 'fromBayes' %in% names(x)
		z$jacobianwarn1 <- rep(FALSE, z$pp)
		if ((any(diag(dfra)[!z$fixed] <= 0)) && (!fromBayes))
		{
			for (i in 1 : z$pp)
			{
				if ((dfra[i, i] < 0)  && (!z$fixed[i]))
				{
					##browser()
					z$jacobianwarn1[i] <- TRUE
					Report(c('Warning: diagonal value', i,
							'is non-positive: ', round(dfra[i, i],8),'  \n\n'), cf)
				}
			}
			if (z$n1 < 200)
			{
				z$n1 <- z$n1 * 2
				Report(c('New phase 1 of increased length because of',
						'unreliable derivative matrix\n'), cf)
				Report(c('New phase 1 of increased length because of',
						'unreliable derivative matrix\n'), lf)
				z$DerivativeProblem <- TRUE
				z$LongerPhase1 <-  TRUE
			}
			else
			{
				z$FinDiff.method <- TRUE
				Report(c('New phase 1 with finite differences because of',
						'unreliable derivative matrix\n'), cf)
				Report(c('New phase 1 with finite differences because of',
						'unreliable derivative matrix\n'), lf)
				z$DerivativeProblem <- TRUE
			}
			return(z)
		}
	}
	if (!z$gmm)
  	{
		dfra[outer(z$fixed,z$fixed,'|')] <- 0
		diag(dfra)[z$fixed] <- 1.0
  	}
  	else
  	{
    	dfra[outer(z$fixed & !z$gmmEffects, z$fixed & !z$gmmEffects,'|')] <- 0
    	diag(dfra)[z$fixed & !z$gmmEffects] <- 1.0
  	}
	Report('Diagonal values of derivative matrix :\n', cf)
	Report(format(diag(dfra), digits = 4, nsmall = 4, width = 8), cf, fill=80)
	someFixed <- FALSE
	if (any(diag(dfra) <= 0 & !z$fixed))
	{
		fromBayes <- 'fromBayes' %in% names(x)
		if (fromBayes)
		{
			# Use the values for diag(dfra) from startupGlobal computed in sienaBayes
			neg <- which((diag(dfra) <= 0 & !z$fixed)[!z$effects$basicRate])
			diag(dfra)[which(!z$effects$basicRate)[neg]] <- x$ddfra[neg]
		}
		else
		{
			neg<- which(diag(dfra) <= 0 & !z$fixed)
			z$fixed[neg] <- TRUE
			z$newFixed[neg] <- TRUE
			someFixed <- TRUE
			Report(c('*** nonpositive diagonal value(s):', neg,
					' is/are fixed.\n'), cf)
			Report(c('Estimation problem with parameter(s)', neg,
					'this/these parameter(s) is/are fixed.\n'), outf)
		}
	}
	z$dfra <- dfra
	z$cdSomeFixed <- someFixed
	z
}

##@FiniteDifferences siena07 Does the extra iterations for finite differences
FiniteDifferences <- function(z, x, fra, fra2)
{
	int <- z$int
	fras <- array(0, dim = c(int, z$pp, z$pp))
	if (z$byWave)
	{
		fras2 <- array(0, dim = c(int, z$f$observations - 1, z$pp, z$pp))
	}
	else
	{
		fras2 <- NULL
	}
	xsmall <- NULL
	for (i in 1 : z$pp)
	{
		zdummy <- makeZsmall(z)
		if (z$Phase == 3 || !z$fixed[i])
		{
			zdummy$theta[i] <- z$theta[i] + z$epsilon[i]
		}
		if (int == 1)
		{
			zz <- x$FRAN(zdummy, xsmall, fromFiniteDiff=TRUE)
			if (!zz$OK)
			{
				z$OK <- zz$OK
				return(z)
			}
		}
		else
		{
			zz <- clusterCall(z$cl, simstats0c, zdummy, xsmall,
				fromFiniteDiff=TRUE)
		}
		if (int == 1)
		{
			fras[1, i, ] <- colSums(zz$fra) - fra
			if (z$byWave)
			{
				fras2[1, , i, ] <- zz$fra - fra2
			}
		}
		else
		{
			for (j in 1 : int)
			{
				fras[j, i, ] <- colSums(zz[[j]]$fra) - fra[j, ]
				if (z$byWave)
				{
					fras2[j, , i, ] <- zz[[j]]$fra - fra2[j, , ]
				}
			}
		}
	}
	if (z$Phase == 1 && z$nit <= 10)
	{
		for (ii in 1: min(10 - z$nit + 1, int))
		{
			z$npos <- z$npos +
				ifelse(abs(diag(matrix(fras[ii, , ], nrow=z$pp))) > 1e-6, 1, 0)
		}
	}
	z$sdf0 <- fras / rep(rep(z$epsilon, each=int), z$pp)
	if (z$byWave)
	{
		z$sdf02 <- fras2 / rep(rep(z$epsilon, each=int * dim(fras2)[2]), z$pp)
	}
	z
}
##@derivativeFromScoresAndDeviations siena07 create dfra from scores and deviations
derivativeFromScoresAndDeviations <- function(scores, deviations, sumdfra,
	sumscores, sumdeviations, byIterations, nIter)
{
	if (byIterations)
	{
		## replaced nested applies by memory saving for loops after problems
		nIterations <- dim(scores)[1]
		nWaves <- dim(scores)[2]
		nParameters <- dim(scores)[3]
		dfra <- matrix(0, nrow=nParameters, ncol=nParameters)
		for (i in 1:nIterations)
		{
			for (j in 1:nWaves)
			{
				dfra <- dfra + outer(scores[i, j, ], deviations[i, j, ])
			}
		}
		tmp <- matrix(vapply(1 : nWaves, function(i){
				outer(colMeans(deviations)[i,],
					colMeans(scores)[i,])}, 
					FUN.VALUE=matrix(0,nParameters,nParameters)), ncol=nWaves)
		dfra <- t(dfra) / nIterations
	}
	else
	{
		nWaves <- dim(sumscores)[1]
		nParameters <- dim(sumscores)[2]
		dfra <- sumdfra
		tmp <- matrix(vapply(1 : nWaves,
				function(i){outer(sumdeviations[i,], sumscores[i,])},
				FUN.VALUE=matrix(0,nParameters,nParameters)),
						ncol=nWaves) / (nIter^2)
		dfra <- t(dfra) / nIter
	}
	dfra - matrix(rowSums(tmp), nrow=nParameters)
}

##@updateDerivativeSde siena07 replace dfra estimates for sde parameters by analytic approx.
updateDerivativeSde <- function(z, dfra)
{
  wienerId <- which(vapply(z$effects$effectName, 
			function(x){grepl("wiener", x, fixed = TRUE)}, FUN.VALUE=TRUE))
  feedbackId <- which(vapply(z$effects$effectName, 
			function(x){grepl("feedback from", x, fixed = TRUE)}, FUN.VALUE=TRUE))
  interceptId <- which(vapply(z$effects$effectName, 
			function(x){grepl("intercept", x, fixed = TRUE)}, FUN.VALUE=TRUE))
  scaleId <- which(z$effects$name == "sde")
  onePeriodSde <- (length(wienerId) > 0) # option set by the user
  nPeriods <- z$observations - 1
  n <- length(z$f$Data1$nodeSets$Actors)
    
  g <- ifelse(onePeriodSde, z$effects$initialValue[wienerId], 1)
  a <- z$effects$initialValue[feedbackId]
  b <- z$effects$initialValue[interceptId]
  tau <- z$effects$initialValue[scaleId]
  y <- z$f$Data1$contbehavs[[1]][[1]]

  if (onePeriodSde) {
    dfra[c(wienerId, feedbackId, interceptId), c(wienerId, feedbackId, interceptId)] <- 0
  } else {
    dfra[c(feedbackId, interceptId, scaleId), c(feedbackId, interceptId, scaleId)] <- 0
  } 
  
  for (i in 1:nPeriods) 
  {
    dSa_da <- sum(y[,i+1] * (tau[i] * exp(a*tau[i]) * y[,i] + 
                  (1 + a*tau[i]*exp(a*tau[i]) - exp(a*tau[i])) * b / (a^2)))
    dSa_db <- sum((exp(a*tau[i])-1)/a * y[,i+1])
    dSa_dg <- 0
    dSa_dt <- sum(exp(a*tau[i]) * (b + a * y[,i]) * y[,i+1])
    
    dSb_da <- sum(tau[i] * exp(a*tau[i]) * y[,i] + 
                  (1 + a*tau[i]*exp(a*tau[i]) - exp(a*tau[i])) * b / (a^2))
    dSb_db <- n * (exp(a*tau[i])-1)/a 
    dSb_dg <- 0
    dSb_dt <- sum(exp(a*tau[i]) * (b + a * y[,i]))
    
    dSgt_da <- sum(1/(2*a^3) * (-4*b^2 + a*g^2 - 4*a*b*y[,i] - 4*exp(a*tau[i])*(b+a*y[,i]) *
                   (b * (-2+a*tau[i]) + a^2*tau[i]*y[,i]) + exp(2*a*tau[i]) *
                   (-4*b^2 + 4*a^3*tau[i]*y[,i]^2 + 2*a^2*tau[i]*(g^2 + 4*b*y[,i]) - 
                   a * (g^2 + 4*b*(-b*tau[i] + y[,i])))))
    dSgt_db <- sum(1/(a^2) * (2*(-1+exp(a*tau[i]))^2 * (b+a*y[,i])))
    dSgt_dg <- n * (exp(2*a*tau[i])-1) * g / a
    dSgt_dt <- sum((-2*exp(a*tau[i])*(b + a*y[,i])^2 + 
                    exp(2*a*tau[i])*(2*b^2 + a*(g^2 + 2*y[,i]*(2*b + a*y[,i])))) / a)

    if (onePeriodSde) 
    {
      dfra[wienerId, c(wienerId, feedbackId, interceptId)] <- c(dSgt_dg, dSgt_da, dSgt_db)
      dfra[feedbackId, c(wienerId, feedbackId, interceptId)] <- c(dSa_dg, dSa_da, dSa_db)
      dfra[interceptId, c(wienerId, feedbackId, interceptId)] <- c(dSb_dg, dSb_da, dSb_db)
    }
    else
    {
      dfra[feedbackId, c(feedbackId, interceptId, scaleId[i])] <- 
        dfra[feedbackId, c(feedbackId, interceptId, scaleId[i])] + c(dSa_da, dSa_db, dSa_dt)
      dfra[interceptId, c(feedbackId, interceptId, scaleId[i])] <- 
        dfra[interceptId, c(feedbackId, interceptId, scaleId[i])] + c(dSb_da, dSb_db, dSb_dt)
      dfra[scaleId[i], c(feedbackId, interceptId, scaleId[i])] <- 
        dfra[scaleId[i], c(feedbackId, interceptId, scaleId[i])] + c(dSgt_da, dSgt_db, dSgt_dt)
    }
  }
  dfra
}

##@makeZsmall siena07 create a minimal version of z to pass between processors.
makeZsmall <- function(z)
{
	zsmall <- NULL
	zsmall$theta <- z$theta
	zsmall$Deriv <- z$Deriv
	zsmall$Phase <- z$Phase
	zsmall$nit <- z$nit
	zsmall$FinDiff.method <- z$FinDiff.method
	zsmall$int2 <- z$int2
	zsmall$cl <- z$cl
	zsmall$maxlike <- z$maxlike
	zsmall$gmm <- z$gmm
	zsmall$cconditional <- z$cconditional
	zsmall$condvar <- z$condvar
	zsmall$pp <- z$pp
	zsmall$nrunMH <- z$nrunMH
	zsmall$returnDeps <- z$returnDeps
	zsmall$returnChains <- z$returnChains
	zsmall$returnChangeContributions <- z$returnChangeContributions
	zsmall$returnDataFrame <- z$returnDataFrame
	zsmall$addChainToStore <- z$addChainToStore
	zsmall$callGrid <- z$callGrid
	zsmall$thetaMat <- z$thetaMat
	zsmall$byWave <- z$byWave
	zsmall
}

##@createSiena07Stores siena07 set up the storage areas used in phase 1 and 3
createSiena07stores <- function(z, nIterations, f)
{
	z$sf <- matrix(0, nrow = nIterations , ncol = z$pp)
	if (z$sf2.byIteration) # standard situation
	{
		z$sf2 <- array(0, dim=c(nIterations, f$observations - 1, z$pp))
		if ((!z$maxlike) && (!z$FinDiff.method))
		{
			z$ssc <- array(0, dim=c(nIterations, f$observations - 1, z$pp))
		}
	}
	else # store only sums over iterations
	{
		z$sf2s <- matrix(0, nrow=(f$observations - 1), ncol=z$pp)
		if (!z$maxlike && !z$FinDiff.method)
		{
			z$sscs <- matrix(0, nrow=(f$observations - 1), ncol=z$pp)
			z$scores <- matrix(0, nrow=nIterations, ncol=z$pp)
			z$dfras <- matrix(0, nrow=z$pp, ncol=z$pp)
		}
	}
	z$sdf <- vector("list", nIterations)
	z$sdf2 <- vector("list", nIterations)

	if (!z$sf2.byIteration){ z$ssc <- NULL}
	if (z$maxlike)
	{
		## misdat steps are separated out giving 10 types
		z$accepts <- array(0, dim=c(nIterations, z$nDependentVariables, 10))
		z$rejects <- array(0, dim=c(nIterations, z$nDependentVariables, 10))
		z$aborts <- array(0, dim=c(nIterations, z$nDependentVariables, 10))
		dimnames(z$accepts) <- list(NULL, NULL,
				c("InsDiag", "CancDiag", "Permute", "InsPerm",
				"DelPerm", "InsMiss",
				"DelMiss", "InsMisdat", "DelMisdat", "Move"))
		dimnames(z$rejects) <- list(NULL, NULL,
				c("InsDiag", "CancDiag", "Permute", "InsPerm",
				"DelPerm", "InsMiss",
				"DelMiss", "InsMisdat", "DelMisdat", "Move"))
		dimnames(z$aborts) <- list(NULL, NULL,
				c("InsDiag", "CancDiag", "Permute", "InsPerm",
				"DelPerm", "InsMiss",
				"DelMiss", "InsMisdat", "DelMisdat", "Move"))
	}
	z$npos <- rep(0, z$pp)
	if (!is.null(z$cconditional) && z$cconditional)
	{
		z$ntim <- matrix(NA, nrow=nIterations, ncol=f$observations - 1)
	}
	z$sims <- vector("list", nIterations)
	if (z$thetaFromFile)
	{
		z$thetaUsed <- matrix(NA, nrow = nIterations , ncol = z$pp)
	}
	z
}

stdError <- function(z, x, subphase,...)
{
	z$addChainToStore <- TRUE
	z$nbrNodes <- 1
	z$storedChains <- FALSE
	
	theta <- z$thav/z$thavn
	
	N <- length(z$myloglik2)

	z$iterSeq <- (N:1)
	z$muLik <- getLikelihoods(theta, z, getScores=TRUE,
		iterSequence= z$iterSeq)

	z
}

entropy <- function(x)
{
	y<- x/sum(x)
	l <- length(y)
	
	-sum(y*log(y))/log(l)
}

E <- function(t, z, x)
{
	theta <- t*x$thetaEnd + (1-t)*z$theta
	cat(paste("\n",theta,"\n"))
	z$muLik1 <- getLikelihoods(theta, z, getScores=TRUE,
	iterSequence= z$iterSeq)
	z$w1 <- exp(z$muLik1$lik - z$myloglik)
	z$bridgeEntropy <- entropy(z$w1)
	
	z
}

bridge1 <- function(z, x, subphase,...)
{
	z$addChainToStore <- TRUE
	z$nbrNodes <- 1
	z$storedChains <- FALSE
	
	z$iterSeq <- (z$nit:1)
	
	converge <- TRUE
	
	if (ifelse(is.null(x$checkTstat), FALSE, x$checkTstat))
	{
		tstat <- apply(z$sf,2,mean)/apply(z$sf,2,sd)
		if (max(abs(tstat)) > 0.2) ### don't want to apply this if an effect is fixed?
		{
			converge <- FALSE
		}
	}
	
if (converge)
{
	c <- ifelse(is.null(x$c), 0.8, x$c)
	z$min1 <- 0
	z$max1 <- 1

	if (ifelse(is.null(x$finalBridge), TRUE, !x$finalBridge))
	{

		z <- E(1,z,x)

		if (z$bridgeEntropy > c)
		{
			z$max1 <- 1
		} else {
			
			for (i in 1:6)
			{
				z$t1 <- (z$min1 + z$max1)/2
				z <- E(z$t1,z,x)
				cat("t1: ");cat(z$t1);cat("\n")
				cat("entropy: ");cat(z$bridgeEntropy);cat("\n")
				if (is.na(z$bridgeEntropy)) {break}
		
				if (z$bridgeEntropy < c)
				{
					z$max1 <- 0.5*(z$min1 + z$max1)
				} else {
					z$min1 <- 0.5*(z$max1 + z$min1)
				}
			}
	
			z$max1 <- z$t1
		}

		z$thetaNext <- z$max1*x$thetaEnd + (1-z$max1)*z$theta

		if (!is.null(z$thetaNext))
		{
			newLik <- getLikelihoods(z$thetaNext, z, getScores=TRUE,
													iterSequence = z$iterSeq)
			z$loglikNext <- newLik$lik
		}
	}

	if (!is.null(x$thetaPrevious))
	{
		newLik <- getLikelihoods(x$thetaPrevious, z, getScores=TRUE,
													iterSequence = z$iterSeq)

		z$loglikPrevious <- newLik$lik
	}
	
}

	z
}

eThermo <- function(h,H, z, x)
{
	if (!is.null(z$likelihoodsList[[h+1]]))
	{
		cat("\nAlready have likelihoods\n")
		z$muLik1 <- z$likelihoodsList[[h+1]]
	} else {
		cat("\nNeed to calculate likelihoods\n")
		theta <- (h/H)*x$thetaEnd + (1-(h/H))*z$theta
		z$muLik1 <- getLikelihoods(theta, z, getScores=TRUE,
		iterSequence= z$iterSeq)
		z$likelihoodsList[[h+1]] <- z$muLik1
	}

	z$w1 <- exp(z$muLik1$lik - z$myloglik)
	z$bridgeEntropy <- entropy(z$w1)
	
	z
}

zThermo <- function(z, x)
{
	z$r <- rep(0, length(z$theta))
	
	for (h in z$hmin:z$hmax)
	{
		cat("h: ");cat(h);cat("\n")
		z$r <- z$r + rThermo(z,x,h)
		cat("z$r: ");cat(z$r);cat("\n")
	}
	
	sum((x$thetaStart - x$thetaEnd)*z$r)/(z$H + 1)
}

rThermo <- function(z,x,h)
{
	r <- rep(0, z$pp)
	z <- E(h/z$H,z,x)
	scores <- t(z$muLik1$sc)

	for (i in 1:z$pp)
	{
		r[i] <- mean(z$w1*scores[,i])/mean(z$w1)
	}
	
	r
}

thermo <- function(z, x, subphase,...)
{
	z$addChainToStore <- TRUE
	z$nbrNodes <- 1
	z$storedChains <- FALSE

	z$iterSeq <- (z$nit:1)
	
	#### Interval bisection ####
	
	c <- ifelse(is.null(x$c), 0.8, x$c)

	z$H <- ifelse(is.null(x$H), 10, x$H)
	z$hmin <- ifelse(is.null(x$hmin), 0, x$hmin)
	z$hmaxMax <- z$H
	z$hmaxMin <- z$hmin
	
	z$likelihoodsList <- vector("list",z$H+1)

	z <- eThermo(z$hmaxMax,z$H,z,x)
	z$likelihoodsList[[z$H+1]] <- z$muLik1
	
	if (z$bridgeEntropy > c)
	{
		z$hmax <- z$H
	} else {
			
		for (i in 1:6) 
		{
			if (z$hmaxMin < z$hmaxMax)
			{
				z$t1 <- floor((z$hmaxMin + z$hmaxMax)/2)
				z <- eThermo(z$t1,z$H,z,x)
				cat("t1: ");cat(z$t1);cat("\n")
				cat("entropy: ");cat(z$bridgeEntropy);cat("\n")
				if (is.na(z$bridgeEntropy)) {break}
			
				if (z$bridgeEntropy < c)
				{
					z$hmaxMax <- ceiling(0.5*(z$hmaxMin + z$hmaxMax))
				} else {
					z$hmaxMin <- floor(0.5*(z$hmaxMax + z$hmaxMin))
				}
				cat("hmaxMin:");cat(z$hmaxMin);cat("\n")
				cat("hmaxMax:");cat(z$hmaxMax);cat("\n")
			}
		}
		
		z$hmax <- z$t1
	}
	
	cat("hmin:");cat(z$hmin);cat("\n")
	cat("hmax:");cat(z$hmax);cat("\n")
	
	z$zThermo <- zThermo(z,x)
	
	z
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: transformScript.r
# *
# * Description: This module contains the code for the meta analysis of a
# * collection of Siena fits.
# *****************************************************************************/

##@transformScript transform R scripts to the new names
transformScript <- function(theoldscript, fileName="newScript.R",
					linelength=80, initialblanks=6,	logfile="log.txt",
					keepOldAlgorithms=FALSE)
{
# This function transforms theoldscript,
# supposed to be a script utilizing RSiena,
# to utilize the new function names as of RSiena version 1.6.2.
# Written by Tom Snijders, version February 24, 2026.

# Operation of TransformScript; see structure after "proper"
# 1. For functions of which nothing changes except the name,
#    all old function names are replaced by the new names.
#    These are in list 'namePairs' of name pairs.
# 2. For all other function names in 'furtherNames',
#    if it is "sienaAlgorithmCreate":
#             this command is executed, to have the algorithm object available.
#    for all others in 'furtherNames':
#             the function name is prepended by the single character 'z',
#             and this modified function, which is defined below, is executed.
#             This creates the new function call.
#    if it is "siena07" or "sienaBayes":
#             "zsiena07" creates not only the new call of "siena",
#             and "zsienaBayes" creates not only the new call of "multi_siena",
#             but also, if necessary, calls of algorithm creation functions.
#             For this purpose, it takes the algorithm object,
#             which was created by processing 
#             a preceding call of "sienaAlgorithmCreate",
#             and compares its values to the default values;
#             if any are different, then the appropriate is constructed
#             among set_model_saom, set_algorithm_saom, and set_output_saom,
#             or set_algorithm_mcm in the case of "sienaBayes",
#             and added to the new script 
#             before the call of "siena" or "multi_siena". 
#             This is done again for each new call of "siena07" or "sienaBayes",
#             which may result in superfluous calls of algorithm constructions.
#    For these function calls (i.e., in 'furtherNames'):
#             * Everything that comes after a hash (#) sign is disregarded;
#               if the function call itself comes after a hash sign,
#               it is not processed and the line is left unchanged.
#             * If the new function call is longer than linelength characters,
#               it is broken up in multiple lines,
#               after the position of commas,
#               and each line except the first starts with initialblanks blanks.
#             * For creation of a time-constant dyadic covariate ("coDyadCovar")
#               the keyword 'oneMode' is always used;
#               the user should check whether,
#               instead, it should be 'bipartite'.
# 3. Finally, for the remaining occurrences of names in 'furtherNames'
#             which will occur in lines after a "#" or without a "<-",
#             or for "Wald.RSiena", "testSame.RSiena", "score.Test" without a "("", 
#             the old function name is replaced by the new name.
# A log file is created indicating changes made
# and their line numbers in theoldscript.


################################################################################
### Data for alias substitution    #############################################
################################################################################

namePairs <- list(c("sienaDependent", "as_dependent_rsiena"),
   c("coCovar", "as_covariate_rsiena"),
   c("varDyadCovar", "as_covariate_rsiena"),
   c("sienaCompositionChange", "as_composition_rsiena"),
   c("sienaCompositionChangeFromFile", "as_composition_file_rsiena"),
   c("sienaNodeSet", "as_nodeset_rsiena"),
   c("sienaDataCreate", "make_data_rsiena"),
   c("sienaModelCreate", "sienaAlgorithmCreate"),
   c("sienaGroupCreate", "make_group_rsiena"),
   c("getEffects", "make_specification"),
   c("sienaDataConstraint", "make_constraint"),
   c("siena.table", "write_result"),
   c("sienaTimeTest", "test_time"),
   c("sienaGOF", "test_gof"),
   c("siena08", "meta_siena"),
   c("descriptives.sienaGOF", "descriptives"),
   c("selectionTable", "interpret_selection"),
   c("influenceTable", "interpret_influence"),
   c("sienaRIDynamics", "interpret_size_dynamics"),
   c("print01Report", "write_report"),
   c("modelname", "outputName"),
   c("updateTheta", "update_theta"),
   c("updateSpecification", "update_specification"),
   c("extract.posteriorMeans", "extract_posteriorMeans"),
	c("glueBayes", "multi_glue"),
	c("extract.posteriorMeans", "extract_posteriorMeans"),
	c("simpleBayesTest", "test_parameter"),
	c("multipleBayesTest", "test_parameter"),
	c("testedPar", "tested"),
	c("tested0", "tested_null")
	)

################################################################################
### Names of functions for which the change    #################################
### involves more than name replacement:       #################################
################################################################################

furtherNames <- list("sienaAlgorithmCreate", "siena07",
					"varCovar", "coDyadCovar", "includeEffects", "setEffect",
					"includeInteraction", "sienaRI", "Multipar.RSiena",
					"Wald.RSiena", "testSame.RSiena", "score.Test", 
					"sienaBayes", "simulateData")
found <- rep(0, length(namePairs) + length(furtherNames))
names(found) <- c(sapply(namePairs, function(x){x[[1]]}), furtherNames)

# With their new names and some others:
furtherNamePairs <- list(
		c("siena07", "siena"),
		c("varCovar", "as_covariate_rsiena"),
		c("coDyadCovar", "as_covariate_rsiena"),
		c("includeEffects", "set_effect"),
		c("setEffect", "set_effect"),
		c("includeInteraction", "set_interaction"),
		c("sienaRI", "interpret_size"),
		c("Multipar.RSiena", "test_parameter"),
		c("Wald.RSiena", "test_parameter"),
		c("testSame.RSiena", "test_parameter"),
		c("score.Test", "test_parameter"),
	c("sienaBayes", "multi_siena"),
	c("simulateData", "simulate"),
    c("extract.sienaBayes", "extract_sienaBayesFit"),
# perhaps the next ones are superfluous:
	c("interaction1", "covar1"),
	c("interaction2", "covar2"),
	c("interaction3", "covar3"))
furtherfound <- rep(0, length(furtherNamePairs))
names(furtherfound) <- c(sapply(furtherNamePairs, function(x){x[[1]]}))


################################################################################
### Helper functions    ########################################################
################################################################################

Reduce2 <- function(x, i=NULL){
# searches in string x for an occurrence of namePairs[[h]][[1]]
# and replaces it by namePairs[[h]][[2]]
# reports this as line i in file logfile and in array found
	out <- x
	for (h in 1:length(namePairs))
	{
		find <- grep(namePairs[[h]][[1]], out, useBytes=TRUE)
		if  (any(find))
		{
			write(paste("line ", i,'; found and replaced ', namePairs[[h]][[1]]),
							file=logfile, append=TRUE)
			out <- sub(namePairs[[h]][[1]], namePairs[[h]][[2]], out)
			found[h] <<- found[h]+1
		}
	}
	out
}


Reduce22 <- function(x, i=NULL){
# searches in string x for an occurrence of furtherNamePairs[[h]][[1]]
# and replaces it by furtherNamePairs[[h]][[2]]
# reports this as line i in file logfile and in array found
	out <- x
	for (h in 1:length(furtherNamePairs))
	{
		find <- grep(furtherNamePairs[[h]][1], out, useBytes=TRUE)
		if  (any(find))
		{
			write(paste("line ", i,'; found and replaced ', 
			            furtherNamePairs[[h]][[1]]), file=logfile, append=TRUE)
			out <- sub(furtherNamePairs[[h]][1], furtherNamePairs[[h]][2], out)
			furtherfound[h] <<- furtherfound[h]+1
		}
	}
	out
}

Reduce3 <- function(x, sep=", "){
# pastes a list of strings to one string
	out <- NULL
	if (length(x) == 0){return(NULL)}
	for (h in which(sapply(x, function(a){!is.null(a)})))
	{
		if (is.null(out))
		{
			out <- x[[h]]
		}
		else
		{
			out <- paste(out,x[[h]], sep=sep)
		}
	}
	return(out)
}

arr <- function(x,xx){
	paste(", ", x , "=", xx, sep="")
}

adtext <- function(s,t){
	return(paste(", ", s, "='", as.character(t), "'", sep=""))
}

pastent <- function(a,b){
	if (b)
	{
		return(paste(a, "=", b, sep=""))
	}
	else
	{
		return(NULL)
	}
}

noccur <- function(x, theline)
# number of occurrences of x in theline
{
	gr <- gregexpr(x, theline, fixed=TRUE, useBytes=TRUE)[[1]]
	ngr <- ifelse(gr >= 1, length(gr), max(gr,0))
	ngr
}

balanceParentheses <- function(x)
{
	nstarts <- noccur("(", x)[[1]]
	nends <- noccur(")", x)[[1]]
	while (nends != nstarts)
	{
		if (nends > nstarts)
		{
			x <- paste("(", x, sep="")
		}
		else
		{
			x <- paste(x, ")", sep="")
		}
		nstarts <- noccur("(", x)[[1]]
		nends <- noccur(")", x)[[1]]
	}
	x
}

textBeforeHash <- function(st, dropBlanks=FALSE)
# The text in st before "#" and perhaps without blanks
 {
	position <- regexpr("#", st, useBytes=TRUE)
	if (position >0 )
	{
		stnew <- substring(st, 1, position-1)
	}
	else
	{
		stnew <- st
	}
	if (dropBlanks)
	{
		stnew <- gsub(" ", "", stnew, fixed=TRUE, useBytes=TRUE)
	}
	stnew
}

shortentext <- function(stext, linewidth=linelength,
							startingblanks=initialblanks){
# transforms string stext to vector of strings of length <= linewidth,
# with breaks occurring after commas (if any),
# prepends startingblanks blanks at the start of follow-up lines
	if (linewidth <= startingblanks)
	{
		stop("startingblanks should be less than linewidth")
	}
	blanks <- Reduce(paste, rep(" ", startingblanks))
	if (is.null(stext)) return(NULL)
	stext_remain <- stext
	shorttext <- list(stext_remain)
	commas <- gregexpr(",", stext_remain, fixed=TRUE, useBytes=TRUE)[[1]]
	if (min(commas) > 0)
	{
		h <- 1
		textlength <- nchar(stext_remain)
		comma_pos <- ifelse((any(commas <= linewidth)),  max(commas[commas<=linewidth]),
					 min(commas[commas > linewidth]))
		while ((textlength > linewidth) && (comma_pos < textlength)){
			breakpos <- ifelse(textlength <= linewidth, textlength, comma_pos)
			shorttext[[h]] <- substr(stext_remain, 1, breakpos)
			shorttext[[h+1]] <- paste(blanks,
							substr(stext_remain, breakpos+1, textlength ))
			stext_remain <- shorttext[[h+1]]
			textlength <- nchar(stext_remain)
			commas <- gregexpr(",", stext_remain, fixed=TRUE, useBytes=TRUE)[[1]]
			if (min(commas) < 0)
			{
				comma_pos <- textlength + 1
			}
			else
			{
				comma_pos <- ifelse((any(commas <= linewidth)),  max(commas[commas<=linewidth]),
					 min(commas[commas > linewidth]))
			}
			h <- h+1
		}
	}
	unlist(shorttext)
}

findTheLines <- function(i, script){
# is the call distributed over several lines?
# Then these lines are concatenated and processed together.
# This considers only text before "#".
	linei <- textBeforeHash(script[i])
	nstarts <- noccur("(", linei)[[1]]
	nends <- noccur(")", linei)[[1]]
	newline <- linei
	ii <- i
	while (nends < nstarts)
	{
		ii <- ii+1
		if (ii > length(script))
		{
			stop(paste("No matching parentheses in what follows after line ", ii, sep=""))
		}
		lineii <- textBeforeHash(script[ii])
		newline <- paste(newline, lineii, collapse="")
		nstarts <- nstarts + noccur("(", lineii)[[1]]
		nends <- nends +  noccur(")", lineii)[[1]]
	}
	list(newline, ii)
}


################################################################################
### Function to process each function defined below that starts with z #########
################################################################################


transformCommand <- function(funName, oldscript, i, logfile)
{
# funName is a character string,
# oldscript is a string vector, newscript is a list
# transformCommand produces a string;
# but if funName=="siena07" or "sienaBayes", a vector of strings
# 	require(stringr) # for str_squish; not used now
	newline <- NULL
	oldline <- textBeforeHash(oldscript[i], dropBlanks=TRUE)
	if (funName %in% c("Multipar.RSiena", "Wald.RSiena", "testSame.RSiena", "score.Test"))
	{ # These may be used without an assignment to a name
		funNameWith <- paste(funName, "(", sep="")
	}
	else
	{
		funNameWith <- paste("<-",funName, "(", sep="")
	}
	if (any(grep(funNameWith, oldline, fixed=TRUE, useBytes=TRUE)))
	{
		found[funName] <<- found[funName]+1
# Define new function name, starting with z:
		newFunName <- paste("z", funName, sep="")
	# is the call distributed over several lines?
		TheLines <- findTheLines(i, oldscript)
		newline <-  Reduce(paste, TheLines[[1]])
#		newline <-  str_squish(Reduce(paste, TheLines[[1]]))
		ii <- TheLines[[2]]
		if (ii > length(oldscript))
		{
			newline <- paste("### The folliwng command is incomplete: ", newline, sep="")
		}
		else
		{
			theparts <- strsplit(newline, "<-", fixed=TRUE)
			if (is.list(theparts))
			{
				theparts <- theparts[[1]]
			}
			if (length(theparts)==1) # No occurrence of "<-" in newline
			{ 
				assignment <- FALSE
				newcommand <- newline
			}
			else
			{
				assignment <- TRUE
				newcommand <- theparts[2]
			}
			newcommand <- sub(funName, newFunName, newcommand)
			newcommand1 <- balanceParentheses(newcommand)
			if ((funName=="siena07") || (funName=="sienaBayes"))
# zsiena07 and zsienaBayes can produce a string vector with more than one element
			{
# Suppose that siena07 or sienaBayes is used in a function with a "..." argument:
				newcommand1 <- sub(", ...", "", newcommand1, fixed=TRUE)
				newcommand1 <- sub(",...", "", newcommand1, fixed=TRUE)
				thelines <- eval(parse(text=newcommand1))
				thelines$siena_text <- balanceParentheses(paste(theparts[1], "<-",
							thelines$siena_text, collapse=""))
				thelines$siena_text <- shortentext(thelines$siena_text)
# the other components of thelines were shortened in zsiena07 and zsienaBayes
				newline <- unlist(thelines)
			}
			else
			{
				newcommand <- unlist(eval(parse(text=newcommand1)))
				if (assignment)
				{
					newline <- paste(theparts[1], "<- ", newcommand, sep="")
				}
				else
				{
					newline <- newcommand
				}
				newline1 <- balanceParentheses (newline)
				newline <- shortentext(newline1)
			}
			write(paste("\nline ", i, "to", ii, '; found and transformed', funName),
							file=logfile, append=TRUE)
			i <- ii
			if (funName=="simulateData")
			{
				write("NOTE SCRIPT For simulate, nsim=100 is arbitrary; please modify." ,
								file=logfile, append=TRUE)
				cat("NOTE: For simulate (replacing simulateData), nsim=100 is arbitrary; please modify.\n" )
			}
		}
		write("lines \r", file=logfile, append=TRUE)
	}
	list(newline=newline, i=i)
}

################################################################################
### Functions to process the algorithm given in siena07 or sienaBayes   ########
################################################################################

modelalgorithm <- function(algo){
#  model algorithm
	alg0 <- list()
	if (!is.null(algo$modelType))
	{
		textmt <- paste(names(algo$modelType),
			algo$modelType, sep = "=", collapse = ",")
		alg0[[1]] <- paste("modelType=c(", textmt, ")", sep="")
	}
	if (!is.null(algo$behModelType))
	{
		textmt <- paste(names(algo$behModelType),
			algo$behModelType, sep = "=", collapse = ",")
		alg0[[2]] <- paste("behModelType=c(", textmt, ")", sep="")
	}
	if (is.null(algo$MaxDegree))
	{
		alg0[[3]] <- NULL
	}
	else
	{
		textmd <- paste(names(algo$MaxDegree),
				algo$MaxDegree, sep = "=", collapse = ",")
		alg0[[3]] <- paste("MaxDegree=c(", textmd, ")", sep="")
	}

	if (!is.null(algo$UniversalOffset))
	{
		textoff <- paste(names(algo$UniversalOffset),
			algo$UniversalOffset, sep = "=", collapse = ",")
		alg0 <- c(alg0, paste("OffSet=c(", textoff, ")", sep=""))
	}
	mod_content <- Reduce3(alg0)
	if (is.null(mod_content))
	{
		alg_model_text <- NULL
	}
	else
	{
		alg_model_text <- paste("alg_model <- set_model_saom(",
									mod_content, ")", sep="")
		alg_model_text <- shortentext(alg_model_text)
	}
	alg_model_text
}

algalgorithm <- function(algo){
	ndtext <- function(a) # not default
	{
		a_value <- with(algo, eval(parse(text=a)))
		defalg <- sienaAlgorithmCreate(silent=TRUE)
		def_value <- with(defalg, eval(parse(text=a)))
		if ((as.character(a_value) != as.character(def_value)) &
		      (!(is.null(a_value) & (def_value == "NULL")))) # comparison with default value
		{
			if (is.character(a_value))
			{
#				a_value <- paste("'", a_value, "'", sep="")
			}
			return(paste(a,"=",a_value, sep=""))
		}
		else
		{
			return(NULL)
		}
	}
# Begin algorithm algorithm
	alg0 <- list()
  # Sets SAOM estimation procedure:
	alg0 <- c(alg0, ndtext("maxlike"))
	alg0 <- c(alg0, ndtext("gmm"))
	if (!is.na(algo$cconditional))
	{
		alg0 <- c(alg0, paste("cond=", algo$cconditional, sep=""))
	}
	alg0 <- c(alg0, ndtext("condvarno"))
	alg0 <- c(alg0, ndtext("condname"))
	alg0 <- c(alg0, ndtext("simOnly"))
	if (!is.null(algo$randomSeed))
	{
		if (algo$randomSeed != "NULL")
		{
			alg0 <- c(alg0, paste("seed=",algo$randomSeed, "", sep=""))
		}
	}
  # Specification of Robbins Monro algorithm:
	alg0 <- c(alg0, ndtext("n3"))
	alg0 <- c(alg0, ndtext("nsub"))
	alg0 <- c(alg0, ndtext("n2start"))
#	if (!is.null(algo$n2start)) #  ndtext("n2start")
#	{
#		alg0 <- c(alg0, paste("n2start=", algo$n2start), sep="")
#	}
	alg0 <- c(alg0, ndtext("firstg"))
	alg0 <- c(alg0, ndtext("reduceg"))
	alg0 <- c(alg0, ndtext("truncation"))
	alg0 <- c(alg0, ndtext("doubleAveraging"))
	alg0 <- c(alg0, ndtext("diagonalize"))
	alg0 <- c(alg0, ndtext("standardizeVar"))
	alg0 <- c(alg0, ndtext("dolby"))
	alg0 <- c(alg0, ndtext("useStdInits"))
	alg0 <- c(alg0, pastent("findiff", algo$FinDiff.method))
  # Specification of likelihood algorithm:
	alg0 <- c(alg0, ndtext("mult"))
#	if ((!is.null(algo$mult)))
#	{
#		alg0 <- c(alg0, paste("mult=", algo$mult, sep=""))
#	}
	alg0 <- c(alg0, ndtext("prML"))
	alg0 <- c(alg0, ndtext("maximumPermutationLength"))
	alg0 <- c(alg0, ndtext("minimumPermutationLength"))
	alg0 <- c(alg0, ndtext("initialPermutationLength"))
	alg0 <- c(alg0, ndtext("localML"))
#browser()
	alg0
}

################################################################################
### Functions for handling function names in furtherNames   ####################
################################################################################


zsiena07 <- function(alg, data=NULL, effects=NULL, batch=FALSE,
		verbose=FALSE, silent=FALSE,
        useCluster=FALSE, nbrNodes=2,
        thetaValues = NULL,
        returnThetas = FALSE,
        thetaBound = 50,
        targets = NULL,
        initC=TRUE, clusterString=rep("localhost", nbrNodes), tt=NULL,
        parallelTesting=FALSE, clusterIter=!alg$maxlike,
        clusterType=c("PSOCK", "FORK"), cl=NULL,
		prevAns=NULL, returnDeps=FALSE,
		returnChains=FALSE, returnDataFrame=FALSE,
		returnChangeContributions=FALSE, ...)
{
# default algorithm:
	defalg <- sienaAlgorithmCreate(silent=TRUE)
	algname <- deparse(substitute(alg))	
	available <- suppressWarnings(!inherits(try(alg, silent=TRUE),
									"try-error"))
	if (available)
	{
		available <- inherits(alg, "sienaAlgorithm")
	}
	if (!available) 
	{ # not available from a previous run of zsienaAlgorithmCreate in this script
		alg <- sienaAlgorithmCreate(silent=TRUE)
		cat("The algorithm for siena07 is not available in this script.\n")
		cat("The name given in the script is ",algname ,".\n")
		cat("If", algname, "is not a default algorithm object,\n ")
		cat("you either have to supply it in the old script, or modify the new script.\n")
		write(paste("\nAlgorithm ", algname, "of siena07 not found.", sep=" "),
						file=logfile, append=TRUE)
		commentline <- paste("### Algorithm ", algname,
							"is not available in this script. ###", sep="")
		alg_model_text <- NULL
		alg_out_text <- NULL
		alg_alg_text <- NULL
	}
	else # alg does exist
	{
# Define output algorithm
		commentline <- NULL
		alg_out_text <- NULL
		algo <- alg # The algorithm given to siena07
		alg0 <- list()
		alg0[[1]] <- pastent("lessMem", (!algo$sf2.byIteration))
		alg0[[2]] <- pastent("returnThetas", returnThetas)
		alg0[[3]] <- pastent("returnChains", returnChains)
		alg0[[4]] <- pastent("returnDataFrame", returnDataFrame)
		alg0[[5]] <- pastent("returnChangeContributions", returnChangeContributions)
		out_content <- Reduce3(alg0)
		if (is.null(out_content))
		{
			alg_out_text <- NULL
		}
		else
		{
			alg_out_text <- paste("alg_out <- set_output_saom(", out_content, ")", sep="")
			alg_out_text <- shortentext(alg_out_text)
		}
# Define model algorithm
		alg_model_text <- modelalgorithm(algo)
# Define algorithm algorithm
		alg0 <- algalgorithm(algo)
		if (!is.null(targets))
		{
			alg0 <- c(alg0, paste("targets=",
						deparse1(substitute(targets)), "", sep=""))
		}
		if (!is.null(thetaValues))
		{
			alg0 <- c(alg0, paste("thetaValues=",
						deparse1(substitute(thetaValues)), "", sep=""))
		}
		alg_content <- Reduce3(alg0)
		if (is.null(alg_content))
		{
			alg_alg_text <- NULL
		}
		else
		{
			alg_alg_text <- paste("alg_alg <- set_algorithm_saom(",
									alg_content, ")", sep="")
			alg_alg_text <- shortentext(alg_alg_text)
		}
	}
## Text of call of siena:
	coretext <- paste("siena(data=", deparse1(substitute(data)),
						", effects=", deparse1(substitute(effects)), sep="")
	text0 <- list(coretext)
	if (hasArg("thetaBound"))
	{
		text0 <- c(text0, arr("thetaBound", substitute(thetaBound)))
	}
	if (hasArg("returnDeps"))
	{
		text0 <- c(text0, arr("returnDeps", substitute(returnDeps)))
	}
	if (hasArg("batch"))
	{
		text0 <- c(text0, arr("batch", substitute(batch)))
	}
	if (hasArg("silent"))
	{
		text0 <- c(text0, arr("silent" ,substitute(silent)))
	}
	if (hasArg("verbose"))
	{
		text0 <- c(text0, arr("verbose", substitute(verbose)))
	}
	if (hasArg("initC"))
	{
		text0 <- c(text0, arr("initC", substitute(initC)))
	}
	if (useCluster & (nbrNodes >= 2))
	{
		text0 <- c(text0, arr("nbrNodes", substitute(nbrNodes)))
	}
	if (hasArg("clusterString"))
	{
		text0 <- c(text0, arr("clusterString", substitute(clusterString)))
	}
	if (hasArg("clusterType"))
	{
		text0 <- c(text0, adtext("clusterType", clusterType))
	}
	if (hasArg("cl"))
	{
		text0 <- c(text0, arr(cl, substitute()))
	}
	if (hasArg(prevAns))
	{
		text0 <- c(text0, paste(", prevAns=",
					deparse1(substitute(prevAns)), sep=""))
	}
	text0 <- Reduce3(text0, sep="")
	textc <- list()
	anyalg <- FALSE
	if (!is.null(alg_model_text))
	{
		textc <- c(textc, "control_model=alg_model")
		anyalg <- TRUE
	}
	if (!is.null(alg_alg_text))
	{
		textc <- c(textc, "control_algo=alg_alg")
		anyalg <- TRUE
	}
	if (!is.null(alg_out_text))
	{
		textc <- c(textc, "control_out=alg_out")
		anyalg <- TRUE
	}
	if (anyalg)
	{
		siena_text <- paste(text0, ", ", Reduce3(textc), ")", sep="")
	}
	else
	{
		siena_text <- paste(text0, ")", sep="")
	}
# siena_text will be shortened in transformCommand,
# because something will be prepended
	return(list(commentline=commentline, alg_out_text=alg_out_text,
			alg_model_text=alg_model_text, alg_alg_text=alg_alg_text,
			siena_text=siena_text))
}


zvarCovar<- function(val, centered=TRUE, nodeSet="Actors", warn=TRUE,
						imputationValues=NULL)
{
	text0 <- paste("as_covariate_rsiena(", deparse1(substitute(val)), sep="")
	text0 <- paste(text0, "type='monadic'", sep=", ")
	if (!centered)
	{
		text0 <- paste(text0, "centered=FALSE", sep=", ")
	}
	if (hasArg(nodeSet))
	{
		text0 <- paste(text0, ", nodeSet=",
					deparse1(substitute(nodeSet)), sep="")
	}
	if (!warn)
	{
		text0 <- paste(text0, "warn=FALSE", sep=", ")
	}
	if (hasArg(imputationValues))
	{
		text0 <- paste(text0, ", imputationValues=",
					deparse1(substitute(imputationValues)), sep="")
	}
	cov_text <- paste(text0, ")", sep="")
	cov_text
}

zcoDyadCovar<- function(val, centered=TRUE, nodeSets=c("Actors","Actors"),
					   warn=TRUE, sparse=inherits(val,"TsparseMatrix"),
					   type=c("oneMode", "bipartite"))
{
	text0 <- paste("as_covariate_rsienas(", deparse1(substitute(val)), sep="")
	text0 <- paste(text0, "type='oneMode'", sep=", ")
	if (!centered)
	{
		text0 <- paste(text0, "centered=FALSE", sep=", ")
	}
	if (hasArg(nodeSets))
	{
		text0 <- paste(text0, ", nodeSet=",
					deparse1(substitute(nodeSets)), sep="")
	}
	if (!warn)
	{
		text0 <- paste(text0, "warn=FALSE", sep=", ")
	}
	cov_text <- paste(text0, ")", sep="")
	cov_text
}

zincludeEffects <- function(myeff, ..., include=TRUE, name=myeff$name[1],
						   type="eval", interaction1="", interaction2="",
						   fix=FALSE, test=FALSE,
						   character=FALSE, verbose=TRUE)
{
	text0 <- paste("set_effect(", deparse1(substitute(myeff)), sep="")
	dots <- substitute(list(...))
	if (length(dots)==2)
	{
		dots <- as.character(dots)[2]
	}
	text0 <- toString(c(text0, dots))
	if (hasArg(type))
	{
		text0 <- paste(text0, ", type=",
					deparse1(substitute(type)), sep="")
	}
	if (hasArg(name))
	{
		text0 <- paste(text0, ", depvar=",
					deparse1(substitute(name)), sep="")
	}
	if (hasArg(interaction1))
	{
		text0 <- paste(text0, ", covar1=",
					deparse1(substitute(interaction1)), sep="")
	}
	if (hasArg(interaction2))
	{
		text0 <- paste(text0, ", covar2=",
					deparse1(substitute(interaction2)), sep="")
	}
	if (!include)
	{
		text0 <- paste(text0, "include=FALSE", sep=", ")
	}
	if (fix)
	{
		text0 <- paste(text0, "fix=TRUE", sep=", ")
	}
	if (test)
	{
		text0 <- paste(text0, "test=TRUE", sep=", ")
	}
	if (!verbose)
	{
		text0 <- paste(text0, "verbose=FALSE", sep=", ")
	}
	setef_text <- paste(text0, ")", sep="")
	setef_text
}

zsetEffect <- function(x, shortName, parameter = NULL,
    fix = FALSE, test = FALSE, random=FALSE, initialValue = 0,
    timeDummy = ",", include = TRUE,
    name = x$name[1], type = "eval", interaction1 = "",
    interaction2 = "", effect1=0, effect2=0, effect3=0,
    period=1, group=1, character=FALSE, verbose = TRUE)
{
	text0 <- paste("set_effect(", deparse1(substitute(x)), sep="")
	text0 <- paste(text0,
					deparse1(substitute(shortName)), sep=", ")
	if (hasArg(type))
	{
		text0 <- paste(text0, ", type=",
					deparse1(substitute(type)), sep="")
	}
	if (hasArg(name))
	{
		text0 <- paste(text0, ", depvar=",
					deparse1(substitute(name)), sep="")
	}
	if (hasArg(interaction1))
	{
		text0 <- paste(text0, ", covar1=",
					deparse1(substitute(interaction1)), sep="")
	}
	if (hasArg(interaction2))
	{
		text0 <- paste(text0, ", covar2=",
					deparse1(substitute(interaction2)), sep="")
	}
	if (hasArg(effect1))
	{
		text0 <- paste(text0, ", effect1=", effect1, sep="")
	}
	if (hasArg(effect2))
	{
		text0 <- paste(text0, ", effect2=", effect2, sep="")
	}
	if (hasArg(effect3))
	{
		text0 <- paste(text0, ", effect3=", effect3, sep="")
	}
	if (hasArg(period))
	{
		text0 <- paste(text0, ", period=", period, sep="")
	}
	if (hasArg(group))
	{
		text0 <- paste(text0, ", group=", group, sep="")
	}
	if (hasArg(parameter))
	{
		text0 <- paste(text0, ", parameter=", parameter, sep="")
	}
	if (hasArg(initialValue))
	{
		text0 <- paste(text0, ", initialValue=",
						deparse1(substitute(initialValue)), sep="")
	}
	if (hasArg(random))
	{
		text0 <- paste(text0, ", random=", random, sep="")
	}
	if (hasArg(timeDummy))
	{
		text0 <- paste(text0, ", timeDummy=",  deparse1(substitute(timeDummy)), sep="")
	}
	if (!include)
	{
		text0 <- paste(text0, "include=FALSE", sep=", ")
	}
	if (fix)
	{
		text0 <- paste(text0, "fix=TRUE", sep=", ")
	}
	if (test)
	{
		text0 <- paste(text0, "test=TRUE", sep=", ")
	}
	if (!verbose)
	{
		text0 <- paste(text0, "verbose=FALSE", sep=", ")
	}
	sete_text <- paste(text0, ")", sep="")
	sete_text
}

zincludeInteraction <- function(x, ..., include = TRUE, name = x$name[1],
    type = "eval", interaction1 = rep("", 3), interaction2 = rep("", 3),
    fix=FALSE, test=FALSE, random=FALSE,
    initialValue=0,
    character = FALSE, verbose = TRUE)
{
	text0 <- paste("set_interaction(", deparse1(substitute(x)), sep="")
	dots <- substitute(list(...))
	text0 <- toString(c(text0, dots))
	if (hasArg(type))
	{
		text0 <- paste(text0, ", type=",
					deparse1(substitute(type)), sep="")
	}
	if (hasArg(name))
	{
		text0 <- paste(text0, ", depvar=",
					deparse1(substitute(name)), sep="")
	}
	if (hasArg(interaction1))
	{
		text0 <- paste(text0, ", covar1=",
					deparse1(substitute(interaction1)), sep="")
	}
	if (hasArg(interaction2))
	{
		text0 <- paste(text0, ", covar2=",
					deparse1(substitute(interaction2)), sep="")
	}
	if (hasArg(initialValue))
	{
		text0 <- paste(text0, ", initialValue=",
					deparse1(substitute(initialValue)), sep="")
	}
	if (hasArg(random))
	{
		text0 <- paste(text0, ", random=", random, sep="")
	}
	if (!include)
	{
		text0 <- paste(text0, "include=FALSE", sep=", ")
	}
	if (fix)
	{
		text0 <- paste(text0, "fix=TRUE", sep=", ")
	}
	if (test)
	{
		text0 <- paste(text0, "test=TRUE", sep=", ")
	}
	if (!verbose)
	{
		text0 <- paste(text0, "verbose=FALSE", sep=", ")
	}
	seti_text <- paste(text0, ")", sep="")
	seti_text
}

zsienaRI <- function(data, ans=NULL, theta=NULL, effects=NULL,
	getChangeStats=FALSE)
{
	if (!is.null(effects))
	{
		text0 <- paste("interpret_size(",
						deparse1(substitute(effects)), sep="")
		if (hasArg(theta))
		{
			text0 <- paste(text0, ", theta=",
					deparse1(substitute(theta)), sep="")
		}
	}
	else if (hasArg(ans))
# or replace by (deparse1(substitute(ans)) != "NULL") ?
	{
		text0 <- paste("interpret_size(",
						deparse1(substitute(ans)), sep="")
	}
	else
	{
		stop("sienaRI called without effects and ans")
	}
	text0 <- paste(text0, ", data=",
					deparse1(substitute(data)), sep="")
	if (getChangeStats)
	{
		text0 <- paste(text0, "getChangeStats=TRUE", sep=", ")
	}
	setin_text <- paste(text0, ")", sep="")
	setin_text
}


zWald.RSiena <- function(A, x)
{
	paste("test_parameter(", deparse1(substitute(x)), ", tested=", 
				deparse1(substitute(A)), ")", sep="")
}


zMultipar.RSiena <- function(x, tested)
{	
	text0 <- paste("test_parameter(", deparse1(substitute(x)), ", tested=", 
				deparse1(substitute(tested)), ")", sep="")
	text0
}

zscore.Test <- function(x, tested=NULL)
{
	if (is.null(tested))
	{
	text0 <- paste("test_parameter(", deparse1(substitute(x)), ", method='score')", 
						sep="")
	}
	else
	{
	text0 <- paste("test_parameter(", deparse1(substitute(x)), ", method='score', tested=", 
				deparse1(substitute(tested)), ")", sep="")
	}
	text0
}

ztestSame.RSiena <- function(x, e1, e2)
{
	paste("test_parameter(", deparse1(substitute(x)), ", method='same', tested=", 
				deparse1(substitute(e1)), ", tested2=", 
				deparse1(substitute(e2)), ")", sep="")
}

zset_algorithm_mcmc <- function(nprewarm=50, nwarm=50, nmain=250, nrunMHBatches=20, nImproveMH=100,
    targetMHProb=0.25,
    nSampVarying=1, nSampConst=1, nSampRates=0,
    priorPrecFactor=1,
    initgainGlobal=0.1, initgainGroupwise = 0.001, delta=1e-10,
    initML=FALSE,
    incidentalBasicRates=FALSE,
	initfgain=0.2, gamma=0.05)
# This is set_algorithm_mcmc of multiSiena version 1.2.36,
# without class and version statements.
# Sets details of mcmc algorithm specification
{
	algo <- list()
	algo$nprewarm <- nprewarm
	algo$nwarm <- nwarm
	algo$nmain <- nmain
	algo$nrunMHBatches <- nrunMHBatches
	algo$nImproveMH <- nImproveMH
	algo$targetMHProb <- targetMHProb
	algo$nSampVarying <- nSampVarying
	algo$nSampConst <- nSampConst
	algo$nSampRates <- nSampRates
	algo$priorPrecFactor <- priorPrecFactor
	algo$initgainGlobal <- initgainGlobal
	algo$initgainGroupwise <- initgainGroupwise
	algo$initML <- initML
	algo$incidentalBasicRates <- incidentalBasicRates
	algo$initfgain <- initfgain
	algo$gamma <- gamma
	algo$delta <- delta
	algo
}


zsienaBayes <- function(data, effects, algo, saveFreq=100,
				initgainGlobal=NULL, initgainGroupwise =NULL, initfgain=NULL,
				gamma=NULL, initML=NULL,
				priorMeanEta=NULL, priorSigEta=NULL,
				priorMu=NULL, priorSigma=NULL, priorDf=NULL, priorKappa=0,
				priorRatesFromData=2,
				priorPrecFactor=NULL,
				frequentist=FALSE, incidentalBasicRates=NULL,
				reductionFactor=0.5,
				delta=NULL,
				nprewarm=NULL, nwarm=NULL, nmain=NULL, nrunMHBatches=NULL,
				nSampVarying=NULL, nSampConst=NULL, nSampRates=NULL,
				nImproveMH=NULL, targetMHProb=NULL,
				lengthPhase1=round(nmain/5), lengthPhase3=round(nmain/5),
				storeScores = FALSE,
				storeAll = FALSE, prevAns=NULL, usePrevOnly=TRUE,
				prevBayes = NULL, newProposalFromPrev=(prevBayes$nwarm >= 1),
				silentstart=TRUE,
				nbrNodes=1, clusterType=c("PSOCK", "FORK"),
				getDocumentation=FALSE)
{
# Construct mcmc algorithm
# In the call of zsienaBayes, I changed all entries for set_algorithm_mcmc to NULL
# alg mcmc text
	setmcmc <- zset_algorithm_mcmc()
	alg_text <- ""
	i <- 0
# Go through the arguments of sienaBayes that were transferred to set_algorithm_mcmc:
	for (nam in names(setmcmc))
	{
		if (!is.null(eval(parse(text=nam))))
		{
			i <- i+1
			if (i <= 1)
			{
			alg_text <- paste(nam, "=", eval(parse(text=nam)), sep=" ")
			}
			else
			{
			alg_text <- paste(alg_text, ", ", nam, " = ", eval(parse(text=nam)), sep="")
			}
		}
	}
	if (alg_text == "")
	{
		alg_text <- NULL
	}
	else
	{
		alg_text <- paste("alg_mcmc <- set_algorithm_mcmc(",
									alg_text, ")", sep="")
		alg_text <- shortentext(alg_text)
	}

	algname <- deparse(substitute(algo))
	if (!exists(algname)) # Is it available?
	{ # not available from a previous run of zsienaAlgorithmCreate in this script
		alg <- sienaAlgorithmCreate(silent=TRUE)
		cat("The algorithm for sienaBayes is not available in this script.\n")
		cat("The name given in the script is ",algname ,".\n")
		cat("If", algname, "is not a default algorithm object,\n ")
		cat("you either have to supply it in the old script, or modify the new script.\n")
		write(paste("\nAlgorithm ", algname, "of sienaBayes not found.", sep=" "),
						file=logfile, append=TRUE)
		commentline <- paste("### Algorithm ", algname,
							"is not available in this script. ###", sep="")
		alg_model_text <- NULL
		alg_out_text <- NULL
		alg_alg_text <- NULL
	}
	else
	{
		commentline <- NULL
# Define model algorithm
		alg_model_text <- modelalgorithm(algo)
# Define algorithm algorithm
		alg_content <- Reduce3(algalgorithm(algo))
		if (is.null(alg_content))
		{
			alg_alg_text <- NULL
		}
		else
		{
			alg_alg_text <- paste("alg_alg <- set_algorithm_saom(",
									alg_content, ")", sep="")
			alg_alg_text <- shortentext(alg_alg_text)
		}
	}
## Define text of call of multi_siena:
	coretext <- paste("multi_siena(data=", deparse1(substitute(data)),
						", effects=", deparse1(substitute(effects)), sep="")
	text0 <- list()
	if (hasArg("priorMu"))
	{
         text0 <- c(text0, arr("priorMu" ,substitute(priorMu)))
	}

	if (hasArg("priorSigma"))
	{
        text0 <- c(text0, arr("priorSigma" ,substitute(priorSigma)))
	}
	if (hasArg("priorDf"))
	{
		text0 <- c(text0, arr("priorDf" ,substitute(priorDf)))
	}
	if (hasArg("priorKappa"))
	{
		text0 <- c(text0, arr("priorKappa" ,substitute(priorKappa)))
	}
	if (hasArg("priorMeanEta"))
	{
        text0 <- c(text0, arr("priorMeanEta" ,substitute(priorMeanEta)))
	}
	if (hasArg("priorSigEta"))
	{
        text0 <- c(text0, arr("priorSigEta" ,substitute(priorSigEta)))
	}
	if (hasArg("priorRatesFromData"))
	{
		text0 <- c(text0, arr("priorRatesFromData", substitute(priorRatesFromData)))
	}
	if (hasArg("reductionFactor"))
	{
        text0 <- c(text0, arr("reductionFactor" ,substitute(reductionFactor)))
	}
	if (hasArg("saveFreq"))
	{
		text0 <- c(text0, arr("saveFreq" , substitute(saveFreq)))
	}
	if (hasArg("storeScores"))
	{
		text0 <- c(text0, arr("storeScores", substitute(storeScores)))
	}
	if (hasArg(prevAns))
	{
		text0 <- c(text0, paste(", prevAns=",
					deparse1(substitute(prevAns)), sep=""))
	}
	if (hasArg("usePrevOnly"))
	{
		text0 <- c(text0, arr("usePrevOnly", usePrevOnly))
	}
	if (hasArg("newProposalFromPrev"))
	{
		text0 <- c(text0, arr("newProposalFromPrev", substitute(newProposalFromPrev)))
	}
	if (hasArg("silentstart"))
	{
		text0 <- c(text0, arr("silentstart" , substitute(silentstart)))
	}
	if (hasArg("nbrNodes"))
	{
		text0 <- c(text0, arr("nbrNodes", substitute(nbrNodes)))
	}
	if (hasArg("clusterType"))
	{
		text0 <- c(text0, adtext("clusterType", clusterType))
	}
	text0 <- Reduce3(text0, sep="")

	textc <- list()
	anyalg <- FALSE
	if (!is.null(alg_model_text))
	{
		textc <- c(textc, "control_model=alg_model")
		anyalg <- TRUE
	}
	if (!is.null(alg_alg_text))
	{
		textc <- c(textc, "control_algo=alg_alg")
		anyalg <- TRUE
	}
	if (!is.null(alg_text))
	{
		textc <- c(textc, "control_mcmc=alg_mcmc")
		anyalg <- TRUE
	}
	if (anyalg)
	{
		text1 <- paste(coretext, Reduce3(textc), sep=", ")
		siena_text <- paste(text1, text0, ")", sep="")
	}
	else
	{
		siena_text <- paste(coretext, text0, ")", sep="")
	}
# siena_text will be shortened in transformCommand,
# because something will be prepended
	return(list(commentline=commentline, alg_text=alg_text,
			alg_model_text=alg_model_text, alg_alg_text=alg_alg_text,
			siena_text=siena_text))
}

zsimulateData <- function(g, x, xd, thin=1, nstart=x$nwarm,
                            seed=NULL, nbrNodes=1)
{
	text0 <- paste("simulate(", substitute(x), sep="")
#	nm <- sum(!is.na(x$ThinParameters[,1,1])) - x$nwarm
# but x may be unavailable
#   nsim <- nm %/% thin # integer division
	nsim <- 100  # arbitrary
	text0 <- paste(text0, ", nsim=", nsim, sep="")
	if (hasArg(seed))
	{
		text0 <- paste(text0, ", seed=", seed, sep="")
	}
	text0 <- paste(text0, ", data=",
					deparse1(substitute(xd)), sep="")
	if (hasArg(nbrNodes))
	{
		text0 <- paste(text0, ", nbrNodes=", nbrNodes, sep="")
	}
	paste(text0, ")", sep="")
}


################################################################################
### Function transformScript proper   ##########################################
################################################################################

# To enforce "silent=TRUE" in calls of sienaAlgorithmCreate:
FromTransformScript <- TRUE

cat("Processing ", deparse1(substitute(theoldscript)), ".\n")
# write beginning of log file:
write(c(paste("This is file  ", deparse1(substitute(logfile)), sep=""),
		"log file for function TransformScript ",
		paste("transforming '", deparse1(substitute(theoldscript)),
					"' to the new names of version 1.6.", sep=""),
		date(), ""),
		file = logfile)

thenewscript <- list()

write("PASS 1", append=TRUE, file=logfile)
# Replace function names by new names ('alias')
# for those functions which further are unaltered:
i <- 0
repeat{
	i <- i+1
	if (i > length(theoldscript)) break
	thenewscript[[i]] <- Reduce2(theoldscript[i], i )
}

thenewscript <- unlist(thenewscript)
# This makes it into a vector of strings, each line a string.
# This is necessary because commands might be spread over several lines,
# and findTheLines, used below, uses string vectors.


write(c("", "PASS 2"), append=TRUE, file=logfile)
# Process functions in furtherNames.

thenewerscript <- list()
i <- 0
write("lines ", file=logfile, append=TRUE)
repeat{
	i <- i+1
	if (i > length(thenewscript)) break
	write(paste(i,'\r'), file=logfile, append=TRUE)
	if ((i %% 20) == 0)
	{
		write('\n', file=logfile, append=TRUE)
	}
# "\r" suppresses the linefeed
	sumfound <- sum(found)
	theline <-  textBeforeHash(thenewscript[i])
	theShortline <-  textBeforeHash(theline, dropBlanks=TRUE)
	if (any(grep("<-sienaAlgorithmCreate(", theShortline, fixed=TRUE, useBytes=TRUE)))
	{
		found["sienaAlgorithmCreate"] <- found["sienaAlgorithmCreate"]+1
		TheLines <- findTheLines(i, thenewscript)
	# the command sienaAlgorithmCreate is executed
	# to have the algorithm object available for future calls of siena07
		eval(parse(text=TheLines[[1]]))
		if (keepOldAlgorithms)
		{
			thenewerscript <- c(thenewerscript,
			"### The following call of sienaAlgorithmCreate is redundant")
			for (j in (i:TheLines[[2]]))
			{
				thenewerscript <- c(thenewerscript, thenewscript[j])
			}
		}
		i <- TheLines[[2]]
		write(paste('\nline ', i,'; found and executed', "sienaAlgorithmCreate"),
							file=logfile, append=TRUE)
		write("lines ", file=logfile, append=TRUE)
	}
	theTransform <- transformCommand("siena07", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("sienaBayes", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("simulateData", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("varCovar", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("coDyadCovar", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("includeEffects", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("setEffect", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("includeInteraction", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("sienaRI", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("Multipar.RSiena", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline)) 
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("Wald.RSiena", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("testSame.RSiena", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	theTransform <- transformCommand("score.Test", thenewscript, i, logfile)
	i <- theTransform$i
	if (!is.null(theTransform$newline))
	{
		thenewerscript <- c(thenewerscript, theTransform$newline)
	}
	if (sumfound == sum(found))
	{
		thenewerscript <- c(thenewerscript, thenewscript[i])
	}
}


write(c("", "", "PASS 3"), append=TRUE, file=logfile)
# Replace remaining occurrences of function names in furtherNamePairs
# in lines containing no "<-" (presumably after "#")"<-",
# or for "Wald.RSiena", "testSame.RSiena", "score.Test" without a "("", 
# by the new names:

i <- 1
while((i <= length(thenewerscript))){
	thenewerscript[i] <- Reduce22(thenewerscript[i], i )
	i <- i+1
}

thenewerscript <- c("#  Script for RSiena",
		paste("#", "  transforming '", deparse1(substitute(theoldscript)),
					"' to the new names of version 1.6.", sep=""),
		paste("#  log file : ", deparse1(substitute(logfile)), sep=""),
		thenewerscript)

if (sum(found) + sum(furtherfound) > 0)
{
	write(c("", "SUMMARY", ""), append=TRUE, file=logfile)
}

if (sum(found) > 0)
{
write("Total found and processed:", file=logfile, append=TRUE)
	write(paste(names(found)[found>0],  found[found>0],
					sep = ": ", collapse = "\n"), file=logfile, append=TRUE)
}

if (sum(furtherfound) > 0)
{
	write(c("", "Total further found and replaced:"), file=logfile, append=TRUE)
	write(paste(names(furtherfound)[furtherfound>0],
				furtherfound[furtherfound>0],
					sep = ": ", collapse = "\n"), file=logfile, append=TRUE)
}

script <- unlist(thenewerscript)
write(script,  file=fileName)
if (logfile=="")
{
	logfile <- "console"
}
if (fileName=="")
{
	fileName <- "console"
}
cat("New script written to",  fileName, "; log file is",
		logfile, ". \n")
invisible(script)
}

## /***************************************************************************
##  * SIENA: Simulation Investigation for Empirical Network Analysis
##  *
##  * Web: https://www.stats.ox.ac.uk/~snijders/siena
##  *
##  * File: sienatable.r
##  *
##  * Description: This file contains the code to save a latex or html table of
##  * estimates for a sienaFit or sienaBayesFit object
##  * Written by Charlotte Greenan; modifications by Tom Snijders.
##  *
##  ***************************************************************************/


##@shortBayesResult abbreviated sienaBayesFit results
shortBayesResults <- function(x, nfirst=NULL){
	if (!inherits(x, "sienaBayesFit"))
	{
		stop('x must be a sienaBayesFit object')
	}
	if (is.null(nfirst))
	{
		nfirst <- x$nwarm+1
	}
	df1 <- sienaFitThetaTable(x, fromBayes=TRUE, nfirst=nfirst)[[1]][,
		c("text", "value", "se", "cFrom", "cTo", "postSd", "cSdFrom", "cSdTo" )]
	df1$postSd[is.na(df1$cSdFrom)] <- NA
	df1$postSd <- as.numeric(df1$postSd)
	df1$cSdFrom <- as.numeric(df1$cSdFrom)
	df1$cSdTo <- as.numeric(df1$cSdTo)
	df2 <- as.data.frame(x$requestedEffects[,c("name","shortName", "interaction1", "interaction2",
		"type", "randomEffects", "fix", "parm", "period", "effect1", "effect2", "effect3", "group")])
	df2$period <- as.numeric(df2$period)
	replace1 <- function(x){ifelse(x=="text", "effectName", x)}
	replace2 <- function(x){ifelse(x=="value", "postMeanGlobal", x)}
	replace3 <- function(x){ifelse(x=="se", "postSdGlobal", x)}
	replace4 <- function(x){ifelse(x=="postSd", "postSdBetween", x)}
	dfs <- cbind(df2, df1)
	dfr <- dfs
	names(dfr) <- replace1(replace2(replace3(replace4(names(dfs)))))
	dfr
}

write_result <- function(x, ...) UseMethod("write_result", x)

##@write_result.sienaFit method for siena.Fit
write_result.sienaFit <- function(x, type='tex',
	fileName=NULL,
	vertLine=TRUE, tstatPrint=FALSE,
	sig=FALSE, d=3, ...)
{	
	if (is.null(fileName))
	{
		fileName <- deparse(substitute(x))
# Check if it looks like a pipe chain 
		if (any((grepl("%>%|\\|>", fileName))))
		{
			stop("Cannot auto-generate filename from piped data. Please provide filename explicitly.") 
		}
	}
	fileName <- paste(fileName, '.', type, sep="")
	siena_table(x, type=type, fileName=fileName,
		vertLine=vertLine, tstatPrint=tstatPrint, sig=sig, d=d, ...)
}


##@siena_table siena07 Saves latex or html table of estimates
## for a sienaFit or sienaBayesFit object
# Also see fromObjectToText in siena07.r,which does this more drastically
siena_table <- function(x, type='tex',
	fileName=NULL,
	vertLine=TRUE, tstatPrint=FALSE,
	sig=FALSE, d=3, nfirst=NULL)
{
	## fromObjectToText transforms names from the effects object
	## to names readable in LateX or text.

    fromObjectToText <- function(a, type='tex'){
		b <- as.character(a)
		if (type == 'tex')
		{
			b <- gsub('->', '$\\rightarrow$', fixed=TRUE, b)
			b <- gsub('<-', '$\\leftarrow$', fixed=TRUE, b)
			b <- gsub('<>', '$\\leftrightarrow$', fixed=TRUE, b)
			b <- gsub('=>', '$\\Rightarrow$', fixed=TRUE, b)
			b <- gsub('>=', '$\\geq$', fixed=TRUE, b)
			b <- gsub('<=', '$\\leq$', fixed=TRUE, b)
			b <- gsub('sqrt', '$\\sqrt{}$', fixed=TRUE, b)
			b <- gsub('&', '.', fixed=TRUE, b)
			b <- gsub('#', '.', fixed=TRUE, b)
		}
		else
		{
			b <- gsub(' ', '_', fixed=TRUE, b)
			b <- gsub('.', '_', fixed=TRUE, b)
			b <- gsub('#', '_', fixed=TRUE, b)
		}
# Note: R changes \\ to \ but still displays \\ in printing the string.
		b <- gsub('^(1/1)', '', fixed=TRUE, b)
		b <- gsub('^(1/2)', '(sqrt)', fixed=TRUE, b)
		b <- gsub('^', '', fixed=TRUE, b)
		b <- gsub('_', '-', fixed=TRUE, b)
		b
	}

	objectName <- deparse(substitute(x))
	if (is.null(fileName))
	{	
		if (grepl("%>%|\\|>", objectName)) 
		{
			stop("Cannot auto-generate filename from piped data. Please provide filename explicitly.") 
		}
		fileName <- paste(objectName,'.',type,sep="")
	}
	fromBayes <- FALSE
	xkind.string <- "sienaFit"
	tstat <- tstatPrint
	if (!inherits(x, "sienaFit"))
	{
		if (inherits(x, "sienaBayesFit"))
		{
			fromBayes <- TRUE
			x$gmm <- FALSE # for compatibility
			sig <- FALSE
			tstat <- TRUE # for sienaBayesFit, the place for t-stats is used for between-groups sd.
			if (is.null(nfirst))
			{
				nfirst <- x$nwarm + 1
				message("Note: the print function for sienaBayesFit objects can also use a parameter nfirst,")
				message("    indicating the first run from which convergence is assumed.")
				message("    The default value used now is nfirst =",
					x$nwarm + 1, ".")
			}
			xkind.string <- "sienaBayesFit"
		}
		else
		{
			stop('x must be a sienaFit or sienaBayesFit object')
		}
	}
	if (!(type %in% c('tex','html')))
	{
		stop('type should be either "tex" or "html"')
	}
	if (!gmm(x))
	{
	  effects <- x$requestedEffects
	  p <- x$pp
	}
	else
	{
	  effects <- x$requestedEffects[!x$gmmEffects,]
	  p <- x$pp - sum(x$gmmEffects)
	}
	condrates <- 0
	if (fromBayes)
	{
		nwaves <- attr(x$initialResults$f,'observations')
	}
	else
	{
	nwaves <- dim(x$targets2)[2]
	}
	if (x$cconditional)
	{
		condrates <- length(x$rate)
	}

	if (fromBayes)
	{
		xx <- shortBayesResults(x, nfirst=nfirst)
		theta <- xx$postMeanGlobal
		ses <- xx$postSdGlobal
		sd.between <- xx$postSdBetween
		se.string <- '(psd)'
		sdb.string <- 'psd-between'
	}
	else
	{
	  if (!x$gmm)
	  {
	    theta <- x$theta
#	    theta[diag(x$covtheta) < 0.0 | x$fixed] <- NA
# This is misleading, theta could be nonzero for fixed
	    ses <- sqrt(diag(x$covtheta))
	    ses[x$fixed] <- NA
	    max.t1 <- max(abs(x$tstat[!x$fixed]))
	    dd <- 2
	    max.t <- round(max.t1, digits = dd)
	  }
	  else
	  {
	    fixednogmm <- x$fixed[!x$gmmEffects]
	    theta <- x$theta
#	    theta[diag(x$covtheta) < 0.0 | fixednogmm] <- NA
# This is misleading, theta could be nonzero for fixed
	    theta <- theta[!x$gmmEffects]
	    ses <- sqrt(diag(x$covtheta))
	    ses[fixednogmm] <- NA
	    max.t1 <- max(abs(x$tstat[!fixednogmm]))
	    dd <- 2
	    max.t <- round(max.t1, digits = dd)
	  }
	if (max.t < max.t1)
	{
		max.t <- max.t + 10^{-dd} #needs to be rounded up
	}
	maxlincomb.t1 <- x$tconv.max
	maxlincomb.t <- round(maxlincomb.t1, digits = dd)
	if (maxlincomb.t < maxlincomb.t1)
	{
		maxlincomb.t <- maxlincomb.t + 10^{-dd} #needs to be rounded up
	}
		se.string <- '(s.e.)'
	}
	if (length(x$condvarno) == 0)
	{
		condvarno <- 0
	}
	else
	{
		condvarno <- x$condvarno
	}

	effects$effectName <- fromObjectToText(effects$effectName, type=type)
	extraType <- effects$type
	extraType <- vapply(extraType, function(tp){switch(tp,
			'creation'='creation', 'endow'='maintenance', '')},
			FUN.VALUE='a', USE.NAMES =FALSE)
	effects$effectName <- paste(effects$effectName, extraType)
	max.eff.width <- max(nchar(effects$effectName))
	effects$effectName <- format(effects$effectName,width=max.eff.width)

	max.width <- function(theta)
	{
		max(nchar(as.character(round(abs(theta[!is.na(theta)]))))) +
			2*(min(theta[!is.na(theta)])<0)
	}

	max.ses.width <- max.width(ses)
	max.theta.width <- max.width(theta)
	max.tstat.width <- ifelse(fromBayes, max.width(sd.between), max.width(theta/ses))

	## signif converts t values into daggers and asterisks

	signif <- function(a)
	{
		s <- format("",width=17)

		if (!is.na(a))
		{
			a <- abs(a)
			signif1 <- qnorm(1-0.5*c(0.001,0.01,0.05,0.1))

			if (type=="html")
			{
				signif2 <- c("&#134","*","**","***")
			}
			else
			{
				signif2 <- c(format("$^\\dagger$",width=18),
					format("$^\\ast$",width=18),
					format("$^{\\ast\\ast}$",width=19),
					"$^{\\ast\\ast\\ast}$")
			}
			s2 <- signif2[sum(a>signif1)]

			if (length(signif2[sum(a>signif1)])>0)
			{
				s <- s2
			}
		}
		s
	}

	## mystr rounds a number and splits into its integer and fractional parts

	mystr <- function(theta,int.width=0)
	{
		if (!is.na(theta))
		{
			tc <- as.character(round(theta,d))
			tcsplit <- unlist(strsplit(tc,split=".",fixed=T))

			if (length(tcsplit)==1)
			{
				tcsplit <- c(tcsplit,paste(rep("0",d),sep="",collapse=""))
			}

			if (nchar(tcsplit[2])<d)
			{
				tcsplit[2] <- paste(tcsplit[2],paste(rep("0",d-nchar(tcsplit[2])),
						sep="",collapse=""),sep="")
			}

			if (theta < 0)
			{
				if (tcsplit[1] == "0")
				{
					if (type=='tex')
					{
						tcsplit[1] <- "--0"
					}
					else
					{
						tcsplit[1] <- "&nbsp;&ndash;0"
					}
				}
				else
				{
					if (type=='tex')
					{
						tcsplit[1] <- paste(c("-",tcsplit[1]),sep="",collapse="")
					}
					else
					{
						tcsplit[1] <- sub("-","&nbsp;&ndash;",tcsplit[1])
					}
				}
			}
		}
		else
		{
			if (fromBayes)
			{
				tcsplit <- c("","")
			}
			else
			{
			tcsplit <- c("N","A.")
		}
		}

		if (int.width>0)
		{
			tcsplit[1] <- format(tcsplit[1],width=int.width,justify="right")
		}
		tcsplit
	}

	## mydf creates a data.frame; these will be bound together to form the table

	mydf <- function(pp)
	{
		if (type == "html")
		{
		data.frame(first=rep("", pp),
			effect=rep("", pp),
			amp1 = rep("", pp),
			par1 = rep("", pp),
			amp2 = rep("", pp),
			par2 = rep("", pp),
			amp3s = rep("", pp),
			signif = rep("",pp),
			amp3lpar = rep("", pp),
			se1 = rep("", pp),
			amp4 = rep("", pp),
			se2 = rep("", pp),
			rpar = rep("", pp),
			amp5 = rep("", pp),
			tstat1 = rep("",pp),
			amp6 = rep("",pp),
			tstat2 = rep("",pp),
			ent = rep("",pp),
			stringsAsFactors =FALSE)
		}
		else
		{
		data.frame(first=rep("", pp),
			effect=rep("", pp),
			amp1 = rep("", pp),
			par1 = rep("", pp),
			amp2 = rep("", pp),
			par2 = rep("", pp),
			signif = rep("",pp),
			amp3lpar = rep("", pp),
			se1 = rep("", pp),
			amp4 = rep("", pp),
			se2 = rep("", pp),
			rpar = rep("", pp),
			amp5 = rep("", pp),
			tstat1 = rep("",pp),
			amp6 = rep("",pp),
			tstat2 = rep("",pp),
			ent = rep("",pp),
			stringsAsFactors =FALSE)
	}
	}

	## mydf2 creates a data.frame with latex symbols required
	## on each line of the main body of the table

	mydf2 <- function(pp)
	{
		if (type == "html")
		{
			if (tstat)
			{
				amp5 <- rep("</TD><TD align=\"right\">",pp)
				amp6 <- rep(".",pp)
			}
			else
			{
				amp5 <- rep("", pp)
				amp6 <- rep("", pp)
			}

			data.frame(first = rep("<TR><TD>", pp),
				effect = rep("", pp),
				amp1 = rep("</TD><TD align=\"right\">", pp),
				par1 = rep("", pp),
				amp2 = rep(".", pp),
				par2 = rep("", pp),
				amp3s = rep("</TD><TD align=\"left\">", pp),
				signif = rep("",pp),
				amp3lpar = rep("</TD><TD align=\"right\">&nbsp;&nbsp;&nbsp;(", pp),
				se1 = rep("", pp),
				amp4 = rep(".", pp),
				se2 = rep("", pp),
				rpar = rep(")", pp),
				amp5 = amp5,
				tstat1 = rep("",pp),
				amp6 = amp6,
				tstat2 = rep("",pp),
				ent = rep("</TD></TR>",pp),
				stringsAsFactors =FALSE)
		}
		else
		{
			if (tstat == TRUE)
			{
				amp <- rep(" & ",pp)
			}
			else
			{
				amp <- rep("", pp)
			}

			data.frame(first=rep("", pp),
				effect=rep("", pp),
				amp1 = rep(" & ", pp),
				par1 = rep("", pp),
				amp2 = rep(" & ", pp),
				par2 = rep("", pp),
				signif = rep("",pp),
				amp3lpar = rep(" & (", pp),
				se1 = rep("", pp),
				amp4 = rep(" & ", pp),
				se2 = rep("", pp),
				rpar = rep(")", pp),
				amp5 = amp,
				tstat1 = rep("",pp),
				amp6 = amp,
				tstat2 = rep("",pp),
				ent = rep("\\\\",pp),
				stringsAsFactors =FALSE)
		}
	}

	## tableSection creates lines of latex which can be appended to the table;
	## eg. headings, subtitles

	tableSection <- function(latex)
	{
		r <- mydf(length(latex))
		r$effect <- latex
		r
	}

	## lines of latex to include

	if (type == "html")
	{
		if (tstat)
		{
			if (fromBayes)
			{
				start.tstat <- "<TD>betw. sd.</TD>"
			}
			else
		{
			start.tstat <- "<TD>t stat.</TD>"
		}
		}
		else
		{
			start.tstat <- ""
		}

		startTable <- tableSection(c("<TABLE border=1  rules=none frame = hsides>",
				paste("<TR><TD>Effect</TD><TD>&nbsp;&nbsp;&nbsp;par.</TD><TD></TD>
					<TD>&nbsp;&nbsp;&nbsp;",se.string,"</TD>",start.tstat,"</TR>")))
					midTable <- tableSection(c("",""))
					indentTable <- tableSection("")
		ruleTable <- tableSection("<TR> <TD colspan='9'><HR/></TD> </TR>")

		footnoteStart <- c("</TABLE>","<TABLE border=1  rules=none frame = below>")
		if (fromBayes)
		{
			footnote <- c(paste("<TR> <TD colspan=9 align=left> par=posterior mean;
							psd = posterior standard deviation;
							</TD> </TR> "),
						paste("<TR> <TD colspan=9 align=left>
							betw. sd = posterior between-groups stand. deviation.
							</TD> </TR> <TR> </TR>"), "</TABLE>")
		}
		else
		{
					footnote <- c(paste(" <TR> <TD colspan=9 align=left>
							all convergence t ratios < ",
							max.t,".</TD> </TR> <TR> </TR>",
							" <TR> <TD colspan=9 align=left>
							Overall maximum convergence ratio ",
							maxlincomb.t,".</TD> </TR> <TR> </TR>",
							sep="",collapse=""),"</TABLE>")
		}
		if (sig)
					{
			footnote <- c(footnoteStart, "<TR> <TD colspan=4 align=left> &#134 p < 0.1;
							* p < 0.05; ** p < 0.01; *** p < 0.001;  </TD> </TR> <TR> </TR> " ,footnote)
					}
		else
		{
			footnote <- c(footnoteStart, footnote)
		}

	}
	else
	{
		if (vertLine)
		{
			linesep="|"
		}
		else
		{
			linesep=""
		}
		if (tstat)
		{
			start.tstat <- "r@{.}l"
			if (fromBayes)
			{
				start.tstat2 <- paste("&\\multicolumn{2}{c", linesep, "}{betw. sd}")
		}
		else
		{
				start.tstat2 <- paste("&\\multicolumn{2}{c", linesep, "}{$t$ stat.}")
		}
		}
		else
		{
			start.tstat <- ""
			start.tstat2 <- ""
		}
		if (is.null(x$startingDate))
		{
			startdate <- NULL
		}
		else
		{
			startdate <- paste("%Estimation date",x$startingDate)
		}
		startTable <- tableSection(c(paste("% Table based on", xkind.string, "object",
					objectName, ',', date()),
					startdate,
				paste("\\begin{tabular}{l",
					linesep,
					"r@{.}l r@{.}l",start.tstat,
					linesep,"}"),
				"\\hline",
				"\\rule{0pt}{2ex}\\relax",
				paste("Effect &\\multicolumn{2}{c}{par.}&\\multicolumn{2}{c",
					"}{",se.string,"}",
					start.tstat2,"\\\\[0.5ex]"),
				"\\hline"))
		midTable <- tableSection(c("\\hline",
				"\\rule{0pt}{2ex}\\relax"))
		indentTable <- tableSection("\\rule{0pt}{2ex}\\relax")
		ruleTable <- tableSection("\\hline")
		if (fromBayes)
		{
			footnote <- c(paste("\\multicolumn{7}{l}\n   ",
				"{\\footnotesize{par = posterior mean; psd = posterior standard deviation;}}\\\\\n",
				"\\multicolumn{7}{l}\n   ",
				"{\\footnotesize{betw. sd = posterior between-groups stand. deviation.}}\\\\\n",
				sep="",collapse=""),
			"\\end{tabular}")
		}
		else
		{
		footnote <- c(paste("\\multicolumn{5}{l}\n   ",
				"{\\footnotesize{convergence $t$ ratios all $<$ ", max.t,
				".}}\\\\\n",
				"\\multicolumn{5}{l}",
				"{\\footnotesize{Overall maximum convergence ratio ",
				maxlincomb.t,".}}",
				sep="",collapse=""),
			"\\end{tabular}")

		if (sig == TRUE)
		{
			footnote <- c("\\multicolumn{5}{l}{\\footnotesize{$^\\dagger$ $p$ $<$ 0.1;
				$^\\ast$ $p$ $<$ 0.05; $^{\\ast\\ast}$ $p$ $<$ 0.01;
				$^{\\ast\\ast\\ast}$ $p$ $<$ 0.001;}}\\\\" ,footnote)
		}
	}
	}

	endTable <- tableSection(footnote)

	subtitleLatex <- function(subtitle)
	{
		if (type == "html")
		{
			tableSection(paste("<TR> <TD colspan=4 align=left>",subtitle,
					"  </TD> </TR> <TR> </TR>"))
		}
		else
		{
			tableSection(c(paste("\\multicolumn{",4+2*tstat,"}{l}{\\emph{",subtitle,"}}&\\\\",sep=""),
					"\\hline"))
		}
	}

	## main body of table for each dependent variable

	mainLatex <- function(rows,sections)
	{
		nn <- length(rows)

		if (condvarno == sections)
		{
			nrate <- 0
			m <- 2
		}
		else
		{
			nrate <- sum(effects[rows,]$type == 'rate')
			m <- 0

			if (nn>nrate)
			{
				m <- 2
			}
		}

		mainTable <- rbind(mydf2(nrate),mydf(m),mydf2(nn-nrate))
		mid <- nrate+(1:2)

		if (m == 2)
		{
			mainTable[mid,] <- midTable
		}
		mainTable$effect[-mid] <- effects[rows,]$effectName
		mainTable$par1[-mid] <- sapply(theta[rows],mystr,max.theta.width)[1,]
		mainTable$par2[-mid] <- sapply(theta[rows],mystr)[2,]
		mainTable$se1[-mid] <- sapply(ses[rows],mystr,max.ses.width)[1,]
		mainTable$se2[-mid] <- sapply(ses[rows],mystr)[2,]

		basicRates <- (1:nwaves)
		if (!x$gmm)
		{
		  fixed.2 <- (1:nn)[x$fixed[rows]]
		}
		else
		{
		  fixed.2 <- (1:nn)[fixednogmm[rows]]
		}

		if (condvarno == sections)
		{
			basicRates <- nn+m+1 #so none are removed
		}

		remove <- unique(c(basicRates,fixed.2))

		if (tstat)
		{
			if (fromBayes)
			{
				remove <- is.na(sd.between)
				mainTable$tstat1[-mid][-remove] <-
					sapply(sd.between[rows],mystr,max.tstat.width)[1,][-remove]
				mainTable$tstat2[-mid][-remove] <-
					sapply(sd.between[rows],mystr)[2,][-remove]

				if (any(remove))
				{
					if (type=='tex')
					{
# This had to be dropped, gave a warning; or that is perhaps not serious?
#						mainTable$tstat1[-mid][remove] <- "\\omit"
#						mainTable$tstat2[-mid][remove] <- "-"
					}
				}
			}
			else
		{
			mainTable$tstat1[-mid][-remove] <-
				sapply(theta[rows]/ses[rows],mystr,max.tstat.width)[1,][-remove]
			mainTable$tstat2[-mid][-remove] <-
				sapply(theta[rows]/ses[rows],mystr)[2,][-remove]

			if (min(remove)<nn+m+1)
			{
				if (type=='tex')
				{
					mainTable$tstat1[-mid][remove] <- "\\omit"
					mainTable$tstat2[-mid][remove] <- "-"
				}
			}
		}

		}

		if (sig)
		{
			mainTable$signif[-mid][-remove] <- sapply(theta[rows]/ses[rows],signif)[-remove]
		}

		if (condvarno == sections)
		{
			rateTable <- mydf2(condrates)
			rateTable$effect <- paste('Rate',1:condrates)
			max.rate.width <- max.width(x$rate)
			max.vrate.width <- max.width(x$vrate)
			rateTable$par1 <- sapply(x$rate,mystr,max.rate.width)[1,]
			rateTable$par2 <- sapply(x$rate,mystr)[2,]
			rateTable$se1 <- sapply(x$vrate,mystr,max.vrate.width)[1,]
			rateTable$se2 <- sapply(x$vrate,mystr)[2,]

			if (tstat)
			{
				if (type=='tex')
				{
					rateTable$tstat1 <- "\\omit"
					rateTable$tstat2 <- "-"
				}
			}

			rbind(indentTable,rateTable,mainTable,ruleTable)
		}
		else
		{
			rbind(indentTable,mainTable,ruleTable)
		}
	}

	## Builds the table

	sections <- 0
	table <- startTable

	nBehavs <- sum(x$f$types == "behavior")
	nNetworks <- length(x$f$types) - nBehavs

	if (nBehavs > 0 && nNetworks > 0)
	{
		table <- rbind(table,subtitleLatex("Network Dynamics"))
	}

	if (nNetworks > 0)
	{
		netEffects <- effects[effects$netType != "behavior",]
		netNames <- unique(netEffects$name)
		sections <- 0
		for (i in 1:nNetworks)
		{
			sections <- sections+1
			thisNetTable <- mainLatex((1:p)[effects$name == netNames[i]],sections)
			table <- rbind(table,thisNetTable)
		}
	}

	if (nBehavs > 0 && nNetworks > 0)
	{
		table <- rbind(table,subtitleLatex("Behaviour Dynamics"))
	}

	if (nBehavs > 0)
	{
		behEffects <- effects[effects$netType == 'behavior',]
		behNames <- unique(behEffects$name)

		for (i in 1:nBehavs)
		{
			sections <- sections+1
			thisBehTable <- mainLatex((1:p)[effects$name == behNames[i]],sections)
			table <- rbind(table,thisBehTable)
		}
	}

	table <- rbind(table,endTable)

	##Saves the table to a file
	write.table(table,file=fileName,row.names=F,col.names=F,sep="", quote=FALSE)
	cat('Results for', objectName, 'written to', fileName,'.\n')
}

##@getActorStatistics. Use as RSiena:::getTheActorStatistics
getTheActorStatistics <- function(data, effects) {
    setup <- sienaSetupForCpp(data, effects,
                              includeBehavior = TRUE,
                              includeBipartite = TRUE,
                              returnActorStatistics = TRUE,
                              returnStaticChangeContributions = FALSE,
                              parallelrun = TRUE)
    ans <- .Call(C_getTargets, PACKAGE=pkgname,
                 setup$pData, setup$pModel, setup$myeffects,
                 parallelrun = TRUE,
                 returnActorStatistics = TRUE,
                 returnStaticChangeContributions = FALSE)
    ans
}

calclk<- function(pars,chain,mat1,mat2,startmat)
{
    lambda<- exp(pars[1])
    beta <- pars[2:3]
    nc<- nrow(chain)
    ##   cat(lambda,beta,nc,'\n')
    tmpnet<- startmat
    nactors<- nrow(startmat)
    pvec<- rep(0,nc)
    for (step in 1:nc)
    {
        i<- chain[step,1]
        j<- chain[step,2]
        ss<- matrix(0,nrow=nactors,ncol=2)
        if (!is.na(mat1[i,j])&!is.na(mat2[i,j]))
        {
            for (ii in 1:nactors)
            {
                tmp<- tmpnet[i,]
                if (ii!=i)
                    tmp[ii]<- 1-tmp[ii]
                ss[ii,1]<- sum(tmp)
                ss[ii,2]<- sum(tmp&tmp==tmpnet[,i])
            }
            fs<-drop(ss%*%beta)
            fs<- fs-fs[i]
            ps<- exp(fs)
            ps[is.na(mat1[i,])|is.na(mat2[i,])]<- 0
            ps<- ps/sum(ps)
            ##  ps<- ps*nactors/sum(ps==0) ##
            ##  cat(sum(ps),'\n')
            ##  ps[is.na(mat1[i,])|is.na(mat2[i,])]<- 1/nactors##
            pvec[step]<- ps[j]
        }
        else
            pvec[step]<-0# 1/nactors
        if (i!=j)
            tmpnet[i,j]<- 1-tmpnet[i,j]
    }
    ##   llik<-nactors*lambda - nc*log(lambda) +log(factorial(nc))-
    ##       sum(log(pvec))
    ##  cat(llik,kappa(nactors,nc,lambda)-sum(log(pvec)),'\n')
    llik <- kappa1(nactors,nc,lambda) - sum(log(pvec))
    llik
}
calcfn<- function(pars,Z,startmat,mat1,mat2)
{
    llik<- sapply(Z,function(x,pars,mat1,mat2,startmat)
                  calclk(pars,x$chain,mat1,mat2,startmat),pars=pars,
                  startmat=startmat,mat1=mat1,mat2=mat2)
    ## cat (pars,llik,'\n')
    ##    browser()
    mean(llik)
}
calcgrad<- function(pars,Z,startmat,varmat=FALSE)
{
    derivs<- function(pars,chain, varmat)
    {
        lambda<- pars[1]
        beta <- pars[2:3]
        nc<- nrow(chain)
        nactors<- nrow(startmat)
      ##  cat(lambda,beta,nc,'\n')
        tmpnet<- startmat
        sc<- rep(0,2)
        st<- rep(0,2)
        ss<- matrix(0,nrow=nactors,ncol=2)
        dff<- matrix(0,nrow=3,ncol=3)
                                        ##  browser()
        for (k in 1:nc)
        {
            i<- chain[k,1]
            j<- chain[k,2]
         ##   if (!is.na(mat1[i,j])&!is.na(mat2[i,j]))
            {
                myout<- tmpnet[i,]
                myin<- tmpnet[,i]
                ss[,1]<- 1-2*myout
                ss[,2]<- (1-2*myout)*myin
                ss[i,]<- 0
               fs<-drop(ss%*%beta)
                ps<- exp(fs)
            ##    ps[is.na(mat1[i,])|is.na(mat2[i,])]<- 0
                ps<- ps/sum(ps)
               ## ss<- sweep(ss,2,ss[i,])
               ## browser()
                wmean<- apply(ss*ps,2,sum)
                if (varmat)
                {
                    ssp<- ss*ps
                    w2mean<-sapply(1:nactors,function(x)outer(ssp[x,],ss[x,]))
                    dim(w2mean)<- c(2,2,nactors)
                    w2mean<- apply(w2mean,c(1,2),sum)
                    dff[-1,-1] <- dff[-1,-1] + outer(wmean,wmean) - w2mean
                }
                sc<- sc + ss[j,]-wmean
                st<- st + ss[j,]
            }
            if (i!=j)
                tmpnet[i,j]<- 1-tmpnet[i,j]
            ##  browser()
        }
        ## cdscore<- c(-lambda,sc)
        ## dsum[i,,]<- outer(obs[i,],cdscore)
        ## browser()
          dlambda<- nactors-nc/lambda
      ##  dlambda<- nactors*lambda - nc
        dff[1,1] <- -nc/lambda/lambda
     #   cat(dlambda,-sc,'\n')
        list(sc= c(dlambda,-sc), dff= -dff)
    }
    if (is.list(Z))
    {
       # browser()
        ##   pars[1]<- log(pars[1])
        grad <- lapply(Z, function(x, pars)
                       derivs(pars, x, varmat=FALSE), pars=pars)
        gradsc <- rowMeans(sapply(grad, function(x)x$sc))
        grad <- list(sc=gradsc, dff=NULL)
       # gradsc <- sapply(grad, function(x)x$sc)
       # gradmn <- rowMeans(gradsc)
       # ##   ##    browser()
       # ##   ##  cat('gradient',gradmn,'\n')
       # gradmn # - gradmn/length(Z)
    }
    else
        grad <- derivs(pars, Z, varmat)
    grad
}
kappa1 <- function(n, nc, lambda)
{
    nc <- nc + 1
    mu <- nc/ n / lambda
    sigma2 <- nc/ n / n / lambda / lambda
    logp <- -(1 - mu)^2 / 2 / sigma2 - log(sigma2) / 2 - log(2 * pi) / 2 -
        log(n * lambda)
    ##   pp <- dpois(nc, n * lambda, log=TRUE)
    ##   cat(logp, pp, '\n')
    -logp + nc * log(n)
}
kappasigmamu<- function(n, nc, lambda, add1=FALSE)
{
    if (add1)
    {
        nc <- nc + 1
    }
    mu <- nc / n / lambda
    sigma2 <- nc / n / n / lambda / lambda
    kappa1 <- -(1 - mu)^2 / 2 / sigma2 - log(sigma2) / 2
    list(kappa=kappa1, mu=mu, sigma2=sigma2)
}

doMoreUpdates <- function(z, x, n)
{
    f <- FRANstore()
    for (i in 1:n)
    {
        ans <- calcgrad(z$theta, f$Z, f$startmat, varmat=FALSE)

        fchange <- as.vector(z$gain * ans$sc %*% z$dinv)
        fchange <- ifelse(z$posj & (fchange > z$theta), z$theta * 0.5, fchange)

        z$theta <- z$theta - fchange
        z$thav <- z$thav + z$theta
        z$thavn <- z$thavn + 1
    }
    z
}

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: globals.r
## *
## * Description: This file contains the code to create and use global objects
## *
## ****************************************************************************/
##@outf Objects/File project .txt file
outf <- NULL
##@lf Objects/File suppressed or to console
lf <- NULL
##@bof Objects/File suppressed or to console
bof <- NULL
##@cf Objects/File suppressed or to console
cf <- NULL

##@Reportfun Reporting Part of global mechanism
Reportfun<- function(x, verbose = FALSE, silent=FALSE)
{
    x <- x
    beverbose <- verbose
    besilent <- silent
    noReportFile <- FALSE
    function(txt, dest, fill=FALSE, sep=" ", hdest, openfiles=FALSE,
             closefiles=FALSE, type=c("a", "w", "n"),  projname="Siena" ,
             verbose=FALSE, silent=FALSE)
    {
        if (openfiles)
        {
            type <- match.arg(type)
            beverbose <<- verbose
            besilent <<- silent
			noReportFile <<- FALSE
            if (type =='w')
            {
                x$outf <<- file(paste(projname, ".txt", sep=""), open="w")
            }
            else if (type =="a")
            {
                x$outf <<- file(paste(projname, ".txt", sep=""), open="a")
            }
            else if (type == "n")
            {
                noReportFile <<- TRUE
            }

        }
        else if (closefiles)
        {
			close(x[["outf"]])
			x$outf <<- NULL
		}
        else
        {
            if (missing(dest) && missing(hdest))
            {
                if (!besilent)
                {
                    cat(txt, fill = fill, sep = sep)
                }
            }
            else
            {
                if (missing(dest))
                {
                    if (hdest  %in% c("cf", "lf", "bof"))
                    {
                        if (beverbose)
                        {
                            cat(txt, fill=fill, sep=sep)
                        }
                    }
                    else
                    {
                        if (!noReportFile)
                        {
                            cat(txt, file = x[[hdest]], fill = fill, sep = sep)
                        }
                    }
                }
                else
                {
                    if (deparse(substitute(dest)) %in% c("cf", "lf", "bof"))
                    {
                        if (beverbose)
                        {
                            cat(txt, fill=fill, sep=sep)
                        }
                    }
                    else
                    {
                        if (is.null(x[[deparse(substitute(dest))]]))
                        {
                            if (!besilent)
                            {
                                cat(txt, fill=fill, sep=sep)
                            }
                        }
                        else
                        {
                            if (!noReportFile)
                            {
                                cat(txt, file=x[[deparse(substitute(dest))]],
                                    fill=fill, sep=sep)
                            }
                        }
                    }
                }
            }
        }
	  }
}


##@Report Globals
Report <- local({verbose <-  NULL; silent <- NULL;
                 Reportfun(list(outf=outf, lf=lf, cf=cf, bof=bof), verbose,
                           silent)})
##@UserInterrupt Siena07/GlobalFunctions Global (within siena07)
UserInterrupt <- local({A <-  FALSE;function(x){if (!missing(x))A<<-x;A}})
##@EarlyEndPhase2 siena07/GlobalFunctions
EarlyEndPhase2 <- local({A <-  FALSE;function(x){if (!missing(x))A<<-x;A}})
##@UserRestart siena07/GlobalFunctions Global (within siena07)
UserRestart <- local({A <-  FALSE;function(x){if (!missing(x))A<<-x;A}})
##@UserInterruptFlag siena07/GlobalFunctions Global (within siena07)
UserInterruptFlag <- local({A <-  FALSE;function(x){if (!missing(x))A<<-x;A}})
##@EarlyEndPhase2Flag siena07/GlobalFunctions Global (within siena07)
EarlyEndPhase2Flag <- local({A <-  FALSE;function(x){if (!missing(x))A<<-x;A}})
##@UserRestartFlag siena07/GlobalFunctions Global (within siena07)
UserRestartFlag <- local({A <-  FALSE;function(x){if (!missing(x))A<<-x;A}})
##@is.batch siena07/GlobalFunctions Global (within siena07)
is.batch <- local({A <-  FALSE;function(x){if (!missing(x))A<<-x;A}})
##@DONE siena01/GlobalFunctions Used to communicate with siena.exe and sienaScript
DONE <- local({A <-  FALSE;function(x){if (!missing(x))A<<-x;invisible(A)}})
##@FRANstore siena07/GlobalFunctions Used to pass data to other processes
FRANstore <- local({A <-  NULL;function(x){if (!missing(x)) A<<-x;A}})

##@Heading Reporting Global function
Heading<- function(level=1, dest, text, fill=FALSE)
{
	ch <- c("=", "-", " ")[level]
	if (missing(dest))
	{
        Report(c("\n", "@", level, "\n", text, "\n"), sep="", fill=fill)
        Report(rep(ch, sum(nchar(text)) + 3), sep="", fill=fill)
		Report("\n\n")
	}
	else
	{
		dest <- deparse(substitute(dest))
		Report(c("\n", "@", level, "\n", text, "\n"), hdest=dest, sep="", fill=fill)
        Report(rep(ch, sum(nchar(text))), hdest=dest, sep="", fill=fill)
		if (level < 3)
			{
			Report("\n\n", hdest = dest)
		}
		else
		{
			Report("\n", hdest = dest)
		}
	}
}

##@PrtOutMat Reporting
PrtOutMat<- function(mat, dest)
	{
	if (is.null(mat))
		{
			return()
		}
		testing <- Sys.getenv("RSIENATESTING")
		testing <- testing != ""
		if (missing(dest))
		{
			Report(format(t(mat), scientific=testing),
				sep=c(rep.int(" ", ncol(mat) - 1), "\n"))
		}
		else
		{
			Report(format(t(mat), scientific=testing),
				sep=c(rep.int(" ", ncol(mat) - 1), "\n"),
				hdest=deparse(substitute(dest)))
			Report("\n", hdest=deparse(substitute(dest)))
		}
}
##@NullChecks siena07/GlobalFunctions Resets global flags
NullChecks <- function()
{
    UserInterrupt(FALSE)
    EarlyEndPhase2(FALSE)
    UserRestart(FALSE)
    UserInterruptFlag(FALSE)
    EarlyEndPhase2Flag(FALSE)
    UserRestartFlag(FALSE)
}

##@CheckBreaks siena07/GlobalFunctions Reads global flags
CheckBreaks <- function()
{
    UserInterruptFlag(UserInterrupt())
    EarlyEndPhase2Flag(EarlyEndPhase2())
    UserRestartFlag(UserRestart())
}


##@gmm siena07/GlobalFunctions checks gmm in a list, if any
# purpose: backward compatibility
gmm <- function(x)
{
    ifelse(is.null(x$gmm), FALSE, x$gmm)
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: robmon.r
# *
# * Description: This module contains the function robmon which controls
# * the phases of the robbins-monro stochastic approximation algorithm.
# *****************************************************************************/
##args:x: model/algorithm object - intended to be read only
##     z: model fitting object
## returns updated z
##@robmon siena07 Controls MOM process
robmon <- function(z, x, useCluster, nbrNodes, initC, clusterString,
                   clusterIter, clusterType, cl, ...)
{
    z$FinDiff.method<- x$FinDiff.method
    z$n <- 0
    z$OK <-  TRUE
    z$error <- FALSE
    z$restarted <- FALSE
    z$DerivativeProblem <- FALSE
    z$ForceFinDifPhase1 <- FALSE
    z$Phase3Interrupt <- FALSE
    z$repeatsForEpsilon <- 0
    z$maxrepeatsubphase <- 4
    z$gain <- x$firstg
    z$haveDfra <- FALSE
    z$maxlike <- x$maxlike
    z$gmm <- gmm(x)	
	if (is.null(x$sf2.byIteration)) # keep compatible
	{
		z$sf2.byIteration <- TRUE
	}
	else
	{
		z$sf2.byIteration <- x$sf2.byIteration
	}
	if (z$maxlike && !is.batch())
	{
		tcltk::tkconfigure(z$tkvars$phaselabel, text="MCMC Burnin")
	}
    #######################################################
    ##do initial setup call of FRAN
    #######################################################
    if (!is.function(x$FRAN))
    {
        x$FRAN <- getFromNamespace(x$FRANname, pkgname)
    }
    z <- initializeFRAN(z, x, initC=FALSE, ...)
   ## If gmm=TRUE and no gmm effects are included, the algorithm stops and 
    ## asks the user to select the regular MoM estimation
    if (gmm(x) & sum(z$gmmEffects)==0)
    {
        stop ("\n No gmm effects are selected. Use the regular Method of Moments estimation. \n")
    }
    ## If gmm=FALSE and gmm effects are included, the algorithm stops and 
    ## asks the user to select use the GMoM estimation
    if (!gmm(x) & sum(z$gmmEffects)>0)
    {
      stop ("\n gmm effects are selected. Use the Generalized Methods of Moments estimation. \n")
    }

	if (z$maxlike && !is.batch())
	{
		tcltk::tkconfigure(z$tkvars$phaselabel, text="Phase")
	}
    ##
    ##if conditional, FRAN changes z$theta etc
    #######################################################
    if (useCluster)
    {
		## check the version has not been updated since we started R: possible
		## on linux and Mac. Not very stringent check, as environment
		## variables or path could lead to different R or different RSiena.
		packageVersion <- packageDescription(pkgname,
											 fields=c("Version", "Date"))
		if (packageVersion$Version != pkgvers$Version ||
			packageVersion$Date != pkgvers$Date)

		{
			stop(c("Incompatible RSiena versions for sub process: exit R ",
				 "and restart"))
		}
       # if (!is.null(x$FRANname) && x$FRANname != "simstats0c")
       # {
       #     stop("Multiple processors only for simstats0c at present")
       # }
		if (z$returnChains)
		{
			stop("returnChains and useCluster are incompatible")
		}
        if (!clusterIter && nbrNodes >= z$observations)
        {
            stop("Not enough observations to use the nodes")
        }
		if (!length(cl)) {
			unlink("cluster.out")
			if (clusterType == "FORK")
			{
				cl <- makeCluster(nbrNodes, type = clusterType,
					outfile = "cluster.out")
			}
			else
			{
				cl <- makeCluster(clusterString, type = clusterType,
					outfile = "cluster.out")
			}
		}
        clusterCall(cl, library, pkgname, character.only = TRUE)
		##parLapply(cl, c('f1','f2'), sink)
		z$oldRandomNumbers <- .Random.seed
		## The possibility to use snow now has been dropped
		## because RSiena requires R >= 2.15.0
		## and snow is superseded.
		## Therefore the call to clusterSetupRNG was dropped.
		#if (getRversion() < "2.14.0")
			## fake this to recreate old results
			##	if (TRUE)
		#{
		#	clusterSetupRNG(cl, seed = as.integer(runif(6,
		#						max=.Machine$integer.max)))
		#}
		#else
		#{
		clusterSetRNGStream(cl, iseed = as.integer(runif(1,
							max=.Machine$integer.max)))
		#}
        clusterCall(cl, storeinFRANstore,  FRANstore())
        if (initC)
        {
			clusterCall(cl, initializeFRAN, z, x, initC=initC)
        }
        z$cl <- cl
    }
    else
    {
        z$cl  <-  NULL
    }

	# check dimensionality of thetaValues
	if (z$thetaFromFile)
	{
		if (dim(z$thetaValues)[2] != z$pp)
		{
			stop(paste('Matrix thetaValues, if given, should have', z$pp, 'columns.'))
		}
	}

    z$newFixed <- rep(FALSE, z$pp)
    z$AllNowFixed <- FALSE
    if (!z$haveDfra)
    {
        z$dinv <- matrix(NA, nrow = z$pp, ncol = z$pp)
    }
    z$scale <- rep(0.1, z$pp)
    Report('\n', outf)
    Report('\nStochastic approximation algorithm.\n', cf)
    if (x$firstg <= 0)
    {
        Report(c('Initial value of the gain parameter is ', x$firstg,
                 '.\n'), outf)
        Report('This is not allowed; changed to 0.0001.\n', outf)
        z$gain <- 0.0001
    }
    if (x$reduceg <= 0)
    {
        Report(c('Reduction factor for the gain parameter is ', x$reduceg,
                 '.\n'), outf)
        Report('This is not allowed; changed to 0.2.\n', outf)
        x$reduceg <- 0.2
    }
    Report(c('Initial value for gain parameter = ', format(z$gain),
			 '.\nReduction factor for gain parameter = ', format(x$reduceg),
             '.\nStart of the algorithm.\n'), cf, sep='')
    Report('Observed function values are \n', cf)
	targets <- if (!z$maxlike) z$targets else z$maxlikeTargets
    ftargets <- format(targets, width = 10, nsmall = 4)
    fnum <- format(1 : z$pp, width = 3)
    Report(c(paste(fnum, '. ', ftargets, sep = '')), cf, fill=80)
    z$epsilon<- pmin(0.1,z$scale)
    z$epsilon[z$posj] <- 0.1 * z$theta[z$posj]
    z$theta0 <- z$theta
	## store starting value without any conditioning variables
    z$anyposj <- any(z$posj)
    if (!gmm(x))
    {
      z$n1 <- 7 + 3 * z$pp
    }
    else
    {
      z$n1 <- 100 + 7 * z$pp
    }
	if (x$dolby){z$n1 <- max(z$n1, 50)}
    if (any(!z$fixed))
    {
        z$AllUserFixed<- FALSE
    }
    else
    {
        z$AllUserFixed <- TRUE
    }

    repeat  ##this is startagain:
    {
      z$epsilonProblem <- FALSE
      repeat ## this loop is simply to break out of! only intend to do it once
        {
            if (any(!z$fixed))
                z$AllNowFixed<- FALSE
            else
                z$AllNowFixed <- TRUE
            phase3Only <- FALSE
            if (!is.batch())
            {
                tcltk::tkdelete(z$tkvars$subphase, 0, "end")
                tcltk::tkconfigure(z$tkvars$earlyEndPhase2, state="disabled")
                tcltk::tkconfigure(z$tkvars$subphase, state="disabled")
                tcltk::tkconfigure(z$tkvars$subphaselabel, state="disabled")
            }
            if (z$AllNowFixed || x$nsub == 0)
            {
                if (z$AllNowFixed)
                {
                    Report('All parameters are fixed.\n', outf)
                    Report(c('All parameters fixed; no estimation;',
                             'only Phase 3.\n'), lf)
                }
                else
                {
                    Report('Number of subphases is specified as 0.\n', outf)
                    Report('0 subphases; no estimation: only phase 3.\n', lf)
                }
                Report(c('Therefore the estimation phase is skipped\n',
                         'and the program passes on immediately to phase 3\n',
                         'for checking the current parameter values and',
                         'calculating standard errors.\n'),
                       outf)
                phase3Only <- TRUE
            }
            NullChecks() ## reset user interrupt variables and flags
            z$Phase <- 0
            z <- AnnouncePhase(z, x)
            WriteOutTheta(z)
            z$fixed[z$newFixed] <- FALSE
            z$newFixed <- rep(FALSE,z$pp) ##not clear what we should do here
            z$restart <- FALSE
            z$OK <- TRUE
            if (!phase3Only)
            {
                if (!z$haveDfra)
                {
                    ##start phase1 and do 10 (approx) iterations,
                    z <- phase1.1(z, x, ...)
                    ##check epsilon
                    if (!z$OK || UserInterruptFlag() || UserRestartFlag())
                    {
                        break
                    }
                    if (z$epsilonProblem && z$repeatsForEpsilon<4)
                    {
                        z$repeatsForEpsilon<- z$repeatsForEpsilon+1
                        z$restart<- !z$restarted
                        break
                    }
                    z<- phase1.2(z, x, ...)
                    if (!z$OK || z$DerivativeProblem ||
                        UserInterruptFlag() || UserRestartFlag())
                    {
                        break
                    }
                }
                if (x$nsub > 0)
                {
                    z <- phase2.1(z, x, ...)
                }
                if (!z$OK || UserInterruptFlag() || UserRestartFlag())
                {
                    if (!is.batch())
                    {
                        tcltk::tkdelete(z$tkvars$subphase,0,'end')
                        tcltk::tkconfigure(z$tkvars$earlyEndPhase2,state='disabled')
                        tcltk::tkconfigure(z$tkvars$subphase,state='disabled')
                        tcltk::tkconfigure(z$tkvars$subphaselabel,state='disabled')
                    }
                    break
                }
                if (x$nsub>1 && !EarlyEndPhase2Flag())
                {
                    z <- proc2subphase(z, x, 2, ...)
                }
                if (!z$OK || z$restart||
                    UserInterruptFlag() || UserRestartFlag() )
                {
                    if (!is.batch())
                    {
                        tcltk::tkdelete(z$tkvars$subphase,0,'end')
                        tcltk::tkconfigure(z$tkvars$earlyEndPhase2, state='disabled')
                        tcltk::tkconfigure(z$tkvars$subphase, state='disabled')
                        tcltk::tkconfigure(z$tkvars$subphaselabel, state='disabled')
                    }
                    break
                }
                if (x$nsub > 2 && !EarlyEndPhase2Flag())
                {
                    for (i in 3 : x$nsub)
                    {
                        z <- proc2subphase(z, x, i, ...)
                        if (!z$OK || UserInterruptFlag() ||
                            UserRestartFlag() || EarlyEndPhase2Flag())
                        {
                            if (!is.batch())
                            {
                                tcltk::tkdelete(z$tkvars$subphase, 0, 'end')
                                tcltk::tkconfigure(z$tkvars$earlyEndPhase2,
                                            state='disabled')
                                tcltk::tkconfigure(z$tkvars$subphase, state='disabled')
                                tcltk::tkconfigure(z$tkvars$subphaselabel,
                                            state='disabled')
                            }
                            break
                        }
                    }
                    if (!z$OK || UserInterruptFlag() ||
                        UserRestartFlag() )
                    {
                        if (!is.batch())
                        {
                            tcltk::tkdelete(z$tkvars$subphase, 0, 'end')
                            tcltk::tkconfigure(z$tkvars$subphase, state='disabled')
                            tcltk::tkconfigure(z$tkvars$subphaselabel,
                                        state='disabled')
                            tcltk::tkconfigure(z$tkvars$earlyEndPhase2,
                                        state='disabled')
                        }
                        break
                    }
                }
            }
            if (!is.batch())
            {
                tcltk::tkdelete(z$tkvars$subphase, 0, 'end')
                tcltk::tkconfigure(z$tkvars$subphase, state='disabled')
                tcltk::tkconfigure(z$tkvars$subphaselabel, state='disabled')
                tcltk::tkconfigure(z$tkvars$earlyEndPhase2, state='disabled')
            }
            z<- phase3(z, x, ...)
            break
        }
        ##stop if not OK or user has asked to
        if (!z$OK || UserInterruptFlag())
        {
            break
        }
        ##break unless we want to start at phase 1 again.
        if (!z$DerivativeProblem && !z$restart && !UserRestartFlag())
        {
            break
        }
    }
    if (!z$OK || UserInterruptFlag())
    {
        if (z$OK)
        {
            if (!z$Phase3Interrupt)
                z$termination <- 'UserInterrupt'
            else
                z$termination <- 'OK'
        }
        else
        {
            z$termination <- 'Error'
        }
        if (!z$OK || !z$Phase3Interrupt)
		{
			if (useCluster)
			{
				stopCluster(cl)
			}
			useCluster <- FALSE
            return(z)
		}
    }
    ## #####################################################
    ## do final call of FRAN
    ## #####################################################
    z <- terminateFRAN(z, x)
    ## #####################################################
    ## call to FRAN changes covariance matrix for conditional estimation
	if (x$simOnly)
	{
		if (z$thetaFromFile)
		{
			z$theta <- colMeans(z$thetaUsed)
			z$covtheta <- cov(z$thetaUsed)
			z$se <- sqrt(diag(z$covtheta))
			z$thetaValues <- NULL # not needed any longer, superseded by z$thetaUsed
		}
		else if (!gmm(x))
		{
			z$covtheta <- matrix(0, z$pp, z$pp )
			z$se <- rep(0, z$pp)
		}
		else if (gmm(x))
	  	{
	    	z$covtheta <- matrix(0, z$pp - sum(z$gmmEffects), z$pp- sum(z$gmmEffects))
	    	z$se <- rep(0, z$pp- sum(z$gmmEffects))
	  	}
	}
	else if (!gmm(x))
	{
		z$diver<- (z$fixed | z$diver | diag(z$covtheta) < 1e-9) & (!z$AllUserFixed)
		z$covtheta[z$diver, ] <- NA # was Root(diag(z$covtheta)) * 33
		##not sure this does not use very small vals
		z$covtheta[, z$diver] <- NA # was Root(diag(z$covtheta)) * 33
		diag(z$covtheta)[z$diver] <- NA # was 999
		z$se <- sqrt(diag(z$covtheta))
	}
#	z$gmm <- FALSE
    z$termination <- 'OK'
    if (useCluster)
    {
		stopCluster(cl)
	}
    z
}

##/*****************************************************************************
## * SIENA: Simulation Investigation for Empirical Network Analysis
## *
## * Web: https://www.stats.ox.ac.uk/~snijders/siena
## *
## * File: siena07.r
## *
## * Description: This file contains the main controlling module for the
## * estimation.
## * Also contains utility functions used within siena07
## ****************************************************************************/

##@siena estimation
siena <- function(data = NULL, effects = NULL,
	control_model = NULL, control_algo = NULL, control_out = NULL,
	thetaBound = 50,
	returnDeps = FALSE,
	batch = FALSE, verbose = FALSE, silent = TRUE,
	initC = TRUE,
	nbrNodes = 1,
	clusterString = rep("localhost", nbrNodes),
	clusterType = c("PSOCK", "FORK"), cl = NULL, ...)
{
	exitfn <- function()
	{
		 if (!is.batch())
		{
		  tcltk::tkdestroy(tkvars$tt)
		}
		## close the report file
		Report(closefiles=TRUE)
		RNGkind("default")
	}
	on.exit(exitfn())

	checking <- any(grepl("_R_CHECK", names(Sys.getenv())))

	dataName <- deparse(substitute(data))
	if (dataName=="") # impossible?
	{
		dataName <- "Siena"
	}

	if (!inherits(effects, "sienaEffects"))
	{
		stop("effects should be an RSiena effects object")
	}
	if (!inherits(data, "sienadata") && !inherits(data, "siena"))
	{
		stop("data should be a siena data structure")
	}
# "siena" is added here, because in pre-1.6.0 versions
# this was the class name that now is defined as "sienadata".


	if (is.null(control_model))
	{
		control_model <- set_model_saom()
	}
	if (is.null(control_algo))
	{
		control_algo <- set_algorithm_saom()
	}
	if (is.null(control_out))
	{
		control_out <- set_output_saom()
	}
	if (is.null(control_out$outputName))
	{
		if (checking)
		{
			control_out$outputName <- tempfile("Siena")
#			cat('siena will create/use an output file',
#				paste('Siena','.txt',sep=''),'.\n')
#			cat('This is a temporary file for this R session.\n')
		}
		else
		{
			if (grepl("%>%|\\|>", dataName))
			{
				stop("For piped data, you need to give an outputName in the output control object.")
			}				
			control_out$outputName <- paste(dataName, "_out", sep="")
			cat('siena will create/use an output file',
				paste(control_out$outputName,'.txt',sep=''),'.\n')
		}
		control_out$projname <- control_out$outputName
	}
	if (control_algo$maxlike)
	{
		if (!is.null(control_model$MaxDegree))
		{
			if (control_model$MaxDegree > 0)
			{
				stop("maxlike and MaxDegree are incompatible")
			}
		}
		if (control_algo$FinDiff.method)
		{
			stop("Finite differences estimation of derivatives",
				"is not possible with maximum likelihood estimation")
		}
	}

	# If the user is passing clusters through -cl- then change the
	# useCluster to TRUE, and assign the -nbrNodes- to number of nodes
	useCluster <- ((nbrNodes >= 2) | (!is.null(cl)))
	if (!useCluster & length(cl))
	{
		useCluster <- TRUE
		nbrNodes   <- length(cl)
	}

	time0 <-  proc.time()['elapsed']
	z <- list() ## z is the object for all control information which may change.
	z$newNames <- TRUE
	z$thetaBound <- thetaBound

	if (control_algo$maxlike)
	{
		if (!is.null(control_model$MaxDegree))
		{
			stop("maxlike and MaxDegree are incompatible")
		}
		if (control_algo$FinDiff.method)
		{
			stop("Finite differences estimation of derivatives",
				"is not possible with maximum likelihood estimation")
		}
	}
	x <- c(control_model, control_algo, control_out)
# Note that, if this call of siena comes from sienaBayes,
# there will be components control_algo$fromBayes
# and control_algo$ddfra
	z$returnThetas <- control_out$returnThetas

	parallelTesting <- FALSE
	clusterIter <- !control_algo$maxlike
			# clusterIter = clustering for parallelization goes by iteration
	if (useCluster)
	{
		if (!length(cl))
		{
			clusterType <- match.arg(clusterType)
			if (.Platform$OS.type == "windows" && clusterType != "PSOCK")
			{
				stop("cannot use forking processes on Windows")
			}
		}
		if (clusterIter)
		{
			x$firstg <- control_algo$firstg * sqrt(nbrNodes)
			z$int <- nbrNodes
			z$int2 <- 1
		}
		else
		{
			z$int <- 1
			z$int2 <- nbrNodes
		}
	}
	else
	{
		z$int <- 1
		z$int2 <- 1
	}
	if (!is.null(control_algo$randomSeed))
	{
		set.seed(control_algo$randomSeed, kind="default")
		seed <- control_algo$randomSeed
	}
	else
	{
		newseed <- NULL
		seed <- NULL
	}

	x$targets <- control_algo$targets
	z$x <- x
# x is designed to be readonly. Only z is returned.

	## set the global is.batch
	batchUse <- batch
	if (!batch)
	{
		if (!requireNamespace("tcltk", quietly = TRUE))
		{
				batchUse <- TRUE
				message("Package tcltk not available, forcing use of batch mode")
		}
		else
		{
			if (.Platform$OS.type != "windows")
			{
				if (!capabilities("X11"))
				{
					batchUse <- TRUE
					message("No X11 device available, forcing use of batch mode")
				}
			}
		}
		if(nzchar(Sys.getenv("RSIENA_TESTING")))
		{
			silent <- TRUE
		}
	}
	is.batch(batchUse)
	## open the output file
	Report(openfiles=TRUE, projname=control_out$projname, verbose=verbose, silent=silent)
	z <- InitReports(z, seed, newseed)

	## reset the globals for interrupts
	NullChecks()

	## create the screen
	tt <- NULL # this is set from a keyword in siena07; here the default, NULL
	if (!is.batch())
	{
		tkvars <- siena07Gui(tt=tt)
		z$tkvars <- tkvars
		z$pb <- list(pb=tkvars$pb, pbval=0, pbmax=1)
	}
	else
	{
		z$pb <- list(pb=NULL, pbval=0, pbmax=1)
	}

	## create theta values for phase 3, if necessary
	if (is.null(control_algo$thetaValues))
	{
		z$thetaValues <- NULL
		z$thetaFromFile <- FALSE
	}
	else
	{
		z$thetaValues <- control_algo$thetaValues
		z$thetaFromFile <- TRUE
	}				   	
	z <- robmon(z, x, useCluster, nbrNodes, initC, clusterString,
		clusterIter, clusterType, cl, data=data, effects=effects,
		returnDeps=returnDeps, 
		returnChains=control_out$returnChains,
		returnDataFrame=control_out$returnDataFrame,
		returnChangeContributions=control_out$returnChangeContributions, ...)
	time1 <-  proc.time()['elapsed']
	Report(c("Total computation time", round(time1 - time0, digits=2),
			"seconds.\n"), outf)
	z$compTime <- round(time1 - time0, digits=2)

	if (useCluster)
	{
		# Only stop cluster if it wasn't provided by the user
		#	  if (!length(cl))
		#		  stopCluster(z$cl)

		## need to reset the random number type to the normal one
		assign(".Random.seed", z$oldRandomNumbers, pos=1)
	}

	class(z) <- "sienaFit"
	attr(z, "version") <- packageDescription(pkgname, fields = "Version")
	z$tkvars <- NULL
	z$pb <- NULL
	z
}


##@siena07 siena07
siena07 <- function(alg, batch = FALSE, verbose = FALSE, silent = FALSE,
	useCluster = FALSE, nbrNodes = 2,
	thetaValues = NULL,
    returnThetas = FALSE,
	thetaBound = 50,
	targets = NULL,
	initC = TRUE,
	clusterString = rep("localhost", nbrNodes), tt = NULL,
	parallelTesting = FALSE, clusterIter = !alg$maxlike,
	clusterType = c("PSOCK", "FORK"), cl = NULL, ...)
{
	exitfn <- function()
	{
		if (!is.batch())
		{
		  tcltk::tkdestroy(tkvars$tt)
		}
		## close the report file
		Report(closefiles=TRUE)
		RNGkind("default")
	}
	on.exit(exitfn())

	# If the user is passing clusters through -cl- then change the
	# useCluster to TRUE, and assign the -nbrNodes- to number of nodes
	if (!useCluster & length(cl))
	{
		useCluster <- TRUE
		nbrNodes   <- length(cl)
	}

	time0 <-  proc.time()['elapsed']
	x <- alg
	z <- NULL ## z is the object for all control information which may change.
	## x is designed to be readonly. Only z is returned.
	z$x <- x
	z$returnThetas <- returnThetas
	z$thetaBound <- thetaBound
	if (useCluster)
	{
		if (parallelTesting)
		{
			stop("cannot parallel test with multiple processes")
		}
		if (!length(cl))
		{
			clusterType <- match.arg(clusterType)
			if (.Platform$OS.type == "windows" && clusterType != "PSOCK")
			{
				stop("cannot use forking processes on Windows")
			}
		}
		# The possibility to use snow now has been dropped
		# because RSiena requires R >= 2.15.0
		# and snow is superseded.
		# if (getRversion() < "2.14.0")
		## fake this to recreate old results
		## if (TRUE)
		#{
		#	require(snow, warn.conflicts=FALSE)
		#	require(rlecuyer)
		#	clusterType <- "SOCK"
		#}
		#else
		#{
		## require(parallel)
		#}
		if (clusterIter)
		{
			x$firstg <- x$firstg * sqrt(nbrNodes)
			z$int <- nbrNodes
			z$int2 <- 1
		}
		else
		{
			z$int <- 1
			z$int2 <- nbrNodes
		}
	}
	else
	{
		z$int <- 1
		z$int2 <- 1
	}
	if (parallelTesting)
	{
		set.seed(1, kind='Wich')
		## randomseed2 is for second generator needed only for parallel testing
		randomseed2 <- .Random.seed
		## .Random.seed[2:4] <- as.integer(c(1,2,3))
		## randomseed2[2:4] <- as.integer(c(3,2,1))
		randomseed2[2:4] <- as.integer(c(1, 2, 3))
		seed <- 1
		newseed <- 1
		z$parallelTesting <- TRUE
	}
	else
	{
		randomseed2 <- NULL
		## x$randomSeed is the user seed, if any
		if (!is.null(x$randomSeed))
		{
			set.seed(x$randomSeed, kind="default")
			seed <- x$randomSeed
		}
		else
		{
			if (!nzchar(Sys.getenv("RSIENA_TESTING")))
			{
				if (exists(".Random.seed"))
				{
					rm(.Random.seed, pos=1)
				}
				newseed <- trunc(runif(1) * 1000000)
				set.seed(newseed)  ## get R to create a random number seed for me.
				seed <- NULL
			}
			else
			{
				newseed <- NULL
				seed <- NULL
			}
		}
	}
	z$randomseed2 <- randomseed2

	x$targets <- targets

	## set the global is.batch
	batchUse <- batch
	if (!batch)
	{
		if (!requireNamespace("tcltk", quietly = TRUE))
		{
				batchUse <- TRUE
				message("Package tcltk not available, forcing use of batch mode")
		}
		else
		{
			if (.Platform$OS.type != "windows")
			{
				if (!capabilities("X11"))
				{
					batchUse <- TRUE
					message("No X11 device available, forcing use of batch mode")
				}
			}
		}
		if(nzchar(Sys.getenv("RSIENA_TESTING")))
		{
			silent <- TRUE
		}
	}
	is.batch(batchUse)

	## open the output file
	Report(openfiles=TRUE, projname=x$projname, verbose=verbose, silent=silent)
	z <- InitReports(z, seed, newseed)

	## reset the globals for interrupts
	NullChecks()

	## create the screen
	if (!is.batch())
	{
		tkvars <- siena07Gui(tt=tt)
		z$tkvars <- tkvars
		z$pb <- list(pb=tkvars$pb, pbval=0, pbmax=1)
	}
	else
	{
		z$pb <- list(pb=NULL, pbval=0, pbmax=1)
	}

	## create theta values for phase 3, if necessary
	if (is.null(thetaValues))
	{
		z$thetaValues <- NULL
		z$thetaFromFile <- FALSE
	}
	else
	{
		if (!x$simOnly)
		{
			cat('The thetaValues parameter was given\n')
			cat('but simOnly was not specified in the algorithm object.\n')
			cat('This is inconsistent.\n')
			stop('Inconsistent combination simOnly - thetaValues')
		}
		if (!is.matrix(thetaValues))
		{
			stop('thetaValues should be a matrix.')
		}
		z$thetaValues <- thetaValues
		z$thetaFromFile <- TRUE
	}

	z <- robmon(z, x, useCluster, nbrNodes, initC, clusterString,
		clusterIter, clusterType, cl, ...)

	time1 <-  proc.time()['elapsed']
	Report(c("Total computation time", round(time1 - time0, digits=2),
			"seconds.\n"), outf)
	z$compTime <- round(time1 - time0, digits=2)

	if (useCluster)
	{
		# Only stop cluster if it wasn't provided by the user
		#	  if (!length(cl))
		#		  stopCluster(z$cl)

		## need to reset the random number type to the normal one
		assign(".Random.seed", z$oldRandomNumbers, pos=1)
	}

	class(z) <- "sienaFit"
	attr(z, "version") <- packageDescription(pkgname, fields = "Version")
	z$tkvars <- NULL
	z$pb <- NULL
	z
}

##@InitReports siena07 Print report
InitReports <- function(z, seed, newseed)
{
	Report("\n\n-----------------------------------\n", outf)
	Report("New Analysis started.\n", outf)
	Report(c("Date and time:", format(Sys.time(),"%d/%m/%Y %H:%M:%S")), outf)
	Report("\nNew results follow.\n", outf)
	Report("-----------------------------------\n", outf)
	version <- packageDescription(pkgname, fields = "Version")
	Report(c(paste("\n", pkgname, " version ", sep = ""), version, " (",
		format(as.Date(packageDescription(pkgname, fields = "Date")), "%d %b %y"),
		")", "\n\n"), sep = "", outf)
	if (z$x$simOnly)
	{
		Heading(1, outf, "Simulations.")
	}
	else
	{
		Heading(1, outf, "Estimation by stochastic approximation algorithm.")
	}
	if (is.null(seed))
	{
		Report("Random initialization of random number stream.\n", outf)
		if (!is.null(newseed))
		{
			Report(sprintf("Current random number seed is %d.\n", newseed), outf)
		}
	}
	else
	{
		Report(sprintf("Current random number seed is %d.\n", seed), outf)
	}
	z$version <- version
	z$startingDate <- date()
	z
}

##@AnnouncePhase siena07 Progress reporting
AnnouncePhase <- function(z, x, subphase=NULL)
{
	## require(tcltk)
	if (!is.batch())
	{
		tcltk::tkdelete(z$tkvars$phase, 0, "end")
		tcltk::tkinsert(z$tkvars$phase, 0, paste(" ", z$Phase))
		tcltk::tkdelete(z$tkvars$subphase, 0, "end")
		tcltk::tkdelete(z$tkvars$iteration, 0, "end")
		tcltk::tkinsert(z$tkvars$iteration, 0, format(0, width=6))
	}
	if (missing(subphase))
	{
		if (is.batch())
		{
			##Report(c("\nStart phase", z$Phase, "\n"), cf)
			Report(c("\nStart phase", z$Phase, "\n"))
		}
	}
	else
	{
		if (!is.batch())
		{
			tcltk::tkinsert(z$tkvars$subphase, 0, paste(" ", subphase))
		}
		else
		{
			##Report(c("\nStart phase ", z$Phase, ".", subphase, "\n"),
			##sep="", cf)
			Report(c("\nStart phase ", z$Phase, ".", subphase, "\n"), sep="")
		}
	}
	if (z$Phase == 0)
	{
		if (!is.batch())
		{
			tcltk::tkconfigure(z$tkvars$current, height=min(z$pp, 30))
			tcltk::tkconfigure(z$tkvars$deviation, height=min(z$pp, 30))
			tcltk::tkconfigure(z$tkvars$quasi, height=min(z$pp, 30))
		}
		n1pos <- z$n1 * (z$pp + 1)
		z$n2min0 <- 7 + z$pp
		z$n2min0 <- max(5, z$n2min0 / z$int)
		z$n2minimum<- rep(0, x$nsub)
		z$n2maximum<- rep(0, x$nsub)
		## 2.5198421 = 2^(4/3); this gives a gain parameter of order n^(-3/4) ##
		if (x$nsub > 0)
		{
			z$n2minimum[1] <-
				ifelse(is.null(x$n2start), trunc(z$n2min0 * 2.52), x$n2start)
			z$n2maximum[1] <- z$n2minimum[1] + 200
			if (x$nsub > 1)
			{
				for (i in 2:x$nsub)
				{
					z$n2minimum[i] <- trunc(z$n2minimum[i-1] * 2.52)
					z$n2maximum[i] <- z$n2minimum[i] + 200
				}
			}
		}
		z$n2partsum <- c(0, cumsum(z$n2maximum))
		n2sum <- sum(z$n2maximum)
		##Progress bar
		pbmax <- n1pos + n2sum + x$n3
		z$n1pos<- n1pos
		if (!x$maxlike && z$FinDiff.method)
			pbmax <- pbmax + x$n3 * z$pp
		z$pb$pbval <- 0
		z$pb <- createProgressBar(z$pb, maxvalue=pbmax)
		z$pb$pbmax <- pbmax
	}
	if (z$Phase==2)
	{
		propo <- z$n1pos + z$n2partsum[max(subphase,1)]
		if (propo> getProgressBar(z$pb))
			z$pb <-setProgressBar(z$pb,propo)
	}
	if (z$Phase ==3)
	{
		propo <- z$n1pos + z$n2partsum[x$nsub + 1]
		if (!z$AllUserFixed)
			z$pb <- setProgressBar(z$pb,propo)
		else
		{
			max <- x$n3
			z$pb <-createProgressBar(z$pb,max)
		}
	}
	z
}
##@roundfreq siena07 Prettify interval between progress reports
roundfreq <- function(w)
{
	vec1 <- c(1, 2, 3, 4, 31, 66, 101, 300, 500)
	vec2 <- c(1, 2, 3, 20, 50, 100, 200, 500)
	if (is.batch())
		w <- max(10,vec2[findInterval(w, vec1, all.inside=TRUE)])
	else
		w <- vec2[findInterval(w, vec1[1:7], all.inside=TRUE)]
	w
}

##@WriteOutTheta siena07 Progress reporting
WriteOutTheta <- function(z)
{
	if (!is.batch())
	{
		DisplayTheta(z)
	}
	else
	{
		Report(c("theta:", format(z$theta, digits=3), "\n"))
	}
	Report("Current parameter values:\n", cf)
	Report(format(z$theta), cf, fill=80)
}

##@DisplayThetaAutocor siena07 Progress reporting
DisplayThetaAutocor <- function(z)
{
	if (!is.batch())
	{
		DisplayTheta(z)
		tcltk::tkdelete(z$tkvars$quasi, "1.0", "end")
		tcltk::tkinsert(z$tkvars$quasi, "1.0", FormatString(z$pp, z$ac))
	}
	else
	{
		Report(c("theta", format(z$theta, digits=3),"\n"))
		Report(c("ac", format(z$ac, digits=3), "\n"))
	}

}
##@DisplayandWriteTheta siena07 Progress reporting
DisplayandWritetheta <- function(z)
{
	if (!is.batch())
	{
		DisplayTheta(z)
	}
	else
	{
		Report(c("theta", format(z$theta, digits=3), "\n"))
	}
}
##@DisplayTheta siena07 Progress reporting
DisplayTheta <- function(z)
{
	if (!is.batch())
	{
		tcltk::tkdelete(z$tkvars$current, "1.0", "end")
		tcltk::tkinsert(z$tkvars$current, "1.0", FormatString(z$pp, z$theta))
	}

}
##@FormatString siena07 Progress Reporting
FormatString <- function(pp, value)
{
	ppuse <- min(30, pp)
	nbrs <- format(1:ppuse)
	nch <- nchar(nbrs[1])
#	formatstr <- paste("%", nch, "d.%", (13 - nch), ".4f\n", sep="",
#		collapse="")
#	paste(sprintf(formatstr, 1:ppuse, value[1:ppuse]), collapse="")
# In version 1.4.13 changed to:
	formatstr1 <- paste("%", nch, "d.%", (13 - nch), ".4f\n", sep="",
		collapse="")
	formatstr2 <- paste("%", nch, "d.%", (13 - nch), ".4g\n", sep="",
		collapse="")
	formatstr <- ifelse(abs(value[1:ppuse]) >= 1e5, formatstr2, formatstr1)
	formatstr[is.na(formatstr)] <- formatstr1 
	paste(sprintf(formatstr, 1:ppuse, value[1:ppuse]), collapse="")	
}
##@DisplayDeviations siena07 Progress reporting
DisplayDeviations <- function(z, fra)
{
	if (!is.batch())
	{
		tcltk::tkdelete(z$tkvars$deviations, "1.0", "end")
		tcltk::tkinsert(z$tkvars$deviations, "1.0", FormatString(z$pp, fra))
	}
}
##@DisplayIteration siena07 Progress reporting
DisplayIteration <- function(z)
{
	if (!is.batch())
	{
		tcltk::tkdelete(z$tkvars$iteration, 0, "end")
		tcltk::tkinsert(z$tkvars$iteration, 0, format(z$nit, width=6))
		tcltk::tcl("update")
	}
}
##@Root siena07 Safe square root for compatibility with siena3. Probably not necessary in R.
Root<- function(x)
{
	ifelse(abs(x) > 1e-36, sqrt(abs(x)), 1e-18)
}

##@getProgressBar siena07 Progress reporting
getProgressBar <- function(pb)
{
	if (is.batch())
		val <- pb$pbval
	else
		val <- as.numeric(tcltk::tclvalue(tcltk::tkcget(pb$pb, "-value")))
	val
}

##@setProgressBarProgress siena07 reporting
setProgressBar <- function(pb, val)
{
	if (is.batch())
	{
		pb$pbval <- val
	}
	else
	{
		tcltk::tkconfigure(pb$pb, value=val)
		tcltk::tcl("update")
	}
	pb
}
##@createProgressBar siena07 Progress reporting
createProgressBar <- function(pb, maxvalue)
{
	if (is.batch())
		pb$pbmax <- maxvalue
	else
		tcltk::tkconfigure(pb$pb, maximum=maxvalue)
	pb
}
##@tkErrorMessage Miscellaneous Not used
tkErrorMessage <- function()
{
	tcltk::tkmessageBox(geterrmessage(), icon="error")
}

##@errorHandler Miscellaneous Not used
errorHandler <- function()
{
	## opts <- options()
	if (!is.batch())
	{
		options(show.error.messages=FALSE)
		options(error=tkErrorMessage)
	}
}

##@coef.sienaFit method for sienaFit
coef.sienaFit <- function(object, dropRates=TRUE, shortenNames=TRUE, ...)
{
	result <- object$theta
	if (shortenNames)
	{
		names(result) <- fromObjectToText(object$requestedEffects$effectName)
	}
	else
	{
		names(result) <- object$requestedEffects$effectName
	}	
	if (dropRates)
	{
		return(result[!object$requestedEffects$basicRate])
	}
	else
	{
		return(result)
	}
}

##@vcov.sienaFit method for sienaFit
vcov.sienaFit <- function(object, dropRates=TRUE, shortenNames=TRUE, ...)
{
	result <- object$covtheta
	if (shortenNames)
	{
		colnames(result) <- fromObjectToText(object$requestedEffects$effectName)
	}
	else
	{
		colnames(result) <- object$requestedEffects$effectName
	}	
	rownames(result) <- colnames(result)
	if (dropRates)
	{
		return(result[!object$requestedEffects$basicRate,!object$requestedEffects$basicRate])
	}
	else
	{
		return(result)
	}
}

fromObjectToText <- function(a, type='notex'){
# This is an extended version of the function in sienatable.r
	b <- as.character(a)
	if (type == 'tex')
	{
		b <- gsub('->', '$\\rightarrow$', fixed=TRUE, b)
		b <- gsub('<-', '$\\leftarrow$', fixed=TRUE, b)
		b <- gsub('<>', '$\\leftrightarrow$', fixed=TRUE, b)
		b <- gsub('=>', '$\\Rightarrow$', fixed=TRUE, b)
		b <- gsub('>=', '$\\geq$', fixed=TRUE, b)
		b <- gsub('<=', '$\\leq$', fixed=TRUE, b)
		b <- gsub('sqrt', '$\\sqrt{}$', fixed=TRUE, b)
		b <- gsub('&', '.', fixed=TRUE, b)
	}
# Note: R changes \\ to \ but still displays \\ in printing the string.
	b <- gsub(" (density)", "", fixed=TRUE, b)
	b <- gsub("ocity", "", fixed=TRUE, b)
	b <- gsub("itive", "", fixed=TRUE, b)
	b <- gsub("iated", "", fixed=TRUE, b)
	b <- gsub("basic ", "", fixed=TRUE, b)
	b <- gsub("rate constant ", "", fixed=TRUE, b)
	b <- gsub("(period ", "(per_", fixed=TRUE, b)
	b <- gsub("degree", "deg", fixed=TRUE, b)
	b <- gsub("arity", "", fixed=TRUE, b)
	b <- gsub("ativity", "", fixed=TRUE, b)
	b <- gsub("ivity", "", fixed=TRUE, b)
	b <- gsub("erence", "", fixed=TRUE, b)
	b <- gsub("tot.", "tot", fixed=TRUE, b)
	b <- gsub("av.", "av", fixed=TRUE, b)
	b <- gsub("aver.", "av", fixed=TRUE, b)
	b <- gsub(" at ", " ", fixed=TRUE, b)
	b <- gsub(" of ", " ", fixed=TRUE, b)
	b <- gsub(" by ", " ", fixed=TRUE, b)
	b <- gsub("compression", "compr", fixed=TRUE, b)
	b <- gsub("triplets", "tripl", fixed=TRUE, b)
	b <- gsub("weighted", "wght", fixed=TRUE, b)
	b <- gsub("weight", "wght", fixed=TRUE, b)
	b <- gsub("maximum", "max", fixed=TRUE, b)
	b <- gsub("minimum", "min", fixed=TRUE, b)
	b <- gsub(" -> ", ">", fixed=TRUE, b)
	b <- gsub(" <- ", "<", fixed=TRUE, b)
	b <- gsub(" <> ", "=", fixed=TRUE, b)
	b <- gsub(" - ", "-", fixed=TRUE, b)
	b <- gsub(" * ", "*", fixed=TRUE, b)
	b <- gsub(" x ", "*", fixed=TRUE, b)
	b <- gsub("ilarity", "", fixed=TRUE, b)
	b <- gsub('^(1/1)', '(1)', fixed=TRUE, b)
	b <- gsub('^(1/2)', '(sqrt)', fixed=TRUE, b)
	b <- gsub(' (', '(', fixed=TRUE, b)
	b <- gsub('^', '', fixed=TRUE, b)
	b <- gsub('_', '-', fixed=TRUE, b)
	b <- gsub('#', '.', fixed=TRUE, b)	
	if (type != 'tex')
	{
		b <- gsub(' ', '_', fixed=TRUE, b)
		b <- gsub('.', '_', fixed=TRUE, b)
		b <- gsub('__', '_', fixed=TRUE, b)
		b <- gsub('_(', '(', fixed=TRUE, b)
	}
	b
}

# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: printDatareport.r
# *
# * Description: This module contains the function to produce the data
# * report from siena07
# *
# *****************************************************************************/
##@DataReport siena07 Print report
DataReport <- function(z, x, f)
{
	if (!is.null(z$effectsName))
	{
		Report(c("Effects object used:", z$effectsName, "\n"), outf)
	}

	if (z$maxlike)
	{
		Report(c(z$nrunMH,
				"MCMC steps per RM step (multiplication factor =",
				x$mult), outf)
		Report(")\n", outf)
	}
	## f could be a group, but has attributes like a group even if not!
	oneMode <- attr(f, "types") == "oneMode"
	bipartite <- attr(f, "types") == "bipartite"
	behavior <- attr(f, "types") == "behavior"
	nOneMode <- sum(oneMode)
	nBehavior <- sum(behavior)
	nBipartites <- sum(bipartite)
	oneModeNames <- attr(f, "netnames")[oneMode]
	behaviorNames <- attr(f, "netnames")[behavior]
	##  behaviorNames <- attr(f, "netnames")[behavior]
	##  bipartiteNames <- attr(f, "netnames")[bipartite]
	nDepVars <- nOneMode + nBehavior + nBipartites
	observations <- attr(f, "observations") ##note this is total number of
	##  periods to process
	exogenous <- attr(f, 'compositionChange')
	exoOptions <- attr(f, 'exooptions')
	if (nOneMode > 0)
	{
		Report("Model Type:\n", outf)
		for (i in 1:nOneMode)
		{
			if (nOneMode > 1)
			{
				Report(sprintf("Network %d %s :", i, oneModeNames[i]), outf)
			}
			Report(sprintf(" %s \n",ModelTypeStrings(z$modelType[i])), outf)
		}
		if (any(z$modelType != x$modelType))
		{
			Report(c("Note that the model type requested has been",
					"over-ridden\n"), outf)
		}
	}
	if (nBehavior > 0)
	{
		Report("Behavioral Model Type:\n",outf)
		for (i in 1:nBehavior)
		{
			if (nBehavior > 1)
			{
				Report(sprintf("Behavior %d %s :", i, behaviorNames[i]), outf)
			}
			Report(sprintf(" %s\n",BehaviorModelTypeStrings(z$behModelType[i])), outf)
		}
	}
	if (x$cconditional  != z$cconditional)
	{
		Report("\nNB. Request for conditional estimation has been over-ridden.\n\n",
			outf)
	}
	Report("Estimation method: ", outf)
	if (z$cconditional)
	{
		Report("conditional moment estimation", outf)
		if (gmm(x))
		{
			Report(' by the Generalized Method of Moments.\n', outf)
		}
		else
		{
			Report('.\n', outf)
		}
		Report(c('Conditioning variable is the total number of observed',
				'changes ("distance") \n'), outf)
		if (z$condtype == 'oneMode')
		{
			if (nOneMode == 1)
			{
				Report('in the network variable.\n', outf)
			}
			else
			{
				Report(c("in network variable", x$condname, '\n'), outf)
			}
		}
		else
		{
			Report(c("in behavioral variable", x$condname, '\n'), outf)
		}
		if (gmm(x))
		{
			Report('in the network variable.\n', outf)
		}
		if (observations == 1)
		{
			## note can only be one group
			Report(c("Distance for simulations is",
					format(f[[1]]$distances, width=4), '.\n'), outf)
		}
		else
		{
			Report("Distances for simulations are\n", outf)
			#            Report(c("period   :", format(1:observations, width=4),
			Report(c("period   :", sapply(f[1], function(x)
						sapply(x$depvars, function(y)
							format(1:observations, width=4))),
					'\n'), sep='', outf)
			Report(c("distance : ", format(sapply(f[1],
							function(x)x$distances),
						width = 4), '.\n'),sep='',  outf)
		}
	}
	else if (!z$maxlike)
	{
		Report("unconditional moment estimation\n", outf)
		if (gmm(x))
		{
			Report(' by the Generalized Method of Moments.\n', outf)
		}
		else
		{
			Report('.\n', outf)
		}

		#if (exogenous)
		#{
		#    Report("Changing composition: no conditional moment estimation.\n",
		#           outf)
		#    ## TODO report separately for different node sets if appropriate
		#    Report(c("Joiners/leavers option:", exoOptions, "\n"), sep="",
		#           outf)
		#}
		Report('\nTime duration for simulations ', outf)
		if (observations >= 2)
		{
			Report('in each period ', outf)
		}
		Report('is 1.0.\n', outf)
	}
	else
	{
		Report("Maximum likelihood estimation\n", outf)
	}
	if (exogenous)
	{
		Report("Changing composition.\n",  outf)
		## TODO report separately for different node sets if appropriate
		Report(c("Joiners/leavers option: ", exoOptions, "\n"), sep="",
			outf)
	}
	if (z$FinDiff.method)
	{
		if ((x$nsub == 0)&(x$simOnly))
		{
			Report(c('Derivatives are estimated with the finite difference',
					'method.\n'), outf)
		}
		else
		{
			Report(c('Standard errors are estimated with the finite difference',
					'method.\n'), outf)
		}
		if (!x$FinDiff.method)
		{
			Report("Note that the option requested has been over-ridden\n",
				outf)
		}
	}
	else
	{
		if ((x$nsub == 0)&(x$simOnly))
		{
			Report(c('Derivatives are estimated with the likelihood ratio',
					'method.\n'), outf)
		}
		else
		{
			Report(c('Standard errors are estimated with the likelihood ratio',
					'method.\n'), outf)
		}
	}
	if (x$dolby)
	{
		Report('Dolby method (regression on scores) is used.\n', outf)
	}
	## any max degree restrictions?
	if (any(x$MaxDegree > 0))
	{
		for (i in 1:length(x$MaxDegree))
		{
			if (x$MaxDegree[i])
			{
				mdnet <- names(x$MaxDegree)[i]
				Report(c("Dependent network variable", mdnet, ':\n'), outf)
				maxod <- max(
					attr(f$Data1$depvars[[match(mdnet, attr(f, "netnames"))]],
						"maxObsOutDegree"))
				if (maxod > x$MaxDegree[i])
				{
					Report(c("The algorithm object requires outdegrees not",
							"larger than", x$MaxDegree[i], '\n',
							"but the maximum observed outdegree is", maxod,
							".\n"), outf)
					Report("This is incompatible.\n", outf)
					cat('Some observed outdegrees higher than MaxDegree.\n')
					stop("Incompatibility between data and MaxDegree in algorithm object.")
				}
				if (attr(f, 'symmetric')[match(mdnet, attr(f, "netnames"))])
				{
					Report(c("All graphs are constrained to having degrees not",
							"larger than", x$MaxDegree[i], '.\n'), outf)
				}
				else
				{
					Report(c("All digraphs are constrained to having",
							"outdegrees not larger than", x$MaxDegree[i],
							'.\n'), outf)
				}

			}
		}
	}
	## any universal offsets?
	if (any(x$UniversalOffset != 0))
	{
		Report("Offset for universal setting or symmetric type-3 model \n", outf)
		for (i in 1:length(x$UniversalOffset))
		{
			if (x$UniversalOffset[i] != 0)
			{
				mdnet <- names(x$UniversalOffset)[i]
				Report(c("Dependent network variable", mdnet, ': '), outf)
				Report(c(sprintf("%9.4f",x$UniversalOffset[i]),'\n'), outf)
			}
		}
		Report("\n", outf)
	}
	if ((x$nsub == 0)&(x$simOnly))
	{
		Report('Parameter values are \n', outf)
	}
	else
	{
		Report(sprintf("Initial value of gain parameter is %10.7f.\n",
				x$firstg), outf)
		Report(sprintf("Reduction factor for gain parameter is %10.7f.\n",
				x$reduceg), outf)
		Report(sprintf("Number of subphases in Phase 2 is %d.\n\n", x$nsub),
			outf)
		Report('Initial parameter values are \n', outf)
	}
	if (z$cconditional)
	{
		if (observations == 1)
		{
			if (nDepVars == 1)
			{
				Report(format('  0. Rate parameter', width = 43), outf)
			}
			else
			{
				Report(format('  0. Rate parameter of conditioning variable',
						width = 43), outf)
			}
			Report(sprintf("%9.4f\n", attr(f, "condEffects")$initialValue[1]),
				outf)
		}
		else ## observations > 1
		{
			for (i in 1:observations)
			{
				if (nDepVars == 1)
				{
					Report(format(paste('  0.', format(format(i), width=2),
								' Rate parameter', sep=''),
							width = 43), outf)
				}
				else
				{
					Report(format(paste('  0.', format(format(i), width=2),
								' Rate parameter cond.',
								'variable period', i, sep=''),
							width = 43), outf)
				}
				Report(sprintf("%9.4f\n",
						attr(f, "condEffects")$initialValue[i]), outf)
			}
		}
	}

	if (!gmm(x))
	{
	  fixed <- ifelse(z$fixed, '  (fixed) ', '')
	}
	else
	{
	  if (!z$cconditional)
	  {
	    fixed <- ifelse((z$fixed & !z$gmmEffects), '  (fixed) ', '')
	  }
	  else
	  {
	    fixed <- ifelse((z$fixed & !(z$requestedEffects$type=="gmm")), '  (fixed) ', '')
	  }
	}
	if (!gmm(x))
	{	tmp <- paste(sprintf("%3d",1:length(z$requestedEffects$effectName)), '. ',
	               format(paste(z$requestedEffects$type, ':  ',
	                            z$requestedEffects$effectName,
	                            sep = ''), width = 52),
	               sprintf("%9.4f", z$requestedEffects$initialValue), fixed, '\n',
	               sep = '', collapse = '')
	}
	else
	{
	  tmp <- paste(sprintf("%3d",1:(length(z$requestedEffects$effectName)-sum(z$gmmEffects))), '. ',
	               format(paste(z$requestedEffects$type[!z$gmmEffects], ':  ',
	                            z$requestedEffects$effectName[!z$gmmEffects],
	                            sep = ''), width = 52),
	               sprintf("%9.4f", z$requestedEffects$initialValue[!z$gmmEffects]), fixed[!z$gmmEffects], '\n',
	               sep = '', collapse = '')
	}
	Report(tmp, outf)
	## targets:
	Report("\n\nValues of target statistics are\n", outf)
	if (z$maxlike)
	{
		targets <- z$maxlikeTargets
	}
	else
	{
		targets <- z$targets
	}
	tmp <- paste(sprintf("%3d",1:length(z$requestedEffects$effectName)), '. ',
		format(z$requestedEffects$functionName, width = 66),
		sprintf("%9.4f",
			ifelse(z$requestedEffects$type=='endow', -targets,
				targets)),
		'\n', sep = '', collapse = '')
	Report(tmp, outf)
	if (attr(z$targets, "fromData"))
	{
		Report("These were calculated from the data.\n", outf)
	}
	else
	{
		Report("These were given by the user in the call of siena07.\n", outf)
	}
	Report(c('\n', nrow(z$requestedEffects)-sum(z$requestedEffects$type=="gmm"), 'parameters,',
			nrow(z$requestedEffects),
			'statistics\n'),outf)
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena/
# *
# * File: sienaRI.r
# *
# * Description: Used to determine, print, and plots relative importances of effects
# * for potential decisions of actors at observation moments.
# *****************************************************************************/

##@interpret_size
sienaRI <- function(data, ans=NULL, theta=NULL, effects=NULL,
	getChangeStats=FALSE)
{
	if (!inherits(data, "sienadata") && !inherits(data, "siena"))
	{
		stop("not a legitimate Siena data specification")
	}
	datatypes <- sapply(data$depvars, function(x){attr(x,"type")})
	if (any(datatypes == "bipartite"))
	{
#		stop("interpret_size works only for dependent variables of type 'oneMode' or 'behavior'")
	}
	if(!is.null(ans))
	{
		if (!inherits(ans, "sienaFit"))
		{
			stop(paste("ans is not a legitimate Siena fit object", sep=""))
		}
		if (!is.null(theta)||!is.null(effects))
		{
			warning(paste("some informations are multiply defined \n",
					"results will be based on 'theta' and 'effects'\n",
					"stored in 'ans' (as 'ans$theta','ans$effects')\n", sep=""))
		}
		if (sum(ans$effects$include==TRUE &
				(ans$effects$type =="endow"|ans$effects$type =="creation")) > 0)
		{
			stop("interpret_size does not yet work for models containing endowment or creation effects")
		}
		if (any(ans$effects$shortName %in% c("unspInt", "behUnspInt", "contUnspInt"))){
			stop("interpret_size does not work for models containing interaction effects")
		}
		contributions <- getChangeContributions(data = data,
			effects = ans$effects)
		# contributions[[1]] is periods by effects by actors by actors
		RI <- expectedRelativeImportance(conts = contributions,
			effects = ans$effects, theta =ans$theta, thedata=data,
			getChangeStatistics=getChangeStats)
	}
	else
	{
		if (!inherits(effects, "sienaEffects"))
		{
			stop(paste("effects is not a legitimate Siena effects object", sep=""))
		}
		if(sum(effects$include==TRUE &
				(effects$type =="endow"|effects$type =="creation")) > 0)
		{
stop("interpret_size does not yet work for models containing endowment or creation effects")
		}
		if (any(effects$include==TRUE &
				(effects$shortName %in% c("unspInt", "behUnspInt", "contUnspInt"))))
		{
			stop("interpret_size does not work for models containing interaction effects")
		}
		effs <- effects
		if (!is.numeric(theta))
		{
			stop("theta is not a legitimate parameter vector")
		}
		if(length(theta) != sum(effs$include==TRUE & effs$type!="rate"))
		{
			if(length(theta) != sum(effs$include==TRUE))
			{
				stop("theta is not a legitimate parameter vector \n number of
					parameters has to match number of effects")
			}
			warning(paste("length of theta does not match the number",
					" of objective function effects\n", 
					"theta is treated as if containing rate parameters"))
			paras <- theta
			## all necessary information available
			contributions <- getChangeContributions(data = data, effects = effs)
			RI <- expectedRelativeImportance(conts = contributions,
				effects = effs, theta = paras, thedata=data,
				getChangeStatistics=getChangeStats)
		}else{
			paras <- theta
			## all necessary information available
			contributions <- getChangeContributions(data = data, effects = effs)
			RI <- expectedRelativeImportance(conts = contributions,
				effects = effs, theta = paras, thedata=data,
				getChangeStatistics=getChangeStats)
		}
	}
	RI
}



interpret_size <- function(x, ...) UseMethod("interpret_size", x)

##@interpret_size  method for sienaEffects objects
interpret_size.sienaEffects <- function(x, data,
		theta, getChangeStats=FALSE, ...)
{
    sienaRI(data, theta=theta, effects=x,
					getChangeStats=getChangeStats)
}

##@interpret_size  method for sienaFit objects
interpret_size.sienaFit <- function(x, data, getChangeStats=FALSE, ...)
{
    sienaRI(data=data, ans=x, getChangeStats=getChangeStats)
}

##@getChangeContributions. Use as RSiena:::getChangeContributions
getChangeContributions <- function(data, effects)
{
	# How to check for settings without algorithm? Is it relevant?
    setup <- sienaSetupForCpp(data, effects,
                              includeBehavior = TRUE,
                              includeBipartite = TRUE,
                              returnActorStatistics = FALSE,
                              returnStaticChangeContributions = TRUE,
                              parallelrun = FALSE)

    ans <- .Call(C_getTargets, PACKAGE=pkgname,
                 setup$pData, setup$pModel, setup$myeffects,
                 parallelrun = FALSE,
                 returnActorStatistics = FALSE,
                 returnStaticChangeContributions = TRUE)
    ans
}

expectedRelativeImportance <- function(conts, effects, theta, thedata=NULL,
	getChangeStatistics=FALSE, effectNames = NULL)
{
	rms <- function(xx){sqrt((1/dim(xx)[2])*rowSums(xx^2,na.rm=TRUE))}
	periods <- length(conts[[1]])
	effects <- effects[effects$include == TRUE,]
	noRate <- effects$type != "rate"
	effects <- effects[noRate,]
	if(sum(noRate)!=length(theta))
	{
		theta <- theta[noRate]
	}
	effectNa <- attr(conts,"effectNames")
	effectTypes <- attr(conts,"effectTypes")
	networkNames <- attr(conts,"networkNames")
	networkTypes <- attr(conts,"networkTypes")
	networkInteraction <- effects$interaction1
	effectIds <- paste(effectNa,effectTypes,networkInteraction, sep = ".")
	currentDepName <- ""
	depNumber <- 0
	for(eff in 1:length(effectIds))
	{
		if(networkNames[eff] != currentDepName)
		{
			currentDepName <- networkNames[eff]
			actors <- length(conts[[1]][[1]][[1]])
			depNumber <- depNumber + 1
			currentDepEffs <- effects$name == currentDepName
			effNumber <- sum(currentDepEffs)
			depNetwork <- thedata$depvars[[depNumber]]
			if (networkTypes[eff] == "oneMode")
			{
				choices <- actors
			}
			else if (networkTypes[eff] == "behavior")
			{
				choices <- 3
			}
			else if (networkTypes[eff] == "bipartite")
			{
				if (dim(depNetwork)[2] >= actors)
				{
					stop("interpret_size does not work for bipartite networks with
						second mode >= first mode")
				}
				choices <- dim(depNetwork)[2] + 1
			}
			else
			{
				stop("interpret_size does not work for dependent variables of type
					'continuous'")
			}

			# impute for period 1
			if (networkTypes[eff] %in% c("oneMode", "bipartite"))
			{
				depNetwork[,,1][is.na(depNetwork[,,1])] <- 0
			}
			else
			{
				depNetwork[,,1][is.na(depNetwork[,,1])] <- attr(depNetwork, 'modes')[1]
			}
			# impute for next periods;
			# this may be undesirable for structurals immediately followed by NA...
			for (m in 2:dim(depNetwork)[3]){depNetwork[,,m][is.na(depNetwork[,,m])] <-
				depNetwork[,,m-1][is.na(depNetwork[,,m])]}
				# Make sure the diagonals are not treated as structurals
				if (networkTypes[eff] == "oneMode")
				{
					for (m in 1:(dim(depNetwork)[3])){diag(depNetwork[,,m]) <- 0}
				}
			structurals <- (depNetwork >= 10)
			if (networkTypes[eff] == "oneMode"){
				if (attr(depNetwork, 'symmetric')){
					message('\nNote that for symmetric networks,
						effect sizes are for modelType 2
						(forcing).')
					}
				}

			# currentDepObjEffsNames <- paste(effects$shortName[currentDepEffs],
			# 	effects$type[currentDepEffs],effects$interaction1[currentDepEffs],sep=".")
			# otherObjEffsNames <- paste(effects$shortName[!currentDepEffs],
			# 	effects$type[!currentDepEffs],effects$interaction1[!currentDepEffs],sep=".")

			entropies <- vector(mode="numeric", length = actors)
			expectedRI <- list()
			expectedI <- list()
			RIActors <- list()
			IActors <- list()
			absoluteSumActors <- list()
			RHActors <-list()
			changeStats <-list()
			sigma <- list()
			sigmas <- matrix(NA, sum(currentDepEffs), periods)
			if (networkTypes[eff] == "behavior")
			{
				toggleProbabilities <- array(0, dim=c(actors, 3, periods))
			}
			else
			{
				toggleProbabilities <- array(0, dim=c(actors, choices, periods))
			}
			for(w in 1:periods)
			{
				currentDepEffectContributions <- conts[[1]][[w]][currentDepEffs]
				if (networkTypes[eff] == "bipartite")
				{
					currentDepEffectContributions <- lapply(
							currentDepEffectContributions,
							function(x){lapply(x,function(xx){xx[1:choices]})})
				}
				# conts[[1]] is periods by effects by actors by actors
				currentDepEffectContributions <-
					sapply(lapply(currentDepEffectContributions, unlist),
						matrix, nrow=actors, ncol=choices, byrow=TRUE,
						simplify="array")
				cdec <- apply(currentDepEffectContributions, c(2,1), as.matrix)
				# cdec is effects by actors (alters) by actors (egos)
				if (dim(currentDepEffectContributions)[3] <= 1) # only one effect
				{
					cdec <- array(cdec, dim=c(1,dim(cdec)))
				}
				rownames(cdec) <- effectNa[currentDepEffs]
				if (getChangeStatistics)
				{
					changeStats[[w]] <- cdec
				}
				# replace structural 0s and 1s by NA,
				# so they are omitted from calculation of RI, R_H, sigma
				if (networkTypes[eff] == "oneMode")
				{
					#	structuralsw <- structurals[,,w]
					for (ff in 1:(dim(cdec)[1])){cdec[ff,,][t(structurals[,,w])] <- NA}
				}
				distributions <- apply(cdec, 3,
					calculateDistributions, theta[which(currentDepEffs)])

				distributions <-
					lapply(apply(distributions, 2, list),
						function(x){matrix(x[[1]], nrow=effNumber+1,
							ncol=choices, byrow=F)})
				# distributions is a list, length = number of actors
				# distributions[[i]] is for actor i, a matrix of dim (effects + 1)
				# * (actors as alters) giving the probability of toggling the
				# tie variable to the alters;
				# the first row is for the unchanged parameter vector theta,
				# each of the following has put one element of theta to 0.
				# for behavior it is a matrix of dim (effects + 1) * 3
				if (networkTypes[eff] == "behavior")
				{
					toggleProbabilities[,,w] <-
						t(vapply(distributions, function(x){x[1,]}, rep(0,3)))
				}
				else
				{
					toggleProbabilities[,,w] <-
						t(vapply(distributions, function(x){x[1,]}, rep(0,choices)))
				}
				# toggleProbabilities is an array referring to ego * choices *
				# period, giving the probability of ego in a ministep at the
				# start of the period to make this choice.
				# For oneMode networks choices are alters;
				# for  bipartite choices are second mode nodes,and the last is "no change";
				# for behavior choices are to add -1, 0, +1.
				entropy_vector <- unlist(lapply(distributions,
						function(x){entropy(x[1,])}))
				## If one wishes another measure than the
				## L^1-difference between distributions, here is
				## the right place to call some new function instead of "L1D".
				RIs_list <- lapply(distributions,function(x){L1D(x[1,],
						x[2:dim(x)[1],])})
				RIs_matrix <-(matrix(unlist(RIs_list),nrow=effNumber,
						ncol=actors, byrow=F))
				entropies <- entropy_vector
				# divide by column sums:
				RIActors[[w]] <- t(t(RIs_matrix)/rowSums(t(RIs_matrix), na.rm=TRUE))
				rownames(RIActors[[w]]) <- effectNa[currentDepEffs]
				absoluteSumActors[[w]] <- colSums(RIs_matrix, na.rm=TRUE)
				RHActors[[w]] <- entropies
				expectedRI[[w]] <- rowMeans(RIActors[[w]], na.rm=TRUE)
				IActors[[w]] <- RIs_matrix
				rownames(IActors[[w]]) <- effectNa[currentDepEffs]
				expectedI[[w]] <- rowMeans(RIs_matrix, na.rm=TRUE)
				sigma[[w]] <- apply(cdec, c(1,3), sd, na.rm=TRUE)
				rownames(sigma[[w]]) <- effectNa[currentDepEffs]
				sigmas[,w] <- rms(sigma[[w]])
			}
			rownames(sigmas) <- effectNa[currentDepEffs]
			meansigmas <- rms(sigmas)
			names(meansigmas) <- effectNa[currentDepEffs]
			RItmp <- NULL
			RItmp$dependentVariable <- currentDepName
			RItmp$expectedRI <- expectedRI
			RItmp$RIActors <- RIActors
			RItmp$expectedI <- expectedI
			RItmp$IActors <- IActors
			RItmp$absoluteSumActors <- absoluteSumActors
			RItmp$RHActors <- RHActors
			RItmp$sigma <- sigma
			RItmp$sigmas <- sigmas
			RItmp$meansigmas <- meansigmas
			if(!is.null(effectNames))
			{
				RItmp$effectNames <- effectNames[currentDepEffs]
			}else{
				RItmp$effectNames <-
					paste(effectTypes[currentDepEffs], " ",
						effects$effectName[currentDepEffs], sep="")
			}
			if (getChangeStatistics){
				RItmp$changeStatistics <- changeStats
			}
			RItmp$toggleProbabilities <- toggleProbabilities
			class(RItmp) <- "sienaRI"
			attr(RItmp, "version") <- packageDescription(pkgname, fields = "Version")
			if(depNumber == 1){
				RI <- RItmp
			}else if(depNumber == 2){
				RItmp1 <- RI
				RI <- list()
				RI[[1]]<-RItmp1
				RI[[2]]<-RItmp
			}else{
				RI[[depNumber]]<-RItmp
			}
		}
	}
	if(depNumber>1)
	{
		message(paste("more than one dependent variable\n",
				"return value is therefore not of class 'sienaRI'\n",
				"but a list of objects of class 'sienaRI'."))
	}
	attr(RI, "version") <- packageDescription(pkgname, fields = "Version")
	RI
}

calculateDistributions <- function(effectContributions = NULL, theta = NULL)
{
	neffects <- dim(effectContributions)[1]
	nchoices <- dim(effectContributions)[2]
	distributions <- array(NA, dim = c(neffects+1,nchoices))
	the.choices <- !is.na(colSums(effectContributions))
	if (sum(the.choices) >= 2)
	{
		distributions[1,the.choices] <-
			exp(colSums(theta*effectContributions[,the.choices,drop=FALSE], na.rm=TRUE))/
			sum(exp(colSums(theta*effectContributions[,the.choices,drop=FALSE], na.rm=TRUE)))
		for(eff in 1:neffects)
		{
			th <- theta
			th[eff] <- 0
			distributions[eff+1,the.choices] <-
				exp(colSums(th*effectContributions[,the.choices,drop=FALSE], na.rm=TRUE))/
				sum(exp(colSums(th*effectContributions[,the.choices,drop=FALSE], na.rm=TRUE)))
		}
	}
	distributions
}

entropy <- function(distribution = NULL)
{
	if (sum(!is.na((distribution))) <= 1) # only constant choice
	{
		certainty <- NA
	}
	else
	{
		entropy <- -1*(sum(distribution * log(distribution), na.rm=TRUE)/
			log(sum(!is.na((distribution)))))
		certainty <- 1 - entropy
	}
	certainty
}

KLD <- function(referenz = NULL, distributions = NULL)
{
	if (sum(!is.na((referenz))) <= 1) # only constant choice
	{
		kld <- rep(NA, dim(distributions)[1])
	}
	else
	{
		if(is.vector(distributions))
		{
			kld <- sum(referenz * (log(referenz)-log(distributions)),
				na.rm=TRUE)/log(sum(!is.na((referenz))))
		}
		else
		{
			kld <- colSums(referenz * (log(referenz)-t(log(distributions))),
				na.rm=TRUE)/log(sum(!is.na((referenz))))
		}
	}
	kld
}

## calculates the L^1-differenz between distribution "reference"
## (which is a vector of length n)
## and each row of distributions (which is a matrix with n columns)
L1D <- function(referenz = NULL, distributions = NULL)
{
	if (sum(!is.na((referenz))) <= 1) # only constant choice
	{
		l1d <- rep(NA, dim(distributions)[1])
	}
	else
	{
		if(is.vector(distributions))
		{
			l1d <- sum(abs(referenz-distributions), na.rm=TRUE)
		}
		else
		{
			l1d <- colSums(abs(referenz-t(distributions)), na.rm=TRUE)
		}
	}
	l1d
}

##@print.sienaRI Methods
print.sienaRI <- function(x, printSigma = FALSE, ...){
	if (!inherits(x, "sienaRI"))
	{
		if (inherits(x[[1]], "sienaRI"))
		{
			cat("This object is a list, the components of which\n")
			cat("are Siena relative importance of effects objects.\n")
			cat("Apply the print function to the separate components.\n")
		}
		stop("not a legitimate Siena relative importance of effects object")
	}
	cat(paste("\n  Expected relative importance of effects for dependent variable '",
			x$dependentVariable,"' at observation moments:\n\n\n", sep=""))
	periods <- length(x$expectedRI)
	effs <- length(x$effectNames)
	colNames = paste("period ", 1:periods, sep="")
	line1 <- format("", width =63)
	line2 <- paste(format(1:effs,width=3), '. ',
		format(x$effectNames, width = 56),sep="")
	line3 <- line2
	line4 <- format(" R_H ('degree of certainty')", width = 61)
	line5 <- line2
	for (w in 1:length(colNames))
	{
		line1 <- paste(line1, format(colNames[w], width=8),"  ", sep = "")
		line2 <- paste(line2, format(round(x$expectedRI[[w]], 4),
				width=8, nsmall=4),"  ",sep="")
		line3 <- paste(line3, format(round(x$expectedI[[w]], 4),
				width=8, nsmall=4),"  ",sep="")
		line4 <- paste(line4,
			format(round(mean(x$RHActors[[w]], na.rm=TRUE), 4),
				width=8, nsmall=4),"  ",sep="")
		if (printSigma)
		{
			line5 <- paste(line5,
				format(round(x$sigmas[,w], 4), width=8, nsmall=4),"  ",sep="")
		}
	}
	line2 <- paste(line2, rep('\n',effs), sep="")
	line3 <- paste(line3, rep('\n',effs), sep="")
	cat(as.matrix(line1),'\n \n', sep='')
	cat(as.matrix(line2),'\n', sep='')
	cat("\n  Expected importance of effects for this dependent variable:\n\n")
	cat(as.matrix(line3),'\n\n', sep='')
	cat(as.matrix(line4),'\n', sep='')
	if (printSigma)
	{
		cat("\n sigma (average within-ego standard deviation of change statistics):\n\n")
		line5 <- paste(line5, rep('\n',effs), sep="")
		cat('\n',as.matrix(line5),'\n', sep='')
	}
	invisible(x)
}

##@summary.sienaRI Methods
summary.sienaRI <- function(object, ...)
{
	if (!inherits(object, "sienaRI"))
	{
		stop("not a legitimate Siena relative importance of effects object")
	}
	class(object) <- c("summary.sienaRI", class(object))
	object
}
##@print.summary.sienaRI Methods
print.summary.sienaRI <- function(x, ...)
{
	if (!inherits(x, "summary.sienaRI"))
	{
		stop("not a legitimate summary of a Siena relative importance of effects object")
	}
	print.sienaRI(x)
	invisible(x)
}


##@plot.sienaRI Methods
plot.sienaRI <- function(x, actors = NULL, col = NULL, addPieChart = FALSE,
	radius = 1, width = NULL, height = NULL, legend = TRUE,
	legendColumns = NULL, legendHeight = NULL, cex.legend = NULL,
	cex.names = NULL, ...)
{
	if (!inherits(x, "sienaRI"))
	{
		stop("not a legitimate Siena relative importance of effects object")
	}
	periods <- length(x$expectedRI)
	if (is.null(actors))
	{
		nactors <- dim(x$RIActors[[1]])[2]
		actors <- (1:nactors)
	}
	else
	{
		if ((!inherits(actors,"integer")) ||
			(min(actors) < 1) || (max(actors) > dim(x$RIActors[[1]])[2]))
		{
			stop(paste("parameter <actors> must be a set of integers from 1 to",
					dim(x$RIActors[[1]])[2]))
		}
		nactors <- length(actors)
	}
	if(legend)
	{
		if(!is.null(legendColumns))
		{
			if(is.numeric(legendColumns))
			{
				legendColumns <- as.integer(legendColumns)
			}else{
				legendColumns <- NULL
warning("legendColumns has to be of type 'numeric' \n used default settings")
			}
		}
		if(is.null(legendColumns))
		{
			legendColumns <-floor((nactors+2)/11)
		}
		if(!is.null(legendHeight))
		{
			if(is.numeric(legendHeight))
			{
				legendHeight <- legendHeight
			}else{
				legendHeight <- NULL
warning("legendHeight has to be of type 'numeric' \n used default settings")
			}
		}
		if(is.null(legendHeight))
		{
			legendHeight <-
				max(0.8,ceiling(length(x$effectNames)/legendColumns)*0.2)
		}
	}
	if(!is.null(height))
	{
		if(is.numeric(height))
		{
			height <- height
		}else{
			height <- NULL
warning("height has to be of type 'numeric' \n used default settings")
		}
	}
	if(is.null(height))
	{
		height <- 1
	}

	if(!is.null(width))
	{
		if(is.numeric(width))
		{
			width <- width
		}else{
			width <- NULL
warning("width has to be of type 'numeric' \n used default settings")
		}
	}
	if(is.null(width))
	{
		if(addPieChart)
		{
			width = (nactors/3+4)
		}else{
			width = (nactors/3+3)
		}
	}

	if(!is.null(cex.legend))
	{
		if(is.numeric(cex.legend))
		{
			cex.legend <- cex.legend
		}else{
			cex.legend <- NULL
warning("cex.legend has to be of type 'numeric' \n used default settings")
		}
	}
	if(is.null(cex.legend))
	{
		cex.legend <- 1.3
	}

	if(!is.null(cex.names))
	{
		if(is.numeric(cex.names))
		{
			cex.names <- cex.names
		}else{
			cex.names <- NULL
warning("cex.names has to be of type 'numeric' \n used default settings")
		}
	}
	if(is.null(cex.names))
	{
		cex.names <- 1
	}

	if(!is.null(radius))
	{
		if(is.numeric(radius))
		{
			rad <- radius
		}else{
			rad <- NULL
		warning("radius has to be of type 'numeric' \n used default settings")
		}
	}
	if(is.null(radius))
	{
		rad <- 1
	}

	if(!is.null(col))
	{
		cl <- col
	}else{
		alph <- 175
		green <- rgb(127, 201, 127,alph, maxColorValue = 255)
		lila <-rgb(190, 174, 212,alph, maxColorValue = 255)
		orange <- rgb(253, 192, 134,alph, maxColorValue = 255)
		yellow <- rgb(255, 255, 153,alph, maxColorValue = 255)
		blue <- rgb(56, 108, 176,alph, maxColorValue = 255)
		lightgray <- rgb(184,184,184,alph, maxColorValue = 255)
		darkgray <- rgb(56,56,56,alph, maxColorValue = 255)
		gray <- rgb(120,120,120,alph, maxColorValue = 255)
		pink <- rgb(240,2,127,alph, maxColorValue = 255)
		brown <- rgb(191,91,23,alph, maxColorValue = 255)
		cl <- c(green,lila,orange,yellow,blue,lightgray,darkgray,gray,pink,brown)
		while(length(cl)<length(x$effectNames)){
			alph <- (alph+75)%%255
			green <- rgb(127, 201, 127,alph, maxColorValue = 255)
			lila <-rgb(190, 174, 212,alph, maxColorValue = 255)
			orange <- rgb(253, 192, 134,alph, maxColorValue = 255)
			yellow <- rgb(255, 255, 153,alph, maxColorValue = 255)
			blue <- rgb(56, 108, 176,alph, maxColorValue = 255)
			lightgray <- rgb(184,184,184,alph, maxColorValue = 255)
			darkgray <- rgb(56,56,56,alph, maxColorValue = 255)
			gray <- rgb(120,120,120,alph, maxColorValue = 255)
			pink <- rgb(240,2,127,alph, maxColorValue = 255)
			brown <- rgb(191,91,23,alph, maxColorValue = 255)
			cl <- c(cl,green,lila,orange,yellow,blue,lightgray,darkgray,gray,pink,brown)
		}
	}
	bordergrey <-"gray25"

	if(addPieChart)
	{
		if(legend)
		{
			layoutMatrix <- matrix(c(1:(2*periods+1),(2*periods+1)), byrow= TRUE,
				ncol=2, nrow=(periods+1))
			layout(layoutMatrix,widths= c((nactors/6)+10,3.5+2.5*(rad^2)),
				heights=c(rep(height,periods),legendHeight))
		}else{
			layoutMatrix <- matrix((1:(2*periods)), byrow= TRUE,
				ncol=2, nrow=periods)
			layout(layoutMatrix,widths = c((nactors/6)+10,7+2.5*(rad^2)),
				heights=rep(height,periods))
		}
	}else{
		if(legend)
		{
			layoutMatrix <- matrix((1:(periods+1)), byrow= TRUE,
				ncol=1, nrow=(periods+1))
			layout(layoutMatrix)
		}else{
			layoutMatrix <- matrix((1:periods), byrow= TRUE, ncol=1, nrow=periods)
			layout(layoutMatrix, heights=2*rep(height,periods))
			# no widths, because these are only relative numbers,
			# so requiring constant widths is redundant
		}
		par( oma = c( 1, 1, 2, 1 ), xpd=T , cex = 0.75, no.readonly = TRUE )
	}
	par(mar = c(3,3,1,1))
	for(w in 1:periods)
	{
		barplot(cbind(x$RIActors[[w]][,actors], x$expectedRI[[w]]),
			space=c(rep(0.1,nactors),1.5),width=c(rep(1,nactors),1),
			beside =FALSE, yaxt = "n", xlab="Actor", cex.names = cex.names,
			ylab=paste("period ", w, sep=""),border=bordergrey,
			col = cl, names.arg=c(actors,"exp. rel. imp."))
		axis(2, at=c(0,0.25,0.5,0.75,1),labels=c("0","","0.5","","1"))
		axis(4, at=c(0,0.25,0.5,0.75,1),labels=c("0","","0.5","","1"))
		if(addPieChart)
		{
			pie(x$expectedRI[[w]], col = cl, labels=NA, border = bordergrey,
				radius = rad)
			mtext("exp. rel. imp.",side = 1, line = 1, cex=cex.names*0.75)
		}
	}
	if(legend)
	{
		plot(c(0,1), c(0,1), col=rgb(0,0,0,0), axes=FALSE, ylab = "", xlab = "")
		legend(0, 1, x$effectNames, fill=cl, ncol = legendColumns,
			bty = "n", cex=cex.legend)
	}
	invisible(cl)
}

##@sienaSetupForCpp Use as RSiena:::sienaSetupForCpp
sienaSetupForCpp <- function(data, effects,
                             includeBehavior = TRUE,
                             includeBipartite = TRUE,
                             returnActorStatistics = FALSE,
                             returnStaticChangeContributions = FALSE,
                             parallelrun = FALSE,
							 initC = FALSE) {
    f <- unpackData(data)
    effects <- effects[effects$include,]
    effects$setting <- rep("", nrow(effects))
    pData <- .Call(C_setupData, PACKAGE=pkgname,
                   list(as.integer(f$observations)),
                   list(f$nodeSets))
    reg.finalizer(pData, clearData, onexit = FALSE)
    .Call(C_OneMode, PACKAGE=pkgname, pData, list(f$nets))
    if (includeBipartite && !is.null(f$bipartites)) {
        .Call(C_Bipartite, PACKAGE=pkgname, pData, list(f$bipartites))
    }
    if (includeBehavior && !is.null(f$behavs)) {
        .Call(C_Behavior, PACKAGE=pkgname, pData, list(f$behavs))
    }
    .Call(C_ConstantCovariates, PACKAGE=pkgname, pData, list(f$cCovars))
    .Call(C_ChangingCovariates, PACKAGE=pkgname, pData, list(f$vCovars))
    .Call(C_DyadicCovariates, PACKAGE=pkgname, pData, list(f$dycCovars))
    .Call(C_ChangingDyadicCovariates, PACKAGE=pkgname, pData, list(f$dyvCovars))
    # if (is.null(f$exog) || !is.list(f$exog)) {
	# 	f$exog <- vector("list", length(f$depvars))
	# }
	# ## split the names of the constraints
	# higher <- attr(f, "allHigher")
	# disjoint <- attr(f, "allDisjoint")
	# atLeastOne <- attr(f, "allAtLeastOne")
	# froms <- sapply(strsplit(names(higher), ","), function(x)x[1])
	# tos <- sapply(strsplit(names(higher), ","), function(x)x[2])
	# .Call(C_Constraints, PACKAGE=pkgname,
	# 	pData, froms[higher], tos[higher],
	# 	froms[disjoint], tos[disjoint],
	# 	froms[atLeastOne], tos[atLeastOne])
	# siena07 only does this with !initC
	storage.mode(effects$parm) <- 'integer'
    storage.mode(effects$group) <- 'integer'
    storage.mode(effects$period) <- 'integer'
    effects$effectPtr <- rep(NA, nrow(effects))
    depvarnames <- names(data$depvars)
    tmpeffects <- split(effects, effects$name)
    myeffectsOrder <- match(depvarnames, names(tmpeffects))
    ans <- .Call(C_effects, PACKAGE=pkgname, pData, tmpeffects)
    pModel <- ans[[1]][[1]]
	reg.finalizer(pModel, clearModel, onexit = FALSE)

    for (i in 1:length(ans[[2]])) {
        effectPtr <- ans[[2]][[i]]
        tmpeffects[[i]]$effectPtr <- effectPtr
    }
    myeffects <- tmpeffects
    for (i in 1:length(myeffectsOrder)) {
        myeffects[[i]] <- tmpeffects[[myeffectsOrder[i]]]
    }
    list(pData = pData, pModel = pModel, myeffects = myeffects)
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: phase3.r
# *
# * Description: This module contains the function phase3 which runs final
# * iterations
# * with constant theta for estimating rate parameter, derivative matrix and
# * covariance matrix
# *****************************************************************************/
##args: x model object, z control object
##@phase3 siena07 Does phase 3
phase3 <- function(z, x, ...)
{
	if (is.null(x$lrt))
	{
		x$lrt <- FALSE
	}
	if (x$lrt)
	{
		z$myloglik <- NULL
		z$myfra <- NULL
		z$addChainToStore <- TRUE
	}
	
    ## initialize phase 3
    f <- FRANstore()
    DisplayTheta(z)
    z$Phase <-  3
    int <- z$int
    z$returnDeps <- z$returnDepsStored

    if (x$checktime) z$ctime <- proc.time()[3]

    ## fix up iteration numbers if using multiple processors
    if (x$n3 %% int > 0)
    {
        endNit <- x$n3 + int - x$n3 %%int
    }
    else
    {
        endNit <- x$n3
    }
	z$n3 <- endNit
    z$Phase3nits <- endNit

    ## revert to original requested method for phase 3 unless symmetric
    if (z$FinDiff.method && !x$FinDiff.method &&
        (!is.null(z$FinDiffBecauseSymmetric)) && z$FinDiffBecauseSymmetric)
    {
        z$Deriv <- FALSE
        z$FinDiff.method <- TRUE
    }
    else
    {
        z$Deriv <- !x$FinDiff.method
        z$FinDiff.method <- x$FinDiff.method
    }
    if (z$FinDiff.method)
        Report('Estimation of derivatives by the finite difference method.\n\n',
               outf)
    else
        Report('Estimation of derivatives by the LR method (type 1).\n\n', outf)

	z <- createSiena07stores(z, z$n3, f)
    xsmall<- NULL
    zsmall <- makeZsmall(z)
    if (!x$maxlike && !is.null(z$writefreq))
    {
        if (z$FinDiff.method)
            z$writefreq <- z$writefreq %/% z$pp
        else
            z$writefreq <- z$writefreq %/% 2
        z$writefreq <- roundfreq(z$writefreq)
    }
    z <- AnnouncePhase(z, x)
    Report('Simulated values, phase 3.\n', cf)
    nits <- seq(1, endNit, int)
    nits11 <- min(c(endNit, nits[nits >= 11]))
    writefreq <- z$writefreq
    if (is.null(z$writefreq))
    {
        z$writefreq <- 20
    }
	z <- doPhase1or3Iterations(3, z, x, zsmall, xsmall, nits, 0, nits11,
							   writefreq)
    z <- phase3.2(z,x)
    z
}

##@phase3.2 siena07 Processing at end of phase 3
phase3.2 <- function(z, x, ...)
{
    z$timePhase3 <- (proc.time()['elapsed'] - z$ctime) / z$Phase3nits
    if (x$checktime)
	{
        Report(c('Time per iteration in phase 3   = ',
                 format(z$timePhase3, nsmall=4, digits=4), '\n'), lf)
	}
	z <- CalculateDerivative3(z, x)
	if (!x$simOnly)
	{
		z <- PotentialNR(z, x, FALSE) # this calculates z$dinv
	}
    if (any(z$newfixed))
    {
        Report('There was a problem in obtaining convergence.\n', outf)
        Report(c('Therefore, the program decided tentatively to fix',
				 'parameter(s)',
					(1:z$pp)[z$newfixed], '.\n'), outf)
        Report(c('It may be better to start all over again,',
                 'with better initial values or a reduced model.\n',
                 '(Check that you entered the data properly!)\n'), outf)
    }
	if ((x$nsub == 0)&(x$simOnly))
	{
        dmsf <- diag(z$msf)
        sf <- colMeans(z$sf)
	}
	else
	{
        Heading(2, outf, c('End of stochastic approximation algorithm, phase ',
                           z$Phase, '.'))
        Report(c('Total of', z$n,'iterations.\n'), outf)
        Report(c('Parameter estimates based on', z$n - z$Phase3nits,
                 'iterations,\n'), outf)
        if (!x$maxlike && z$cconditional)
 		{
            Report(c('basic rate parameter',
                     c('', 's')[as.integer(z$f$observations > 2) + 1],
                     ' as well as \n'), sep='', outf)
 		}
 		Report(c('convergence diagnostics, covariance and derivative matrices ',
 				 'based on ', z$Phase3nits, ' iterations.\n\n'), sep='', outf)
        Report('Information for convergence diagnosis.\n', outf)
        Report(c('Averages, standard deviations, ',
               'and t-ratios for deviations from targets:\n'), sep='', outf)
   #  #  Report(c(date(),'\n'),bof)
        if (x$maxlike)
 		{
            Report('\nMaximum Likelihood estimation.', bof)
 		}
        else if (z$gmm)
        {
          Report('\nGeneralized Method of Moments estimation.', bof)
        }
        else
 		{
 			if (z$cconditional)
 			{
 				Report('\nconditional moment estimation.', bof)
 			}
 			else
 			{
 				Report('\nunconditional moment estimation.', bof)
 			}
 		}
        Report('\nInformation for convergence diagnosis.\n', bof)
        Report(c('Averages, standard deviations, ',
            'and t-ratios for deviations from targets:\n'), bof, sep='')
        ##calculate t-ratios
        dmsf <- diag(z$msf)
        sf <- colMeans(z$sf)
        toosmall <- dmsf < 1e-20 * z$scale * z$scale
        toosmall2 <- abs(sf) < 1e-10 * z$scale
        dmsf[toosmall] <- 1e-20 * z$scale[toosmall] * z$scale[toosmall]
        tstat <- rep(NA, z$pp)
        tstat[!toosmall]<- sf[!toosmall] / sqrt(dmsf[!toosmall])
        tstat[toosmall & toosmall2] <- 0
        tstat[toosmall & !toosmall2] <- NA
        z$tstat <- tstat
		if (z$gmm)
        {
			cvsf <- cov(z$sf)
			if (inherits(try(W <- solve(cvsf), silent=TRUE),
 						"try-error"))
			{
				warning("Results not reliable")
				Report("Statistics collinear. Results not reliable.\n", outf)					
				cvsf[ z$fixed & !z$gmmEffects, ] <- 0
				cvsf[ , z$fixed & !z$gmmEffects] <- 0
				diag(cvsf)[z$fixed & !z$gmmEffects] <- 1
				cvsfr <- cvsf + 0.01*diag(cvsf)
				W <- solve(cvsfr)
			}
			gamma <- z$dfra[,-which(z$gmmEffects==TRUE)]
			B0 <- t(gamma) %*% W
			B <- solve(diag(sqrt(rowSums(B0*B0)))) %*% B0 # Row-normalized matrix B
			D0 <- B%*%gamma # Matrix D = B * gammaT 
# I (TS) think the following is superfluous, because z$dinvv is never used in Phase 3.
			if (inherits(try(D0inv <- solve(D0), silent=TRUE),
 						"try-error"))
			{
				z$dinvv <- NULL
			}
			else
			{
				z$dinvv <- D0inv%*%B			
			}
          sf <- B%*%colMeans(z$sf)
          dmsf <- diag(B%*%z$msf%*%t(B))
          dimTheta <- length(sf)
          scaleGmm <- z$scale[1:dimTheta]
          toosmall <- dmsf < 1e-20 * scaleGmm * scaleGmm
          toosmall2 <- abs(sf) < 1e-10 * scaleGmm
          dmsf[toosmall] <- 1e-20 * scaleGmm [toosmall] * scaleGmm [toosmall]
          tstat <- rep(NA, length(sf))
          tstat[!toosmall]<- sf[!toosmall] / sqrt(dmsf[!toosmall])
          tstat[toosmall & toosmall2] <- 0
          tstat[toosmall & !toosmall2] <- NA
          z$tstat <- tstat
          z$B <- B
          z$D0 <- D0
          z$gamma <- t(gamma)
          z$W <- W
        }
 		# tconv.max = Maximum value of t-ratio for convergence,
 		# for any linear combination.
 		z$tconv.max <- NA
 		if (sum(!z$fixed) > 0)
 		{
 		  if (!z$gmm)
 		  {
 			mean.dev <- colSums(z$sf)[!z$fixed]/dim(z$sf)[1]
 			cov.dev <- z$msf[!z$fixed,!z$fixed]
		  }
 		  else
 		  {
 		    fixednoGmm <- z$fixed[-which(z$gmmEffects)]
 		    mean.dev <- (B%*%colSums(z$sf)/dim(z$sf)[1])[!fixednoGmm]
 		    cov.dev <- (B%*%z$msf%*%t(B))[!fixednoGmm, !fixednoGmm]
 		  }
 			if (inherits(try(thisproduct <- solve(cov.dev, mean.dev), silent=TRUE),
 						"try-error"))
 			{
 	Report('Overall maximum t-ratio for convergence not computable.\n', outf)
 			}
 			else
 			{
 				z$tconv.max <- sqrt(t(mean.dev) %*% thisproduct)
 			}
 		}
 		if (!z$gmm)
 		{
 		  mymess1 <- paste(format(1:z$pp,width=3), '. ',
 		                   format(round(sf, 4), width=8, nsmall=4), ' ',
 		                   format(round(sqrt(dmsf), 4) ,width=8, nsmall=4), ' ',
 		                   format(round(tstat, 4), width=8, nsmall=3), sep='')
 		  mymess2 <- c('', '    (fixed parameter)')[as.numeric(z$fixed) + 1]
 		} 
    else
    {
      mymess1 <- paste(format(1:(z$pp-sum(z$gmmEffects)),width=3), '. ',
                       format(round(sf, 4), width=8, nsmall=4), ' ',
                       format(round(sqrt(dmsf), 4) ,width=8, nsmall=4), ' ',
                       format(round(tstat, 4), width=8, nsmall=3), sep='')
      mymess2 <- c('', '    (fixed parameter)')[as.numeric(z$fixed[-which(z$gmmEffects)]) + 1]
    }
        mymess <- paste(mymess1, mymess2)
        PrtOutMat(as.matrix(mymess), outf)
        PrtOutMat(as.matrix(mymess1), bof)
        ##  Report(mymess1, bof, fill=80)
        if (!z$gmm)
         {
          tmax <- max(abs(tstat)[!z$fixed])
         }
        else
         {
           tmax <- max(abs(tstat)[which(!z$fixed[-which(z$gmmEffects)])])
         }
        z$tconv <- tstat
		z$tmax <- tmax
        error <- (is.na(tmax)) ||
						(abs(tmax) > 4.0 / sqrt(z$Phase3nits)) && (abs(tmax) > 0.3)
        if ((is.na(tmax)) || (tmax >= 0.4))
 		{
            z$error <- TRUE
 		}
 		Report('Good convergence is indicated by the t-ratios ', outf)
        if (any(z$fixed) & !z$gmm)
 		{
 			Report('of non-fixed parameters ', outf)
 		}
        Report('being close to zero.\n', outf)
		Report(c('\nOverall maximum convergence ratio = ',
			round(z$tconv.max, digits=4), '.\n'), outf)
        if (z$Phase3nits < 100)
 		{
            Report(c('(Since the diagnostic checks now are based only on ',
                     z$Phase3nits,
                     ' iterations,', '\nthey are not reliable.)\n'), sep='', outf)
 		}
        if (error) ## also test subphase here but not relevant to phase 3, I think
        {
            Report('One or more of the t-statistics are rather large.\n', outf)
			doubts <- ifelse(is.na(tmax), TRUE, tmax > 0.5)
            if (doubts)
 			{
                Report('Convergence of the algorithm is doubtful.\n', outf)
 			}
 			## removed repfortotal loop possibility here as not functioning now
            if (z$Phase3nits <= 50)
 			{
                Report(c('Note that the standard deviations are based on',
                         'few simulations.\n'), outf)
 			}
 		}
	}
    if (x$maxlike)
    {
        sfl <- apply(z$sf, 2,
                       function(x)acf(x, plot=FALSE, lag.max=1)[[1]][[2]])
        Report('Autocorrelations during phase 3 : \n', outf)
        Report(paste(format(1:z$pp, width=3), '. ',
                     format(sfl, width=8, digits=4),
                     '\n', collapse="", sep=""), outf)
        Report ('\n', outf)
        Report('Autocorrelations during phase 3 : \n', cf)
        Report(paste(format(1:z$pp, width=3), '. ',
                     format(sfl, width=8, digits=4),
                     '\n', collapse="", sep=""), cf)
        Report ('\n', cf)
		z$ac3 <- sfl
    }
    for (j in 1:z$pp)
	{
        if (z$diver[j]) ### don't understand this condition, as AllFixed is true
        {
            Report(c('Warning. Extremely large standard error of parameter',j,
                     '.\n'), outf)
            if (sf[j] < 0.5 * sqrt(dmsf[j]))
			{
                Report('Presumably this parameter must be fixed.\n', outf)
			}
			else
			{
                Report('Maybe the algorithm diverged.\n', outf)
			}
		}
	}
	errorMessage.cov <- ''
	if (!x$simOnly)
	{
		if (x$maxlike)
		{
			Report('Estimated complete data information matrix: \n', cf)
			PrtOutMat(z$dfra, cf)
			Report(c('Estimated conditional covariance matrix score function ',
				'(unobserved information):\n'), cf)
			PrtOutMat(z$msf, cf)
			Report('\n', cf)
			dfrac <- z$dfra - z$msf
			## dfrac[z$fixed[row(dfrac)] | z$fixed[col(dfrac)]] <- 0
			## a clever way to do it
			dfrac[z$fixed, ] <- 0
			dfrac[ ,z$fixed] <- 0
			diag(dfrac)[z$fixed] <- 1
			if (inherits(try(cov.est <- solve(dfrac), silent=TRUE),"try-error"))
			{
				Report('Noninvertible estimated covariance matrix : \n', outf)
	errorMessage.cov <- '***Warning: linear dependencies between statistics ***'
				cov.est <- NA * dfrac
			}
		}
		else if (z$gmm)
		{		  
			if (inherits(try(D0inv <- solve(D0), silent=TRUE),
 						"try-error"))
			{
				cov.est <- matrix(NA, nrow(D0), nrow(D0))
			}
			else
			{
				cov.est <-  D0inv %*% (B%*%z$msfc%*%t(B)) %*% t(D0inv)	
			}
		}
		else
		{
			cov.est <- z$dinv %*% z$msfc %*% t(z$dinv)
		}
		error <- FALSE
		ei <- eigen(z$msfc)
		mineivalue <- min(ei[[1]])
		if ((mineivalue < 1e-12) && (x$n3 > z$pp + 20)) # 1e-12 seems a small enough bound
		{
			Report('*** Warning: Covariance matrix not positive definite *** \n', outf)
			Report('***            Standard errors not reliable           *** \n', outf)
			maxei <- max(abs(ei[[2]][,z$pp]))
			# last eigenvector corresponds to smallest eigenvalue
			smallei <- ei[[2]][,z$pp]/maxei
			Report(
			 'The approximate linear combination that has variance 0 is\n', outf)
			thetext <- paste(round(smallei,2)[which(abs(smallei) > 0.1)],
							' * beta[',
							(1:z$pp)[which(abs(smallei) > 0.1)], ']',
							sep="", collapse=" + ")
			Report(thetext, outf)
			Report('\n',outf)
			fromBayes <- 'fromBayes' %in% names(x)
			if (!fromBayes)
			{
				warning('*** Warning: Covariance matrix not positive definite *** \n')
				message('*** Standard errors not reliable ***')
				message('The following is approximately a linear combination ')
				message('for which the data carries no information:\n', thetext)
				message('It is advisable to drop one or more of these effects.')
				if (any(z$fixed) || any(z$newfixed))
				{
					Report(c('(This may be unimportant, and related to the fact\n',
							'that some parameters are fixed.)\n'), outf)
				}
				else
				{
					Report('Do not use any reported standard errors.\n', outf)
					errorMessage.cov <- '*** Warning: Noninvertible estimated covariance matrix ***'
				}
			}
			cov.est <- NA * z$msfc
		}
		if (!is.null(cov.est) & !z$gmm)
		{
			zerovar <- ((diag(cov.est) < 1e-9) | (is.na(diag(cov.est))))
			z$diver <- (z$fixed | z$diver | zerovar) & (!z$AllUserFixed)
			cov.est[z$diver, ] <- NA
			cov.est[, z$diver] <- NA
		}
		else if (!is.null(cov.est) & z$gmm)
		{ 
		  zerovar <- ((diag(cov.est) < 1e-9) | (is.na(diag(cov.est))))
		  z$diver <- (z$fixed[-which(z$gmmEffects)] | z$diver[-which(z$gmmEffects)] | zerovar) & (!z$AllUserFixed)
		  cov.est[z$diver, ] <- NA
		  cov.est[, z$diver] <- NA
		}
		z$covtheta <- cov.est
	}
	z$errorMessage.cov <- errorMessage.cov
	z$sf.invcov <- NULL
	## ans<-InstabilityAnalysis(z)
	
	if (x$lrt)
	{
		z$nGroup <- 1
		
		if (ifelse(is.null(x$thermo), FALSE, x$thermo))
		{
			z <- thermo(z,x,subphase=4, ...) 
		}
		
		if (ifelse(is.null(x$bridge1), FALSE, x$bridge1))
		{
			z <- bridge1(z,x,subphase=4, ...)
		}
		
		z$nbrNodes <- 1
		f <- FRANstore()
		.Call("clearStoredChains", PACKAGE=pkgname,
		f$pModel, keep = 0,1)
	}
	
	z
}

##@CalculateDerivative3 siena07 Calculates derivative at end of phase 3
CalculateDerivative3<- function(z,x)
{
	z$mnfra <- colMeans(z$sf)
	estMeans <- z$mnfra + z$targets
	z$regrCoef <- rep(0, z$pp)
	z$regrCor <- rep(0, z$pp)
	if (z$FinDiff.method || x$maxlike)
	{
		dfra <- t(as.matrix(Reduce("+", z$sdf) / length(z$sdf)))
	}
	else
	{
		dfra <- derivativeFromScoresAndDeviations(z$ssc, z$sf2,
			z$dfras, z$sscs, z$sf2s, z$sf2.byIteration, z$Phase3nits)
		if (any(diag(dfra) < 0))
		{
			sub <- which(diag(dfra) < 0)
			dfra[sub,] <- 0
			dfra[,sub] <- 0
			diag(dfra)[sub] <- 1
			Report(c("Warning: diagonal element(s)", sub,
					" of derivative matrix < 0\n"), cf)
		}
			if (z$sf2.byIteration)
			{
				scores <- apply(z$ssc, c(1,3), sum)  # z$nit by z$pp matrix
			}
			else
			{
				scores <- z$scores
			}
			for (i in 1:z$pp)
			{
				oldwarn <- getOption("warn")
				options(warn = -1)
				if ((var(scores[,i]) > 0)&&(var(z$sf[,i]) > 0))
				{
					z$regrCoef[i] <- cov(z$sf[,i], scores[,i])/var(scores[,i])
					z$regrCor[i] <- cor(z$sf[,i], scores[,i])
				}
				if (is.na(z$regrCor[i])){z$regrCor[i] <- 0}
				if (is.na(z$regrCoef[i])){z$regrCoef[i] <- 0}
				options(warn = oldwarn)
			}
		if (x$dolby)
		{
			estMeans <- estMeans - (z$regrCoef * colMeans(scores))
		}
		Report('Correlations between scores and statistics:\n', cf)
		PrtOutMat(format(as.matrix(t(z$regrCor)), digits = 2, nsmall = 2), cf)
	}
	z$estMeans <- estMeans
	z$diver <- rep(FALSE, z$pp)
	if (z$AllUserFixed & any(abs(diag(dfra)) < 1e-6))
	{
		z$diver[abs(diag(dfra)) < 1e-6] <- TRUE
	}
	z$msf <- cov(z$sf)
	z$dfra1 <- z$dfra
	z$dfra <- dfra
	if (x$simOnly)
	{
		dmsf <- diag(z$msf)
		sem <- sqrt(dmsf/dim(z$sf)[1])
		if ((x$dolby) & (!z$thetaFromFile))
		{
			sem <- sem*sqrt(1 - (z$regrCor)^2)
		}
		z$estMeans.sem <- sem
	}
	z
}

##@PotentialNR siena07 Calculates change if NR step done now
PotentialNR <-function(z,x,MakeStep=FALSE)
{
	z$dfrac <- z$dfra
	z$msfc <- z$msf
	if (!z$AllUserFixed)
	{
	  if (!z$gmm)
	  {
	    z$dfrac[z$fixed, ] <- 0
	    z$dfrac[, z$fixed] <- 0
	    diag(z$dfrac)[z$fixed]<- 1
	    z$msfc[z$fixed, ] <- 0
	    z$msfc[, z$fixed] <- 0
	    diag(z$msfc)[z$fixed] <- 1
	  }
	  else 
	  {
	    z$dfrac[(z$fixed & !z$gmmEffects), ] <- 0
	    z$dfrac[, (z$fixed & !z$gmmEffects)] <- 0
	    diag(z$dfrac)[(z$fixed & !z$gmmEffects)]<- 1
	    z$msfc[(z$fixed & !z$gmmEffects), ] <- 0
	    z$msfc[, (z$fixed & !z$gmmEffects)] <- 0
	    diag(z$msfc)[(z$fixed & !z$gmmEffects)] <- 1
	  }
	}
	if (inherits(try(dinv <- solve(z$dfrac), silent=TRUE), "try-error"))
	{
		Report('Error message from inversion of dfra: \n', cf)
		diag(z$dfrac) <- diag(z$dfrac)+0.1*z$scale
		Report('Intervention 3.4: ridge added after phase 3.\n', cf)
		if (inherits(try(dinv <- solve(z$dfrac), silent=TRUE), "try-error"))
		{
			Report(c('Warning. After phase 3, derivative matrix non-invertible',
					'even with a ridge.\n'), cf)
			z$errorMessage.cov <- '*** Warning: Noninvertible derivative matrix ***'
			fchange <- 0
			z$dinv <- z$dfrac
			z$dinv[,] <- NA # 999
		}
		else
		{
			fchange <- dinv %*% colMeans(z$sf)
			z$dinv <- dinv
		}
	}
	else
	{
		fchange <- dinv%*%colMeans(z$sf)
		z$dinv <- dinv
	}
	z$fchange <- fchange
	Report('dfrac :\n', cf)
	PrtOutMat(z$dfrac, cf)
	Report('inverse of dfra :\n', cf)
	PrtOutMat(z$dinv, cf)
	try(if (x$errorMessage.cov > '') {Report(z$errorMessage.cov, cf)}, silent=TRUE)
	# "Try" for downward compatilibity
	Report(c('A full Quasi-Newton-Raphson step after phase 3\n',
			'would add the following numbers to the parameters, yielding ',
			'the following results:\n'), sep='', cf)
	Report('         change     new value \n', cf)
	Report(c(paste('  ', format(1:z$pp, width=2), '. ',
				format(round(-fchange, digits=6), width=12, nsmall=6),
				format(round(z$theta - fchange, 6), width=12, nsmall=6),
				sep='', collapse='\n'), '\n'), cf)
	if (MakeStep) ##currently not used
	{
		Report(c('\nAt the end of phase ', z$phase,
				', parameter values are \n'), outf)
		Report(paste(1:z$pp,'. ',format(z$theta,width=18,digits=6)),outf)
		try(Report(z$errorMessage.cov, outf), silent=TRUE)
		Report(c('A full Quasi-Newton-Raphson step after phase 3\n',
				'would add the ',
				'following numbers to the parameters:\n'), outf)
		Report(paste(1:z$pp, '. ', format(-round(fchange, 6), width=12)), outf)
		Report('\n\n', outf)
		if (z$SomeFixed)
		{
			fchange[z$fixed] <- 0
		}
		##check positivity
		if (z$anyposj)
		{
			neg <- z$posj & fchange >= z$theta
			fchange[neg] <- 0.5 * z$theta[neg]
			Report(c('Intervention 3.4.3: positivity restriction after phase 3',
					'coordinate(s)', neg, '.\n'), cf)
		}
	}
	z
}

doPhase1or3Iterations <- function(phase, z, x, zsmall, xsmall, nits, nits6=0,
	nits11=0, writefreq)
{
	int <- z$int
	nWaves <- z$observations - 1
	for (nit in nits)
	{
		z$nit <- nit
		## initial steps differ between phases
		if (phase == 1)
		{
			if (length(nits) > 1 && nit == nits[2])
			{
				time1 <- proc.time()['elapsed']
				if (x$checktime)
				{
					z$ctime <- time1
				}
			}
			if (nit == nits6)
			{
				time1 <- proc.time()['elapsed']-time1
				if (time1 > 1e-5)
				{
					z$writefreq <- round(20.0/time1)
				}
				else
				{
					z$writefreq <- 5
				}
				if (is.batch())
				{
					z$writefreq <- 10 * z$writefreq
				}
				z$writefreq <- roundfreq(z$writefreq)
			}
			if (nit > nits6 && is.null(z$ctime))
			{
				time1 <- proc.time()['elapsed']
				if (x$checktime)
				{
					z$ctime <- time1
				}
			}
			DisplayIteration(z)
		}
		else ## phase 3
		{
			if (is.null(writefreq))
			{
				if (nit == nits[2])
				{
					z$time1 <- proc.time()[[3]]
				}
				else if (nit == nits11)
				{
					time1 <- proc.time()[[3]] - z$time1
					if (time1 > 1e-5)
					{
						z$writefreq <- max(1, round(20.0 / time1))
					}
					else
					{
						z$writefreq <- 20
					}
					if (is.batch())
					{
						z$writefreq <-  z$writefreq * 5 ##compensation for it
						## running faster with no tcl/tk
					}
					z$writefreq <- roundfreq(z$writefreq)
					writefreq <- z$writefreq
				}
			}
			if (nit <= 5 || nit == 10 || (int==1 && nit %% z$writefreq == 0 ) ||
				(int > 1 && (z$writefreq + 1) < z$n3%/%int &&
					nit %in% nits[seq(z$writefreq + 1, x$n3 %/% int,
						z$writefreq)]))
			{
				if (!is.batch())
				{
					DisplayIteration(z)
					##  DisplayTheta(z)
					DisplayDeviations(z, z$sf[nit-1,])
				}
				## else
				## {
				##     Report(c('Phase ', z$Phase,' Iteration ',nit,'\n'))
				## }
				if (nit %% z$writefreq == 0 || (int > 1 &&
						nit %% z$writefreq == 1))
				{
					increment <- ifelse(nit <= 5, int,
						ifelse(nit <= 10, 5, z$writefreq * int))
					val<- getProgressBar(z$pb)
					if (z$FinDiff.method)
						val <- val + increment * (z$pp + 1)
					else
						val <- val + increment
					## sort out progress bar
					z$pb <- setProgressBar(z$pb, val)
					if (is.batch())
						Report(c("Phase ", z$Phase, " Iteration ", nit,
								" Progress ",
								round(val / z$pb$pbmax * 100), "%\n"), sep='')
				}
			}
		}
		## iteration proper

		if (z$thetaFromFile)
		{
			if (nit <= dim(z$thetaValues)[1])
			{
				zsmall$theta <- z$thetaValues[nit,]
			}
		}
		if (z$int == 1)
		{
			if (is.null(x$lrt))
			{
				x$lrt <- FALSE
			}
			if (!x$lrt)
			{
				zz <- x$FRAN(zsmall, xsmall)
			} else {
				zz <- x$FRAN(zsmall, xsmall, returnLoglik=TRUE, returnChains=TRUE)
				z$myloglik <- c(z$myloglik, sum(zz$loglik))
				fra <- colSums(zz$fra) - z$targets
				z$myfra <- rbind(z$myfra, fra)
			}
			
			if (!zz$OK)
			{
				z$OK <- zz$OK
				z$zz <- zz
				return(z)
			}
			z$n <- z$n + 1
			if (phase == 1)
			{
				z$phase1Its <- z$phase1Its + 1
			}
		}
		else ### only do parallels at this level for forward simulation
		{
			zz <- clusterCall(z$cl, simstats0c, zsmall, xsmall)
			z$n <- z$n + z$int
			if (phase == 1)
			{
				z$phase1Its <- z$phase1Its + z$int
			}
		}
		## now update the stores
		if (z$int == 1)
		{
			fra <- colSums(zz$fra)
			fra <- fra - z$targets
			if (z$FinDiff.method)
			{
				fra2 <- zz$fra
			}
			z$sf[z$nit, ] <- fra
			if (z$sf2.byIteration)
			{
				z$sf2[z$nit, , ] <- zz$fra
			}
			else
			{
				z$sf2s <- z$sf2s + zz$fra
			}
			z$sims[[z$nit]] <- zz$sims
			z$chain[[z$nit]] <- zz$chain
			z$changeContributions[[z$nit]] <- zz$changeContributions

			fra <- fra + z$targets
			if (z$thetaFromFile)
			{
				z$thetaUsed[z$nit, ] <- zsmall$theta
			}
		}
		else
		{
			for (i in 1:int)
			{
				fra <- colSums(zz[[i]]$fra)
				fra <- fra - z$targets
				z$sf[z$nit + (i - 1), ] <- fra
				if (z$sf2.byIteration)
				{
					z$sf2[z$nit + (i - 1), , ] <- zz[[i]]$fra
				}
				else
				{
					z$sf2s <- z$sf2s + zz[[i]]$fra
				}
				z$sims[[z$nit + (i - 1)]] <- zz[[i]]$sims
				## To allow returning chains and changeContributions in parallel?
				# z$chain[[z$nit + (i - 1)]] <- zz[[i]]$chain
				# z$changeContributions[[z$nit + (i - 1)]] <- zz[[i]]$changeContributions
				if (z$thetaFromFile)
				{
					z$thetaUsed[z$nit + (i - 1), ] <- zsmall$theta
				}
			}
			if (z$FinDiff.method)
			{
				fra2 <- t(sapply(zz, function(x)x$fra))
				dim(fra2) <- c(int, nrow(zz[[1]]$fra), z$pp)
			}
			fra <- t(sapply(zz, function(x) colSums(x$fra)))
		}
		if (x$maxlike)
		{
# length of zz$accepts, zz$rejects, zz$aborts determined in siena07models.cpp 
# in lines 621-627 function mlPeriod
			z$sdf[[z$nit]] <- zz$dff
			z$sdf2[[z$nit]] <- zz$dff2
			z$accepts[z$nit, , ] <- zz$accepts
			z$rejects[z$nit, , ] <- zz$rejects
			z$aborts[z$nit, , ] <- zz$aborts
		}
		else if (z$FinDiff.method)
		{
			z <- FiniteDifferences(z, x, fra, fra2)
			for (i in 0:(z$int - 1))
			{
				z$sdf[[z$nit + i]] <- z$sdf0[i + 1, , ]
				if (z$byWave)
				{
					z$sdf2[[z$nit + i]] <- z$sdf02[i + 1, , , ]
				}
			}
		}
		else
		{
			if (int==1)
			{
				if (!is.null(zz[['sc']]))
				{
					if (z$sf2.byIteration)
					{
						z$ssc[z$nit , ,] <- zz$sc
					}
					else
					{
						z$sscs <- z$sscs + zz$sc
						z$scores[z$nit,] <- colSums(zz$sc)
						# z$dfras + rowSums(sapply(1:nWaves, function(j){outer(zz$sc[j,], zz$fra[j, ])}))
						for (j in 1:nWaves)
						{
							z$dfras <- z$dfras + outer(zz$sc[j,], zz$fra[j, ])
						}
					}
				}
			}
			else
			{
				for (i in 1:int)
				{
					if (!is.null(zz[[i]][['sc']]))
					{
						if (z$sf2.byIteration)
						{
							z$ssc[z$nit + (i - 1), , ] <- zz[[i]]$sc
						}
						else
						{
							z$sscs <- z$sscs + zz[[i]]$sc
							z$scores[z$nit + (i - 1), ] <- colSums(zz[[i]]$sc)
							for (j in 1:nWaves)
							{
								z$dfras <- z$dfras + outer(zz[[i]]$sc[j,], zz[[i]]$fra[j, ])
							}
						}
					}
				}
			}
		}
		if ((!x$maxlike) && z$cconditional && z$Phase == 3)
		{
			if (int == 1)
			{
				z$ntim[z$nit, ] <- zz$ntim0
			}
			else
			{
				for (i in 1:int)
				{
					z$ntim[z$nit + (i-1), ] <- zz[[i]]$ntim0
				}
			}
		}
		## post processing also differs
		if (phase == 1)
		{
			CheckBreaks()
			if (UserInterruptFlag() || UserRestartFlag())
			{
				return(z)
			}
			val <- getProgressBar(z$pb)
			if (z$FinDiff.method)
			{
				val <- val + z$pp + 1
			}
			else
			{
				val <- val + 1
			}
			z$Phase1nits <- nit
			z$pb <- setProgressBar(z$pb, val)
			progress <- val / z$pb$pbmax * 100
			if (z$nit <= 5 || z$nit %% z$writefreq == 0 || z$nit %%5 == 0 ||
				x$maxlike || z$FinDiff.method ||
				(z$int > 1 && z$nit %% z$writefreq < z$int))
			{
				##  Report(c('Phase', z$Phase, 'Iteration ', z$nit, '\n'))
				if (is.batch())
				{
					Report(c('Phase ', z$Phase, ' Iteration ', z$nit,
							' Progress: ', round(progress), '%\n'), sep='')
				}
				else
				{
					DisplayTheta(z)
					DisplayDeviations(z, z$sf[z$nit, ])
				}
			}
			if (!z$OK || UserRestartFlag() || UserInterruptFlag())
				return(z)
		}
		else ## phase 3
		{
			if (!is.batch())
			{
				if (nit < 10)
				{
					Report(c("  ", nit, " ",
							format(z$sf[nit,], width=1, digits=4, nsmall=4),
							"\n"), cf)
				}
				if (nit >= 10)
				{
					CheckBreaks()
					##    if (nit==10) set up stopkey hint
					##   Early termination of estimation
					if( UserInterruptFlag())
					{
						Report(c("The user asked for an early stop of the ",
								"algorithm during phase 3.\n"), outf)
						z$Phase3Interrupt <- TRUE
						if (nit < 500)
						{
							if (EarlyEndPhase2Flag())
							{
								Report('This implies that ', outf)
							}
							else
							{
								Report(c("This implies that the estimates",
										"are as usual,\nbut the "), outf)
							}
							Report(c("diagnostic checks, covariance",
									"matrices and t-values \nare less ",
									"reliable, because they ",
									'are now based on only', nit,
									'phase-3 iterations.\n\n'), outf)
						}
						z$sf <- z$sf[1:nit, , drop=FALSE]


						if (z$sf2.byIteration)
						{
							z$sf2 <- z$sf2[1:nit, , , drop=FALSE]
						}
						if (!z$maxlike && !z$FinDiff.method)
						{
							if (z$sf2.byIteration)
							{
								z$ssc <- z$ssc[1:nit, , , drop=FALSE]
							}
							else
							{
								z$scores <- z$scores[1:nit, , drop=FALSE]
							}
						}
						else
						{
							z$sdf <- z$sdf[1:nit]
							z$sdf2 <- z$sdf2[1:nit]
						}
						z$sims <-z$sims[1:nit]
						z$Phase3nits <- nit
						z$n3 <- nit
						break
					}
					if (UserRestartFlag())
						break
				}
			}
		}
		if (!z$OK || UserRestartFlag())
		{
			return(z)
		}
	}
	z
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: siena07Gui.r
# *
# * Description: This module contains the code controlling the gui for siena07.
# *
# *****************************************************************************/
##@siena07Gui siena07 Create program screen for siena07
## This is called only if !batch.
siena07Gui <- function(tt, getDocumentation=FALSE)
{
    ##@myInterrupt internal siena07Gui
    myInterrupt<- function()
    {
        UserInterrupt(TRUE)
    }
    ##@myEndPhase2 internal siena07Gui
    myEndPhase2<- function()
    {
        EarlyEndPhase2(TRUE)
    }
    ##@myRestart internal siena07Gui
    myRestart<- function()
    {
        UserRestart(TRUE)
    }
    if (getDocumentation)
    {
        return(getInternals())
    }
    if (is.null(tt))
    {
        ## require(tcltk)
        tt <- tcltk::tktoplevel()
    }
    tcltk::tkwm.title(tt,'Siena07')
    frame <- tcltk::tkframe(tt, width=300, height=300, relief='ridge')
    tcltk::tkpack(frame, side='top', padx=5)
    button1 <- tcltk::tkbutton(frame, command=myInterrupt, text='Interrupt')
    button2 <- tcltk::tkbutton(frame, command=myEndPhase2, text='End Phase2',
                       state='disabled')
    button3 <- tcltk::tkbutton(frame, command=myRestart, text='Restart')
    tcltk::tkgrid.configure(button1, column=1, columnspan=2, row=1, padx=20, pady=20)
    tcltk::tkgrid.configure(button2, column=3, row=1, padx=20)
    tcltk::tkgrid.configure(button3, column=4, row=1, padx=20)
    phaselabel <- tcltk::tklabel(frame, text='Phase')
    subphaselabel <- tcltk::tklabel(frame, text='Subphase', state='disabled')
    iterationlabel <- tcltk::tklabel(frame, text='Iteration')
    label1 <- tcltk::tklabel(frame, text='ProgressBar')

    phase <- tcltk::tkentry(frame, width=2)

    subphase <- tcltk::tkentry(frame, width=2, state='disabled')
    iteration <- tcltk::tkentry(frame, width=6)
    progressbar <- tcltk::ttkprogressbar(frame, max=2000, length=120)

    tcltk::tkgrid.configure(phaselabel, column=1, row=2, pady=5)
    tcltk::tkgrid.configure(subphaselabel, column=2, row=2)
    tcltk::tkgrid.configure(iterationlabel, column=3, row=2)
    tcltk::tkgrid.configure(label1, column=4, row=2, padx=5)
    tcltk::tkgrid.configure(phase, column=1, row=3, pady=3)
    tcltk::tkgrid.configure(subphase, column=2, row=3, padx=10)
    tcltk::tkgrid.configure(iteration, column=3, row=3, padx=10)

    tcltk::tkgrid.configure(progressbar, column=4, padx=5, row=3)
    label2 <- tcltk::tklabel(frame, text='Current parameter values')
    label3 <- tcltk::tklabel(frame, text='Quasi-autocorrelations')
    label4 <- tcltk::tklabel(frame, text='Deviation values')

    tcltk::tkgrid.configure(label2, column=1, columnspan=2, row=4, padx=10)
    tcltk::tkgrid.configure(label3, column=3, row=4, padx=10)
    tcltk::tkgrid.configure(label4, column=4, row=4, padx=10)

    text1 <- tcltk::tktext(frame, height=6, width=14)

    text2 <- tcltk::tktext(frame, height=6, width=14)
    text3 <- tcltk::tktext(frame, height=6, width=14)
    tcltk::tkgrid.configure(text1, column=1, columnspan=2, row=5, padx=20, pady=5)

    tcltk::tkgrid.configure(text2, column=3, row=5, padx=20)
    tcltk::tkgrid.configure(text3, column=4, row=5, padx=20)
    ilcampo <- tcltk::tclVar()
    tcltk::tcl("image", "create", "photo", ilcampo, file=imagepath)
    frame2 <- tcltk::tkframe(tt, width=300, height=300, relief='ridge')
    tcltk::tkpack(frame2, side='bottom', padx=5)
    imgAsLabel <- tcltk::tklabel(frame2, image=ilcampo)
    tcltk::tkgrid.configure(imgAsLabel, pady=10)
    tcltk::tkinsert(phase, 0, ' 1')
    tcltk::tkgrab.set(tt)
    tcltk::tcl('update')
    tcltk::tkfocus(tt)
    list(tt=tt, pb=progressbar, earlyEndPhase2=button2, current=text1,
         quasi=text2, deviations=text3, phase=phase, subphase=subphase,
         iteration=iteration, subphaselabel=subphaselabel,
		 phaselabel=phaselabel)
}


#tcltk::tkconfigure(button2,state='normal')

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: print01Report.r
# *
# * Description: This module contains the function to print the initial report
# *****************************************************************************/
##@print01Report Reporting
print01Report <- function(data, modelname="Siena", getDocumentation=FALSE)
{
	##@reportDataObject1 internal print01Report
	reportDataObject1 <- function(x)
	{
		Report(c(x$observations, "observations,\n"), outf)
		##
		if (length(x$nodeSets) > 1)
		{
			Report("Node Sets:\n", outf)
			lapply(x$nodeSets, function(z)
			   {
				   Report(c(" ", format(attr(z, "nodeSetName"), width=15),
							":",
							format(length(z), width=3), "nodes\n"), outf)
			   })
			Report("\n", outf)
		}
		else
		{
			Report(c(length(x$nodeSets[[1]]), "actors\n"), outf)
		}
	}
	##@reportDataObject internal print01Report
	reportDataObject <- function(x, periodFromStart=0, multi=FALSE)
	{
		##@reportStart internal print01Report
		reportStart <- function()
		{
			multipleNodeSets <- length(x$nodeSets) > 1
			if (multipleNodeSets)
			{
				Report("Dependent variables	  Type		 NodeSet(s) (R, C)\n",
					   outf)
				Report("-------------------	  ----		 -----------------\n",
					   outf)
				for (i in seq_along((x$depvars)))
				{
					atts <- attributes(x$depvars[[i]])
					Report(c(format(atts$name, width=20),
							 format(atts$type, width=12)), outf)
					for (j in seq_along((atts$nodeSet)))
					{
						if (j > 1)
						{
							Report(', ', outf)
						}
						Report(c(format(atts$nodeSet[j]),
								 " (", atts$netdims[j], ")"), sep="", outf)
					}
					Report("\n", outf)
				}
			}
			else
			{
				Report(c(x$observations, "observations,\n"), outf)
				Report(c(length(x$nodeSets[[1]]), "actors,\n"), outf)
				Report(c(sum(types=="oneMode"),
						 "dependent network variables,\n"),
					   outf)
				Report(c(sum(types=="bipartite"),
						 "dependent bipartite variables,\n"), outf)
				Report(c(sum(types=="behavior"),
						 "dependent discrete behavior variables,\n"),
					   outf)
				Report(c(sum(types=="continuous"),
						 "dependent continuous behavior variables,\n"),
					   outf)
			}
			Report(c(length(x$cCovars), "constant actor covariates,\n"), outf)
			Report(c(length(x$vCovars),
					 "exogenous changing actor covariates,\n"), outf)
			Report(c(length(x$dycCovars), "constant dyadic covariates,\n"),
				   outf)
			Report(c(length(x$dyvCovars),
					 "exogenous changing dyadic covariates,\n"), outf)
			Report(c(length(x$compositionChange),
					c('no files','file',
						'files')[1 + as.numeric(length(x$compositionChange))],
					 "with times of composition change.\n"), outf)
			if ((length(x$cCovars) > 0 || length(x$dycCovars) > 0) && multi)
			{
				Report(c("For multi-group projects, constant covariates are",
						 "treated as changing covariates.\n"), outf)
				if (length(x$dycCovars) > 0)
				{
					Report(c("Note that missings in changing dyadic",
							 "covariates are not (yet) supported!\n"), outf)
				}
			}
		 Report("\n", outf)
		}

		##@reportNetworks internal print01Report
		reportNetworks <- function()
		{
			Heading(2, outf, "Reading network variables.")
			anymissings <- FALSE
			for (i in seq_along((x$depvars)))
			{
				depvar <- x$depvars[[i]]
				atts <- attributes(depvar)
				netname <- atts$name
				type <- atts$type
				if (!(type %in% c("behavior", "continuous")))
				{
					Report("Name of ", outf)
					if (nNetworks > 1)
					{
						Report("this ", outf)
					}
					Report(c("network variable: ", netname, '.\n'),
						   sep="", outf)
					Report(c(type, "network.\n"), outf)
					if (type == "bipartite")
					{
						Report("This is a two-mode network.\n", outf)
						Report(c("The number of units in the second mode is ",
								 atts$netdims[2], ".\n"), sep="", outf)
					}
					for (k in 1:x$observations)
					{
						Report(c("For observation moment ", k + periodFromStart,
								 ", degree distributions are as ",
								 "follows:\nNodes\n"),
							   sep="", outf)
						## remove structurals ? NA or 0/1
						if (attr(depvar, "sparse"))
						{
						   # require(Matrix)
							tmpdepvar <- depvar[[k]]
							tmpx1 <- tmpdepvar@x
							use <- tmpx1 %in% c(10, 11)
							tmpx1[use] <- tmpx1[use] - 10
							tmpdepvar@x <- tmpx1
							outdeg <- rowSums(tmpdepvar, na.rm=TRUE)
							indeg <- colSums(tmpdepvar, na.rm=TRUE)
							diag(tmpdepvar) <- 0
							missrow <- rowSums(is.na(depvar[[k]]))
							misscol <- colSums(is.na(depvar[[k]]))
						}
						else
						{
							tmpdepvar <- depvar[, , k]
							use <- tmpdepvar %in% c(10, 11)
							tmpdepvar[use] <- tmpdepvar[use] - 10
							if (attr(depvar, "type") != "bipartite")
							{
								diag(tmpdepvar) <- 0
							}
							outdeg <- rowSums(tmpdepvar, na.rm=TRUE)
							indeg <- colSums(tmpdepvar, na.rm=TRUE)
							missrow <- rowSums(is.na(tmpdepvar))
							misscol <- colSums(is.na(tmpdepvar))
						}
						if (attr(depvar, "type") == "bipartite")
						{
							tmp <- format(cbind(1:atts$netdims[1], outdeg))
							tmp2 <- format(cbind(1:atts$netdims[2], indeg))
						}
						else
						{
							tmp <- format(cbind(1:atts$netdims[1], outdeg,
												indeg))
						}

						Report(tmp[, 1], fill=60, outf)
						Report("out-degrees\n", outf)
						Report(tmp[, 2], fill=60, outf)
						if (attr(depvar, "type") == "bipartite")
						{
							Report("in-degrees\n", outf)
							Report(tmp2[, 2], fill=60, outf)
						}
						else
						{
							Report("in-degrees\n", outf)
							Report(tmp[, 3], fill=60, outf)
						}
						## report structural values
						if (attr(depvar, "structural"))
						{
							if (attr(depvar, "sparse"))
							{
								nstruct0 <- sum(depvar[[k]]@x %in% c(10))
								nstruct1 <- sum(depvar[[k]]@x %in% c(11))
							}
							else
							{
								nstruct0 <- sum(depvar[, , k] %in% c(10))
								nstruct1 <- sum(depvar[, , k] %in% c(11))
							}
							if (nstruct0 + nstruct1 > 0)
							{
								Report(c("\nThe input file contains codes for ",
										 "structurally determined values:\n"),
									   sep="", outf );
								if (attr(depvar, "sparse"))
								{
									nstruct0 <- sum(depvar[[k]]@x %in% c(10))
									nstruct1 <- sum(depvar[[k]]@x %in% c(11))
								}
								else
								{
									nstruct0 <- sum(depvar[, , k] %in% c(10))
									nstruct1 <- sum(depvar[, , k] %in% c(11))
								}
								Report(c('  ', nstruct0, ' structural zero'),
									sep='', outf)
								Report(ifelse(nstruct0 > 1,
										"s were found (code 10).\n",
										" was found (code 10).\n"), outf)
								Report(c('  ', nstruct1, ' structural one'),
									sep='', outf)
								Report(ifelse(nstruct1 > 1,
										"s were found (code 11).\n",
										" was found (code 11).\n"),
									outf)
								##
								if (attr(depvar, 'sparse'))
								{
									nnonactive <-
										rowSums(depvar[[k]] == 10 |
												depvar[[k]] == 11, na.rm=TRUE)
									nnonactive <- nnonactive >= nrow(depvar[[k]])
								}
								else
								{
									nnonactive <-
										rowSums(depvar[, , k] == 10 |
												depvar[, , k] == 11, na.rm=TRUE)
									nnonactive <- nnonactive >=
										nrow(depvar[, , k])
								}
								if (sum(nnonactive)	 == 1)
								{
									Report(c("Actor ", which(nnonactive),
											 " is inactive at this ",
											 "observation.\n"), sep='', outf)
								}
								else if (sum(nnonactive) > 1)
								{
									Report(c("Actors", which(nnonactive),
											 "are inactive at this",
											 "observation.\n"), fill=80, outf)
								}
							}
						}
						if (attr(depvar, "sparse"))
						{
							depvark <- depvar[[k]]
							diag(depvark) <- 0
							anymissings <- any(is.na(depvark))
						}
						else
						{
							depvark <- depvar[, , k]
							diag(depvark) <- 0
							anymissings <- any(is.na(depvark))
						}
						if (anymissings)
						{
							Report(c("\nFor observation moment ",
									 k + periodFromStart,
									 ", number of missing values ",
									 "are:\n"),
								   sep="", outf)
							if (attr(depvar, "type") == "bipartite")
							{
								Report("Senders\n", outf)
								tmp <- format(cbind(1:atts$netdims[1],
													missrow))
								Report(tmp[, 1], fill=60, outf)
								Report("missing in rows\n", outf)
								Report(tmp[, 2], fill=60, outf)
								tmp <- format(cbind(1:atts$netdims[2],
													misscol))
								Report("Receivers\n", outf)
								Report(tmp[, 1], fill=60, outf)
								Report("missing in columns\n", outf)
								Report(tmp[, 2], fill=60, outf)
								mult <- atts$netdims[2]
							}
							else
							{
								Report("Nodes\n", outf)
								tmp <- format(cbind(1:atts$netdims[1],
													missrow, misscol))
								Report(tmp[, 1], fill=60, outf)
								Report("missing in rows\n", outf)
								Report(tmp[, 2], fill=60, outf)
								Report("missing in columns\n", outf)
								Report(tmp[, 3], fill=60, outf)
								mult <- atts$netdims[1] - 1
							}
							Report(c("Total number of missing data: ",
									 sum(missrow),
									 ", corresponding to a fraction of ",
									 format(round(sum(missrow)/
												  atts$netdims[1] /
												  mult, 3),
											nsmall=3),
									 ".\n"), sep="", outf)

							if (k > 1)
								Report(c("In reported in- and outdegrees,",
										 "missings are not counted.\n"), outf)
							Report("\n", outf)
						}
						else
						{
							Report(c("\nNo missing data for observation ",
									 k + periodFromStart, ".\n\n"),
								   sep= "", outf)
						}
					}
					if (anymissings)
					{
						Report(c("There are missing data for this",
							   "network variable,\n"), outf)
						Report(c("and the <<carry missings forward>>",
							   "option is active.\n"), outf)
						Report("This means that for each tie variable,\n", outf)
						Report(c("the last previous nonmissing value (if any)",
								 "is imputed.\n"), outf)
						Report(c("If there is no previous nonmissing value,",
								 "the value 0 is imputed.\n"), outf)
					}
				}
			Report("\n", outf)
			}
			Report("\n", outf)
		}
		##@reportBehaviors internal print01Report
		reportBehaviors <- function()
		{
			Heading(2, outf, "Reading dependent actor variables.")
			iBehav <- 0
			for (i in seq_along((x$depvars)))
			{
				if (types[i] %in% c("behavior", "continuous"))
				{
					depvar <- x$depvars[[i]]
					atts <- attributes(depvar)
					netname <- atts$name

					iBehav <- iBehav + 1
					mystr <- paste(iBehav, switch(as.character(iBehav),
												  "1"=, "21"=, "31"= "st",
												  "2"=, "22"=, "32"= "nd",
												  "3"=, "23"=, "33"= "rd",
												  "th"), sep="")
					Report(c(mystr, " dependent actor variable named ",
							 netname,".\n"), sep="", outf)
					ranged <- atts$range2

					if (types[i] == "behavior")
						ranged <- round(ranged)
					else
						ranged <- signif(ranged, 4)							
					Report(c("Maximum and minimum ", 
							 ifelse(types[i] == "behavior", "rounded ", ""), 
							 "values are ", ranged[1], " and ", ranged[2], 
							 ".\n"), sep="", outf)
					if (types[i] == "behavior")
					{
					if (ranged[1] < 0 )
							stop("Negative minima not allowed for discrete ",
								 "dependent actor variables.\n")
					if (ranged[2] > 255 )
							stop("Maxima more than 255 not allowed for ",
								 "discrete dependent actor variables.\n")
					}
					if (ranged[1] >= ranged[2] )
						stop("Dependent actor variables must not be",
							 " constant.\n")
					if (any(is.na(depvar)))
					{
						Report(c("Missing values in this actor variable are",
								 "imputed",
								 "by the mode per observation.\n"), outf)
						Report(c("But if there is a previous (or later)",
								 "nonmissing value,",
								 "this is used as the imputed value.\n"), outf)
						Report("Modal values:\nObservation	", outf)
						Report(c(format(1:x$observations+periodFromStart,
										width=4), '\n'), outf)
						Report(c(format("Modes", width=12),
								 format(atts$modes, width=4)), outf)
						Report("\n", outf)
					}
					depvar2 <- depvar
					depvar2[is.na(depvar2)] <- 0
					if (types[i] == "behavior" &&
						!isTRUE(all.equal(as.vector(depvar2),
										  round(as.vector(depvar2)))))
					{
						 Report(c("Non-integer values noted in this behavior",
								  "variable: they will be truncated.\n")
								 , outf)
					}
					Report('\n', outf)
				}
			}
			Report(c("\nA total of",
					 nBehavs, "dependent actor variable"), outf)
			Report(ifelse(nBehavs > 1, "s.\n\n", ".\n\n"), outf)
			Report("Number of missing cases per observation:\n", outf)
			Report(c(" observation", format(1:x$observations+periodFromStart,
											width=10),
					 "		overall\n"), sep="", outf)
			for (i in seq_along((x$depvars)))
			{
				if (types[i] %in% c("behavior", "continuous"))
				{
					depvar <- x$depvars[[i]][, 1, ]
					atts <- attributes(x$depvars[[i]])
					netname <- atts$name
					missings <- colSums(is.na(depvar))
					Report(c(format(netname, width=12),
							 format(c(missings, sum(missings)),
									width=10), "	  (",
							 format(round(100 * sum(missings)/
										  nrow(depvar)/ncol(depvar), 1),
									nsmall=1, width=4), ' %)\n'), sep="", outf)
				}
			}
			Report("\nMeans per observation:\n", outf)
			Report(c(" observation", format(1:x$observations+periodFromStart,
											width=10),
					 "		overall\n"), sep="", outf)
			for (i in seq_along((x$depvars)))
			{
				if (types[i] %in% c("behavior", "continuous"))
				{
					depvar <- x$depvars[[i]][, 1, ]
					atts <- attributes(x$depvars[[i]])
					netname <- atts$name
					means <- colMeans(depvar, na.rm=TRUE)
					Report(c(format(netname, width=14),
							 format(round(means, 3), nsmall=3,
									width=10), format(round(mean(means),
									3), width=10), '\n'), sep="", outf)
				}
			}
		}
		##@reportConstantCovariates internal print01Report
		reportConstantCovariates <- function()
		{
			nCovars <- length(x$cCovars)
			covars <- names(x$cCovars)
			Heading(2, outf, "Reading constant actor covariates.")
			Report(c(nCovars, "variable"),outf)
			Report(ifelse(nCovars == 1, ", named:\n", "s, named:\n"), outf)
			for (i in seq(along=covars))
			{
				Report(c(format(covars[i], width=15), '\n'), outf)
			}
			Report(c("\nA total of", nCovars,
					 "non-changing individual covariate"), outf)
			Report(ifelse(nCovars == 1, ".\n\n", "s.\n\n"), outf)
			Report("Number of missing cases:\n", outf)
			for (i in seq(along=covars))
			{
				Report(c(format(covars[i], width=15),
						 sum(is.na(x$cCovars[[i]])), "	(",
						 format(round(100 * sum(is.na(x$cCovars[[i]]))/
									  length(x$cCovars[[i]]), 1),
								width=3, nsmall=1), '%)\n'), outf)
			}
			Report("\nInformation about covariates:\n", outf)
			Report(c(format("minimum  maximum	  mean  centered", width=48,
							justify="right"), "\n"), outf)
			any.cent <- 0
			any.noncent <- 0
			for (i in seq(along=covars))
			{
				atts <- attributes(x$cCovars[[i]])
				if (atts$centered)
				{
					cent <- "   Y"
					any.cent <- any.cent+1
				}
				else
				{
					cent <- "   N"
					any.noncent <- any.noncent+1
				}
				Report(c(format(covars[i], width=10),
						 format(round(atts$range2[1], 1),
								nsmall=1, width=8),
						 format(round(atts$range2[2], 1),
								nsmall=1, width=7),
						 format(round(atts$mean, 3),
								nsmall=3, width=10), cent, "\n"), outf)
			}
			if (nData <= 1)
			{
				if (any.noncent <= 0)
				{
					Report(c("The mean value", ifelse(nCovars == 1, " is", "s are"),
						" subtracted from the",
						ifelse(nCovars == 1, " centered", ""), " covariate",
						ifelse(nCovars == 1, ".\n\n", "s.\n\n")), sep="", outf)
				}
				else if (any.cent >= 1)
				{
					s.plural <- ""
					if (any.cent >= 2){s.plural <- "s"}
					Report(c("For the centered variable", s.plural,
					", the mean value", ifelse(any.cent == 1, " is", "s are"),
						" subtracted from the covariate", s.plural,
						".\n"), sep="", outf)
				}
			}
		}
		##@reportChangingCovariates internal print01Report
		reportChangingCovariates <- function()
		{
			nCovars <- length(x$vCovars)
			covars <- names(x$vCovars)
			use <- ! covars %in% names(x$cCovars)
			nCovars <- length(x$vCovars[use])
			Heading(2, outf, "Reading exogenous changing actor covariates.")
			Report(c(nCovars, "variable"),outf)
			Report(ifelse(nCovars == 1, ", named:\n", "s, named:\n"), outf)
			for (i in seq(along=covars[use]))
			{
				Report(c(format(covars[use][i], width=15), '\n'), outf)
			}
			Report(c("\nA total of", nCovars,
					 "exogenous changing actor covariate"), outf)
				Report(ifelse(nCovars == 1, ".\n\n", "s.\n\n"), outf)
			Report("Number of missing cases per period:\n", outf)
			Report(c(" period             ", format(1:(x$observations - 1) +
										 periodFromStart, width=8),
					 "     overall\n"), sep="", outf)
			for (i in seq(along=covars))
			{
				if (use[i])
				{
					thiscovar <- x$vCovars[[i]] ## matrix
					misscols <- colSums(is.na(thiscovar))
					Report(c(format(covars[i], width=20),
							 format(misscols, width=7),
							 format(sum(misscols), width=8), "	   (",
							 format(round(100 * sum(misscols)/nrow(thiscovar)/
										  ncol(thiscovar), 1), nsmall=1,
									width=3), '%)\n'), outf)
				}
			}
			Report("\nInformation about changing covariates:\n\n", outf)
			Report(c(format("minimum  maximum	  mean  centered", width=48,
							justify="right"), "\n"), outf)
			any.cent <- 0
			any.noncent <- 0
			for (i in seq(along=covars))
			{
				if (use[i])
				{
					atts <- attributes(x$vCovars[[i]])
					if (atts$centered)
					{
						cent <- "   Y"
						any.cent <- any.cent+1
					}
					else
					{
						cent <- "   N"
						any.noncent <- any.noncent+1
					}
					Report(c(format(covars[i], width=39), cent, '\n'), outf) # name
					for (j in seq_len(ncol(x$vCovars[[i]])))
					{
						Report(c("	period", format(j + periodFromStart,
												   width=3),
								 format(round(atts$rangep[1, j], 1),
										nsmall=1, width=7),
								 format(round(atts$rangep[2, j], 1),
										nsmall=1, width=7),
								 format(round(atts$meanp[j], 3),
										nsmall=3, width=10), "\n"), outf)
					}
					Report(c(format("Overall", width=29),
							 format(round(atts$mean, 3), width=10, nsmall=3),
							 "\n\n"), outf)
				}
			}
			if (nData <= 1)
			{
				if (any.noncent <= 0)
				{
					Report(c("The mean value", ifelse(nCovars == 1, " is", "s are"),
						" subtracted from the",
						ifelse(nCovars == 1, " centered", ""), " covariate",
						ifelse(nCovars == 1, ".\n\n", "s.\n\n")), sep="", outf)
				}
				else if (any.cent >= 1)
				{
					s.plural <- ""
					if (any.cent >= 2){s.plural <- "s"}
					Report(c("For the centered variable", s.plural,
					", the mean value", ifelse(any.cent == 1, " is", "s are"),
						" subtracted from the covariate", s.plural,
						".\n"), sep="", outf)
				}
			}
		}
		##@reportConstantDyadicCovariates internal print01Report
		reportConstantDyadicCovariates <- function()
		{
			nCovars <- length(x$dycCovars)
			covars <- names(x$dycCovars)
			Heading(2, outf, "Reading constant dyadic covariates.")
			for (i in seq(along=covars))
			{
				Report(c("Dyadic covariate named ", covars[i], '.\n'),
					   sep="", outf)
			}
			Report(c("\nA total of", nCovars,
					 "dyadic individual covariate"), outf)
			Report(ifelse(nCovars == 1, ".\n\n", "s.\n\n"), outf)
			Report("Number of tie variables with missing data:\n", outf)
			for (i in seq(along=covars))
			{
				if (attr(x$dycCovars[[i]], "sparse"))
				{
					myvar <- x$dycCovars[[i]][[1]]
				}
				else
				{
					myvar <- x$dycCovars[[i]]
				}
				diag(myvar) <- 0
				Report(c(format(covars[i], width=30),
						 sum(is.na(myvar)), "  (",
						 format(round(100 * sum(is.na(myvar))/
									  (length(myvar) - nrow(myvar)), 1),
								width=3, nsmall=1), '%)\n'), outf)
			}
			Report("\nInformation about dyadic covariates:\n", outf)
			Report(c(format("minimum  maximum	  mean  centered", width=67,
							justify="right"), "\n"), outf)
			any.cent <- 0
			any.noncent <- 0
			for (i in seq(along=covars))
			{
				atts <- attributes(x$dycCovars[[i]])
				if (atts$centered)
				{
					cent <- "   Y"
					any.cent <- any.cent+1
				}
				else
				{
					cent <- "   N"
					any.noncent <- any.noncent+1
				}
				Report(c(format(covars[i], width=30),
						 format(round(atts$range2[1], 1),
								nsmall=1, width=8),
						 format(round(atts$range2[2], 1),
								nsmall=1, width=7),
						 format(round(atts$mean, 3),
								nsmall=3, width=10), cent, "\n"), outf)
			}
			Report('\n', outf)

			s.plural <- ifelse((any.cent >= 2),"s","")
			if (any.noncent >= 1)
			{
				Report(c('The <mean> listed for the non-centered variable',
						s.plural, ' is the attribute, not the observed mean.',
						'\n'), sep="", outf)
			}
			if (any.noncent <= 0)
			{
				Report(c("The mean value", ifelse(nCovars == 1, " is", "s are"),
					" subtracted from the",
					ifelse(nCovars == 1, " centered", ""), " covariate",
					ifelse(nCovars == 1, ".\n\n", "s.\n\n")), sep="", outf)
			}
			else if (any.cent >= 1)
			{
				Report(c("For the centered variable", s.plural,
				", the mean value", ifelse(any.cent == 1, " is", "s are"),
					" subtracted from the covariate", s.plural,
					".\n"), sep="", outf)
			}
		}

		##@reportChangingDyadicCovariates internal print01Report
		reportChangingDyadicCovariates <- function()
		{
			covars <- names(x$dyvCovars)
			use <- ! covars %in% names(x$dycCovars) ## need an attributes to say
			nCovars <- length(x$dyvCovars[use])
			Heading(2, outf, "Reading exogenous dyadic covariates.")
			## Report(c("Note that no missing values are considered yet for",
			##			"changing dyadic covariates.\n"), outf)
			for (i in seq(along=covars))
			{
				Report(c("Exogenous dyadic covariate named ", covars[i], '.\n'),
					   sep="", outf)
			}
			Report("Number of tie variables with missing data per period:\n",
				   outf)
			Report(c(" period   ", format(1:(x$observations - 1) +
										  periodFromStart, width=7),
					 "      overall\n"), sep="", outf)
			for (i in seq(along=covars))
			{
				if (use[i])
				{
					sparse <- attr(x$dyvCovars[[i]], "sparse")
					vardims <- attr(x$dyvCovars[[i]], "vardims")
					thiscovar <- x$dyvCovars[[i]] ## array/list of sparse mats
					if (!sparse)
					{
						missvals <- colSums(is.na(thiscovar), dims=2)
					}
					else
					{
						missvals <- sapply(thiscovar, function(x)sum(is.na(x)))
					}
					Report(c(format(covars[i], width=10),
							 format(missvals, width=6),
							 format(sum(missvals), width=9), "	   (",
							 format(round(100 * sum(missvals)/vardims[1]/
										  vardims[2]), nsmall=1,
										  width=3), '%)\n'), outf)
				}
			}
			Report("\nInformation about changing dyadic covariates:\n", outf)
			Report(c(format("mean     centered", width=36,
							justify="right"), "\n"), outf)
			any.cent <- 0
			any.noncent <- 0
			for (i in seq(along=covars))
			{
				atts <- attributes(x$dyvCovars[[i]])
				if (atts$centered)
				{
					cent <- "   Y"
					any.cent <- any.cent+1
				}
				else
				{
					cent <- "   N"
					any.noncent <- any.noncent+1
				}
				Report(c(format(covars[i], width=28), cent, '\n'), outf) # name
				for (j in 1:(atts$vardims[3]))
				{
					Report(c("	period", format(j + periodFromStart,
											   width=3),
							 format(round(atts$meanp[j], 3),
									nsmall=3, width=10), "\n"), outf)
				}
				if (!atts$centered) # else atts$mean is 0
				{
					Report(c(format("Overall", width=29),
						 format(round(atts$mean, 3), width=10, nsmall=3),
						 "\n"), outf)
				}
				Report("\n", outf)
			}
			Report('\n', outf)
			s.plural <- ifelse((any.cent >= 2),"s","")
			if (any.noncent >= 1)
			{
				Report(c('The <mean> listed for the non-centered variable',
						s.plural, ' is the attribute, not the observed mean.',
						'\n'), sep="", outf)
			}
			if (nCovars >= 1)
			{
				if (any.noncent <= 0)
				{
					Report(c("The mean value",
						ifelse(nCovars == 1, " is", "s are"),
						" subtracted from the",
						ifelse(nCovars == 1, " centered", ""), " covariate",
						ifelse(nCovars == 1, ".\n\n", "s.\n\n")), sep="", outf)
				}
				else if (any.cent >= 1)
				{
					Report(c("For the centered variable", s.plural,
					", the mean value", ifelse(any.cent == 1, " is", "s are"),
						" subtracted from the covariate", s.plural,
						".\n"), sep="", outf)
				}
			}
		}

		##@reportCompositionChange internal print01Report
		reportCompositionChange <- function()
		{
			comps <- x$compositionChange
			Heading(2, outf, "Reading files with times of composition change.")
			for (i in seq(along=comps))
			{
				nodeSet <- attr(comps[[i]], "nodeSet")
				Report(c("\nComposition changes for nodeSet ", nodeSet, '.\n\n'),
					   sep="", outf)
				events <- attr(comps[[i]], "events")
				for (j in seq_len(nrow(events)))
				{
					x <- events[j, ]
					Report(c("Actor ", format(x$actor, width=2),
							 ifelse(x$event=="join", " joins ", " leaves"),
							 " network at time ",
							 format(round(x$period + x$time, 4), nsmall=4),
							 ".\n"), sep="", outf)
				}
				pertab <- table(events$period, events$event)
				for (period in row.names(pertab))
				{
					joiners <- pertab[period, "join"]
					leavers <- pertab[period, "leave"]
					Report(c("\nIn period ", period, ", ", joiners,
							 ifelse(joiners == 1, " actor", " actors"),
							 " joined and ", leavers,
							 ifelse(leavers == 1, " actor", " actors"),
							 " left the network.\n"), sep="", outf)
				}
			}
		}
		types <- lapply(x$depvars, function(z) attr(z, "type"))
		reportStart()
		nNetworks <- sum(types != "behavior")
		nBehavs <- sum(types %in% c("behavior", "continuous"))
		if (nNetworks > 0)
		{
			reportNetworks()
		}
		if (nBehavs > 0)
		{
			reportBehaviors()
		}
		if (length(x$cCovars) > 0)
		{
			reportConstantCovariates()
		}
		if (nData > 1 && length(x$vCovars) > length(x$cCovars) ||
			(nData ==1	&& length(x$vCovars) > 0))
		{
			reportChangingCovariates()
		}
		if (length(x$dycCovars) > 0)
		{
			reportConstantDyadicCovariates()
		}
		if (nData > 1 && length(x$dyvCovars) > length(x$dycCovars) ||
			(nData ==1	&& length(x$dyvCovars) > 0))
		{
			reportChangingDyadicCovariates()
		}
		if (length(x$compositionChange) > 0)
		{
			reportCompositionChange()
		}
		Report("\n\n", outf) ## end of reportDataObject
	}
	## create output file. ## start of print01Report proper
	if 	(!(inherits(data, "sienadata") | inherits(data, "siena")))
	{
		stop("The first argument needs to be a siena data object.")
	}
	if (is.null(attr(data, "dataname"))) 
	{
		dataname <- modelname
	}
	else # then the call comes from write_report.sienadata
	{
		dataname <- attr(data, "dataname")
	}
	if (!(inherits(modelname, "character")))
	{
		cat("Since version 1.1-279, an effects object should not be given\n")
		cat(" in the call of print01Report. Consult the help file.\n")
		stop("print01Report needs no effects object.")
	}
	if (!inherits(getDocumentation, 'logical'))
	{
		stop('wrong parameters; note: do not include an effects object as parameter!')
	}
	if (getDocumentation)
	{
		tt <- getInternals()
		return(tt)
	}
	Report(openfiles=TRUE, type="w", projname=modelname)
	Report("							************************\n", outf)
	Report(c("									 ", modelname, ".txt\n"),
		sep='', outf)
	Report("							************************\n\n", outf)
	Report(c("Filename is ", modelname, ".txt.\n\n"), sep="", outf)
	Report(c("This file contains basic descriptives for SIENA data set <<",
		dataname, ">>.\n\n"), sep="", outf)
	Report(c("Date and time:", format(Sys.time(), "%d/%m/%Y %X"), "\n\n"), outf)
	packageValues <- packageDescription(pkgname, fields=c("Version", "Date"))
	Report(c(paste(pkgname, "version "), packageValues[[1]],
			" (", format(as.Date(packageValues[[2]]), "%d %m %Y"), ")",
			"\n\n"), sep="", outf)

	if (!inherits(data, 'sienaGroup'))
	{
		nData <- 1
		data <- sienaGroupCreate(list(data), singleOK=TRUE)
	}
	else
	{
		nData <- length(data)
	}
	if (nData > 1)
	{
		Report("Multi-group input detected\n\n", outf)
		for (i in seq_len(nData))
		{
			Report(c("Subproject ", i, ": <", names(data)[i], ">\n"), sep="",
				   outf)
			reportDataObject1(data[[i]])
		}
		Report(c("Multi-group project", modelname, "contains", nData,
				 "subprojects.\n\n"), outf)
		periodFromStart <- 0
		for (i in seq_len(nData))
		{
			Heading(1, outf,
					paste("Subproject ", i, ": <", names(data)[i], ">",
						  sep="", collapse="")
					)
			reportDataObject(data[[i]], periodFromStart, multi=TRUE)
			periodFromStart <- periodFromStart + data[[i]]$observations
	   }
	}
	else
	{
		Heading(1, outf, "Data input.")
		reportDataObject(data[[1]], 0, multi=FALSE)
	}
	atts <- attributes(data)
	nets <- !(atts$types %in% c("behavior", "continuous"))
	behs <- atts$types == "behavior"
	if (length(data) > 1)
	{
		Heading(1, outf, "Further processing of multi-group data.")
		Report("Series of observations for the multi-group project:\n", outf)
		periodFromStart <- 0
		for (i in seq(along=data))
		{
			Report(c(format(1:data[[i]]$observations + periodFromStart), '\n'),
				   outf)
			periodFromStart <- periodFromStart + data[[i]]$observations
		}
		Report("\n", outf)
		if (length(atts$vCovars) == 1)
		{
			Report(c("The overall mean value ",
				format(round(atts$vCovarMean, 4), nsmall=3, width=12),
					 " is subtracted from covariate ", atts$vCovars,
					  ".\n\n"), sep="", outf)
		}
		else if (length(atts$vCovars) >= 2)
		{
			Report(c("The mean values are subtracted from the covariates:\n"), outf)
			for (i in seq(along=atts$vCovars))
			{
				Report(c(format(atts$vCovars[i], width=15),
				format(round(atts$vCovarMean[i], 4), nsmall=3, width=12), '\n'), outf)
			}
		}
	}
	periodNos <- attr(data, "periodNos")
	if (any(atts$anyUpOnly[nets]))
	{
		netnames <- atts$netnames[nets]
		upOnly <- atts$anyUpOnly[nets]
		allUpOnly <- atts$allUpOnly[nets]
		for (i in which(upOnly))
		{
			if (sum(nets) > 1)
			{
				Report(c("Network ", netnames[i], ":\n"), sep = "", outf)
			}
			if (allUpOnly[i])
			{
				Report("All network changes are upward.\n", outf)
				Report("This will be respected in the simulations.\n", outf)
				Report("Therefore, there is no outdegree parameter.\n\n", outf)
			}
			else
			{
				Report(c("All network changes are upward for the following",
						 "periods:\n"), outf)
				periodsUp <- unlist(lapply(data, function(x)
					{
						attr(x$depvars[[match(netnames[i], names(x$depvars))]],
							"uponly")
					}))
				periods <- periodNos[(seq_along(periodsUp))[periodsUp]]
				Report(paste(periods, " => ", periods + 1, ";",
							 sep=""), fill=80, outf)
				Report("This will be respected in the simulations.\n\n", outf)
			}
		}
	}
	if (any(atts$anyDownOnly[nets]))
	{
		netnames <- atts$netnames[nets]
		downOnly <- atts$anyDownOnly[nets]
		allDownOnly <- atts$allDownOnly[nets]
		for (i in which(downOnly))
		{
			if (sum(nets) > 1)
			{
				Report(c("Network ", netnames[i], ":\n"), sep = "", outf)
			}
			if (allDownOnly[i])
			{
				Report("All network changes are downward.\n", outf)
				Report("This will be respected in the simulations.\n", outf)
				Report("Therefore, there is no outdegree parameter.\n\n", outf)
			}
			else
			{
				periodsDown <-
					unlist(lapply(data, function(x)
					   {
						   attr(x$depvars[[match(netnames[i],
												 names(x$depvars))]],
								"downonly")
					   }))
				Report(c("All network changes are downward for the",
						 "following periods:\n"), outf)
				periods <- periodNos[(seq_along(periodsDown))[periodsDown]]
				Report(paste(periods, " => ", periods + 1, ";",
							 sep=""), fill=80, outf)
				Report("This will be respected in the simulations.\n\n", outf)
			}
		}
	}
	if (any(atts$anyUpOnly[behs])) # only for discrete behavior
	{
		netnames <- atts$netnames[behs]   
		upOnlyAndBeh <- atts$anyUpOnly[behs] 
		allUpOnly <- atts$allUpOnly[behs]
		for (i in which(upOnlyAndBeh))
		{
			Report(c("\nBehavior variable ", netnames[i], ":\n"), sep = "",
				   outf)
			if (allUpOnly[i])
			{
				Report("All behavior changes are upward.\n", outf)
				Report("This will be respected in the simulations.\n", outf)
				Report("Therefore, there is no linear shape parameter.\n\n",
						outf)
			}
			else
			{
				Report(c("All behavior changes are upward for the following",
						 "periods:\n"), outf)
				periodsUp <-
					sapply(data, function(x)
					   {
						   attr(x$depvars[[match(netnames[i],
												 names(x$depvars))]],
								"uponly")
					   })
				periods <- periodNos[(seq_along(periodsUp))[periodsUp]]
				Report(paste(periods, " => ", periods + 1, ";",
							 sep=""), fill=80, outf)
				Report("This will be respected in the simulations.\n\n", outf)
			}
		}
	}
	if (any(atts$anyDownOnly[behs]))
	{
		netnames <- atts$netnames[behs]
		downOnly <- atts$anyDownOnly[behs]
		allDownOnly <- atts$allDownOnly[behs]
		for (i in which(downOnly))
		{
			Report(c("\nBehavior ", netnames[i], ":\n"), sep = "", outf)
			if (allDownOnly[i])
			{
				Report("All behavior changes are downward.\n", outf)
				Report("This will be respected in the simulations.\n", outf)
				Report("Therefore, there is no linear shape parameter.\n\n",
						outf)
			}
			else
			{
				periodsDown <-
					sapply(data, function(x)
					   {
						   attr(x$depvars[[match(netnames[i],
												 names(x$depvars))]],
								"downonly")
					   })
				Report(c("All behavior changes are downward for the",
						 "following periods:\n"), outf)
				periods <- periodNos[(seq_along(periodsDown))[periodsDown]]
				Report(paste(periods, " => ", periods + 1, ";",
							 sep=""), fill=80, outf)
				Report("This will be respected in the simulations.\n\n", outf)
			}
	   }
	}
	if (any(atts$anyMissing[nets]))
	{
		netnames <- atts$netnames[nets]
		missings <- atts$anyMissing[nets]
		for (i in seq(along=netnames[missings]))
		{
			Report(c("There are missing data for network variable ",
					 netnames[i], ".\n"), sep = "", outf)
		}
	}
	 if (any(atts$anyMissing[!nets]))
	{
		netnames <- atts$netnames[!nets]
		missings <- atts$anyMissing[!nets]
		for (i in seq(along=netnames[missings]))
		{
			Report(c("There are missing data for behavior variable ",
					 netnames[i], ".\n"), sep = "", outf)
		}
	}

# The following is dropped in version 1.6.3 because the balance effect is hardly used.
	if ((sum(atts$types == 'oneMode') > 0) & (FALSE))
	{
		netnames <- atts$netnames[nets]
		if (nData > 1)
		{
			balmean <-
				lapply(data, function(x)
					   sapply(x$depvars, function(y) attr(y, "balmean")))
		}
		else
		{
			balmean <- atts$"balmean"
		}
		if (nData > 1 || sum(atts$types == "oneMode") > 1)
		{
			Report(c("The mean structural dissimilarity values subtracted",
					 "in the\n"), outf)
			Report("balance calculations are\n", outf)
		}
		else
		{
			Report(c("The mean structural dissimilarity value subtracted",
					 "in the\n"), outf)
			Report("balance calculations is ", outf)
		}
		for (i in seq(along=atts$types))
		{
			if (atts$types[i] == "oneMode")
			{
				if (nData > 1)
				{
					thisbalmean <- sapply(balmean, function(x)x[[netnames[i]]])
					##	if (sum(atts$types == "oneMode") > 1)
					if (sum(atts$types != "behavior") > 1)
					{
						Report(c("for network ", netnames[i],":"), sep="",
							   outf)
					}
					Report("\n", outf)
					mystr <- format(paste("Subproject ", 1:nData, " <",
								  atts$names, "> ", sep=""))
					for (j in seq(along=thisbalmean))
					{
						Report(c(mystr[j], ": ",
								 format(round(thisbalmean[j], 4), nsmall=4,
										width=14), "\n"), sep="", outf)
					}
				}
				else
				{
					##	if (sum(atts$types == "oneMode") > 1)
					if (sum(atts$types != "behavior") > 1)
					{
						Report(c("for network ", format(netnames[i], width=12),
								 format(round(balmean[i], 4),
										nsmall=4, width=14), '.\n'),
							   sep="", outf)
					}
					else
					{
						Report(c(format(round(balmean[i], 4), nsmall=4,
										width=14), '.\n'), sep="", outf)
					}
				}
			}
		}
	}
	if (sum(atts$types %in% c("behavior", "continuous")) > 0 ||
		(nData ==1 && length(atts$cCovars) > 0) ||
		length(atts$vCovars) > 0)
	{
		netnames <- atts$netnames
#		vCovarSim2 <-
#			lapply(data, function(x)
#				   lapply(x$vCovars, function(y) attr(y, "simMeans")))
#		behSim2 <-
#			lapply(data, function(x)
#				   lapply(x$depvars, function(y) attr(y, "simMeans")))
#		cCovarSim2 <-
#			lapply(data, function(x)
#				   lapply(x$cCovars, function(y) attr(y, "simMeans")))
		if (nData > 1)
		{
			vCovarSim <-
				lapply(data, function(x)
					   sapply(x$vCovars, function(y) attr(y, "simMean")))
			behSim <-
				lapply(data, function(x)
					   sapply(x$depvars, function(y) attr(y, "simMean")))
		}
		else
		{
			vCovarSim <- atts$"vCovarSim"
			behSim <- atts$"bSim"
		}
		Report(c("\nFor the similarity variable calculated from each actor",
				 "covariate,\nthe mean is subtracted.\nThese means are:\n"),
			   outf)
		if (nData == 1) ## ie we may have constant covariates
		{
			for (i in seq(along=atts$cCovars))
			{
				if (atts$cCovarPoszvar[i])
				{
					Report(c("Similarity", format(atts$cCovars[i], width=24),
							 ':', format(round(atts$cCovarSim[i], 4), width=12,
										 nsmall=4), '\n'), outf)
				}
			}
		}
		for (i in seq(along=atts$netnames))
		{
			if ((atts$types[i] %in% c("behavior", "continuous")) && atts$bPoszvar[i])
			{
				if (nData > 1)
				{
					thisSim <- sapply(behSim, function(x)x[[netnames[i]]])
					Report(c("Similarity ", format(atts$netnames[i], width=24),
							 ":\n"), sep="", outf)
					mystr <- format(paste("	 Subproject ", 1:nData, " <",
								  atts$names, "> ", sep=""))
					for (j in seq(along=thisSim))
					{
						Report(c(mystr[j], format(round(thisSim[j], 4),
												  nsmall=4, width=12), "\n"),
							   sep="", outf)
					}
					Report("\n", outf)
				}
				else
				{
					Report(c("Similarity", format(atts$netnames[i], width=24),
							 ':', format(round(atts$bSim[i], 4), nsmall=4,
										 width=12), '\n'), outf)
				}
			}
		}
		for (i in seq(along=atts$vCovars))
		{
			covarnames <- atts$vCovars
			if (atts$vCovarPoszvar[i])
			{
				if (nData > 1)
				{
					thisSim <- sapply(vCovarSim, function(x)x[[covarnames[i]]])
					Report(c("Similarity ", format(covarnames[i], width=24),
							 ":\n"), sep="", outf)
					mystr <- format(paste("	 Subproject ", 1:nData, " <",
										  atts$names, "> ", sep=""))
					for (j in seq(along=thisSim))
					{
						Report(c(mystr[j], format(round(thisSim[j], 4),
												  nsmall=4, width=12), "\n"),
							   sep="", outf)
					}
					Report("\n", outf)
				}
				else
				{
					Report(c("Similarity", format(atts$vCovars[i], width=24),
							 ':', format(round(atts$vCovarSim[i], 4), width=12,
										 nsmall=4), '\n'), outf)
				}
			}
		}
	}
	## report on constraints
	if (any(atts$anyHigher) || any(atts$anyDisjoint) || any(atts$anyAtLeastOne))
	{
		Report("\n", outf)
		highers <- atts[["anyHigher"]]
		disjoints <- atts[["anyDisjoint"]]
		atleastones <- atts[["anyAtLeastOne"]]
		if (any(highers))
		{
			higherSplit <- strsplit(names(highers)[highers], ",")
			lapply(higherSplit, function(x)
			   {
				   Report(c("Network ", x[1], " is higher than network ", x[2],
							".\n"), sep="", outf)
				   Report("This will be respected in the simulations.\n\n",
						  outf)
			  })
		}
		if (any(disjoints))
		{
			disjointSplit <- strsplit(names(disjoints)[disjoints],',')
			lapply(disjointSplit, function(x)
			   {
				   Report(c("Network ", x[1], " is disjoint from network ",
							x[2], ".\n"), sep="", outf)
				   Report("This will be respected in the simulations.\n\n",
						  outf)
			  })
		}
		if (any(atleastones))
		{
			atLeastOneSplit <- strsplit(names(atleastones)[atleastones],',')
			lapply(atLeastOneSplit, function(x)
			   {
				   Report(c("A link in at least one of networks ",
							x[1], " and", x[2],
						   " always exists.\n"), sep="", outf)
				   Report("This will be respected in the simulations.\n\n",
						  outf)
			  })
		}
	}
	myeff <- getEffects(data)
	attr(data, "dataname") <- dataname
	printInitialDescription(data, myeff, modelName=modelname)
	##close the files
	Report(closefiles=TRUE)
}

##@write_report Methods Reporting
write_report <- function(x, ...) UseMethod("write_report", x)

##@ write_report Interpretation method for siena data sets
write_report.sienadata <- function(x, outputName=NULL, ...)
{	
	dataName <- deparse(substitute(x))
# Check if it looks like a pipe chain 
	if (grepl("%>%|\\|>", dataName)) 
	{
		stop("Cannot auto-generate filename from piped data. Please provide filename explicitly.") 
	}
	if (is.null(outputName))
	{
		outputName <- paste(dataName, "_report", sep="")
	}	
# to get the dataName into print01Report
	attr(x, "dataname") <- dataName
	print01Report(data=x, modelname=outputName,	getDocumentation=FALSE)
	cat("Basic data description written to text file", 
						paste(outputName, ".txt. \n", sep=""))
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: maxlikec.r
# *
# * Description: This module contains the code for simulating the process,
# * communicating with C++. For use with maximum likelihood method, so
# * never conditional or from finite differences, or parallel testing.
# *****************************************************************************/
##@maxlikec siena07 ML Simulation Module
maxlikec <- function(z, x, data=NULL, effects=NULL,
                     returnChains=FALSE, byGroup=FALSE, byWave=FALSE,
					 returnDataFrame=FALSE,
					 returnLoglik=FALSE, onlyLoglik=FALSE)
{
# note: parameter x is not used. Just for consistency with other possibilities for FRAN.
    ## retrieve stored information
    f <- FRANstore()
    callGrid <- z$callGrid
    if (nrow(callGrid) == 1)
    {
		if (byGroup)
		{
			theta <- z$thetaMat[1,]
		}
		else
		{
			theta <- z$theta
		}
        ans <- .Call(C_mlPeriod, PACKAGE=pkgname, z$Deriv, f$pData,
                     f$pModel, f$myeffects, theta,
					 1, 1, z$nrunMH, z$addChainToStore,
                     z$returnDataFrame, z$returnDeps,
                     z$returnChains, returnLoglik, onlyLoglik)
		if (!onlyLoglik)
		{
			ans[[6]] <- list(ans[[6]])
			ans[[7]] <- list(ans[[7]])
			if (byGroup)
			{
				ans[[8]] <- list(ans[[8]])
				ans[[9]] <- list(ans[[9]])
				ans[[10]] <- list(ans[[10]])
				ans[[11]] <- list(ans[[11]])
			}
		}
		else
		{
			ans[[2]] <- list(ans[[2]])
			ans[[3]] <- list(ans[[3]])
			ans[[4]] <- list(ans[[4]])
		}
		if (z$returnDeps)
		{
			sims <- ans[[12]]
		}
		else
		{
			sims <- 'there are no simulated dependent variables'
		}
	}
    else
    {
		## z$int2 is the number of processors if iterating by period, so 1 means
		## we are not. Can only parallelize by period withmaxlike.
        if (z$int2 == 1)
        {
            anss <- apply(cbind(callGrid, 1:nrow(callGrid)),
						  1, doMLModel, z$Deriv, z$thetaMat,
                          z$nrunMH, z$addChainToStore,
                          z$returnDataFrame, z$returnDeps,
                          z$returnChains, byGroup, z$theta, returnLoglik,
						  onlyLoglik)
        }
        else
        {
            use <- 1:(min(nrow(callGrid), z$int2))
            anss <- parRapply(z$cl[use], cbind(callGrid, 1:nrow(callGrid)),
							  doMLModel, z$Deriv, z$thetaMat,
                              z$nrunMH, z$addChainToStore,
                              z$returnDataFrame,  z$returnDeps, z$returnChains, byGroup,
							  z$theta, returnLoglik, onlyLoglik)
        }
        ## reorganize the anss so it looks like the normal one
        ans <- list()
 		if (!onlyLoglik)
		{
			ans[[1]] <- sapply(anss, "[[", 1) ## statistics
			ans[[2]] <- NULL ## scores
			ans[[3]] <- NULL ## seeds
			ans[[4]] <- NULL ## ntim
			ans[[5]] <- NULL # randomseed
			if (z$returnChains)
			{
				fff <- lapply(anss, function(x) x[[6]][[1]])
				fff <- split(fff, callGrid[, 1 ]) ## split by group
				ans[[6]] <- fff
			}
			ans[[7]] <- lapply(anss, "[[", 7) ## derivative
			ans[[8]] <- lapply(anss, "[[", 8)
			ans[[9]] <- lapply(anss, "[[", 9)
			ans[[10]] <- lapply(anss, "[[", 10)
			if (!byGroup)
			{
				ans[[8]] <- Reduce("+",  ans[[8]]) ## accepts
				ans[[9]] <- Reduce("+",  ans[[9]]) ## rejects
				ans[[10]] <- Reduce("+",  ans[[10]]) ## aborts
			}
			ans[[11]] <- sapply(anss, "[[", 11)
			if (z$returnDeps)
			{
				fff <- lapply(anss, function(x) x[[12]])
				sims <- split(fff, callGrid[, 1 ]) ## split by group
			}
			else
			{
				sims <- 'no simulated dependent variables'
			}
		}
		else ##onlyLoglik is always byGroup (sienaBayes)
		{
			ans[[1]] <- sum(sapply(anss, '[[', 1))## loglik
			ans[[2]] <- lapply(anss, "[[", 2)
			ans[[3]] <- lapply(anss, "[[", 3)
			ans[[4]] <- lapply(anss, "[[", 4)
		}
    }
	
    FRANstore(f)

    if (z$Deriv && !onlyLoglik) ## need to reformat the derivatives
    {
        resp <- reformatDerivs(z, f, ans[[7]])
        dff <- resp[[1]]
        dff2 <- resp[[2]]
    }
    else
    {
        dff <- NULL
        dff2 <- NULL
    }
	if (!onlyLoglik)
	{
		fra <- -t(ans[[1]]) ##note sign change

		list(fra = fra, ntim0 = NULL, feasible = TRUE, OK = TRUE,
			 sims=sims, dff=dff, dff2=dff2,
			 chain = ans[[6]], accepts=ans[[8]],
			 rejects= ans[[9]], aborts=ans[[10]], loglik=ans[[11]])
	}
	else
	{
		list(loglik=ans[[1]], accepts=ans[[2]],
			 rejects= ans[[3]], aborts=ans[[4]])
	}
}

##@doMLModel Maximum likelihood
doMLModel <- function(x, Deriv, thetaMat, nrunMH, addChainToStore,
                      returnDataFrame, returnDeps, returnChains,
					  byGroup, theta, returnLoglik, onlyLoglik)
{
    f <- FRANstore()
	if (byGroup)
	{
		theta <- thetaMat[x[1], ]
	}
	else
	{
		theta <- theta
	}
    .Call(C_mlPeriod, PACKAGE=pkgname, Deriv, f$pData,
          f$pModel, f$myeffects, theta,
          as.integer(x[1]), as.integer(x[2]), nrunMH[x[3]], addChainToStore,
		  returnDataFrame, returnDeps, returnChains,
		  returnLoglik, onlyLoglik)
}

##@reformatDerivs Maximum likelihood move this back to inline or internal function
reformatDerivs <- function(z, f, derivList)
{
    ## current format is a list of vectors of the lower? triangle
    ## of the matrix. Need to be put into a symmetric matrix.
    ## Tricky part is getting the rates in the right place!
    ## Note that we do not yet deal with rate effects other than the basic
    dff <- as(matrix(0, z$pp, z$pp), "symmetricMatrix")
    nPeriods <- length(derivList)
	dff2 <- vector("list", nPeriods)
	if (z$byWave)
	{
		tmp <- as(matrix(0, z$pp, z$pp), "symmetricMatrix")
		dff2[] <- tmp
	}
    for (period in 1:nPeriods)
    {
        dffraw <- derivList[[period]]
        ## start indexes row/col for first effect for the variable
        start <- 1
        ## rawsub is subscript in the vector
        rawsub <- 1
        ## f$myeffects is a list of data frames per dependent variable
        ## of selected effects.
        for (i in 1:length(f$myeffects))
        {
            dffPeriod <- matrix(0, nrow=z$pp, ncol=z$pp)
            ## rows is the number of effects for this variable
            rows <- nrow(f$myeffects[[i]])
            ## nRates is the number of rate effects for this variable
            ## at present nRates will be the number of periods.
            nRates <- sum(f$myeffects[[i]]$type == 'rate')
            ## nonRates is the number of rows/cols in the objective function
            ## part of the derivative matrix dff.
            nonRates <- rows - nRates
            ## first put the basic rate for this variable in the right place
            dffPeriod[cbind(start:(start + nRates -1),
                            start:(start + nRates - 1))] <-
                                dffraw[rawsub:(rawsub + nRates - 1)]
            ##
            ## now the matrix of objective function effects
            ##
            start <- start + nRates
            ##
            rawsub <- rawsub + nRates
            ##
            dffPeriod[start : (start + nonRates - 1 ),
                      start : (start + nonRates - 1)] <-
                          dffraw[rawsub:(rawsub + nonRates * nonRates - 1)]
            start <- start + nonRates
            rawsub <- rawsub + nonRates * nonRates
            dffPeriod <- dffPeriod + t(dffPeriod)
            diag(dffPeriod) <- diag(dffPeriod) / 2
			if (z$byWave)
			{
				dff2[[period]] <- dff2[[period]] - dffPeriod
			}
            dff <- dff - dffPeriod
        }
    }
    list(dff, dff2)
}

##@getProbabilitiesFromC siena07 with maxLike gets loglik from chains in C
## new for rsiena version 1.3.18
# used in getLikelihoods
getProbabilitiesFromC <- function(z, index=1, getScores=FALSE)
{
	## expects maximum likelihood parallelisations
	f <- FRANstore()

	callGrid <- z$callGrid
	## z$int2 is the number of processors if iterating by period, so 1 means
	## we are not. Can only parallelize by period!
	if (nrow(callGrid) == 1)
	{
		theta <- z$thetaMat[1,]
		ans <- .Call(C_getChainProbabilities, PACKAGE = pkgname, f$pData,
					 f$pModel, as.integer(1), as.integer(1),
					 as.integer(index), f$myeffects, theta, getScores)
		anss <- list(ans)
	}
	else
	{
		if (z$int2 == 1 )
		{
			anss <- apply(callGrid, 1,
						  doGetProbabilitiesFromC, z$thetaMat, index, getScores)
		}
		else
		{
			use <- 1:(min(nrow(callGrid), z$int2))
			anss <- parRapply(z$cl[use], callGrid,
							  doGetProbabilitiesFromC, z$thetaMat, index,
							  getScores)
		}
	}
	ans <- list()
# It was the following - must be wrong
#	ans[[1]] <- sum(sapply(anss, "[[", 1))
	if (nrow(callGrid) != length(anss))
	{
		cat("Error: nrow(callGrid) = ", nrow(callGrid), "; length(anss) = ",
			length(anss),"\n")
		stop("Error in getProbabilitiesFromC")
	}
	# Sum the log probabilities for the periods corresponding to each group
# TODO: change the following sapply into vapply;
# what is the type of result???
	logprob <- sapply(anss, "[[", 1)
	ans[[1]] <- sapply(1:z$nGroup, function(i){sum(logprob[callGrid[,1]==i])})
	if (getScores)
	{
		ans[[2]] <- sapply(anss, "[[", 2) # it was rowSums of this
	}
	ans[[3]] <- sapply(anss, "[[", 3)
	ans
}


##@doGetProbabilitiesFromC Maximum likelihood
doGetProbabilitiesFromC <- function(x, thetaMat, index, getScores)
{
# thetaMat <- z$thetaMat # [1,]
# getScores <- TRUE
# x <- c(1,1)
	f <- FRANstore()
	theta <- thetaMat[x[1], ]
#	gcp <-
	.Call(C_getChainProbabilities, PACKAGE = pkgname, f$pData,
		  f$pModel, as.integer(x[1]), as.integer(x[2]),
		  as.integer(index), f$myeffects, theta, getScores)
}


##@getLikelihoods algorithms Get likelihoods from C for stored chainss
getLikelihoods <- function(theta, z, getScores=FALSE, iterSequence)
{
	z$thetaMat <- matrix(theta, nrow=1)
	if (z$maxlike || z$nbrNodes == 1)
	{
		if (missing(iterSequence))
		{
			iterSequence <- length(z$lik0) : 1
		}
		if (any(iterSequence < 0))
		{
			cat("length(z$lik0) = ", length(z$lik0), " in getLikelihoods\n")
			stop("Error in getLikelihoods")
		}
		anss <-	lapply(iterSequence, function(i, z)
				   getProbabilitiesFromC(z, i, getScores=getScores), z=z)
	}
	else
	{
		stop("getLikelihoods used in different case; look at algorithm.r")
#		if (missing(iterSequence))
#		{
#			blocksize <- ceiling(z$nIter / z$nbrNodes)
#			iterSequence <- blocksize : 1
#			iterSequence <- rep(iterSequence, z$nbrNodes)[1:z$nIter]
#		}
#		anss <- parLapply(z$cl, iterSequence, forwardGetProbabilitiesFromC,
#						z, getScores=getScores)
	}
# TODO: change the following sapply into vapply;
# what is the type of result???
	lik <- sapply(anss, "[[", 1)
	sc <- sapply(anss, "[[", 2)
	deriv <- sapply(anss, "[[", 3)
	list(lik=lik, sc=sc, deriv=deriv)
}

#/******************************************************************************
#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: printInitialDescription.r
# *
# * Description: This module contains the code to print details of the project.
# *
# *****************************************************************************/
##@printInitialDescription Reporting
printInitialDescription <- function(data, effects, modelName="Siena",
	getDocumentation=FALSE)
{
	##@initialNetworks internal printInitialDescription
	initialNetworks <- function()
	{
		Heading(2, outf,'Change in networks:')
		difficult <- FALSE
		for (net in which(types %in% c("oneMode", "bipartite")))
		{
			bipartite <- types[net] == "bipartite"
			if ((nOneMode + nBipartite) > 1)
			{
				Heading(3, outf, c("Network: ", netnames[net]))
			}

			Report(c("For the following statistics, missing values (if any)",
					"are not counted.\n"), outf)
			if (any(attr(data, "structural")))
			{
				Report(c("Further, structurally determined entries are",
						"treated as observed entries,\n"), outf)
				Report(c("except for calculating the distances, where missings",
						"and structurally determined entries\n"), outf)
				Report(c("at the beginning or end of a period",
						"are disregarded.\n"), outf)
				Report(c("This may imply that the columns 0 => 1 and 1 => 0",
						"do not add up to the distance column.\n"), outf)
			}
			## is this network symmetric?
			if (gpatts$symmetric[net])
			{
				Report(c(ifelse(nobs == 2, "Both", c("All ", nobs)),
						" observed networks are symmetric.\n"), outf)
				Report(c("Therefore, it is assumed that this is an",
						"analysis of an non-directed relation.\n"), outf)
			}
			Report("\nNetwork density indicators:\n", outf)
			density <- rep(NA, nobs)
			degree <- rep(NA, nobs)
			nties <- rep(NA, nobs)
			missings <- rep(NA, nobs)
			periodFromStart <- 0
			for (group in 1:nData)
			{
				j <- match(netnames[net], names(data[[group]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[group]]$depvars[[j]]
				atts <- attributes(depvar)
				subs <- 1:data[[group]]$observations + periodFromStart
				density[subs] <- atts$density
				if (any(atts$ones >= atts$nval, na.rm = TRUE))
				{
					difficult <- TRUE
				}
				degree[subs] <- atts$degree
				missings[subs] <-atts$missings
				nties[subs] <- atts$ones
				if (gpatts$symmetric[net])
				{
					nties <- nties / 2
				}
				periodFromStart <- periodFromStart + data[[group]]$observations
			}
			## now do the format
			tmp <- rbind(format(round(density, 3), nsmall=3, width=7),
				format(round(degree, 3), nsmall=3, width=7),
				format(nties, width=7),
				format(round(missings, 3), nsmall=3, width=7))
			startCol <- 1
			text <- c("observation time", "density", "average degree",
				"number of ties", "missing fraction")
			text <- c(format(text[1], width=24), format(text[-1], width=25))
			eols <- rep("\n", length(text))
			repeat
			{
				endCol <- startCol + 6
				endCol <- min(endCol, nobs)
				tmp3 <- rbind(format(startCol:endCol, width=7),
					tmp[, startCol:endCol, drop=FALSE])
				tmp2 <- cbind(text, tmp3, eols)
				Report(t(tmp2), sep="", outf)
				startCol <- startCol + 7
				if (startCol > nobs)
					break
			}
			averageOutDegree <- rep(NA, nData)
			for (group in 1:nData)
			{
				j <- match(netnames[net], names(data[[group]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[group]]$depvars[[j]]
				atts <- attributes(depvar)
				averageOutDegree[group] <- atts$"averageOutDegree"
			}
			Report("\n", outf)
			if (nData > 1)
			{
				Report("The average degrees over all waves are: \n ", outf)
				Report(paste(names(data), round(averageOutDegree, 3), "\n",
						sep=': '), outf)
				Report("\n", outf)
			}
			else
			{
				Report(c("The average degree is",
						round(averageOutDegree, 3), "\n"), outf)
			}
			Report("\n\n", outf)
			Report(c(ifelse(gpatts$symmetric[net], "Edge", "Tie"),
					"changes between subsequent observations:\n"), outf)
			valmin <- gpatts$netRanges[1, net]
			valmax <- gpatts$netRanges[2, net]
			tmp <- expand.grid(valmin:valmax, valmin:valmax)
			heads <- paste(tmp[, 2], "=> ", tmp[,1])
			Report(c(format(" periods", width=16),
					format(heads, width=10),
					ifelse(valmin == 0 && valmax == 1,
						"Distance Jaccard   Missing\n",
						"Distance     Missing\n")), sep = "", outf)
			periodFromStart <- 0

			for (group in 1:nData)
			{
				j <- match(netnames[net], names(data[[group]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[group]]$depvars[[j]]
				if (bipartite)
				{
					tmp <- getBipartiteStartingVals(depvar)
				}
				else
				{
					tmp <- getNetworkStartingVals(depvar)
				}
				atts <- attributes(depvar)
				matchange <- tmp$tmp[-(1:2), , drop=FALSE]
				## if symmetric: divide by 2
				if (gpatts$symmetric[net])
				{
					tmp$tmp <- tmp$tmp %/% 2
					matchange <- matchange %/% 2
				}
				for (per in 1:(atts$netdims[3] - 1))
				{
					ntot <- tmp$tmp["matcnt", per]
					if (gpatts$symmetric[net])
					{
						misd <- atts$netdims[1] * (atts$netdims[1] - 1) / 2 -
							ntot
					}
					else if (bipartite)
					{
						misd <- atts$netdims[1] * atts$netdims[2] - ntot
					}
					else
					{
						misd <- atts$netdims[1] * (atts$netdims[1] - 1) - ntot
					}
					if (valmin == 0 && valmax == 1)
					{
						jaccard <- format(round(matchange[4, per] /
								(matchange[4, per] +
									matchange[3, per] +
									matchange[2, per]), 3), nsmall=3,
							width=10)

						Report(c(format(per + periodFromStart, width=3),
								" ==> ",
								format(per + 1 + periodFromStart, width=3),
								format(matchange[, per], width=10),
								format(attr(depvar, "distance")[per],
									width=10),
								jaccard, format(misd, width=10), " (",
								round(100 * misd/(ntot + misd)), "%)\n"),
							sep="", outf)
					}
					else
					{
						Report(c(per + periodFromStart, " ==> ",
								format(per + 1 + periodFromStart, width=3),
								format(matchange[, per], width=10),
								format(attr(depvar,"distance")[per], width=10),
								format(misd, width=10), " (",
								round(100 * misd/(ntot + misd)), "%)\n"),
							sep="", outf)

					}
					if (valmin == 0 && valmax == 1)
					{
						if (matchange[4, per]*
							(matchange[2, per] + matchange[1, per])
						<
						matchange[2, per] *
							(matchange[3, per] + matchange[4, per]) )
						{
							Report(c("\nThis means that in period ", per,
									", proportionately fewer 1-ties stayed 1,\n",
									" than 0-ties became 1. A great reversal",
									" of the network pattern!\n",
									"For some model specifications this may",
									" lead to problems in estimation.\n"), outf)
						}
					}
				}
				periodFromStart <- periodFromStart + atts$netdims[3]
				Report("\n", outf)
			}
			if (gpatts$symmetric[net])
			{
				Report(c("The distances reported in the output file",
						"for conditional estimation\n",
						"for the network variable refer to the total",
						"symmetric adjacency matrix,\n",
						"and therefore are double the distance",
						"reported above.)\n\n"), outf)
			}
			if (!bipartite)
			{
				Report("Directed dyad Counts:\n", outf)
				if (valmin == 0 & valmax == 1)
				{
					Report(" observation    total    mutual    asymm.     null\n",
						outf)
				}
				else
				{
					Report(" observation    total \n", outf)
					heads <- expand.grid(valmin:valmax, valmin:valmax)
					heads <- heads[, c(2, 1)] ## reverse the column order
					heads <- heads[heads[,2] >= heads[,1], ] ## remove col2 < col1
					heads <- paste("(", heads[, 1], ",", heads[, 2], ")", sep="")
					Report(heads, outf)
					Report("\n", outf)
					## this one needs more work!
				}
				periodFromStart <- 0
				for (group in 1:nData)
				{
					j <- match(netnames[net], names(data[[group]]$depvars))
					if (is.na(j))
						stop("network names not consistent")
					depvar <- data[[group]]$depvars[[j]]
					atts <- attributes(depvar)

					for (per in 1:(atts$netdims[3]))
					{
						## find the dyad tables
						if (atts$sparse)
						{
							##  require(Matrix)
							mymat <- depvar[[per]]
							diag(mymat) <- 0
							mymat1 <- mymat@i
							mymat2 <- mymat@j
							mymat3 <- mymat@x
							if (any(duplicated(paste(mymat1, mymat2))))
								stop("sparse matrix has duplicate triples")
							mymat3[mymat3==10] <- 0
							mymat3[mymat3==11] <- 1
							mymat1 <- mymat1[is.na(mymat3) | mymat3 != 0]
							mymat2 <- mymat2[is.na(mymat3) | mymat3 != 0]
							mymat3 <- mymat3[is.na(mymat3) | mymat3 != 0]
							## for not 0/1 need to tabulate the equivs to mutuals
							## and asymms by
							## the non zero one.
							if (valmin == 0 && valmax ==1)
							{
								ij <- paste(mymat1[!is.na(mymat3)],
									mymat2[!is.na(mymat3)])
								ji <- paste(mymat2[!is.na(mymat3)],
									mymat1[!is.na(mymat3)])
								missij <- paste(mymat1[is.na(mymat3)],
									mymat2[is.na(mymat3)])
								missji <- paste(mymat2[is.na(mymat3)],
									mymat1[is.na(mymat3)])
								mutual <- sum(ij %in% ji) / 2
								## nondyads are ones where we have a link and
								## its partner is missing
								nondyads <- sum(ji %in% missij)
								asymm <- length(ij) - nondyads - mutual * 2
								missdyads <- sum(!(missij %in% missji)) +
									sum(missij %in% missji) / 2
								nulls <- atts$netdims[1] *
									(atts$netdims[2] - 1) / 2 -
									missdyads - mutual - asymm
								totDyad <- nulls + mutual + asymm
							}
						}
						else
						{
							mymat <- depvar[, , per]
							mymat[mymat == 10] <- 0
							mymat[mymat == 11] <- 1
							diag(mymat) <- NA
							tmymat <- t(mymat)
							# dyadTable <- table(mymat, t(mymat))
							# Changed to protect against zero rows or columns
							nulls <- sum((1 - mymat)*(1 - tmymat), na.rm=TRUE)
							asymm <- sum(mymat*(1 - tmymat), na.rm=TRUE) +
								sum((1 - mymat)*tmymat, na.rm=TRUE)
							mutual <- sum(mymat*tmymat, na.rm=TRUE)
							# diag(dyadTable) <- diag(dyadTable) / 2
							#if (valmin == 0 && valmax ==1)
							#{
							#    mutual <- dyadTable[2, 2]
							#    asymm <- dyadTable[2, 1]
							#    nulls <- dyadTable[1, 1]
							totDyad <- nulls + asymm + mutual
							#}
						}
						if (valmin == 0 && valmax == 1)
						{
							Report(c(format(per + periodFromStart, width=6),
									".", format(totDyad, width=14),
									format(mutual, width=9),
									format(asymm, width=10),
									format(nulls, width=10), "\n"),
								sep="", outf)
						}
					}
					periodFromStart <- periodFromStart + atts$netdims[3]
				}
			}
			if (difficult)
			{
				Report(c("There is a density equal to 0.0 or 1.0.\n",
						"This may lead to difficulties.\n"), outf)
			}
			Report("\n", outf)
			Report(c("Standard values for initial parameter values\n",
					"-------------------------------------------------\n\n"),
				sep = "", outf)
			myeff <- effects[effects$name == netnames[net],]
			myrate <- myeff[myeff$shortName == "Rate",]
			for (i in 1:nrow(myrate))
			{
				Report(c(format(myrate$effectName[i], width=35),
						format(round(myrate$initialValue[i], 4), nsmall=4,
							width=10), "\n"), outf)
			}
			myobj <- myeff[myeff$shortName == "density" &
				myeff$type == "eval",]
			if (nrow(myobj) > 0)
			{
				untrimmed <- myobj$untrimmedValue
				Report(c(format(myobj$effectName, width=46),
						format(round(untrimmed, 4), nsmall=4, width=10),
						"\n"), outf)
				if (untrimmed > 3)
				{
					Report(c("The initial parameter value is very low.",
							"It is truncated to -3. \n"), outf)
				}
				else
					if (untrimmed < -3)
					{
						Report(c("The initial parameter value is very high.",
								"It is truncated to 3.\n"), outf)
					}
			}
		}
		Report("\n", outf)
	}

	##@initialBehaviors internal printInitialDescription
	initialBehaviors <- function()
	{
		Report("\n", outf)
     		Heading(2, outf,"Dependent discrete actor variables:")
		if (nBehav == 1)
		{
			Report(c(netnames[types == "behavior"], "\n\n"), outf)
			Heading(3, outf, "Marginal distribution")
		}
		else
		{
			Heading(3, outf, "Marginal distributions")
		}
		for (net in which(types=="behavior"))
		{
			if (nBehav > 1)
			{
				Report(c("Dependent actor variable ", net, ": ",
						netnames[net], "\n"), sep="", outf)
			}
			Report(c(rep(" ", 3*nobs + 7), "Observations\n"), sep="", outf)
			Report(c(format("values", width=16),
					format(1 : nobs, width=6), "\n"), sep="", outf)
			Report(c(rep(" ", 16), rep("------", nobs), "\n"), sep="", outf)
			periodFromStart <- 0
			bRange <- gpatts$bRange[net]
			minval <- gpatts$behRange[, net][1]
			vals <- matrix(0, ncol=nobs, nrow=bRange + 1)
			missings <- rep(0, nobs)
			missingN <- rep(NA, nobs)
			for (group in 1:nData)
			{
				j <- match(netnames[net], names(data[[group]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[group]]$depvars[[j]]
				atts <- attributes(depvar)
				for (i in 1: atts$netdims[3])
				{
					mytab <- table(round(depvar[, 1, i]))
					vals[as.numeric(names(mytab)) + 1 - minval,
						periodFromStart + i] <- mytab
					# vals[factor(names(mytab)), periodFromStart + i] <- mytab
					missings[periodFromStart + i] <- sum(is.na(depvar[, 1, i]))
					missingN[periodFromStart + i] <- atts$netdims[1]
				}
				periodFromStart <- periodFromStart + atts$netdims[3]
			}
			for (i in 1:nrow(vals))
			{
				Report(c(format(i - 1 + minval, width=3), rep(" ", 13),
						format(vals[i, ], width=6), "\n"), sep="", outf)
			}
			if (gpatts$anyMissing[net])
			{
				Report(c(format("missing", width=16),
						format(missings, width=6), "\n\n"), sep="", outf)
				Report(c(format("(fraction missing"),
						format(round(missings / missingN, 2), nsmall=2,
							width=6), "  )\n"), sep="", outf)
			}
			else
			{
				Report("No missings\n", outf)
			}

		}
		Report("\n\n", outf)
		Heading(3, outf, "Changes")
		for (net in which(types=="behavior"))
		{
			if (nBehav > 1)
			{
				Report(c("Dependent actor variable ", net, ": ",
						netnames[net], "\n"), sep="", outf)
			}
			Report(c(" periods    actors:  down   up   constant  missing  ;",
					"   steps:   down    up  total\n"), sep="", outf)
			periodFromStart <- 0
			for (group in 1:nData)
			{
				j <- match(netnames[net], names(data[[group]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[group]]$depvars[[j]]
				atts <- attributes(depvar)
				for (i in 1:(atts$netdims[3] - 1))
				{
					mytab <- table(round(depvar[, 1, i + 1]) - 
						round(depvar[, 1, i]))
					numvals <- as.numeric(names(mytab))
					ups <- mytab[numvals > 0]
					downs <- mytab[numvals < 0]
					constants <- mytab[numvals == 0]
					stepsup <- sum(ups * numvals[numvals > 0])
					stepsdown <- sum(-1 * downs * numvals[numvals < 0])
					Report(c(format(i + periodFromStart, width=3),
							"  =>", format(i + 1 + periodFromStart, width=3),
							format(sum(downs), width=14),
							format(sum(ups), width=6),
							format(sum(constants), width=8),
							format(sum(is.na(depvar[, 1, i + 1]) |
									is.na(depvar[, 1, i])), width=10),
							format(stepsdown, width=21),
							format(stepsup, width=6),
							format(stepsup + stepsdown, width=6), "\n"),
						sep="", outf)
				}
				Report("\n", outf)
				periodFromStart <- periodFromStart + atts$netdims[3]
			}
			myeff <- effects[effects$name == netnames[net],]
			myobj <- myeff[myeff$shortName == "linear" &
				myeff$type == "eval",]
			Report(c("For this variable, the standard initial ",
					"behavioral tendency parameter is ",
					format(round(myobj$initialValue,4), nsmall=4,width=8),
					"\n"), sep="",
				outf)
		}

		Report("\n", outf)
	}
	
	##@initialContinuous internal printInitialDescription
	initialContinuous <- function()
	{
		Report("\n", outf)
		Heading(2, outf,"Dependent continuous actor variables:")
		Report("\n", outf)
		Heading(3, outf, "Changes")
		for (net in which(types=="continuous"))
		{
			Report(c("Dependent actor variable: ",
				netnames[net], "\n"), sep="", outf)
			Report(c(" periods    actors:  down   up   constant  missing  ;",
				" average:   down    up  total\n"), sep="", outf)
			periodFromStart <- 0
			for (group in 1:nData)
			{
				j <- match(netnames[net], names(data[[group]]$depvars))
				if (is.na(j))
					stop("network names not consistent")
				depvar <- data[[group]]$depvars[[j]]
				atts <- attributes(depvar)
				for (i in 1:(atts$netdims[3] - 1))
				{
					mytab <- table(depvar[, 1, i + 1] - depvar[, 1, i])
					numvals <- as.numeric(names(mytab))
					ups <- mytab[numvals > 0]
					downs <- mytab[numvals < 0]
					constants <- mytab[numvals == 0]
					averageup <- sum(ups * numvals[numvals > 0]) / length(ups)
					averagedown <- sum(downs * numvals[numvals < 0]) /
																  length(downs)
					averagechange <- sum(numvals) / length(numvals != 0) 
					Report(c(format(i + periodFromStart, width=3),
						"  =>", format(i + 1 + periodFromStart, width=3),
						format(sum(downs), width=14),
						format(sum(ups), width=6),
						format(sum(constants), width=8),
						format(sum(is.na(depvar[, 1, i + 1]) |
						is.na(depvar[, 1, i])), width=10),
						format(round(averagedown,2), width=21),
						format(round(averageup,2), width=6),
						format(round(averagechange,2), width=6), "\n"),
						sep="", outf)
					}
					Report("\n", outf)
					periodFromStart <- periodFromStart + atts$netdims[3]
				}
			}

		Report("\n", outf)
	}
	##### start of printInitialDescription function	
	if (is.null(attr(data, "dataname")))
	{
		dataname <- modelName
	}
	else # then the call comes originally from write_report.sienadata
	{
		dataname <- attr(data, "dataname")
	}
	if (getDocumentation)
	{
		return(getInternals())
	}
	if (!inherits(data, "sienaGroup"))
	{
		nData <- 1
		data <- sienaGroupCreate(list(data), singleOK=TRUE)
	}
	else
	{
		nData <- length(data)
	}
	Report("\n\n", outf)
	Heading(1, outf, "Initial data description.")
	gpatts <- attributes(data)
	types <- gpatts$types
	netnames <- gpatts$netnames
	nobs <- gpatts$observations + length(data)
	nOneMode <- sum(types == "oneMode")
	nBehav <- sum(types == "behavior")
	nContinuous <- sum(types == "continuous")
	nBipartite <- sum(types == "bipartite")
	if (nOneMode  + nBipartite > 0)
	{
		initialNetworks()
	}
	if (nBehav > 0)
	{
		initialBehaviors()
	}
	if (nContinuous > 0)
	{
		initialContinuous()
	}
	Report(c("Initialisation of data set <<", dataname,
			">> executed succesfully.\n"), sep="", outf)
}

#/******************************************************************************
#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena
# *
# * File: sienaeffects.r
# *
# * Description: This module contains utilities for updating an effects object
# *****************************************************************************/
##@includeEffect DataCreate
includeEffects <- function(myeff, ..., include=TRUE, name=myeff$name[1],
						   type="eval", interaction1="", interaction2="",
						   fix=FALSE, test=FALSE,
						   character=FALSE, verbose=TRUE)
{
	if (!inherits(myeff, 'sienaEffects'))
	{
		stop("The first argument is not of class <sienaEffects>.")
	}
	if (character)
	{
		dots <- sapply(list(...), function(x)x)
	}
	else
	{
		dots <- substitute(list(...))[-1] ##first entry is the word 'list'
	}
	if (length(dots) == 0)
	{
		stop("This function needs some effect short names.")
	}
	if (!character)
	{
		effectNames <- sapply(dots, function(x)deparse(x))
	}
	else
	{
		effectNames <- dots
	}
	if ("AltsAvAlt" %in% effectNames)
	{
		stop("Effect AltsAvAlt now is called avXAlt. Use the new name please.")
	}
	if ("cycle4ND" %in% effectNames)
	{
		stop("Effect cycle4ND now is called cycle4. Use the new name please.")
	}
	use <- myeff$shortName %in% effectNames &
	myeff$type==type &
	myeff$name==name &
	myeff$interaction1 == interaction1 &
	myeff$interaction2 == interaction2
	myeff[use, "include"] <- include
	myeff[use, "test"] <- test
	myeff[use, "fix"] <- fix
  	if (sum(myeff[use, "type"]=="gmm") > 0)
  	{
	    stop("\n To include a GMoM statistic use the function includeGMoMStatistics.")
	}
	if (sum(use) <= 0)
	{
		cat(paste("There is no effect with short name "))
		cat(paste(effectNames,", \n", sep=""))
		cat(paste("and with interaction1 = <",interaction1,">, ", sep=""))
		cat(paste("interaction2 = <",interaction2,">, ", sep=""))
		cat(paste("and type = <",type,">, \n", sep=""))
		cat(paste("for dependent variable",name,".\n"))
		cat("See effectsDocumentation() for this effects object.\n")
	}
	else
	{
#		print.data.frame(myeff[use, c("name", "shortName", "type",
#			"interaction1", "interaction2", "include")])
		if (verbose)
		{		
			myeff2 <- myeff[use,]		
			print.sienaEffects(myeff2, includeOnly=FALSE, 
								includeRandoms=any(myeff$random & (myeff$shortName != 'density')),
								includeShortNames=TRUE)
		}
	}
	if (hasArg('initialValue'))
	{
		warning("Warning: argument 'initialValue' has no effect in includeEffects; use setEffect.\n")
	}
	if (hasArg('parameter'))
	{
		warning("Warning: argument 'parameter' has no effect in includeEffects; use setEffect.\n")
	}
	if (hasArg('random'))
	{
		warning("Warning: argument 'random' has no effect in includeEffects; use setEffect.\n")
	}
	myeff
}

##@includeInteraction DataCreate
includeInteraction <- function(x, ...,
				include=TRUE, name=x$name[1],
				type="eval", interaction1=rep("", 3), interaction2=rep("", 3),
				fix=FALSE, test=FALSE, random=FALSE,
				initialValue=0,
				character=FALSE, verbose=TRUE)
{
	if (character)
	{
		dots <- sapply(list(...), function(x)x)
	}
	else
	{
		## check we have 2 or 3 short names
		dots <- substitute(list(...))[-1] ##first entry is the word 'list'
	}
	if (length(dots) == 0)
	{
		stop("need some effect short names")
	}
	if (length(dots) < 2 || length(dots) > 3)
	{
		 stop("need exactly two or three effect short names")
	}
	if (hasArg("parameter"))
	{
		stop("includeInteraction should not mention a parameter; see the help page")
	}
	if (!character)
	{
		shortNames <- sapply(dots, function(x)deparse(x))
	}
	else
	{
		shortNames <- dots
	}
	## find the first underlying effect
	shortName <- shortNames[1]
	interact1 <- interaction1[1]
	interact2 <- interaction2[1]
	use <- x$shortName == shortName &
	x$type==type &
	x$name==name &
	x$interaction1 == interact1 &
	x$interaction2 == interact2
	if (sum(use) == 0)
	{
		stop("First effect not found")
	}
	if (sum(use) > 1)
	{
		stop("First effect not unique")
	}
	effect1 <- x[use, "effectNumber"]
	## find the second underlying effect
	shortName <- shortNames[2]
	interact1 <- ifelse (length(interaction1) > 1, interaction1[2], "")
	interact2 <- ifelse (length(interaction2) > 1, interaction2[2], "")
	use <- x$shortName == shortName &
	x$type==type &
	x$name==name &
	x$interaction1 == interact1 &
	x$interaction2 == interact2
	if (sum(use) == 0)
	{
		stop("Second effect not found")
	}
	if (sum(use) > 1)
	{
		stop("Second effect not unique")
	}
	effect2 <- x[use, "effectNumber"]
	## find the third underlying effect, if any
	if (length(shortNames) > 2)
	{
		shortName <- shortNames[3]
		interact1 <- ifelse (length(interaction1) > 2, interaction1[3], "")
		interact2 <- ifelse (length(interaction2) > 2, interaction2[3], "")
		use <- x$shortName == shortName &
		x$type==type &
		x$name==name &
		x$interaction1 == interact1 &
		x$interaction2 == interact2
		if (sum(use) == 0)
		{
			stop("Third effect not found")
		}
		if (sum(use) > 1)
		{
			stop("Third effect not unique")
		}
		effect3 <- x[use, "effectNumber"]
	}
	else
	{
		effect3 <- 0
	}
    ## interaction effects not yet implemented for continuous behavior
 #   if (any(x$netType[c(effect1, effect2, effect3)] == "continuous"))
 #   {
 #       stop("Interaction effects not yet implemented for continuous behavior")
 #   }
	## does the effect already exist?
	intn <- (x$effect1 == effect1) & (x$effect2 == effect2)
	if (effect3 > 0)
	{
		intn <- intn & (x$effect3 == effect3)
	}
	# intn indicates the rows of the effects object with this interaction
	if (sum(intn) >= 2)
	{
		cat('The given effects object is corrupted:\n')
		cat('It already contains more than one copy of this interaction.\n')
		cat('Make a new effects object from scratch.\n')
		stop('Corrupted effects object')
	}

	## if want to include, check that we have a spare row
	if ((include) && (sum(intn) == 0))
	{
		# The interaction must be created
		ints <- x[x$name == name & x$shortName %in%
			c("unspInt", "behUnspInt", "contUnspInt") &
			(is.na(x$effect1) | x$effect1 == 0)&
			x$type == type, ]
		if (nrow(ints) == 0)
		{
			baseEffect<- x[x$name == name, ][1, ]
			if (baseEffect$netType == "behavior")
			{
				stop("Use getEffects() with a larger value for behNintn.")
			}
			else
			{
				stop("Use getEffects() with a larger value for nintn.")
			}
		}
		ints <- ints[1, ]
		intn <- x$effectNumber == ints$effectNumber
	}
	if (include)
	{
		x[intn, "include"] <- include
		x[intn, c("effect1", "effect2", "effect3")] <-
			c(effect1, effect2, effect3)
		x[intn, "fix"] <- fix
		x[intn, "test"] <- test
		x[intn, "randomEffects"] <- random
		x[intn, "initialValue"] <- initialValue
	}
	else
	{
		if (sum(intn) == 0)
		{
		warning('Note: there was no such interaction in this effects object.')
		}
		else
		{
			x[intn, "include"] <- FALSE
		}
	}
#
	x <- fixUpEffectNames(x)
	if (verbose)
	{
		intn2 <- intn
		intn[effect1] <- TRUE
		intn[effect2] <- TRUE
		if (effect3 > 0)
		{
			intn[effect3] <- TRUE
		}
		myeff2 <- x[intn,]	
		print.sienaEffects(myeff2, includeOnly=FALSE, includeRandoms=random,
								includeShortNames=TRUE)
	}
	x
}

##@setEffect DataCreate
setEffect <- function(x, shortName, parameter=NULL,
					fix=FALSE, test=FALSE, random=FALSE,
					initialValue=0,
					timeDummy=",",
					include=TRUE, name=x$name[1],
					type="eval", interaction1="", interaction2="",
					effect1=0, effect2=0, effect3=0,
					period=1, group=1, character=FALSE, verbose=TRUE)
{
	replace.myeff <- function(meff, use, oldPar, newPar){
		meff[use, "effectName"] <- 
              gsub(oldPar, newPar, meff[use, "effectName"], fixed=TRUE)
		meff[use, "functionName"] <- 
              gsub(oldPar, newPar, meff[use, "functionName"], fixed=TRUE)	
		meff[use,]
	}
	if (!character)
	{
		shortName <- deparse(substitute(shortName))
	}
	if (shortName=="AltsAvAlt")
	{
		stop("Effect AltsAvAlt renamed to avXAlt.")
	}
	if (type=="gmm")
	{
	    stop("\n To include a GMoM statistic use the function includeGMoMStatistics.")
	}
	use <- x$shortName == shortName &
			x$name == name &
			x$type == type &
			x$interaction1 == interaction1 &
			x$interaction2 == interaction2 &
			(is.na(x$period) | x$period == period) &
			x$group == group
	if (shortName %in% c("unspInt", "behUnspInt", "contUnspInt"))
	{
		use <- use & (x$include) & (x$effect1 == effect1) &
			(x$effect2 == effect2) & (x$effect3 == effect3)
	}
	if (sum(use) == 0)
	{
		cat(paste("There is no effect with short name "))
		cat(paste(shortName,", \n", sep=""))
		cat(paste("and with interaction1 = <",interaction1,">, ", sep=""))
		cat(paste("interaction2 = <",interaction2,">, ", sep=""))
		cat(paste("type = <",type,">, ", sep=""))
		cat(paste("period = <",period,">, ", sep=""))
		if (shortName %in% c("unspInt", "behUnspInt", "contUnspInt"))
		{
		cat(paste("effects1-2-3 = <",effect1, effect2, effect3,">,", sep=" "))
		}
		cat(paste("and group = <",group,">, \n ", sep=""))
		cat(paste("for dependent variable",name,".\n"))
		stop("Effect not found")
	}
	if (sum(use) > 1)
	{
		stop("Effect not unique")
	}
	if (!is.null(parameter)) 
	{
		olderParameter <- x[use, "parm"]
		x[use, "parm"] <- parameter
	}
	x[use, "include"] <- include
	x[use, "fix"] <- fix
	x[use, "test"] <- test
	x[use, "initialValue"] <- initialValue
	x[use, "timeDummy"] <- timeDummy
	x[use, "randomEffects"] <- random
	if (grepl("#", x[use, "effectName"], fixed=TRUE))
	{
# This means the original names in row[use,] in allEffects.csv were not yet changed
		x[use, "effectName"] <- 
              gsub("#", x[use, "parm"], x[use, "effectName"], fixed=TRUE)
		x[use, "functionName"] <- 
              gsub("#", x[use, "parm"], x[use, "functionName"], fixed=TRUE)
	}
	else if (!is.null(parameter))
    {
# replace what originally were strings "1/#" or "(#)" or "= #" or  "+ #" or "- #" or "#-" 
# Other original uses of # will not be replaced!
		oldParameter <- paste("1/", olderParameter, sep="")
		newParameter <- paste("1/", x[use, "parm"], sep="")
		x[use,] <- replace.myeff(x, use, oldParameter, newParameter)
		oldParameter <- paste("(", olderParameter, ")", sep="")
		newParameter <- paste("(", x[use, "parm"], ")", sep="")
		x[use,] <- replace.myeff(x, use, oldParameter, newParameter)
		oldParameter <- paste("= ", olderParameter, sep="")
		newParameter <- paste("= ", x[use, "parm"], sep="")
		x[use,] <- replace.myeff(x, use, oldParameter, newParameter)
		oldParameter <- paste("+ ", olderParameter, sep="")
		newParameter <- paste("+ ", x[use, "parm"], sep="")
		x[use,] <- replace.myeff(x, use, oldParameter, newParameter)
		oldParameter <- paste("- ", olderParameter, sep="")
		newParameter <- paste("- ", x[use, "parm"], sep="")
		x[use,] <- replace.myeff(x, use, oldParameter, newParameter)
		oldParameter <- paste(olderParameter, "-", sep="")
		newParameter <- paste(x[use, "parm"], "-", sep="")
		x[use,] <- replace.myeff(x, use, oldParameter, newParameter)
	}
	if (verbose)
	{		
		myeff2 <- x[use,]		
		print.sienaEffects(myeff2, includeOnly=FALSE, includeRandoms=random,
								includeShortNames=TRUE)
	}
	x
}

##@includeGMoMStatistics DataCreate
includeGMoMStatistics <- function(myeff, ..., include=TRUE, depvar=myeff$name[1],
                                  covar1="", covar2="",
                                  character=FALSE, verbose=TRUE)
{
  if (character)
  {
    dots <- sapply(list(...), function(x)x)
  }
  else
  {
    dots <- substitute(list(...))[-1] ##first entry is the word 'list'
  }
  if (length(dots) == 0)
  {
    stop("This function needs some effect short names.")
  }
  if (!character)
  {
    effectNames <- sapply(dots, function(x)deparse(x))
  }
  else
  {
    effectNames <- dots
  }
  if (hasArg("name"))
  {
		stop("The name of the dependent variable now should be given as 'depvar', not 'name'.
      See the help file for includeGMoMStatistics.  ")
  }
  if (hasArg("interaction1") | hasArg("interaction2"))
  {
		stop("The name of the explanatory variable now should be given 
    as 'covar1' and 'covar2', not 'interaction1' and 'interaction2'.
    See the help file for includeGMoMStatistics.  ")
  }
  use <- myeff$shortName %in% effectNames &
    myeff$type=="gmm" &
    myeff$name==depvar &
    myeff$interaction1 == covar1 &
    myeff$interaction2 == covar2
  myeff[use, "include"] <- include
  myeff[use, "test"] <- FALSE
  myeff[use, "fix"] <- TRUE
  myeff[use, "initialValue"] <- 0
  if (sum(use) <= 0)
  {
    cat(paste("There is no GMoM statistic with short name "))
    cat(paste(effectNames,", \n", sep=""))
    cat(paste("and with interaction1 = <",covar1,">, ", sep=""))
    cat(paste("interaction2 = <",covar2,">, ", sep=""))
    cat(paste("for dependent variable",depvar,".\n"))
  }
  else
  {
    #		print.data.frame(myeff[use, c("name", "shortName", "type",
    #			"interaction1", "interaction2", "include")])
    if (verbose)
    {
      myeff2 <- as.data.frame(myeff[use,])
      rownames(myeff2) <- 1:nrow(myeff2)
      print.data.frame(myeff2[, c("name", "shortName", "type","include")])
    }
  }
  myeff
}


fixUpEffectNames <- function(effects)
##@fixUpEffectNames siena07 Replace # and construct interaction names
{
    ## replace # by the parm value in function and effect names:
    effects$effectName <-
        sapply(1:nrow(effects), function(x, y)
           {
               y <- y[x, ]
               gsub("#", y$parm, y$effectName)
           }, y=effects)
    effects$functionName <-
        sapply(1:nrow(effects), function(x, y)
           {
               y <- y[x, ]
               gsub("#", y$parm, y$functionName)
           }, y=effects)
    ##validate user-specified network interactions
    interactions <- effects[effects$shortName == "unspInt" & effects$include &
                            effects$effect1 > 0 , ]
    if (nrow(interactions) > 0)
    {
        unspIntNames <-
            sapply(1:nrow(interactions), function(x, y, z)
               {
                   y <- y[x, ] ## get the interaction effect
                   twoway <- y$effect3 == 0
                   ## now get the rows which are to interact
                   inter1 <- z[z$effectNumber == y$effect1, ]
                   if (nrow(inter1) != 1 )
                   {
                       stop("invalid network interaction specification: ",
                            "effect number 1")
                   }
                   inter2 <- z[z$effectNumber == y$effect2, ]
                   if (nrow(inter2) != 1 )
                   {
                       stop("invalid network interaction specification: ",
                            "effect number 2")
                   }
                   if (!twoway)
                   {
                       inter3 <- z[z$effectNumber == y$effect3, ]
                       if (nrow(inter3) != 1)
                       {
                           stop("invalid network interaction specification: ",
                                "effect number 3")
                       }
                   }
                   else
                   {
                       inter3 <- z[is.na(z$effectNumber), ]
                       ## should be empty row
                   }
                   if (twoway)
                   {
                       if (inter1$name != inter2$name)
                       {
                           stop("invalid network interaction specification: ",
                                "must all be same network")
                       }
                       if (inter1$type != inter2$type)
                       {
                           stop("invalid network interaction specification: ",
                                "must all be same type: ",
                                "evaluation, endowment or creation")
                       }
                   }
                   else
                   {
                       if (inter1$name != inter2$name ||
                           inter1$name != inter3$name)
                       {
                           stop("invalid network interaction specification: ",
                                "must all be same network")
                       }
                       if (inter1$type != inter2$type ||
                           inter1$type != inter3$type)
                       {
                           stop("invalid network interaction specification: ",
                                "must all be ",
                                "same type: evaluation, endowment or creation ")
                       }
                   }
                   ## check types
                   inters <- rbind(inter1, inter2, inter3)
                   egos <- which(inters$interactionType == "ego")
                   egoCount <- length(egos)
                   dyads <- which(inters$interactionType == "dyadic")
                   dyadCount <- length(dyads)
                   if (twoway)
                   {
                       if (egoCount < 1 && dyadCount != 2)
                       {
                           stop("invalid network interaction specification: ",
                                "must be at least one ego or both dyadic ",
                                "effects")
                       }
                   }
                   else
                   {
                       if (egoCount < 2 && (egoCount + dyadCount < 3))
                       {
                      stop("invalid network 3-way interaction specification: ",
									"must be at least two ego effects ",
									"or all ego or dyadic effects")
                       }
                   }
                   ## construct a name
                   ## make sure the egos are at the front of inters
                   if (egoCount > 0)
                   {
                       inters <- rbind(inters[egos, ], inters[-egos, ])
                   }
 				   tmpnames <- inters$effectName
				   tmpnames[-1] <- sub(paste(inters$name[1], ": ",
											 sep=""), "", tmpnames[-1])
				   tmpname <- paste(tmpnames, collapse = " x ")
# following lines dropped, might be restored if desired
#                   if (twoway && nchar(tmpname) < 38)
#                   {
#                       tmpname <- paste("int. ", tmpname)
#                   }
#                   if (!twoway)
#                  {
#                      tmpname <- paste("i3.", tmpname)
#                  }
                   tmpname
               }, y=interactions, z=effects)
        effects[effects$shortName == "unspInt" & effects$include &
                !is.na(effects$effect1), c("effectName", "functionName")] <-
                    unspIntNames
    }
    ##validate user-specified behavior interactions
    interactions <- effects[effects$shortName == "behUnspInt" &
                            effects$include &
                            effects$effect1 > 0 , ]
    if (nrow(interactions) > 0)
    {
        unspIntNames <-
            sapply(1:nrow(interactions), function(x, y, z)
               {
                   y <- y[x, ] ## get the interaction effect
                   twoway <- y$effect3 == 0
                   ## now get the rows which are to interact
                   inter1 <- z[z$effectNumber == y$effect1, ]
                   if (nrow(inter1) != 1 )
                   {
                       stop("invalid behavior interaction specification: ",
                            "effect number 1")
                   }
                   inter2 <- z[z$effectNumber == y$effect2, ]
                   if (nrow(inter2) != 1 )
                   {
                       stop("invalid behavior interaction specification: ",
                            "effect number 2")
                   }
                   if (!twoway)
                   {
                       inter3 <- z[z$effectNumber == y$effect3, ]
                       if (nrow(inter3) != 1)
                       {
                           stop("invalid behavior interaction specification: ",
                                "effect number 3")
                       }
                   }
                   else
                   {
                       inter3 <- z[is.na(z$effectNumber), ]
                       ## should be empty row
                   }
                   if (twoway)
                   {
                       if (inter1$name != inter2$name)
                       {
                           stop("invalid behavior interaction specification: ",
                                "must all be same behavior variable")
                       }
                       if (inter1$type != inter2$type)
                       {
                           stop("invalid behavior interaction specification: ",
                                "must be same type: evaluation, endowment ",
                                "or creation")
                       }
                   }
                   else
                   {
                       if (inter1$name != inter2$name ||
                           inter1$name != inter3$name)
                       {
                           stop("invalid behavior interaction specification: ",
                                "must all be same behavior variable")
                       }
                       if (inter1$type != inter2$type ||
                           inter1$type != inter3$type)
                       {
                           stop("invalid behavior interaction specification: ",
                                "must all be ",
                                "same type: evaluation, endowment or creation")
                       }
                   }
                   ## check types - at most one should be not OK here
                   inters <- rbind(inter1, inter2, inter3)
                   if (length(which(inters$interactionType != "OK")) > 1)
				   {
				   	   stop("invalid behavior interaction specification: ",
				   			"at most one effect with interactionType ",
				   			"not OK is allowed")
                   }
                   ##if (any(inters$interactionType != "OK"))
                   ##{
                   ##    stop("invalid behavior interaction specification: ",
                   ##         "only effects with interactionType OK are allowed")
                   ##}
                   ## construct a name
				   tmpnames <- inters$effectName
				   tmpnames[-1] <- sub(paste("behavior ", inters$name[1], " ",
											 sep=""), "", tmpnames[-1])
				   tmpname <- paste(tmpnames, collapse = " x ")
                   if (twoway && nchar(tmpname) < 38)
                   {
                       tmpname <- paste("int. ", tmpname)
                   }
                   if (!twoway)
                   {
                       tmpname <- paste("i3.", tmpname)
                   }
                   tmpname
               }, y=interactions, z=effects)
        effects[effects$shortName == "behUnspInt" & effects$include &
                !is.na(effects$effect1), c("effectName", "functionName")] <-
                    unspIntNames
    }
    ##validate user-specified continuous interactions
    interactions <- effects[effects$shortName == "contUnspInt" &
                            effects$include &
                            effects$effect1 > 0 , ]
    if (nrow(interactions) > 0)
    {
        unspIntNames <-
            sapply(1:nrow(interactions), function(x, y, z)
               {
                   #browser()
				   y <- y[x, ] ## get the interaction effect
                   twoway <- y$effect3 == 0
                   ## now get the rows which are to interact
                   inter1 <- z[z$effectNumber == y$effect1, ]
                   if (nrow(inter1) != 1 )
                   {
                       stop("invalid behavior interaction specification: ",
                            "effect number 1")
                   }
                   inter2 <- z[z$effectNumber == y$effect2, ]
                   if (nrow(inter2) != 1 )
                   {
                       stop("invalid behavior interaction specification: ",
                            "effect number 2")
                   }
                   if (!twoway)
                   {
                       inter3 <- z[z$effectNumber == y$effect3, ]
                       if (nrow(inter3) != 1)
                       {
                           stop("invalid behavior interaction specification: ",
                                "effect number 3")
                       }
                   }
                   else
                   {
                       inter3 <- z[is.na(z$effectNumber), ]
                       ## should be empty row
                   }
                   if (twoway)
                   {
                       if (inter1$name != inter2$name)
                       {
                           stop("invalid behavior interaction specification: ",
                                "must all be same behavior variable")
                       }
                       if (inter1$type != inter2$type)
                       {
                           stop("invalid behavior interaction specification: ",
                                "must be same type: evaluation")
                       }
                   }
                   else
                   {
                       if (inter1$name != inter2$name ||
                           inter1$name != inter3$name)
                       {
                           stop("invalid behavior interaction specification: ",
                                "must all be same behavior variable")
                       }
                       if (inter1$type != inter2$type ||
                           inter1$type != inter3$type)
                       {
                           stop("invalid behavior interaction specification: ",
                                "must all be same type: evaluation")
                       }
                   }
                   ## check types - all should be OK here
                   inters <- rbind(inter1, inter2, inter3)
                   if (any(inters$interactionType != "OK"))
				   {
				   	   stop("invalid behavior interaction specification: ",
				   			"only effects with interactionType OK are allowed")
                   }
                   ## construct a name
				   tmpnames <- inters$effectName
				   tmpnames[-1] <- sub(paste("behavior ", inters$name[1], " ",
											 sep=""), "", tmpnames[-1])
				   tmpname <- paste(tmpnames, collapse = " x ")
                   if (twoway && nchar(tmpname) < 38)
                   {
                       tmpname <- paste("int. ", tmpname)
                   }
                   if (!twoway)
                   {
                       tmpname <- paste("i3.", tmpname)
                   }
                   tmpname
               }, y=interactions, z=effects)
        effects[effects$shortName == "contUnspInt" & effects$include &
                !is.na(effects$effect1), c("effectName", "functionName")] <-
                    unspIntNames
    }
    effects
}

#/******************************************************************************
# * SIENA: Simulation Investigation for Empirical Network Analysis
# *
# * Web: https://www.stats.ox.ac.uk/~snijders/siena/
# *
# * File: sienaRIDynamics.r
# *
# * Description: Used to determine, print, and plots relative importances of effects
# * in sequences of simulated mini-steps
# *****************************************************************************/

##@interpret_size_dynamics
interpret_size_dynamics <- function(data, ans=NULL, theta=NULL, 
				algorithm=NULL, effects=NULL, depvar=NULL, intervalsPerPeriod=NULL, 
				n3 = NULL, useChangeContributions = FALSE,
				silent = TRUE, seed = NULL)
{
	if(length(data$depvars)>1){
		if(is.null(depvar)){
			currentNetName <- attr(data$depvars,"names")[1]
			warning("If the models contains more than one dependent variables, 
			\n it should be specified by variable 'depvar' for which dependent 
			variable the relative importances should be calculated. \n\n")
			warning(paste("As 'depvar = NULL', relative importances are 
			calculated for variable ", currentNetName, sep=""))
		}else if(!(depvar %in% attr(data$depvars,"names"))){
			stop("'depvar' is not a name of a dependent variable")
		} else {
			depvar <- depvar
		}
	} else {
		depvar <- attr(data$depvars,"names")[1]
	}
	if (!(inherits(data, "sienadata") | inherits(data, "siena")))
	{
		stop("data is not a legitimate Siena data specification")
	}
	if(!is.null(intervalsPerPeriod))
	{
		if(is.numeric(intervalsPerPeriod))
			{
				intervalsPerPeriod <- as.integer(intervalsPerPeriod)
			} else {
				intervalsPerPeriod <- NULL
				warning("'intervalsPerPeriod' has to be of type 'numeric' \n 
					used default settings")
			}
		}
	if(is.null(intervalsPerPeriod))
	{
			intervalsPerPeriod <- 10
	}
	if(!is.null(ans))
	{
		## Take from ans if available
		if (!inherits(ans, "sienaFit"))
		{
			stop("ans is not a legitimate Siena fit object")
		}
		paras <- c(ans$rate, ans$theta)
		algo <- ans$x
		effs <- ans$effects # will be overwritten if conditional
		# could data also be taken from ans?
		if(ans$cconditional)
		{
			if(is.null(effects))
			{
				stop("effects = NULL! In case of conditional estimation, the 
				'sienaEffects' object has to be given to function 
				sienRIDynamics directly.")
			}
			if(!inherits(effects, "sienaEffects"))
			{
				stop("effects is not a legitimate Siena effects object")
			}
			effs <- effects
			if(!is.null(algorithm)||!is.null(theta))
			{
				warning("some information are multiply defined \n results will 
					be based on 'theta' and 'algorithm' stored in 'ans' 
					(as 'ans$theta', 'ans$x')")
			}
		} else {
			if(!is.null(algorithm)||!is.null(theta)||!is.null(effects))
			{
				warning("some information are multiply defined \n results will 
				be based on 'theta', 'algorithm', and 'effects' stored in 'ans' 
				(as 'ans$theta', 'ans$x', 'ans$effects')")
			}
		}
		RIValues <- calculateRIDynamics(data = data, 
			ans=ans, 
			theta = paras, 
			algorithm = algo, 
			effects = effs, 
			depvar = depvar, 
			intervalsPerPeriod=intervalsPerPeriod, 
			n3 = n3, 
			useChangeContributions = useChangeContributions,
			silent = silent,
			seed = seed)
	} else {
		## Use theta, algorithm, and effects given in the function call
		if (!inherits(algorithm, "sienaAlgorithm"))
		{
			stop("algorithm is not a legitimate Siena algorithm specification")
		}
		if (!inherits(effects, "sienaEffects"))
		{
			stop("effects is not a legitimate Siena effects object")
		}
		if(!is.numeric(theta))
		{
			stop("theta is not a legitimate parameter vector")
		}
		if(length(theta) != sum(effects$include==TRUE))
		{
			if(length(theta) == sum(effects$include==TRUE & effects$type!="rate"))
			{
				stop("vector of model parameters has wrong dimension, 
					maybe rate parameters are missing")
			}
			stop("theta is not a legitimate parameter vector \n number 
				of parameters has to match number of effects")
		}
		RIValues <- calculateRIDynamics(data = data, 
		ans= NULL, 
		theta = theta, 
		algorithm = algorithm, 
		effects = effects, 
		depvar = depvar, 
		intervalsPerPeriod=intervalsPerPeriod, 
		n3 = n3, 
		useChangeContributions = useChangeContributions,
		silent = silent,
		seed = seed)
	}
	RIValues
}

##@calculateRIDynamics calculateRIDynamics simulates sequences ministeps, 
## and aggregates the relative importances of effects over ministeps of same 
## time intervals
calculateRIDynamics <- function(data, 
	ans, 
	theta, 
	algorithm, 
	effects, 
	depvar, 
	intervalsPerPeriod=10, 
	returnActorStatistics=NULL, 
	n3 = NULL, 
	useChangeContributions = FALSE,
	silent = TRUE,
	seed = NULL)
{
    x <- algorithm # why not use algorithm directly?
	if(!is.null(seed))
	{
		if(is.numeric(seed))
		{
			x$seed <- as.integer(seed)
		} else {
			warning("'seed' has to be of type 'numeric' \n used default settings")
		}
	}
	if (!is.null(n3)) {
    	x$n3 <- as.integer(n3)
	}
	if (useChangeContributions) {
        if (is.null(ans) || is.null(ans$changeContributions)) {
            warning("useChangeContributions=TRUE, but 'ans' does not 
			contain 'changeContributions'. Will rerun siena07 to generate them.")
            useChangeContributions <- FALSE
        }
		if (!is.null(n3)) {
		    warning("n3 cannot be changed when useChangeContributions=TRUE; 
			using the number of chains stored in 'ans'.")
    		n3 <- NULL
		}
    }
    if (!useChangeContributions) 
	{
		x$nsub <- 0
        if (!is.null(ans)) 
		{
			prevAns <- ans
		} else {
			prevAns <- NULL
			effects$initialValue[effects$include] <- theta
		}
        ans <- siena07(
			x, 
			data=data, 
			effects=effects,
            prevAns=prevAns, 
			initC=FALSE, 
			returnDeps=FALSE, 
			returnChangeContributions=TRUE,
			silent = silent, batch=TRUE)
    }
	chains <- x$n3
	periods <- data$observation-1
	effects <- effects[effects$include==TRUE,]
	noRate <- effects$type != "rate"
	thetaNoRate <- theta[noRate]
	effectNames <- effects$shortName[noRate]
	effectTypes <- effects$type[noRate]
	depvar <- depvar
#	networkName <- effects$name[noRate]
	networkInteraction <- effects$interaction1[noRate]
	effectIds <- paste(effectNames,effectTypes,networkInteraction, sep = ".")
	currentNetObjEffs <- effects$name[noRate] == depvar
# The old code leading to a crash...
	RIintervalValues <- list()
# ...threfore, the following line is added
	blanks <- matrix(0, sum(currentNetObjEffs),intervalsPerPeriod)
	for(period in 1:periods)
	{
		RIintervalValues[[period]] <- data.frame(blanks, 
			row.names = effectIds[currentNetObjEffs])
	}
	for (chain in (1:chains))
	{
#cat("The following line leads to an error\n")
#browser()
		for(period in 1:periods)
		{
			ministeps <- length(ans$changeContributions[[chain]][[1]][[period]])
			stepsPerInterval <- ministeps/intervalsPerPeriod
# Tom's new code TS (7 lines).... Natalie slightly modified the next 4 lines
# and moved them one loop higher (out of the "chains"-loop)
#			blanks <- matrix(NA, sum(currentNetObjEffs),
#									ceiling(ministeps/stepsPerInterval))
#			RIintervalValues <-
#						as.list(rep(data.frame(blanks,
#							row.names = effectIds[currentNetObjEffs]), periods))
			blanks <- matrix(0, sum(currentNetObjEffs), intervalsPerPeriod)
			RItmp <- data.frame(blanks, row.names = effectIds[currentNetObjEffs])
			interval <- 1
			stepsInIntervalCounter <- 0
			for(ministep in 1:ministeps)
			{
				#depvar <- attr(ans$changeContributions[[1]][[period]][[ministep]],"networkName")
				if(attr(ans$changeContributions[[chain]][[1]][[period]][[ministep]],
												"networkName")==depvar)
				{
					stepsInIntervalCounter <- stepsInIntervalCounter + 1
					## distributions[1,] contains
					## the probabilities of the available choices in this micro-step
					## distributions[2,],distributions[3,],distributions[4,], ...contains
					## the probabilities of the available choices
					## if the parameter of the first, second, third ... effects is set to zero.
					distributions <- calculateDistributions(
						ans$changeContributions[[chain]][[1]][[period]][[ministep]],
						thetaNoRate[currentNetObjEffs])
					## If one wishes another measure
					## than the L^1-difference between distributions,
					## here is the right place
					## to call some new function instead of "L1D".
					RItmp[,stepsInIntervalCounter] <-
						standardize(L1D(distributions[1,],
									distributions[2:dim(distributions)[1],]))
				}
				if (ministep >
						interval * stepsPerInterval || ministep == ministeps)
				{
					if(chain == 1)
					{
						RIintervalValues[[period]][,interval] <-
										rowSums(RItmp)/length(RItmp)
					} else {
						RIintervalValues[[period]][,interval] <-
							RIintervalValues[[period]][,interval] +
											rowSums(RItmp)/length(RItmp)
					}
					interval <- interval + 1
					stepsInIntervalCounter <- 0
				}
			}
		}
	}
	for(period in 1:periods)
	{
		RIintervalValues[[period]] <- RIintervalValues[[period]]/chains
	}
	RIDynamics <- NULL
	RIDynamics$intervalValues <-RIintervalValues
	RIDynamics$dependentVariable <- depvar
	RIDynamics$effectNames <- paste(effectTypes[currentNetObjEffs], " ",
						(effects$effectName[noRate])[currentNetObjEffs], sep="")
	class(RIDynamics) <- "sienaRIDynamics"
	attr(RIDynamics, "version") <- packageDescription(pkgname, fields = "Version")
	RIDynamics
}

standardize <- function(values = NULL)
{
	newValues <- values/sum(values)
	newValues
}


##@print.sienaRIDynamics Methods
print.sienaRIDynamics <- function(x, ...)
{
	if (!inherits(x, "sienaRIDynamics"))
	{
		stop("x is not of class 'sienaRIDynamics' ")
	}
	periods <- length(x$intervalValues)
	intervals <- length(x$intervalValues[[1]])
	effs <- length(x$effectNames)
	cat(paste("\n  Relative importance of effects in micro-steps of 
	dependent variable '", x$dependentVariable,"'. \n \n",sep=""))
	cat(paste("  Periods between observations are divided into ", intervals, 
	" intervals. \n \n",sep=""))
	cat(paste("  Displayed results are aggregations over intervals:\n", sep=""))
	for(p in 1:periods){
		cat(paste("\n\nPeriod ",p,":\n", sep=""))
		colNames = paste("int ", 1:intervals, sep="")
		line1 <- paste(format("", width =52), sep="")
		line2 <- paste(format(1:effs,width=3), '. ', format(x$effectNames, 
			width = 45),sep="")
		col <- 0
		for(i in 1:length(colNames))
		{
			col <- col + 1
			line1 <- paste(line1, format(colNames[i], width=8),"  ", sep = "")
			line2 <- paste(line2, format(round(x$intervalValues[[p]][[i]], 4), 
				width=8, nsmall=4),"  ",sep="")
			if(col == 5)
			{
				line2 <- paste(line2, rep('\n',effs), sep="")
				cat(as.matrix(line1),'\n \n', sep='')
				cat(as.matrix(line2),'\n', sep='')
				line1 <- paste(format("", width =52), sep="")
				line2 <- paste(format(1:effs,width=3), '. ', 
					format(x$effectNames, width = 45),sep="")
				col<-0
			}
		}
		if(col>0)
		{
			line2 <- paste(line2, rep('\n',effs), sep="")
			cat(as.matrix(line1),'\n \n', sep='')
			cat(as.matrix(line2),'\n', sep='')
		}
	}
	invisible(x)
}
#
##@summary.sienaRIDynamics Methods
summary.sienaRIDynamics <- function(object, ...)
{
	if (!inherits(x, "sienaRIDynamics"))
	{
		stop("x is not of class 'sienaRIDynamics' ")
	}
	class(x) <- c("summary.sienaRIDynamics", class(x))
	x
}
##@print.summary.sienaRIDynamics Methods
print.summary.sienaRIDynamics <- function(x, ...)
{
	if (!inherits(x, "summary.sienaRIDynamics"))
	{
		stop("not a legitimate summary of a 'sienaRIDynamics' object")
	}
	print.sienaRIDynamics(x)
	invisible(x)
}


##@plot.sienaRIDynamics Methods
plot.sienaRIDynamics <- function(x, staticRI = NULL, col = NULL,
			ylim=NULL, width = NULL, height = NULL, legend = TRUE,
			legendColumns = NULL, legendHeight = NULL, cex.legend = NULL, ...)
{
	if(!inherits(x, "sienaRIDynamics"))
	{
		stop("x is not of class 'sienaRIDynamics' ")
	}
	if(!is.null(staticRI))
	{
		if(!inherits(staticRI, "sienaRI"))
		{
			warning("staticRI is not of class 'sienaRI' and is therefore ignored")
			staticValues <- NULL
		} else {
			if(staticRI$dependentVariable != x$dependentVariable)
			{
				warning("staticRI does not correspond to x and is therefore 
				ignored,\n staticRI and x do not refer to the same dependent variable")
				staticValues <- NULL
			}
			else if(length(staticRI$effectNames) != length(x$effectNames) || 
				sum(x$effectNames==staticRI$effectNames)!=length(x$effectNames))
			{
				warning("staticRI does not correspond to x and is therefore 
				ignored,\n staticRI and x do not refer to the same effects")
				staticValues <- NULL
			}
			else if(length(staticRI$expectedRI) != length(x$intervalValues))
			{
				warning("staticRI does not correspond to x and is therefore 
				ignored,\n staticRI and x do not refer to the same number of 
				observation moments")
				staticValues <- NULL
			}
			else
			{
				staticValues <- staticRI$expectedRI
			}
		}
	} else {
		staticValues <- NULL
	}
	if(legend)
	{
		if(!is.null(legendColumns))
		{
			if(is.numeric(legendColumns))
			{
				legendColumns <- as.integer(legendColumns)
			} else {
				legendColumns <- NULL
				warning("legendColumns has to be of type 'numeric' \n used 
					default settings")
			}
		}
		if(is.null(legendColumns))
		{
			legendColumns <- 2
		}
		if(!is.null(legendHeight))
		{
			if(is.numeric(legendHeight))
			{
				legendHeight <- legendHeight
			} else {
				legendHeight <- NULL
				warning("legendHeight has to be of type 'numeric' \n used 
					default settings")
			}
		}
		if(is.null(legendHeight))
		{
			legendHeight <- 1
		}
	}
	if(!is.null(height))
	{
		if(is.numeric(height))
		{
			height <- height
		} else {
			height <- NULL
			warning("height has to be of type 'numeric' \n used default settings")
		}
	}
	if(is.null(height))
	{
		if(legend)
		{
			height <- 4
		} else {
			height <- 3
		}
	}

	if(!is.null(width))
	{
		if(is.numeric(width))
		{
			width <- width
		} else {
			width <- NULL
			warning("width has to be of type 'numeric' \n used default settings")
		}
	}
	if(is.null(width))
	{
		width <- 8
	}

	if(!is.null(cex.legend))
	{
		if(is.numeric(cex.legend))
		{
			cex.legend <- cex.legend
		} else {
			cex.legend <- NULL
			warning("cex.legend has to be of type 'numeric' \n used default 
				settings")
		}
	}
	if(is.null(cex.legend))
	{
		cex.legend <- 1
	}

	if(!is.null(col))
	{
		cl <- col
	}
	if(is.null(col))
	{
		alph <- 175
		green <- rgb(127, 201, 127,alph, maxColorValue = 255)
		lila <-rgb(190, 174, 212,alph, maxColorValue = 255)
		orange <- rgb(253, 192, 134,alph, maxColorValue = 255)
		yellow <- rgb(255, 255, 153,alph, maxColorValue = 255)
		blue <- rgb(56, 108, 176,alph, maxColorValue = 255)
		lightgray <- rgb(184,184,184,alph, maxColorValue = 255)
		darkgray <- rgb(56,56,56,alph, maxColorValue = 255)
		gray <- rgb(120,120,120,alph, maxColorValue = 255)
		pink <- rgb(240,2,127,alph, maxColorValue = 255)
		brown <- rgb(191,91,23,alph, maxColorValue = 255)
		cl <- c(green,lila,orange,yellow,blue,lightgray,darkgray,gray,pink,brown)
		while(length(cl)<length(x$effectNames)){
			alph <- (alph+75)%%255
			green <- rgb(127, 201, 127,alph, maxColorValue = 255)
			lila <-rgb(190, 174, 212,alph, maxColorValue = 255)
			orange <- rgb(253, 192, 134,alph, maxColorValue = 255)
			yellow <- rgb(255, 255, 153,alph, maxColorValue = 255)
			blue <- rgb(56, 108, 176,alph, maxColorValue = 255)
			lightgray <- rgb(184,184,184,alph, maxColorValue = 255)
			darkgray <- rgb(56,56,56,alph, maxColorValue = 255)
			gray <- rgb(120,120,120,alph, maxColorValue = 255)
			pink <- rgb(240,2,127,alph, maxColorValue = 255)
			brown <- rgb(191,91,23,alph, maxColorValue = 255)
			cl <- c(cl,green,lila,orange,yellow,blue,lightgray,darkgray,gray,pink,brown)
		}
	}
#	bordergrey <-"gray25"
	values <- x$intervalValues
	periods <- length(values)
	effectNames <- x$effectNames
	effectNumber <- length(effectNames)
	lineTypes <- rep("solid",effectNumber)
	legendNames <- effectNames
	if(is.null(ylim))
	{
		ylim <-c(0,ceiling(max(unlist(lapply(values, max)))*10)*0.1)
	} else {
		ylim <-ylim
	}
	if(legend)
	{
		layout(rbind(1:periods, rep(periods+1,periods)),
				widths=rep(4, periods),heights=c(3,legendHeight))
	} else {
		layout(rbind(1:periods),widths=rep(4, periods),heights=c(3))
	}
#	par( oma = c( 1, 3, 1, 3 ),mar = par()$mar+c(-5,-4.1,-4,-2.1), xpd=T )
	par( oma = c( 1, 3, 1, 3 ),mar = c(0.1, 0.1, 0.1, 0.1), xpd=T )
	for(period in 1:periods){
		timeseries<-ts(t(values[[period]]))
		plot.ts(timeseries, plot.type = "single",  col = cl,
				lty = lineTypes, lwd = rep(1.5,effectNumber), bty = "n",
				xaxt = "n",yaxt = "n", ylab ="", xlab = "", ylim = ylim)
		for(eff in 1:effectNumber)
		{
			points(ts(t(values[[period]]))[,eff], col = cl[eff], type = "p", pch = 20)
			if(!is.null(staticValues))
			{
				points(xy.coords(1,staticValues[[period]][eff]),col = cl[eff],
						type = "p", pch = 1, cex = 1.75)
			}
		}
		ax <- ((ylim[1]*10):(ylim[2]*10))/10
		if(period==1){
			if(period == periods){
				axis(4, ax)
				axis(2, ax)
			}else{
				axis(4, ax, labels = FALSE)
				axis(2, ax)
			}
		} else if(period == periods) {
			axis(4, ax)
			axis(2, ax, labels = FALSE)
		} else {
			axis(2, ax, labels = FALSE)
			axis(4, ax, labels = FALSE)
		}
	}
	if(legend)
	{
		plot(c(0,1), c(0,1), col=rgb(0,0,0,0),axes=FALSE,  ylab = "", xlab = "")
		legend(0, 1, legendNames, col = cl[1:effectNumber], lwd = 2,
				lty = lineTypes, ncol = legendColumns, bty = "n",cex=cex.legend)
	}
	invisible(cl)
}

%file RSiena/man/iwlsm.Rd
% based on file MASS/man/rlm.Rd
% copyright (C) 1994-9 W. N. Venables and B. D. Ripley
%
\name{iwlsm}
\alias{iwlsm}
\alias{iwlsm.default}
\alias{iwlsm.formula}
\alias{print.iwlsm}
\alias{predict.iwlsm}
\alias{psi.iwlsm}
\title{
Function to fit an iterated weighted least squares model.
}
\description{
  Fits an iterated weighted least squares model.
}
\usage{
iwlsm(x, ...)

\method{iwlsm}{formula}(formula, data, weights, ses, \dots, subset, na.action,
    method = c("M", "MM", "model.frame"),
    wt.method = c("inv.var", "case"),
    model = TRUE, x.ret = TRUE, y.ret = FALSE, contrasts = NULL)

\method{iwlsm}{default}(x, y, weights, ses, \dots, w = rep(1/nrow(x), nrow(x)),
    init = "ls", psi = psi.iwlsm,
    scale.est = c("MAD", "Huber", "proposal 2"), k2 = 1.345,
    method = c("M", "MM"), wt.method = c("inv.var", "case"),
    maxit = 20, acc = 1e-4, test.vec = "resid", lqs.control = NULL)

psi.iwlsm(u, k, deriv = 0, w, sj2, hh)
}
\arguments{
  \item{formula}{
    a formula of the form \code{y ~ x1 + x2 + \dots}.
  }
  \item{data}{
    data frame from which variables specified in \code{formula} are
    preferentially to be taken.
  }
  \item{weights}{
    a vector of prior weights for each case.
  }
  \item{subset}{
    An index vector specifying the cases to be used in fitting.
  }
  \item{ses}{Estimated variance of the responses. Will be passed to
	\code{psi} as sj2
  }
  \item{na.action}{
    A function to specify the action to be taken if \code{NA}s are found.
    The \sQuote{factory-fresh} default action in \R is
    \code{\link{na.omit}}, and can be changed by
    \code{\link{options}(na.action=)}.
  }
  \item{x}{
    a matrix or data frame containing the explanatory variables.
  }
  \item{y}{
    the response: a vector of length the number of rows of \code{x}.
  }
  \item{method}{Must be "M". (argument not used here).
  }
  \item{wt.method}{
    are the weights case weights (giving the relative importance of case,
    so a weight of 2 means there are two of these) or the inverse of the
    variances, so a weight of two means this error is half as variable?
	This will not work at present.
  }
  \item{model}{
    should the model frame be returned in the object?
  }
  \item{x.ret}{
    should the model matrix be returned in the object?
  }
  \item{y.ret}{
    should the response be returned in the object?
  }
  \item{contrasts}{
    optional contrast specifications: se \code{\link{lm}}.
  }
  \item{w}{
    (optional) initial down-weighting for each case. Will not work at present.
  }
  \item{init}{
    (optional) initial values for the coefficients OR a method to find
    initial values OR the result of a fit with a \code{coef} component.  Known
    methods are \code{"ls"} (the default) for an initial least-squares fit
    using weights \code{w*weights}, and \code{"lts"} for an unweighted
    least-trimmed squares fit with 200 samples. Probably not functioning.
  }
  \item{psi}{
    the psi function is specified by this argument.  It must give
    (possibly by name) a function \code{g(x, \dots, deriv, w)} that for
    \code{deriv=0} returns psi(x)/x and for \code{deriv=1} returns
    some value.  Extra arguments may be passed in via \code{\dots}.
  }
  \item{scale.est}{
    method of scale estimation: re-scaled MAD of the residuals (default)
    or Huber"s proposal 2 (which can be selected by either \code{"Huber"}
    or \code{"proposal 2"}).
  }
  \item{k2}{
    tuning constant used for Huber proposal 2 scale estimation.
  }
  \item{maxit}{
    the limit on the number of IWLS iterations.
  }
  \item{acc}{
    the accuracy for the stopping criterion.
  }
  \item{test.vec}{
    the stopping criterion is based on changes in this vector.
  }
  \item{\dots}{
    additional arguments to be passed to \code{iwlsm.default} or to the \code{psi}
    function.
  }
  \item{lqs.control}{
    An optional list of control values for \code{\link[MASS]{lqs}}.
  }
  \item{u}{
    numeric vector of evaluation points.
  }
  \item{k}{    tuning constant. Not used.
  }
  \item{deriv}{
    \code{0} or \code{1}: compute values of the psi function or of its
    first derivative. (Latter not used).
  }
  \item{sj2}{Estimated variance of the responses}
  \item{hh}{Diagonal values of the hat matrix}
}
\details{This function is very slightly adapted from \code{rlm} in
  packages \code{MASS}. It alternates between weighted
  least squares and estimation of variance on the basis of a common
  variance. The function \code{psi.iwlsm} calculates the weights
  for the next iteration. 
  Used by \code{\link{meta_siena}} to combine estimates
  from different \code{sienaFits}.
}
\value{
  An object of class \code{"iwlsm"} inheriting from \code{"lm"}.
  Note that the \code{df.residual} component is deliberately set to
  \code{NA} to avoid inappropriate estimation of the residual scale from
  the residual mean square by \code{"lm"} methods.

  The additional components not in an \code{lm} object are

  \item{s}{
    the robust scale estimate used
  }
  \item{w}{
    the weights used in the IWLS process
  }
  \item{psi}{
    the psi function with parameters substituted
  }
  \item{conv}{
    the convergence criteria at each iteration
  }
  \item{converged}{
    did the IWLS converge?
  }
  \item{wresid}{
    a working residual, weighted for \code{"inv.var"} weights only.
  }
}
\references{
  Venables, W. N. and Ripley, B. D. (2002)
  \emph{Modern Applied Statistics with S.}  Fourth edition.  Springer.
  See also \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Ruth Ripley}
\note{The function has been changed as little as possible, but
   has only been used with default arguments. The other options have
    been retained just in
    case they may prove useful.
}
\seealso{
  \code{\link{meta_siena}}, \code{\link{sienaMeta}}, 
  \code{\link{sienaFit}}
  }
  \examples{
\dontrun{
##not enough data here for a sensible example, but shows the idea.
Group1 <- as_dependent_rsiena(array(c(N3401, HN3401), dim=c(45, 45, 2)))
Group3 <- as_dependent_rsiena(array(c(N3403, HN3403), dim=c(37, 37, 2)))
Group4 <- as_dependent_rsiena(array(c(N3404, HN3404), dim=c(33, 33, 2)))
dataset.1 <- make_data_rsiena(Friends = Group1)
dataset.3 <- make_data_rsiena(Friends = Group3)
dataset.4 <- make_data_rsiena(Friends = Group4)
effects.1 <- make_specification(dataset.1)
effects.3 <- make_specification(dataset.3)
effects.4 <- make_specification(dataset.4)
effects.1 <- set_effect(effects.1, transTrip)
effects.1 <- set_effect(effects.1, transRecTrip, fix=TRUE, test=TRUE)
effects.3 <- set_effect(effects.3, transTrip)
effects.3 <- set_effect(effects.3, transRecTrip, fix=TRUE, test=TRUE)
effects.4 <- set_effect(effects.4, transTrip)
effects.4 <- set_effect(effects.4, transRecTrip, fix=TRUE, test=TRUE)
ans.1 <- siena(dataset.1, effects=effects.1, batch=TRUE)
ans.3 <- siena(dataset.3, effects=effects.3, batch=TRUE)
ans.4 <- siena(dataset.4, effects=effects.4, batch=TRUE)
meta <- meta_siena(ans.1, ans.3, ans.4)
metadf <- split(meta$thetadf, meta$thetadf$effects)[[1]]
(metalm <- iwlsm(theta ~ tconv, metadf, ses=se^2))
}
}
\keyword{models }

\name{print.sienaTest}
\alias{print.sienaTest}
\alias{sienaTest.methods}
\title{
  Print method for Wald and score tests for RSiena results
}
\description{
  This method prints Wald-type and score-type tests for results   
  estimated by \code{\link{siena}}.
}
\usage{
\method{print}{sienaTest}(x, \dots)
}
\arguments{
  \item{x}{An object of type \code{sienaTest}, produced by
    \code{\link{test_parameter}}.}
  \item{\dots}{Extra arguments (not used at present).}
}

\details{The function  \code{\link{test_parameter}} produces
    an object of type \code{\link{sienaTest}}.
    These can be printed by this method.}
\value{An object of type  \code{\link{sienaTest}}.
}
\author{Tom Snijders}
\seealso{ \code{\link{siena}},  \code{\link{test_parameter}}.}

\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mydata <- make_data_rsiena(mynet)
myeff <- make_specification(mydata)
myalgo <- set_algorithm_saom(nsub=1, n3=40, seed=1291)
# nsub=1, n3=40 is used here for having a brief computation, not for practice.
myeff <- set_effect(myeff, list(transTrip, transTies))
myeff <- set_effect(myeff, list(outAct, outPop), fix=TRUE, test=TRUE)
(ans <- siena(mydata, effects=myeff, control_algo=myalgo,
                                            silent=TRUE, batch=TRUE))
mprs <- test_parameter(ans, tested=c(3, 4))
print(mprs)
}
\keyword{print}

\name{interpret_size}
\alias{interpret_size}
\alias{sienaRI}
\alias{interpret_size}
\alias{interpret_size.sienaFit}
\alias{interpret_size.sienaEffects}
\alias{entropy}
\alias{plot.interpret_size}
\alias{print.interpret_size}
\alias{plot.sienaRI}
\alias{print.sienaRI}
\alias{interpret_size.methods}
\title{Functions to assess the relative importance of effects at observation
       moments}
\description{
 The function \code{interpret_size} computes various effect sizes for
 Stochastic Actor-oriented Models.
 As its value it returns the relative importance of effects
 of a SAOM according to the measure of relative importance described in
 Section 3.1 of Indlekofer and Brandes (2013).
 This is based on the influence of effects on potential tie change or
 behavior change decisions of individual actors at the given observation
 moments (waves).  \cr
 Other measures useful for effect sizes are entropy-based
 effect sizes as in Snijders (2004) and the within-ego
 standard deviations of change statistics.
 If \code{getChangeStats=TRUE}, the arrays of change statistics are
 stored in the \code{sienaRI} object that is created.

 It takes the data as well as the complete model specification into account.
 Therefore, necessary arguments are the analysed data given as a
 \code{\link{sienadata}} data object as well as the complete model
 specification represented either by an estimated \code{sienaFit}
 object or by the combination of a
 suitable parameter vector \code{theta} and the corresponding
 \code{\link{sienaEffects}} object.
}
\usage{
\method{interpret_size}{sienaFit}(x, data, getChangeStats=FALSE, ...)
\method{interpret_size}{sienaEffects}(x, data, theta, getChangeStats=FALSE, ...)
\method{print}{sienaRI}(x, printSigma=FALSE,\dots)
\method{plot}{sienaRI}(x, actors = NULL, col = NULL, addPieChart = FALSE,
              radius = 1, width = NULL, height = NULL, legend = TRUE,
              legendColumns = NULL, legendHeight = NULL,
              cex.legend = NULL, cex.names = NULL,\dots)
}
\arguments{
  \item{x}{for \code{interpret_size.sienaFit}: a \code{sienaFit} object,
           resulting from a call of \code{\link{siena}};\cr
           for \code{interpret_size.sienaEffects}: a \code{sienaEffects} object
           that was used for calling \code{\link{siena}};\cr
           for \code{print.sienaRI} and \code{plot.sienaRI}:
           a \code{sienaRI} object resulting
           from a call to \code{interpret_size}.}
  \item{data}{\code{siena} data object representing the analyzed data and
              resulting from a call to \code{\link{make_data_rsiena}}.}
  \item{theta}{Vector of parameter values of evaluation effects
               included in the model.
               Length of \code{theta} has to be equal to the number of
               included evaluation effects in the effects object \code{x}.}
  \item{getChangeStats}{Boolean: If \code{TRUE}, an array of change statistics
               is added to the \code{interpret_size} object.}
  \item{printSigma}{Boolean: If \code{TRUE},
       average within-actor standard deviations of change statistics
       ('\code{sigma}') are included in the print.}
  \item{actors}{vector of integers: set of actors to be included in the plot;
       if \code{NULL}, all actors.}
  \item{col}{Colors used in the plot. If \code{col=NULL} a default color
       scheme is used.}
  \item{addPieChart}{Boolean: If \code{TRUE}, pie charts of aggregated relative
       importances for the complete set of actors will be added to the plot.}
  \item{radius}{Radius of pie charts. Only effective if \code{addPieCharts = TRUE}.}
  \item{width}{Width of the plot. If \code{width=NULL} a default value is used.}
  \item{height}{Height of the plot. If \code{height=NULL} a default value is used.}
  \item{legend}{Boolean: if \code{TRUE} a legend is added to the plot.
        \code{x$effectNames} are used as labels.}
  \item{legendColumns}{Number of columns in legend. If \code{legendColumns=NULL}
        a default value is used. Only effective if \code{legend=TRUE}.}
  \item{legendHeight}{Height of legend. If \code{legendHeight=NULL} a default
        value is used. Only effective if \code{legend=TRUE}.}
  \item{cex.legend}{Specifies the relative font size of legend labels.}
  \item{cex.names}{Specifies the relative font size of bar graph labels.}
  \item{\dots}{Other arguments.}
}
\details{\code{interpret_size} works only for estimation by the Method of Moments
      using \code{\link{modelType}} 1 (directed networks) or 2
      ('forcing' model for non-directed networks).
      For dependent behavior variables, \code{behModelType=1} ('standard') is assumed.
      It does not yet work for endowment or creation effects
      (i.e., included effects have to be of type \code{eval}) or \code{rate},
    and also not for models with interaction effects.
    For two-mode (bipartite) networks as dependent variables,
    it works only if the number of second-mode nodes is less than the
    number of actors.

    If there are any missing tie values in the network data set, they are imputed by
    initial zeros and Last Observation Carried Forward. Structural zeros and ones
    are replaced by \code{NA} and treated as impossible choices in the probability
    vectors and ignored in the standard deviations; but the change statistics
    for these dyads still are given in \code{changeStatistics} (if requested).

    The averages reported in the components \code{sigmas} (average across actors)
    and \code{meansigmas} (average across waves) are obtained by averaging at the
    variance level and then taking square roots.
}
\value{
 If the model contains only one dependent variable, \code{interpret_size} returns
 an object of class \code{sienaRI}. Otherwise, it returns a list of objects
 of class \code{sienaRI}, each corresponding to one dependent variable.

 A returned \code{sienaRI} object stores the expected relative importances of
 effects of one dependent variable at observation moments as defined in
 Section 3.1 of Indlekofer and Brandes (2013).

A \code{sienaRI} object is a list with the following components.
For the components referred to as lists themselves, these are lists
corresponding to the observation moments.
    \describe{
    \item{- \code{dependentVariable}}{the name of the corresponding
          dependent variable.}
    \item{- \code{effectNames}}{the names of considered effects.}
    \item{- \code{RIActors}}{a list that contains the expected
          relative importances of effects
          for each potential actor decision at observation moments.
          This is equation (3) in Indlekofer and Brandes (2013).}
    \item{- \code{expectedRI}}{a list that contains the expected
          relative importances of effects aggregated over all actors for each
          network observation. These are the averages of the actor related
          values in \code{RIActors}.
          This is equation (4) in Indlekofer and Brandes (2013).}
    \item{- \code{IActors}}{a list that contains the expected
          importances of effects for each potential actor decision at
          observation moments. This is the numerator of equation (3)
          in Indlekofer and Brandes (2013).}
    \item{- \code{expectedI}}{a list that contains the expected
          importances of effects
          aggregated over all actors in each observation.
          More precisely, it contains the averages of the actor related values
          in \code{IActors}.}
    \item{- \code{absoluteSumActors}}{a list that contains the sum of
          the (unstandardized) L1-differences
          calculated for each potential actor decision at observation moments.
          This is the denominator of equation (3)
          in Indlekofer and Brandes (2013).}
    \item{- \code{RHActors}}{a list that contains the degree of certainty,
          also called degree of determination, in
          the potential ministep taken by an actor at the observation moments;
          this is R_H(i,x) of formula (6) in Snijders (2004).
          The mean over actors of these degrees of certainty, given by
          formula (7) in Snijders (2004), is printed by the
          \code{print} method for \code{interpret_size} objects.}
    \item{- \code{sigma}}{a list of effects by ego matrices of the values of
          the within-ego standard deviations of the change statistics.}
    \item{- \code{sigmas}}{effects by wave matrices of averages
         (see above for how this is done; across actors) of \code{sigma}.
          These are printed if \code{printSigma=TRUE}.}
    \item{- \code{meansigmas}}{average (see above for how this is done)
          over waves of \code{sigmas}.}
    \item{- \code{changeStatistics}}{a list of arrays
          (effects by choices by egos)
          containing, for each observation wave,
          the values of the change statistics for making this choice,
          where for one-mode networks the choice is defined by the alters
          with alter=ego referring to 'no change',
          for two-mode networks the choice is defined by the second-mode nodes
          with the last choice referring to 'no change',
          and for behavior the choice is defined as
          going down, staying constant, or going up;
          this output is produced only if \code{getChangeStats=TRUE}.}
    \item{- \code{toggleProbabilities}}{an array
         (egos by choices by waves), where "choices" are as directly above,
         giving the choice probabilities of ego in a ministep,
         when the data are as in this wave.}
    }
}

\references{
 Indlekofer, N. and Brandes, U. (2013),
 Relative Importance of Effects in Stochastic Actor-oriented Models.
 \emph{Network Science}, \bold{1}, 278--304.

 Snijders, T.A.B. (2004),
  Explained Variation in Dynamic Network Models.
  \emph{Mathematics and Social Sciences}, \bold{168}, 31--41.
}
\author{Natalie Indlekofer, Tom Snijders, Daniel Gotthardt}
\examples{
myalgo <- set_algorithm_saom(nsub=1, n3=50, seed=1293)
mynet1 <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
mydata <- make_data_rsiena(mynet1)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, list(density, recip, outAct, inPop))
(myeff <- set_effect(myeff, reciAct, parameter=1))
ans <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE)
RI <- interpret_size(ans, mydata)
RI
print(RI, printSigma=TRUE)
# average within-ego standard deviations of change statistics by wave:
RI$sigmas
# sigma averaged over waves:
RI$meansigmas
# to get the change statistics:
RI.cs <- interpret_size(ans, mydata, getChangeStats=TRUE)
# For the network at the first wave:
dim(RI.cs$changeStatistics[[1]])
# E.g., the change statistics for the first actor (note the 0 first column):
RI.cs$changeStatistics[[1]][,1,]
# semi-standardized parameters by wave:
ans$theta * RI$meansigmas
\dontrun{
plot(RI, addPieChart=TRUE)
plot(RI, actors=1:20, addPieChart=TRUE, radius=1.08)
}
# For two dependent variables:
myalgo <- set_algorithm_saom(nsub=1, n3=50, seed=1293)
mynet2 <- as_dependent_rsiena(array(c(s502, s503), dim=c(50, 50, 2)))
mybeh <- as_dependent_rsiena(s50a[,2:3], type="behavior")
mydata2 <- make_data_rsiena(mynet2, mybeh)
myeff2 <- make_specification(mydata2)
myeff2 <- set_effect(myeff2, list(density, recip, transTies))
myeff2 <- set_effect(myeff2, avAlt, depvar="mybeh", covar1="mynet2")
ans2 <- siena(mydata2, effects=myeff2, control_algo=myalgo, batch=TRUE)
# Use only the parameters for the evaluation function:
(theta.eval <- coef(ans2, dropRates=TRUE))
RI <- interpret_size(myeff2, data=mydata2, theta=theta.eval)
RI[[1]]
RI[[2]]
RI[[1]]$sigmas
RI[[2]]$sigmas
\dontrun{
plot(RI[[2]], col = c("red", "green"), legend=FALSE)
plot(RI[[1]], addPieChart = TRUE, legendColumns=2)
}
}
\keyword{models}

\name{make_specification}
\alias{getEffects}
\alias{sienaEffects}
\alias{sienaGroupEffects}
\alias{make_specification}
\alias{make_specification.sienadata}
\alias{make_specification.sienaGroup}
\title{Function to create a Siena effects object}
\description{
  Creates a basic list of effects for all dependent
  variables in the input siena object.\cr
  The functions \code{make_specification} and \code{getEffects}
  are identical. \cr
  The first name is preferred, 
  the other is kept for backward compatibility.
}
\usage{
\method{make_specification}{sienadata}(x, nintn = 10, behNintn=4, onePeriodSde=FALSE, ...)

\method{make_specification}{sienaGroup}(x, nintn = 10, behNintn=4, onePeriodSde=FALSE, ...)

getEffects(x, nintn = 10, behNintn=4, getDocumentation=FALSE, 
           onePeriodSde=FALSE)
}
\arguments{
  \item{x}{an object of class "sienadata" or "sienaGroup"}
  \item{nintn}{Number of user-defined network interactions
       that can later be created.}
  \item{behNintn}{Number of user-defined behavior interactions
       that can later be created.}
  \item{onePeriodSde}{Flag to indicate that the stochastic
       differential equation (SDE) model dZ(t) = [aZ(t) + b] dt + g dW(t)
       should be used, instead of the regular SDE with a scale parameter.
       This is only relevant in case the model includes a continuous
       dependent variable and one period is studied.}
  \item{getDocumentation}{Flag to allow documentation of internal
       functions, not for use by users.}
  \item{...}{Additional arguments, not used now.}
}
\details{
Creates a data frame of effects for use in siena model estimation.
The regular way of changing this object is by the functions
\code{\link{includeEffects}}, \code{\link{setEffect}},
and \code{\link{includeInteraction}}.

Note that the class of the return object may be lost if the data.frame
is edited using \command{fix}.
See \code{\link{fix}} and \code{\link{edit.data.frame}}.
}
\value{An object of class \code{sienaEffects} or \code{sienaGroupEffects}:
this is a data frame of which the rows are the effects available
for data set \code{x}.\cr
  The effects object consists of consecutive parts, each of which relates
  to one dependent variable in the input object.
  The columns are:
  \item{name}{name of the dependent variable}
  \item{effectName}{name of the effect}
  \item{functionName}{name of the function}
  \item{shortName}{short name for the effect}
  \item{interaction1}{second variable to define the effect, if any}
  \item{interaction2}{third variable to define the effect, if any}
  \item{type}{"eval", "endow", "creation", "rate", or "gmm"}
  \item{basicRate}{boolean: whether a basic rate parameter}
  \item{include}{boolean: include in the model to be fitted or not}
  \item{randomEffects}{boolean: random or fixed effect. Currently not used.}
  \item{fix}{boolean: fix parameter value or not}
  \item{test}{boolean: test parameter value or not}
  \item{timeDummy}{comma separated list of periods, or "all",
    or "," for none --
    which time dummy interacted parameters should be included?}
  \item{initialValue}{starting value for estimation, also used for
    \code{fix} and \code{test}.}
  \item{parm}{internal effect parameter values}
  \item{functionType}{"objective" or "rate"}
  \item{period}{period for basic rate parameters}
  \item{rateType}{"Structural", "covariate", "diffusion"}
  \item{untrimmedValue}{Used to store initial values which could be
    trimmed}
  \item{effect1}{Used to indicate effect number in user-specified interactions}
  \item{effect2}{Used to indicate effect number in user-specified interactions}
  \item{effect3}{Used to indicate effect number in user-specified interactions}
  \item{interactionType}{Defines "dyadic" or "ego" or "OK" effects, used in
    \code{\link{includeInteraction}}}
  \item{local}{whether a local effect; used for the option \code{localML}
    in \code{\link{set_algorithm_saom}}}
  \item{effectFn}{here NULL, but could be replaced by a function later}
  \item{statisticFn}{here NULL, but could be replaced by a function
    later}
  \item{netType}{Type of dependent variable:
    "oneMode", "behavior", or "bipartite"}
  \item{groupName}{name of relevant group data object}
  \item{group}{sequential number of relevant group data object in total}
  \item{effectNumber}{the sequence number of the row}
  The combination of \code{name}, \code{shortName}, \code{interaction1},
  \code{interaction2}, and \code{type} uniquely identifies any effect
  other than basic rate effects and user-specified interaction effects.
  For the latter, \code{effect1}, \code{effect2} and \code{effect3}
  are also required for the identification. The combination \code{name},
  \code{shortName}, \code{period} and \code{group} uniquely identifies a
  basic rate effect.

  The columns not used for identifying the effect define 
  how the effect is used for the estimation.

  The columns \code{initialValue} and \code{parm} should not be confused:
  \code{initialValue} gives the initial value for the parameter to be estimated,
  indicated in the manual by \code{theta};
  \code{parm} gives the internal value of the parameter defining the effect,
  indicated in the manual (Chapter 12) by \code{p}, and is fixed
  during the estimation.

  Note that if an effects object is printed by \code{print(...)},
  by default only the included rows are printed.

  A list of all effects in a given effects object (e.g., \code{myeff}),
  including their names of dependent variables, effect names, short names,
  and values of interaction1 and interaction2 (if any),
  is obtained by executing \code{\link{effectsDocumentation}(myeff)}.

  As from version 1.3.24, effects object have a "version" attribute.
  Effects objects including interaction effects created by
  \code{\link{set_interaction}} are not necessarily compatible
  between versions of \code{RSiena}. Therefore it is recommended,
  for effects objects including any interaction effects,
  to create them again when changing to a new version of \code{RSiena}.
  If an effects object including any interaction effects is used from an
  old version of \code{RSiena}, this will lead to a warning when running
  \code{\link{siena}}. 
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley, Tom Snijders}
\seealso{\code{\link{make_data_rsiena}}, \code{\link{make_group_rsiena}},
        \code{\link{set_effect}}, \cr
        \code{\link{includeGMoMStatistics}},
        \code{\link{updateSpecification}},
        \code{\link{print.sienaEffects}}, \cr
        \code{\link{effectsDocumentation}}}
\examples{
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <- as_dependent_rsiena(s50a, type="behavior")
mycovar <- as_covariate_rsiena(rnorm(50))
mydyadcovar <- as_covariate_rsiena(
                        matrix(as.numeric(rnorm(2500) > 2), nrow=50))
mydata <- make_data_rsiena(mynet1, mybeh, mycovar, mydyadcovar)
myeff <- make_specification(mydata)
myeff
}

% Add one or more standard keywords, see file "KEYWORDS" in the
% R documentation directory.
\keyword{classes}
\keyword{methods}

\name{edit.sienaEffects}
\alias{edit.sienaEffects}
\title{
  Allow editing of a sienaEffects object if a gui is available.
}
\description{
  Interactive editor for an effects object. A wrapper to edit.data.frame.
}
\usage{
\method{edit}{sienaEffects}(name, ...)
}
%- maybe also "usage" for other objects documented here.
\arguments{
  \item{name}{An object of class \code{sienaEffects}}
  \item{\dots}{For extra arguments (none used at present)
}}
\details{Will be invoked by \code{fix(name)} for an object of class
  \code{sienaEffects}.
}
\value{
  The updated object. There is no backup copy, and the edits cannot be undone.
}
\seealso{\code{\link{make_specification}}}
\references{
  See \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Ruth Ripley
}
\examples{
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <- as_dependent_rsiena(s50a, type="behavior")
mycovar <- as_covariate_rsiena(rnorm(50))
mydyadcovar <- as_covariate_rsiena(
                  matrix(as.numeric(rnorm(2500) > 2), nrow=50))
(mydata <- make_data_rsiena(mynet1, mybeh, mycovar, mydyadcovar))
myeff <- make_specification(mydata)
\dontrun{
fix(myeff)
}
}
\keyword{methods}

\name{as_nodeset_rsiena}
\alias{as_nodeset_rsiena}
\alias{sienaNodeSet}
\title{Function to create a node set }
\description{Creates a Siena node set which can be used as the nodes in a  
  Siena network.\cr
  The functions \code{as_nodeset_rsiena} and \code{sienaNodeSet}
  are identical. The first name is preferred, 
  the other is kept for backward compatibility.
}
\usage{
as_nodeset_rsiena(n, nodeSetName="Actors", names=NULL)

sienaNodeSet(n, nodeSetName="Actors", names=NULL)
}
%- maybe also "usage" for other objects documented here.
\arguments{
  \item{n}{integer, size of set.}
  \item{nodeSetName}{character string naming the node set.}
  \item{names}{optional character string vector of length n of the names of the
 nodes.}
}
\details{
  This function is important for data sets having more than one node set,
  but not otherwise.
}
\value{
  Returns a Siena node set, an integer vector, possibly with names, plus the
  attributes, class equal to "as_nodeset_rsiena", and nodeSetName equal to
  the argument nodeSetName.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/ }}
\author{Ruth Ripley }
\seealso{\code{\link{as_dependent_rsiena}}, \code{\link{make_data_rsiena}}}
\examples{
senders <- as_nodeset_rsiena(50, nodeSetName="senders")
receivers <- as_nodeset_rsiena(30, nodeSetName="receivers")
senders.attribute <- as_covariate_rsiena(rep(1:10, each=5), 
                        type="monadic", nodeSet="senders")
receivers.attribute <- as_covariate_rsiena(rep(1:5, each=6), 
                        type="monadic", nodeSet="receivers")
mynet <- as_dependent_rsiena(
      array(c(s501[,1:30], s502[,1:30]), dim=c(50, 30, 2)),
      nodeSet=c("senders", "receivers"))
(mydata <- make_data_rsiena(mynet, senders.attribute, receivers.attribute,
      nodeSets=list(senders, receivers)))
}
% Add one or more standard keywords, see file "KEYWORDS" in the
% R documentation directory.
\keyword{classes }

\name{interpret_selection}
\alias{interpret_selection}
\alias{interpret_selection.sienaFit}
\alias{selectionTable}
\alias{selectiontable}
\alias{selection.Table}
\alias{selection.table}
\title{Function to construct selection tables for SAOMs}
\description{
 The function \code{selectionTable} constructs a selection table
 which may be helpful for the interpretation of results
 for network dynamics,
 for an estimation result represented by a
 \code{sienaFit} object created by \code{\link{siena}}.
}

\usage{
\method{interpret_selection}{sienaFit}(x, xd, name, vname,
    as.matrix=FALSE,levls=NULL, levls.alt=levls,
    nfirst=x$nwarm+1, multiplier=1,
    include.endow=FALSE, include.creation=FALSE,
    silent=FALSE, ...)
    
selectionTable(x, xd, name, vname,
    as.matrix=FALSE,levls=NULL, levls.alt=levls,
    nfirst=x$nwarm+1, multiplier=1,
    include.endow=FALSE, include.creation=FALSE,
    silent=FALSE)
}
\arguments{
  \item{x}{An object of class \code{\link{sienaFit}}, produced by a
    call to \code{\link{siena}} for a model including
    a behavioral dependent variable;
    or an object of class \code{\link{sienaMeta}}, produced by a
    call to \code{\link{meta_siena}};
    or an object of class \code{sienaBayesFit}, produced by a
    call to \code{sienaBayes}.  }
  \item{xd}{If \code{x} is of class \code{\link{sienaFit}},
    a \code{\link{siena}} data set used to produce \code{x}.\cr
    If \code{x} is of class \code{\link{sienaMeta}} or \code{sienaBayesFit},
    one of the data sets used to produce \code{x}; preferable a representative
    one.}
  \item{name}{character string: name of network dependent variable.}
  \item{vname}{character string: name of actor covariate (should be centered!).}
  \item{as.matrix}{boolean: will the table be returned as a matrix.}
  \item{levls}{levels for ego.}
  \item{levls.alt}{levels for alter.}
  \item{nfirst}{If of class \code{sienaBayesFit}: first run in
       posterior sample used for constructing the table.}
  \item{multiplier}{multiplier for the range of the actor covariate.}
  \item{include.endow}{boolean, used only for models utilizing selection and
        endowment effects: by specifying \code{include.endow=TRUE},
        the sum of evaluation and endowment effects is used.}
  \item{include.creation}{boolean, used only for models utilizing selection and
         creation effects: by specifying \code{include.creation=TRUE},
        the sum of evaluation and creation effects is used.}
  \item{silent}{boolean; if \code{FALSE}, some information
     is reproduced at the console.}
  \item{\dots}{Other arguments.}

}
\details{
 This functions is used for constructing selection tables
 for the interpretation of results for network dynamics
 obtained with the \code{RSiena} or \code{multiSiena} packages.

 In matrix form, each row corresponds to ego's value of the covariate,
 and each column to alter's value.
 The table entries are the joint contributions of covariate effects
 to the objective function, for this covariate and for
 the combinations of ego and alter values.
  Effects currently implemented are \code{"altX"}, \code{"altSqX"},
  \code{"egoX"}, \code{"egoSqX"},
  \code{"egoXaltX"}, \code{"simX"}, \code{"diffX"}, \code{"diffSqX"},
  \code{"higher"}, \code{"sameX"}, \code{"egoDiffX"}, and
  \code{"egoPlusAltX"}.\cr

 If \code{levls} is \code{NULL} (the default), the levels of
 ego's behavior are taken as the integer range of
 the dependent actor variable.
 In most applications \code{levls.alt} will be the same as \code{levls},
 in which case it does not have to be specified.

 The \code{multiplier} is used in case the variable \code{vname} has a different
 natural scale, and the values for ego and alter as reported
 should be multiplied.
 The values given for \code{levls} and \code{levls.alt} are before this
 multiplication, i.e., they are the values as in the data set.

 The data set \code{xd} is only used to get the means and similarity means
 which are subtracted somewhere in the effects.

 This function has its own print method.\cr
 It can be plotted by package \code{autograph}.
 
}

\value{
  An object of class  \code{selectionTable}.\cr
  If \code{as.matrix=TRUE}, this object is a matrix;
  if \code{as.matrix=FALSE} (the default), it is a data frame, where the rows
  are all combinations of \code{levls} and \code{levls.alt}, and the columns:
  \item{ego}{ego's values as a factor;}
  \item{vego}{the values \code{levls}}
  \item{valter}{the values \code{levls.alt}}
  \item{select}{the value of the selection table.}\cr
  This object has an attribute \code{quad}, which specifies whether the plot
  will be a quadratic function.  
}

\references{
  \url{https://www.stats.ox.ac.uk/~snijders/siena/}\cr
  See the manual, Sections 14.2 and 14.4.
}
\author{Tom Snijders}
\seealso{\code{\link{siena}}, \code{\link{meta_siena}} }
\examples{

mynet <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mycov <- as_covariate_rsiena(s50a[,1])
mydata <- make_data_rsiena(mynet, mycov)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, simX, covar1="mycov")
myalgo <- set_algorithm_saom(nsub=2, n3=100, seed=1291)
# nsub=2, n3=100 is used here for having a brief computation, not for practice.
(ans <- siena(mydata, effects=myeff, control_algo=myalgo, 
                                            silent=TRUE, batch=TRUE))
interpret_selection(ans, mydata, "mynet", "mycov")
}
\keyword{models}
\keyword{methods}

\name{allEffects}
\Rdversion{1.1}
\alias{allEffects}
\docType{data}
\title{
Internal data frame used to construct effect objects.
}
\description{
This data frame is used internally to construct effect objects.  
}
\usage{data(allEffects)}
\format{
  A data frame with values for the following 23 variables.
  \describe{
    \item{\code{effectGroup}}{a character vector}
    \item{\code{effectName}}{a character vector}
    \item{\code{functionName}}{a character vector}
    \item{\code{shortName}}{a character vector}
    \item{\code{endowment}}{a logical vector}
    \item{\code{interaction1}}{a character vector}
    \item{\code{interaction2}}{a character vector}
    \item{\code{type}}{a character vector}
    \item{\code{basicRate}}{a logical vector}
    \item{\code{include}}{a logical vector}
    \item{\code{randomEffects}}{a logical vector}
    \item{\code{fix}}{a logical vector}
    \item{\code{test}}{a logical vector}
    \item{\code{timeDummy}}{a character vector, default ","}
    \item{\code{initialValue}}{a numeric vector}
    \item{\code{parm}}{a numeric vector}
    \item{\code{functionType}}{a character vector}
    \item{\code{period}}{a character vector}
    \item{\code{rateType}}{a character vector}
    \item{\code{untrimmedValue}}{a numeric vector}
    \item{\code{effect1}}{a logical vector}
    \item{\code{effect2}}{a logical vector}
    \item{\code{effect3}}{a logical vector}
    \item{\code{interactionType}}{a character vector}
    \item{\code{local}}{a logical vector}
    \item{\code{setting}}{Settings name: '' (no settings), 'primary', 'universal' or the name of the defining covariate.}
  }
}
\details{
Used to define effects. Not for general user use.}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\keyword{datasets}

\name{s50}
\alias{s50}
\docType{data}
\title{Network data: excerpt from "Teenage Friends and Lifestyle Study" data.
}
\description{
An excerpt of the network, alcohol consumption, and smoking data     
for 50 randomly chosen girls from the
Teenage Friends and Lifestyle Study data set. Useful as a small    
example of network and behaviour, for which models can be    
fitted quickly, and for which there are no missing values.
}
\format{
 Adjacency matrix for the network at time points 1, 2, 3; 50 by 3 matrices
 of alcohol consumption and smoking data for the three time points.
}
\source{
\url{https://www.stats.ox.ac.uk/~snijders/siena/s50_data.zip}
}
\references{
West, P. and Sweeting, H. (1995), \emph{Background Rationale and Design of the
West of Scotland 11-16 Study}.
Working Paper No. 52. MRC Medical Sociology Unit Glasgow.

See \url{https://www.stats.ox.ac.uk/~snijders/siena/s50_data.htm}
}
\seealso{\code{\link{s501}}, \code{\link{s502}}, \code{\link{s503}},
         \code{\link{s50a}}, \code{\link{s50s}}
}
\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh  <- as_dependent_rsiena(s50a, type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
mydata
}
\keyword{datasets}

\name{n3401}
\alias{n3401}
\alias{N3401}
\alias{N3403}
\alias{N3404}
\alias{N3406}
\docType{data}
\title{Network data: excerpt from "Dutch Social Behavior Data Set"
of Chris Baerveldt.
}
\description{
Matrices N3401, N3403, N3404, N3406, and HN3401, HN3403, HN3404,  
HN3406 are two waves of networks for four schools 
(numbered 1, 3, 4, 6).  
}
\format{
 Adjacency matrices for the network at two time points.
 The matrices with name N... are the first wave,
 those with name HN... are the second wave.

 There is a tie from pupil i to pupil j if i says that he/she   
 receives and/or gives emotional support from/to pupil j.  
 The data are part of a larger data set (see source below)  
 and were collected under the direction of Chris Baerveldt.
}
\source{
\url{https://www.stats.ox.ac.uk/~snijders/siena/CB_data.zip}
}
\references{
Houtzager, B. and Baerveldt, C. (1999), Just like Normal.
A Social Network Study of the Relation between Petty Crime
and the Intimacy of Adolescent Friendships.
\emph{Social Behavior and Personality} \bold{27}, 177--192.

Snijders, Tom A.B, and Baerveldt, C. (2003),
A Multilevel Network Study of the Effects of Delinquent Behavior
on Friendship Evolution.
\emph{Journal of Mathematical Sociology} \bold{27}, 123--151.

See \url{https://www.stats.ox.ac.uk/~snijders/siena/BaerveldtData.html}
}
\examples{
mynet <- as_dependent_rsiena(array(c(N3401, HN3401), dim=c(45, 45, 2)))
mydata <- make_data_rsiena(mynet)
}
\keyword{datasets}

\name{print.sienaMeta}
\alias{print.sienaMeta}
\alias{summary.sienaMeta}
\alias{plot.sienaMeta}
\alias{print.summary.sienaMeta}
\alias{meta.table}
\title{Methods for processing sienaMeta objects}
\usage{
\method{print}{sienaMeta}(x, file=FALSE, reportEstimates=FALSE, ...)

\method{summary}{sienaMeta}(object, file=FALSE, extra=TRUE, ...)

\method{print}{summary.sienaMeta}(x, file=FALSE, extra=TRUE, ...)

\method{plot}{sienaMeta}(x, ..., which = 1:length(x$theta),
                         useBound=TRUE, layout = c(2,2))
meta.table(x, d=3, option=2,
    filename=paste(deparse(substitute(x)),'_global.tex',sep=""), align=TRUE)
}
\description{
  \code{print}, \code{summary}, and \code{plot} methods for   
   \code{sienaMeta} objects; and a function to write a LaTeX table.
}
\arguments{
  \item{object}{An object of class \code{sienaMeta}.}
  \item{x}{An object of class \code{sienaMeta}, or
    \code{summary.sienaMeta} as appropriate.}
  \item{file}{Boolean: if TRUE, sends output to file named
    \code{x$projname.txt}. If FALSE, output is to the terminal.}
  \item{reportEstimates}{Boolean: whether to report all estimates
    and standard errors.}
  \item{extra}{Boolean: if TRUE, prints more information.}
  \item{which}{Set of effects contained in the plot
    (given by sequence numbers).}
  \item{useBound}{Boolean: whether to restict plotted symbols to the bound
    used in the call of \code{sienaMeta}.}
  \item{layout}{Vector giving number of rows and columns
    in the arrangement of the several panels in a
    rectangular array, possibly spanning multiple pages.}
  \item{d}{Number of decimals to be used in table.}
  \item{option}{1: results without normality assumptions;
    2: results with normality assumptions, with confidence intervals;
    3: results with normality assumptions, with standard errors.}
  \item{filename}{filename for output; if "", printed to the console.}
  \item{align}{Whether to align numbers at the decimal point.}
  \item{\dots}{For extra arguments (none used at present).}
}
\value{
 The function \code{print.sienaMeta} prints details of the merged
 estimates of the meta-analysis carried out by \code{\link{meta_siena}},
 with test statistics. See the help page for \code{\link{meta_siena}}
 for what is produced by this function.

 The function \code{summary.sienaMeta} prints details as for the
 \code{print} method, but also details of the \code{sienaFit} objects
 included.

 Output from either can be directed to a file by using the argument
 \code{file}. It will be appended to any existing file of the same
 name: \code{outputName.txt}  where \code{outputName} is the value of the
 argument to \code{meta_siena}.

 The function \code{meta.table} writes a combined table of results
 for all parameters to a LaTeX file in (as default) the current working
 directory. This table is a more compact version of
 the results presented by \code{print.sienaMeta}.

 The function \code{plot.sienaMeta} plots estimates against standard
 errors for each effect, with reference lines added at the two-sided
 significance threshold 0.05. It returns an object of class \code{trellis},
 of the \pkg{lattice} package. Effects for which a score test
 was requested are not plotted.\cr
 Another funnel plot, not using \code{\link{meta_siena}},
 is available as \code{\link{funnelPlot}}.
}
\references{ 
Snijders, T.A.B, and Baerveldt, C. (2003),
A Multilevel Network Study of the Effects of Delinquent Behavior
on Friendship Evolution.
\emph{Journal of Mathematical Sociology} \bold{27}, 123--151.

  See also the Siena manual and
  \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Ruth Ripley, Tom Snijders
}
\seealso{\code{\link{meta_siena}}}

\examples{
\dontrun{
# A meta-analysis for three groups does not make much sense
# for generalizing to a population of networks,
# but it the Fisher combinations of p-values are meaningful.
# But using three groups shows the idea.
Group1 <- as_dependent_rsiena(array(c(N3401, HN3401), dim=c(45, 45, 2)))
Group3 <- as_dependent_rsiena(array(c(N3403, HN3403), dim=c(37, 37, 2)))
Group4 <- as_dependent_rsiena(array(c(N3404, HN3404), dim=c(33, 33, 2)))
dataset.1 <- make_data_rsiena(Friends = Group1)
dataset.3 <- make_data_rsiena(Friends = Group3)
dataset.4 <- make_data_rsiena(Friends = Group4)
effects.1 <- make_specification(dataset.1)
effects.3 <- make_specification(dataset.3)
effects.4 <- make_specification(dataset.4)
effects.1 <- set_effect(effects.1, transTrip)
effects.1 <- set_effect(effects.1, transRecTrip, fix=TRUE, test=TRUE)
effects.3 <- set_effect(effects.3, transTrip)
effects.3 <- set_effect(effects.3, transRecTrip, fix=TRUE, test=TRUE)
effects.4 <- set_effect(effects.4, transTrip)
effects.4 <- set_effect(effects.4, transRecTrip, fix=TRUE, test=TRUE)
ans.1 <- siena(dataset.1, effects=effects.1, batch=TRUE)
ans.3 <- siena(dataset.3, effects=effects.3, batch=TRUE)
ans.4 <- siena(dataset.4, effects=effects.4, batch=TRUE)
ans.1
ans.3
ans.4
meta <- meta_siena(ans.1, ans.3, ans.4)
print(meta, reportEstimates=FALSE)
print(meta)
summary(meta)
# For specifically presenting the Fisher combinations:
# First determine the number of estimated effects:
(neff <- sum(sapply(meta, function(x){ifelse(is.list(x),
        !is.null(x$cjplus),FALSE)})))
Fishers <- t(sapply(1:neff,
     function(i){c(meta[[i]]$cjplus, meta[[i]]$cjminus,
                   meta[[i]]$cjplusp, meta[[i]]$cjminusp, 2*meta[[i]]$n1 )}))
Fishers <- as.data.frame(Fishers, row.names=names(meta)[1:neff])
names(Fishers) <- c('Fplus', 'Fminus', 'pplus', 'pminus', 'df')
Fishers
round(Fishers, 4)
# For plotting:
plo <- plot(meta, layout = c(3,1))
plo
plo[3]
# Show effects of bound (bounding at 0.4 is not reasonable, just for example)
meta <- meta_siena(ans.1, ans.3, ans.4, bound=0.4)
plot(meta, which=c(2,3), layout=c(2,1))
plot(meta, which=c(2,3), layout=c(2,1), useBound=FALSE)
meta.table(meta, option=3, file='')
}}
\keyword{print}

\name{update_theta}
\alias{update_theta}
\alias{updateTheta}
\alias{update_theta.sienaEffects}
\alias{update_specification}
\alias{updateSpecification}
\alias{update_specification.sienaEffects}
\title{
A function to update the initial values of the parameters, and a function
to update an effects object.
}
\description{
  \code{update_theta} copies the final values of 
  the parameter estimates for any matching selected effects
  from a \code{\link{sienaFit}} object to a Siena effects object.\cr
  \code{update_specification} includes in a Siena effects object
  a set of effects that are included in another effects object.\cr
  The functions \code{update_theta} and
  \code{updateTheta} are identical;\cr
  the same holds for
  \code{update_specification} and \code{updateSpecification}.
  The first of these names are preferred,
  the others are kept for backward compatibility.
}
\usage{
\method{update_theta}{sienaEffects}(x, prevAns, varName=NULL, ...)
\method{update_specification}{sienaEffects}(x, effects.from,
         effects.extra=NULL, name.to=NULL, name.from=NULL, ...)

updateTheta(effects, prevAns, varName=NULL)
updateSpecification(effects.to, effects.from,
         effects.extra=NULL, name.to=NULL, name.from=NULL)
}
\arguments{
  \item{x}{Object of class \code{\link{sienaEffects}}.}
  \item{prevAns}{Object of class \code{\link{sienaFit}}
    as returned by \code{\link{siena07}}.}
  \item{varName}{Character string or vector of character strings;
       is this is not \code{NULL}, the update will only applied to this
       dependent variable / these dependent variables.}
  \item{effects.to}{Object of class \code{\link{sienaEffects}}.}
  \item{effects.from}{Object of class \code{\link{sienaEffects}}.}
  \item{effects.extra}{Object of class \code{\link{sienaEffects}}.}
  \item{name.to}{Character string, name of dependent variable in \code{object.to}.}
  \item{name.from}{Character string, name of dependent variable in
       \code{object.from}.}
  \item{effects}{Object of class \code{\link{sienaEffects}}.}
  \item{...}{Additional arguments, not used now.}
}
\details{
  The initial values of any selected effects
  in the input effects object which match an effect estimated in
  \code{prevAns} will be updated by \code{update_theta}
  and set to the parameter estimates obtained in \code{prevAns}.\cr
  If the previous run was conditional, the estimated rate parameters for
  the dependent variable on which the run was conditioned are added to
  the final value of theta. If \code{varName} is not \code{NULL},
  this update is restricted to effects for the dependent variable/s
  specified by \code{varName}.
  \cr
  By \code{update_specification}, the effects included in \code{effects.from}
  are also included in \code{x} (for \code{update_specification}) \code{effects.to}
  (for \code{updateSpecification}); if \code{name.to} and/or
  \code{name.from} is specified, this is restricted to effects
  for those dependent variables.\cr
  If \code{effects.from} contains interaction effects, the corresponding
  main effects will be looked for in \code{effects.from}; if they are not
  found there, they will be looked for in \code{effects.extra}.
  It is not guaranteed that this will be successful.
  For \code{effects.extra}, it is best to use an effects object constructed for
  the same data set as \code{effects.from}, and by the same version of RSiena.
}
\value{
  Updated effects object.
}
\note{Using \code{update_theta} explicitly before calling \code{\link{siena}}
  rather than using it via the argument
  \code{prevAns} of \code{\link{siena}} will not permit the use of
  the previous derivative matrix for option \code{Dolby}.
  In most cases, using \code{\link{siena}} with \code{prevAns}
  will therefore be more efficient;
  but sometimes the use of  \code{update_theta} gives better results.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley, Tom A.B. Snijders}
\seealso{
    \code{\link{siena}}, \code{\link{make_specification}}
}
\examples{
## For updateTheta:
mynet1 <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
mydata <- make_data_rsiena(mynet1)
myeff1 <- make_specification(mydata)
myeff1 <- set_effect(myeff1, transTrip)
myeff1 <- set_interaction(myeff1, list(recip, inPop))
myalgo <- set_algorithm_saom(nsub=1, n3=100, seed=1291)
(ans <- siena(mydata, effects=myeff1, control_algo=myalgo,
                                          batch=TRUE))
ans$theta
(myeff <- update_theta(myeff1, ans))
##
## For updateSpecification:
myeff2 <- make_specification(mydata)
myeff2 <- set_effect(myeff2, outAct)
update_specification(myeff2, myeff1)
}
\keyword{models}

\name{effectsDocumentation}
\alias{effectsDocumentation}
\title{
Function to create a table of documentation of effect names, short names etc.
}
\description{
  Produces a table of the shortnames and other information for effects,  
  either in html or latex.
}
\usage{
effectsDocumentation(effects = NULL, type = "html", display = (type=="html"),
     filename = ifelse(is.null(effects), "effects", 
                                         deparse(substitute(effects))))
}
%- maybe also "usage" for other objects documented here.
\arguments{
  \item{effects}{
    A Siena effects object, or NULL.
}
  \item{type}{
    Type of output required. Valid options are "html" or "latex".
}
\item{display}{
  Boolean: should the output be displayed after creation. Only
  applicable to html output.
}
\item{filename}{
  Character string denoting file name.
}
}
\details{
  If \code{effects=NULL}, the \code{allEffects} object is written to
  a table, either latex or html. This table presents all the available
  effects present in this version of RSiena, not delimited
  by a particular data set. The default file name is "effects.tex" or
  "effects.html", respectively.

  The table lists all effects, with their name,
  shortName, whether an endowment (and creation) effect exists,
  the value of an effect parameter - if any -,
  and the interactionType (which can be empty or: "ego" or "dyadic" for
  dependent network variables; "OK" for dependent behavior variables).
  The latter is important for knowing how the effects can be used
  in interaction effects. (See \code{\link{includeInteraction}}).

  If an existing effects object is specified for \code{effects}, then all
  available effects in this effects object are listed.
  This table lists the name (i.e., dependent variable), effect name,
  shortName, type (rate/evaluation/endowment/creation),
  the variables defined as \code{interaction1} and \code{interaction2}
  (see \code{\link{includeEffects}}) that specify this effect,
  the value of an effect parameter - if any -, and the interactionType.\cr
  The GMoM effects, which are those with \code{type=gmm}, are listed at the end.
  For these, the distinction between the fields \code{name} and
  \code{interaction1}, referring to the dependent and the explanatory roles
  of the variables, has no meaning.\cr
  The default root file name is the name of the input effects object.
}
\value{
  Nothing returned. Output files are created in the current working directory.
}
\seealso{\code{\link{make_specification}}, \code{\link{includeEffects}},
    \code{\link{summary.sienaEffects}}, \cr
    \code{\link{includeInteraction}}.}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley, Tom A.B. Snijders}
\examples{
\dontrun{effectsDocumentation()}
}
% Add one or more standard keywords, see file "KEYWORDS" in the
% R documentation directory.
\keyword{print}

\name{as_covariate_rsiena}
\alias{as_covariate_rsiena}
\alias{covariate}
\title{Function to create a covariate object}
\description{
  This function creates a monadic or dyadic covariate object
}
\usage{
as_covariate_rsiena(val, type="monadic", centered=TRUE, 
                                nodeSet="Actors", warn=TRUE, 
                                imputationValues=NULL)
}
\arguments{
  \item{val}{Covariate values, given as a vector, or matrix
    (may be sparse, of type \cr
    \code{"TsparseMatrix"}), or 3-dimensional array, 
    or list of 3 sparse matrices 
    of type \code{"TsparseMatrix"}; which of these structures
    determines the kind of covariate (see Details).}
  \item{type}{\code{"monadic"}, \code{"oneMode"} or \code{"bipartite"}: 
    whether the 
    values refer to a monadic covariate, and in case of a dyadic
    covariate, whether it is for a one-mode or a bipartite 
    (two-mode) network (see Details).}
  \item{centered}{Boolean: if TRUE, then the mean value is subtracted.}
  \item{nodeSet}{Name of node set: character string;
    for \code{type="bipartite"}, referring to
    bipartite (two-mode) dyadic covariates, a vector of two node set names
    (character strings).\cr  
    If the entire data set contains more than one node set,
    then the node sets must be specified in all data objects.}
  \item{warn}{Logical: determines whether a warning given if all values are 
   \code{NA}, or all non-missing values are the same.}
  \item{imputationValues}{For monadic covariates only:
    Vector or matrix of covariate values of same
    length or dimensions as \code{val}, to be used for imputation of
    \code{NA} values (if any) in \code{val}. Must not contain
    any \code{NA}.}
}
\details{This function determines the kind of covariate
  from the kind of arguments given:
  \itemize{
  \item if \code{val} is a vector, 
         it will be a time-constant monadic (actor) covariate; 
  \item if \code{val} is a matrix and \code{ type="monadic"},
     it will be a changing monadic (actor) covariate;  
  \item if \code{val} is a matrix and 
     \code{type="oneMode"} or \code{"bipartite"}, 
     it will be a time-constant dyadic covariate; 
  \item if \code{val} is a  3-dimensional array or list of sparse matrices 
    of type \code{"TsparseMatrix"}, it will be a changing dyadic covariate. 
 }
     
  When part of a \code{siena} data object,
  the created covariate is associated with the node set
  \code{nodeSet} of this data object.
  In practice, the node set needs to be specified only in the case of
  the use of the covariate with a two-mode network.
  
  If there are any \code{NA} values in \code{val}, and
  \code{imputationValues} is given, and the covariate is monadic,
  then the corresponding elements
  of \code{imputationValues} are used for imputation.
  If \code{imputationValues} is \code{NULL}, imputation
  is by the mean value.  In both cases, cases with imputed values
  are not used for calculating target statistics (see the manual).
  
  For changing covariates (monadic or dyadic), the values for wave \code{m} 
  are supposed in the simulations to be valid in the whole period from 
  wave \code{m} to wave \code{m+1}.
  If the data set has \code{M} waves, this means that the values,
  if any, for wave \code{M} will not be used.  
  These values may be given or omitted as part of \code{val}.
  
  The function replaces \code{\link{coCovar}}, \code{\link{varCovar}},
  \code{\link{coDyadCovar}}, \code{\link{varDyadCovar}},
  which now are obsolete.
}
\value{
  The covariate is returned as an object of class \code{\link{coCovar}}, 
  \code{\link{varCovar}},
  \code{\link{coDyadCovar}}, or  \cr
  \code{\link{varDyadCovar}}, as the case may be.  This object
  can be used as an argument to \code{\link{make_data_rsiena}}.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Tom A.B. Snijders}
\seealso{\code{\link{make_data_rsiena}}, \code{\link{as_nodeset_rsiena}}; 
  this function replaces the functions \code{\link{coCovar}},
  \code{\link{varCovar}}, \code{\link{coDyadCovar}}, \code{\link{varDyadCovar}}.
}
\examples{
# constant monadic covariate:
myconstcovar <- as_covariate_rsiena(s50a[,1])
senders <- as_nodeset_rsiena(50, nodeSetName="senders")
receivers <- as_nodeset_rsiena(30, nodeSetName="receivers")
senders.attribute <- as_covariate_rsiena(rep(1:10, each=5), 
                                         nodeSet="senders")
receivers.attribute <- as_covariate_rsiena(rep(1:5, each=6), 
                                         nodeSet="receivers")
# varying monadic covariate:
myvarcovar <- as_covariate_rsiena(s50a, type="monadic")
senders.covariate <- as_covariate_rsiena(s50a, nodeSet="senders", 
                                               type="monadic")
receivers.covariate <- as_covariate_rsiena(s50s[1:30,], 
                            nodeSet="receivers", type="monadic")
# constant dyadic covariate:
mydyadvar <- as_covariate_rsiena(s503, type="oneMode")
# varying dyadic covariate:
mydyadvar <- as_covariate_rsiena(array(c(s501, s502), dim=c(50, 50, 2)), 
                                type="oneMode")
# using sparse matrices for a constant dyadic covariate:
require(Matrix)
sps501 <- as(s501,"TsparseMatrix")
sps502 <- as(s502,"TsparseMatrix")
sps502r <- as(s502[1:30,],"TsparseMatrix")
spcDyadvar <- as_covariate_rsiena(sps501, type="oneMode")
spvDyadvar <- as_covariate_rsiena(list(sps501,sps502), type="oneMode")
spvDyadBipvar <- as_covariate_rsiena(list(sps501,sps502), 
                   type="bipartite", nodeSet=c("senders","receivers"))
}
\keyword{classes}

\name{xtable}
\alias{xtable}
\title{
Access xtable in package xtable}
\description{
Dummy function to allow access to xtable in package xtable
}
\usage{
xtable(x, ...)
}
\arguments{
  \item{x}{\code{\link{sienaFit}} object

}
\item{\dots}{
  Other arguments for \code{\link{xtable.sienaFit}}
}
}
\value{
  Value returned from \code{\link{xtable.sienaFit}}
}
\references{
\url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley}
\seealso{\code{\link{xtable.sienaFit}}}

\examples{

## The function is currently defined as
function (x, ...)
{
    xtable::xtable(x, ...)
  }
\dontrun{
myalgo <- set_algorithm_saom(nsub=2, n3=100, seed=1293)
mynet <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
mydata <- make_data_rsiena(mynet)
myeff <- make_specification(mydata)
ans <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE)
ans
summary(ans)
xtable(ans, type="html", file="ans.html")
}
}
% Add one or more standard keywords, see file "KEYWORDS" in the
% R documentation directory.
\keyword{print}

\name{sienaFit.methods}
\alias{sienaFit.methods}
\alias{print.summary.sienaFit}
\alias{sienaFit}
\alias{xtable.sienaFit}
\alias{print.xtable.sienaFit}
\alias{siena_table}
\alias{print.sienaFit}
\alias{summary.sienaFit}
\alias{write_result.sienaFit}
\alias{write_result}
\alias{coef}
\alias{coef.sienaFit}
\alias{vcov}
\alias{vcov.sienaFit}
\title{Methods for processing sienaFit objects,
         produced by \code{\link{siena}}
}
\usage{
\method{print}{sienaFit}(x, tstat=TRUE, ...)

\method{summary}{sienaFit}(object, ...)

\method{print}{summary.sienaFit}(x, matrices=TRUE, ...)

\method{coef}{sienaFit}(object, dropRates=TRUE, shortenNames=TRUE, ...)

\method{vcov}{sienaFit}(object, dropRates=TRUE, shortenNames=TRUE, ...)

\method{write_result}{sienaFit}(x, type="tex",
    fileName=NULL,
    vertLine=TRUE, tstatPrint=FALSE, sig=FALSE, d=3, ...)

\method{xtable}{sienaFit}(x, caption = NULL, label = NULL, align = NULL,
    digits = NULL, display = NULL, ...)

siena_table(x, type="tex",
    fileName=NULL,
    vertLine=TRUE, tstatPrint=FALSE, sig=FALSE, d=3, nfirst=NULL)
}
\arguments{
  \item{x}{An object of class \code{sienaFit}, or
    \code{summary.sienaFit} as appropriate. For \code{siena_table},
    objects of class \code{sienaBayes} are also permitted.}
  \item{object}{An object of class \code{\link{sienaFit}}, produced by
    \code{\link{siena}}. For \code{siena_table}, objects of class
    \code{sienaBayes} are also permitted.}
  \item{dropRates}{Logical. Should the results for the basic rate parameters
    also be given?}
  \item{shortenNames}{Logical. Should the effect names
    be shortened?}
  \item{matrices}{Boolean: whether also to print in the summary the
   covariance matrix of the estimates, the derivative matrix of expected
   statistics X by parameters, and the covariance matrix of the statistics.}
  \item{tstat}{Boolean: if this is NULL, the t-statistics for convergence
     will not be added to the report.}
  \item{type}{Type of output to produce; must be either \code{"tex"} or
    \code{"html"}.}
  \item{fileName}{Name of the file; defaults to the name of the \code{sienaFit}
    object. "" indicates output to the console.}
  \item{vertLine}{Boolean: add vertical lines separating the columns in
    \code{siena_table}.}
  \item{tstatPrint}{Boolean: add a column of significance t values (parameter
    estimate/standard error estimate) to \code{siena_table}.}
  \item{sig}{Boolean: adds symbols (daggers and asterisks) indicating
    significance levels for the parameter estimates to \code{siena_table}.}
  \item{d}{The number of decimals places used in \code{siena_table}.}
  \item{caption}{
    See documentation for \code{\link[xtable]{xtable}}.}
  \item{label}{
    See documentation for \code{\link[xtable]{xtable}}.}
  \item{align}{
    See documentation for \code{\link[xtable]{xtable}}.}
  \item{digits}{
    See documentation for \code{\link[xtable]{xtable}}.}
  \item{display}{
    See documentation for \code{\link[xtable]{xtable}}}
  \item{nfirst}{Only relevant for the \code{multiSiena} package.}
  \item{\dots}{
    Add extra parameters for  \code{\link[xtable]{print.xtable}}
    here. e.g.
    \code{type}, \code{file}.}
}
\description{
  \code{print}, \code{summary}, and \code{xtable} methods for
  \code{sienaFit} objects.
}
\value{
  Function \code{coef.sienaFit} gives the estimated parameters,
  and \code{vcov.sienaFit} their estimated variance-covariance matrix.

  The function \code{print.sienaFit} prints a table containing estimated
  parameter values, standard errors and (optionally)
  t-statistics for convergence.

  The function \code{summary.sienaFit} prints a table containing
  estimated parameter values, standard errors and t-statistics for
  convergence together with the covariance matrix of the estimates, the
  derivative matrix of expected statistics \code{X} by parameters, and the
  covariance matrix of the expected statistics \code{X}.

  The function \code{xtable.sienaFit} creates an object of class
  \code{xtable.sienaFit} which inherits from class \code{xtable} and
  passes an extra arguments to the \code{print.xtable}.

  The function \code{siena_table} outputs a latex or html table of
  the estimates and standards errors of a \code{\link{sienaFit}} object.
  The table will be written to a file in the current directory and
  has a footnote reporting the maximum of the convergence t-ratios.
  Endowment or creation effects will be denoted, respectively,
  by 'maintenance' or 'creation'.

  See the manual for how to import the html tables easily into MS-Word.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Ruth Ripley, Charlotte Greenan, Tom Snijders}

\seealso{\code{\link[xtable]{xtable}},
 \code{\link[xtable]{print.xtable}}, \code{\link{siena}}}
\examples{
myalgorithm <- set_algorithm_saom(nsub=2, n3=100)
mynet <-  as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
mydata <- make_data_rsiena(mynet)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, outAct)
ans <- siena(mydata, effects=myeff,
                     control_algo=myalgorithm, batch=TRUE)
ans
coef(ans)
round(vcov(ans), 4)
round(sqrt(diag(vcov(ans))), 4)
summary(ans)
\dontrun{
xtable(ans, type="html", file="ans.html")
write_result(ans, type="html", tstat=TRUE, d=2)
}
}
\keyword{methods}

\name{plot.sienaTimeTest}
\alias{plot.sienaTimeTest}
\title{Functions to plot assessment of time heterogeneity of parameters}
\description{
 Plot method for \code{\link{sienaTimeTest}} objects.
 }

\usage{
\method{plot}{sienaTimeTest}(x, pairwise=FALSE, effects,
     scale=.2, plevels=c(.1, .05, .025), ...)
}
\arguments{
  \item{x}{ A sienaTimeTest object returned by \code{\link{test_time}}. }
  \item{pairwise}{ A Boolean value corresponding to whether the user
  would like a pairwise plot of the simulated statistics to assess
  correlation among the effects (\code{pairwise=TRUE}), or a plot of the
  estimates across waves in order to assess graphically the results of
  the score type test. }
\item{effects}{ A vector of integers corresponding to the indices given
  in the \code{sienaTimeTest} output for effects which are to be
  plotted.}
\item{scale}{ A positive number corresponding to the number of standard
  deviations on one step estimates to use for computing the maximum and
  minimum of the plotting range. We recommend experimenting with this
  number when the y-axes of the plots are not satisfactory. Smaller
  numbers shrink the axes.}
\item{plevels}{ A list of three decimals indicating the gradients at
  which to draw the confidence interval bars.}
 \item{\dots}{For extra arguments. The Lattice parameter \code{layout}
   can be used to control the layout of the graphs.}
}
\details{
 The \code{pairwise=TRUE} plot may be used to assess whether effects are
 highly correlated. This information may be important when considering
 forward-model selection, since highly correlated effects may have
 highly correlated one-step estimates, particularly since the individual
 score type tests are not orthogonalized against the scores and
 deviations of yet-unestimated dummies.  For example, reciprocity and
 outdegree may have highly correlated statistics as indicated by a
 strong, positive correlation coefficient.  When considering whether to
 include dummy terms, it may be a good idea to include, e.g., outdegree,
 estimate the parameter, and see whether reciprocity dummies remain
 significant after method of moments estimation of the updated model--as
 opposed to including both outdegree and reciprocity.

 The \code{pairwise=FALSE} plot displays the most of the information
 garnered from \code{sienaTimeTest} in a graphical fashion.  For each
 effect, the method of moments parameter estimate for the base period
 (i.e. wave 1) is given as a blue, horizontal reference line. One step
 estimates are given for all of the parameters by dots at each wave. The
 dots are colored black if the parameter has been included in the model
 already (i.e. has been estimated via method of moments), or red if they
 have not been included. Confidence intervals are given based on pivots
 given at \code{pvalues}. Evidence of time heterogeneity is suggested by
 points with confidence intervals not overlapping with the base period.
}
\value{None
}

\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}
  for general information on RSiena.

}
\author{Josh Lospinoso}
\seealso{\code{\link{siena}}, \code{\link{test_time}},
  \code{\link[lattice]{xyplot}}}
\examples{
\dontrun{
## Estimate a restricted model
myalgo <- set_algorithm_saom(nsub=2, n3=200)
# It makes no sense to put together the following data set,
# but just for demonstration:
mynet1 <- as_dependent_rsiena(
          array(c(s501, s502, s503, s501, s503, s502), dim=c(50, 50, 6)))
mydata <- make_data_rsiena(mynet1)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, transTrip)
myeff <- includeTimeDummy(myeff, density, timeDummy="all")
myeff <- includeTimeDummy(myeff, recip, timeDummy="2,3,5")
myeff <- includeTimeDummy(myeff, transTrip, timeDummy="2,3")
(ansp <- siena(mydata, effects=myeff, batch=TRUE, control_algo=myalgo))
ttp <- test_time(ansp)
summary(ttp)

## Pairwise plots show
plot(ttp, pairwise=TRUE)

## Time test plots show
plot(ttp, effects=1:3) ## default layout
plot(ttp, effects=1:3, layout=c(3,1))
}
}

\keyword{methods}

\name{includeTimeDummy}
\alias{includeTimeDummy}
\title{Function to include time dummy effects in a Siena model}
\description{
  This function specifies time heterogeneity for selected effects   
  in a Siena model, by interacting them with time dummies,   
  without explicitly using time-dependent covariates.
 }

\usage{
includeTimeDummy(myeff, ..., timeDummy="all", depvar=myeff$name[1], 
       type="eval", covar1="", covar2="", include=TRUE, character=FALSE)
}
\arguments{
  \item{myeff}{A Siena effects object as created by 
              \code{\link{make_specification}}.}
  \item{\dots}{
   Short names to identify the effects for which interactions with time dummies
   should be included or excluded.
   This function cannot be used for regular interaction effects.}
  \item{timeDummy}{Character string. Either "all" or the periods for which
   to create dummies (from 1 to (number of waves - 1)), space delimited.}
  \item{include}{Boolean. default TRUE, but can be switched to FALSE to
    turn off an effect.}
  \item{depvar}{Name of dependent network or behavioral variable for which
    effects are being included. Defaults to the first in the effects object.}
  \item{type}{Type of dummy effects to be interacted.}
  \item{covar1}{
   Name of variable where needed to completely identify the
   effects e.g. covariate name or behavior variable name.}
  \item{covar2}{
   Name of variable where needed to completely identify the
   effects e.g. covariate name or behavior variable name.}
  \item{character}{Boolean: are the effect names character strings or not}
}
\details{
  The arguments (\ldots, \code{name, covar1, covar2})
  should identify the effects completely.\cr
  See \code{\link{make_specification}} and \code{\link{effectsDocumentation}}
  for more information about this.

  This function operates by setting the \code{timeDummy} column
  on selected rows of a Siena effects object, thereby specifying
  interactions of the specified effect or effects with dummy variables
  for the specified periods.
  The \code{timeDummy} column of \code{myeff} will be set to include the
  values requested if \code{include=TRUE}, and to exclude them for
  \code{include=FALSE}.

  For an effects object in which the \code{timeDummy} column
  of some of the included effects includes some or all period numbers,
  interactions of those effects with ego effects of time dummies for the
  indicated periods will also be estimated by \code{\link{siena}}.
  For the outdegree effect this is just the ego effect of the time dummies.
  If \dots does not include the outdegree effect, then still this
  ego effect will be created, but its parameter will be fixed to 0.

  An alternative to the use of \code{\link{includeTimeDummy}}
  is to define time-dependent actor covariates (dummy variables or other
  functions of wave number that are the same for all actors),
  include these in the data set through \code{\link{make_data_rsiena}},
  and include interactions of other effects with ego effects of these
  time-dependent actor covariates by \code{\link{set_interaction}}.
  This is illustrated in an example in the help file for
  \code{\link{test_time}}.
  Using \code{\link{includeTimeDummy}} is easier; on the other hand,
  using self-defined interactions with time-dependent variables
  gives more control (e.g., it will allow to specify linear time dependence
  and test time heterogeneity for interaction effects).
}
\value{
    An updated version of \code{myeff}, with the \code{timeDummy}
    column for one or more rows updated.
    Details of the rows altered will be printed.
}

\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}
  for general information on RSiena.

}
\author{Josh Lospinoso}
\seealso{\code{\link{sienaTimeTest}}, \code{\link{make_specification}},
  \code{\link{includeEffects}}, \code{\link{effectsDocumentation}}.}
\examples{
\dontrun{
## Estimate a restricted model
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mydata <- make_data_rsiena(mynet)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, list(outAct, transTrip))
(ans <- siena(mydata, effects=myeff))

## Conduct the score type test to assess whether heterogeneity is present.
tt <- sienaTimeTest(ans)
summary(tt)

## Suppose that we wish to include a time dummy.
## Since there are three waves, the number of periods is two.
## This means that only one time dummy can be included for
## the interactions. The default is for period 2;
## an equivalent model, but with different parameters
## (that can be transformed into each other) is obtained
## when the dummies are defined for period 1.
myeff <- includeTimeDummy(myeff, density, recip, timeDummy="2")
myeff       # Note the \code{timeDummy} column.
(ans2 <- siena(mydata, effects=myeff))

## Re-assess the time heterogeneity
tt2 <- sienaTimeTest(ans2)
summary(tt2)

## And so on..
}
}
\keyword{models}

\name{RSiena-package}
\alias{RSiena-package}
\alias{RSiena}
\docType{package}
\title{
Simulation Investigation for Empirical Network Analysis
}
\description{
Fits statistical models to longitudinal sets of networks, and to   
longitudinal sets of networks and behavioral variables.   
Not only one-mode networks but also two-mode networks   
and multivariate networks are allowed.   
The models are stochastic actor-oriented models,   
described in Snijders (2017).

Recent versions of the package are distributed through GitHub,   
see \url{https://github.com/stocnet/rsiena/}.

Bug reports can be submitted at 
\url{https://github.com/stocnet/rsiena/issues}.
}
\details{
  The main flow of operations of this package is as follows.

  Data objects can be created from matrices and
  vectors using \code{\link{as_dependent_rsiena}}, \cr
  \code{\link{as_covariate_rsiena}},
  possibly \code{\link{as_composition_rsiena}}, \cr
  and finally \code{\link{make_data_rsiena}}.

  Effects are selected using an \code{\link{sienaEffects}} object,
  which can be created using \code{\link{make_specification}}
  and may be further specified by \code{\link{set_effect}},
  and \code{\link{set_interaction}}.

  Control of the model family, the estimation algorithm,
  and the output produced  requires control objects that
  define the settings of the algorithm,
  and which can be created by \code{\link{set_algorithm_saom}}, 
  \code{\link{set_model_saom}}, and \code{\link{set_output_saom}}.

  Function \code{\link{siena}} is used to fit a model.
  Function \code{\link{test_gof}} can be used for studying goodness of fit.
  
  Version 1.6 has established an overhaul of the function names used,
  while retaining the old function names for backward compatibility.

  A general introduction to the method is available in the tutorial paper
  Snijders, van de Bunt, and Steglich (2010).
  Next to the help pages, more detailed help is available in the manual
  (see below) and a lot of information is at the website (also see below).
  \tabular{ll}{
    Package: \tab RSiena\cr
    Type: \tab Package\cr
    Version: \tab 1.6.4\cr
    Date: \tab 2026-04-02\cr
    Depends: \tab R (>= 4.5.0)\cr
    Imports: \tab Matrix, lattice, parallel, MASS, methods, xtable\cr
    Suggests: \tab network, tools, codetools, tcltk\cr
    SystemRequirements: \tab GNU make\cr
    License: \tab GPL-2 | GPL-3 \cr
    LazyData: \tab yes\cr
    NeedsCompilation: \tab yes\cr
    BuildResaveData: \tab no\cr
  }
}
\author{
  Ruth Ripley, Krists Boitmanis, Tom Snijders, Felix Schoenenberger,
  Nynke Niezink, Christian Steglich, Viviana Amati.
  Contributions by Josh Lospinoso, Charlotte Greenan,
  Johan Koskinen, Mark Ortmann, Natalie Indlekofer, Mark Huisman,
  Christoph Stadtfeld, Per Block, Marion Hoffman, Michael Schweinberger,
  Robert Hellpap, Alvaro Uzaheta, Robert Krause, James Hollway, 
  Daniel Gotthardt, and Steffen Triebel.

  Maintainer: Christian Steglich <c.e.g.steglich@rug.nl>
}
\references{
    Amati, V., Schoenenberger, F., and Snijders, T.A.B. (2015),
    Estimation of stochastic actor-oriented models for the evolution of 
    networks by generalized method of moments.
    \emph{Journal de la Societe Francaise de Statistique} \bold{156}, 140--165.
    
    Amati, V., Schoenenberger, F., and Snijders, T.A.B. (2019),
    Contemporaneous statistics for estimation in stochastic actor-oriented
    co-evolution models. \emph{Psychometrika} \bold{84}, 1068--1096.
    
    Greenan, C. (2015),
    \emph{Evolving Social Network Analysis: developments in statistical
    methodology for dynamic stochastic actor-oriented models}.
    DPhil dissertation, University of Oxford.
    
    Niezink, N.M.D., and Snijders, T.A.B. (2017),
    Co-evolution of Social Networks and Continuous Actor Attributes.
    \emph{The Annals of Applied Statistics} \bold{11}, 1948--1973.
    
    Schweinberger, M., and Snijders, T.A.B. (2007),
    Markov models for digraph panel data:
    Monte Carlo based derivative estimation.
    \emph{Computational Statistics and Data Analysis} \bold{51}, 4465--4483.
    
     Snijders, T.A.B. (2001),
    The statistical evaluation of social network dynamics.
    \emph{Sociological Methodology} \bold{31}, 361--395.
    
    Snijders, T.A.B. (2017), Stochastic Actor-Oriented Models for Network
    Dynamics. \emph{Annual Review of Statistics and Its Application}
    \bold{4}, 343--363.
    
    Snijders, T.A.B., Koskinen, J., and Schweinberger, M. (2010).
    Maximum likelihood estimation for social network dynamics.
    \emph{Annals of Applied Statistics} \bold{4}, 567--588.
    
    Snijders, T.A.B., Steglich, C.E.G., and Schweinberger, Michael (2007), 
    Modeling the co-evolution of networks and behavior.
    Pp. 41--71 in \emph{Longitudinal models in the behavioral
    and related sciences},
    edited by van Montfort, K., Oud, H., and Satorra, A.; Lawrence Erlbaum.
    
    Steglich, C.E.G., Snijders, T.A.B., and Pearson, M.A. (2010),
    Dynamic networks and behavior:     Separating selection from influence.
    \emph{Sociological Methodology} \bold{40}, 329--393.
    Information about the implementation of the algorithm is in
     \url{https://www.stats.ox.ac.uk/~snijders/siena/Siena_algorithms.pdf}.
     
    Further see \url{https://www.stats.ox.ac.uk/~snijders/siena/} and \cr
    \url{https://github.com/stocnet/rsiena/wiki}.
}
\seealso{\code{\link{siena}}}

\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mybeh  <- as_dependent_rsiena(s50a[,1:2], type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, avAlt, depvar="mybeh", covar1="mynet")
myalgo <- set_algorithm_saom(nsub=2, n3=100, seed=1291)
# nsub=2, n3=100 is used here for having a brief computation, not for practice.
(ans <- siena(mydata, effects=myeff, control_algo=myalgo,
                                            silent=TRUE, batch=TRUE))
summary(ans)
}
\keyword{package}

\name{make_constraint}
\alias{make_constraint}
\alias{make_constraint.sienadata}
\alias{make_constraint.sienaGroup}
\alias{sienaDataConstraint}
\title{
Function to change the values of the constraints between networks
}
\description{This function allows the user to change the constraints   
  of "higher", "disjoint" and "atLeastOne" for a specified pair of  
  networks in a Siena data object.\cr  
  The functions \code{make_constraint} and \code{sienaDataConstraint}
  are identical. The first name is preferred,
  the other is kept for backward compatibility.\cr
  The functions \code{make_constraint} and 
  \code{sienaDataConstraint} are identical. 
  The first name is preferred, 
  the other is kept for backward compatibility.
}
\usage{
\method{make_constraint}{sienadata}(x, net1, net2, type, value = FALSE, ...)
\method{make_constraint}{sienaGroup}(x, net1, net2, type, value = FALSE, ...)

sienaDataConstraint(x, net1, net2, type, value = FALSE)
}
\arguments{
  \item{x}{\code{sienadata} object or a \code{sienaGroup} object}
  \item{net1}{name of first network}
  \item{net2}{name of second network}
  \item{type}{one of "higher", "disjoint", "atleastOne".}
  \item{value}{Boolean giving the value.}
  \item{...}{Additional arguments, not used now.}
}
\details{
  The value of the appropriate attribute is set to the value requested.
  Note that, for \code{value=TRUE}, the correspondence of this value 
  to the data is not checked.
}
\value{
  Updated \code{sienadata} object.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley}
\seealso{\code{\link{make_data_rsiena}}, \code{\link{make_group_rsiena}}}
\examples{
nowFriends <- as_dependent_rsiena(
                array(c(s501, s502, s503), dim=c(50, 50, 3)))
ever <- array(c(s501, s502, s503), dim=c(50, 50, 3))
ever[,,2] <- pmax(ever[,,1], ever[,,2])
ever[,,3] <- pmax(ever[,,2], ever[,,3])
everFriends <- as_dependent_rsiena(ever)
# Note: this data set serves to illustrate this function,
# but it is not an appropriate data set for estimation by siena07,
# because everFriends (for the three waves together) 
# depends deterministically on nowFriends (for the three waves together).
nowOrEver <- make_data_rsiena(nowFriends, everFriends)
attr(nowOrEver, "higher")
nowOrEver
nowOrEver.unconstrained <-
 make_constraint(nowOrEver, everFriends, nowFriends, "higher", FALSE)
nowOrEver.unconstrained
attr(nowOrEver.unconstrained, "higher")
}
\keyword{methods}

\name{set_effect} 
\alias{set_effect}
\alias{set_effect.sienaEffects}
\alias{setEffect}
\title{
  Function to change the specification of a Siena model
  by setting various columns in an effects object.
}
\description{
  This function provides an interface to change the 
  specification of a Siena model by setting 
  various columns in an effects object.\cr
  The function \code{set_effect} combines the functionality
  of \code{setEffect} and \code{\link{includeEffects}}.
  \code{set_effect} now is preferred.
  The oher two are kept for backward compatibility.\cr
}
\usage{
\method{set_effect}{sienaEffects}(x, shortNames, type="eval",
            depvar=x$name[1], covar1="", covar2="",
            effect1=0, effect2=0, effect3=0, period=1, group=1, 
            parameter=NULL, initialValue = 0, random=FALSE,
            timeDummy = ",", 
            include=TRUE, fix=FALSE, test=FALSE, verbose=TRUE, ...)

setEffect(x, shortName, parameter = NULL,
    fix = FALSE, test = FALSE, random=FALSE, initialValue = 0,
    timeDummy = ",", include = TRUE,
    name = x$name[1], type = "eval", interaction1 = "",
    interaction2 = "", effect1=0, effect2=0, effect3=0,
    period=1, group=1, character=FALSE, verbose = TRUE)
}
%- maybe also "usage" for other objects documented here.
\arguments{
  \item{x}{a Siena effects object as created by \code{\link{make_specification}}
}
\item{shortNames}{A shortName or list of shortNames
  (all with or all without quotes)
  to identify the effect which should be changed.
}
\item{shortName}{A shortName to identify the effect 
    which should be changed.
}
  \item{depvar}{Name of dependent variable (network or behavior)
    for which interactions are being defined.
    Defaults to the first in the effects object.
  }
  \item{covar1}{
  Name of siena object where needed to completely identify the
  effect e.g. covariate name or behavior variable name.
  }
  \item{covar2}{
  Name of siena object where needed to completely identify the
  effect e.g. covariate name or behavior variable name.
  }
  \item{parameter}{Value of internal effect parameter. If \code{NULL},
  no change is made.
  }
  \item{fix}{For fixing effects. Boolean required. Default \code{FALSE}.
  }
  \item{test}{For testing effects by score-type tests. Boolean required.
  Default \code{FALSE}.
  }
  \item{random}{For specifying that effects will vary randomly;
  used only in function \code{sienaBayes} in package \code{multiSiena}.
  Not relevant for \code{RSiena} at this moment.
  Boolean required. Default \code{FALSE}.
  }
  \item{initialValue}{Initial value for estimation. Default 0.
   }
\item{timeDummy}{string: Comma delimited string of which periods to
   dummy. Alternatively, use \code{\link{includeTimeDummy}}.
  }
  \item{include}{Boolean. default \code{TRUE}, but can be switched to
    \code{FALSE} to turn off an effect.}
  \item{name}{Name of dependent variable (network or behavior)
    for which the effect is being modified.
    Defaults to the first in the effects object, which is the
   first dependent variable specified in \code{\link{make_data_rsiena}}.
}
  \item{type}{Character string indicating the
  type of the effect to be changed : "rate", "eval",
  "endow", or "creation". Default "eval".
}
  \item{interaction1}{
  Name of siena object where needed to completely identify the
  effect e.g. covariate name or behavior variable name.
}
  \item{interaction2}{
  Name of siena object where needed to completely identify the
  effect e.g. covariate name or behavior variable name.
}
\item{effect1}{
  Only for \code{shortName=unspInt, behUnspInt} or \code{contUnspInt},
  which means this is a user-defined interaction effect: \code{effect1} is
  the row number in \code{x} of the first component of
  the interaction effect.
}
\item{effect2}{
  See \code{effect1}: second component of interaction effect.
}
\item{effect3}{
  See \code{effect1}: third component of interaction effect.
}
\item{period}{
  Number of period if basic rate. Use numbering within groups.
}
\item{group}{
  Number of group if basic rate.
  Only relevant for \code{\link{sienaGroup}} data sets.
  }
\item{character}{Boolean: whether the short name is a character string.}
\item{verbose}{Boolean: should the print of altered effects be produced.}
\item{...}{Additional arguments, not used now.}
}
\details{Recall from the help page for \code{\link{make_specification}}
   that a Siena effects object (class \cr
   \code{sienaEffects} or \code{sienaGroupEffects}) 
   is a \code{\link{data.frame}};
   the rows in the data frame are the effects for this data set;
   some of the columns/variables of the data frame
   are used to identify the effect, other columns/variables define how this
   effect is used in the estimation. \cr
   The function \code{set_effect} can operate on several
   effects simultaneously, but in a less detailed way.
   Using \code{set_effect} for only one effect can change not only the
   value of the column \code{include}, but also
   those of \code{initialValue} and \code{parm}.
   The arguments \code{shortName}, \code{name}, \code{type},
   \code{interaction1}, \code{interaction2}, \code{effect1}, \code{effect2},
   \code{effect3}, \code{period}, and \code{group} should identify one
   effect completely. \cr
   (Not all of them are needed; see \code{\link{make_specification}}.)\cr
   The call of \code{setEffect} will set, for this effect,
   the column elements of the resulting effects object for \code{parm},
   \code{fix}, \code{test}, \code{randomEffects}, \code{initialValue},
    \code{timeDummy}, and \code{include} to the values requested.\cr
    For \code{setEffect} the \code{shortName} must not 
    be set between quotes, unless \code{character=TRUE}.

   The input names \code{interaction1} and \code{interaction2}
   for \code{setEffect} do not themselves
   refer to created interactions, but to dependence of the base effects on
   other variables in the data set.
   They are used to completely identify the effects.

   A value for \code{parameter} should be given only if only one effect
   is specified in \code{shortNames}. If it is given,
   the occurrences of \code{#} in the original effect and function names
   are replaced by this value. If a value for \code{parameter} is not given,
   the current value of the internal effect parameter of this effect is used.
}
\value{
  An object of class \code{\link{sienaEffects}} or
  \code{\link{sienaGroupEffects}}. This will be an updated version of the
  input effects object, with one row updated.
  Details of the rows altered will be printed, unless \code{verbose=FALSE}.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley, Tom Snijders}
\seealso{
  \code{\link{make_specification}}, 
  \code{\link{set_interaction}}, \code{\link{includeGMoMStatistics}}, \cr
  \code{\link{update_specification}},
  \code{\link{print.sienaEffects}}, \code{\link{effectsDocumentation}}.
}

\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <- as_dependent_rsiena(s50a, type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, gwespFF)
myeff <- set_effect(myeff, list(inPop, outAct))
myeff <- set_effect(myeff, "gwespFB")
myeff <- set_effect(myeff, simX, covar1="mybeh")
myeff
# Specify an effect parameter:
myeff <- set_effect(myeff, outTrunc, parameter=1)
myeff
# Set the initial rate parameter for one period:
myeff <- set_effect(myeff, Rate, initialValue=1.5, depvar="mybeh",
                   type="rate", period=2)
myeff
}
% Add one or more standard keywords, see file "KEYWORDS" in the
% R documentation directory.
\keyword{classes}
\keyword{methods}

\name{make_group_rsiena}
\alias{sienaGroupCreate}
\alias{sienaGroup}
\alias{make_group_rsiena}
\title{Function to group together several Siena data objects }
\description{Creates an object of class "sienaGroup" from a list of  
  Siena data objects.\cr
  The functions \code{make_group_rsiena} and \code{sienaGroupCreate}
  are identical. The first name is preferred, 
  the other is kept for backward compatibility.
}
\usage{
make_group_rsiena(objlist, singleOK = FALSE)

sienaGroupCreate(objlist, singleOK = FALSE, getDocumentation=FALSE)
}
\arguments{
  \item{objlist}{ List of objects of class \code{siena}. }
  \item{singleOK}{ Boolean: is it OK to only have one object?}
  \item{getDocumentation}{Flag to allow documentation of internal
  functions, not for use by users.}
}
\details{

This function creates a Siena group object from several Siena data
objects ('groups'), all of which use networks, covariates and actor sets
with the same names. The variables must correspond exactly
between all data objects; the numbers of waves may differ.
It can be used as data input to \code{\link{siena07}} for the
multigroup option.
Also used internally for convenience with a single Siena data object.

Each covariate should either be centered in all groups, or non-centered
in all groups.
Centered actor covariates are re-centered at the overall mean.
This means that the original values are used, and the overall
mean of all non-missing observations is subtracted. Note that this implies
that group-dependent variables that are constant for all actors in each
group, can be used as centered actor covariates.

For combining two-wave with more-wave groups in one group object,
covariates that are changing covariates for the more-wave groups have to
be specified as changing covariates also for the two-wave groups.
This can be done by specifying them with values for the two waves;
for actor covariates this will be by using an \eqn{n \times 2}{n * 2} matrix,
for dyadic covariates an \eqn{n \times n \times 2}{n * n * 2} array
(or \eqn{n \times m \times 2}{n * m * 2} for the two-mode case).
The values for the second wave should be identical to those for the
first wave (they will be used only for centering operations).

For later use in \code{\link{siena}},
it will often (but not always...) be helpful when creating the
Siena data objects in \code{objlist}
to use \code{allowOnly=FALSE} in the call of
\code{\link{as_dependent_rsiena}}; see the help page for this function.

If there are multiple dependent networks, it may be necessary to run \cr
\code{\link{make_constraint.sienadata}} before \code{make_group_rsiena}
to ensure that these constraints are equal for all groups.
}

\value{
An object of class \code{sienaGroup}; this is a list
containing the input objects, with attributes:
  \item{netnames}{names of the dependent variables in each set}
  \item{symmetric}{vector of booleans, one for each dependent
    variable. TRUE if all occurrences of the network are symmetric.}
  \item{structural}{vector of booleans, indicating whether structurally
    fixed values occur in this network}
  \item{allUpOnly}{vector of booleans, indicating whether changes are
    all upwards in all the occurrences of this network}
  \item{allDownOnly}{similar to previous, but for downward changes}
  \item{anyUpOnly}{vector of booleans, indicating whether changes are
    all upwards in any of the occurrences of this network}
  \item{anyDownOnly}{similar to previous, but for downward changes}
  \item{types}{vector of network types of the dependent variables}
  \item{observations}{Total number of periods to process}
  \item{periodNos}{Sequence of numbers of periods which are not skipped
    in multigroup processing}
  \item{netnodeSets}{list of names of the node sets corresponding to the
    dependent variables}
  \item{cCovars}{names of the constant covariates, if any}
  \item{vCovars}{names of the changing  covariates, if any}
  \item{dycCovars}{names of the constant dyadic covariates, if any}
  \item{dyvCovars}{names of the changing dyadic covariates, if any}
  \item{ccnodeSets}{list of the names of the node sets corresponding to
    the constant covariates}
  \item{cvnodeSets}{list of the names of the node sets corresponding to
    the changing covariates}
  \item{dycnodeSets}{list of the names of the node sets corresponding to
    the constant dyadic covariates}
  \item{dyvcnodeSets}{list of the names of the node sets corresponding to
    the changing dyadic covariates}
  \item{compositionChange}{boolean: any composition change at all?}
  \item{exooptions}{named vector of composition change options for the
    node sets}
  \item{names}{Either from the input objects or "Data1", "Data2" etc}
  \item{class}{"sienaGroup" inheriting from "siena"}
  \item{balmean}{vector of means for balance calculations}
  \item{bRange}{vector of difference between maximum and minimum values
    for behavior variables, NA for other dependent variables}
  \item{behRange}{matrix of  maximum and minimum values
    for behavior variables, NA for other dependent variables}
  \item{bSim}{vector of similarity means for behavior variables, NA for
    other dependent variables}
  \item{bPoszvar}{vector of booleans indicating positive variance for
    behavior variables. NA for other dependent variables}
  \item{bMoreThan2}{vector of booleans indicating whether the behavior
    variables take more than 2 distinct values}
  \item{cCovarPoszvar}{vector of booleans indicating positive variance for
    constant covariates}
  \item{cCovarMoreThan2}{vector of booleans indicating whether the
    constant covariates take more than 2 distinct values}
  \item{cCovarRange}{vector of difference between maximum and minimum
    values for constant covariates}
  \item{cCovarRange2}{matrix of maximum and minimum values for constant
    covariates}
  \item{cCovarSim}{vector of similarity means for constant covariates}
  \item{cCovarMean}{vector of means for constant covariates}
  \item{vCovarRange}{vector of difference between maximum and minimum
    values for changing covariates}
  \item{vCovarSim}{vector of similarity means for changing covariates}
  \item{vCovarMoreThan2}{vector of booleans indicating whether the
    changing covariates take more than 2 distinct values}
  \item{vCovarPoszvar}{vector of booleans indicating positive variance for
    changing covariates}
  \item{vCovarMean}{vector of means for changing covariates}
  \item{dycCovarMean}{vector of means for constant dyadic covariates}
  \item{dycCovarRange}{vector of ranges for constant dyadic covariates}
  \item{dycCovarRange2}{matrix of maximum and minimum values
    for constant dyadic covariates}
  \item{dyvCovarRange}{vector of ranges for changing dyadic covariates}
  \item{dyvCovarMean}{vector of means for changing dyadic covariates}
  \item{anyMissing}{vector of booleans, one for each dependent variable,
    indicating the presence of any missing values}
  \item{netRanges}{matrix of maximum and minimum values for dependent
    networks, NA for behavior variables}
}
\references{See the Section on Multi-group Siena analysis in the manual
   available from \url{https://www.stats.ox.ac.uk/~snijders/siena/ }.}
\author{ Ruth Ripley, Modification by Tom Snijders }
\seealso{\code{\link{make_data_rsiena}} }
\examples{
Group1 <- as_dependent_rsiena(array(c(N3401, HN3401), dim=c(45, 45, 2)))
Group3 <- as_dependent_rsiena(array(c(N3403, HN3403), dim=c(37, 37, 2)))
Group4 <- as_dependent_rsiena(array(c(N3404, HN3404), dim=c(33, 33, 2)))
Group6 <- as_dependent_rsiena(array(c(N3406, HN3406), dim=c(36, 36, 2)))
# Illustration of the use of group-level variables:
# dum1 is a dummy variable for group 1,
# having constant value 1 in group 1, and constant value 0 in the other groups.
dum1.1  <- as_covariate_rsiena(c(rep(1,45)), warn = FALSE)
dum1.3  <- as_covariate_rsiena(c(rep(0,37)), warn = FALSE)
dum1.4  <- as_covariate_rsiena(c(rep(0,33)), warn = FALSE)
dum1.6  <- as_covariate_rsiena(c(rep(0,36)), warn = FALSE)
# In a similar way, dummies for the other groups can be defined.
dataset.1 <- make_data_rsiena(Friends = Group1, dum1 = dum1.1)
dataset.3 <- make_data_rsiena(Friends = Group3, dum1 = dum1.3)
dataset.4 <- make_data_rsiena(Friends = Group4, dum1 = dum1.4)
dataset.6 <- make_data_rsiena(Friends = Group6, dum1 = dum1.6)
(FourGroups <- make_group_rsiena(list(dataset.1, dataset.3, dataset.4,
                                     dataset.6)))
class(FourGroups)
# The main effect of the group-level variable is the \code{egoX} effect:
myeff <- make_specification(FourGroups)
(myeff <- set_effect(myeff, egoX, covar1 = "dum1"))
}
\keyword{classes}

\name{coDyadCovar}
\alias{coDyadCovar}
\title{Function to create a constant dyadic covariate object}
\description{
  This function creates a constant dyadic covariate object from a matrix.\cr
   This function now is superseded by function \code{\link{as_covariate_rsiena}}.
}
\usage{
coDyadCovar(val, centered=TRUE, nodeSets=c("Actors", "Actors"),
     warn=TRUE, sparse=inherits(val,"TsparseMatrix"), 
     type=c("oneMode", "bipartite"))
}
\arguments{
  \item{val}{Matrix of covariate values. May be sparse, of type
     \code{"TsparseMatrix"}. }
  \item{centered}{Boolean: if TRUE, then the mean value is subtracted.}
  \item{nodeSets}{The name of the node sets with which this
    covariate is associated. If the entire data set contains more than
    one node set, then the node sets must be specified in all data objects.}
  \item{warn}{Logical: is a warning given if all values are \code{NA},
  or all non-missing values are the same.}
  \item{sparse}{Boolean: whether a sparse matrix or not.}
  \item{type}{oneMode or bipartite: whether the matrix refers to a one-mode
  or a bipartite (two-mode) network.}
}
\details{
  When part of a Siena data object, the covariate is assumed to be
  associated with the node sets named in nodeSets
  of the Siena data object. The name of the associated node sets will only
  be checked when the Siena data object is created.
}
\value{
   Returns the covariate as an object of class "coDyadCovar", in which
  form it can be used as an argument to \code{\link{make_data_rsiena}}.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley}
\seealso{\code{\link{as_covariate_rsiena}},
                    \code{\link{as_nodeset_rsiena}}}
\examples{
mydyadvar <- as_covariate_rsiena(s503, type="oneMode")
class(mydyadvar)
}
\keyword{classes}

\name{includeGMoMStatistics}
\alias{includeGMoMStatistics}
\title{
  Function to include GMoM statistics in a Siena model
}
\description{
  This function can be used for including one or more GMoM statistics   
  by modifying a Siena effects object.
}
\usage{
  includeGMoMStatistics(myeff, ..., include=TRUE, 
                        depvar=myeff$name[1], covar1="", covar2="",
                        character=FALSE, verbose=TRUE)
}
\arguments{
  \item{myeff}{a Siena effects object as created by 
      \code{\link{make_specification}}.
  }
  \item{\dots}{
    short names to identify the GMoM statistics which should be included or
    excluded.}
  \item{include}{Boolean; default TRUE, but can be switched to FALSE to
    turn off an effect.}
  \item{depvar}{Name of dependent variable (network or behavior) for which
    statistics are being included. Defaults to the first in the effects object.}
  \item{covar1}{
    Name of siena object where needed to completely identify the
    effects e.g. covariate name or behavior variable name.}
  \item{covar2}{
    Name of siena object where needed to completely identify the
    effects e.g. covariate name or behavior variable name.}
  \item{character}{Boolean: are the statistic names character strings or not.}
  \item{verbose}{Boolean: should the print of altered statistic be produced.}
}
\details{The names \code{covar1} and \code{covar2} refer to
   the dependence of the GMoM statistics on other variables in the data set.
  The arguments should identify the GMoM statistic completely.
  The type does not have to be specified, as it is \code{gmm} for all
  GMoM statistiscs in the effects object. \cr
  The short names must not be set between quotes,
  unless you use \code{character=TRUE}.

  The function \code{includeGMoMStatistics} operates as an interface
  setting the "include" column on selected rows of the effects object,
  to the value requested (\code{TRUE} or \code{FALSE}).
}
\value{
  An updated version of the input effects object, with the \code{include}
  column for one or more rows updated.
  Details of the rows altered will be printed.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Viviana Amati.
}
\seealso{
  \code{\link{make_specification}},
  \code{\link{set_effect}},
  \code{\link{set_interaction}},
  \code{\link{print.sienaEffects}}
}

\examples{
  mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
  mybeh  <- as_dependent_rsiena(s50a, type="behavior")
  mydata <- make_data_rsiena(mynet1, mybeh)
  myeff <- make_specification(mydata)
  myeff <- includeGMoMStatistics(myeff, egoX_gmm, covar1="mybeh")
  myeff
}
\keyword{classes}

\name{test_parameter}
\alias{test_parameter}
\alias{test_parameter.sienaFit}
\alias{Wald}
\alias{Wald.RSiena}
\alias{Multipar.RSiena}
\alias{score.Test}
\alias{scoreTest}
\alias{sienaTest}
\alias{siena.Test}
\alias{scoretest}
\alias{sienatest}
\alias{testSame.RSiena}
\title{  Wald and score tests for RSiena results }
\description{
  These functions test parameters in RSiena results
  estimated by \code{\link{siena}}.
  Tests can be Wald-type (if the parameters were estimated)
  or score-type tests (if the parameters were fixed and tested).\cr
  The function \code{test_parameter} chooses among the
  functions \code{Wald.RSiena},  \code{Multipar.RSiena},
  \code{testSame.RSiena},   and \code{score.Test},
  based on the input given. The export of the latter three functions is superfluous,
  but they are kept for backward compatibility.
}
\usage{

\method{test_parameter}{sienaFit}(x, method=NULL, tested=NULL, tested2=NULL, ...)

Wald.RSiena(A, x)

Multipar.RSiena(x, tested)

testSame.RSiena(x, e1, e2)

score.Test(x, tested=x$test)

}
\arguments{
  \item{x}{An object of class \code{\link{sienaFit}},
    resulting from a call to \code{\link{siena}.}}
  \item{method}{\code{NULL}, \code{"all"}, \code{"same"}, or \code{"score"}.}
  \item{tested}{One number or a vector of numbers between 1 and \code{p},
      where \code{p = x$pp}, the number of parameters in \code{x}
      excluding the basic rate parameters used for
      conditional estimation; these indicate the tested parameters.
      Can also be a logical vector of length \code{p}; \cr
      or, for \code{test_parameter}, a \code{k} * \code{p} matrix. }
  \item{tested2}{A single number or vector as mentioned for \code{tested};
     these two parameters are tested for equality for \code{method = "same"}.}
  \item{A}{A \code{k} * \code{p} matrix, where \code{p = ans$pp},
  the number of parameters in \code{ans} excluding the basic rate parameters
  used for conditional estimation.}
  \item{e1,e2}{Each an integer number between 1 and \code{p}, or a vector
    of such numbers; the hypothesis tested is that the parameters for effects
    with number/s \code{e1} are equal to those in \code{e2}.}
 \item{...}{Additional arguments (currently not used.)}
}

\details{ The tests for \code{test_parameter}, \code{Wald.RSiena},
    \code{Multipar.RSiena}, and \cr
    \code{testSame.RSiena} are Wald-type tests
    (but see below for \code{test_parameter} with \code{method=score}.\cr
    For \code{test_parameter} and \code{Wald.RSiena},
    if \code{tested} is a matrix \code{A}, the hypothesis tested
   is \eqn{A\theta = 0}{A theta = 0}, where \eqn{\theta}{theta} is
   the parameter estimated in the process leading to \code{x}.\cr
    For \code{test_parameter} and \code{Multipar.RSiena}
    if \code{tested} is a number or vector, a multivariate
    (i.e., simultaneous) test  is given of the hypothesis that
    the  parameters mentioned in \code{tested} are 0.   \cr
    For \code{test_parameter} if \code{method="all"},
    a test is given of all parameters in the model.\cr
    For \code{test_parameter}
    if \code{method="same"}, the items \code{"tested"} and  \code{"tested2"}
    should be single numbers or vectors of equal length, and the test
    of equality of the corresponding two parameters is given.\cr
    For \code{test_parameter} if \code{method="score"}, and for \code{score.Test},
    the parameters with numbers in \code{"tested"}
    should all be used in \code{x} as fixed-and-tested parameters,
    and the multivariate score test of these parameters is given
    of the hypothesis that the tested parameters have
    the value indicated in the effects object used for obtaining \code{x}.  \cr
    The numbering of parameters is as in \code{print(x)};
    if conditional estimation was used, numbered
    as the 'Other parameters').

   These tests should be carried out only when convergence is adequate
   (overall maximum convergence ratio less than 0.25
   and all \emph{t}-ratios for convergence less than 0.1 in absolute value).

   These functions have their own print method, see \code{\link{print.sienaTest}}.
}
\value{An object of class \code{sienaTest}, which is a list with elements:
  \item{chisquare: }{ The test statistic, assumed to have a chi-squared null
    distribution.}
  \item{df: }{ The degrees of freedom.}
  \item{pvalue: }{ The associated \emph{p}-value.}
  \item{onesided: }{For \code{df}=1, the onesided test statistic.}
  \item{efnames: }{For \code{Multipar.RSiena} and \code{score.Test}, the names
              of the tested effects.}
}
\references{See the manual and \url{https://www.stats.ox.ac.uk/~snijders/siena/}

M. Schweinberger (2012). Statistical modeling of network panel data:
Goodness-of-fit. \emph{British Journal of Statistical
and Mathematical Psychology} \bold{65}, 263--281.}

\author{Tom Snijders}

\seealso{ \code{\link{siena}}, \code{\link{print.sienaTest}} }

\examples{
myalgo <- set_algorithm_saom(nsub=1, n3=40, seed=1777)
# nsub=1 and n3=40 is used here for having a brief computation,
# not for practice.
mynet <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mydata <- make_data_rsiena(mynet)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, list(transTrip, transTies))
myeff <- set_effect(myeff, list(outAct, outPop), fix=TRUE, test=TRUE)
(ans <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE))
A <- matrix(0, 2, 6)
A[1, 3] <- 1
A[2, 4] <- 1
(wa <- test_parameter(ans, tested=A))
wa
# An alternative specification of the above is:
test_parameter(ans, tested=c(3, 4))
# The following two are also equivalent:
sct <- test_parameter(ans, method="score",
                tested=c(FALSE, FALSE, FALSE, FALSE, FALSE, TRUE))
sct <- test_parameter(ans, method="score", tested=6)
print(sct)
# Getting all 1-df score tests separately:
for (i in which(ans$test)){
   sct <- test_parameter(ans, method="score", tested=i)
   print(sct)}
# Testing that endowment and creation effects are identical:
myeff1 <- make_specification(mydata)
myeff1 <- set_effect(myeff1, list(transTrip, transTies))

myeff1 <- make_specification(mydata)
myeff1 <- set_effect(myeff1, recip, include=FALSE)
myeff1 <- set_effect(myeff1, recip, type='creation')
(myeff1 <- set_effect(myeff1, recip, type='endow'))
ans1 <- siena(mydata, effects=myeff1, control_algo=myalgo, batch=TRUE)
test_parameter(ans1, method="same", tested=2, tested2=3)
}
\keyword{tests}

\name{meta_siena}
\alias{siena08}
\alias{sienaMeta}
\alias{meta_siena}
\title{
Function to perform a meta analysis of a collection of Siena fits.
}
\description{
  Estimates a meta analysis based on a collection of Siena fits.\cr
  \code{meta_siena()} and \code{siena08()}
  are identical functions, differing only in the keyword
  for the output file. The first name is preferred,
  the other is kept for backward compatibility.
}
\usage{

meta_siena(..., outputName = "sienaMeta", bound = 10, alpha = 0.05, maxit=20)

siena08(..., projname = "sienaMeta", bound = 10, alpha = 0.05, maxit=20)
}
\arguments{
  \item{\dots}{names of \code{\link{sienaFit}} objects, returned from
    \code{\link{siena}}.
    They will be renamed if entered in format \code{newname=oldname}.
    It is also allowed to give for \dots a list of \code{\link{sienaFit}}
    objects.
}
\item{outputName}{
  Base name of report file if required.
}
\item{projname}{
  Base name of report file if required.
}
\item{bound}{
  Upper limit of standard error for inclusion in the meta analysis.
}
\item{alpha}{
  1 minus confidence level of confidence intervals.
}
\item{maxit}{
  Number of iterations of iterated least squares procedure.
}
}
\details{
  A meta analysis is performed as described
  in the Siena manual, section "Meta-analysis of Siena results". This
  consists of three parts: an iterated weighted least squares (IWLS)
  modification of the method described in the reference below;
  maximum likelihood estimates and confidence intervals based on
  profile likelihoods under normality assumptions;
  and Fisher combinations of left-sided and right-sided \eqn{p}-values.
  These are produced for all effects separately.

  Note that the corresponding effects must have the same effect name in each
  model fit. This implies that at least covariates and behavior variables must
  have the same name in each model fit.
}
\value{An object of class \code{\link{sienaMeta}}.  There are \code{print},
  \code{summary} and \code{plot} methods for this class.
  This object contains at least the following.
  \item{thetadf}{Data frame containing the coefficients, standard errors
    and score test results}
  \item{projname}{Root name for any output file to be produced by the print method}
  \item{bound}{Estimates with standard error above this value were
    excluded from the calculations}
  \item{scores}{Object of class \code{by} indicating, for each effect in
    the models, whether score test  information was present.}
  \item{requestedEffects}{The \code{requestedEffects} component of the first
    \code{sienaFit} object in ....}
  \item{muhat}{The vector of IWLS estimates.}
  \item{se.muhat}{The vector of standard errors of the IWLS estimates.}
  \item{theta}{The vector of ML estimates \code{mu.ml} (see below).}
  \item{se}{The vector of standard errors of the ML estimates \code{mu.ml.se}
    (see below).}

  Then for each effect, there is a list with at least the following.

  \item{cor.est}{Spearman rank correlation coefficient between estimates
                 and their standard errors.}
  \item{cor.pval}{p-value for above}
  \item{regfit}{Part of the result of the fit of \code{\link{iwlsm}}.}
  \item{regsummary}{The summary of the fit, which includes the
                    coefficient table.}
  \item{Tsq}{test statistic for effect zero in every model}
  \item{pTsq}{p-value for above}
  \item{tratio}{test statistics that mean effect is 0}
  \item{ptratio}{p-value for above}
  \item{Qstat}{Test statistic for variance of effects is zero}
  \item{pttilde}{p-value for above}
  \item{cjplus}{Test statistic for at least one theta strictly greater than 0}
  \item{cjminus}{Test statistic for at least one theta strictly less than 0}
  \item{cjplusp}{p-value for \code{cjplus}}
  \item{cjminusp}{p-value for \code{cjminus}}
  \item{mu.ml}{ML estimate of population mean}
  \item{mu.ml.se}{standard error of ML estimate of population mean}
  \item{sigma.ml}{ML estimate of population standard deviation}
  \item{mu.confint}{confidence interval for population mean
                    based on profile likelihood}
  \item{sigma.confint}{confidence interval for population standard deviation
                    based on profile likelihood}
  \item{n1}{Number of fits on which the meta analysis is based}
  \item{cjplus}{Test statistic for combination of right one-sided
  Fisher combination tests}
  \item{cjminus}{Test statistic for combination of left one-sided
  Fisher combination tests}
  \item{cjplusp}{p-value for \code{cjplus}}
  \item{cjminusp}{p-value for \code{cjminus}}
  \item{scoreplus}{Test statistic for combination of right one-sided
  \eqn{p}-values from score tests}
  \item{scoreminus}{Test statistic for combination of left one-sided
  \eqn{p}-values from score tests}
  \item{scoreplusp}{p-value for \code{scoreplus}}
  \item{scoreminusp}{p-value for \code{scoreminus}}
  \item{ns}{Number of fits on which the score test analysis is based}
}
\references{
  Snijders, T.A.B, and Baerveldt, C. (2003),
  A Multilevel Network Study of the Effects of Delinquent Behavior
  on Friendship Evolution.
  \emph{Journal of Mathematical Sociology} \bold{27}, 123--151.

  See also the manual (Section 12.2)
  and \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Ruth Ripley, Tom Snijders}

\seealso{\code{\link{print.sienaMeta}}, \code{\link{funnelPlot}},
  \code{\link{meta.table}}, \code{\link{iwlsm}},
  \code{\link{siena}}
}
\examples{
\dontrun{
# A meta-analysis for three groups does not make much sense
# for generalizing to a population of networks,
# but the Fisher combinations of p-values are meaningful.
# However, using three groups does show the idea.

Group1 <- as_dependent_rsiena(array(c(N3401, HN3401), dim=c(45, 45, 2)))
Group3 <- as_dependent_rsiena(array(c(N3403, HN3403), dim=c(37, 37, 2)))
Group4 <- as_dependent_rsiena(array(c(N3404, HN3404), dim=c(33, 33, 2)))
dataset.1 <- make_data_rsiena(Friends = Group1)
dataset.3 <- make_data_rsiena(Friends = Group3)
dataset.4 <- make_data_rsiena(Friends = Group4)
OneAlgorithm <- set_algorithm_saom(seed=128, nsub=2, n3=100)
effects.1 <- make_specification(dataset.1)
effects.3 <- make_specification(dataset.3)
effects.4 <- make_specification(dataset.4)
effects.1 <- set_effect(effects.1, transTrip)
effects.1 <- set_effect(effects.1, transRecTrip, fix=TRUE, test=TRUE)
effects.3 <- set_effect(effects.3, transTrip)
effects.3 <- set_effect(effects.3, transRecTrip, fix=TRUE, test=TRUE)
effects.4 <- set_effect(effects.4, transTrip)
effects.4 <- set_effect(effects.4, transRecTrip, fix=TRUE, test=TRUE)
ans.1 <- siena(dataset.1, effects=effects.1, batch=TRUE)
ans.3 <- siena(dataset.3, effects=effects.3, batch=TRUE)
ans.4 <- siena(dataset.4, effects=effects.4, batch=TRUE)
ans.1
ans.3
ans.4
meta <- meta_siena(ans.1, ans.3, ans.4)
print(meta)
plot(meta, which=2:3, layout = c(2,1))
# For specifically presenting the Fisher combinations:
# First determine the components of meta with estimated effects:
which.est <- sapply(meta,
                function(x){ifelse(is.list(x),!is.null(x$cjplus),FALSE)})
Fishers <- t(sapply(1:sum(which.est),
 function(i){c(meta[[i]]$cjplus, meta[[i]]$cjminus,
               meta[[i]]$cjplusp, meta[[i]]$cjminusp, 2*meta[[i]]$n1 )}))
Fishers <- as.data.frame(Fishers, row.names=names(meta)[which.est])
names(Fishers) <- c('Fplus', 'Fminus', 'pplus', 'pminus', 'df')
Fishers
round(Fishers,4)
}
}
\keyword{models}

\name{s50a}
\alias{s50a}
\docType{data}
\title{Alcohol use data: excerpt from "Teenage Friends and Lifestyle Study" data}
\description{
Alcohol use data from an excerpt of 50 girls from    
an excerpt of the network, alcohol consumption, and smoking data     
for 50 randomly chosen girls from the
Teenage Friends and Lifestyle Study data set. Useful as a small    
example of network and behaviour, for which models can be    
fitted quickly, and for which there are no missing values.
}
\format{
 A matrix of variables relating to the use of alcohol for the actors in
 the network. Three columns, one for each time point. Coding is 1--5, high
 values indicating higher consumption.
}
\source{
\url{https://www.stats.ox.ac.uk/~snijders/siena/s50_data.zip}
}
\references{
West, P. and Sweeting, H. (1995), \emph{Background Rationale and Design of the
West of Scotland 11-16 Study}.
Working Paper No. 52. MRC Medical Sociology Unit Glasgow.

See \url{https://www.stats.ox.ac.uk/~snijders/siena/s50_data.htm}
}
\seealso{\code{\link{s501}}, \code{\link{s502}}, \code{\link{s503}}, \code{\link{s50s}}
}
\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh  <- as_dependent_rsiena(s50a, type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
mydata
}
\keyword{datasets}

\name{test_time}
\alias{test_time}
\alias{test_time.sienaFit}
\alias{sienaTimeTest}
\alias{print.sienaTimeTest}
\title{Function to assess and account for time heterogeneity of parameters}
\description{
 Takes a \code{sienaFit} object estimated by Method of Moments,
 and tests for time heterogeneity by the addition of interactions
 with time dummy variables at waves \code{m=2...(M-1)}.
 The test used is the score-type test of Schweinberger (2012).

 Tests for joint significance, parameter-wise significance,
 period-wise significance, individual significance,
 and one-step estimates of the unrestricted model parameters
 are returned in a list.\cr
  \code{test_time()} and \code{sienaTimeTest()}
  are identical functions. The first name is preferred,
  the other is kept for backward compatibility.
 }
\usage{
\method{test_time}{sienaFit}(x, effects=NULL,
              excludedEffects=NULL, condition=FALSE, ...)

sienaTimeTest(x, effects=NULL,
              excludedEffects=NULL, condition=FALSE)
}
\arguments{
  \item{x}{ A \code{sienaFit} object
      returned by \code{\link{siena}}. }
  \item{effects}{Optional vector of effect numbers to test. Use the
    numbering on the print of the sienaFit object.}
  \item{excludedEffects}{Optional vector of effect numbers
    for which time heterogeneity is not to be tested. Use the
    numbering on the print of the sienaFit object.}
  \item{condition}{ Whether to orthogonalize effect-wise score-type tests
    and individual significance tests against estimated effects
    and un-estimated dummy terms, or just against estimated effects.}
  \item{\dots}{For other arguments. Not used currently.  }
}
\details{
  This test follows the score type test of Schweinberger (2012) as
 elaborated by Lospinoso et al. (2011) by using statistics already
 calculated at each wave to obtain vectors of partitioned moment
 functions corresponding to a restricted model (the model in the
 \code{sienaFit} object; used as null hypothesis) and an
 unrestricted model (which contains dummies for waves
 \code{m=2...(M-1)}; used as alternative hypothesis).

 \code{condition=TRUE} leads to a rough-and-easy approximation to
 controlling the mentioned tests also for the unestimated effects.

 After assessing time heterogeneity, effects objects can be modified by
 adding numbers of all or some periods to the \code{timeDummy} column.
 This is facilitated by the \code{\link{includeTimeDummy}} function.
 For an effects object in which the \code{timeDummy} column
 of some of the included effects includes some or all period numbers,
 interactions of those effects  with time dummies for the indicated
 periods will also be estimated.

 An alternative to the use of \code{\link{includeTimeDummy}}
 is to define time-dependent actor covariates (dummy variables or other
 functions of wave number that are the same for all actors),
 include these in the data set through \code{\link{make_data_rsiena}},
 and include interactions of other effects with ego effects of these
 time-dependent actor covariates by \code{\link{set_interaction}}.
 This is illustrated in an example below.
 Using \code{\link{includeTimeDummy}} is easier;
 using self-defined interactions with time-dependent variables
 gives more control.

 If you wish to use this function with \code{sienaFit} objects that use the
 finite differences method of derivative estimation, or which use maximum
 likelihood estimation, you must request the derivatives to be returned
 by wave using \code{byWave=TRUE} option in the call of \code{\link{siena07}}.

 Effects leading to dummy interactions that are collinear with the
 model originally fitted, after excluding the effects mentioned,
 will be automatically excluded from the time heterogeneity testing.

 If \code{test_time} gives errors that there are too
 many collinear effects, run it with a smaller set of effects as specified
 by the \code{effects} parameter.
 For example, if the model has 40 effects of which the
 first 8 are rate parameters and therefore uninteresting,
 and there is such an error message, try \code{effects=9:30};
 if that still does not work, decrease the upper limit of 30, if it does work
 increase it, to find the largest possible set of effects for which
 heterogeneity assessment still is possible; then as a next step
 try the remaining effects in a similar way.

 Also if the execution is time-consuming, e.g., for a multi-group
 \code{sienaFit} object with many groups and many effects,
 it can be helpful to carry out the function in smaller subsets of effects.
}
\value{
  \code{test_time} returns a list containing many items,
    including the following:
  \item{JointTest }{A chi-squared test for joint significance of the dummies.}
  \item{EffectTest }{A chi-squared test for joint significance
      across dummies for each separate effect.}
  \item{GroupTest }{A chi-squared test for joint significance
      across dummies; if \code{sienaFit} is a fit for a multi-group object
      then these refer to each group; else they refer to each period.}
  \item{IndividualTest }{A matrix displaying initial estimates, one-step
    estimates, and \eqn{p}-values for the individual interactions.}
}

\references{
J.A. Lospinoso, M. Schweinberger, T.A.B. Snijders, and R.M. Ripley
(2011), Assessing and Accounting for Time Heterogeneity in Stochastic
Actor Oriented Models. \emph{Advances in Data Analysis and Computation},
\bold{5}, 147--176.

M. Schweinberger (2012), Statistical modeling of network panel data:
Goodness-of-fit. \emph{British Journal of Statistical
and Mathematical Psychology} \bold{65}, 263--281.
}

\author{Josh Lospinoso, Tom Snijders}
\seealso{\code{\link{siena}}, \code{\link{plot.sienaTimeTest}},
  \code{\link{includeTimeDummy}}}
\examples{
## Estimate a restricted model
myalgorithm <- set_algorithm_saom(nsub=1, n3=50)
# Short estimation not for practice, just for having a quick demonstration
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mydata <- make_data_rsiena(mynet1)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, transTrip)
ans <- siena(mydata, effects=myeff, batch=TRUE, control_algo=myalgorithm)

## Conduct the score-type test to assess whether heterogeneity is present.
tt <- test_time(ans)
summary(tt)

## Suppose that we wish to include time dummies.
## Add them in the following way:
myeff <- includeTimeDummy(myeff, recip, transTrip, timeDummy="2")
ans2 <- siena(mydata, effects=myeff, batch=TRUE, control_algo=myalgorithm)

## Re-assess the time heterogeneity
(tt2 <- test_time(ans2))

## And so on..

## A demonstration of the plotting facilities, on a larger dataset:
## (Of course pasting these identical sets of three waves after each other
##  in a sequence of six is not really meaningful. It's just a demonstration.)

myalgo <- set_algorithm_saom(nsub=1, n3=50, seed=654)
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503, s501, s503, s502),
                               dim=c(50, 50, 6)))
mydata <- make_data_rsiena(mynet1)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, list(transTrip, outAct))
myeff <- includeTimeDummy(myeff, density, timeDummy="all")
myeff <- includeTimeDummy(myeff, recip, timeDummy="2,3,5")
myeff <- includeTimeDummy(myeff, outAct, timeDummy="4")
\dontrun{
(ansp <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE))
ttp <- test_time(ansp, effects=2:3)
# Using effects=1:4 leads to an informative error message.

## Pairwise plots show
plot(ttp, pairwise=TRUE)
}

## Instead of working with includeTimeDummy,
## you can also define time dummies explicitly;
## this may give more control and more clarity:
dum2 <- matrix(c(0,1,0,0,0), nrow=50, ncol=5, byrow=TRUE)
dum3 <- matrix(c(0,0,1,0,0), nrow=50, ncol=5, byrow=TRUE)
dum4 <- matrix(c(0,0,0,1,0), nrow=50, ncol=5, byrow=TRUE)
dum5 <- matrix(c(0,0,0,0,1), nrow=50, ncol=5, byrow=TRUE)
time2 <- as_covariate_rsiena(dum2, type="monadic")
time3 <- as_covariate_rsiena(dum3, type="monadic")
time4 <- as_covariate_rsiena(dum4, type="monadic")
time5 <- as_covariate_rsiena(dum5, type="monadic")
mydata <- make_data_rsiena(mynet1, time2, time3, time4, time5)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, list(transTrip, outAct))
## corresponding to includeTimeDummy(myeff, density, timeDummy="all"):
myeff <- set_effect(myeff, egoX, covar1='time2')
myeff <- set_effect(myeff, egoX, covar1='time3')
myeff <- set_effect(myeff, egoX, covar1='time4')
myeff <- set_effect(myeff, egoX, covar1='time5')
## corresponding to myeff <- includeTimeDummy(myeff, recip, timeDummy="2,3,5"):
myeff <- set_interaction(myeff, list(egoX, recip), covar1=c('time2', ''))
myeff <- set_interaction(myeff, list(egoX, recip), covar1=c('time3', ''))
myeff <- set_interaction(myeff, list(egoX, recip), covar1=c('time5', ''))
## corresponding to myeff <- includeTimeDummy(myeff, outAct, timeDummy="4"):
myeff <- set_interaction(myeff, list(egoX, outAct), covar1=c('time4', ''))
\dontrun{
(anspp <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE))
## anspp contains identical results as ansp above.
}

## A demonstration of RateX heterogeneity.
\dontrun{
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
myccov <- as_covariate_rsiena(s50a[,1], type="monadic")
mydata <- make_data_rsiena(mynet1, myccov)
myeff <- make_specification(mydata)
myeff <-  set_effect(myeff, list(transTrip, outAct))
myeff <- includeTimeDummy(myeff, RateX, type="rate", covar1="myccov")
(ans <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE))
}
}
\keyword{tests}
\keyword{methods}

\name{as_dependent_rsiena}
\alias{sienaDependent}
\alias{sienaNet}
\alias{as_dependent_rsiena}
\title{Function to create a dependent variable for a Siena model}
\description{
  Creates a Siena dependent variable: either a network,  
  created from a matrix or array or list of sparse matrix of triples;   
  or a behavior variable, created from a matrix.\cr   
  \code{as_dependent_rsiena()}, \code{sienaDependent()} and \code{sienaNet()} 
  are identical functions. The first name is preferred, 
  the two others are kept for backward compatibility.
}
\usage{
as_dependent_rsiena(netarray, 
      type=c("oneMode", "bipartite", "behavior", "continuous"),
      nodeSet="Actors", sparse=is.list(netarray), 
      allowOnly=TRUE, imputationValues=NULL)

sienaDependent(netarray, 
      type=c("oneMode", "bipartite", "behavior", "continuous"),
      nodeSet="Actors", sparse=is.list(netarray), 
      allowOnly=TRUE, imputationValues=NULL)

sienaNet(netarray, 
      type=c("oneMode", "bipartite", "behavior", "continuous"),
      nodeSet="Actors", sparse=is.list(netarray), 
      allowOnly=TRUE, imputationValues=NULL)
}
\arguments{
  \item{netarray}{\code{type="behavior"} or \code{"continuous"}: matrix
    (actors \eqn{\times}{*} waves).\cr
    \code{type="oneMode"} or \code{"bipartite"}: array of values or list of
    sparse matrices of type \code{"TsparseMatrix"},
    see the \code{\link[Matrix]{Matrix}} package;
    if an array is used, it should have dimensions:
    for a one-mode network, \eqn{n \times n \times M}{n * n * M},
    and for a two-mode network \eqn{n \times m \times M}{n * m * M},
    where \eqn{n} is the number of actors, \eqn{m} is the number
    of nodes in the second mode, and \eqn{M} is the number of waves.}
  \item{type}{type of dependent variable, default \code{oneMode}.}
  \item{nodeSet}{character string naming the appropriate node set.
    For a bipartite network, a vector containing 2 character strings:
    "rows" first, then "columns".}
  \item{sparse}{logical: TRUE indicates the data is in sparse matrix
    format, FALSE otherwise.}
  \item{allowOnly}{logical: If TRUE, it will be detected when between any
    two consecutive waves the changes are non-decreasing or non-increasing,
    and if this is the case, this will also be a constraint for the
    simulations between these two waves.
    This is done by means of the internal parameters \code{uponly} and
    \code{downonly}.
    If FALSE, the parameters \code{uponly} and \code{downonly} always are
    set to FALSE, and changes in dependent variables will not be constrained
    to be non-decreasing or non-increasing.
    This also will imply that some effects are excluded because they are
    superfluous in such constrained situations.
    This will be reported in the output of \code{\link{print01Report}}.

    For normal operation when this is the case for all periods, usually
    TRUE is the appropriate option. When it is only the case for some of the
    periods, and for data sets that will be part of a multi-group object
    created by \code{\link{make_group_rsiena}}, FALSE usually is preferable.}
  \item{imputationValues}{for \code{behavior} or \code{continuous} dependent variables,
    a matrix with imputation values can be included that will be used instead
    of the default imputation values.}
}
\details{
  Adds attributes so that the array or list of matrices can be used in a
  Siena model fit.
}
\value{
  An object of class \code{sienaDependent}. An array or (networks only) a list of
  sparse matrices with attributes:
  \item{netdims}{Dimensions of the network or behavior
    variable: senders, receivers (1 for behavior), periods}
  \item{type}{oneMode, bipartite or behavior}
  \item{sparse}{Boolean: whether the network is given as a list of sparse
    matrices or not}
  \item{nodeSet}{Character string with name(s) of node set(s)}
  \item{allowOnly}{The value of the \code{allowOnly} parameter}
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/} .}
\author{Ruth Ripley and Tom A.B. Snijders}
\seealso{\code{\link{make_data_rsiena}}, \code{\link{as_nodeset_rsiena}},
      \code{\link{make_constraint.sienadata}}}
\examples{
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <- as_dependent_rsiena(s50a, type="behavior")
## note that the following example works 
## although the node sets do not yet exist!
mynet3 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)),
       type="bipartite", nodeSet=c("senders", "receivers"))
## sparse matrix input
## To show this, we first go back from the adjacency matrices to edgelists.
## The manual shows one way to do this.
## Another way is to use the sparse matrix representation which internally
## indeed is an edge list:
library(Matrix)
sp501 <- as(Matrix(s501), "TsparseMatrix")
sp502 <- as(Matrix(s502), "TsparseMatrix")
sp503 <- as(Matrix(s503), "TsparseMatrix")
## If you are interested in the internal structure 
## of these sparse matrices, you can request
str(sp501)
## Slot @i is the row, @j is the column, and @x the value;
## here the values all are 1.
## Slots @i and @j do not contain information about the number of nodes,
## so that is supplied additionally by @Dim.
mymatlist <- list(sp501, sp502, sp503)
mynet.sp <- as_dependent_rsiena(mymatlist)
# For a bipartite (two-mode) network:
senders <- as_nodeset_rsiena(50, nodeSetName="senders")
receivers <- as_nodeset_rsiena(30, nodeSetName="receivers")
mynet <- as_dependent_rsiena(
          array(c(s501[,1:30], s502[,1:30]), dim=c(50, 30, 2)),
          nodeSet=c("senders", "receivers"))
mynet
}
\keyword{classes}

\name{test_gof_auxiliary}
\alias{test_gof_auxiliary}
\alias{sienaGOF-auxiliary}
\alias{sparseMatrixExtraction}
\alias{networkExtraction}
\alias{behaviorExtraction}
\alias{OutdegreeDistribution}
\alias{IndegreeDistribution}
\alias{BehaviorDistribution}
\alias{TriadCensus}
\alias{mixedTriadCensus}
\alias{dyadicCov}
\alias{egoAlterCombi}
\alias{egoAlterCovarComb}

\title{Auxiliary functions for goodness of fit assessment by \code{\link{test_gof}}}
\description{
 The functions given here are auxiliary to function \code{\link{test_gof}}
 which assesses goodness of fit for actor-oriented models.

 The auxiliary functions are, first, some functions of networks or behaviour
 (i.e., statistics) for which the simulated values for the fitted model
 are compared to the observed value; second, some extraction functions
 to extract the observed and simulated networks and/or behaviour
 from the \code{\link{sienaFit}} object produced by \code{\link{siena}}
 with \code{returnDeps=TRUE}.

 These functions are exported here mainly to enable users to write
 their own versions. At the end of this help page some non-exported functions
 are listed. These are not exported because they depend on packages that
 are not in the R base distribution; and to show templates for readers
 wishing to contruct their own functions.
 }

\usage{
OutdegreeDistribution(i, obsData, sims, period, groupName, varName,
         levls=0:8, cumulative=TRUE)

IndegreeDistribution(i, obsData, sims, period, groupName, varName,
         levls=0:8, cumulative=TRUE)

BehaviorDistribution(i, obsData, sims, period, groupName, varName,
         levls=NULL, cumulative=TRUE)

TriadCensus(i, obsData, sims, period, groupName, varName, levls=1:16)

mixedTriadCensus(i, obsData, sims, period, groupName, varName)

dyadicCov(i, obsData, sims, period, groupName, varName, dc)

egoAlterCombi(i, obsData, sims, period, groupName, varName, trafo=NULL)

egoAlterCovarComb(i, obsData, sims, period, groupName, varName, covar)

sparseMatrixExtraction(i, obsData, sims, period, groupName, varName)

networkExtraction(i, obsData, sims, period, groupName, varName)

behaviorExtraction(i, obsData, sims, period, groupName, varName)
}
\arguments{
  \item{i}{ Index number of simulation to be extracted, ranging from 1 to
    \code{length(sims)}; if NULL, the data observation will be extracted.}
  \item{obsData}{ The observed data set to which the model was fitted; normally
    this is \code{x$f} where \code{x} is the \code{\link{sienaFit}}
    object for which the fit is being assessed.}
  \item{sims}{ The simulated data sets to be compared with the observed data;
    normally this is
    \code{x$sims} where \code{x} is the \code{\link{sienaFit}}
    object for which the fit is being assessed.}
  \item{period}{ Period for which data and simulations are used (may run from
    1 to number of waves - 1).}
  \item{groupName}{ Name of group; relevant for multi-group data sets;
    defaults in \code{\link{test_gof}} to "\code{Data1}".}
  \item{varName}{ Name of dependent variable. }
  \item{levls}{ Levels used as values of the auxiliary statistic.
    For \code{BehaviorDistribution}, this defaults to the observed
    range of values.}
  \item{cumulative}{ Are the distributions to be considered as raw or
    cumulative (<=) distributions?}
  \item{dc}{Dyadic covariate: either a matrix with dimensions
    \eqn{n \times n}{n * n}; or, as period-dependent values,
    an array with dimensions \eqn{n \times n \times (M-1)}{n * n * (M-1)};
    where \eqn{n} is the number of actors and \eqn{M} is the number of waves.
    There may be more time points, but those after \eqn{(M-1)} will not be used.}
  \item{trafo}{ For \code{egoAlterCombi}:
    transformation of the dependent behavior variable.
    Default: identity function. }
  \item{covar}{ For \code{egoAlterCovarComb}: arbitrary monadic variable, 
    vector of length \eqn{n},  the number of actors. }
}
\details{
 The statistics should be chosen to represent features of the network
 that are not explicitly fit by the estimation procedure but can be
 considered important properties that the model at hand should represent well.
 The three given here are far from a complete set;
 they will be supplemented in due time by statistics depending on networks and
 behavior jointly.
 The examples below give a number of other statistics, using the packages
 \code{sna} and \code{igraph}.

 The \code{levls} parameter must be adapted to the range of values that is
 considered important. For indegrees and outdegrees, the whole range should
 usually be covered. If the range is large, which could be the case, e.g.,
 for indegrees of two-mode networks where the second mode has few nodes,
 think about the possibility of making a selection such as
 \code{levls=5*(0:20)} or \code{levls=c(0:4,5*(1:20))};
 which in most cases will make sense only if \code{cumulative=TRUE}.

 The method signature for the auxiliary statistics generally is\cr
 \code{function(i, obsData, sims, period, groupName, varName, \dots)}.
 For constructing new auxiliary statistics, it is helpful to
 study the code of \code{OutdegreeDistribution}, \cr
 \code{IndegreeDistribution},
 and \code{BehaviorDistribution} and of the example functions below.

 \code{TriadCensus} returns the distribution of the Holland-Leinhardt triad
 census according to the algorithm by Batagelj and Mrvar
 (implementation by Parimalarangan, Slota, and Madduri). An alternative is the
 \code{TriadCensus.sna} function mentioned below, from package \code{sna}, which
 gives the same results. Here the \code{levls} parameter can be used to exclude
 some triads, e.g., for non-directed networks.\cr
 The Batagelj-Mrvar algorithm is optimized for sparse, large graphs and may be
 much faster than the procedure implemented in \code{sna}. For dense graphs
 the \code{sna} procedure may be faster.

 \code{dyadicCov} assumes that \code{dc} is a categorical dyadic variable,
 and returns the frequencies of the non-zero values for realized ties.
 It is recommended to do this for a categorical covariate.
 Since zero values of dc are not counted, it may be advisable
 to code dc so that all non-diagonal values are non-zero,
 and all diagonal values are zero.

 \code{egoAlterCombi} returns the frequencies of realized ties for each
  ego-alter combination of values of the dependent behavior variable,
  transformed by trafo; if \code{NULL}, untransformed.\cr
  The range of this transformation should preferably have 5 or fewer values,
  and definitely not 10 or more, as this would lead to problems in
  the coding of ego-alter combinations.\cr
 \code{egoAlterCovarComb} does the same, for arbitrary monadic 
  variables; e.g., transformed or untransformed covariates.\cr
  Here also, the range of \code{covar} should preferably have 5 or fewer values.
  Missing values are allowed.
}

\value{
 \code{OutdegreeDistribution} returns a named vector, the distribution
  of the observed or simulated outdegrees for the values in \code{levls}.

 \code{IndegreeDistribution} returns a named vector, the distribution
  of the observed or simulated indegrees for the values in \code{levls}.

 \code{BehaviorDistribution} returns a named vector, the distribution of the
  observed or simulated behavioral variable for the values in \code{levls}.

 \code{TriadCensus} returns a named vector, the distribution of the
  Holland-Leinhardt triad census according to the algorithm by
  Batagelj and Mrvar.

 \code{mixedTriadCensus} returns a named vector, the distribution of the
  mixed triad census of Hollway, Lomi, Pallotti, and Stadtfeld (2017).
  See their Figure 1 for the meaning of the codes.
  In this figure, ties between the bottom nodes are for the first network,
  ties from the bottom to the top nodes are for the second network.
  The mixed triad census can be used for pairs of dependent networks of which
  the first must be one-mode and the second can be one-mode or two-mode.
  If the second is one-mode, the set of triads considered is only a subset
  of all mixed triads, and ties in the figure are directed upward;
  existence of other ties is not considered.

 \code{dyadicCov} returns a named vector, the frequencies of the non-missing
  non-zero values dc(ego,alter) of the observed or simulated (ego,alter) ties.
  
 \code{egoAlterCombi} and \code{egoAlterCovarComb} return  a named vector
  with the frequencies of realized ties for ego-alter combinations
  as indicated above. The range of \code{trafo} or \code{covar}, respectively,
  should be a subset of the values \code{0, 1, 2, ..., 9}. Then the name
  is composed of a number having two digits; the first digit is ego's value,
  the second alter's  value.  
  
 \code{sparseMatrixExtraction} returns the simulated network as a sparse matrix
  in the \code{"TsparseMatrix-class"};
  this is the virtual class for sparse numeric matrices
  represented by triplets in the \code{Matrix} package.\cr
  Tie variables for ordered pairs with a missing value for wave=period or
  period+1 are zeroed; note that this also is done
  in \code{RSiena} for calculation of target statistics.
  Tie variables that are structurally determined at the beginning of a period
  are used to replace observed values at the end of the period;
  tie variables that are structurally determined at the end, but not
  the beginning, of a period are used to replace
  simulated values at the end of the period.
  \cr
  To treat the objects returned by this function as regular matrices,
  it is necessary to attach the \code{Matrix} package in your session.

\code{networkExtraction} returns the network as an edge list of class
  \code{network} according to the \code{network} package
  (used for package \code{sna}).
  Missing values and structural values are treated as in
  \code{sparseMatrixExtraction}, see above.

\code{behaviorExtraction} returns the dependent behavior
  variable as an integer vector.
  Values for actors with a missing value for wave=period or period+1 are
  transformed to \code{NA}.
}

\references{
   Batagelj, V., and Mrvar, A. (2001),
  A subquadratic triad census algorithm for large sparse networks with small
  maximum degree. \emph{Social Networks}, \bold{23}, 237--243.

   Holland, P.W., and Leinhardt, S. (1976),
  Local structure in social networks. \emph{Sociological Methodology}, \bold{6}, 1--45.

   Hollway, J., Lomi, A., Pallotti, F., and Stadtfeld, C. (2017),
   Multilevel social spaces: The network dynamics of organizational fields.
   \emph{Network Science}, \bold{5}, 187--212.

   Lospinoso, J.A. and Snijders, T.A.B. (2019),
  Goodness of fit for stochastic actor-oriented models.
  \emph{Methodological Innovations}, \bold{12}:2059799119884282.

   Parimalarangan S., Slota, G.M., and Madduri, K. (2017),
 Fast parallel graph triad census and triangle counting on shared-memory
  platforms, \emph{2017 IEEE International Parallel and Distributed Processing
  Symposium Workshops (IPDPSW)}, Lake Buena Vista, FL, pp. 1500-1509.
}
\author{Josh Lospinoso, Tom Snijders}
\seealso{\code{\link{siena}}, \code{\link{test_gof}} }
\examples{
### For use out of the box:

mynet1 <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mybeh <- as_dependent_rsiena(s50a[,1:2], type="behavior")
mydycov <- matrix(rep(1:5, 500), 50, 50) # artificial, just for trying
mydata <- make_data_rsiena(mynet1, mybeh)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, list(transTies, cycle3))
# Shorter phases 2 and 3, just for example:
myalgo <- set_algorithm_saom(nsub=1, n3=50, seed=122)
(ans <- siena(mydata, effects=myeff, control_algo=myalgo,
                                returnDeps=TRUE, silent=TRUE, batch=TRUE))
# NULL for the observations:
OutdegreeDistribution(NULL, ans$f, ans$sims, period=1, groupName="Data1",
  levls=0:7, varName="mynet1")
dyadicCov(NULL, ans$f, ans$sims, period=1, groupName="Data1",
  dc=mydycov, varName="mynet1")
# An arbitrary selection for simulation run i:
IndegreeDistribution(5, ans$f, ans$sims, period=1, groupName="Data1",
  varName="mynet1")
BehaviorDistribution(20, ans$f, ans$sims, period=1, groupName="Data1",
  varName="mybeh")
sparseMatrixExtraction(50, ans$f, ans$sims, period=1, groupName="Data1",
  varName="mynet1")
networkExtraction(40, ans$f, ans$sims, period=1, groupName="Data1",
  varName="mynet1")
behaviorExtraction(50, ans$f, ans$sims, period=1, groupName="Data1",
  varName="mybeh")

gofi <- test_gof(ans, IndegreeDistribution, verbose=TRUE, join=TRUE,
  varName="mynet1")
gofi
plot(gofi)

(gofo <- test_gof(ans, OutdegreeDistribution, verbose=TRUE, join=TRUE,
    varName="mynet1", cumulative=FALSE))
# cumulative is an example of "\dots".
plot(gofo)

traf <- function(z){pmax(pmin(z,4),2)}
(gofea <- test_gof(ans, egoAlterCombi, verbose=TRUE, join=TRUE,
    varName=c("mynet1","mybeh"), trafo=traf))
plot(gofea)
descriptives(gofea, showAll=TRUE)

cova <- c(rep(1:3, 16), NA, 1)
(gofea <- test_gof(ans, egoAlterCovarComb, verbose=TRUE, join=TRUE,
    varName="mynet1", covar=cova))
plot(gofea)

(gofdc <- test_gof(ans, dyadicCov, verbose=TRUE, join=TRUE,
    dc=mydycov, varName="mynet1"))
plot(gofdc)

(gofb <- test_gof(ans, BehaviorDistribution, varName = "mybeh",
    verbose=TRUE, join=TRUE, cumulative=FALSE))
plot(gofb)

(goftc <- test_gof(ans, TriadCensus, verbose=TRUE, join=TRUE,
    varName="mynet1"))
plot(goftc, center=TRUE, scale=TRUE)
# For this type of auxiliary statistics
# it is advisable in the plot to center and scale.
# note the keys at the x-axis (widen the plot if they are not clear).
descriptives(goftc)

### The mixed triad census for co-evolution of one-mode and two-mode networks:
actors <- as_nodeset_rsiena(50, nodeSetName="actors")
activities <- as_nodeset_rsiena(20, nodeSetName="activities")
onemodenet <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)),
                nodeSet="actors")
# Not meaningful, just for example:
twomodenet <- as_dependent_rsiena(
                array(c(s502[1:50, 1:20], s503[1:50, 1:20]), dim=c(50, 20, 2)),
                type= "bipartite", nodeSet=c("actors", "activities"))
twodata <- make_data_rsiena(onemodenet, twomodenet,
                nodeSets=list(actors, activities))
twoeff <- make_specification(twodata)
twoeff <- set_effect(twoeff, outActIntn, depvar="onemodenet",
                covar1="twomodenet")
twoeff <- set_effect(twoeff, outActIntn, depvar="twomodenet",
                covar1="onemodenet")
twoeff <- set_effect(twoeff, from, depvar="onemodenet", covar1="twomodenet")
twoeff <- set_effect(twoeff, to, depvar="twomodenet", covar1="onemodenet")
twoeff
# Shorter phases 2 and 3, just for example:

twoalgo <- set_algorithm_saom(nsub=1, n3=50, seed=1291)
(ans <- siena(twodata, effects=twoeff, control_algo=twoalgo,
                returnDeps=TRUE, silent=TRUE, batch=TRUE))
(gof.two <- test_gof(ans, mixedTriadCensus,
                varName=c("onemodenet", "twomodenet"), verbose=TRUE))
plot(gof.two, center=TRUE, scale=TRUE)

\dontrun{
### Here come some useful functions for building your own auxiliary statistics:
### First an extraction function.

# igraphNetworkExtraction extracts simulated and observed networks
# from the results of a siena run.
# It returns the network as an edge list of class "graph"
# according to the igraph package.
# Ties for ordered pairs with a missing value for wave=period or period+1
# are zeroed;
# note that this also is done in RSiena for calculation of target statistics.
# However, changing structurally fixed values are not taken into account.
igraphNetworkExtraction <- function(i, data, sims, period, groupName, varName) {
  require(igraph)
  dimsOfDepVar <- attr(data[[groupName]]$depvars[[varName]], "netdims")[1]
  missings <- is.na(data[[groupName]]$depvars[[varName]][,,period]) |
    is.na(data[[groupName]]$depvars[[varName]][,,period+1])
  if (is.null(i)) {
    # test_gof wants the observation:
    original <- data[[groupName]]$depvars[[varName]][,,period+1]
    original[missings] <- 0
    returnValue <- graph.adjacency(original)
  }
  else
  {
    missings <- graph_from_adjacency_matrix(missings)
    #test_gof wants the i-th simulation:
    returnValue <- difference(
      make_empty_graph(dimsOfDepVar) +
        edges(t(sims[[i]][[groupName]][[varName]][[period]][,1:2])),
      missings)
  }
  returnValue
}

### Then some auxiliary statistics.

# GeodesicDistribution calculates the distribution of non-directed
# geodesic distances; see ?sna::geodist
# The default for \code{levls} reflects that
# geodesic distances larger than 5 do not differ appreciably
# with respect to interpretation.
# Note that the levels of the result are named;
# these names are used in the \code{plot} method.
GeodesicDistribution <- function (i, data, sims, period, groupName,
  varName, levls=c(1:5,Inf), cumulative=TRUE, ...) {
  x <- networkExtraction(i, data, sims, period, groupName, varName)
  require(network)
  require(sna)
  a <- sna::geodist(symmetrize(x))$gdist
  if (cumulative)
  {
    gdi <- sapply(levls, function(i){ sum(a<=i) })
  }
  else
  {
    gdi <- sapply(levls, function(i){ sum(a==i) })
  }
  names(gdi) <- as.character(levls)
  gdi
}

# Holland and Leinhardt Triad Census from sna; see ?sna::triad.census.
# For undirected networks, call this with levls=1:4
TriadCensus.sna <- function(i, data, sims, period, groupName,
                            varName, levls=1:16){
  unloadNamespace("igraph") # to avoid package clashes
  require(network)
  require(sna)
  x <- networkExtraction(i, data, sims, period, groupName, varName)
  if (network.edgecount(x) <= 0){x <- symmetrize(x)}
  # because else triad.census(x) will lead to an error
  tc <- sna::triad.census(x)[levls]
  # names are transferred automatically
  tc
}

# Holland and Leinhardt Triad Census from igraph; see ?igraph::triad_census.
TriadCensus.i <- function(i, data, sims, period, groupName, varName){
  unloadNamespace("sna") # to avoid package clashes
  require(igraph)
  x <- igraphNetworkExtraction(i, data, sims, period, groupName, varName)
# suppressWarnings is used because else warnings will be generated
# when a generated network happens to be symmetric.
  setNames(suppressWarnings(triad_census(x)),
        c("003", "012", "102", "021D","021U", "021C", "111D", "111U",
          "030T", "030C", "201",  "120D", "120U", "120C", "210", "300"))
}

# CliqueCensus calculates the distribution of the clique census
# of the symmetrized network; see ?sna::clique.census.
CliqueCensus<-function (i, obsData, sims, period, groupName,
                        varName, levls = 1:5){
  require(sna)
  x <- networkExtraction(i, obsData, sims, period, groupName, varName)
  cc0 <- sna::clique.census(x, mode='graph', tabulate.by.vertex = FALSE,
    enumerate=FALSE)[[1]]
  cc <- 0*levls
  names(cc) <- as.character(levls)
  levels.used <- as.numeric(intersect(names(cc0), names(cc)))
  cc[levels.used] <- cc0[levels.used]
  cc
}

# Distribution of Bonacich eigenvalue centrality; see ?igraph::eigen_centrality.
EigenvalueDistribution <- function (i, data, sims, period, groupName,
                   varName, levls=c(seq(0,1,by=0.125)), cumulative=TRUE){
  require(igraph)
  x <- igraphNetworkExtraction(i, data, sims, period, groupName, varName)
  a <- igraph::eigen_centrality(x)$vector
  a[is.na(a)] <- Inf
  lel <- length(levls)
  if (cumulative)
  {
    cdi <- sapply(2:lel, function(i){sum(a<=levls[i])})
  }
  else
  {
    cdi <- sapply(2:lel, function(i){
      sum(a<=levls[i]) - sum(a <= levls[i-1])})
  }
  names(cdi) <- as.character(levls[2:lel])
  cdi
}

## Finally some examples
## of the three auxiliary statistics constructed above.
mynet1 <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <- as_dependent_rsiena(s50a, type="behavior")
mydata <- make_data_rsiena(mynet1, mybeh)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, list(transTrip, cycle3))
myeff <- set_effect(myeff, outdeg, depvar="mybeh", covar1="mynet1")
myeff <- set_effect(myeff,  outdeg, depvar="mybeh", covar1="mynet1")
# Shorter phases 2 and 3, just for example:
myalgo <- set_algorithm_saom(nsub=1, n3=100, seed=1291)
(ans2 <- siena(mydata, effects=myeff, control_algo=myalgo,
           returnDeps=TRUE, batch=TRUE))
gofc <- test_gof(ans2, EigenvalueDistribution, varName="mynet1",
  verbose=TRUE, join=TRUE)
plot(gofc)
descriptives(gofc, showAll=TRUE)

goftc <- test_gof(ans2, TriadCensus, varName="mynet1",
           verbose=TRUE, join=TRUE)
plot(goftc, center=TRUE, scale=TRUE)
# For this type of auxiliary statistics
# it is advisable in the plot to center and scale.
# note the keys at the x-axis; these names are given by sna::triad.census
descriptives(goftc)
round(descriptives(goftc))

gofgd <- test_gof(ans2, GeodesicDistribution, varName="mynet1",
           verbose=TRUE, join=TRUE, cumulative=FALSE)
plot(gofgd)
# and without infinite distances:
gofgdd <- test_gof(ans2, GeodesicDistribution, varName="mynet1",
           verbose=TRUE, join=TRUE, levls=1:7, cumulative=FALSE)
plot(gofgdd)
}
}
\keyword{models}
\keyword{methods}

\name{as_composition_rsiena}
\Rdversion{1.1}
\alias{sienaCompositionChange}
\alias{sienaCompositionChangeFromFile}
\alias{as_composition_rsiena}
\alias{as_composition_file_rsiena}
\title{
Functions to create a Siena composition change object
}
\description{Used to create a list of events describing the changes    
  over time of a Siena actor set.\cr  
  The functions \code{as_composition_rsiena} and 
  \code{sienaCompositionChange} \cr
  are identical; the same holds for 
  \code{as_composition_file_rsiena} and \cr
  \code{sienaCompositionChangeFromFile}. 
  The first of these names are preferred, 
  the others are kept for backward compatibility.
}
\usage{
as_composition_rsiena(changelist, nodeSet = "Actors", option = 1)
as_composition_file_rsiena(filename, nodeSet = "Actors",
    fileobj=NULL, option = 1)

sienaCompositionChange(changelist, nodeSet = "Actors", option = 1)
sienaCompositionChangeFromFile(filename, nodeSet = "Actors",
    fileobj=NULL, option = 1)
}
\arguments{
  \item{changelist}{
    A list with an entry for each actor in the node set. Each entry a
    vector of numbers (may be as characters) indicating intervals during
    which the corresponding actor was present.
    Each entry must have an even number of digits.
    The actor is assumed to be present from the first to the second,
    third to fourth, etc., time points.
}
\item{filename}{
  Name of file containing change information. One line per actor, each
  line a series
  of space delimited numbers indicating intervals.
}
\item{fileobj}{
  The result of \command{readLines} on filename.
}
\item{nodeSet}{
  Character string containing the name of a Siena node set.
  If the entire data set contains more than one node set,
  then the node sets must be specified in all data objects.
}
\item{option}{
  Integer controlling the processing of the tie variables for the
  actors not currently present. Values (default is 1)
  \tabular{rl}{
1 \tab  0 before entry, final value carried forward after leaving,\cr
  \tab  and used for calculating statistics in Method of Moments estimation\cr
2 \tab  0 before entry, missing after (final value carried forward, but
          treated as missing) \cr
3 \tab  missing whenever not in the network. Previous values will be used
        where available, \cr
  \tab  but always treated as missing values.\cr
4 \tab  Convert to structural zeros (not available at present).
}
}
}
\details{
  If there is a composition change object for the first node set in the
  data object, then this will be used in estimation by the Method of Moments
  to make actors active
  (able to send and receive ties) only for the time intervals
  when this is indicated in the composition change object.
  This is done according to the procedure of Huisman and Snijders (2003).
  See the manual for further details.\cr
  For bipartite networks, composition change objects for the second node set
  have no effect and will lead to an error message.\cr
  For \code{M} waves, time starts at 1 and ends at \code{M};
  so all numbers must be between 1 and the number of waves (bounds included).
  Intervals are treated as closed at each end.
  For example, an entry \code{(2, 4)} means that the actor corresponding
  to this entry arrived at wave 2 and left at wave 4, but did give
  valid date for both of these waves. An entry \code{(1.01, 2.99)} means that
  the actor arrived just after wave 1 and left just before wave 3, and gave
  valid data only for wave 2. An entry \code{(1, 2), (3.5, 4)} means that the
  actor was there at the start and left at wave 2 (giving valid data for wave 2),
  came back halfway between waves 3 and 4, and gave valid data still at wave 4;
  if there would be more than 4 waves in the data set, this entry would also
  mean that the actor left at wave 4.\cr
  Function \code{as_composition_rsiena} reads input from a list, \cr
  and \code{as_composition_file_rsiena} from a file.
  Further, they do the same.  
\cr
  For data sets including a composition change object, estimation
  by Method of Moments is forced to be unconditional, overriding
  the specification in the \code{\link{sienaAlgorithm}} object.
}
\value{
  An object of class "compositionChange", a list of numeric vectors,
  with attributes:
  \item{NodeSet}{Name of node set}
  \item{Option}{Option}
}
\references{
    Huisman, M.E. and Snijders, T.A.B. (2003),
   Statistical analysis of longitudinal network data with changing composition.
   \emph{Sociological Methods & Research}, \bold{32}, 253--287.
   
   See also \url{https://www.stats.ox.ac.uk/~snijders/siena/RSiena_Manual.pdf}
    
    Further see \url{https://www.stats.ox.ac.uk/~snijders/siena/}
   }
\author{Ruth Ripley }
\seealso{
 \code{\link{as_nodeset_rsiena}},
 \code{\link{make_data_rsiena}}
}
\examples{
  clist <- list(c(1, 3), c(1.4, 2.5))
  #or
  clist <- list(c("1", "3"), c("1.4", "2.5"))

  compChange <- as_composition_rsiena(clist)

  s50net <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
  s50list <- rep(list(c(1,3)), 50)
  # This is a trivial composition change: all actors are present in all waves.
  compChange <- as_composition_rsiena(s50list)
  s50data <- make_data_rsiena(s50net, compChange)
  s50data

  \dontrun{
  filedata <- c("1 3", "1.4 2.5")
  write.table(filedata, "cc.dat",row.names=FALSE, col.names=FALSE,
          quote=FALSE)
  ## file will be
  ## 1 3
  ## 1.4 2.5
  compChange <- as_composition_file_rsiena("cc.dat")
  }
}
% Add one or more standard keywords, see file "KEYWORDS" in the
% R documentation directory.
\keyword{classes}

\name{test_gof}
\alias{test_gof}
\alias{test_gof.sienaFit}
\alias{sienaGOF}
\alias{plot.sienaGOF}
\alias{descriptives.sienaGOF}
\alias{descriptives}
\title{Functions to assess goodness of fit for SAOMs}
\description{
 The function \code{sienaGOF} assesses goodness of fit
 for a model specification as represented by an estimated
 \code{sienaFit} object created by \code{\link{siena}}.
 This is done by simulations of auxiliary
 statistics, that differ from the statistics used for estimating
 the parameters. The auxiliary statistics must be given explicitly.

 The fit is good if the average values
 of the auxiliary statistics over many simulation runs are close
 to the values observed in the data.
 A Monte Carlo test based on the Mahalanobis
 distance is used to calculate frequentist \eqn{p}-values.

 Plotting functions can be used to diagnose bad fit. There are basic
 functions for calculating auxiliary statistics available out of the
 box, and the user is invited to create additional ones.

  \code{test_gof} and \code{sienaGOF}
  perform the same estimation. The first of these is preferred,
  the other is kept for backward compatibility.
 }

\usage{
\method{test_gof}{sienaFit}(object, auxiliaryFunction,
    period=NULL, verbose=FALSE, join=TRUE, twoTailed=FALSE,
    cluster=NULL, robust=FALSE, groupName="Data1",
    varName, tested=FALSE, iterations=NULL, giveNAWarning=TRUE, \dots)

sienaGOF(object, auxiliaryFunction,
    period=NULL, verbose=FALSE, join=TRUE, twoTailed=FALSE,
    cluster=NULL, robust=FALSE, groupName="Data1",
    varName, tested=FALSE, iterations=NULL, giveNAWarning=TRUE, \dots)
\method{plot}{sienaGOF}(x, center=FALSE, scale=FALSE, violin=TRUE, key=NULL,
    perc=.05, period=1, showAll=FALSE, position=4, fontsize=12, \dots)
\method{descriptives}{sienaGOF}(x, center=FALSE, scale=FALSE, perc=.05, key=NULL,
    period=1, showAll=FALSE, \dots)
}
\arguments{
  \item{object}{An object of class \code{\link{sienaFit}}, produced by a
    call to \code{\link{siena}} with \code{returnDeps = TRUE} and
    \code{maxlike=FALSE} (the latter is the default, the former is not);
    or a list of such objects; if a list, then the first period
    of each \code{\link{sienaFit}} object will be used.\cr
    If this is a list of \code{\link{sienaFit}} objects,
    where \code{object} is mentioned below, it refers to the first
    element of this list.}
  \item{auxiliaryFunction}{Function to be used to calculate
    the auxiliary statistics;
    this can be a user-defined function, e.g. depending on the \code{sna}
    or \code{igraph} packages.

    See \code{Examples} and \code{\link{test_gof_auxiliary}}
    for more information on the signature of this function.
    The basic signature is\cr
    \code{function(index, data, sims, period, groupName, varName, \dots)},
    where \code{index} is the index of the simulated network,
    or \code{NULL} if the observed variable is needed;
    \code{data} is the observed data object from which
    the relevant variables are extracted; \code{sims} is the list of simulations
    returned from \code{\link{siena}};
    \code{period} is the index of the period;
    and \dots are further arguments (like \code{levls} in the examples below
    and in \code{\link{test_gof_auxiliary}}).}
  \item{period}{Vector of period(s) to be used (may run from 1 to
    number of waves - 1). Has an effect only if \code{join=FALSE}.\cr
    May be only a single number if \code{object} is a list of
    \code{\link{sienaFit}} objects.}
  \item{verbose}{Whether to print intermediate results. This may give some
       peace of mind to the user because calculations can take some time.}
  \item{join}{Boolean: should sienaGOF do tests on all of the periods
       individually (\code{FALSE}), or sum across periods (\code{TRUE})?}
  \item{twoTailed}{Whether to use two tails for calculating \eqn{p}-values on
        the Monte Carlo test. Recommended for advanced users only,
        as it is probably only applicable in rare cases.}
  \item{cluster}{Optionally, a \code{parallel} or \code{snow} cluster to execute
        the auxiliary function calculations on.}
  \item{robust}{ Whether to use robust estimation of the covariance matrix.}
  \item{groupName}{ Name of group; relevant for multi-group data sets.}
  \item{varName}{ Name of dependent variable.}
  \item{tested}{ A logical vector of length \code{object$pp}
          (number of parameters), indicating a subset of tested parameters;
          or \code{NULL}, indicating all tested parameters (see below);
          or \code{FALSE}, indicating nothing is to be tested.}
  \item{iterations}{ Number of iterations for the goodness of fit calculations.
        If \code{NULL}, the number of simulated data sets in
        \code{object}.  }
  \item{giveNAWarning}{If \code{TRUE}, a warning is given if any simulated
        values are missing.}
  \item{x}{ Result from a call to sienaGOF. }
  \item{center}{ Whether to center the statistics by median during plotting.}
  \item{scale}{ Whether to scale the statistics by range during plotting.
                scale=\code{TRUE} makes little sense without also
                center=\code{TRUE}.}
  \item{violin}{ Use violin plots (vs. box plots only)? }
  \item{key}{ Keys in the plot for the levels of the auxiliary statistic
        (as given by parameter \code{levls} in the examples). }
  \item{perc}{ 1 minus confidence level for the confidence bands (two sided). }
  \item{position}{ Position where the observed value is plotted: 1=under,
        2=to the left, 3=above, 4=to the right of the red dot. Can be a single
        number from 1 to 4, or a vector with positions for each statistic
        (possibly recycled).}
  \item{fontsize}{ Font size for the observed values plotted.}
  \item{\dots}{Other arguments; for \code{test_gof}, e.g., \code{levls} as a
       parameter for the auxiliary statistic in
       \code{\link{test_gof_auxiliary}};\cr
       for \code{plot.sienaGOF()}, e.g., the usual plotting parameters
       \code{main}, \code{xlab}, \code{ylab}, \code{cex}, \code{cex.main},
       \code{cex.lab}, and \code{cex.axis}.}
  \item{showAll}{If \code{FALSE}, drops statistics with variance 0.}
}
\details{
 This function is used to assess the goodness of fit of an estimated stochastic
 actor-oriented model for an arbitrarily defined multidimensional auxiliary
 statistic. It operates basically by comparing the observed values, at the ends
 of the periods, with the simulated values for the ends of the periods.
 The differences are assessed by combining the components of the auxiliary
 statistic using the Mahalanobis distance.

 For sienaFit objects that were made for a multi-group data set,
 if you are not sure about the \code{groupName}s to use, these can be retrieved
 by the command \code{"names(dataObject)"} (where \code{dataObject} is
 the data used to produce the input \code{object}).
 Mostly they are \code{"Data1", "Data2"}, etc.

 To save computation time, \code{iterations} can be set to a lower number
 than what is available in \code{object}; this will yield
 a less precise result.

 The function does not work properly for data sets that include a
 \code{\link{sienaCompositionChange}} object. If you wish to test the
 fit for such a data set, you need (for the purpose of fit assessment only)
 to replace the data set by a data set where absent actors are represented
 by structural zeros, and estimate the same model for this data set with the
 corresponding effects object, and use \code{sienaGOF} for this
 \code{\link{sienaFit}} object.

 To achieve comparability between simulated and observed dependent variables,
 variables that are missing in the data at the start or end of a period
 are replaced by 0 (for tie variables) or NA (for behavior variables).\cr
 If there are any differences between structural values at the beginning and
 at the end of a period, these are dealt with as follows.
 For tie variables that have a structural value at the start of the period,
 this value is used to replace the observed value at the end of the period
 (for the goodness of fit assessment only).
 For tie variables that have a structural value at the end of the period but a
 free value value at the start of the period, the reference value for the
 simulated values is lacking; therefore, the simulated values at the end of
 the period then are replaced by the structural value at the end of the period
 (again, for the goodness of fit assessment only).

 The auxiliary statistics documented in \code{\link{test_gof_auxiliary}}
 are calculated for the simulated dependent
 variables in Phase 3 of the estimation algorithm,
 returned in \code{object} because of having used
 \code{returnDeps = TRUE} in the call to \code{\link{siena}}.
 These statistics should be chosen to represent features of the network
 that are not explicitly fit by the estimation procedure but can be
 considered important properties that the model at hand should represent well.
 Some examples are:
 \itemize{
 \item Outdegree distribution
 \item Indegree distribution
 \item Distribution of the dependent behavior variable (if any).
 \item Distribution of geodesic distances
 \item Triad census
 \item Edgewise homophily counts
 \item Edgewise shared partner counts
 \item Statistics depending on the combination of network
       and behavioral variables.
}
The function is written so that the user can easily define other functions to
capture some other relevant aspects of the network, behaviors, etc. \cr
This is further illustrated in the help page
\code{\link{test_gof_auxiliary}}.

We recommend the following heuristic approach to model checking:
\enumerate{
 \item Check convergence of the estimation.
 \item Assess goodness of fit (primarily using \code{join=TRUE}) on
  auxiliary statistics, and if necessary refine the model.
 \item Assess time heterogeneity by \code{\link{sienaTimeTest}}
   and if there is evidence for time heterogeneity
   either modify the base effects or include time dummy terms.
}
 No general rules can be given about whether time heterogeneity
 (\code{\link{test_time}}) or goodness of fit using \code{test_gof}
 have precedence. This is an explorative issue.

 The \code{summary} function will display some useful information
 to help with model selection if some effects are set in the effects object
 to be fixed and tested. In that case, for all parameters indicated
 in the vector \code{tested}, a rough estimator is computed for the Mahalanobis
 distance that would be obtained at each proposed specification.
 This is then given in the \code{summary}. This can help guide model selection.
 This estimator is called the modified Mahalanobis distance (MMD).
 See Lospinoso and Snijders (2019) or the manual for more information.

 The following functions are pre-fabricated for ease of use, and can
 be passed in as the \cr
 \code{auxiliaryFunction} with no extra effort;
 see \code{\link{test_gof_auxiliary}} and the examples below.
\itemize{
 \item \code{\link{IndegreeDistribution}}
 \item \code{\link{OutdegreeDistribution}}
 \item \code{\link{BehaviorDistribution}}
 \item \code{\link{TriadCensus}}
 \item \code{\link{mixedTriadCensus}}
 \item \code{\link{dyadicCov}}
}

}
\value{
  The result is of class \code{sienaGOF};
    this is a list of elements of class \code{sienaGofTest};
    if \code{join=TRUE}, the list has length 1; if \code{join=FALSE},
    each list element corresponds to a period analyzed;
    the list elements are themselves lists again, including the
    following elements:
    \item{- sienaFitName}{The name of the input \code{object}.}
    \item{- auxiliaryStatisticName}{The name of auxiliaryFunction.}
    \item{- Observations}{The observed values for the auxiliary statistics.}
    \item{- Simulations}{The simulated auxiliary statistics.}
    \item{- ObservedTestStat}{The observed Mahalanobis distance in the data.}
    \item{- SimulatedTestStat}{The Mahalanobis distance for the simulations.}
    \item{- TwoTailed}{Whether the \eqn{p}-value corresponds to a one- or
                     two-tailed Monte Carlo test.}
    \item{- p}{The \eqn{p}-value for the observed Mahalanobis distance in the
             permutation distribution of the
             simulated Mahalanobis distances.}
    \item{- Rank}{ Rank of the covariance matrix of the simulated auxiliary
              statistics.}
    In addition there are several attributes which give, for model
    specifications with fixed-and-tested effects, approximations
    to the expected Mahalanobis distance for model specifications where
    each of these effects would be added. This is reported
    in the \code{summary} method. \cr
    The \code{plot} method makes violin plots or box plots, with superimposed
    confidence bands, for the simulated distributions of all elements
    of the \code{auxiliaryFunction},
    with the observed values indicated by red dots;
    but statistics with variance 0 are dropped.\cr
    \code{sienaGOF} objects can also be plotted by package \code{autograph}.

    \code{descriptives.sienaGOF} returns a matrix giving numerical
    information about what is plotted in the \code{plot} method:
    maximum, upper percentile, mean, median, lower percentile, minimum,
    and standard deviation
    of the simulated distributions of the auxiliary statistics, the
    observed values, and the proportions of simulated values greater
    and greater-or-equal than the observed values.
    If \code{center=TRUE} the median is subtracted
    from mean, median, and percentiles; if \code{scale=TRUE}
    these numbers and the standard deviation
    are divided by (maximum - minimum). \cr
    If \code{showAll=FALSE}, statistics with variance 0 will be dropped.
}

\references{
 Lospinoso, J.A. and Snijders, T.A.B. (2019,
  Goodness of fit for stochastic actor-oriented models.
  \emph{Methodological Innovations}, \bold{12}:2059799119884282.

  Also see \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Josh Lospinoso, modifications by Ruth Ripley and Tom Snijders}
\seealso{\code{\link{siena}}, \code{\link{test_gof_auxiliary}},
         \code{\link{test_time}} }
\examples{
mynet  <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mybeh  <- as_dependent_rsiena(s50a[,1:2], type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
myeff  <- make_specification(mydata)
myeff  <- set_effect(myeff, transTrip)
myeff  <- set_effect(myeff, cycle3, fix=TRUE, test=TRUE)
myeff  <- set_effect(myeff, transTies, fix=TRUE, test=TRUE)
myalgo <- set_algorithm_saom(nsub=1, n3=20, seed=1291)
# Shorter phases 2 and 3, just for example.
(ans <- siena(mydata, effects=myeff, control_algo=myalgo,
                    silent=TRUE, batch=TRUE, returnDeps=TRUE))
gofi <- test_gof(ans, IndegreeDistribution, verbose=TRUE, join=TRUE,
                    varName="mynet")
summary(gofi)
plot(gofi)

# Illustration just for showing a case with two dependent networks;
# running time backwards is not meaningful!
mynet1 <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mynet2 <- as_dependent_rsiena(array(c(s503, s501), dim=c(50, 50, 2)))
mybeh <- as_dependent_rsiena(s50a[,1:2], type="behavior")
mydata <- make_data_rsiena(mynet1, mynet2, mybeh)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, transTrip)
myeff <- set_effect(myeff, recip, depvar="mynet2")
# Shorter phases 2 and 3, just for example.
(ans <- siena(mydata, effects=myeff, control_algo=myalgo,
              silent=TRUE, batch=TRUE, returnDeps=TRUE))
gofi <- test_gof(ans, IndegreeDistribution, verbose=TRUE, join=TRUE,
                 varName="mynet1")
summary(gofi)
plot(gofi)

\dontrun{
(gofi.nc <- test_gof(ans, IndegreeDistribution, cumulative=FALSE,
                     varName="mynet1"))
# cumulative is an example of "...".
plot(gofi.nc)
descriptives(gofi.nc)

(gofi2 <- test_gof(ans, IndegreeDistribution, varName="mynet2"))
plot(gofi2)

(gofb <- test_gof(ans, BehaviorDistribution, varName = "mybeh"))
plot(gofb)

(gofo <- test_gof(ans, OutdegreeDistribution, varName="mynet1",
                  levls=0:6, cumulative=FALSE))
# levls is another example of "...".
plot(gofo)
}

## A demonstration of using multiple processes
\dontrun{
library(parallel)
(n.clus <- detectCores() - 1)
n.clus <- min(n.clus, 4)  # keep time for other processes
myalgorithm.c <- sienaAlgorithmCreate(nsub=4, n3=1000, seed=1265)
(ans.c <- siena(mydata, effects=myeff, control_algo=myalgo,
                nbrNodes=n.clus, silent=TRUE, batch=TRUE,
				returnDeps=TRUE))
(ans.c <- siena07(myalgorithm.c, data=mydata, effects=myeff, batch=TRUE,
                returnDeps=TRUE, useCluster=TRUE, nbrNodes=n.clus))
gofi.1 <- test_gof(ans.c, TriadCensus, verbose=TRUE, varName="mynet1")
cl <- makeCluster(n.clus)
gofi.cl <- test_gof(ans.c, TriadCensus, varName="mynet1", cluster=cl)
cl2 <- makeCluster(2)
gofi.cl2 <- test_gof(ans.c, TriadCensus, varName="mynet1", cluster=cl2)
# compare simulation times
attr(gofi.1,"simTime")
attr(gofi.cl,"simTime")
attr(gofi.cl2,"simTime")
}
}
\keyword{models}

\name{print.sienaEffects}
\alias{print.sienaEffects}
\alias{summary.sienaEffects}
\alias{print.summary.sienaEffects}
\title{
  Print methods for Siena effects objects
}
\description{
  Prints the major columns of the effects object.
  Or all, with any non-atomic columns listed separately.
}
\usage{
\method{print}{sienaEffects}(x, fileName = NULL, includeOnly=TRUE,
   expandDummies = FALSE, includeRandoms = FALSE, dropRates=FALSE,
   includeShortNames=FALSE, ...)
\method{summary}{sienaEffects}(object, fileName = NULL,
   includeOnly=TRUE, expandDummies = FALSE, ...)
\method{print}{summary.sienaEffects}(x, fileName = NULL, ...)
}
\arguments{
  \item{object}{An object of class \code{sienaEffects}.}
  \item{x}{An object of class \code{sienaEffects} or
    \code{summary.sienaEffects} as appropriate.
  }
  \item{fileName}{Character string denoting file name if file output desired.}
  \item{includeOnly}{Boolean. If \code{TRUE}, only effects with the include
    flag \code{TRUE} will be printed.}
  \item{expandDummies}{Interpret the \code{timeDummy} column and show
    any effects which would be added by \code{sienaTimeFix}.}
  \item{includeRandoms}{Boolean. If \code{TRUE}, also the
    \code{randomEffects} column will be printed.}
  \item{includeShortNames}{Boolean. If \code{TRUE}, also the
    \code{shortName} column will be printed.}
  \item{dropRates}{Boolean. If \code{TRUE}, do not print the rows for basic rate
    effects.}
  \item{\dots}{For extra arguments (none used at present).}
}
\value{
  The function \code{print.sienaEffects} prints details of the main
  columns of the selected rows of the effects object.\cr
  If the effects object includes statistics for the
  Generalized Method of Moments (GMoM), as included by function
 \code{\link{includeGMoMStatistics}} and for which \code{type=gmm},
  the print consists of two parts: the first consists of the
  included effects for the probability model, the second
  of the statistics used for GMoM estimation.

 The function \code{summary.sienaEffects} checks the rows for valid
 printing via \cr
 \code{print.data.frame} and excludes any that will
 fail. The OK columns are printed first, followed by any others.

 Output from either can be directed to a file by using the argument
 \code{filename}.
}
\references{
  See \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Ruth Ripley, modifications by Tom Snijders and Viviana Amati.
}
\seealso{\code{\link{sienaEffects}},
    \code{\link{make_specification}}, \code{\link{set_effect}},
    \code{\link{includeGMoMStatistics}}, \cr
    \code{\link{test_time}},
    \code{\link{effectsDocumentation}}
}
\examples{
mynet <- as_dependent_rsiena(
           array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <- as_dependent_rsiena(s50a, type="behavior")
mycovar <- as_covariate_rsiena(rnorm(50))
mydyadcovar <- as_covariate_rsiena(
              matrix(as.numeric(rnorm(2500) > 2), nrow=50),
              type="oneMode")
(mydata <- make_data_rsiena(mynet, mybeh, mycovar, mydyadcovar))
myeff <- make_specification(mydata)
print(myeff)
print(myeff, includeShortNames=TRUE)
summary(myeff)
}
\keyword{print}

\name{includeEffects}
\alias{includeEffects}
\title{
  Function to include effects in a Siena model
}
\description{
  This function can be used for model specification  
  by modifying a Siena effects object.\cr
  This function now is superseded by \code{\link{set_effect}}.
}
\usage{
includeEffects(myeff, ..., include = TRUE, name = myeff$name[1], 
 type = "eval", interaction1 = "", interaction2 = "", 
 fix=FALSE, test=FALSE, character=FALSE, verbose = TRUE)
}
\arguments{
  \item{myeff}{a Siena effects object as created by \code{\link{make_specification}}
}
  \item{\dots}{
short names to identify the effects which should be included or excluded.}
  \item{include}{Boolean. default TRUE, but can be switched to FALSE to
  turn off an effect.}
  \item{name}{Name of dependent variable (network or behavior) for which effects
  are being included. Defaults to the first in the effects object,
  which is the first dependent variable specified in \code{\link{sienaDataCreate}}.}
  \item{type}{
  Type of effects to be included: "eval", "endow", "creation", or "rate".}
  \item{interaction1}{
  Name of siena object where needed to completely identify the
  effects, e.g., covariate name or behavior variable name.}
  \item{interaction2}{
  Name of siena object where needed to completely identify the
  effects, e.g., covariate name or behavior variable name.}
  \item{fix}{
  Boolean. Are the effects to be fixed at the value stored in
  \code{myeff$initialValue} or not.}
  \item{test}{
  Boolean. Are the effects to be tested or not (requires \code{fix}).}
\item{character}{Boolean: are the effect names character strings or not.}
  \item{verbose}{Boolean: should the print of altered effects be produced.}
}
\details{Recall from the help page for \code{\link{make_specification}}
   that a Siena effects object (class \code{sienaEffects} or
   \code{sienaGroupEffects}) is a \code{\link{data.frame}};
   the rows in the data frame are the effects for this data set;
   some of the columns/variables of the data frame
   are used to identify the effect, other columns/variables define how this
   effect is used in the estimation. \cr   
   
  The function \code{includeEffects} operates as an interface
  setting the "include" column on selected rows of the effects object,
  to the value requested (\code{TRUE} or \code{FALSE}).
   The selected effects must be indicated by the arguments \ldots, \code{type},
  and (if necessary) \code{interaction1} and \code{interaction2}.
  The names \code{interaction1} and \code{interaction2} do not refer
  to interactions between effects, but to dependence of effects on
  other variables in the data set.
  The arguments should identify the effects completely.\cr
  The short names must not be set between quotes,
  unless you use \code{character=TRUE}.

  Note that the internal effect parameter has a default value which differs
  between effects. This can be set by function \code{\link{setEffect}}.
  Also the value of \code{myeff$initialValue} can be set by this function.
  The function \code{\link{setEffect}} operates on the effects object
  in a more detailed way, but applies to one effect at the time.

  Further information about Siena effects objects is given
  in the help page for \code{\link{make_specification}}.

  A list of all effects available in a given effects object (e.g.,
  \code{myeff}), including their names of dependent variables,
  effect names, short names,
  and values of \code{interaction1} and \code{interaction2} (if any),
  is obtained by executing \code{\link{effectsDocumentation}(myeff)}.

  The input names \code{interaction1} and \code{interaction2} do not themselves
  refer to created interactions, but to dependence of the base effects on
  other variables in the data set.
  They are used to completely identify the effects.
}
\value{
    An updated version of the input effects object, with the
    \code{include}, \code{test}, and \code{fix}
    columns for one or more rows updated.
    Details of the rows altered will be printed.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley }
\seealso{\code{\link{set_effect}},
  \code{\link{make_specification}}, 
  \code{\link{set_interaction}},
  \code{\link{includeGMoMStatistics}},
  \code{\link{update_specification}},
  \code{\link{print.sienaEffects}}, \code{\link{effectsDocumentation}}
}

\examples{
   
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh  <- as_dependent_rsiena(s50a, type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, avAlt, depvar="mybeh", covar1="mynet")
myeff <- set_effect(myeff, list(transTrip, outAct))
myeff
}
\keyword{classes}

\name{tmp3}
\alias{tmp3}
\docType{data}
\title{van de Bunt's Freshman dataset, time point 3}
\description{
Third timepoint of van de Bunt's freshman dataset.\cr

Codes: 1 = best friendship; 2 = friendship; 3 = friendly relationship;   
4 = neutral relationship; 5 = troubled relationship; 0 = unknown person.
}
\format{
 Adjacency matrix for the "at least friendly relationship"
 network at time point 3.
}
\source{
  vrnd32t3.dat from
  \url{https://www.stats.ox.ac.uk/~snijders/siena/vdBunt_data.zip}
}
\references{
  Van de Bunt, G.G., van Duijn, M.A.J., and Snijders, T.A.B.  (1999),
  Friendship networks through time: An actor-oriented statistical network model.
  \emph{Computational and Mathematical Organization Theory}, \bold{5}, 167--192.
  
  Also see \url{https://www.stats.ox.ac.uk/~snijders/siena/vdBunt_data.htm}.
}
\seealso{\code{\link{tmp4}}}
\keyword{datasets}

\name{interpret_size_dynamics}
\alias{interpret_size_dynamics}
\alias{sienaRIDynamics}
\alias{plot.sienaRIDynamics}
\title{Functions to assess the dynamics of relative importance of effects
       between observation moments}
\description{
   The function \code{interpret_size_dynamics} returns the relative importance of
   effects in a SAOM according to the measure of relative importance described
   in Indlekofer and Brandes (2013).
   More precisely, it returns aggregates of relative importances over
   simulated ministeps between observation moments as described in Section
   3.2 of this paper but only for the starting observation at
   each period.
   The measure is based on the influence of effects on simulated
   micro-steps and takes the complete model specification into account.
   Therefore, required arguments are the analysed data given as a
   \code{siena} data object (necessary for the micro-steps simulations)
   as well as the complete model specification represented either by an
   estimated \code{sienaFit} object or by the triple consisting of a
   suitable parameter vector \code{theta} and the corresponding
   \code{sienaAlgorithm} and \code{sienaEffects} objects.
 }
\usage{
  interpret_size_dynamics(data, ans=NULL, theta=NULL,
     algorithm=NULL, effects=NULL, depvar=NULL,
     intervalsPerPeriod=NULL, n3 = NULL, useChangeContributions=FALSE,
     silent=TRUE, seed=NULL)
  \method{plot}{sienaRIDynamics}(x, staticRI=NULL, col=NULL,
     ylim=NULL, width=NULL, height=NULL, legend=TRUE,
     legendColumns=NULL, legendHeight=NULL, cex.legend=NULL, \dots)
}
\arguments{
  \item{data}{\code{siena} data object representing the analyzed data and
  resulting from a call to \code{\link{sienaDataCreate}}.}
  \item{ans}{\code{sienaFit} object resulting from a call to
    \code{\link{siena07}}.
    The \code{sienaFit} object contains all necessary information on the model
    specification, in particular, the vector of parameter values
    \code{ans$theta}, the used algorithm for estimation \code{ans$x},
    and information on included model effects \code{ans$effects}.
    If \code{ans} is a valid \code{sienaFit} object, the calculations of
    relative importances are based on \code{ans$theta}, \code{ans$x},
    and \code{ans$effects}.
    Alternatively, the necessary information can be given directly
    as a suitable parameter vector \code{theta}, a \code{sienaAlgorithm} object,
    and a \code{sienaEffects} object.
    In this case, \code{ans} has to be unspecified (i.e., \code{ans=NULL}).}
  \item{theta}{Vector of parameter values of effects included in the model.
    Length of \code{theta} has to be equal to the number of included effects
    (rate effects as well as evaluation effects).}
  \item{algorithm}{\code{sienaAlgorithm} object as resulting from a call to
    \code{\link{sienaAlgorithmCreate}}.
    Works only for estimation by Method of Moments (i.e., if
    \code{maxlike = FALSE}).}
  \item{effects}{\code{sienaEffects} object specifying which effects are
    included the model.}
  \item{depvar}{If the model contains more than one dependent variable,
    it has to be specified for which dependent variable the relative importances
    of associated effects should be determined.
    Only those simulated ministeps that refer to the selected dependent
    variable are considered in the aggregated values of relative importance
    of the corresponding effects.
    If \code{depvar=NULL}, relative importances will be determined for
    the first element in the list of dependent variables of \code{data}
    (\code{attr(data$depvars,"names")[1]}).}
  \item{intervalsPerPeriod}{For analyzing the dynamics of relative
    importances between observation moments, the time interval between
    observation moments is divided into \cr
    \code{intervalsPerPeriod} subintervals.
    The values of relative importance are aggregated over all ministeps
    of one subinterval. Default value is 10 subintervals per period.}
  \item{n3}{Integer specifying the number of chains to be simulated.
    Defaults to the value in \code{algorithm} or what was set when estimating \code{ans}.
    A very large value will result in a long computation time and large memory usage.}
  \item{silent}{Logical specifying whether output during internal
    \code{\link{siena}} calls should be suppressed. Default is \code{TRUE}.}
  \item{seed}{Integer specifying the seed for random number generation
    during internal \code{\link{siena}} calls.}
  \item{useChangeContributions}{Logical specifying whether previously extracted change
    contributions should be used instead of rerunning phase 3 simulations.}
  \item{x}{\code{interpret_size_dynamics} object resulting from a call to \cr
    \code{interpret_size_dynamics}.}
  \item{staticRI}{\code{\link{sienaRI}} object corresponding to the same model
    and data as \code{x}.
    If \code{staticRI} is specified, the expected relative importances of
    effects at observation moment are indicated in the plot by circlets
    at the beginning of each period.}
  \item{col}{Colors used in the plot. If \code{col=NULL} a default color
    scheme is used.}
  \item{ylim}{A vector of two numbers specifying the maximum and minimum
    value of the y-range visible in the plot. If \code{ylim=NULL} default
    values are used.}
  \item{width}{Width of the plot. If \code{width=NULL} a default value is used.}
  \item{height}{Height of the plot. If \code{height=NULL} a
    default value is used.}
  \item{legend}{Boolean: if \code{TRUE} a legend is added to the plot.
    \code{x$effectNames} are used as labels.}
  \item{legendColumns}{Number of columns in legend. If
    \code{legendColumns=NULL} a default value is used.
    Only effective if \code{legend=TRUE}.}
  \item{legendHeight}{Height of legend. If \code{legendHeight=NULL}
    a default value is used. Only effective if \code{legend=TRUE}.}
  \item{cex.legend}{Specifies the relative font size of legend labels.}
  \item{\dots}{Other arguments.}
}
\details{
\code{interpret_size_dynamics} takes the data as well as the complete model
specification into account.
Therefore, necessary arguments are the analyzed data given as a
\code{siena} data object as well as the complete model specification
represented either by an estimated \code{sienaFit} object or by the
triple consisting of a suitable parameter vector \code{theta}
and the corresponding \code{sienaAlgorithm} and \code{sienaEffects} objects.

Note that \code{interpret_size_dynamics} works only with Method of Moments \cr
(i.e., for \code{sienaAlgorithm} objects with \code{maxlike = FALSE}).

If \code{ans} is a valid \code{sienaFit} object that resulted from a
call to \code{\link{siena07}} with unconditional estimation (i.e.,
for \code{sienaAlgorithm} objects with \code{cond = FALSE}),
the calculations of relative importances are based on \code{ans$theta},
\code{ans$x}, and \code{ans$effects}.
(Note that if the estimation of \code{ans} has been conditional on a
dependent variable, it is necessary to give the associated
\code{sienaEffects} object explicitly to \code{interpret_size_dynamics}).

Alternatively, the necessary information can be given directly as a
suitable parameter vector \code{theta} (containing necessary rate
and evaluation parameters), a \code{sienaAlgorithm} object, \cr
and a \code{sienaEffects} object.
In this case, \code{ans} has to be unspecified, i.e., \code{ans=NULL}.

If the sienaFit object \code{ans} was already estimated with
\code{returnChangeContributions=TRUE}, the change contributions
are stored in \code{ans$changeContributions} and can be used to
estimate the relative importances of effects in \code{interpret_size_dynamics}
without running any additional simulations.
In this case, \code{useChangeContributions} has to be set to \code{TRUE}
and \code{n3} is ignored, since the number of chains is determined by the
previous \code{siena07} run.
If \code{useChangeContributions=FALSE}, new chains are simulated internally
with \code{siena07} and the change contributions are calculated from these
chains. The number of simulations is determined by the value of \code{n3} or
the value of \code{algorithm$n3} if \code{algorithm} is specified. Otherwise,
it defaults to the value of \code{ans$n3} if \code{ans} is specified.
}
\value{
\code{interpret_size_dynamics} returns an object of class \code{sienaRIDynamics}.

 A returned \code{interpret_size_dynamics} object stores the relative importances
   of effects in simulated mini-steps aggregated to \code{intervalsPerPeriod}
   averages per period. For details, see Section 3.2 of
   Indlekofer and Brandes (2013).

A \code{interpret_size_dynamics} object consists of following components:
    \describe{
    \item{\code{intervalValues}}{the relative importances of included
          effects for simulated ministeps of the considered dependent
          variable aggregated over subintervals.}
    \item{\code{dependentVariable}}{the name of the dependent variable
          the results refer to.}
    \item{\code{effectNames}}{the names of considered effects.}
    }
}
\seealso{\code{\link{sienaRI}} and \code{\link{siena07}}}
\references{
  Indlekofer, N. and Brandes, U. (2013),
  Relative Importance of Effects in Stochastic Actor-oriented Models.
  \emph{Network Science}, \bold{1 (3)}, 275--297.
}
\author{Natalie Indlekofer}
\examples{

myalgo <- set_algorithm_saom(nsub=1, n3=32, seed=1777, cond=FALSE)
myout <- set_output_saom(returnChangeContributions=TRUE)
mynet1 <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
mydata <- make_data_rsiena(mynet1)
mymodel <- make_specification(mydata)
mymodel <- set_effect(mymodel, list(inPop, outAct, transTrip))
(ans <- siena(mydata, effects=mymodel,
            control_algo=myalgo, control_out=myout, batch=TRUE))

RIDynamics1 <- interpret_size_dynamics(mydata, ans=ans,
                 useChangeContributions=TRUE)
RIDynamics1
\dontrun{
plot(RIDynamics1)


myalgo2 <- set_algorithm_saom(nsub=1, n3=32, seed=1777, cond=TRUE)
(ans2 <- siena(mydata, effects=mymodel, control_algo=myalgo2,
            control_out=myout, batch=TRUE))

RIDynamics2 <- interpret_size_dynamics(mydata, ans=ans2, effects=mymodel)
RIDynamics2


myalgo3 <- sienaAlgorithmCreate(nsub=1, n3=32, seed=1777, cond=TRUE)
RIDynamics3 <- interpret_size_dynamics(data=mydata,
             theta=c(ans2$rate,ans2$theta),
             algorithm=myalgo3, effects=mymodel, intervalsPerPeriod=4)
RIDynamics3

myalgo4 <- set_algorithm_saom(nsub=1, n3=32)
mynet2 <-  as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <-  as_dependent_rsiena(s50a, type="behavior")
mydata2 <- make_data_rsiena(mynet2, mybeh)
mymodel2 <- make_specification(mydata2)
mymodel2 <- set_effect(mymodel2, list(inPop, outAct, transTrip))

(ans3 <- siena(mydata2, effects=mymodel2, control_algo=myalgo4,
            control_out=myout,  batch=TRUE))

RIDynamics4 <- interpret_size_dynamics(mydata2, ans=ans3, depvar="mybeh")
RIDynamics4

RI5 <- interpret_size(ans3, data=mydata2)

RIDynamics5 <- interpret_size_dynamics(mydata2, ans=ans3,
                                       depvar="mynet2", n3 = 50)
RIDynamics5
plot(RIDynamics5, staticRI=RI5[[1]])
}
}
\keyword{models}

\name{varCovar}
\alias{varCovar}
\title{Function to create a changing covariate object.}
\description{
  This function creates a changing covariate object from a matrix.\cr
  It now is superseded by function \code{\link{as_covariate_rsiena}}.
}
\usage{
varCovar(val, centered=TRUE, nodeSet="Actors",  warn=TRUE,
  imputationValues=NULL)
}
\arguments{
  \item{val}{ Matrix of covariate values, one row for each actor, one
    column for each period. }
  \item{centered}{Boolean: if TRUE, then the overall mean value is subtracted.}
  \item{nodeSet}{Character string containing the name of the associated
    node set. If the entire data set contains more than one node set,
    then the node sets must be specified in all data objects.}
  \item{warn}{Logical: is a warning given if all values are \code{NA},
  or all non-missing values are the same.}
  \item{imputationValues}{Matrix of covariate values of same
   dimensions as \code{val}, to be used for imputation of
   \code{NA} values (if any) in \code{val}. Must not contain
   any \code{NA}.}
}
\details{
  When part of a Siena data object, the covariate is assumed to be
  associated with node set \code{nodeSet} of the Siena data object.
  In practice, the node set needs to be specified only in the case of
  the use of the covariate with a two-mode network.\cr
  If there are any \code{NA} values in \code{val}, and
  \code{imputationValues} is given, then the corresponding elements
  of \code{imputationValues} are used for imputation.
  If \code{imputationValues} is \code{NULL}, imputation
  is by the overall mean value. In both cases, cases with imputed values
  are not used for calculating target statistics (see the manual).\cr
  The value of the changing covariate for wave \code{m} is supposed
  in the simulations to be valid in the whole period from wave \code{m}
  to wave \code{m+1}.
  If the data set has \code{M} waves, this means that the values,
  if any, for wave \code{M} will not be used. Therefore, the number of columns
  can be \code{M} or \code{M-1}; if the former, the values in the last
  column will not be used.
}
\value{
  Returns the covariate as an object of class "varCovar", in which
  form it can be used as an argument to \code{\link{make_data_rsiena}}.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley}
\seealso{\code{\link{make_data_rsiena}}, 
\code{\link{as_covariate_rsiena}},
\code{\link{as_nodeset_rsiena}}}
\examples{
myvarCovar <- as_covariate_rsiena(s50a)
senders <- as_nodeset_rsiena(50, nodeSetName="senders")
receivers <- as_nodeset_rsiena(30, nodeSetName="receivers")
senders.covariate <- as_covariate_rsiena(s50a, nodeSet="senders")
receivers.covariate <- as_covariate_rsiena(s50s[1:30,], nodeSet="receivers")
}
\keyword{classes}

\name{write_report}
\alias{write_report}
\alias{print01Report}
\alias{write_report.sienadata}
\title{
Function to produce a basic descriptive report of SIENA data sets}
\description{
Prints a report of a Siena data object and its default effects.\cr
  The functions \code{write_report} and \code{print01Report}
  are identical. \cr
  The first name is preferred, 
  the other is kept for backward compatibility.
}
\usage{
\method{write_report}{sienadata}(x, outputName=NULL, ...)

print01Report(data, modelname="Siena", getDocumentation=FALSE)
}
\arguments{
  \item{x}{a Siena data object of class \code{\link{sienadata}}
}
  \item{data}{a Siena data object of class \code{\link{sienadata}}
}
\item{outputName}{If \code{outputName = NULL} and the Siena data set
   has name \code{dataName}, output will be written by
   \code{write_report} to a file with name "\code{dataName}_report.txt"
   in the working directory.\cr
    If \code{outputName} is not \code{NULL}, it should be a character string
    without embedded spaces, and output will be written
    to a file with name "\code{outputName}_report.txt".}
\item{modelname}{
  Character string used to name the output file "\code{modelname}.txt"
}
\item{getDocumentation}{Flag to allow documentation of internal
  functions, not for use by users.}
\item{...}{
   Additional arguments, not used now.
}
}
\details{
  First deletes any file of the given name, then writes a new one.
  }
\value{
  No value returned.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley, Tom Snijders }

\examples{
mynet1 <- as_dependent_rsiena(
             array(c(s501, s502, s503), dim=c(50, 50, 3)))
mydata <- make_data_rsiena(mynet1)
write_report(mydata, outputName=tempfile())
}
\keyword{print}

\name{set_interaction}
\alias{set_interaction}
\alias{includeInteraction}
\alias{set_interaction.sienaEffects}
\title{
Function to create user-specified interactions for a Siena model.
}
\description{This function allows the user to include or exclude
  an interaction effect in a Siena effects object.
}
\usage{
\method{set_interaction}{sienaEffects}(x, shortNames, type="eval",
            depvar=x$name[1], covar1=rep("", 3), covar2=rep("", 3),
            initialValue = 0, random=FALSE,
            include=TRUE, fix=FALSE, test=FALSE, verbose=TRUE, ...)

includeInteraction(x, ..., include = TRUE, name = x$name[1],
    type = "eval", interaction1 = rep("", 3), interaction2 = rep("", 3),
    fix=FALSE, test=FALSE, random=FALSE,
    initialValue=0,
    character = FALSE, verbose = TRUE)
}
\arguments{
  \item{x}{A Siena effects object as created by \code{\link{make_specification}},
    which is either an object of class \code{sienaEffects}
    or \code{sienaGroupEffects}.
  }
  \item{shortNames}{List of 2 or 3 shortNames to identify the effects
    which should be interacted.
  }
  \item{type}{
    Type of effects to be interacted.
  }
  \item{depvar}{Name of dependent variable (network or behavior)
    for which interactions are being defined.
    Defaults to the first in the effects object.
  }
  \item{covar1}{
    Vector of Siena objects where needed to completely identify the
    effect e.g. covariate name or behavior variable name.
    See the examples below. Trailing blanks may be omitted.
  }
  \item{covar2}{
    Vector of Siena objects where needed to completely identify the
    effect e.g. covariate name or behavior variable name.
    Trailing blanks may be omitted.
  }
  \item{initialValue}{Initial value for estimation. Default 0.}
  \item{random}{For specifying that the interaction effect will vary randomly;
    not relevant for \code{RSiena} at this moment.
    Boolean required. Default FALSE.}
  \item{include}{
    Boolean. default TRUE, but can be switched to FALSE to
    turn off an interaction.
  }
  \item{fix}{
    Boolean. Are the effects to be fixed at the value stored in
    \code{x$initialValue} or not.}
  \item{test}{
    Boolean. Are the effects to be tested or not (requires \code{fix}).}
  \item{character}{Boolean: are the effect names character strings or not.}
  \item{verbose}{Boolean: should the print of altered effects be produced.}
  \item{\dots}{for \code{includeInteraction}: 2 or 3 shortNames to identify the effects
    which should be interacted. Not used for \code{set_interaction}.
  }
  \item{name}{Name of dependent variable (network or behavior)
    for which interactions are being defined.
    Defaults to the first in the effects object.
  }
  \item{interaction1}{
    Vector of Siena objects where needed to completely identify the
    effect e.g. covariate name or behavior variable name.
    See the examples below.
    Trailing blanks may be omitted.
  }
  \item{interaction2}{
    Vector of Siena objects where needed to completely identify the
    effect e.g. covariate name or behavior variable name.
    Trailing blanks may be omitted.
  }
}
\details{The details provided should uniquely identify two or three
  effects. If so, an interaction effect will be created and included or
  not in the model. \cr
  Whether interactions between two or three given effects
  can be created depends on their
  \code{interactionType} (which can be, for
  dependent network variables, \code{empty}, \code{ego}, or \code{dyadic};
  and for dependent behavioral variables, \code{empty} \code{or OK}).
  Consult the section on Interaction
  Effects in the manual for this. The \code{interactionType} is shown
  in the list of effects obtained from the function \cr
  \code{\link{effectsDocumentation}}.\cr
  The short names must not be set between quotes,
  unless you use \code{character=TRUE}.\cr
  From the point of view of model building it is usually advisable, when
  including an interaction effect in a model, also to include the
  corresponding main effects. This is however not enforced
  by \code{set_interaction}.\cr

  As from version 1.3.24, effects object have a "version" attribute.
  Effects objects including interaction effects are not necessarily compatible
  between versions of \code{RSiena}. Therefore it is recommended to create
  such effects objects again when changing to a new version of \code{RSiena}.
  If an effects object including any interaction effects is used from an
  old version of \code{RSiena}, this will lead to a warning when running
  \code{\link{siena07}}. \cr

  An interaction effect does not have its own internal effect parameter.
  The internal effect parameters of the interacting main effects are used,
  whether or not these are included in the model. This implies that if an
  interaction effect is included but not the corresponding main effects,
  or not all of them, then nevertheless the internal effect parameters as
  specified in the effects object are used for the interaction.
  These can be set using function \code{setEffect} with the desired value
  of \code{parameter} and (in this case) \code{include=FALSE} or
  \code{fix=TRUE, initialValue=0}. \cr
  If an internal effect parameter is changed for one of the main effects
  after the last call of \cr
  \code{set_interaction} for a given interaction
  effect, this will not be visible in the name of the interaction effect
  when the effects object is printed.
  However, the correct value of the internal effect parameter
  will be used by \code{\link{siena07}}.\cr
  The values of the internal effect parameters can be checked for a
  \code{\link{sienaFit}} object \code{ans} produced by \code{\link{siena07}}
  by looking at \code{ans$effects}, which is the requested effects object
  to which the main effects of the user-defined interactions were added,
  if they were not included.\cr

  Interaction effects are constructed from effects with shortName
  \code{unspInt} (for networks), \cr
  \code{behUnspInt} (for discrete behavior),
  and \code{contUnspInt} (for continuous behavior) by specifying their
  elements \code{effect1} and \code{effect2}, and possibly \code{effect3}.
  The shortName is not altered by this function.

  The number of possible user-specified interaction effects is limited
  by the parameters \code{nintn} (for dependent network variables)
  and \code{behNintn} (for dependent behavior variables) in the call of
  \code{\link{make_specification}}, which determine the numbers of effects with
  shortNames \code{unspInt}, \code{behUnspInt}, and \code{contUnspInt}
  (for the latter two, the maximum is \code{behNintn} each).

  The input names \code{interaction1} and \code{interaction2},
  which are vectors as indicated above, do not themselves
  refer to created interactions, but to dependence of the base effects on
  other variables in the data set.
  They are used to completely identify the effects.\cr
  The first element of these vectors applies to the first shortName,
  the second to the second, and if \dots contains 3 shortNames,
  the third to the third. See the list of examples below for the interaction
  between \code{recip} and \code{simX} for an example.

  Further information about Siena effects objects is given
  in the help page for \cr
  \code{\link{make_specification}}.

  A list of all effects in a given effects object (e.g., \code{myeff}),
  including their names of dependent variables, effect names, short names,
  and values of interaction1 and interaction2 (if any),
  is obtained by executing \code{\link{effectsDocumentation}(myeff)}.
}
\value{
  An updated version of the input effects object; if \code{include},
  containing the interaction effect between "effect1" and "effect2" and
  possibly "effect3"; if not, without this interaction effect.
  The shortName of the interaction effect is "unspInt" for network effects,
  "behUnspInt" for discrete behavior effects, and "contUnspInt" for
  continuous behavior effects. \cr
  If \code{verbose=TRUE}, the interacting effects and the interaction effect
  will be printed, with their row numbers in the effects object.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley, Tom Snijders}
\seealso{
  \code{\link{make_specification}}, \code{\link{set_effect}},
  \code{\link{includeEffects}}, \code{\link{effectsDocumentation}
}}
\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
alc <- as_covariate_rsiena(s50a)
sm <- as_covariate_rsiena(s50s)
mydata <- make_data_rsiena(mynet, alc, sm)
myeff <- make_specification(mydata)
myeff
myeff <- set_effect(myeff, inPop)
myeff <- set_interaction(myeff, list(recip, inPop))
myeff <- set_interaction(myeff, list("recip", "inPop"))
myeff
myeff <- set_interaction(myeff, list(egoX, altX, simX), covar1=c("alc", "alc", "alc"))
myeff <- set_interaction(myeff, list(egoX, altX), covar1=c("alc", "sm"))
myeff <- set_interaction(myeff, list(recip, simX), covar1=c("", "alc"))
myeff
myeff <- set_effect(myeff, gwespFF, parameter=20)
myeff <- set_interaction(myeff, list(recip, gwespFF))
myeff
(myeff <- set_effect(myeff, gwespFF, parameter=69, include=FALSE))
myeff <- set_interaction(myeff, list(recip, gwespFF))
myeff
}
\keyword{classes}

\name{s50s}
\alias{s50s}
\docType{data}
\title{Smoking data: excerpt from "Teenage Friends and Lifestyle Study" data}
\description{
Smoking data from    
an excerpt of the network, alcohol consumption, and smoking data     
for 50 randomly chosen girls from the
Teenage Friends and Lifestyle Study data set. Useful as a small    
example of network and behaviour, for which models can be    
fitted quickly, and for which there are no missing values.
}
\format{
 A matrix of variables relating to the smoking habits for the actors in
 the network. Three columns, one for each time point. Coding is 1--3:
 1 = no smoking, 2 = moderate smoking, 3 = serious smoking.
}
\source{
\url{https://www.stats.ox.ac.uk/~snijders/siena/s50_data.zip}
}
\references{
West, P. and Sweeting, H. (1995), \emph{Background Rationale and Design of the
West of Scotland 11-16 Study}.
Working Paper No. 52. MRC Medical Sociology Unit Glasgow.

See \url{https://www.stats.ox.ac.uk/~snijders/siena/s50_data.htm}
}
\seealso{\code{\link{s501}}, \code{\link{s502}}, \code{\link{s503}}, \code{\link{s50a}}
}
\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
myvar  <- as_covariate_rsiena(s50s)
mydata <- make_data_rsiena(mynet, myvar)
mydata
}
\keyword{datasets}

\name{set_algorithm}
\alias{set_algorithm_saom}
\alias{set_model_saom}
\alias{set_output_saom}
\alias{set_algorithm}
\alias{set_model}
\alias{set_output}
\alias{sienaModelCreate}
\alias{sienaAlgorithmCreate}
\alias{sienaModel}
\alias{sienaAlgorithm}
\alias{sienaModel}
\alias{sienaAlgorithmSettings}
\alias{sienaOutputOptions}
\alias{model.create}
\alias{modelType}
\alias{behModelType}

\title{Functions to create objects containing various aspects of
  algorithm specifications for parameter estimation in RSiena}
\description{
  Create objects containing various aspects of
  algorithm specifications for parameter estimation in RSiena.  \cr
  \code{set_model_saom} sets the specifications of the Stochastic
  Actor-oriented Model. \cr
  \code{set_algorithm_saom} sets the specifications of the algorithm
  for parameter estimation of the model. \cr
  \code{set_output_saom} sets the specifications of the generated output.

  These three aspects are combined in the older functions
  \code{sienaAlgorithmCreate} and \cr
  \code{sienaModelCreate},
  which are identical functions although having different names.
  The older functions are not preferred any more,
  but are kept for backward compatibility.
}
\usage{
  set_model_saom(modelType = NULL, behModelType = NULL,
                        MaxDegree = NULL, Offset = NULL)

  set_algorithm_saom(maxlike = FALSE, gmm = FALSE,
    cond = NA, condvarno = 0, condname = "",
    simOnly = FALSE,
    targets = NULL, thetaValues = NULL, seed = NULL,
    n3 = 1000, nsub = 4, n2start = NULL, firstg = 0.2, reduceg = 0.5,
    truncation = 5, doubleAveraging = 0,
    diagonalize = 0.2*!maxlike, standardizeVar = (diagonalize<1),
    dolby = TRUE, splitDepvars = 0, 
	useStdInits = FALSE, findiff = FALSE,
    mult = 5, prML = 1, maximumPermutationLength = 40,
    minimumPermutationLength = 2, initialPermutationLength = 20,
    localML = FALSE)

set_output_saom(outputName = NULL, lessMem = FALSE,
    returnThetas = FALSE, 
    returnChains = FALSE, returnDataFrame = FALSE,
    returnChangeContributions = FALSE)

sienaAlgorithmCreate(fn, projname = "Siena", MaxDegree = NULL,
     Offset = NULL, useStdInits = FALSE, 
     n3 = 1000, nsub = 4, n2start = NULL,
     dolby = TRUE, splitDepvars = 0, 
	 maxlike = FALSE, gmm = FALSE, 
     diagonalize = 0.2*!maxlike,
     condvarno = 0, condname = "", firstg = 0.2, reduceg = 0.5,
     cond = NA, findiff = FALSE, seed = NULL,
     prML = 1, maximumPermutationLength = 40,
     minimumPermutationLength = 2, initialPermutationLength = 20,
     modelType = NULL, behModelType = NULL, mult = 5, simOnly = FALSE,
     localML = FALSE, truncation = 5, doubleAveraging = 0,
     standardizeVar = (diagonalize<1), lessMem = FALSE, silent = FALSE)

sienaModelCreate(fn, projname = "Siena", MaxDegree = NULL,
     Offset = NULL, useStdInits = FALSE, 
     n3 = 1000, nsub = 4, n2start = NULL,
     dolby = TRUE,  splitDepvars = 0, 
	 maxlike = FALSE, gmm = FALSE, 
     diagonalize = 0.2*!maxlike,
     condvarno = 0, condname = "", firstg = 0.2, reduceg = 0.5,
     cond = NA, findiff = FALSE, seed = NULL,
     prML = 1, maximumPermutationLength = 40,
     minimumPermutationLength = 2, initialPermutationLength = 20,
     modelType = NULL, behModelType = NULL, mult = 5, simOnly = FALSE,
     localML = FALSE, truncation = 5, doubleAveraging = 0,
     standardizeVar = (diagonalize<1), lessMem = FALSE, silent = FALSE)
}
\arguments{
  For \code{set_model_saom}:
  \item{modelType}{Named vector indicating the type of model to be fitted for
    dependent network variables.
    (See the examples below for how to specify a named vector.)\cr
    Possible values are:\cr
    1 = directed standard,
    \cr
    2:6 for symmetric networks only: 2 = dictatorial forcing (D.1),
    3 = Initiative model with reciprocal confirmation (M.1),
    4 = Pairwise dictatorial forcing model (D.2),
    5 = Pairwise mutual model (M.2), 6 = Pairwise joint model (C.2),\cr
    7:10 for directed one-mode only:
    7 = Double Step model with double step probability 0.25,
    8 = Double Step model with double step probability 0.50,
    9 = Double Step model with double step probability 0.75,
    10 = Double Step model with double step probability 1.00,\cr
    11 = Contemporaneous evaluation statistics model.
    \cr
    Names should be the names of all dependent network variables,
    in the same order as in the Siena data set.\cr
    See Snijders and Pickup (2016) for the meanings of the various models
    for symmetric networks.\cr
    Default \code{NULL} implies 1 for directed or two-mode, 2 for symmetric.}
  \item{behModelType}{Named vector indicating the type of model to be fitted for
    behavioral dependent variables.
    (See the examples below for how to specify a named vector.)\cr
    Possible values are:\cr
    1 = standard (restricted), 2 = absorbing.\cr
    Names should be the names of all dependent behavioral variables,
    in the same order as in the Siena data set.\cr
    Default \code{NULL} implies values 1.}
  \item{MaxDegree}{Named vector of maximum degree values for
    corresponding networks. Allows to restrict the model to networks
    with degrees not higher than this maximum.
    Names should be the names of all dependent network variables,
    in the same order as in the Siena data set.\cr
    Default as well as value 0 imply no restrictions.\cr
    This option is not available for maximum likelihood estimation.}
  \item{Offset}{Named vector of offset values for symmetric networks with
    \code{modelType = 3} (M.1), and for universal setting in Settings model.
    Names should be the names of all dependent network variables,
    in the same order as in the Siena data set.
    Default \code{NULL} implies values 0.}
  For \code{set_algorithm_saom}:
  \item{maxlike}{Whether to use maximum likelihood method or
    Method of Moments estimation.}
  \item{gmm}{Whether to use the Generalized Method of Moments or the regular
    Method of Moments estimation.}
  \item{cond}{Boolean. Only relevant for Method of Moments
    simulation/estimation.
    If TRUE, use conditional simulation; if FALSE, unconditional simulation.
    If missing, decision is deferred until \code{\link{siena}},
    when it is set to TRUE if there is only one dependent variable,
    FALSE otherwise.}
  \item{condvarno}{If \code{cond} (conditional simulation), the
    sequential number of the network
    or behavior variable on which to condition.}
  \item{condname}{If conditional, the name of the dependent variable on
    which to condition. Use one or other of \code{condname} or
    \code{condvarno} to specify the variable.}
  \item{simOnly}{Logical: If TRUE, then the calculation of the covariance
    matrix and standard errors of the estimates at the end of
    Phase 3 of the estimation algorithm is skipped.
    This is suitable if nsub = 0 and \code{\link{siena}}
    is used only for the purpose of simulation.}
  \item{targets}{Numeric vector of length equal to the number of estimated
    parameters, meant to supersede the targets calculated from the data set;
    see "Details". Not for regular use.}
  \item{thetaValues}{If not \code{NULL}, this should be a matrix with parameter
    values to be used in Phase 3. The number of columns must be equal to the
    number of estimated parameters in the effects object (if conditional
    estimation is used, without the rate parameters for the conditioning
    dependent variable). Can only be used if \code{simOnly=TRUE}.}
  \item{seed}{Integer. Starting value of random seed.}
  \item{n3}{Number of iterations in phase 3. For regular use with the
    Method of Moments, n3 = 1000 mostly suffices.
    For use in publications and for Maximum Likelihood,
    at least n3 = 3000 is advised. Sometimes much higher values are required
    for stable estimation of standard errors.}
  \item{nsub}{Number of subphases in phase 2.}
  \item{n2start}{Minimum number of iterations in subphase 1 of phase 2;
    default is \code{2.52*(p+7)}, where \code{p} = number of estimated
    parameters; if \code{useCluster = TRUE} in the call of
    \code{\link{siena07}}, this is divided by \code{nbrNodes}.  }
  \item{firstg}{Initial value of scaling ("gain") parameter for updates
    in the Robbins-Monro procedure.}
  \item{reduceg}{Reduction factor for scaling ("gain") parameter for updates
    in the Robbins-Monro procedure (MoM only).}
  \item{truncation}{Used for step truncation in the Robbins Monro algorithm
    (applied to deviate/(standard deviation)).}
  \item{doubleAveraging}{subphase after which double averaging is used
    in the Robbins Monro algorithm,
    which probably increases algorithm efficiency.}
  \item{diagonalize}{Number between 0 and 1 (bounds included),
    values outside this interval will be truncated;
    for diagonalize = 0 the complete estimated derivative matrix will be used
    for updates in the Robbins-Monro procedure;
    for diagonalize = 1 only the diagonal entries will be used;
    for values between 0 and 1, the weighted average will be used
    with weight diagonalize for the diagonalized matrix.
    Has no effect for ML estimation.\cr
    Higher values are more stable, lower values potentially more efficient.
    Default: for ML estimation, diagonalize = 0; for MoM estimation,
    diagonalize = 0.2.}
  \item{standardizeVar}{Logical: whether to limit deviations used in
    Robbins-Monro updates to unit variances.}
  \item{dolby}{Boolean. Should there be noise reduction by regression
    on augmented data score. In most cases dolby = TRUE yields better
    convergence, but takes some extra computing time;
    if convergence is problematic, however, dolby = FALSE may be tried.
    Just use whatever works best.}
  \item{splitDepvars}{Integer with possible values \code{-1, 0, +1}. 
    If \code{-1}, obtain results as in versions
    prior to 1.3.6. If \code{+1}, Robbins-Monro updates for a given dependent
	variable will not depend on deviations for other dependent variables.
	See Details.}
  \item{useStdInits}{Boolean. If TRUE, the initial values in the effects
    object will be ignored and default values used instead.
    If FALSE, the initial values in the effects object will be used.}
  \item{findiff}{ Boolean: If TRUE, estimate derivatives using finite
    differences. If FALSE, use scores.}
  \item{mult}{Multiplication factor for maximum likelihood and Bayes. Number of
    steps per iteration is set to this multiple of the total distance
    between the observations at start and finish of the wave (and rounded).
    Decreasing \code{mult} below a certain value has no further effect.\cr
     \code{mult} can be either a number (which needs to be positive) or a vector
    of numbers, of length equal to the total number of periods. Note that for
    multi-group data, the total number of periods is equal to the number
    of groups times the number of periods per group
    (if the latter is constant).}
  Algorithm parameters only relevant for maximum likelihood
    and Bayesian estimation:
  \item{prML}{Either one real number, or a vector of 7 numbers.
    Determines update probabilities used in Metropolis-Hastings
    routine in ML estimation. Should be nonnegative; if a vector,
    the sum should be < =  1. See \code{Details}.}
  \item{maximumPermutationLength}{Maximum length of permutation
    in steps in ML estimation.}
  \item{minimumPermutationLength}{Minimum length of permutation
    in steps in MLestimation.}
  \item{initialPermutationLength}{Initial length of permutation
    in steps in MLestimation.}
  \item{localML}{Logical: If TRUE, and \code{maxlike}, then calculations
    are sped up for models with all local effects.}
  For \code{set_output_saom}:
  \item{outputName}{If \code{outputName = NULL} and the Siena data set
   has name \code{dataName}, output will be written  by
   \code{\link{siena}} to a file with name \code{dataName_out.txt}
   in the working directory, appending if this file already exists.\cr
    If \code{outputName} is not \code{NULL}, it should be a character string
    without embedded spaces, and output will be written
    to a file with name \code{outputName_out.txt}.}
  \item{lessMem}{Logical: whether to reduce storage during operation of
    \code{\link{siena}}, and of the object produced, by leaving out
    arrays by iteration and by period of simulated statistics \code{sf2}
    and scores \code{ssc}.\cr
    if \code{lessMem = TRUE}, it will be impossible to run
    \code{\link{test_time}} or \code{\link{test_gof}} on
    the object produced by \code{\link{siena}}.}
  \item{returnThetas}{Boolean: whether to return theta values
   and generated estimation statistics of Phase 2 runs.}
  \item{returnChains}{Boolean. Whether to return the chains generated in
    the Phase 3 runs.}
  \item{returnDataFrame}{Boolean. Whether to return the chains as
    lists or data frames.}
  \item{returnChangeContributions}{Boolean. Whether to return the change contributions.
     See \code{\link{interpret_size_dynamics}}.}

  And additional, for \code{sienaAlgorithmeCreate}:
  \item{fn}{Function to do one simulation in the Robbins-Monro
    algorithm. Not to be touched.}
  \item{projname}{Character string name of project; the output file will be
    called projname.txt. No embedded spaces!!!\cr
    If \code{projname = NULL}, output will be written to a file in the temporary
    session directory, created as \code{\link{tempfile}(Siena)}.}
  \item{silent}{Logical: whether to give a note about the output file.}
}
\details{Model specification is done via this object for
  \code{\link{siena}}, which calls function \code{\link{simstats0c}}.
  This function creates an object with the elements required to control the
  Robbins-Monro algorithm. Those not
  available as arguments can be changed manually when desired.\cr
  The matrix \code{D} transforming deviations from targets to updates
  in the Robbins Monro steps are partially diagonalized 
  if \code{diagonalize > 0}; and its between-dependent variable blocks
  are put to \code{0} if \code{splitDepvars=1}. Diagonalization is omitted
  for the update step immediately at the end of Phase if \code{splitDepvars=-1}.
  
  The value \code{prML = 1} defines the defaults valid in RSiena up to version
  1.3.16. For ML estimation with only one dependent variable,
  \code{prML = 2} may be more efficient.\cr
  If \code{prML} is given as a vector of 7 probabilities, these are,
  consecutively: the probabilities of inserting a diagonal step, deleting a
  diagonal step, permuting, inserting a CCP, deleting a CCP, inserting random
  missing, deleting random missing; the residual (1 minus the sum)
  is the probability of a move step.\cr
  Further information about the implementation of the algorithm is in \cr
  \url{https://www.stats.ox.ac.uk/~snijders/siena/Siena_algorithms.pdf}.\cr
  Some of the examples use \code{projname = NULL}; this is just for the sake of
  checking the examples, not necessarily intended for normal use.
}
\value{\code{set_model_saom} returns an object of class \code{sienaModel},
  \code{set_algorithm_saom} returns an object
  of class \code{sienaAlgorithmSettings},
  \code{set_output_saom} returns an object
  of class \code{sienaOutputOptions},
  and \code{sienaAlgorithmCreate} returns an object of class \cr
  \code{sienaAlgorithm}, all being lists containing values implied by the parameters.
}
\references{
    For \code{modelType}:\cr
   Snijders, T.A.B., and Pickup, M. (2016),
   Stochastic Actor-Oriented Models for Network Dynamics.
   In: Victor, J.N., Lubell, M., and Montgomery, A.H.,
   \emph{Oxford Handbook of Political Networks}. Oxford University Press.

    Also see \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Ruth Ripley and Tom A.B. Snijders}
\seealso{\code{\link{siena}}, \code{\link{simstats0c}}.}
\examples{
CondAlgorithm <- set_algorithm_saom(cond = TRUE, condvarno = 1)
Max10Model <- set_model_saom(MaxDegree = c(mynet = 10),
                                modelType = c(mynet = 1))
Beh2Model <- set_model_saom(behModelType = c(mybeh = 2))
# where mynet is the name of the network object
# created by as_dependent_rsiena,
# and mybeh the name of the behavior object created by the same function.
Algorithm2 <- set_algorithm_saom(nsub = 2, n2start = 2000)
OutAlgorithm <- set_output_saom(outputName = "LaterEstimation",
                        returnThetas = TRUE)
}
% Add one or more standard keywords, see file "KEYWORDS" in the
% R documentation directory.
\keyword{classes}

\name{simstats0c}
\alias{simstats0c}
\alias{maxlikec}
\alias{initializeFRAN}
\alias{terminateFRAN}
\title{Versions of FRAN}
\description{
 The functions to be called as "FRAN" by \code{\link{siena}}
 and \code{\link{siena07}}.
 They call compiled C++. Not for general users' use.
}
\usage{
simstats0c(z, x, data=NULL, effects=NULL, fromFiniteDiff=FALSE,
           returnDeps=FALSE, returnChains=FALSE, 
           returnChangeContributions=FALSE,
           byWave=FALSE, returnDataFrame=FALSE, returnLoglik=FALSE)
maxlikec(z, x, data=NULL, effects=NULL,
           returnChains=FALSE, byGroup = FALSE, 
           byWave=FALSE, returnDataFrame=FALSE, 
           returnLoglik=FALSE, onlyLoglik=FALSE)
initializeFRAN(z, x, data, effects, prevAns=NULL, initC,
           profileData=FALSE, returnDeps=FALSE, returnChains=FALSE,
           returnChangeContributions=FALSE, 
           byGroup=FALSE, returnDataFrame=FALSE,
           byWave=FALSE, returnLoglik=FALSE, onlyLoglik=FALSE)
terminateFRAN(z, x)
}
\arguments{
  \item{z}{Control object, passed in automatically in  \code{\link{siena}}.}
  \item{x}{A sienaAlgorithm object, passed in automatically in  \code{\link{siena}}.}
  \item{data}{A \code{\link{sienadata}} object as returned by 
    \code{\link{make_data_rsiena}}.}
  \item{effects}{A sienaEffects object as
    returned by \code{\link{make_specification}}.}
  \item{fromFiniteDiff}{Boolean used during calculation of derivatives
    by finite differences. Not for user use.}
  \item{returnDeps}{Boolean. Whether to return the simulated networks in
    Phase 3.}
  \item{returnChains}{Boolean. Whether to return the chains.}
  \item{returnChangeContributions}{Boolean. Whether to return the change contributions.}
  \item{byWave}{Boolean. Whether to return the finite difference or
    maximum likelihood
    derivatives by wave (uses a great deal of memory). Only necessary
    for \code{\link{test_time}}}
  \item{byGroup}{Boolean. For internal use: allows different thetas for each
    group to be used in \code{sienaBayes}.}
  \item{returnDataFrame}{Boolean. Whether to return the chains as
    lists or data frames.}
  \item{returnLoglik}{Boolean. Whether to return the log likelihood of
    the simulated chain.}
   \item{onlyLoglik}{Boolean: whether to return just the likelihood for
    the simulated chain, plus details of steps accepted and rejected. }
  \item{prevAns}{An object of class "sienaFit" as returned by
   \code{\link{siena}}, from which scaling
    information (derivative matrix and standard deviation of the
    deviations) will be extracted along with the latest version of the
    parameters which will be used as the initial values, unless the
    model requests the use of standard initial values. If the previous
    model is exactly the same as the current one, Phase 1 will be omitted. If
    not, any parameter estimates for effects which are included in
    the new model will be used as initial
    values, but phase 1 will still be carried out.
    If the results used as \code{prevAns} are a reasonable starting point,
    this will increase the efficiency of the algorithm. }
  \item{initC}{If TRUE, call is to setup the data and model in C++. For
    use with multiple processes only.}
 \item{profileData}{Boolean to force dumping of the data for profiling
    with \command{sienaProfile.exe}.}
}
\details{
  Not for general users' use.\cr
  The name of \code{simstats0c} or \code{maxlikec}
  should be used for the element FRAN of the model object, the former when using
  estimation by forward simulation, the latter for maximum likelihood estimation.
  The arguments with no defaults must be passed in on the call to
  \code{\link{siena}}.
  \code{initializeFRAN} and \code{terminateFRAN} are called in both cases.
}
\value{\code{simstats0c} returns a list containing:
  \item{fra}{Simulated statistics.}
  \item{sc}{Scores with which to calculate the derivative (not phase 2
    or if using finite differences or maximum likelihood).}
  \item{dff}{Contributions to the derivative if finite differences}
  \item{ntim}{For conditional processing, time taken.}
  \item{feasible}{Currently set to TRUE.}
  \item{OK}{Could be set to FALSE if serious error has occurred.}
  \item{sims}{A list of simulation results, one for each period. Each
    list consists of a list for each data object, each of which consists of
    a list for each network, each of which consists of a list for each period,
    each component of which is an edgelist in matrix form (the columns
    are from, to, value) (or vector for behavior variables).
    Only if \code{returnDeps} is TRUE.}
  \code{maxlikec} returns a list containing:
  \item{fra}{Simulated scores.}
   \item{dff}{Simulated Hessians: stored as lower triangular matrices}
  \item{ntim}{NULL, compatibility only}
  \item{feasible}{Currently set to TRUE.}
  \item{OK}{Could be set to FALSE if serious error has occurred.}
  \item{dff}{Simulated Hessian}
  \item{sims}{NULL, for compatibility only}
  \item{chain}{A list of sampled chains, one for each period. Each
    list consists of a list for each data object, each of which consists of
    a list for each network, each of which consists of a list for each period,
    each component of which is a list or a data frame depending on the
    value of \code{returnDataFrame}.
    Only if \code{returnChains} is TRUE.}
  \item{changeContributions}{A list of sampled chains, one for
    each period. Each list consists a list for each dependent variable, each 
    of which consists of a list for each period, each component of which 
    is a matrix of effects times choices containing the changeContributions 
    (+- change statistics) for each ministep. \cr
    Only if \code{returnChangeContributions} is TRUE.}
  \item{accepts}{Number of accepted MH steps by dependent variable
    (permute steps are counted under first dependent variable)}
  \item{rejects}{Number of rejected MH steps by dependent variable
    (permute steps are counted under first dependent variable)}
  \item{aborts}{Number of aborted MH steps counted under first dependent
    variable.}
  \item{loglik}{Loglikelihood of the simulations. Only if
    \code{returnLoglik} is TRUE. \cr
    If \code{onlyLoglik} is TRUE, only \code{loglik}, \code{accepts},
    \code{rejects} and \code{aborts} are returned.}
  \code{initializeFRAN} and \code{terminateFRAN} return the control
    object \code{z}.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley }
\seealso{\code{\link{siena}} }
\examples{
mynet1 <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
mydata <- make_data_rsiena(mynet1)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, transTrip)
myalgorithm <- sienaAlgorithmCreate(fn=simstats0c, 
                                    nsub=2, n3=100, projname=NULL)
ans <- siena07(myalgorithm, data=mydata, effects=myeff,  batch=TRUE)
}
\keyword{models}

\name{make_data_rsiena}
\alias{sienaDataCreate}
\alias{make_data_rsiena}
\alias{sienadata}
\title{Function to create a Siena data object}
\description{
    Creates a Siena data object from input dependent variables   
    (networks and possibly behavioural variables), covariates,
     and composition change objects.\cr
  The functions \code{make_data_rsiena} and \code{sienaDataCreate}
  are identical. The first name is preferred, 
  the other is kept for backward compatibility.
}
\usage{
make_data_rsiena(..., nodeSets=NULL)

sienaDataCreate(..., nodeSets=NULL, getDocumentation=FALSE)
}
\arguments{
  \item{\dots}{objects of class \code{\link{sienaDependent}},
    \code{\link{coCovar}}, \code{\link{varCovar}}, \code{\link{coDyadCovar}},
  \code{\link{varDyadCovar}}, and/or \code{\link{sienaCompositionChange}};
  or a list of such objects, of which the first element must not be
  a \code{\link{sienaCompositionChange}} object.
  There should be at least one \code{\link{sienaDependent}} object.\cr
  If there are one-mode as well as two-mode dependent networks,
  the one-mode networks should be mentioned first.}
  \item{nodeSets}{list of Siena node sets. Default is the single node
    set named \code{Actors}, length equal to the number of rows in the first
    object of class \code{sienaDependent}. If the entire data set contains more than
    one node set, then the node sets must have been specified in
    the creation of all data objects mentioned in \dots.}
  \item{getDocumentation}{Flag to allow documentation of internal
    functions, not for use by users.}
}
\details{
The function checks that the objects fit, that there is at least one dependent
variable, and adds various attributes to each variable describing the
data. If there is more than one nodeSet they must all be specified.\cr
Function \code{\link{print01Report}} will give a basic description of the data
object and is a check useful, e.g., for diagnosing problems.
}
\value{
An object of class \code{sienadata} which is designed to be used in a siena
model fit by \code{\link{siena}}. \cr
In \code{RSiena} versions prior to 1.6, the class name was \code{siena}. \cr
The components of the object are:
\item{nodeSets}{List of node sets involved}
\item{observations}{Integer indicating number of waves of data}
\item{depvars}{List of networks and behavior variables}
\item{cCovars}{List of constant covariates}
\item{vCovars}{List of changing covariates}
\item{dycCovars}{List of constant dyadic covariates}
\item{dyvCovars}{List of changing dyadic covariates}
\item{compositionChange}{List of composition change objects
  corresponding to the node sets}
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley}
\seealso{
\code{\link{as_dependent_rsiena}}, \code{\link{as_covariate_rsiena}},
\code{\link{as_nodeset_rsiena}}, \cr
\code{\link{as_composition_rsiena}}, \code{\link{make_group_rsiena}},
\code{\link{make_constraint.sienadata}}, \cr
\code{\link{print01Report}}}
\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh <- as_dependent_rsiena(s50a, type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
# This gives the same result as
mydata <- make_data_rsiena(list(mynet, mybeh))
## And for a two-mode network
mynet1 <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)), 
                              nodeSet="senders")
senders <- as_nodeset_rsiena(50, nodeSetName="senders")
receivers <- as_nodeset_rsiena(30, nodeSetName="receivers")
mynet2 <- as_dependent_rsiena(
            array(c(s501[,1:30], s502[,1:30]), dim=c(50, 30, 2)),
            nodeSet=c("senders", "receivers"))
(mydata <- make_data_rsiena(mynet1, mynet2, nodeSets=list(senders, receivers)))
\dontrun{
print01Report(mydata, modelname = "mydescription")
}
}
\keyword{classes}

\name{coCovar}
\alias{coCovar}
\title{Function to create a constant covariate object}
\description{
  This function creates a constant covariate object from a vector.  \cr
   This function now is superseded by function \code{\link{as_covariate_rsiena}}.
  }
\usage{
coCovar(val, centered=TRUE, nodeSet="Actors", warn=TRUE, 
        imputationValues=NULL)
}
\arguments{
  \item{val}{Vector of covariate values}
  \item{centered}{Boolean: if TRUE, then the mean value is subtracted.}
  \item{nodeSet}{Name of node set: character string.
  If the entire data set contains more than one node set,
  then the node sets must be specified in all data objects.}
  \item{warn}{Logical: is a warning given if all values are \code{NA},
  or all non-missing values are the same.}
  \item{imputationValues}{Vector of covariate values of same
  length as \code{val}, to be used for imputation of
  \code{NA} values (if any) in \code{val}. Must not contain
  any \code{NA}.}
}
\details{
  When part of a Siena data object,
  the covariate is associated with the node set
  \code{nodeSet} of the Siena data object.
  In practice, the node set needs to be specified only in the case of
  the use of the covariate with a two-mode network.\cr
  If there are any \code{NA} values in \code{val}, and
  \code{imputationValues} is given, then the corresponding elements
  of \code{imputationValues} are used for imputation.
  If \code{imputationValues} is \code{NULL}, imputation
  is by the mean value. In both cases, cases with imputed values
  are not used for calculating target statistics (see the manual).
}
\value{
  Returns the covariate as an object of class "coCovar", in which
  form it can be used as an argument to \code{\link{sienaDataCreate}}.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley}
\seealso{\code{\link{as_covariate_rsiena}},
\code{\link{sienaDataCreate}}, \code{\link{varCovar}},
\code{\link{coDyadCovar}}, \cr
\code{\link{varDyadCovar}}, \code{\link{sienaNodeSet}}}
\examples{

myconstCovar <- coCovar(s50a[,1])
senders <- sienaNodeSet(50, nodeSetName="senders")
receivers <- sienaNodeSet(30, nodeSetName="receivers")
senders.attribute <- coCovar(rep(1:10, each=5), nodeSet="senders")
receivers.attribute <- coCovar(rep(1:5, each=6), nodeSet="receivers")
}
\keyword{classes}

\name{varDyadCovar}
\alias{varDyadCovar}
\title{Function to create a changing dyadic covariate object.}
\description{
  This function creates a changing dyadic covariate object from an array. \cr
   It now is superseded by function \code{\link{as_covariate_rsiena}}.
}
\usage{
varDyadCovar(val, centered=TRUE, nodeSets=c("Actors", "Actors"),
     warn=TRUE, sparse=is.list(val), type=c("oneMode", "bipartite"))
}
%- maybe also "usage" for other objects documented here.
\arguments{
  \item{val}{Array of covariate values, third dimension is the
    time. Alternatively, a list of sparse matrices of type \code{"TsparseMatrix"}.}
  \item{centered}{Boolean: if TRUE, then the overall mean value is subtracted.}
  \item{nodeSets}{Names (character string) of the associated node sets.
   If the entire data set contains more than one node set,
   then the node sets must be specified in all data objects.}
  \item{warn}{Logical: is a warning given if, for non-sparse input,
    all values are \code{NA}, or all non-missing values are the same.}
  \item{sparse}{Boolean: whether sparse matrices or not.}
  \item{type}{oneMode or bipartite: whether the matrix refers to a one-mode
  or a bipartite (two-mode) network.}
}
\details{
  When part of a Siena data object, the covariate is assumed to be
  associated with the node sets named \code{NodeSets}
  of the Siena data object. The names of the associated node sets will only
  be checked when the Siena data object is created.
  In practice, the node set needs to be specified only in the case of
  the use of the covariate with a two-mode network.\cr
  The value of the changing covariate for wave \code{m} is supposed
  in the simulations to be valid in the whole period from wave \code{m}
  to wave \code{m+1}.
  If the data set has \code{M} waves, this means that the values,
  if any, for wave \code{M} will not be used. Therefore, the number of columns
  can be \code{M} or \code{M-1}; if the former, the values in the last
  column will not be used.
}
\value{
  Returns the covariate as an object of class "varDyadCovar", in which
  form it can be used as an argument to \code{\link{make_data_rsiena}}.
}
\references{See \url{https://www.stats.ox.ac.uk/~snijders/siena/}}
\author{Ruth Ripley}
\seealso{\code{\link{as_covariate_rsiena}}, 
                \code{\link{as_nodeset_rsiena}}}
\examples{
mydyadvar <- as_covariate_rsiena(array(c(s501, s502), dim=c(50, 50, 2)),
                                type="oneMode")
class(mydyadvar)
}
% Add one or more standard keywords, see file "KEYWORDS" in the
% R documentation directory.
\keyword{classes}

\name{funnelPlot}
\alias{funnelPlot}
\title{Plot function for a list of sienaFit objects}
\usage{
funnelPlot(anslist, k, threshold=NULL, origin=TRUE,
           plotAboveThreshold=TRUE, verbose=TRUE, ...)
}
\description{
  Draws a funnel plot for a list of sienaFit objects that all  
  have estimated the same parameter.
}
\arguments{
  \item{anslist}{A list of object of class \code{sienaFit}.}
  \item{k}{The number of the parameter to be plotted.}
  \item{threshold}{threshold for standard errors:
     all estimations where the standard error for parameter
     \code{k} is larger than this threshold will be disregarded.}
  \item{origin}{Boolean: whether to include the origin in the plot,
    if all estimates have the same sign.}
  \item{plotAboveThreshold}{Boolean: whether to include the estimates
    for which the standard error is larger than \code{threshold},
    and plot them with an asterisk at \code{se=threshold}.}
  \item{verbose}{Boolean: whether to report in the console all estimates
    omitted, because either their standard error is larger than
    \code{threshold}, or they were fixed.}
  \item{\dots}{For extra arguments (passed to \code{plot}).}
}
\details{
 The function \code{funnelPlot} plots estimates against standard
 errors for a given effect \code{k}, with red
 reference lines added at the two-sided
 significance threshold 0.05. Effects for which a score test
 was requested are not plotted (and reported to the console
 if \code{verbose}).\cr
 If not all effects with number \code{k} are the same
 in all \code{sienaFit} objects, a warning is given.
 The effect name for the first object is used as the plot title.\cr
 Another funnel plot is available as \code{\link{print.sienaMeta}}.
}
\value{
 The two-column matrix of
 values of the plotted points is invisibly returned.\cr
}
\author{Tom Snijders
}
\seealso{\code{\link{meta_siena}}, \code{\link{print.sienaMeta}}}

\examples{
# A meta-analysis for three groups does not make much sense.
# But using three groups shows the idea.
Group1 <- as_dependent_rsiena(array(c(N3401, HN3401), dim=c(45, 45, 2)))
Group3 <- as_dependent_rsiena(array(c(N3403, HN3403), dim=c(37, 37, 2)))
Group4 <- as_dependent_rsiena(array(c(N3404, HN3404), dim=c(33, 33, 2)))
dataset.1 <- make_data_rsiena(Friends = Group1)
dataset.3 <- make_data_rsiena(Friends = Group3)
dataset.4 <- make_data_rsiena(Friends = Group4)
OneAlgorithm <- set_algorithm_saom(nsub=1, n3=50, seed=123)
effects.1 <- make_specification(dataset.1)
effects.3 <- make_specification(dataset.3)
effects.4 <- make_specification(dataset.4)
ans.1 <- siena(dataset.1, effects=effects.1, 
							control_algo=OneAlgorithm, batch=TRUE)
ans.3 <- siena(dataset.3, effects=effects.3, 
							control_algo=OneAlgorithm, batch=TRUE)
ans.4 <- siena(dataset.4, effects=effects.4, 
							control_algo=OneAlgorithm, batch=TRUE)
funnelPlot(list(ans.1, ans.3, ans.4), k=2)
funnelPlot(list(ans.1, ans.3, ans.4), k=2, origin=FALSE)
}
\keyword{plot}

\name{transformScript}
\alias{transformScript}
\title{Function to transform \code{RSiena}  and \code{multiSiena} scripts }
\description{
   Function to transform a script with  'old' function names
     of \code{RSiena}  and \code{multiSiena} to the new names
     that are used since version \code{RSiena.1.6.0} and
     \code{multiSiena.1.2.36}.
}
\usage{
transformScript(theoldscript, fileName="newScript.R",
                    linelength=80, initialblanks=6, logfile="log.txt",
                    keepOldAlgorithms=FALSE)
}
\arguments{
  \item{theoldscript}{Character vector: an R script which uses \code{RSiena}
   and/or \code{multiSiena}.}
  \item{fileName}{Character string naming a file for the new script.}
  \item{linelength}{The maximum line length for the new script.}
  \item{initialblanks}{The number of blanks at the start of each follow-up
     line for a given transformed R command.}
  \item{logfile}{Character string naming a file, for the log file.}
  \item{keepOldAlgorithms}{Boolean: should the old calls of
        \code{sienaAlgorithmCreate} be retained in the new script,
        even though they are redundant.}
}
\details{This function transforms \code{theoldscript}, supposedly a script with
     'old' function names of \code{RSiena} and \code{multiSiena},
     and transforms it to \code{fileName} in which the old
     names are replaced with the new names
     that are used since version \code{RSiena.1.6.0}; the use of the functions
     is adapted correspondingly.\cr
     The calls for constructing the algorithms required for
     the  estimation functions \code{siena} and \code{multi_siena}
     are positioned immediately before the calls
     of the respective estimation functions. This may involve duplication.\cr
     For creation of a time-constant dyadic covariate ("\code{coDyadCovar}")
     the argument \code{type=oneMode} is always used;
     the user should check whether, instead,
     it should be \code{type=bipartite}.\cr
     This function is not guaranteed to be correct,
     and the result should be checked!\cr
     Hash signs (\code{#}) in strings that are names of objects
     in \code{RSiena} functions will lead to errors, because it is assumed
     that everything following a has sign in a line 
     is not part of the command.\cr
     Functions utilizing other \code{RSiena} functions 
     may also lead to errors.\cr
     If an error is encountered, the end of the log file will show the line 
     number to which the transformation proceeded without errors.\cr
     The use of \code{keepOldAlgorithms} is just for the sake of
     possible information or checking. These calls of
     \code{sienaAlgorithmCreate} can be dropped from the new script
     without any consequences for its functioning.\cr\cr
     Note that using \code{""} for a filename will write to the console.\cr\cr
     This function transforms calls of \code{RSiena} and
     \code{multiSiena} functions to calls utilizing the new names.
     The only \code{RSiena} functions in \code{theoldscript}
     that are executed are calls of \code{\link{sienaAlgorithmCreate}};
     their results are used for transforming
     calls of \code{\link{siena07}} and \code{sienaBayes}.
}
\value{Invisibly, the new script will be returned as a character vector.
}
\references{See the Chapter on "New function names" in the manual
   available from \url{https://www.stats.ox.ac.uk/~snijders/siena/ }.}
\author{Tom Snijders}
\examples{
oldscript <- c("mynet <- sienaDependent(array(c(s501, s502), dim=c(50, 50, 2)))",
    "# construct actor covariate",
"drink <- coCovar(s50a[1,], centered=FALSE)",
"mydata <- sienaDataCreate(mynet, drink)",
"mymodel <- getEffects(mydata)",
"mymodel <- setEffect(mymodel, transTrip)",
"mymodel <- includeEffects(mymodel, simX ,egoX, altX, interaction1='drink')",
"mycontrols0 <- sienaAlgorithmCreate(projname='ans.txt')",
"(ans <- siena07(mycontrols0, data=mydata, effects=mymodel))",
"mycontrols <- sienaAlgorithmCreate(projname='ans.txt', nsub=1, ",
"                                   n2start=2000, n3=2000, seed=1234)",
"(ans1 <- siena07(mycontrols, data=mydata, effects=mymodel, prevAns=ans))",
"(mtest <- Multipar.RSiena(ans1, tested=3:4))")
transformScript(oldscript, fileName="", logfile=tempfile())
}
\keyword{classes}

\name{s503}
\alias{s503}
\docType{data}
\title{Network 3 data: excerpt from "Teenage Friends and Lifestyle Study" data.}
\description{
Second timepoint network data from   
an excerpt of the network, alcohol consumption, and smoking data     
for 50 randomly chosen girls from the
Teenage Friends and Lifestyle Study data set. Useful as a small    
example of network and behaviour, for which models can be    
fitted quickly, and for which there are no missing values.
}
\format{
 Adjacency matrix for the network at time point 3.
}
\source{
\url{https://www.stats.ox.ac.uk/~snijders/siena/s50_data.zip}
}
\references{
West, P. and Sweeting, H. (1995), \emph{Background Rationale and Design of the
West of Scotland 11-16 Study}.
Working Paper No. 52. MRC Medical Sociology Unit Glasgow.

See \url{https://www.stats.ox.ac.uk/~snijders/siena/s50_data.htm}
}
\seealso{\code{\link{s501}}, \code{\link{s502}}, \code{\link{s50a}}, \code{\link{s50s}}
}
\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502, s503), dim=c(50, 50, 3)))
mybeh  <- as_dependent_rsiena(s50a, type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
mydata
}
\keyword{datasets}

% file RSiena/man/summary.iwlsm.Rd
% based on file MASS/man/summary.rlm.Rd
% copyright (C) 1994-9 W. N. Venables and B. D. Ripley
%
\name{summary.iwlsm}
\alias{summary.iwlsm}
\alias{print.summary.iwlsm}
\title{
Summary method for Iterative Weighted Least Squares Models
}
\description{
\code{summary} method for objects of class \code{"iwlsm"}
}
\usage{
\method{summary}{iwlsm}(object, method = c("XtX", "XtWX"),
 correlation = FALSE, \dots)
}
\arguments{
\item{object}{
the fitted model.
This is assumed to be the result of some fit that produces
an object inheriting from the class \code{iwlsm}, in the sense that
the components returned by the \code{iwlsm} function will be available.
}
\item{method}{
Should the weighted (by the IWLS weights) or unweighted cross-products
matrix be used?
}
\item{correlation}{
logical. Should correlations be computed (and printed)?
}
\item{\dots}{
arguments passed to or from other methods.
}}
\value{
If printing takes place, only a null value is returned.
Otherwise, a list is returned with the following components.
Printing always takes place if this function is invoked automatically
as a method for the \code{summary} function.

\item{correlation}{
The computed correlation coefficient matrix for the coefficients in the model.
}
\item{cov.unscaled}{
The unscaled covariance matrix; i.e, a matrix such that multiplying it by
an estimate of the error variance produces an estimated covariance matrix
for the coefficients.
}
\item{sigma}{
The scale estimate.
}
\item{stddev}{
A scale estimate used for the standard errors.
}
\item{df}{
The number of degrees of freedom for the model and for residuals.
}
\item{coefficients}{
A matrix with three columns, containing the coefficients, their standard errors
and the corresponding t statistic.
}
\item{terms}{
The terms object used in fitting this model.
}}
\details{
This function is a method for the generic function
\code{summary()} for class \code{"iwlsm"}.
It can be invoked by calling \code{summary(x)} for an
object \code{x} of the appropriate class, or directly by
calling \code{summary.iwlsm(x)} regardless of the
class of the object.
}
\references{
  Venables, W. N. and Ripley, B. D. (2002),
  \emph{Modern Applied Statistics with S.} Fourth edition.  Springer.
  See also  \url{https://www.stats.ox.ac.uk/~snijders/siena/}
}
\author{Adapted by Ruth Ripley}
\seealso{
\code{\link{summary}}
}
\examples{
\dontrun{
##not enough data here for a sensible example, but shows the idea.
myalgo <- sienaAlgorithmCreate(nsub=2, n3=100)
mynet1 <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mynet2 <- as_dependent_rsiena(array(c(s502, s503), dim=c(50, 50, 2)))
mydata1 <- make_data_rsiena(mynet1)
mydata2 <- make_data_rsiena(mynet2)
myeff1 <- make_specification(mydata1)
myeff2 <- make_specification(mydata2)
myeff1 <- set_effect(myeff1, transTrip)
myeff2 <- set_effect(myeff2, transTrip)
myeff1 <- set_effect(myeff1, cycle3)
myeff2 <- set_effect(myeff2, cycle3)
ans1 <- siena(mydata1, effects=myeff1, control_algo=myalgo, batch=TRUE)
ans2 <- siena(mydata2, effects=myeff2, control_algo=myalgo, batch=TRUE)
meta <- meta_siena(ans1, ans2)
metadf <- split(meta$thetadf, meta$thetadf$effects)[[1]]
metalm <- iwlsm(theta ~ tconv, metadf, ses=se^2)
summary(metalm)
}
}
\keyword{methods}

\name{interpret_influence}
\alias{interpret_influence}
\alias{interpret_influence.sienaFit}
\alias{influenceTable}
\alias{influencetable}
\alias{influence.Table}
\alias{influence.table}
\title{Function to construct influence tables for SAOMs}
\description{
 The function \code{influenceTable} constructs influence tables
 which may be helpful for the interpretation of results
 for network and behavior dynamics,
 for an estimation result represented by a
 \code{sienaFit} object created by \code{\link{siena}}.
}

\usage{
\method{interpret_influence}{sienaFit}(x, xd, netname, behname,
    as.matrix=FALSE, levls=NULL, levls.alt=levls,
    out.ego=1, silent=FALSE, nfirst=x$nwarm+1,
    include.endow=FALSE, include.creation=FALSE, ...)

influenceTable(x, xd, netname, behname,
    as.matrix=FALSE, levls=NULL, levls.alt=levls,
    out.ego=1, silent=FALSE, nfirst=x$nwarm+1,
    include.endow=FALSE, include.creation=FALSE)
}
\arguments{
  \item{x}{An object of class \code{\link{sienaFit}}, produced by a
    call to \code{\link{siena}} for a model including
    a behavioral dependent variable;
    or an object of class \code{\link{sienaMeta}}, produced by a
    call to \code{\link{meta_siena}};
    or an object of class \code{sienaBayesFit}, produced by a
    call to \code{sienaBayes}.  }
  \item{xd}{If \code{x} is of class \code{\link{sienaFit}},
    a \code{\link{sienadata}} data set used to produce \code{x}.\cr
    If \code{x} is of class \code{\link{sienaMeta}} or \code{sienaBayesFit},
    one of the data sets used to produce \code{x}; preferable a representative
    one.}
  \item{netname}{character string: name of network dependent variable.}
  \item{behname}{character string: name of behavior dependent variable.}
  \item{as.matrix}{boolean: will the table be returned as a matrix.}
  \item{levls}{levels for ego.}
  \item{levls.alt}{levels for alter.}
  \item{out.ego}{presumed outdegree of ego for effects \code{totSim} and
       \code{totAlt}.}
  \item{silent}{boolean; if \code{FALSE}, the parameters taken from \code{x}
     are reproduced at the console.}
  \item{nfirst}{If \code{x} is of class \code{sienaBayesFit}: first run in
       posterior sample used for constructing the table.}
  \item{include.endow}{boolean, used only for models utilizing influence and
        endowment effects: by specifying \code{include.endow=TRUE},                                         #
        the sum of evaluation and endowment effects is used.}
  \item{include.creation}{boolean, used only for models utilizing influence and
         creation effects: by specifying \code{include.creation=TRUE},                                         #
        the sum of evaluation and creation effects is used.}
  \item{\dots}{Other arguments.}
}
\details{
 This functions is used for constructing influence tables
 for the interpretation of results for network and behavior dynamics
 obtained with the \code{RSiena} or \code{multiSiena} packages.

 In matrix form, each row corresponds to a given average behavior of the
 alters, to whom the focal actor (ego) is connected by an outgoing tie;
 the columns are the different potential values of ego’s own behavior.
 The rows are for the values in \code{levls.alt}, the columns
 for the values in \code{levls}.

 If \code{levls} is \code{NULL} (the default), the levels of
 ego's behavior are taken as the integer range of
 the dependent actor variable.
 In most applications \code{levls.alt} will be the same as \code{levls},
 in which case it does not have to be specified.

 The data set \code{xd} is only used to get means and ranges
 which are used somewhere in the effects.
 
 This function has its own print method.\cr
 It can be plotted by package \code{autograph}.
}

\value{
    An object of class \code{influenceTable}.\cr
  This represents the joint contribution to the evaluation function
  of effects \code{"linear"}, \code{"quad"}, \code{"avAlt"}, \code{"avSim"},
  \code{"totAlt"}, \code{"totSim"},  \code{"avAttHigher"}, \code{"avAttLower"},
  \code{"threshold"}, \code{"threshold2"}, \code{"threshold3"},
  and \code{"threshold4"}.\cr
  If \code{as.matrix=TRUE}, the object is a matrix;
  if \code{as.matrix=FALSE} (the default), it is a data frame, where the rows
  are all combinations of \code{levls} and \code{levls.alt}, and the columns:
  \item{alter}{alter's values as a factor;}
  \item{zalter}{the values \code{levls.alt}}
  \item{zego}{the values \code{levls}}
  \item{select}{the value of the influence table.}\cr
  This object has an attribute \code{quad}, which specifies whether the plot
  will be a quadratic function.  
}

\references{
  \url{https://www.stats.ox.ac.uk/~snijders/siena/}\cr
  See the manual, Sections 14.2 and 14.4.
}
\author{Tom Snijders}
\seealso{\code{\link{siena}}, \code{\link{meta_siena}} }
\examples{
mynet <- as_dependent_rsiena(array(c(s501, s502), dim=c(50, 50, 2)))
mybeh  <- as_dependent_rsiena(s50a[,1:2], type="behavior")
mydata <- make_data_rsiena(mynet, mybeh)
myeff <- make_specification(mydata)
myeff <- set_effect(myeff, avAlt, depvar="mybeh", covar1="mynet")
myalgo <- set_algorithm_saom(nsub=2, n3=100, seed=1291)
# nsub=2, n3=100 is used here for having a brief computation, not for practice.
(ans <- siena(mydata, effects=myeff, control_algo=myalgo, 
                                            silent=TRUE, batch=TRUE))
interpret_influence(ans, mydata, "mynet", "mybeh")
}
\keyword{models}
\keyword{methods}

\name{siena}
\alias{siena}
\alias{siena.sienadata}
\alias{siena07}
\concept{estimation}
\title{Function to estimate parameters in a Siena model}
\description{
 Estimates parameters in a Siena model using Method of Moments,
 based on direct simulation, conditional or otherwise;
 or using Generalized Method of Moments;
 or using Maximum Likelihood by MCMC simulation.
 Estimation is done using a Robbins-Monro algorithm.\cr
  \code{siena()} and \code{siena07()}
  perform the same estimation. The first of these is preferred,
  the other is kept for backward compatibility.
}
\usage{
siena(data=NULL, effects=NULL,
        control_model=NULL, control_algo=NULL, control_out=NULL,
        thetaBound = 50,
        returnDeps = FALSE,
        batch = FALSE, verbose = FALSE, silent=TRUE,
        initC=TRUE,
        nbrNodes = 1, clusterString=rep("localhost", nbrNodes),
        clusterType=c("PSOCK", "FORK"), cl=NULL, ...)

siena07(alg, batch=FALSE, verbose=FALSE, silent=FALSE,
        useCluster=FALSE, nbrNodes=2,
        thetaValues = NULL,
        returnThetas = FALSE,
        thetaBound = 50,
        targets = NULL,
        initC=TRUE, clusterString=rep("localhost", nbrNodes), tt=NULL,
        parallelTesting=FALSE, clusterIter=!alg$maxlike,
        clusterType=c("PSOCK", "FORK"), cl=NULL, ...)
      }
\arguments{
  \item{data}{Siena data set of class \code{\link{sienadata}}. }
  \item{effects}{\code{sienaEffects} object specifying which effects are
    included the model.}
  \item{control_model}{Control object for the model,
                       of class \code{\link{sienaModel}}. }
  \item{control_algo}{Control object for the algorithm,
                       of class \code{\link{sienaAlgorithmSettings}}. }
  \item{control_out}{Control object for the output,
                       of class \code{\link{sienaOutputOptions}}. }
  \item{thetaBound}{Numeric: if at some moment during estimation the maximum
   absolute value of the parameters exceeds \code{thetaBound}, estimation
   is interrupted. In interactive use, the user is requested to give
   a higher number; if non-numeric input is given, estimation stops.
   In non-interactive use, estimation stops anyway.
   The check is turned off by using \code{thetaBound=Inf}.}
  \item{returnDeps}{Boolean. Whether to return the simulated networks in
    Phase 3.}
  \item{batch}{ Desired interface: \code{FALSE} gives a gui
     (graphical user interface implemented as a tcl/tk screen),
     \code{TRUE} gives a small (if \code{verbose=FALSE}) amount of printout
     to the console.}
  \item{verbose}{Produces various output to the console if \code{TRUE}.}
  \item{silent}{Produces no output to the console if \code{TRUE},
     even if batch mode.}
  \item{initC}{Boolean: set to \code{TRUE} if the simulation will use C
   routines (currently always needed). Only for use if using
   multiple processors, to ensure all copies are initialised
   correctly. Ignored otherwise, so is set to \code{TRUE} by default.}
  \item{nbrNodes}{Number of processes to use; when using \code{siena07},
        this is ignored if useCluster is \code{FALSE}.}
  \item{clusterString}{Definitions of clusters. Default set up to use
   the local machine only.}
  \item{clusterType}{Either "PSOCK" or "FORK". On Windows, must be
  "PSOCK". On a single non-Windows machine may be "FORK", and
  subprocesses will be formed by forking. If "PSOCK", subprocesses are
  formed using \R scripts.}
  \item{cl}{An object of class c("SOCKcluster", "cluster") (see Details).}
  \item{alg}{A control object, of class \code{\link{sienaAlgorithm}. }}
  \item{useCluster}{Boolean: whether to use a cluster of processes (useful if
  multiple processors are available).}
  \item{thetaValues}{If not \code{NULL}, this should be a matrix with parameter
    values to be used in Phase 3. The number of columns must be equal to the
    number of estimated parameters in the effects object (if conditional
    estimation is used, without the rate parameters for the conditioning
    dependent variable). Can only be used if \code{alg$simOnly=TRUE}.}
  \item{returnThetas}{Boolean: whether to return theta values
   and generated estimation statistics of Phase 2 runs.}
  \item{targets}{Numeric vector of length equal to the number of estimated
    parameters, meant to supersede the targets calculated from the data set;
    see "Details". Not for regular use.}
  \item{tt}{A \code{tcltk} toplevel window. Used if called from the
   model options screen, if \code{tcltk} is available.}
  \item{parallelTesting}{Boolean. If \code{TRUE}, sets up random numbers to
   parallel those in Siena 3.}
  \item{clusterIter}{Boolean. If \code{TRUE}, multiple processes execute
   complete iterations at each call.
   If \code{FALSE}, multiple processes execute a single wave at each call.}
  \item{\dots}{Arguments for the simulation function,
    see \code{\link{simstats0c}}:\cr
    \code{prevAns} if a previous reasonable
    provisional estimate was obtained for a similar model;\cr
    the following are ignored by \code{siena}
    (because included here or in \code{control_out}) 
    but can be mentioned when using \code{siena07}:\cr
    in any case, \code{data} and \code{effects}, as in the examples below;\cr
    possibly also \code{returnDeps} if the simulated dependent variables
    (networks, behaviour) should be returned;\cr
    possibly also \code{returnChains} or \code{returnChangeContributions}
    if the simulated sequences (chains) of ministeps or the simulated
    change statistics (contributions) should be returned;
    this may produce a very big file.}
}
\details{
  This is the main function and workhorse of \code{RSiena}.

  For use of \code{siena07}, it is necessary to specify parameters \code{data}
  (RSiena data set) and \code{effects} (effects object), which are
  required parameters in function \code{\link{simstats0c}}.
  (These parameters are inserted through '\dots'.) See the examples.

 \code{siena} runs a Robbins-Monro algorithm for parameter estimation
  using the three-phase implementation described in Snijders (2001, 2017),
 with (if \code{findiff=FALSE} in \code{control_algo})
 derivative estimation as in Schweinberger and Snijders (2007).
 The default is estimation according to the Method of Moments
 as in Snijders, Steglich and Schweinberger (2007).\cr
 If  \code{gmm=TRUE} in \code{control_algo} and \code{myeff} contains one or more
 \code{gmm} statistics as included by function
 \code{\link{includeGMoMStatistics}}, the algorithm employs the
 Generalized Method of Moments as defined in Amati, Schoenenberger, and
 Snijders (2015, 2019).\cr
 For continuous behavior variables defined with \code{type="continuous"} in \cr
 \code{\link{as_dependent_rsiena}}, estimation is done as
 described in Niezink and Snijders (2017).\cr
 If  \code{maxlike=TRUE} in \code{control_algo}, 
 estimation is done by Maximum Likelihood
 implemented as in Snijders, Koskinen and Schweinberger (2010).\cr
 Phase 1 does a few iterations to
 estimate the derivative matrix of the targets with respect to the
 parameter vector. Phase 2 does the estimation. Phase 3 runs a
 simulation to estimate standard errors and check convergence of the model.
 The simulation function is called once for each iteration in these phases
 and also once to initialise the model fitting and once to complete it.
 Unless in batch mode, a tcl/tk screen is displayed to allow interruption
 and to show progress.

 If \code{targets} is specified in \code{control_algo}
 or in the call of \code{siena07} (which should be done only in special cases),
 and provided that estimation is by the Method of Moments, the data
 is not a multi-group data set and has exactly 2 waves,
 and if the length of the vector \code{targets} is equal to the number
 of estimated parameters (not counting the rate parameters estimated
 by conditional estimation), then the vector \code{targets} supersedes the
 targets calculated from the data set.

 It is necessary to check that convergence has been achieved.
 The rule of thumb is that the all t-ratios for convergence should be
 in absolute value less than 0.1 and
 the overall maximum convergence ratio should be less than 0.25.
 If this was not achieved, the result can be used to start
 another estimation run from the estimate obtained, using
 the parameter \code{prevAns} as illustrated in the example below.
 (This parameter is inserted through '\dots' into the function
 \code{\link{initializeFRAN}}.)

 For good estimation of standard errors, it is necessary that \code{n3}
 in \code{control_algo} is large enough. More about this is in the manual.
 The default value \code{n3} is adequate for most explorative use,
 but for presentation in publications larger values are necessary, depending
 on the data set and model; e.g., \code{n3=3000} or larger.

 Parameters can be tested against zero by dividing the estimate by its
 standard error and using an approximate standard normal null distribution.
 Further, function \code{\link{test_parameter}} is available for
 multi-parameter testing.\cr
 Parameters specified in \code{\link{set_effect}} with
 \code{fix=TRUE, test=TRUE} will not be estimated;
 score tests of their hypothesized values can be obtained
 using \code{\link{test_parameter}} with \code{method="score"}.

 If \code{simOnly=TRUE} in \code{control_algo},
 which is meant to go together with  \code{nsub=0},
 the calculation of the standard errors and covariance
 matrix at the end of Pase 3 is skipped. No estimation is performed.
 If \code{thetaValues} in \code{control_algo} is not \code{NULL},
 the parameter values in  the rows of this matrix will be used
 in the consecutive runs of Phase 3.
 If \code{n3} is larger than the number of rows times \code{nbrNodes}
 (see below), the last row of \code{thetaValues} will continue to be used.
 The parameter values actually used will be stored in the
 output matrix \code{thetaUsed}.

 Multiple processors are used for estimation by MoM to distribute each iteration
 in each subphase over the cluster of nodes. The number of iterations
 accordingly will be divided (approximately) by the number of nodes;
 for phase 2, unless \code{n2start} is specified in \code{control_algo}.
 This implies that if multiple processors are used, think of dividing
 \code{n2start} by \code{nbrNodes}.\cr
 For estimation by ML, multiple processing is done per period. Therefore,
 for one period (two waves) and one group, this will have no effect.

 In the case of using multiple processors, there are two options for telling
 \code{siena} or \code{siena07} to use them.
 By specifying the options \code{useCluster},
 \code{nbrNodes}, \code{clusterString} and \code{initC}, the estimation
 will create a \code{cluster object} that will be used by the
 \code{parallel} package. After finishing the estimation procedure,
 the estimation will automatically stop the cluster. Alternatively, instead of
 having the function to create a cluster, the user may provide its own by
 specifying the option \code{cl}, similar to what the boot function does in
 the \CRANpkg{boot} package. By using the option \code{cl} the user may be
 able to create more complex clusters (see examples below).

 If \code{thetaValues} is not \code{NULL} and \code{nbrNodes >= 2}, parameters
 in Phase 3 will be constant for each set of \code{nbrNodes} consecutive
 simulations. This must be noted in the interpretation, and will be visible
 in \code{thetaUsed} (see below).

 The keyword \code{thetaBound} is used because in practice, parameter values
 of Stochastic Actor-oriented Models will be limited in magnitude,
 and for usual models values larger than 50 never occur, which means
 that when they occur this may be regarded as a signal of divergence
 of the algorithm.\cr
 Note that covariates should have large enough standard deviations
 for parameters to have moderate values; see the manual.
}
\value{
  Returns an object of class \code{\link{sienaFit}}, some parts of which are:
  \item{OK}{Boolean indicating successful termination}
  \item{termination }{Character string, values: "OK", "Error", or
    "UserInterrupt". "UserInterrupt" indicates that the user asked for early
  termination before phase 3. }
  \item{f}{Various characteristics of the data and model definition.}
  \item{requestedEffects}{The included effects in the effects object.}
  \item{effects}{The included effects in the effects object to which are added
    the main effects of the requested interaction effects, if any.}
  \item{theta}{Estimated value of theta, if \code{simOnly=FALSE}
    in \code{control_algo}.}
  \item{thetas}{Matrix, returned if \code{returnThetas} and \code{x$nsub >= 1}.
    First column is subphase; further columns are values of theta as generated
    during this subphase of Phase 2.}
  \item{sfs}{Matrix, returned if \code{returnThetas} and \code{x$nsub >= 1}.
    First column is subphase; further columns are deviations from targets
    generated during this subphase of Phase 2.}
  \item{covtheta}{Estimated covariance matrix of theta; this is not available
    if \code{x$simOnly=TRUE}.}
  \item{se}{Vector of standard errors of estimated theta,
    if \code{x$simOnly=FALSE}.}
  \item{dfra}{Matrix of estimated derivatives.}
  \item{sf}{Matrix of simulated deviations from targets in phase 3.}
  \item{sf2}{Array of periodwise deviations from simulations in phase 3.
    Not included if \code{x$lessMem=TRUE}.}
  \item{W}{If \code{x$gmm=TRUE}: Estimated optimal matrix of weights for
    estimation by the Generalized Method of Moments.}
  \item{B}{If \code{x$gmm=TRUE}: Row-normalized matrix of weights for
    equating the linear combination of estimation statistics to 0,
    for estimation by the Generalized Method of Moments.}
  \item{tconv}{t-statistics for convergence.}
  \item{tmax}{maximum absolute t-statistic for convergence for
    non-fixed parameters.}
  \item{tconv.max}{overall maximum convergence ratio.}
  \item{ac3}{If \code{x$maxlike=TRUE}:
                    autocorrelations of statistics in Phase 3.}
  \item{targets}{Observed statistics; for ML, zero vector.}
  \item{targets2}{Observed statistics by wave,
    starting with second wave; for ML, zero matrix.}
  \item{ssc}{Score function contributions for each wave for each
    simulation in phase 3. Not included if finite difference method is used
    or if \code{x$lessMem=TRUE}.}
  \item{scores}{Score functions, added over waves, for each
    simulation in phase 3. Only included if \code{x$lessMem=FALSE}.}
  \item{regrCoef}{If \code{x$dolby} and not \code{x$maxlike}:
    regression coefficients of estimation statistics on score functions.}
  \item{regrCor}{If \code{x$dolby} and not \code{x$maxlike}:
    correlations between estimation statistics and score functions.}
  \item{estMeans}{Estimated means of estimation statistics.}
  \item{estMeans.sem}{If \code{x$simOnly}: Standard errors of the
    estimated means of estimation statistics.}
 \item{sims}{If \code{returnDeps=TRUE}:
    list of simulated dependent variables (networks, behaviour).
    Networks are given as a list of edgelists, one for each period.\cr
    The structure of sims is a nested list:
    \code{sims[[run]][[group]][[dependent variable]][[period]]}.
    If \code{x$maxlike=TRUE} and there is only one group and one period,
    the structure is \code{[[run]][[dependent variable]]}.}
 \item{chain}{If \code{returnChains = TRUE}:
    list, or data frame, of simulated chains of ministeps. \cr
    The chain has the structure
    \code{chain[[run]][[depvar]][[period]][[ministep]]}.
    }
  \item{changeContributions}{If \code{returnChangeContributions = TRUE}:
    list, or data frame, of simulated change contributions. \cr
    The contributions have the structure
    \code{returnChangeContributions[[run]][[depvar]][[period]][[ministep]]}.
    }
   \item{Phase3nits}{Number of iterations actually performed in phase 3.}
   \item{thetaUsed}{If \code{thetaValues} is not \code{NULL}, the matrix
   of parameter values actually used in the simulations of Phase 3.}
  Writes text output to the file indicated for \code{siena}
  in \code{control_output} and for \code{siena07} in \code{alg}.
}
\references{
    Amati, V., Schoenenberger, F., and Snijders, T.A.B. (2015),
    Estimation of stochastic actor-oriented models for the evolution of
    networks by generalized method of moments.
    \emph{Journal de la Societe Francaise de Statistique} \bold{156}, 140--165.

    Amati, V., Schoenenberger, F., and Snijders, T.A.B. (2019),
    Contemporaneous statistics for estimation in stochastic actor-oriented
    co-evolution models. \emph{Psychometrika} \bold{84}, 1068--1096.

    Greenan, C. (2015),
    \emph{Evolving Social Network Analysis: developments in statistical
    methodology for dynamic stochastic actor-oriented models}.
    DPhil dissertation, University of Oxford.

    Niezink, N.M.D., and Snijders, T.A.B. (2017),
    Co-evolution of Social Networks and Continuous Actor Attributes.
    \emph{The Annals of Applied Statistics} \bold{11}, 1948--1973.

    Schweinberger, M., and Snijders, T.A.B. (2007),
    Markov models for digraph panel data:
    Monte Carlo based derivative estimation.
    \emph{Computational Statistics and Data Analysis} \bold{51}, 4465--4483.

     Snijders, T.A.B. (2001),
    The statistical evaluation of social network dynamics.
    \emph{Sociological Methodology} \bold{31}, 361--395.

    Snijders, T.A.B. (2017), Stochastic Actor-Oriented Models for Network
   Dynamics. \emph{Annual Review of Statistics and Its Application}
    \bold{4}, 343--363.

    Snijders, T.A.B., Koskinen, J., and Schweinberger, M. (2010).
    Maximum likelihood estimation for social network dynamics.
    \emph{Annals of Applied Statistics} \bold{4}, 567--588.

    Snijders, T.A.B., Steglich, C.E.G., and Schweinberger, Michael (2007),
    Modeling the co-evolution of networks and behavior.
    Pp. 41--71 in \emph{Longitudinal models in the behavioral
    and related sciences},
    edited by van Montfort, K., Oud, H., and Satorra, A.; Lawrence Erlbaum.

    Steglich, C.E.G., Snijders, T.A.B., and Pearson, M.A. (2010),
    Dynamic networks and behavior:     Separating selection from influence.
    \emph{Sociological Methodology} \bold{40}, 329--393.
    Information about the implementation of the algorithm is in
     \url{https://www.stats.ox.ac.uk/~snijders/siena/Siena_algorithms.pdf}.\cr
    Further see \url{https://www.stats.ox.ac.uk/~snijders/siena/} .
}
\author{Ruth Ripley, Tom Snijders, Viviana Amati, Felix Schoenenberger,
        Nynke Niezink}

\seealso{\code{\link{sienadata}}, \code{\link{set_algorithm_saom}},
  \code{\link{sienaEffects}},
  \code{\link{test_parameter}}.
  There are print, summary and xtable methods for \code{\link{sienaFit}}
  objects: \code{\link{xtable}}, \code{\link{print.sienaFit}}.}
\examples{
myalgo <- set_algorithm_saom(nsub=2, n3=100, seed=1293)
# nsub=2, n3=100 is used here for having a brief computation, not for practice.
mynet1 <- as_dependent_rsiena(array(c(tmp3, tmp4), dim=c(32, 32, 2)))
mydata <- make_data_rsiena(mynet1)
myeff <- make_specification(mydata)
ans <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE)
# or, when using the old siena07,
myalgorithm <- sienaAlgorithmCreate(nsub=2, n3=100, seed=1293)
ans <- siena07(myalgorithm, data=mydata, effects=myeff, batch=TRUE)
ans
coef(ans)
# or for non-conditional estimation ---------------------------------
myalgo <- set_algorithm_saom(nsub=2, n3=100, cond=FALSE, seed=1273)
ans <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE)
coef(ans)
coef(ans, dropRates=TRUE)
# or if a previous "on track" result ans was obtained ---------------
\dontrun{
ans1 <- siena(mydata, effects=myeff, control_algo=myalgo, prevAns=ans)
         }

# Running in multiple processors ------------------------------------
\dontrun{
# Not tested because dependent on presence of processors
# Find out how many processors there are
library(parallel)
(n.clus <- detectCores() - 1)
n.clus <- min(n.clus, 4)  # keep time for other processes
ans2 <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE,
                nbrNodes=n.clus)

# Suppose 8 processors are going to be used.
# Loading the parallel package and creating a cluster
# with 8 processors (this should be equivalent)

library(parallel)
cl <- makeCluster(n.clus)

ans3 <- siena07(myalgorithm, data=mydata, effects=myeff, batch=TRUE, cl = cl)
# I did not succeed in constructing an alternative with siena for this.
# Notice that now -siena- perhaps won't stop the cluster for you.
# stopCluster(cl)

# You can create even more complex clusters using several computers. In this
# example we are creating a cluster with 3*8 = 24 processors on three
# different machines.
#cl <- makePSOCKcluster(
#    rep(c('localhost', 'machine2.website.com' , 'machine3.website.com'), 8),
#    user='myusername', rshcmd='ssh -p PORTNUMBER')

#ans4 <- siena07(myalgorithm, data=mydata, effects=myeff, batch=TRUE, cl = cl)
#stopCluster(cl)
}

# for a continuous behavior variable --------------------------------
# simulate behavior data according to dZ(t) = [-0.1 Z + 1] dt + 1 dW(t)
set.seed(123)
y1 <- rnorm(50, 0,3)
y2 <- exp(-0.1) * y1 + (1-exp(-0.1)) * 1/ -0.1 + 
                       rnorm(50, 0, (exp(-0.2)- 1) / -0.2 * 1^2)
friend <- as_dependent_rsiena(array(c(s501, s502), dim = c(50,50,2)))
behavior <- as_dependent_rsiena(matrix(c(y1,y2), 50,2), type = "continuous")
(mydata <- make_data_rsiena(friend, behavior))
(myeff <- make_specification(mydata, onePeriodSde = TRUE))
algorithmMoM <- set_algorithm_saom(nsub=1, n3=20, seed=321)
(ans <- siena(mydata, effects=myeff, control_algo=myalgo, batch=TRUE))

# Accessing simulated networks for ML -------------------------------
# The following is an example for accessing the simulated networks for ML,
# which makes sense only if there are some missing tie variables;
# observed tie variables are identically simulated
# at the moment of observation,
# missing tie variable are imputed in a model-based way.
mat1 <- matrix(c(0,0,1,1,
                 1,0,0,0,
                 0,0,0,1,
                 0,1,0,0),4,4, byrow=TRUE)
mat2 <- matrix(c(0,1,1,1,
                 1,0,0,0,
                 0,0,0,1,
                 0,0,1,0),4,4, byrow=TRUE)
mat3 <- matrix(c(0,1,0,1,
                 1,0,0,0,
                 0,0,0,0,
                 NA,1,1,0),4,4, byrow=TRUE)
mats <- array(c(mat1,mat2,mat3), dim=c(4,4,3))
net <- as_dependent_rsiena(mats, allowOnly=FALSE)
sdat <- make_data_rsiena(net)
alg <- set_algorithm_saom(maxlike=TRUE, nsub=3, n3=100, seed=12534)
effs <- make_specification(sdat)
(ans <- siena(sdat, effects=effs, control_algo=alg,
                              batch=TRUE, returnDeps=TRUE))
# See manual Section 10.1 for information about the following functions
edges.to.adj <- function(x,n){
# create empty adjacency matrix
    adj <- matrix(0, n, n)
# put edge values in desired places
    adj[x[, 1:2]] <- x[, 3]
    adj
}
the.edge <- function(x,n,h,k){
    edges.to.adj(x,n)[h,k]
}
# Now show the results
n <- 4
ego <- rep.int(1:n,n)
alter <- rep(1:n, each=n)
# Get the average simulated adjacency matrices for wave 3 (period 2):
ones <- sapply(1:n^2, function(i)
    {mean(sapply(ans$sims,
           function(x){the.edge(x[[1]][[2]][[1]],n,ego[i],alter[i])}))})
# Note that for maximum likelihood estimation,
# if there is one group and one period,
# the nesting levels for group and period are dropped from ans$sims.
cbind(ego,alter,ones)
matrix(ones,n,n)
}
\keyword{models}
